#include "megadrive.h"
#include "burnint.h"

// Driver generated by database on http://www.barryharris.me.uk

static struct BurnInputInfo MegadriveInputList[] = {
	{"P1 Start",		BIT_DIGITAL,	MegadriveJoy1 +  7, "p1 start"  },
	{"P1 Up",		BIT_DIGITAL,	MegadriveJoy1 +  0, "p1 up"     },
	{"P1 Down",		BIT_DIGITAL,	MegadriveJoy1 +  1, "p1 down"   },
	{"P1 Left",		BIT_DIGITAL,	MegadriveJoy1 +  2, "p1 left"   },
	{"P1 Right",		BIT_DIGITAL,	MegadriveJoy1 +  3, "p1 right"  },
	{"P1 Button A",		BIT_DIGITAL,	MegadriveJoy1 +  6, "p1 fire 1" },
	{"P1 Button B",		BIT_DIGITAL,	MegadriveJoy1 +  4, "p1 fire 2" },
	{"P1 Button C",		BIT_DIGITAL,	MegadriveJoy1 +  5, "p1 fire 3" },
	{"P1 Button X",		BIT_DIGITAL,	MegadriveJoy1 + 10, "p1 fire 4" },
	{"P1 Button Y",		BIT_DIGITAL,	MegadriveJoy1 +  9, "p1 fire 5" },
	{"P1 Button Z",		BIT_DIGITAL,	MegadriveJoy1 +  8, "p1 fire 6" },
	{"P1 Mode",		BIT_DIGITAL,	MegadriveJoy1 + 11, "p1 select" },

	{"P2 Start",		BIT_DIGITAL,	MegadriveJoy2 +  7, "p2 start"  },
	{"P2 Up",		BIT_DIGITAL,	MegadriveJoy2 +  0, "p2 up"     },
	{"P2 Down",		BIT_DIGITAL,	MegadriveJoy2 +  1, "p2 down"   },
	{"P2 Left",		BIT_DIGITAL,	MegadriveJoy2 +  2, "p2 left"   },
	{"P2 Right",		BIT_DIGITAL,	MegadriveJoy2 +  3, "p2 right"  },
	{"P2 Button A",		BIT_DIGITAL,	MegadriveJoy2 +  6, "p2 fire 1" },
	{"P2 Button B",		BIT_DIGITAL,	MegadriveJoy2 +  4, "p2 fire 2" },
	{"P2 Button C",		BIT_DIGITAL,	MegadriveJoy2 +  5, "p2 fire 3" },
	{"P2 Button X",		BIT_DIGITAL,	MegadriveJoy2 + 10, "p2 fire 4" },
	{"P2 Button Y",		BIT_DIGITAL,	MegadriveJoy2 +  9, "p2 fire 5" },
	{"P2 Button Z",		BIT_DIGITAL,	MegadriveJoy2 +  8, "p2 fire 6" },
	{"P2 Mode",		BIT_DIGITAL,	MegadriveJoy2 + 11, "p2 select" },

	{"Reset",		BIT_DIGITAL,	&MegadriveReset,     "reset"    },
	{"Dip A",		BIT_DIPSWITCH,	MegadriveDIP  + 0,   "dip"      },
	{"Dip B",		BIT_DIPSWITCH,	MegadriveDIP  + 1,   "dip"      },
};

STDINPUTINFO(Megadrive);

static struct BurnDIPInfo MegadriveDIPList[] = {
	{0x19,	0xFF, 0xFF, 0x21,  NULL               },	// Default Auto-Detect and No-CDROM
	{0x1a,	0xFF, 0xFF, 0x01,  NULL               },

	{0,	0xFE, 0,       5, "Hardware"          },
	{0x19,	0x01, 0xC1, 0x01, "Auto Detect"       },
	{0x19,	0x01, 0xC1, 0x00, "Japan NTSC"        },
	{0x19,	0x01, 0xC1, 0x40, "Japan PAL"         },
	{0x19,	0x01, 0xC1, 0x80, "USA"               },
	{0x19,	0x01, 0xC1, 0xC0, "Europe"            },

	{0,	0xFE, 0,       2, "CD-ROM"            },
	{0x19,	0x01, 0x20, 0x20, "No"                },
	{0x19,	0x01, 0x20, 0x00, "Yes"               },

	{0,	0xFE, 0,       3, "32-Col Mode Draw"  },
	{0x1a,	0x01, 0x03, 0x00, "No Change"         },
	{0x1a,	0x01, 0x03, 0x01, "Center"            },
	{0x1a,	0x01, 0x03, 0x02, "Inner Zoom"        },
};

STDDIPINFO(Megadrive);

// Adventures of Batman and Robin, The (U) [!]
static struct BurnRomInfo g_abatRomDesc[] = {
	{ "g_abat.bin", 0x200000, 0x0CAAA4AC, BRF_PRG },
};

STD_ROM_PICK(g_abat);
STD_ROM_FN(g_abat);

struct BurnDriverD BurnDrvMDg_abat = {
	"g_abat", NULL, NULL, "1995",
	"Adventures of Batman and Robin, The (U) [!]\0", "Sound Problems", "Clockwork Tortoise Inc.", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_SCRFIGHT, 0,
	NULL, g_abatRomInfo, g_abatRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Adventures of Batman and Robin, The (E) [!]
static struct BurnRomInfo g_abateRomDesc[] = {
	{ "g_abate.bin", 0x200000, 0x355E8C39, BRF_PRG },
};

STD_ROM_PICK(g_abate);
STD_ROM_FN(g_abate);

struct BurnDriverD BurnDrvMDg_abate = {
	"g_abate", "g_abat", NULL, "1995",
	"Adventures of Batman and Robin, The (E) [!]\0", "Gfx Problems", "Clockwork Tortoise Inc.", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_SCRFIGHT, 0,
	NULL, g_abateRomInfo, g_abateRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Aero the Acro-Bat (U) [c][!]
static struct BurnRomInfo g_acroRomDesc[] = {
	{ "g_acro.bin", 0x100000, 0xa3a7a8b5, BRF_PRG },
};

STD_ROM_PICK(g_acro);
STD_ROM_FN(g_acro);

struct BurnDriverD BurnDrvMDg_acro = {
	"g_acro", NULL, NULL, "1993",
	"Aero the Acro-Bat (U) [c][!]\0", NULL, "Sunsoft / Iguana", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_acroRomInfo, g_acroRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Aero the Acro-Bat 2 (U) [!]
static struct BurnRomInfo g_acr2RomDesc[] = {
	{ "g_acr2.bin", 0x200000, 0x39eb74eb, BRF_PRG },
};

STD_ROM_PICK(g_acr2);
STD_ROM_FN(g_acr2);

struct BurnDriverD BurnDrvMDg_acr2 = {
	"g_acr2", NULL, NULL, "1994",
	"Aero the Acro-Bat 2 (U) [!]\0", NULL, "Sunsoft / Iguana", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_acr2RomInfo, g_acr2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Aero the Acro-Bat 2 (E) [!]
static struct BurnRomInfo g_acro2eRomDesc[] = {
	{ "g_acro2e.bin", 0x200000, 0xa451f9a1, BRF_PRG },
};

STD_ROM_PICK(g_acro2e);
STD_ROM_FN(g_acro2e);

struct BurnDriverD BurnDrvMDg_acro2e = {
	"g_acro2e", "g_acr2", NULL, "1994",
	"Aero the Acro-Bat 2 (E) [!]\0", NULL, "Sunsoft / Iguana", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_acro2eRomInfo, g_acro2eRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Aerobiz (U) [!]
static struct BurnRomInfo g_aeroRomDesc[] = {
	{ "g_aero.bin", 0x100000, 0xcfaa9bce, BRF_PRG },
};

STD_ROM_PICK(g_aero);
STD_ROM_FN(g_aero);

struct BurnDriver BurnDrvMDg_aero = {
	"g_aero", NULL, NULL, "199?",
	"Aerobiz (U) [!]\0", NULL, "Koei", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_aeroRomInfo, g_aeroRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Air Management - Oozora ni Kakeru (J) [c][!]
static struct BurnRomInfo g_airmRomDesc[] = {
	{ "g_airm.bin", 0x100000, 0xb3db0c71, BRF_PRG },
};

STD_ROM_PICK(g_airm);
STD_ROM_FN(g_airm);

struct BurnDriver BurnDrvMDg_airm = {
	"g_airm", "g_aero", NULL, "199?",
	"Air Management - Oozora ni Kakeru (J) [c][!]\0", NULL, "Koei", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_airmRomInfo, g_airmRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Aerobiz Supersonic (U) [!]
static struct BurnRomInfo g_abz2RomDesc[] = {
	{ "g_abz2.bin", 0x100000, 0x9377f1b5, BRF_PRG },
};

STD_ROM_PICK(g_abz2);
STD_ROM_FN(g_abz2);

struct BurnDriver BurnDrvMDg_abz2 = {
	"g_abz2", NULL, NULL, "1994",
	"Aerobiz Supersonic (U) [!]\0", NULL, "Koei", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_abz2RomInfo, g_abz2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Air Management II - Kouku Ou wo Mezase (J) [!]
static struct BurnRomInfo g_airm2RomDesc[] = {
	{ "g_airm2.bin", 0x100000, 0x4582817b, BRF_PRG },
};

STD_ROM_PICK(g_airm2);
STD_ROM_FN(g_airm2);

struct BurnDriver BurnDrvMDg_airm2 = {
	"g_airm2", "g_abz2", NULL, "1994",
	"Air Management II - Kouku Ou wo Mezase (J) [!]\0", NULL, "Koei", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_airm2RomInfo, g_airm2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Disney's Aladdin (U) [!]
static struct BurnRomInfo g_aladRomDesc[] = {
	{ "g_alad.bin", 0x200000, 0xed427ea9, BRF_PRG },
};

STD_ROM_PICK(g_alad);
STD_ROM_FN(g_alad);

struct BurnDriverD BurnDrvMDg_alad = {
	"g_alad", NULL, NULL, "1993",
	"Disney's Aladdin (U) [!]\0", NULL, "Disney / Virgin", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_aladRomInfo, g_aladRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Disney's Aladdin (E) [!]
static struct BurnRomInfo g_aladeRomDesc[] = {
	{ "g_alade.bin", 0x200000, 0xd1845e8f, BRF_PRG },
};

STD_ROM_PICK(g_alade);
STD_ROM_FN(g_alade);

struct BurnDriverD BurnDrvMDg_alade = {
	"g_alade", "g_alad", NULL, "1993",
	"Disney's Aladdin (E) [!]\0", NULL, "Disney / Virgin", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_aladeRomInfo, g_aladeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Disney's Aladdin (J) [!]
static struct BurnRomInfo g_aladjRomDesc[] = {
	{ "g_aladj.bin", 0x200000, 0xfb5aacf0, BRF_PRG },
};

STD_ROM_PICK(g_aladj);
STD_ROM_FN(g_aladj);

struct BurnDriverD BurnDrvMDg_aladj = {
	"g_aladj", "g_alad", NULL, "1993",
	"Disney's Aladdin (J) [!]\0", NULL, "Disney / Virgin", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_aladjRomInfo, g_aladjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Disney's Aladdin (Beta) [!]
static struct BurnRomInfo g_aladbRomDesc[] = {
	{ "g_aladb.bin", 0x200000, 0x8c60ef73, BRF_PRG },
};

STD_ROM_PICK(g_aladb);
STD_ROM_FN(g_aladb);

struct BurnDriverD BurnDrvMDg_aladb = {
	"g_aladb", "g_alad", NULL, "1993",
	"Disney's Aladdin (Beta) [!]\0", NULL, "Disney / Virgin", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_aladbRomInfo, g_aladbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Alex Kidd in the Enchanted Castle (U) [!]
static struct BurnRomInfo g_alexRomDesc[] = {
	{ "g_alex.bin", 0x40000, 0x47dba0ac, BRF_PRG },
};

STD_ROM_PICK(g_alex);
STD_ROM_FN(g_alex);

struct BurnDriverD BurnDrvMDg_alex = {
	"g_alex", NULL, NULL, "1989",
	"Alex Kidd in the Enchanted Castle (U) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_alexRomInfo, g_alexRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Alex Kidd in the Enchanted Castle (E) (MD Bundle) [!]
static struct BurnRomInfo g_alxkebRomDesc[] = {
	{ "g_alxkeb.bin", 0x40000, 0x778a0f00, BRF_PRG },
};

STD_ROM_PICK(g_alxkeb);
STD_ROM_FN(g_alxkeb);

struct BurnDriverD BurnDrvMDg_alxkeb = {
	"g_alxkeb", "g_alex", NULL, "1989",
	"Alex Kidd in the Enchanted Castle (E) (MD Bundle) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_alxkebRomInfo, g_alxkebRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Alex Kidd in the Enchanted Castle (E) [!]
static struct BurnRomInfo g_alexkeRomDesc[] = {
	{ "g_alexke.bin", 0x40000, 0xc3a52529, BRF_PRG },
};

STD_ROM_PICK(g_alexke);
STD_ROM_FN(g_alexke);

struct BurnDriverD BurnDrvMDg_alexke = {
	"g_alexke", "g_alex", NULL, "1989",
	"Alex Kidd in the Enchanted Castle (E) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_alexkeRomInfo, g_alexkeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Alex Kidd in the Enchanted Castle (J) [!]
static struct BurnRomInfo g_alexkjRomDesc[] = {
	{ "g_alexkj.bin", 0x40000, 0x8a5ed856, BRF_PRG },
};

STD_ROM_PICK(g_alexkj);
STD_ROM_FN(g_alexkj);

struct BurnDriverD BurnDrvMDg_alexkj = {
	"g_alexkj", "g_alex", NULL, "1989",
	"Alex Kidd in the Enchanted Castle (J) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_alexkjRomInfo, g_alexkjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Alex Kidd - Cheongongmaseong (K) [!]
static struct BurnRomInfo g_alexkkRomDesc[] = {
	{ "g_alexkk.bin", 0x40000, 0x5b0678fb, BRF_PRG },
};

STD_ROM_PICK(g_alexkk);
STD_ROM_FN(g_alexkk);

struct BurnDriverD BurnDrvMDg_alexkk = {
	"g_alexkk", "g_alex", NULL, "1989",
	"Alex Kidd - Cheongongmaseong (K) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_alexkkRomInfo, g_alexkkRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Altered Beast (UE) (REV02) [!]
static struct BurnRomInfo g_abeaRomDesc[] = {
	{ "g_abea.bin", 0x80000, 0x154d59bb, BRF_PRG },
};

STD_ROM_PICK(g_abea);
STD_ROM_FN(g_abea);

struct BurnDriverD BurnDrvMDg_abea = {
	"g_abea", NULL, NULL, "1988",
	"Altered Beast (UE) (REV02) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_abeaRomInfo, g_abeaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Juu-Ou-Ki (J) (REV00) [c][!]
static struct BurnRomInfo g_juuo00RomDesc[] = {
	{ "g_juuo00.bin", 0x80000, 0x1b7c96c0, BRF_PRG },
};

STD_ROM_PICK(g_juuo00);
STD_ROM_FN(g_juuo00);

struct BurnDriverD BurnDrvMDg_juuo00 = {
	"g_juuo00", "g_abea", NULL, "1988",
	"Juu-Ou-Ki (J) (REV00) [c][!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_juuo00RomInfo, g_juuo00RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Juu-Ou-Ki (J) (REV01) [!]
static struct BurnRomInfo g_juuo01RomDesc[] = {
	{ "g_juuo01.bin", 0x80000, 0xb2233e87, BRF_PRG },
};

STD_ROM_PICK(g_juuo01);
STD_ROM_FN(g_juuo01);

struct BurnDriverD BurnDrvMDg_juuo01 = {
	"g_juuo01", "g_abea", NULL, "1988",
	"Juu-Ou-Ki (J) (REV01) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_juuo01RomInfo, g_juuo01RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Atomic Runner (U) [!]
static struct BurnRomInfo g_arunRomDesc[] = {
	{ "g_arun.bin", 0x100000, 0x0677c210, BRF_PRG },
};

STD_ROM_PICK(g_arun);
STD_ROM_FN(g_arun);

struct BurnDriver BurnDrvMDg_arun = {
	"g_arun", NULL, NULL, "1992",
	"Atomic Runner (U) [!]\0", NULL, "Data East", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_arunRomInfo, g_arunRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Atomic Runner (E) [!]
static struct BurnRomInfo g_arunreRomDesc[] = {
	{ "g_arunre.bin", 0x100000, 0xb3c05418, BRF_PRG },
};

STD_ROM_PICK(g_arunre);
STD_ROM_FN(g_arunre);

struct BurnDriver BurnDrvMDg_arunre = {
	"g_arunre", "g_arun", NULL, "1992",
	"Atomic Runner (E) [!]\0", NULL, "Data East", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_arunreRomInfo, g_arunreRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Chelnov (J) [!]
static struct BurnRomInfo g_chelnoRomDesc[] = {
	{ "g_chelno.bin", 0x100000, 0xb2fe74d8, BRF_PRG },
};

STD_ROM_PICK(g_chelno);
STD_ROM_FN(g_chelno);

struct BurnDriver BurnDrvMDg_chelno = {
	"g_chelno", "g_arun", NULL, "1992",
	"Chelnov (J) [!]\0", NULL, "Data East", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_chelnoRomInfo, g_chelnoRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Art of Fighting (U) [!]
static struct BurnRomInfo g_aofRomDesc[] = {
	{ "g_aof.bin", 0x200000, 0xc9a57e07, BRF_PRG },
};

STD_ROM_PICK(g_aof);
STD_ROM_FN(g_aof);

struct BurnDriver BurnDrvMDg_aof = {
	"g_aof", NULL, NULL, "1994",
	"Art of Fighting (U) [!]\0", NULL, "SNK / Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_aofRomInfo, g_aofRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Art of Fighting (E) [!]
static struct BurnRomInfo g_aofeRomDesc[] = {
	{ "g_aofe.bin", 0x200000, 0x9970c422, BRF_PRG },
};

STD_ROM_PICK(g_aofe);
STD_ROM_FN(g_aofe);

struct BurnDriver BurnDrvMDg_aofe = {
	"g_aofe", "g_aof", NULL, "1994",
	"Art of Fighting (E) [!]\0", NULL, "SNK / Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_aofeRomInfo, g_aofeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Ryuuko no Ken (J) [!]
static struct BurnRomInfo g_ryuukoRomDesc[] = {
	{ "g_ryuuko.bin", 0x200000, 0x054cf5f6, BRF_PRG },
};

STD_ROM_PICK(g_ryuuko);
STD_ROM_FN(g_ryuuko);

struct BurnDriver BurnDrvMDg_ryuuko = {
	"g_ryuuko", "g_aof", NULL, "1994",
	"Ryuuko no Ken (J) [!]\0", NULL, "SNK / Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ryuukoRomInfo, g_ryuukoRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Beyond Oasis (U) [!]
static struct BurnRomInfo g_boasRomDesc[] = {
	{ "g_boas.bin", 0x300000, 0xc4728225, BRF_PRG },
};

STD_ROM_PICK(g_boas);
STD_ROM_FN(g_boas);

struct BurnDriver BurnDrvMDg_boas = {
	"g_boas", NULL, NULL, "1995",
	"Beyond Oasis (U) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_boasRomInfo, g_boasRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Story of Thor, The - A Successor of The Light (J) [!]
static struct BurnRomInfo g_sthorjRomDesc[] = {
	{ "g_sthorj.bin", 0x300000, 0x4f39783c, BRF_PRG },
};

STD_ROM_PICK(g_sthorj);
STD_ROM_FN(g_sthorj);

struct BurnDriver BurnDrvMDg_sthorj = {
	"g_sthorj", "g_boas", NULL, "1995",
	"Story of Thor, The - A Successor of The Light (J) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sthorjRomInfo, g_sthorjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Story of Thor, The - A Successor of The Light (E) [!]
static struct BurnRomInfo g_sthorRomDesc[] = {
	{ "g_sthor.bin", 0x300000, 0x1110b0db, BRF_PRG },
};

STD_ROM_PICK(g_sthor);
STD_ROM_FN(g_sthor);

struct BurnDriver BurnDrvMDg_sthor = {
	"g_sthor", "g_boas", NULL, "1995",
	"Story of Thor, The - A Successor of The Light (E) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sthorRomInfo, g_sthorRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Story of Thor, The - A Successor of The Light (F)
static struct BurnRomInfo g_sthorfRomDesc[] = {
	{ "g_sthorf.bin", 0x300000, 0xb97cca1c, BRF_PRG },
};

STD_ROM_PICK(g_sthorf);
STD_ROM_FN(g_sthorf);

struct BurnDriver BurnDrvMDg_sthorf = {
	"g_sthorf", "g_boas", NULL, "1995",
	"Story of Thor, The - A Successor of The Light (F)\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sthorfRomInfo, g_sthorfRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Story of Thor, The - A Successor of The Light (G)
static struct BurnRomInfo g_sthorgRomDesc[] = {
	{ "g_sthorg.bin", 0x300000, 0xfa20d011, BRF_PRG },
};

STD_ROM_PICK(g_sthorg);
STD_ROM_FN(g_sthorg);

struct BurnDriver BurnDrvMDg_sthorg = {
	"g_sthorg", "g_boas", NULL, "1995",
	"Story of Thor, The - A Successor of The Light (G)\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sthorgRomInfo, g_sthorgRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Story of Thor, The - A Successor of The Light (K)
static struct BurnRomInfo g_sthorkRomDesc[] = {
	{ "g_sthork.bin", 0x300000, 0xee1603c5, BRF_PRG },
};

STD_ROM_PICK(g_sthork);
STD_ROM_FN(g_sthork);

struct BurnDriver BurnDrvMDg_sthork = {
	"g_sthork", "g_boas", NULL, "1995",
	"Story of Thor, The - A Successor of The Light (K)\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sthorkRomInfo, g_sthorkRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Story of Thor, The - A Successor of The Light (S)
static struct BurnRomInfo g_sthorsRomDesc[] = {
	{ "g_sthors.bin", 0x300000, 0x4631f941, BRF_PRG },
};

STD_ROM_PICK(g_sthors);
STD_ROM_FN(g_sthors);

struct BurnDriver BurnDrvMDg_sthors = {
	"g_sthors", "g_boas", NULL, "1995",
	"Story of Thor, The - A Successor of The Light (S)\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sthorsRomInfo, g_sthorsRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Story of Thor, The - A Successor of The Light (Beta)
static struct BurnRomInfo g_sthorbRomDesc[] = {
	{ "g_sthorb.bin", 0x1b0000, 0xbfc11649, BRF_PRG },
};

STD_ROM_PICK(g_sthorb);
STD_ROM_FN(g_sthorb);

struct BurnDriver BurnDrvMDg_sthorb = {
	"g_sthorb", "g_boas", NULL, "1995",
	"Story of Thor, The - A Successor of The Light (Beta)\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sthorbRomInfo, g_sthorbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Block Out (W) (REV01) [!]
static struct BurnRomInfo g_blocRomDesc[] = {
	{ "g_bloc.bin", 0x20000, 0x5e2966f1, BRF_PRG },
};

STD_ROM_PICK(g_bloc);
STD_ROM_FN(g_bloc);

struct BurnDriver BurnDrvMDg_bloc = {
	"g_bloc", NULL, NULL, "1994",
	"Block Out (W) (REV01) [!]\0", NULL, "Electronic Arts", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_blocRomInfo, g_blocRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Blood Shot (E) (M4) [!]
static struct BurnRomInfo g_bshtRomDesc[] = {
	{ "g_bsht.bin", 0x200000, 0xf9f2bceb, BRF_PRG },
};

STD_ROM_PICK(g_bsht);
STD_ROM_FN(g_bsht);

struct BurnDriverD BurnDrvMDg_bsht = {
	"g_bsht", NULL, NULL, "1994",
	"Blood Shot (E) (M4) [!]\0", NULL, "Domark / Acclaim", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_bshtRomInfo, g_bshtRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// B.O.B (UE) (REV01) [!]
static struct BurnRomInfo g_bobRomDesc[] = {
	{ "g_bob.bin", 0x100000, 0xeaa2acb7, BRF_PRG },
};

STD_ROM_PICK(g_bob);
STD_ROM_FN(g_bob);

struct BurnDriverD BurnDrvMDg_bob = {
	"g_bob", NULL, NULL, "1994",
	"B.O.B (UE) (REV01) [!]\0", NULL, "Electronic Arts", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_bobRomInfo, g_bobRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Space Funky B.O.B. (J) [!]
static struct BurnRomInfo g_sfbobRomDesc[] = {
	{ "g_sfbob.bin", 0x100000, 0xe9310d3b, BRF_PRG },
};

STD_ROM_PICK(g_sfbob);
STD_ROM_FN(g_sfbob);

struct BurnDriverD BurnDrvMDg_sfbob = {
	"g_sfbob", "g_bob", NULL, "1994",
	"Space Funky B.O.B. (J) [!]\0", NULL, "Electronic Arts", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sfbobRomInfo, g_sfbobRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// B.O.B (Beta)
static struct BurnRomInfo g_bobbRomDesc[] = {
	{ "g_bobb.bin", 0x100000, 0xe3e8421e, BRF_PRG },
};

STD_ROM_PICK(g_bobb);
STD_ROM_FN(g_bobb);

struct BurnDriverD BurnDrvMDg_bobb = {
	"g_bobb", "g_bob", NULL, "1994",
	"B.O.B (Beta)\0", NULL, "Electronic Arts", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_bobbRomInfo, g_bobbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Bram Stoker's Dracula (U) [!]
static struct BurnRomInfo g_dracRomDesc[] = {
	{ "g_drac.bin", 0x100000, 0x077084a6, BRF_PRG },
};

STD_ROM_PICK(g_drac);
STD_ROM_FN(g_drac);

struct BurnDriverD BurnDrvMDg_drac = {
	"g_drac", NULL, NULL, "1994",
	"Bram Stoker's Dracula (U) [!]\0", NULL, "Psygnosis / Sony", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_dracRomInfo, g_dracRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Bram Stoker's Dracula (E) [!]
static struct BurnRomInfo g_draceRomDesc[] = {
	{ "g_drace.bin", 0x100000, 0x9ba5a063, BRF_PRG },
};

STD_ROM_PICK(g_drace);
STD_ROM_FN(g_drace);

struct BurnDriverD BurnDrvMDg_drace = {
	"g_drace", "g_drac", NULL, "1994",
	"Bram Stoker's Dracula (E) [!]\0", NULL, "Psygnosis / Sony", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_draceRomInfo, g_draceRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Captain Planet and the Planeteers (E) [!]
static struct BurnRomInfo g_cappRomDesc[] = {
	{ "g_capp.bin", 0x80000, 0x7672efa5, BRF_PRG },
};

STD_ROM_PICK(g_capp);
STD_ROM_FN(g_capp);

struct BurnDriverD BurnDrvMDg_capp = {
	"g_capp", NULL, NULL, "1900",
	"Captain Planet and the Planeteers (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_cappRomInfo, g_cappRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Captain Planet and the Planeteers (Beta)
static struct BurnRomInfo g_cappbRomDesc[] = {
	{ "g_cappb.bin", 0x80000, 0xbf2cbd3a, BRF_PRG },
};

STD_ROM_PICK(g_cappb);
STD_ROM_FN(g_cappb);

struct BurnDriverD BurnDrvMDg_cappb = {
	"g_cappb", "g_capp", NULL, "1900",
	"Captain Planet and the Planeteers (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_cappbRomInfo, g_cappbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Castle of Illusion Starring Mickey Mouse (UE) [!]
static struct BurnRomInfo g_cillRomDesc[] = {
	{ "g_cill.bin", 0x80000, 0xba4e9fd0, BRF_PRG },
};

STD_ROM_PICK(g_cill);
STD_ROM_FN(g_cill);

struct BurnDriverD BurnDrvMDg_cill = {
	"g_cill", NULL, NULL, "1990",
	"Castle of Illusion Starring Mickey Mouse (UE) [!]\0", NULL, "Disney / Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_cillRomInfo, g_cillRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// I Love Mickey Mouse - Fushigi no Oshiro Dai Bouken (J) [!]
static struct BurnRomInfo g_mickeyRomDesc[] = {
	{ "g_mickey.bin", 0x80000, 0xce8333c6, BRF_PRG },
};

STD_ROM_PICK(g_mickey);
STD_ROM_FN(g_mickey);

struct BurnDriverD BurnDrvMDg_mickey = {
	"g_mickey", "g_cill", NULL, "1990",
	"I Love Mickey Mouse - Fushigi no Oshiro Dai Bouken (J) [!]\0", NULL, "Disney / Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mickeyRomInfo, g_mickeyRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Castlevania - Bloodlines (U) [!]
static struct BurnRomInfo g_casvRomDesc[] = {
	{ "g_casv.bin", 0x100000, 0xfb1ea6df, BRF_PRG },
};

STD_ROM_PICK(g_casv);
STD_ROM_FN(g_casv);

struct BurnDriverD BurnDrvMDg_casv = {
	"g_casv", NULL, NULL, "1994",
	"Castlevania - Bloodlines (U) [!]\0", NULL, "Konami", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_casvRomInfo, g_casvRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Castlevania - The New Generation (E) [!]
static struct BurnRomInfo g_cvaneRomDesc[] = {
	{ "g_cvane.bin", 0x100000, 0x4dd4e4a5, BRF_PRG },
};

STD_ROM_PICK(g_cvane);
STD_ROM_FN(g_cvane);

struct BurnDriverD BurnDrvMDg_cvane = {
	"g_cvane", "g_casv", NULL, "1994",
	"Castlevania - The New Generation (E) [!]\0", NULL, "Konami", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_cvaneRomInfo, g_cvaneRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Castlevania - The New Generation (Beta)
static struct BurnRomInfo g_cvtngbRomDesc[] = {
	{ "g_cvtngb.bin", 0x100000, 0x84cd103a, BRF_PRG },
};

STD_ROM_PICK(g_cvtngb);
STD_ROM_FN(g_cvtngb);

struct BurnDriverD BurnDrvMDg_cvtngb = {
	"g_cvtngb", "g_casv", NULL, "1994",
	"Castlevania - The New Generation (Beta)\0", NULL, "Konami", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_cvtngbRomInfo, g_cvtngbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Vampire Killer (J) [!]
static struct BurnRomInfo g_vamkilRomDesc[] = {
	{ "g_vamkil.bin", 0x100000, 0x91b57d2b, BRF_PRG },
};

STD_ROM_PICK(g_vamkil);
STD_ROM_FN(g_vamkil);

struct BurnDriverD BurnDrvMDg_vamkil = {
	"g_vamkil", "g_casv", NULL, "1994",
	"Vampire Killer (J) [!]\0", NULL, "Konami", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_vamkilRomInfo, g_vamkilRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Columns (W) (REV01) [!]
static struct BurnRomInfo g_colRomDesc[] = {
	{ "g_col.bin", 0x20000, 0xd783c244, BRF_PRG },
};

STD_ROM_PICK(g_col);
STD_ROM_FN(g_col);

struct BurnDriverD BurnDrvMDg_col = {
	"g_col", NULL, NULL, "1994",
	"Columns (W) (REV01) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_colRomInfo, g_colRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Columns (W) (REV00) [!]
static struct BurnRomInfo g_col00RomDesc[] = {
	{ "g_col00.bin", 0x20000, 0x03163d7a, BRF_PRG },
};

STD_ROM_PICK(g_col00);
STD_ROM_FN(g_col00);

struct BurnDriverD BurnDrvMDg_col00 = {
	"g_col00", "g_col", NULL, "1994",
	"Columns (W) (REV00) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_col00RomInfo, g_col00RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Columns III - Revenge of Columns (U) [!]
static struct BurnRomInfo g_col3RomDesc[] = {
	{ "g_col3.bin", 0x80000, 0xdc678f6d, BRF_PRG },
};

STD_ROM_PICK(g_col3);
STD_ROM_FN(g_col3);

struct BurnDriverD BurnDrvMDg_col3 = {
	"g_col3", NULL, NULL, "1994",
	"Columns III - Revenge of Columns (U) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_col3RomInfo, g_col3RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Columns III - Taiketsu! Columns World (J) [!]
static struct BurnRomInfo g_col3jRomDesc[] = {
	{ "g_col3j.bin", 0x80000, 0xcd07462f, BRF_PRG },
};

STD_ROM_PICK(g_col3j);
STD_ROM_FN(g_col3j);

struct BurnDriverD BurnDrvMDg_col3j = {
	"g_col3j", "g_col3", NULL, "1994",
	"Columns III - Taiketsu! Columns World (J) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_col3jRomInfo, g_col3jRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Comix Zone (U) [!]
static struct BurnRomInfo g_comxRomDesc[] = {
	{ "g_comx.bin", 0x200000, 0x17da0354, BRF_PRG },
};

STD_ROM_PICK(g_comx);
STD_ROM_FN(g_comx);

struct BurnDriverD BurnDrvMDg_comx = {
	"g_comx", NULL, NULL, "1995",
	"Comix Zone (U) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_comxRomInfo, g_comxRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Comix Zone (E) (M4) [!]
static struct BurnRomInfo g_comixeRomDesc[] = {
	{ "g_comixe.bin", 0x200000, 0x1318e923, BRF_PRG },
};

STD_ROM_PICK(g_comixe);
STD_ROM_FN(g_comixe);

struct BurnDriverD BurnDrvMDg_comixe = {
	"g_comixe", "g_comx", NULL, "1995",
	"Comix Zone (E) (M4) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_comixeRomInfo, g_comixeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Comix Zone (J) [!]
static struct BurnRomInfo g_comixjRomDesc[] = {
	{ "g_comixj.bin", 0x200000, 0x7a6027b8, BRF_PRG },
};

STD_ROM_PICK(g_comixj);
STD_ROM_FN(g_comixj);

struct BurnDriverD BurnDrvMDg_comixj = {
	"g_comixj", "g_comx", NULL, "1995",
	"Comix Zone (J) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_comixjRomInfo, g_comixjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Comix Zone (Beta)
static struct BurnRomInfo g_comixbRomDesc[] = {
	{ "g_comixb.bin", 0x200000, 0x2efcb6ee, BRF_PRG },
};

STD_ROM_PICK(g_comixb);
STD_ROM_FN(g_comixb);

struct BurnDriverD BurnDrvMDg_comixb = {
	"g_comixb", "g_comx", NULL, "1995",
	"Comix Zone (Beta)\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_comixbRomInfo, g_comixbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Contra - Hard Corps (U) [!]
static struct BurnRomInfo g_contRomDesc[] = {
	{ "g_cont.bin", 0x200000, 0xc579f45e, BRF_PRG },
};

STD_ROM_PICK(g_cont);
STD_ROM_FN(g_cont);

struct BurnDriverD BurnDrvMDg_cont = {
	"g_cont", NULL, NULL, "1994",
	"Contra - Hard Corps (U) [!]\0", NULL, "Konami", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_contRomInfo, g_contRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Contra - The Hard Corps (J) [!]
static struct BurnRomInfo g_contrjRomDesc[] = {
	{ "g_contrj.bin", 0x200000, 0x2ab26380, BRF_PRG },
};

STD_ROM_PICK(g_contrj);
STD_ROM_FN(g_contrj);

struct BurnDriverD BurnDrvMDg_contrj = {
	"g_contrj", "g_cont", NULL, "1994",
	"Contra - The Hard Corps (J) [!]\0", NULL, "Konami", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_contrjRomInfo, g_contrjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Probotector (E) [!]
static struct BurnRomInfo g_probotRomDesc[] = {
	{ "g_probot.bin", 0x200000, 0xbc597d48, BRF_PRG },
};

STD_ROM_PICK(g_probot);
STD_ROM_FN(g_probot);

struct BurnDriverD BurnDrvMDg_probot = {
	"g_probot", "g_cont", NULL, "1994",
	"Probotector (E) [!]\0", NULL, "Konami", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_probotRomInfo, g_probotRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// CrossFire (U) [c][!]
static struct BurnRomInfo g_cfirRomDesc[] = {
	{ "g_cfir.bin", 0x80000, 0xcc73f3a9, BRF_PRG },
};

STD_ROM_PICK(g_cfir);
STD_ROM_FN(g_cfir);

struct BurnDriverD BurnDrvMDg_cfir = {
	"g_cfir", NULL, NULL, "1991",
	"CrossFire (U) [c][!]\0", NULL, "Kyugo", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_cfirRomInfo, g_cfirRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Super Airwolf (J) [!]
static struct BurnRomInfo g_sairRomDesc[] = {
	{ "g_sair.bin", 0x80000, 0xfa451982, BRF_PRG },
};

STD_ROM_PICK(g_sair);
STD_ROM_FN(g_sair);

struct BurnDriverD BurnDrvMDg_sair = {
	"g_sair", "g_cfir", NULL, "1991",
	"Super Airwolf (J) [!]\0", NULL, "Kyugo", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sairRomInfo, g_sairRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Crue Ball (UE) [!]
static struct BurnRomInfo g_crueRomDesc[] = {
	{ "g_crue.bin", 0x80000, 0x4b195fc0, BRF_PRG },
};

STD_ROM_PICK(g_crue);
STD_ROM_FN(g_crue);

struct BurnDriverD BurnDrvMDg_crue = {
	"g_crue", NULL, NULL, "1992",
	"Crue Ball (UE) [!]\0", NULL, "Electronic Arts", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_crueRomInfo, g_crueRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Crue Ball (J) [!]
static struct BurnRomInfo g_cruejRomDesc[] = {
	{ "g_cruej.bin", 0x80000, 0x514c53e2, BRF_PRG },
};

STD_ROM_PICK(g_cruej);
STD_ROM_FN(g_cruej);

struct BurnDriverD BurnDrvMDg_cruej = {
	"g_cruej", "g_crue", NULL, "1992",
	"Crue Ball (J) [!]\0", NULL, "Electronic Arts", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_cruejRomInfo, g_cruejRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Twisted Flipper (Beta)
static struct BurnRomInfo g_twisfRomDesc[] = {
	{ "g_twisf.bin", 0x80000, 0x6dd47554, BRF_PRG },
};

STD_ROM_PICK(g_twisf);
STD_ROM_FN(g_twisf);

struct BurnDriverD BurnDrvMDg_twisf = {
	"g_twisf", "g_crue", NULL, "1992",
	"Twisted Flipper (Beta)\0", NULL, "Electronic Arts", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_twisfRomInfo, g_twisfRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Crusader of Centy (U) [!]
static struct BurnRomInfo g_ccRomDesc[] = {
	{ "g_cc.bin", 0x200000, 0x41858f6f, BRF_PRG },
};

STD_ROM_PICK(g_cc);
STD_ROM_FN(g_cc);

struct BurnDriverD BurnDrvMDg_cc = {
	"g_cc", NULL, NULL, "1994",
	"Crusader of Centy (U) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ccRomInfo, g_ccRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Soleil (E) [!]
static struct BurnRomInfo g_soleRomDesc[] = {
	{ "g_sole.bin", 0x200000, 0xa30ebdb1, BRF_PRG },
};

STD_ROM_PICK(g_sole);
STD_ROM_FN(g_sole);

struct BurnDriverD BurnDrvMDg_sole = {
	"g_sole", "g_cc", NULL, "1994",
	"Soleil (E) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_soleRomInfo, g_soleRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Soleil (F) [!]
static struct BurnRomInfo g_soleifRomDesc[] = {
	{ "g_soleif.bin", 0x200000, 0x08dc1ead, BRF_PRG },
};

STD_ROM_PICK(g_soleif);
STD_ROM_FN(g_soleif);

struct BurnDriverD BurnDrvMDg_soleif = {
	"g_soleif", "g_cc", NULL, "1994",
	"Soleil (F) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_soleifRomInfo, g_soleifRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Soleil (G) [!]
static struct BurnRomInfo g_soleigRomDesc[] = {
	{ "g_soleig.bin", 0x200000, 0x332b9ecd, BRF_PRG },
};

STD_ROM_PICK(g_soleig);
STD_ROM_FN(g_soleig);

struct BurnDriverD BurnDrvMDg_soleig = {
	"g_soleig", "g_cc", NULL, "1994",
	"Soleil (G) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_soleigRomInfo, g_soleigRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Soleil (S)
static struct BurnRomInfo g_soleisRomDesc[] = {
	{ "g_soleis.bin", 0x200000, 0x9ed4c323, BRF_PRG },
};

STD_ROM_PICK(g_soleis);
STD_ROM_FN(g_soleis);

struct BurnDriverD BurnDrvMDg_soleis = {
	"g_soleis", "g_cc", NULL, "1994",
	"Soleil (S)\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_soleisRomInfo, g_soleisRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Soleil (Beta)
static struct BurnRomInfo g_soleibRomDesc[] = {
	{ "g_soleib.bin", 0x100000, 0x43797455, BRF_PRG },
};

STD_ROM_PICK(g_soleib);
STD_ROM_FN(g_soleib);

struct BurnDriverD BurnDrvMDg_soleib = {
	"g_soleib", "g_cc", NULL, "1994",
	"Soleil (Beta)\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_soleibRomInfo, g_soleibRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Ragnacenty (J) [!]
static struct BurnRomInfo g_ragnaRomDesc[] = {
	{ "g_ragna.bin", 0x200000, 0x6a3f5ae2, BRF_PRG },
};

STD_ROM_PICK(g_ragna);
STD_ROM_FN(g_ragna);

struct BurnDriverD BurnDrvMDg_ragna = {
	"g_ragna", "g_cc", NULL, "1994",
	"Ragnacenty (J) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ragnaRomInfo, g_ragnaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Shin Changsegi - Ragnacenty (K) [!]
static struct BurnRomInfo g_shinchRomDesc[] = {
	{ "g_shinch.bin", 0x200000, 0x77b5b10b, BRF_PRG },
};

STD_ROM_PICK(g_shinch);
STD_ROM_FN(g_shinch);

struct BurnDriverD BurnDrvMDg_shinch = {
	"g_shinch", "g_cc", NULL, "1994",
	"Shin Changsegi - Ragnacenty (K) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_shinchRomInfo, g_shinchRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Cyber-Cop (U) [c][!]
static struct BurnRomInfo g_ccopRomDesc[] = {
	{ "g_ccop.bin", 0x100000, 0x01e719c8, BRF_PRG },
};

STD_ROM_PICK(g_ccop);
STD_ROM_FN(g_ccop);

struct BurnDriverD BurnDrvMDg_ccop = {
	"g_ccop", NULL, NULL, "1994",
	"Cyber-Cop (U) [c][!]\0", NULL, "Core / Virgin", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ccopRomInfo, g_ccopRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Corporation (E) [c][!]
static struct BurnRomInfo g_corpRomDesc[] = {
	{ "g_corp.bin", 0x100000, 0xa80d18aa, BRF_PRG },
};

STD_ROM_PICK(g_corp);
STD_ROM_FN(g_corp);

struct BurnDriverD BurnDrvMDg_corp = {
	"g_corp", "g_ccop", NULL, "1994",
	"Corporation (E) [c][!]\0", NULL, "Core / Virgin", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_corpRomInfo, g_corpRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Cyborg Justice (W) [!]
static struct BurnRomInfo g_cyboRomDesc[] = {
	{ "g_cybo.bin", 0x80000, 0xab0d1269, BRF_PRG },
};

STD_ROM_PICK(g_cybo);
STD_ROM_FN(g_cybo);

struct BurnDriverD BurnDrvMDg_cybo = {
	"g_cybo", NULL, NULL, "1993",
	"Cyborg Justice (W) [!]\0", NULL, "Sega / Novotrade", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_cyboRomInfo, g_cyboRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Cyborg Justice (Beta)
static struct BurnRomInfo g_cyjusbRomDesc[] = {
	{ "g_cyjusb.bin", 0x80000, 0x91daf11e, BRF_PRG },
};

STD_ROM_PICK(g_cyjusb);
STD_ROM_FN(g_cyjusb);

struct BurnDriverD BurnDrvMDg_cyjusb = {
	"g_cyjusb", "g_cybo", NULL, "1993",
	"Cyborg Justice (Beta)\0", NULL, "Sega / Novotrade", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_cyjusbRomInfo, g_cyjusbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Robot Wreckage (Beta)
static struct BurnRomInfo g_robwrbRomDesc[] = {
	{ "g_robwrb.bin", 0x100000, 0xc67ddb14, BRF_PRG },
};

STD_ROM_PICK(g_robwrb);
STD_ROM_FN(g_robwrb);

struct BurnDriverD BurnDrvMDg_robwrb = {
	"g_robwrb", "g_cybo", NULL, "1993",
	"Robot Wreckage (Beta)\0", NULL, "Sega / Novotrade", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_robwrbRomInfo, g_robwrbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Dino Dini's Soccer (E)
static struct BurnRomInfo g_dinoRomDesc[] = {
	{ "g_dino.bin", 0x100000, 0x4608f53a, BRF_PRG },
};

STD_ROM_PICK(g_dino);
STD_ROM_FN(g_dino);

struct BurnDriverD BurnDrvMDg_dino = {
	"g_dino", NULL, NULL, "1994",
	"Dino Dini's Soccer (E)\0", NULL, "Dino Dini / Virgin", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_dinoRomInfo, g_dinoRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Dragon's Fury (UE) [!]
static struct BurnRomInfo g_dfryRomDesc[] = {
	{ "g_dfry.bin", 0x80000, 0x58037bc6, BRF_PRG },
};

STD_ROM_PICK(g_dfry);
STD_ROM_FN(g_dfry);

struct BurnDriverD BurnDrvMDg_dfry = {
	"g_dfry", NULL, NULL, "1992",
	"Dragon's Fury (UE) [!]\0", NULL, "Technosoft / Tengen", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_dfryRomInfo, g_dfryRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Devil Crash MD (J) [!]
static struct BurnRomInfo g_dcmdRomDesc[] = {
	{ "g_dcmd.bin", 0x80000, 0x4c4caad8, BRF_PRG },
};

STD_ROM_PICK(g_dcmd);
STD_ROM_FN(g_dcmd);

struct BurnDriverD BurnDrvMDg_dcmd = {
	"g_dcmd", "g_dfry", NULL, "1992",
	"Devil Crash MD (J) [!]\0", NULL, "Technosoft", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_dcmdRomInfo, g_dcmdRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Devilish - The Next Possession (U) [c][!]
static struct BurnRomInfo g_deviRomDesc[] = {
	{ "g_devi.bin", 0x80000, 0xd3f300ac, BRF_PRG },
};

STD_ROM_PICK(g_devi);
STD_ROM_FN(g_devi);

struct BurnDriverD BurnDrvMDg_devi = {
	"g_devi", NULL, NULL, "1992",
	"Devilish - The Next Possession (U) [c][!]\0", NULL, "Sage's Creation", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_deviRomInfo, g_deviRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Bad Omen (J) [c][!]
static struct BurnRomInfo g_bdomenRomDesc[] = {
	{ "g_bdomen.bin", 0x80000, 0x975693ce, BRF_PRG },
};

STD_ROM_PICK(g_bdomen);
STD_ROM_FN(g_bdomen);

struct BurnDriverD BurnDrvMDg_bdomen = {
	"g_bdomen", "g_devi", NULL, "1992",
	"Bad Omen (J) [c][!]\0", NULL, "Hot B", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_bdomenRomInfo, g_bdomenRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Dragon Slayer - The Legend of Heroes (J) [!]
static struct BurnRomInfo g_dslayRomDesc[] = {
	{ "g_dslay.bin", 0x200000, 0x01bc1604, BRF_PRG },
};

STD_ROM_PICK(g_dslay);
STD_ROM_FN(g_dslay);

struct BurnDriverD BurnDrvMDg_dslay = {
	"g_dslay", NULL, NULL, "1994",
	"Dragon Slayer - The Legend of Heroes (J) [!]\0", NULL, "Sega / Falcom", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_dslayRomInfo, g_dslayRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Dragon Slayer II - The Legend of Heroes (J) [!]
static struct BurnRomInfo g_dslay2RomDesc[] = {
	{ "g_dslay2.bin", 0x200000, 0x46924dc3, BRF_PRG },
};

STD_ROM_PICK(g_dslay2);
STD_ROM_FN(g_dslay2);

struct BurnDriverD BurnDrvMDg_dslay2 = {
	"g_dslay2", NULL, NULL, "1994",
	"Dragon Slayer II - The Legend of Heroes (J) [!]\0", NULL, "Sega / Falcom", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_dslay2RomInfo, g_dslay2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// ECCO The Dolphin (UE) [!]
static struct BurnRomInfo g_eccoRomDesc[] = {
	{ "g_ecco.bin", 0x100000, 0x45547390, BRF_PRG },
};

STD_ROM_PICK(g_ecco);
STD_ROM_FN(g_ecco);

struct BurnDriverD BurnDrvMDg_ecco = {
	"g_ecco", NULL, NULL, "199?",
	"ECCO The Dolphin (UE) [!]\0", NULL, "Sega / Novotrade", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_eccoRomInfo, g_eccoRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// ECCO The Dolphin (J) [!]
static struct BurnRomInfo g_eccodjRomDesc[] = {
	{ "g_eccodj.bin", 0x100000, 0x6520304d, BRF_PRG },
};

STD_ROM_PICK(g_eccodj);
STD_ROM_FN(g_eccodj);

struct BurnDriverD BurnDrvMDg_eccodj = {
	"g_eccodj", "g_ecco", NULL, "199?",
	"ECCO The Dolphin (J) [!]\0", NULL, "Sega / Novotrade", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_eccodjRomInfo, g_eccodjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// ECCO - The Tides of Time (U) [!]
static struct BurnRomInfo g_eco2RomDesc[] = {
	{ "g_eco2.bin", 0x200000, 0xccb21f98, BRF_PRG },
};

STD_ROM_PICK(g_eco2);
STD_ROM_FN(g_eco2);

struct BurnDriverD BurnDrvMDg_eco2 = {
	"g_eco2", NULL, NULL, "1994",
	"ECCO - The Tides of Time (U) [!]\0", NULL, "Sega / Novotrade", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_eco2RomInfo, g_eco2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// ECCO - The Tides of Time (E) [!]
static struct BurnRomInfo g_ecco2eRomDesc[] = {
	{ "g_ecco2e.bin", 0x200000, 0x7b1bf89c, BRF_PRG },
};

STD_ROM_PICK(g_ecco2e);
STD_ROM_FN(g_ecco2e);

struct BurnDriverD BurnDrvMDg_ecco2e = {
	"g_ecco2e", "g_eco2", NULL, "1994",
	"ECCO - The Tides of Time (E) [!]\0", NULL, "Sega / Novotrade", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ecco2eRomInfo, g_ecco2eRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// ECCO - The Tides of Time (J) [!]
static struct BurnRomInfo g_ecco2jRomDesc[] = {
	{ "g_ecco2j.bin", 0x200000, 0x062d439c, BRF_PRG },
};

STD_ROM_PICK(g_ecco2j);
STD_ROM_FN(g_ecco2j);

struct BurnDriverD BurnDrvMDg_ecco2j = {
	"g_ecco2j", "g_eco2", NULL, "1994",
	"ECCO - The Tides of Time (J) [!]\0", NULL, "Sega / Novotrade", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ecco2jRomInfo, g_ecco2jRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// ECCO - The Tides of Time (Beta)
static struct BurnRomInfo g_ecco2bRomDesc[] = {
	{ "g_ecco2b.bin", 0x200000, 0x1d1470ea, BRF_PRG },
};

STD_ROM_PICK(g_ecco2b);
STD_ROM_FN(g_ecco2b);

struct BurnDriverD BurnDrvMDg_ecco2b = {
	"g_ecco2b", "g_eco2", NULL, "1994",
	"ECCO - The Tides of Time (Beta)\0", NULL, "Sega / Novotrade", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ecco2bRomInfo, g_ecco2bRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// ECCO Jr. (U) (REV00) [!]
static struct BurnRomInfo g_ecjrRomDesc[] = {
	{ "g_ecjr.bin", 0x100000, 0x6c6f4b89, BRF_PRG },
};

STD_ROM_PICK(g_ecjr);
STD_ROM_FN(g_ecjr);

struct BurnDriverD BurnDrvMDg_ecjr = {
	"g_ecjr", NULL, NULL, "1995",
	"ECCO Jr. (U) (REV00) [!]\0", NULL, "Sega / Novotrade", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ecjrRomInfo, g_ecjrRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// ECCO Jr. (U) (REV01) [!]
static struct BurnRomInfo g_ecjr01RomDesc[] = {
	{ "g_ecjr01.bin", 0x100000, 0x3c517975, BRF_PRG },
};

STD_ROM_PICK(g_ecjr01);
STD_ROM_FN(g_ecjr01);

struct BurnDriverD BurnDrvMDg_ecjr01 = {
	"g_ecjr01", "g_ecjr", NULL, "1995",
	"ECCO Jr. (U) (REV01) [!]\0", NULL, "Sega / Novotrade", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ecjr01RomInfo, g_ecjr01RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Ex-Mutants (U) [!]
static struct BurnRomInfo g_exmuRomDesc[] = {
	{ "g_exmu.bin", 0x100000, 0x33b1979f, BRF_PRG },
};

STD_ROM_PICK(g_exmu);
STD_ROM_FN(g_exmu);

struct BurnDriverD BurnDrvMDg_exmu = {
	"g_exmu", NULL, NULL, "1992",
	"Ex-Mutants (U) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_exmuRomInfo, g_exmuRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// F-22 Interceptor (U) (Jun 1992) [c][!]
static struct BurnRomInfo g_f22iRomDesc[] = {
	{ "g_f22i.bin", 0xc0000, 0xdd19b2b3, BRF_PRG },
};

STD_ROM_PICK(g_f22i);
STD_ROM_FN(g_f22i);

struct BurnDriverD BurnDrvMDg_f22i = {
	"g_f22i", NULL, NULL, "1991",
	"F-22 Interceptor (U) (Jun 1992) [c][!]\0", NULL, "Electronic Arts / Edward Lemer", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_f22iRomInfo, g_f22iRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// F-22 Interceptor (U) (Sep 1991) [c][!]
static struct BurnRomInfo g_f22uRomDesc[] = {
	{ "g_f22u.bin", 0xc0000, 0x9cf552c2, BRF_PRG },
};

STD_ROM_PICK(g_f22u);
STD_ROM_FN(g_f22u);

struct BurnDriverD BurnDrvMDg_f22u = {
	"g_f22u", "g_f22i", NULL, "1991",
	"F-22 Interceptor (U) (Sep 1991) [c][!]\0", NULL, "Electronic Arts / Edward Lemer", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_f22uRomInfo, g_f22uRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// F-22 Interceptor (U) (Sep 1991) [a1][c][!]
static struct BurnRomInfo g_f22uaRomDesc[] = {
	{ "g_f22ua.bin", 0xc0000, 0x31e9d1a5, BRF_PRG },
};

STD_ROM_PICK(g_f22ua);
STD_ROM_FN(g_f22ua);

struct BurnDriverD BurnDrvMDg_f22ua = {
	"g_f22ua", "g_f22i", NULL, "1991",
	"F-22 Interceptor (U) (Sep 1991) [a1][c][!]\0", NULL, "Electronic Arts / Edward Lemer", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_f22uaRomInfo, g_f22uaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// F-22 Interceptor - Advanced Tactical Fighter (J) [!]
static struct BurnRomInfo g_f22jRomDesc[] = {
	{ "g_f22j.bin", 0xc0000, 0xfb55c785, BRF_PRG },
};

STD_ROM_PICK(g_f22j);
STD_ROM_FN(g_f22j);

struct BurnDriverD BurnDrvMDg_f22j = {
	"g_f22j", "g_f22i", NULL, "1991",
	"F-22 Interceptor - Advanced Tactical Fighter (J) [!]\0", NULL, "Electronic Arts / Edward Lemer", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_f22jRomInfo, g_f22jRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// F-22 Interceptor (Beta) [c][!]
static struct BurnRomInfo g_f22bRomDesc[] = {
	{ "g_f22b.bin", 0xc0000, 0xd6a880a4, BRF_PRG },
};

STD_ROM_PICK(g_f22b);
STD_ROM_FN(g_f22b);

struct BurnDriverD BurnDrvMDg_f22b = {
	"g_f22b", "g_f22i", NULL, "1991",
	"F-22 Interceptor (Beta) [c][!]\0", NULL, "Electronic Arts / Edward Lemer", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_f22bRomInfo, g_f22bRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Fatal Fury (U) [!]
static struct BurnRomInfo g_fafRomDesc[] = {
	{ "g_faf.bin", 0x180000, 0x98d49170, BRF_PRG },
};

STD_ROM_PICK(g_faf);
STD_ROM_FN(g_faf);

struct BurnDriverD BurnDrvMDg_faf = {
	"g_faf", NULL, NULL, "1993",
	"Fatal Fury (U) [!]\0", NULL, "SNK / Takara", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_fafRomInfo, g_fafRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Fatal Fury (E)
static struct BurnRomInfo g_fatfueRomDesc[] = {
	{ "g_fatfue.bin", 0x180000, 0x2e730a91, BRF_PRG },
};

STD_ROM_PICK(g_fatfue);
STD_ROM_FN(g_fatfue);

struct BurnDriverD BurnDrvMDg_fatfue = {
	"g_fatfue", "g_faf", NULL, "1993",
	"Fatal Fury (E)\0", NULL, "SNK / Takara", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_fatfueRomInfo, g_fatfueRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Garou Densetsu - Shukumei no Tatakai (J) [!]
static struct BurnRomInfo g_garouRomDesc[] = {
	{ "g_garou.bin", 0x180000, 0xbf3e3fa4, BRF_PRG },
};

STD_ROM_PICK(g_garou);
STD_ROM_FN(g_garou);

struct BurnDriverD BurnDrvMDg_garou = {
	"g_garou", "g_faf", NULL, "1993",
	"Garou Densetsu - Shukumei no Tatakai (J) [!]\0", NULL, "SNK / Takara", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_garouRomInfo, g_garouRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Fatal Fury 2 (U) [c][!]
static struct BurnRomInfo g_faf2RomDesc[] = {
	{ "g_faf2.bin", 0x300000, 0x1b1754cb, BRF_PRG },
};

STD_ROM_PICK(g_faf2);
STD_ROM_FN(g_faf2);

struct BurnDriverD BurnDrvMDg_faf2 = {
	"g_faf2", NULL, NULL, "1994",
	"Fatal Fury 2 (U) [c][!]\0", NULL, "SNK / Takara", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_faf2RomInfo, g_faf2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Garou Densetsu 2 - Arata-naru Tatakai (J) [c][!]
static struct BurnRomInfo g_garou2RomDesc[] = {
	{ "g_garou2.bin", 0x300000, 0x2af4427f, BRF_PRG },
};

STD_ROM_PICK(g_garou2);
STD_ROM_FN(g_garou2);

struct BurnDriverD BurnDrvMDg_garou2 = {
	"g_garou2", "g_faf2", NULL, "1994",
	"Garou Densetsu 2 - Arata-naru Tatakai (J) [c][!]\0", NULL, "SNK / Takara", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_garou2RomInfo, g_garou2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Fatal Rewind (UE) [!]
static struct BurnRomInfo g_fatrRomDesc[] = {
	{ "g_fatr.bin", 0x80000, 0xe91aed05, BRF_PRG },
};

STD_ROM_PICK(g_fatr);
STD_ROM_FN(g_fatr);

struct BurnDriverD BurnDrvMDg_fatr = {
	"g_fatr", NULL, NULL, "1991",
	"Fatal Rewind (UE) [!]\0", NULL, "Psygnosis", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_fatrRomInfo, g_fatrRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Killing Game Show, The (J)
static struct BurnRomInfo g_kgsRomDesc[] = {
	{ "g_kgs.bin", 0x80000, 0x21dbb69d, BRF_PRG },
};

STD_ROM_PICK(g_kgs);
STD_ROM_FN(g_kgs);

struct BurnDriverD BurnDrvMDg_kgs = {
	"g_kgs", "g_fatr", NULL, "1991",
	"Killing Game Show, The (J)\0", NULL, "Psygnosis", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_kgsRomInfo, g_kgsRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Killing Game Show, The - 1993 Remix
static struct BurnRomInfo g_kgsrRomDesc[] = {
	{ "g_kgsr.bin", 0x100000, 0xb8e7668a, BRF_PRG },
};

STD_ROM_PICK(g_kgsr);
STD_ROM_FN(g_kgsr);

struct BurnDriverD BurnDrvMDg_kgsr = {
	"g_kgsr", "g_fatr", NULL, "1993",
	"Killing Game Show, The - 1993 Remix\0", NULL, "Psygnosis", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_kgsrRomInfo, g_kgsrRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Flashback - The Quest for Identity (U) [!]
static struct BurnRomInfo g_fbakRomDesc[] = {
	{ "g_fbak.bin", 0x180000, 0x23a9616d, BRF_PRG },
};

STD_ROM_PICK(g_fbak);
STD_ROM_FN(g_fbak);

struct BurnDriver BurnDrvMDg_fbak = {
	"g_fbak", NULL, NULL, "1993",
	"Flashback - The Quest for Identity (U) [!]\0", NULL, "Delphine / US Gold", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_fbakRomInfo, g_fbakRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Flashback - The Quest for Identity (U) [a1][!]
static struct BurnRomInfo g_fbckuaRomDesc[] = {
	{ "g_fbckua.bin", 0x180000, 0x33cd2b65, BRF_PRG },
};

STD_ROM_PICK(g_fbckua);
STD_ROM_FN(g_fbckua);

struct BurnDriver BurnDrvMDg_fbckua = {
	"g_fbckua", "g_fbak", NULL, "1993",
	"Flashback - The Quest for Identity (U) [a1][!]\0", NULL, "Delphine / US Gold", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_fbckuaRomInfo, g_fbckuaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Flashback - The Quest for Identity (J) [!]
static struct BurnRomInfo g_fbckjRomDesc[] = {
	{ "g_fbckj.bin", 0x180000, 0xb790e3b4, BRF_PRG },
};

STD_ROM_PICK(g_fbckj);
STD_ROM_FN(g_fbckj);

struct BurnDriver BurnDrvMDg_fbckj = {
	"g_fbckj", "g_fbak", NULL, "1993",
	"Flashback - The Quest for Identity (J) [!]\0", NULL, "Delphine / US Gold", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_fbckjRomInfo, g_fbckjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Flashback - The Quest for Identity (E) [!]
static struct BurnRomInfo g_fbckeRomDesc[] = {
	{ "g_fbcke.bin", 0x180000, 0x6f311c83, BRF_PRG },
};

STD_ROM_PICK(g_fbcke);
STD_ROM_FN(g_fbcke);

struct BurnDriver BurnDrvMDg_fbcke = {
	"g_fbcke", "g_fbak", NULL, "1993",
	"Flashback - The Quest for Identity (E) [!]\0", NULL, "Delphine / US Gold", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_fbckeRomInfo, g_fbckeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Gauntlet 4 (UE) (Aug 1993) [!]
static struct BurnRomInfo g_gau4RomDesc[] = {
	{ "g_gau4.bin", 0x100000, 0x3bf46dce, BRF_PRG },
};

STD_ROM_PICK(g_gau4);
STD_ROM_FN(g_gau4);

struct BurnDriverD BurnDrvMDg_gau4 = {
	"g_gau4", NULL, NULL, "1994",
	"Gauntlet 4 (UE) (Aug 1993) [!]\0", NULL, "Tengen / Atari", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_gau4RomInfo, g_gau4RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Gauntlet 4 (UE) (Sep 1993)
static struct BurnRomInfo g_gaun4aRomDesc[] = {
	{ "g_gaun4a.bin", 0x100000, 0xf9d60510, BRF_PRG },
};

STD_ROM_PICK(g_gaun4a);
STD_ROM_FN(g_gaun4a);

struct BurnDriverD BurnDrvMDg_gaun4a = {
	"g_gaun4a", "g_gau4", NULL, "1994",
	"Gauntlet 4 (UE) (Sep 1993)\0", NULL, "Tengen / Atari", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_gaun4aRomInfo, g_gaun4aRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Gauntlet (J) [!]
static struct BurnRomInfo g_gauntRomDesc[] = {
	{ "g_gaunt.bin", 0x100000, 0xf9872055, BRF_PRG },
};

STD_ROM_PICK(g_gaunt);
STD_ROM_FN(g_gaunt);

struct BurnDriverD BurnDrvMDg_gaunt = {
	"g_gaunt", "g_gau4", NULL, "1994",
	"Gauntlet (J) [!]\0", NULL, "Tengen / Atari", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_gauntRomInfo, g_gauntRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Gargoyles (U) [!]
static struct BurnRomInfo g_gargRomDesc[] = {
	{ "g_garg.bin", 0x300000, 0x2d965364, BRF_PRG },
};

STD_ROM_PICK(g_garg);
STD_ROM_FN(g_garg);

struct BurnDriverD BurnDrvMDg_garg = {
	"g_garg", NULL, NULL, "1995",
	"Gargoyles (U) [!]\0", NULL, "Buena Vista Interactive", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_gargRomInfo, g_gargRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Ghouls 'N Ghosts (UE) (REV02) [!]
static struct BurnRomInfo g_gngRomDesc[] = {
	{ "g_gng.bin", 0xa0000, 0x4f2561d5, BRF_PRG },
};

STD_ROM_PICK(g_gng);
STD_ROM_FN(g_gng);

struct BurnDriverD BurnDrvMDg_gng = {
	"g_gng", NULL, NULL, "1989",
	"Ghouls 'N Ghosts (UE) (REV02) [!]\0", NULL, "Capcom / Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_gngRomInfo, g_gngRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Ghouls 'N Ghosts (UE) (REV01) [!]
static struct BurnRomInfo g_gng01RomDesc[] = {
	{ "g_gng01.bin", 0xa0000, 0xd31bd910, BRF_PRG },
};

STD_ROM_PICK(g_gng01);
STD_ROM_FN(g_gng01);

struct BurnDriverD BurnDrvMDg_gng01 = {
	"g_gng01", "g_gng", NULL, "1989",
	"Ghouls 'N Ghosts (UE) (REV01) [!]\0", NULL, "Capcom / Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_gng01RomInfo, g_gng01RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Dai Makaimura (J) [!]
static struct BurnRomInfo g_daimRomDesc[] = {
	{ "g_daim.bin", 0xa0000, 0x5659f379, BRF_PRG },
};

STD_ROM_PICK(g_daim);
STD_ROM_FN(g_daim);

struct BurnDriverD BurnDrvMDg_daim = {
	"g_daim", "g_gng", NULL, "1989",
	"Dai Makaimura (J) [!]\0", NULL, "Capcom / Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_daimRomInfo, g_daimRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Great Circus Mystery, The - Starring Mickey & Minnie (U) [!]
static struct BurnRomInfo g_gcmRomDesc[] = {
	{ "g_gcm.bin", 0x200000, 0x14744883, BRF_PRG },
};

STD_ROM_PICK(g_gcm);
STD_ROM_FN(g_gcm);

struct BurnDriverD BurnDrvMDg_gcm = {
	"g_gcm", NULL, NULL, "1994",
	"Great Circus Mystery, The - Starring Mickey & Minnie (U) [!]\0", NULL, "Disney / Capcom", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_gcmRomInfo, g_gcmRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mickey Mouse - Minnie's Magical Adventure 2 (J) [!]
static struct BurnRomInfo g_minnieRomDesc[] = {
	{ "g_minnie.bin", 0x200000, 0x5aa0f3a0, BRF_PRG },
};

STD_ROM_PICK(g_minnie);
STD_ROM_FN(g_minnie);

struct BurnDriverD BurnDrvMDg_minnie = {
	"g_minnie", "g_gcm", NULL, "1994",
	"Mickey Mouse - Minnie's Magical Adventure 2 (J) [!]\0", NULL, "Disney / Capcom", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_minnieRomInfo, g_minnieRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Gunstar Heroes (U) [!]
static struct BurnRomInfo g_gshRomDesc[] = {
	{ "g_gsh.bin", 0x100000, 0xb813cf0d, BRF_PRG },
};

STD_ROM_PICK(g_gsh);
STD_ROM_FN(g_gsh);

struct BurnDriverD BurnDrvMDg_gsh = {
	"g_gsh", NULL, NULL, "1993",
	"Gunstar Heroes (U) [!]\0", NULL, "Sega / Treasure", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_gshRomInfo, g_gshRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Gunstar Heroes (E) [!]
static struct BurnRomInfo g_gsheRomDesc[] = {
	{ "g_gshe.bin", 0x100000, 0x866ed9d0, BRF_PRG },
};

STD_ROM_PICK(g_gshe);
STD_ROM_FN(g_gshe);

struct BurnDriverD BurnDrvMDg_gshe = {
	"g_gshe", "g_gsh", NULL, "1993",
	"Gunstar Heroes (E) [!]\0", NULL, "Sega / Treasure", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_gsheRomInfo, g_gsheRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Gunstar Heroes (J) [!]]
static struct BurnRomInfo g_gshjRomDesc[] = {
	{ "g_gshj.bin", 0x100000, 0x1cfd0383, BRF_PRG },
};

STD_ROM_PICK(g_gshj);
STD_ROM_FN(g_gshj);

struct BurnDriverD BurnDrvMDg_gshj = {
	"g_gshj", "g_gsh", NULL, "1993",
	"Gunstar Heroes (J) [!]]\0", NULL, "Sega / Treasure", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_gshjRomInfo, g_gshjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Gunstar Heroes (Sample) (J) [!]
static struct BurnRomInfo g_gshsamRomDesc[] = {
	{ "g_gshsam.bin", 0x100000, 0x6f90b502, BRF_PRG },
};

STD_ROM_PICK(g_gshsam);
STD_ROM_FN(g_gshsam);

struct BurnDriverD BurnDrvMDg_gshsam = {
	"g_gshsam", "g_gsh", NULL, "1993",
	"Gunstar Heroes (Sample) (J) [!]\0", NULL, "Sega / Treasure", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_gshsamRomInfo, g_gshsamRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Hard Drivin' (W) [!]
static struct BurnRomInfo g_hardRomDesc[] = {
	{ "g_hard.bin", 0x40000, 0x3225baaf, BRF_PRG },
};

STD_ROM_PICK(g_hard);
STD_ROM_FN(g_hard);

struct BurnDriverD BurnDrvMDg_hard = {
	"g_hard", NULL, NULL, "1990",
	"Hard Drivin' (W) [!]\0", NULL, "Tengen / Atari", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_hardRomInfo, g_hardRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// High Seas Havoc (U) [!]
static struct BurnRomInfo g_hshRomDesc[] = {
	{ "g_hsh.bin", 0x100000, 0x17be551c, BRF_PRG },
};

STD_ROM_PICK(g_hsh);
STD_ROM_FN(g_hsh);

struct BurnDriverD BurnDrvMDg_hsh = {
	"g_hsh", NULL, NULL, "1993",
	"High Seas Havoc (U) [!]\0", NULL, "Data East", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_hshRomInfo, g_hshRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Capt'n Havoc (E) [!]
static struct BurnRomInfo g_capthaRomDesc[] = {
	{ "g_captha.bin", 0x100000, 0x76e6d20d, BRF_PRG },
};

STD_ROM_PICK(g_captha);
STD_ROM_FN(g_captha);

struct BurnDriverD BurnDrvMDg_captha = {
	"g_captha", "g_hsh", NULL, "1994",
	"Capt'n Havoc (E) [!]\0", NULL, "Data East / Codemasters", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_capthaRomInfo, g_capthaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// International Superstar Soccer Deluxe (E) [!]
static struct BurnRomInfo g_issRomDesc[] = {
	{ "g_iss.bin", 0x200000, 0x9bb3b180, BRF_PRG },
};

STD_ROM_PICK(g_iss);
STD_ROM_FN(g_iss);

struct BurnDriverD BurnDrvMDg_iss = {
	"g_iss", NULL, NULL, "1996",
	"International Superstar Soccer Deluxe (E) [!]\0", NULL, "Konami / Factor 5", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_issRomInfo, g_issRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Ronaldinho 98 (B) [c][!]
static struct BurnRomInfo g_ron98bRomDesc[] = {
	{ "g_ron98b.bin", 0x200000, 0xdd27c84e, BRF_PRG },
};

STD_ROM_PICK(g_ron98b);
STD_ROM_FN(g_ron98b);

struct BurnDriverD BurnDrvMDg_ron98b = {
	"g_ron98b", "g_iss", NULL, "1998",
	"Ronaldinho 98 (B) [c][!]\0", NULL, "bootleg", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ron98bRomInfo, g_ron98bRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// James Bond 007 - The Duel (UE) [!]
static struct BurnRomInfo g_bondRomDesc[] = {
	{ "g_bond.bin", 0x80000, 0x291a3e4b, BRF_PRG },
};

STD_ROM_PICK(g_bond);
STD_ROM_FN(g_bond);

struct BurnDriver BurnDrvMDg_bond = {
	"g_bond", NULL, NULL, "1993",
	"James Bond 007 - The Duel (UE) [!]\0", NULL, "Domark", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_bondRomInfo, g_bondRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// James Bond 007 - The Duel (W)
static struct BurnRomInfo g_jbondwRomDesc[] = {
	{ "g_jbondw.bin", 0x80000, 0x4e614548, BRF_PRG },
};

STD_ROM_PICK(g_jbondw);
STD_ROM_FN(g_jbondw);

struct BurnDriver BurnDrvMDg_jbondw = {
	"g_jbondw", "g_bond", NULL, "1993",
	"James Bond 007 - The Duel (W)\0", NULL, "Domark", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jbondwRomInfo, g_jbondwRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// 007 Shitou (J) [!]
static struct BurnRomInfo g_007RomDesc[] = {
	{ "g_007.bin", 0x80000, 0xaeb4b262, BRF_PRG },
};

STD_ROM_PICK(g_007);
STD_ROM_FN(g_007);

struct BurnDriver BurnDrvMDg_007 = {
	"g_007", "g_bond", NULL, "1993",
	"007 Shitou (J) [!]\0", NULL, "Domark", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_007RomInfo, g_007RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// James Pond - Underwater Agent (UE) [!]
static struct BurnRomInfo g_jpRomDesc[] = {
	{ "g_jp.bin", 0x80000, 0xd0e7b466, BRF_PRG },
};

STD_ROM_PICK(g_jp);
STD_ROM_FN(g_jp);

struct BurnDriverD BurnDrvMDg_jp = {
	"g_jp", NULL, NULL, "1991",
	"James Pond - Underwater Agent (UE) [!]\0", NULL, "Electronic Arts / Millenium", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jpRomInfo, g_jpRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// James Pond II - Codename RoboCod (UE) [!]
static struct BurnRomInfo g_jp2RomDesc[] = {
	{ "pond ii robocod rob02", 0x80000, 0xc32b5d66, BRF_PRG },
};

STD_ROM_PICK(g_jp2);
STD_ROM_FN(g_jp2);

struct BurnDriver BurnDrvMDg_jp2 = {
	"g_jp2", NULL, NULL, "1991",
	"James Pond II - Codename RoboCod (UE) [!]\0", NULL, "Electronic Arts / Millenium", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jp2RomInfo, g_jp2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveNoByteswapInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// James Pond II - Codename RoboCod (J) [!]
static struct BurnRomInfo g_jp2jRomDesc[] = {
	{ "g_jp2j.bin", 0x80000, 0x98794702, BRF_PRG },
};

STD_ROM_PICK(g_jp2j);
STD_ROM_FN(g_jp2j);

struct BurnDriver BurnDrvMDg_jp2j = {
	"g_jp2j", "g_jp2", NULL, "1991",
	"James Pond II - Codename RoboCod (J) [!]\0", NULL, "Electronic Arts / Millenium", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jp2jRomInfo, g_jp2jRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Aquatic Games - Starring James Pond, The (UE) [!]
static struct BurnRomInfo g_aquaRomDesc[] = {
	{ "g_aqua.bin", 0x80000, 0x400f4ba7, BRF_PRG },
};

STD_ROM_PICK(g_aqua);
STD_ROM_FN(g_aqua);

struct BurnDriver BurnDrvMDg_aqua = {
	"g_aqua", NULL, NULL, "1992",
	"Aquatic Games - Starring James Pond, The (UE) [!]\0", NULL, "Electronic Arts / Vectordean / Millenium", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_aquaRomInfo, g_aquaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// James Pond 3
static struct BurnRomInfo g_jp3RomDesc[] = {
	{ "g_jp3.bin", 0x200000, 0x26f64b2a, BRF_PRG },
};

STD_ROM_PICK(g_jp3);
STD_ROM_FN(g_jp3);

struct BurnDriver BurnDrvMDg_jp3 = {
	"g_jp3", NULL, NULL, "1993",
	"James Pond 3\0", NULL, "Electronic Arts / Vectordean / Millenium", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jp3RomInfo, g_jp3RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Judge Dredd - The Movie (W) [!]
static struct BurnRomInfo g_jdreRomDesc[] = {
	{ "g_jdre.bin", 0x200000, 0xea342ed8, BRF_PRG },
};

STD_ROM_PICK(g_jdre);
STD_ROM_FN(g_jdre);

struct BurnDriverD BurnDrvMDg_jdre = {
	"g_jdre", NULL, NULL, "1995",
	"Judge Dredd - The Movie (W) [!]\0", NULL, "Acclaim / Probe", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jdreRomInfo, g_jdreRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Judge Dredd - The Movie (Beta)
static struct BurnRomInfo g_jdrdbRomDesc[] = {
	{ "g_jdrdb.bin", 0x200000, 0x8d46f4da, BRF_PRG },
};

STD_ROM_PICK(g_jdrdb);
STD_ROM_FN(g_jdrdb);

struct BurnDriverD BurnDrvMDg_jdrdb = {
	"g_jdrdb", "g_jdre", NULL, "1995",
	"Judge Dredd - The Movie (Beta)\0", NULL, "Acclaim / Probe", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jdrdbRomInfo, g_jdrdbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Judge Dredd - The Movie (Beta) [a1]
static struct BurnRomInfo g_jdrdbaRomDesc[] = {
	{ "g_jdrdba.bin", 0x200000, 0xe649f784, BRF_PRG },
};

STD_ROM_PICK(g_jdrdba);
STD_ROM_FN(g_jdrdba);

struct BurnDriverD BurnDrvMDg_jdrdba = {
	"g_jdrdba", "g_jdre", NULL, "1995",
	"Judge Dredd - The Movie (Beta) [a1]\0", NULL, "Acclaim / Probe", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jdrdbaRomInfo, g_jdrdbaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Kid Chameleon (UE) [!]
static struct BurnRomInfo g_kidcRomDesc[] = {
	{ "g_kidc.bin", 0x100000, 0xce36e6cc, BRF_PRG },
};

STD_ROM_PICK(g_kidc);
STD_ROM_FN(g_kidc);

struct BurnDriverD BurnDrvMDg_kidc = {
	"g_kidc", NULL, NULL, "1994",
	"Kid Chameleon (UE) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_kidcRomInfo, g_kidcRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Chameleon Kid (J) [!]
static struct BurnRomInfo g_ckidRomDesc[] = {
	{ "g_ckid.bin", 0x100000, 0x50217c80, BRF_PRG },
};

STD_ROM_PICK(g_ckid);
STD_ROM_FN(g_ckid);

struct BurnDriverD BurnDrvMDg_ckid = {
	"g_ckid", "g_kidc", NULL, "1994",
	"Chameleon Kid (J) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ckidRomInfo, g_ckidRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Landstalker - The Treasures of King Nole (U) [!]
static struct BurnRomInfo g_landRomDesc[] = {
	{ "g_land.bin", 0x200000, 0xfbbb5b97, BRF_PRG },
};

STD_ROM_PICK(g_land);
STD_ROM_FN(g_land);

struct BurnDriverD BurnDrvMDg_land = {
	"g_land", NULL, NULL, "1994",
	"Landstalker - The Treasures of King Nole (U) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_landRomInfo, g_landRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Landstalker - The Treasures of King Nole (E) [!]
static struct BurnRomInfo g_landseRomDesc[] = {
	{ "g_landse.bin", 0x200000, 0xe3c65277, BRF_PRG },
};

STD_ROM_PICK(g_landse);
STD_ROM_FN(g_landse);

struct BurnDriverD BurnDrvMDg_landse = {
	"g_landse", "g_land", NULL, "1994",
	"Landstalker - The Treasures of King Nole (E) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_landseRomInfo, g_landseRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Landstalker - The Treasures of King Nole (G) [!]
static struct BurnRomInfo g_landsgRomDesc[] = {
	{ "g_landsg.bin", 0x200000, 0x10fedb8f, BRF_PRG },
};

STD_ROM_PICK(g_landsg);
STD_ROM_FN(g_landsg);

struct BurnDriverD BurnDrvMDg_landsg = {
	"g_landsg", "g_land", NULL, "1994",
	"Landstalker - The Treasures of King Nole (G) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_landsgRomInfo, g_landsgRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Landstalker - The Treasures of King Nole (F)
static struct BurnRomInfo g_landsfRomDesc[] = {
	{ "g_landsf.bin", 0x200000, 0x5de7d917, BRF_PRG },
};

STD_ROM_PICK(g_landsf);
STD_ROM_FN(g_landsf);

struct BurnDriverD BurnDrvMDg_landsf = {
	"g_landsf", "g_land", NULL, "1994",
	"Landstalker - The Treasures of King Nole (F)\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_landsfRomInfo, g_landsfRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Landstalker - Koutei no Zaihou (J) [!]
static struct BurnRomInfo g_landsjRomDesc[] = {
	{ "g_landsj.bin", 0x200000, 0x60d4cedb, BRF_PRG },
};

STD_ROM_PICK(g_landsj);
STD_ROM_FN(g_landsj);

struct BurnDriverD BurnDrvMDg_landsj = {
	"g_landsj", "g_land", NULL, "1994",
	"Landstalker - Koutei no Zaihou (J) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_landsjRomInfo, g_landsjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Landstalker - The Treasures of King Nole (Beta)
static struct BurnRomInfo g_landsbRomDesc[] = {
	{ "g_landsb.bin", 0x200000, 0x70483d03, BRF_PRG },
};

STD_ROM_PICK(g_landsb);
STD_ROM_FN(g_landsb);

struct BurnDriverD BurnDrvMDg_landsb = {
	"g_landsb", "g_land", NULL, "1994",
	"Landstalker - The Treasures of King Nole (Beta)\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_landsbRomInfo, g_landsbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// OutRun 2019 (U) [!]
static struct BurnRomInfo g_or20RomDesc[] = {
	{ "g_or20.bin", 0x100000, 0xe32e17e2, BRF_PRG },
};

STD_ROM_PICK(g_or20);
STD_ROM_FN(g_or20);

struct BurnDriverD BurnDrvMDg_or20 = {
	"g_or20", NULL, NULL, "1993",
	"OutRun 2019 (U) [!]\0", NULL, "Sims", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_or20RomInfo, g_or20RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// OutRun 2019 (E) [!]
static struct BurnRomInfo g_2019eRomDesc[] = {
	{ "g_2019e.bin", 0x100000, 0x5cb3536a, BRF_PRG },
};

STD_ROM_PICK(g_2019e);
STD_ROM_FN(g_2019e);

struct BurnDriverD BurnDrvMDg_2019e = {
	"g_2019e", "g_or20", NULL, "1993",
	"OutRun 2019 (E) [!]\0", NULL, "Sims", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_2019eRomInfo, g_2019eRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// OutRun 2019 (J) [!]
static struct BurnRomInfo g_2019jRomDesc[] = {
	{ "g_2019j.bin", 0x100000, 0x0eac7440, BRF_PRG },
};

STD_ROM_PICK(g_2019j);
STD_ROM_FN(g_2019j);

struct BurnDriverD BurnDrvMDg_2019j = {
	"g_2019j", "g_or20", NULL, "1993",
	"OutRun 2019 (J) [!]\0", NULL, "Sims", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_2019jRomInfo, g_2019jRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// OutRun 2019 (Beta)
static struct BurnRomInfo g_2019bRomDesc[] = {
	{ "g_2019b.bin", 0x100000, 0xd2ecddfa, BRF_PRG },
};

STD_ROM_PICK(g_2019b);
STD_ROM_FN(g_2019b);

struct BurnDriverD BurnDrvMDg_2019b = {
	"g_2019b", "g_or20", NULL, "1993",
	"OutRun 2019 (Beta)\0", NULL, "Sims", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_2019bRomInfo, g_2019bRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Junkers High (Beta)
static struct BurnRomInfo g_junkbRomDesc[] = {
	{ "g_junkb.bin", 0x100000, 0x23534949, BRF_PRG },
};

STD_ROM_PICK(g_junkb);
STD_ROM_FN(g_junkb);

struct BurnDriverD BurnDrvMDg_junkb = {
	"g_junkb", "g_or20", NULL, "1992",
	"Junkers High (Beta)\0", NULL, "Sims", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_junkbRomInfo, g_junkbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// OutRunners (U) [!]
static struct BurnRomInfo g_orurRomDesc[] = {
	{ "g_orur.bin", 0x200000, 0xede636b9, BRF_PRG },
};

STD_ROM_PICK(g_orur);
STD_ROM_FN(g_orur);

struct BurnDriverD BurnDrvMDg_orur = {
	"g_orur", NULL, NULL, "199?",
	"OutRunners (U) [!]\0", NULL, "Sega / Data East", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_orurRomInfo, g_orurRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// OutRunners (J) [!]
static struct BurnRomInfo g_orunrjRomDesc[] = {
	{ "g_orunrj.bin", 0x200000, 0xe164a09f, BRF_PRG },
};

STD_ROM_PICK(g_orunrj);
STD_ROM_FN(g_orunrj);

struct BurnDriverD BurnDrvMDg_orunrj = {
	"g_orunrj", "g_orur", NULL, "199?",
	"OutRunners (J) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_orunrjRomInfo, g_orunrjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Lemmings (JU) (REV01) [!]
static struct BurnRomInfo g_lemRomDesc[] = {
	{ "g_lem.bin", 0x100000, 0x68c70362, BRF_PRG },
};

STD_ROM_PICK(g_lem);
STD_ROM_FN(g_lem);

struct BurnDriverD BurnDrvMDg_lem = {
	"g_lem", NULL, NULL, "1992",
	"Lemmings (JU) (REV01) [!]\0", NULL, "Psygnosis / Sunsoft", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_lemRomInfo, g_lemRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Lemmings (JU) (REV00) [!]
static struct BurnRomInfo g_lem00RomDesc[] = {
	{ "g_lem00.bin", 0x100000, 0xf015c2ad, BRF_PRG },
};

STD_ROM_PICK(g_lem00);
STD_ROM_FN(g_lem00);

struct BurnDriverD BurnDrvMDg_lem00 = {
	"g_lem00", "g_lem", NULL, "1992",
	"Lemmings (JU) (REV00) [!]\0", NULL, "Psygnosis / Sunsoft", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_lem00RomInfo, g_lem00RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Lemmings (E) [!]
static struct BurnRomInfo g_lemeRomDesc[] = {
	{ "g_leme.bin", 0x100000, 0x6a1a4579, BRF_PRG },
};

STD_ROM_PICK(g_leme);
STD_ROM_FN(g_leme);

struct BurnDriverD BurnDrvMDg_leme = {
	"g_leme", "g_lem", NULL, "1992",
	"Lemmings (E) [!]\0", NULL, "Psygnosis / Sunsoft", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_lemeRomInfo, g_lemeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mazin Saga (U) [!]
static struct BurnRomInfo g_maziRomDesc[] = {
	{ "g_mazi.bin", 0x100000, 0x1bd9fef1, BRF_PRG },
};

STD_ROM_PICK(g_mazi);
STD_ROM_FN(g_mazi);

struct BurnDriverD BurnDrvMDg_mazi = {
	"g_mazi", NULL, NULL, "1993",
	"Mazin Saga (U) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_maziRomInfo, g_maziRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mazin Wars (E) [!]
static struct BurnRomInfo g_mazieRomDesc[] = {
	{ "g_mazie.bin", 0x100000, 0x4b07a105, BRF_PRG },
};

STD_ROM_PICK(g_mazie);
STD_ROM_FN(g_mazie);

struct BurnDriverD BurnDrvMDg_mazie = {
	"g_mazie", "g_mazi", NULL, "1993",
	"Mazin Wars (E) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mazieRomInfo, g_mazieRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mazin Saga (J) [!]
static struct BurnRomInfo g_mazijRomDesc[] = {
	{ "g_mazij.bin", 0x100000, 0x45b3a34b, BRF_PRG },
};

STD_ROM_PICK(g_mazij);
STD_ROM_FN(g_mazij);

struct BurnDriverD BurnDrvMDg_mazij = {
	"g_mazij", "g_mazi", NULL, "1993",
	"Mazin Saga (J) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mazijRomInfo, g_mazijRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mazin Saga (Beta)
static struct BurnRomInfo g_mazibRomDesc[] = {
	{ "g_mazib.bin", 0x100000, 0x36459b59, BRF_PRG },
};

STD_ROM_PICK(g_mazib);
STD_ROM_FN(g_mazib);

struct BurnDriverD BurnDrvMDg_mazib = {
	"g_mazib", "g_mazi", NULL, "1993",
	"Mazin Saga (Beta)\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mazibRomInfo, g_mazibRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// McDonald's Treasure Land Adventure (U) [!]
static struct BurnRomInfo g_mtlaRomDesc[] = {
	{ "g_mtla.bin", 0x100000, 0x04ef4899, BRF_PRG },
};

STD_ROM_PICK(g_mtla);
STD_ROM_FN(g_mtla);

struct BurnDriverD BurnDrvMDg_mtla = {
	"g_mtla", NULL, NULL, "1993",
	"McDonald's Treasure Land Adventure (U) [!]\0", NULL, "Sega / Treasure / McDonalds", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mtlaRomInfo, g_mtlaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// McDonald's Treasure Land Adventure (J) [!]
static struct BurnRomInfo g_mcdtjRomDesc[] = {
	{ "g_mcdtj.bin", 0x100000, 0xfebcfd06, BRF_PRG },
};

STD_ROM_PICK(g_mcdtj);
STD_ROM_FN(g_mcdtj);

struct BurnDriverD BurnDrvMDg_mcdtj = {
	"g_mcdtj", "g_mtla", NULL, "1993",
	"McDonald's Treasure Land Adventure (J) [!]\0", NULL, "Sega / Treasure / McDonalds", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mcdtjRomInfo, g_mcdtjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// McDonald's Treasure Land Adventure (E)
static struct BurnRomInfo g_mcdtleRomDesc[] = {
	{ "g_mcdtle.bin", 0x100000, 0x6ab6a8da, BRF_PRG },
};

STD_ROM_PICK(g_mcdtle);
STD_ROM_FN(g_mcdtle);

struct BurnDriverD BurnDrvMDg_mcdtle = {
	"g_mcdtle", "g_mtla", NULL, "1993",
	"McDonald's Treasure Land Adventure (E)\0", NULL, "Sega / Treasure / McDonalds", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mcdtleRomInfo, g_mcdtleRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// McDonald's Treasure Land Adventure (Beta)
static struct BurnRomInfo g_mcdtlbRomDesc[] = {
	{ "g_mcdtlb.bin", 0x100000, 0x7bf477e8, BRF_PRG },
};

STD_ROM_PICK(g_mcdtlb);
STD_ROM_FN(g_mcdtlb);

struct BurnDriverD BurnDrvMDg_mcdtlb = {
	"g_mcdtlb", "g_mtla", NULL, "1993",
	"McDonald's Treasure Land Adventure (Beta)\0", NULL, "Sega / Treasure / McDonalds", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mcdtlbRomInfo, g_mcdtlbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mega Turrican (U) [!]
static struct BurnRomInfo g_mturRomDesc[] = {
	{ "g_mtur.bin", 0x100000, 0xfe898cc9, BRF_PRG },
};

STD_ROM_PICK(g_mtur);
STD_ROM_FN(g_mtur);

struct BurnDriverD BurnDrvMDg_mtur = {
	"g_mtur", NULL, NULL, "1994",
	"Mega Turrican (U) [!]\0", NULL, "Data East / Factor 5", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mturRomInfo, g_mturRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mega Turrican (E) [!]
static struct BurnRomInfo g_mturreRomDesc[] = {
	{ "g_mturre.bin", 0x100000, 0xb1d15d0f, BRF_PRG },
};

STD_ROM_PICK(g_mturre);
STD_ROM_FN(g_mturre);

struct BurnDriverD BurnDrvMDg_mturre = {
	"g_mturre", "g_mtur", NULL, "1994",
	"Mega Turrican (E) [!]\0", NULL, "Data East / Factor 5", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mturreRomInfo, g_mturreRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mick & Mack as the Global Gladiators (U) [!]
static struct BurnRomInfo g_mickRomDesc[] = {
	{ "g_mick.bin", 0x100000, 0x40f17bb3, BRF_PRG },
};

STD_ROM_PICK(g_mick);
STD_ROM_FN(g_mick);

struct BurnDriverD BurnDrvMDg_mick = {
	"g_mick", NULL, NULL, "1994",
	"Mick & Mack as the Global Gladiators (U) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mickRomInfo, g_mickRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mick & Mack as the Global Gladiators (E) [!]
static struct BurnRomInfo g_micmajRomDesc[] = {
	{ "g_micmaj.bin", 0x100000, 0x5c218c6a, BRF_PRG },
};

STD_ROM_PICK(g_micmaj);
STD_ROM_FN(g_micmaj);

struct BurnDriverD BurnDrvMDg_micmaj = {
	"g_micmaj", "g_mick", NULL, "1994",
	"Mick & Mack as the Global Gladiators (E) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_micmajRomInfo, g_micmajRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mick & Mack as the Global Gladiators (Beta)
static struct BurnRomInfo g_micmabRomDesc[] = {
	{ "g_micmab.bin", 0x100000, 0x08c2af21, BRF_PRG },
};

STD_ROM_PICK(g_micmab);
STD_ROM_FN(g_micmab);

struct BurnDriverD BurnDrvMDg_micmab = {
	"g_micmab", "g_mick", NULL, "1994",
	"Mick & Mack as the Global Gladiators (Beta)\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_micmabRomInfo, g_micmabRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mickey Mania - Timeless Adventures of Mickey Mouse (U) [!]
static struct BurnRomInfo g_mmanRomDesc[] = {
	{ "g_mman.bin", 0x200000, 0x629e5963, BRF_PRG },
};

STD_ROM_PICK(g_mman);
STD_ROM_FN(g_mman);

struct BurnDriverD BurnDrvMDg_mman = {
	"g_mman", NULL, NULL, "1994",
	"Mickey Mania - Timeless Adventures of Mickey Mouse (U) [!]\0", NULL, "Disney / Sony", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mmanRomInfo, g_mmanRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mickey Mania - Timeless Adventures of Mickey Mouse (E) [!]
static struct BurnRomInfo g_mmanieRomDesc[] = {
	{ "g_mmanie.bin", 0x200000, 0xcb5a8b85, BRF_PRG },
};

STD_ROM_PICK(g_mmanie);
STD_ROM_FN(g_mmanie);

struct BurnDriverD BurnDrvMDg_mmanie = {
	"g_mmanie", "g_mman", NULL, "1994",
	"Mickey Mania - Timeless Adventures of Mickey Mouse (E) [!]\0", NULL, "Disney / Sony", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mmanieRomInfo, g_mmanieRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mickey Mania - Timeless Adventures of Mickey Mouse (J) [!]
static struct BurnRomInfo g_mmanijRomDesc[] = {
	{ "g_mmanij.bin", 0x200000, 0x23180cf7, BRF_PRG },
};

STD_ROM_PICK(g_mmanij);
STD_ROM_FN(g_mmanij);

struct BurnDriverD BurnDrvMDg_mmanij = {
	"g_mmanij", "g_mman", NULL, "1994",
	"Mickey Mania - Timeless Adventures of Mickey Mouse (J) [!]\0", NULL, "Disney / Sony", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mmanijRomInfo, g_mmanijRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mickey Mania - Timeless Adventures of Mickey Mouse (Beta)
static struct BurnRomInfo g_mmanibRomDesc[] = {
	{ "g_mmanib.bin", 0x200000, 0x7fc1bdf0, BRF_PRG },
};

STD_ROM_PICK(g_mmanib);
STD_ROM_FN(g_mmanib);

struct BurnDriverD BurnDrvMDg_mmanib = {
	"g_mmanib", "g_mman", NULL, "1994",
	"Mickey Mania - Timeless Adventures of Mickey Mouse (Beta)\0", NULL, "Disney / Sony", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mmanibRomInfo, g_mmanibRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Micro Machines (UE) [c][!]
static struct BurnRomInfo g_micRomDesc[] = {
	{ "g_mic.bin", 0x80000, 0x7ffbd1ad, BRF_PRG },
};

STD_ROM_PICK(g_mic);
STD_ROM_FN(g_mic);

struct BurnDriver BurnDrvMDg_mic = {
	"g_mic", NULL, NULL, "1993",
	"Micro Machines (UE) [c][!]\0", NULL, "Codemasters", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_micRomInfo, g_micRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Micro Machines (UE) [a1][c][!]
static struct BurnRomInfo g_micaRomDesc[] = {
	{ "g_mica.bin", 0x80000, 0xe5cf560d, BRF_PRG },
};

STD_ROM_PICK(g_mica);
STD_ROM_FN(g_mica);

struct BurnDriver BurnDrvMDg_mica = {
	"g_mica", "g_mic", NULL, "1993",
	"Micro Machines (UE) [a1][c][!]\0", NULL, "Codemasters", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_micaRomInfo, g_micaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Micro Machines (C)
static struct BurnRomInfo g_micmcRomDesc[] = {
	{ "g_micmc.bin", 0x80000, 0x54e4cff1, BRF_PRG },
};

STD_ROM_PICK(g_micmc);
STD_ROM_FN(g_micmc);

struct BurnDriver BurnDrvMDg_micmc = {
	"g_micmc", "g_mic", NULL, "1993",
	"Micro Machines (C)\0", NULL, "Codemasters", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_micmcRomInfo, g_micmcRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Micro Machines 2 - Turbo Tournament (E) [c][!]
static struct BurnRomInfo g_mic2RomDesc[] = {
	{ "g_mic2.bin", 0x100000, 0x01c22a5d, BRF_PRG },
};

STD_ROM_PICK(g_mic2);
STD_ROM_FN(g_mic2);

struct BurnDriverD BurnDrvMDg_mic2 = {
	"g_mic2", NULL, NULL, "1994",
	"Micro Machines 2 - Turbo Tournament (E) [c][!]\0", NULL, "Codemasters", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mic2RomInfo, g_mic2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Micro Machines 2 - Turbo Tournament (E) (J-Cart) [c][!]
static struct BurnRomInfo g_micm2eRomDesc[] = {
	{ "g_micm2e.bin", 0x100000, 0x42bfb7eb, BRF_PRG },
};

STD_ROM_PICK(g_micm2e);
STD_ROM_FN(g_micm2e);

struct BurnDriverD BurnDrvMDg_micm2e = {
	"g_micm2e", "g_mic2", NULL, "1994",
	"Micro Machines 2 - Turbo Tournament (E) (J-Cart) [c][!]\0", NULL, "Codemasters", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_micm2eRomInfo, g_micm2eRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Micro Machines - Turbo Tournament '96 (V1.1) (E) (J-Cart) [c][!]
static struct BurnRomInfo g_mic9RomDesc[] = {
	{ "g_mic9.bin", 0x100000, 0x23319d0d, BRF_PRG },
};

STD_ROM_PICK(g_mic9);
STD_ROM_FN(g_mic9);

struct BurnDriverD BurnDrvMDg_mic9 = {
	"g_mic9", NULL, NULL, "1995",
	"Micro Machines - Turbo Tournament '96 (V1.1) (E) (J-Cart) [c][!]\0", NULL, "Codemasters", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mic9RomInfo, g_mic9RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Micro Machines - Turbo Tournament '96 (V1.1) (E)
static struct BurnRomInfo g_micm96RomDesc[] = {
	{ "g_micm96.bin", 0x100000, 0x3137b3c4, BRF_PRG },
};

STD_ROM_PICK(g_micm96);
STD_ROM_FN(g_micm96);

struct BurnDriverD BurnDrvMDg_micm96 = {
	"g_micm96", "g_mic9", NULL, "1995",
	"Micro Machines - Turbo Tournament '96 (V1.1) (E)\0", NULL, "Codemasters", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_micm96RomInfo, g_micm96RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Micro Machines Military - It's a Blast! (E) (J-Cart) [c][!]
static struct BurnRomInfo g_micmRomDesc[] = {
	{ "g_micm.bin", 0x100000, 0xb3abb15e, BRF_PRG },
};

STD_ROM_PICK(g_micm);
STD_ROM_FN(g_micm);

struct BurnDriverD BurnDrvMDg_micm = {
	"g_micm", NULL, NULL, "1996",
	"Micro Machines Military - It's a Blast! (E) (J-Cart) [c][!]\0", NULL, "Codemasters", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_micmRomInfo, g_micmRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Micro Machines Military - It's a Blast! (E) [x]
static struct BurnRomInfo g_mimmxRomDesc[] = {
	{ "g_mimmx.bin", 0x100000, 0xa1ad9f97, BRF_PRG },
};

STD_ROM_PICK(g_mimmx);
STD_ROM_FN(g_mimmx);

struct BurnDriverD BurnDrvMDg_mimmx = {
	"g_mimmx", "g_micm", NULL, "1996",
	"Micro Machines Military - It's a Blast! (E) [x]\0", NULL, "Codemasters", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mimmxRomInfo, g_mimmxRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Out of this World (U) [!]
static struct BurnRomInfo g_ootwRomDesc[] = {
	{ "g_ootw.bin", 0x100000, 0x2da36e01, BRF_PRG },
};

STD_ROM_PICK(g_ootw);
STD_ROM_FN(g_ootw);

struct BurnDriver BurnDrvMDg_ootw = {
	"g_ootw", NULL, NULL, "1992",
	"Out of this World (U) [!]\0", NULL, "Delphine / Virgin", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ootwRomInfo, g_ootwRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Another World (E) [!]
static struct BurnRomInfo g_awldRomDesc[] = {
	{ "g_awld.bin", 0x100000, 0xe9742041, BRF_PRG },
};

STD_ROM_PICK(g_awld);
STD_ROM_FN(g_awld);

struct BurnDriver BurnDrvMDg_awld = {
	"g_awld", "g_ootw", NULL, "1992",
	"Another World (E) [!]\0", NULL, "Delphine / Virgin", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_awldRomInfo, g_awldRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Out of this World (Beta)
static struct BurnRomInfo g_ootwbRomDesc[] = {
	{ "g_ootwb.bin", 0x100000, 0x3aad905a, BRF_PRG },
};

STD_ROM_PICK(g_ootwb);
STD_ROM_FN(g_ootwb);

struct BurnDriver BurnDrvMDg_ootwb = {
	"g_ootwb", "g_ootw", NULL, "1992",
	"Out of this World (Beta)\0", NULL, "Delphine / Virgin", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ootwbRomInfo, g_ootwbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Pagemaster, The (U) [!]
static struct BurnRomInfo g_pageRomDesc[] = {
	{ "g_page.bin", 0x200000, 0x75a96d4e, BRF_PRG },
};

STD_ROM_PICK(g_page);
STD_ROM_FN(g_page);

struct BurnDriverD BurnDrvMDg_page = {
	"g_page", NULL, NULL, "1994",
	"Pagemaster, The (U) [!]\0", NULL, "Probe", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pageRomInfo, g_pageRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Pagemaster, The (E) [!]
static struct BurnRomInfo g_pagemeRomDesc[] = {
	{ "g_pageme.bin", 0x200000, 0x79a180e2, BRF_PRG },
};

STD_ROM_PICK(g_pageme);
STD_ROM_FN(g_pageme);

struct BurnDriverD BurnDrvMDg_pageme = {
	"g_pageme", "g_page", NULL, "1994",
	"Pagemaster, The (E) [!]\0", NULL, "Probe", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pagemeRomInfo, g_pagemeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Pagemaster, The (Beta)
static struct BurnRomInfo g_pagembRomDesc[] = {
	{ "g_pagemb.bin", 0x200000, 0x29895e3d, BRF_PRG },
};

STD_ROM_PICK(g_pagemb);
STD_ROM_FN(g_pagemb);

struct BurnDriverD BurnDrvMDg_pagemb = {
	"g_pagemb", "g_page", NULL, "1994",
	"Pagemaster, The (Beta)\0", NULL, "Probe", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pagembRomInfo, g_pagembRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Phantasy Star II (UE) (REV02) [!]
static struct BurnRomInfo g_pst2RomDesc[] = {
	{ "g_pst2.bin", 0xc0000, 0x904fa047, BRF_PRG },
};

STD_ROM_PICK(g_pst2);
STD_ROM_FN(g_pst2);

struct BurnDriverD BurnDrvMDg_pst2 = {
	"g_pst2", NULL, NULL, "1989",
	"Phantasy Star II (UE) (REV02) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pst2RomInfo, g_pst2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Phantasy Star II (UE) (REV01)
static struct BurnRomInfo g_pst201RomDesc[] = {
	{ "g_pst201.bin", 0xc0000, 0x0d07d0ef, BRF_PRG },
};

STD_ROM_PICK(g_pst201);
STD_ROM_FN(g_pst201);

struct BurnDriverD BurnDrvMDg_pst201 = {
	"g_pst201", "g_pst2", NULL, "1989",
	"Phantasy Star II (UE) (REV01)\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pst201RomInfo, g_pst201RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Phantasy Star II (J) [!]
static struct BurnRomInfo g_pstr2jRomDesc[] = {
	{ "g_pstr2j.bin", 0xc0000, 0xbec8eb5a, BRF_PRG },
};

STD_ROM_PICK(g_pstr2j);
STD_ROM_FN(g_pstr2j);

struct BurnDriverD BurnDrvMDg_pstr2j = {
	"g_pstr2j", "g_pst2", NULL, "1989",
	"Phantasy Star II (J) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pstr2jRomInfo, g_pstr2jRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Phantasy Star II (B) (REV01) [!]
static struct BurnRomInfo g_phst2bRomDesc[] = {
	{ "g_phst2b.bin", 0xc0000, 0xe6688b66, BRF_PRG },
};

STD_ROM_PICK(g_phst2b);
STD_ROM_FN(g_phst2b);

struct BurnDriverD BurnDrvMDg_phst2b = {
	"g_phst2b", "g_pst2", NULL, "1989",
	"Phantasy Star II (B) (REV01) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_phst2bRomInfo, g_phst2bRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Phantasy Star III - Generations of Doom (UE) [!]
static struct BurnRomInfo g_pst3RomDesc[] = {
	{ "g_pst3.bin", 0xc0000, 0xc6b42b0f, BRF_PRG },
};

STD_ROM_PICK(g_pst3);
STD_ROM_FN(g_pst3);

struct BurnDriverD BurnDrvMDg_pst3 = {
	"g_pst3", NULL, NULL, "1991",
	"Phantasy Star III - Generations of Doom (UE) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pst3RomInfo, g_pst3RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Phantasy Star III - Toki no Keishousha (J) [!]
static struct BurnRomInfo g_pstr3jRomDesc[] = {
	{ "g_pstr3j.bin", 0xc0000, 0x6c48c06f, BRF_PRG },
};

STD_ROM_PICK(g_pstr3j);
STD_ROM_FN(g_pstr3j);

struct BurnDriverD BurnDrvMDg_pstr3j = {
	"g_pstr3j", "g_pst3", NULL, "1991",
	"Phantasy Star III - Toki no Keishousha (J) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pstr3jRomInfo, g_pstr3jRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Phantasy Star III - Generations of Doom (B) [!]
static struct BurnRomInfo g_phst3bRomDesc[] = {
	{ "g_phst3b.bin", 0xc0000, 0x2e9b4c23, BRF_PRG },
};

STD_ROM_PICK(g_phst3b);
STD_ROM_FN(g_phst3b);

struct BurnDriverD BurnDrvMDg_phst3b = {
	"g_phst3b", "g_pst3", NULL, "1991",
	"Phantasy Star III - Generations of Doom (B) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_phst3bRomInfo, g_phst3bRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Phantasy Star - The End of the Millenium (U) [!]
static struct BurnRomInfo g_pst4RomDesc[] = {
	{ "g_pst4.bin", 0x300000, 0xfe236442, BRF_PRG },
};

STD_ROM_PICK(g_pst4);
STD_ROM_FN(g_pst4);

struct BurnDriverD BurnDrvMDg_pst4 = {
	"g_pst4", NULL, NULL, "1994",
	"Phantasy Star - The End of the Millenium (U) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pst4RomInfo, g_pst4RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Phantasy Star - The End of the Millenium (J) [!]
static struct BurnRomInfo g_pstr4jRomDesc[] = {
	{ "g_pstr4j.bin", 0x300000, 0xf0bfad42, BRF_PRG },
};

STD_ROM_PICK(g_pstr4j);
STD_ROM_FN(g_pstr4j);

struct BurnDriverD BurnDrvMDg_pstr4j = {
	"g_pstr4j", "g_pst4", NULL, "1994",
	"Phantasy Star - The End of the Millenium (J) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pstr4jRomInfo, g_pstr4jRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Prince of Persia (U) [!]
static struct BurnRomInfo g_prinRomDesc[] = {
	{ "g_prin.bin", 0x100000, 0x13c181a4, BRF_PRG },
};

STD_ROM_PICK(g_prin);
STD_ROM_FN(g_prin);

struct BurnDriverD BurnDrvMDg_prin = {
	"g_prin", NULL, NULL, "1993",
	"Prince of Persia (U) [!]\0", NULL, "Domark", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_prinRomInfo, g_prinRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Prince of Persia (E)
static struct BurnRomInfo g_perseRomDesc[] = {
	{ "g_perse.bin", 0x100000, 0x61de6fe0, BRF_PRG },
};

STD_ROM_PICK(g_perse);
STD_ROM_FN(g_perse);

struct BurnDriverD BurnDrvMDg_perse = {
	"g_perse", "g_prin", NULL, "1993",
	"Prince of Persia (E)\0", NULL, "Domark", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_perseRomInfo, g_perseRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Prince of Persia (Beta 1)
static struct BurnRomInfo g_persb1RomDesc[] = {
	{ "g_persb1.bin", 0x40000, 0x425e6a87, BRF_PRG },
};

STD_ROM_PICK(g_persb1);
STD_ROM_FN(g_persb1);

struct BurnDriverD BurnDrvMDg_persb1 = {
	"g_persb1", "g_prin", NULL, "1993",
	"Prince of Persia (Beta 1)\0", NULL, "Domark", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_persb1RomInfo, g_persb1RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Prince of Persia (Beta 2)
static struct BurnRomInfo g_persb2RomDesc[] = {
	{ "g_persb2.bin", 0x100000, 0x505314b6, BRF_PRG },
};

STD_ROM_PICK(g_persb2);
STD_ROM_FN(g_persb2);

struct BurnDriverD BurnDrvMDg_persb2 = {
	"g_persb2", "g_prin", NULL, "1993",
	"Prince of Persia (Beta 2)\0", NULL, "Domark", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_persb2RomInfo, g_persb2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Quack Shot Starring Donald Duck (W) (REV01) [!]
static struct BurnRomInfo g_quacRomDesc[] = {
	{ "g_quac.bin", 0x140000, 0x1801098b, BRF_PRG },
};

STD_ROM_PICK(g_quac);
STD_ROM_FN(g_quac);

struct BurnDriverD BurnDrvMDg_quac = {
	"g_quac", NULL, NULL, "1991",
	"Quack Shot Starring Donald Duck (W) (REV01) [!]\0", NULL, "Disney / Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_quacRomInfo, g_quacRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Quack Shot Starring Donald Duck (W) (REV00) [!]
static struct BurnRomInfo g_quac00RomDesc[] = {
	{ "g_quac00.bin", 0x80000, 0x88c8dd94, BRF_PRG },
};

STD_ROM_PICK(g_quac00);
STD_ROM_FN(g_quac00);

struct BurnDriverD BurnDrvMDg_quac00 = {
	"g_quac00", "g_quac", NULL, "1994",
	"Quack Shot Starring Donald Duck (W) (REV00) [!]\0", NULL, "Disney / Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_quac00RomInfo, g_quac00RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Quack Shot Starring Donald Duck (W) (REV01) [a1][c][!]
static struct BurnRomInfo g_quac01RomDesc[] = {
	{ "g_quac01.bin", 0x140000, 0xa5c946b3, BRF_PRG },
};

STD_ROM_PICK(g_quac01);
STD_ROM_FN(g_quac01);

struct BurnDriverD BurnDrvMDg_quac01 = {
	"g_quac01", "g_quac", NULL, "1994",
	"Quack Shot Starring Donald Duck (W) (REV01) [a1][c][!]\0", NULL, "Disney / Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_quac01RomInfo, g_quac01RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Race Drivin' (U) [!]
static struct BurnRomInfo g_raceRomDesc[] = {
	{ "g_race.bin", 0x80000, 0xd737cf3d, BRF_PRG },
};

STD_ROM_PICK(g_race);
STD_ROM_FN(g_race);

struct BurnDriverD BurnDrvMDg_race = {
	"g_race", NULL, NULL, "1993",
	"Race Drivin' (U) [!]\0", NULL, "Tengen / Atari", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_raceRomInfo, g_raceRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Revenge of Shinobi, The (W) (REV03) [!]
static struct BurnRomInfo g_revsRomDesc[] = {
	{ "g_revs.bin", 0x80000, 0x4d35ebe4, BRF_PRG },
};

STD_ROM_PICK(g_revs);
STD_ROM_FN(g_revs);

struct BurnDriverD BurnDrvMDg_revs = {
	"g_revs", NULL, NULL, "1990",
	"Revenge of Shinobi, The (W) (REV03) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_revsRomInfo, g_revsRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Revenge of Shinobi, The (W) (REV00) [!]
static struct BurnRomInfo g_rshi00RomDesc[] = {
	{ "g_rshi00.bin", 0x80000, 0x5c7e5ea6, BRF_PRG },
};

STD_ROM_PICK(g_rshi00);
STD_ROM_FN(g_rshi00);

struct BurnDriverD BurnDrvMDg_rshi00 = {
	"g_rshi00", "g_revs", NULL, "1990",
	"Revenge of Shinobi, The (W) (REV00) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_rshi00RomInfo, g_rshi00RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Revenge of Shinobi, The (W) (REV01) [!]
static struct BurnRomInfo g_rshi01RomDesc[] = {
	{ "g_rshi01.bin", 0x80000, 0x05f27994, BRF_PRG },
};

STD_ROM_PICK(g_rshi01);
STD_ROM_FN(g_rshi01);

struct BurnDriverD BurnDrvMDg_rshi01 = {
	"g_rshi01", "g_revs", NULL, "1990",
	"Revenge of Shinobi, The (W) (REV01) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_rshi01RomInfo, g_rshi01RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Revenge of Shinobi, The (W) (REV02) [!]
static struct BurnRomInfo g_rshi02RomDesc[] = {
	{ "g_rshi02.bin", 0x80000, 0xfe91ab7e, BRF_PRG },
};

STD_ROM_PICK(g_rshi02);
STD_ROM_FN(g_rshi02);

struct BurnDriverD BurnDrvMDg_rshi02 = {
	"g_rshi02", "g_revs", NULL, "1990",
	"Revenge of Shinobi, The (W) (REV02) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_rshi02RomInfo, g_rshi02RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Samurai Shodown (U) [!]
static struct BurnRomInfo g_samsRomDesc[] = {
	{ "g_sams.bin", 0x300000, 0x5bb8b2d4, BRF_PRG },
};

STD_ROM_PICK(g_sams);
STD_ROM_FN(g_sams);

struct BurnDriverD BurnDrvMDg_sams = {
	"g_sams", NULL, NULL, "1994",
	"Samurai Shodown (U) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_samsRomInfo, g_samsRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Samurai Shodown (E) [c][!]
static struct BurnRomInfo g_samsheRomDesc[] = {
	{ "g_samshe.bin", 0x300000, 0xc972014f, BRF_PRG },
};

STD_ROM_PICK(g_samshe);
STD_ROM_FN(g_samshe);

struct BurnDriverD BurnDrvMDg_samshe = {
	"g_samshe", "g_sams", NULL, "1994",
	"Samurai Shodown (E) [c][!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_samsheRomInfo, g_samsheRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Samurai Spirits (J) [!]
static struct BurnRomInfo g_samspiRomDesc[] = {
	{ "g_samspi.bin", 0x300000, 0x0ea2ae36, BRF_PRG },
};

STD_ROM_PICK(g_samspi);
STD_ROM_FN(g_samspi);

struct BurnDriverD BurnDrvMDg_samspi = {
	"g_samspi", "g_sams", NULL, "1994",
	"Samurai Spirits (J) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_samspiRomInfo, g_samspiRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Shining in the Darkness (UE) [!]
static struct BurnRomInfo g_sitdRomDesc[] = {
	{ "g_sitd.bin", 0x100000, 0x4d2785bc, BRF_PRG },
};

STD_ROM_PICK(g_sitd);
STD_ROM_FN(g_sitd);

struct BurnDriverD BurnDrvMDg_sitd = {
	"g_sitd", NULL, NULL, "1991",
	"Shining in the Darkness (UE) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sitdRomInfo, g_sitdRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Shining and the Darkness (J) [!]
static struct BurnRomInfo g_shdrkjRomDesc[] = {
	{ "g_shdrkj.bin", 0x100000, 0x496af51c, BRF_PRG },
};

STD_ROM_PICK(g_shdrkj);
STD_ROM_FN(g_shdrkj);

struct BurnDriverD BurnDrvMDg_shdrkj = {
	"g_shdrkj", "g_sitd", NULL, "1991",
	"Shining and the Darkness (J) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_shdrkjRomInfo, g_shdrkjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Shining in the Darkness (B) [!]
static struct BurnRomInfo g_shdrkbRomDesc[] = {
	{ "g_shdrkb.bin", 0x100000, 0x3ee2bbc4, BRF_PRG },
};

STD_ROM_PICK(g_shdrkb);
STD_ROM_FN(g_shdrkb);

struct BurnDriverD BurnDrvMDg_shdrkb = {
	"g_shdrkb", "g_sitd", NULL, "1991",
	"Shining in the Darkness (B) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_shdrkbRomInfo, g_shdrkbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Shining Force (U) [!]
static struct BurnRomInfo g_shf1RomDesc[] = {
	{ "g_shf1.bin", 0x180000, 0xe0594abe, BRF_PRG },
};

STD_ROM_PICK(g_shf1);
STD_ROM_FN(g_shf1);

struct BurnDriverD BurnDrvMDg_shf1 = {
	"g_shf1", NULL, NULL, "1993",
	"Shining Force (U) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_shf1RomInfo, g_shf1RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Shining Force - The Legacy of Great Intention (J) [!]
static struct BurnRomInfo g_shfrcjRomDesc[] = {
	{ "g_shfrcj.bin", 0x180000, 0x9378fbcd, BRF_PRG },
};

STD_ROM_PICK(g_shfrcj);
STD_ROM_FN(g_shfrcj);

struct BurnDriverD BurnDrvMDg_shfrcj = {
	"g_shfrcj", "g_shf1", NULL, "1993",
	"Shining Force - The Legacy of Great Intention (J) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_shfrcjRomInfo, g_shfrcjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Shining Force (Beta)
static struct BurnRomInfo g_sfrcbtRomDesc[] = {
	{ "g_sfrcbt.bin", 0x180000, 0xce67143a, BRF_PRG },
};

STD_ROM_PICK(g_sfrcbt);
STD_ROM_FN(g_sfrcbt);

struct BurnDriverD BurnDrvMDg_sfrcbt = {
	"g_sfrcbt", "g_shf1", NULL, "1993",
	"Shining Force (Beta)\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sfrcbtRomInfo, g_sfrcbtRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Shining Force II (U) [!]
static struct BurnRomInfo g_shf2RomDesc[] = {
	{ "g_shf2.bin", 0x200000, 0x4815e075, BRF_PRG },
};

STD_ROM_PICK(g_shf2);
STD_ROM_FN(g_shf2);

struct BurnDriverD BurnDrvMDg_shf2 = {
	"g_shf2", NULL, NULL, "1994",
	"Shining Force II (U) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_shf2RomInfo, g_shf2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Shining Force II (E) [!]
static struct BurnRomInfo g_shfr2eRomDesc[] = {
	{ "g_shfr2e.bin", 0x200000, 0x83cb46d1, BRF_PRG },
};

STD_ROM_PICK(g_shfr2e);
STD_ROM_FN(g_shfr2e);

struct BurnDriverD BurnDrvMDg_shfr2e = {
	"g_shfr2e", "g_shf2", NULL, "1994",
	"Shining Force II (E) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_shfr2eRomInfo, g_shfr2eRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Shining Force II - Inishie no Fuuin (J) [!]
static struct BurnRomInfo g_shfr2jRomDesc[] = {
	{ "g_shfr2j.bin", 0x200000, 0x0288f3e1, BRF_PRG },
};

STD_ROM_PICK(g_shfr2j);
STD_ROM_FN(g_shfr2j);

struct BurnDriverD BurnDrvMDg_shfr2j = {
	"g_shfr2j", "g_shf2", NULL, "1994",
	"Shining Force II - Inishie no Fuuin (J) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_shfr2jRomInfo, g_shfr2jRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Shinobi III - Return of the Ninja Master (U) [!]
static struct BurnRomInfo g_shi3RomDesc[] = {
	{ "g_shi3.bin", 0x100000, 0x5381506f, BRF_PRG },
};

STD_ROM_PICK(g_shi3);
STD_ROM_FN(g_shi3);

struct BurnDriverD BurnDrvMDg_shi3 = {
	"g_shi3", NULL, NULL, "1993",
	"Shinobi III - Return of the Ninja Master (U) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_shi3RomInfo, g_shi3RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Shinobi III - Return of the Ninja Master (E) [c][!]
static struct BurnRomInfo g_shin3eRomDesc[] = {
	{ "g_shin3e.bin", 0x100000, 0x0b6d3eb5, BRF_PRG },
};

STD_ROM_PICK(g_shin3e);
STD_ROM_FN(g_shin3e);

struct BurnDriverD BurnDrvMDg_shin3e = {
	"g_shin3e", "g_shi3", NULL, "1993",
	"Shinobi III - Return of the Ninja Master (E) [c][!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_shin3eRomInfo, g_shin3eRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Super Shinobi II, The (J) [!]
static struct BurnRomInfo g_sshin2RomDesc[] = {
	{ "g_sshin2.bin", 0x100000, 0x5b412816, BRF_PRG },
};

STD_ROM_PICK(g_sshin2);
STD_ROM_FN(g_sshin2);

struct BurnDriverD BurnDrvMDg_sshin2 = {
	"g_sshin2", "g_shi3", NULL, "1993",
	"Super Shinobi II, The (J) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sshin2RomInfo, g_sshin2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Super Shinobi II, The (Beta)
static struct BurnRomInfo g_sshi2bRomDesc[] = {
	{ "g_sshi2b.bin", 0x100000, 0xc47e8aea, BRF_PRG },
};

STD_ROM_PICK(g_sshi2b);
STD_ROM_FN(g_sshi2b);

struct BurnDriverD BurnDrvMDg_sshi2b = {
	"g_sshi2b", "g_shi3", NULL, "1993",
	"Super Shinobi II, The (Beta)\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sshi2bRomInfo, g_sshi2bRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Super Skidmarks (E) (J-Cart) [!]
static struct BurnRomInfo g_skidRomDesc[] = {
	{ "g_skid.bin", 0x200000, 0x4a9c62f9, BRF_PRG },
};

STD_ROM_PICK(g_skid);
STD_ROM_FN(g_skid);

struct BurnDriverD BurnDrvMDg_skid = {
	"g_skid", NULL, NULL, "1995",
	"Super Skidmarks (E) (J-Cart) [!]\0", NULL, "Codemasters / Acid", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_skidRomInfo, g_skidRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Super Skidmarks (E) [a1]
static struct BurnRomInfo g_skidaRomDesc[] = {
	{ "g_skida.bin", 0x200000, 0xa61a0f0c, BRF_PRG },
};

STD_ROM_PICK(g_skida);
STD_ROM_FN(g_skida);

struct BurnDriverD BurnDrvMDg_skida = {
	"g_skida", "g_skid", NULL, "1995",
	"Super Skidmarks (E) [a1]\0", NULL, "Codemasters / Acid", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_skidaRomInfo, g_skidaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Ranger-X (U) [!]
static struct BurnRomInfo g_ranxRomDesc[] = {
	{ "g_ranx.bin", 0x100000, 0x55915915, BRF_PRG },
};

STD_ROM_PICK(g_ranx);
STD_ROM_FN(g_ranx);

struct BurnDriverD BurnDrvMDg_ranx = {
	"g_ranx", NULL, NULL, "1993",
	"Ranger-X (U) [!]\0", NULL, "Sega / Gau", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ranxRomInfo, g_ranxRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Ranger-X (E) [!]
static struct BurnRomInfo g_ranxeRomDesc[] = {
	{ "g_ranxe.bin", 0x100000, 0xb8c04804, BRF_PRG },
};

STD_ROM_PICK(g_ranxe);
STD_ROM_FN(g_ranxe);

struct BurnDriverD BurnDrvMDg_ranxe = {
	"g_ranxe", "g_ranx", NULL, "1993",
	"Ranger-X (E) [!]\0", NULL, "Sega / Gau", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ranxeRomInfo, g_ranxeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Ex-Ranza (J) [!]
static struct BurnRomInfo g_exranzRomDesc[] = {
	{ "g_exranz.bin", 0x100000, 0x349bb68d, BRF_PRG },
};

STD_ROM_PICK(g_exranz);
STD_ROM_FN(g_exranz);

struct BurnDriverD BurnDrvMDg_exranz = {
	"g_exranz", "g_ranx", NULL, "1993",
	"Ex-Ranza (J) [!]\0", NULL, "Sega / Gau", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_exranzRomInfo, g_exranzRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Ex-Ranza (Beta)
static struct BurnRomInfo g_exrnzbRomDesc[] = {
	{ "g_exrnzb.bin", 0x100000, 0xc642fdf4, BRF_PRG },
};

STD_ROM_PICK(g_exrnzb);
STD_ROM_FN(g_exrnzb);

struct BurnDriverD BurnDrvMDg_exrnzb = {
	"g_exrnzb", "g_ranx", NULL, "1993",
	"Ex-Ranza (Beta)\0", NULL, "Sega / Gau", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_exrnzbRomInfo, g_exrnzbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Road Rash (UE) [c][!]
static struct BurnRomInfo g_rrshRomDesc[] = {
	{ "g_rrsh.bin", 0xc0000, 0xdea53d19, BRF_PRG },
};

STD_ROM_PICK(g_rrsh);
STD_ROM_FN(g_rrsh);

struct BurnDriverD BurnDrvMDg_rrsh = {
	"g_rrsh", NULL, NULL, "1991",
	"Road Rash (UE) [c][!]\0", NULL, "Electronic Arts", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_rrshRomInfo, g_rrshRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Road Rash II (UE) (REV00) [!]
static struct BurnRomInfo g_rrs2RomDesc[] = {
	{ "g_rrs2.bin", 0x100000, 0x7b29c209, BRF_PRG },
};

STD_ROM_PICK(g_rrs2);
STD_ROM_FN(g_rrs2);

struct BurnDriverD BurnDrvMDg_rrs2 = {
	"g_rrs2", NULL, NULL, "1992",
	"Road Rash II (UE) (REV00) [!]\0", NULL, "Electronic Arts", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_rrs2RomInfo, g_rrs2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Road Rash II (UE) (REV02) [c][!]
static struct BurnRomInfo g_rrsh22RomDesc[] = {
	{ "g_rrsh22.bin", 0x100000, 0x0876e992, BRF_PRG },
};

STD_ROM_PICK(g_rrsh22);
STD_ROM_FN(g_rrsh22);

struct BurnDriverD BurnDrvMDg_rrsh22 = {
	"g_rrsh22", "g_rrs2", NULL, "1992",
	"Road Rash II (UE) (REV02) [c][!]\0", NULL, "Electronic Arts", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_rrsh22RomInfo, g_rrsh22RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Road Rash II (J) [!]
static struct BurnRomInfo g_rrsh2jRomDesc[] = {
	{ "g_rrsh2j.bin", 0x100000, 0x9a5723b6, BRF_PRG },
};

STD_ROM_PICK(g_rrsh2j);
STD_ROM_FN(g_rrsh2j);

struct BurnDriverD BurnDrvMDg_rrsh2j = {
	"g_rrsh2j", "g_rrs2", NULL, "1992",
	"Road Rash II (J) [!]\0", NULL, "Electronic Arts", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_rrsh2jRomInfo, g_rrsh2jRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Road Rash 3 (UE) [!]
static struct BurnRomInfo g_rrs3RomDesc[] = {
	{ "g_rrs3.bin", 0x200000, 0x15785956, BRF_PRG },
};

STD_ROM_PICK(g_rrs3);
STD_ROM_FN(g_rrs3);

struct BurnDriverD BurnDrvMDg_rrs3 = {
	"g_rrs3", NULL, NULL, "1995",
	"Road Rash 3 (UE) [!]\0", NULL, "Electronic Arts", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_rrs3RomInfo, g_rrs3RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Robocop Versus The Terminator (U) [!]
static struct BurnRomInfo g_robtRomDesc[] = {
	{ "g_robt.bin", 0x200000, 0xbbad77a4, BRF_PRG },
};

STD_ROM_PICK(g_robt);
STD_ROM_FN(g_robt);

struct BurnDriverD BurnDrvMDg_robt = {
	"g_robt", NULL, NULL, "1993",
	"Robocop Versus The Terminator (U) [!]\0", NULL, "Virgin", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_robtRomInfo, g_robtRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Robocop Versus The Terminator (E)
static struct BurnRomInfo g_robteRomDesc[] = {
	{ "g_robte.bin", 0x200000, 0x85a93f8d, BRF_PRG },
};

STD_ROM_PICK(g_robte);
STD_ROM_FN(g_robte);

struct BurnDriverD BurnDrvMDg_robte = {
	"g_robte", "g_robt", NULL, "1993",
	"Robocop Versus The Terminator (E)\0", NULL, "Virgin", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_robteRomInfo, g_robteRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Robocop Versus The Terminator (Beta 1)
static struct BurnRomInfo g_robtb1RomDesc[] = {
	{ "g_robtb1.bin", 0x200000, 0xecebff29, BRF_PRG },
};

STD_ROM_PICK(g_robtb1);
STD_ROM_FN(g_robtb1);

struct BurnDriverD BurnDrvMDg_robtb1 = {
	"g_robtb1", "g_robt", NULL, "1993",
	"Robocop Versus The Terminator (Beta 1)\0", NULL, "Virgin", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_robtb1RomInfo, g_robtb1RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Robocop Versus The Terminator (Beta 2)
static struct BurnRomInfo g_robtb2RomDesc[] = {
	{ "g_robtb2.bin", 0x200000, 0x2487049f, BRF_PRG },
};

STD_ROM_PICK(g_robtb2);
STD_ROM_FN(g_robtb2);

struct BurnDriverD BurnDrvMDg_robtb2 = {
	"g_robtb2", "g_robt", NULL, "1993",
	"Robocop Versus The Terminator (Beta 2)\0", NULL, "Virgin", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_robtb2RomInfo, g_robtb2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Robocop 3 (UE) [!]
static struct BurnRomInfo g_robo3RomDesc[] = {
	{ "g_robo3.bin", 0x80000, 0x34fb7b27, BRF_PRG },
};

STD_ROM_PICK(g_robo3);
STD_ROM_FN(g_robo3);

struct BurnDriverD BurnDrvMDg_robo3 = {
	"g_robo3", NULL, NULL, "1993",
	"Robocop 3 (UE) [!]\0", NULL, "Acclaim", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_robo3RomInfo, g_robo3RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Rolo to the Rescue (UE) [!]
static struct BurnRomInfo g_roloRomDesc[] = {
	{ "g_rolo.bin", 0x80000, 0x306861a2, BRF_PRG },
};

STD_ROM_PICK(g_rolo);
STD_ROM_FN(g_rolo);

struct BurnDriverD BurnDrvMDg_rolo = {
	"g_rolo", NULL, NULL, "1992",
	"Rolo to the Rescue (UE) [!]\0", NULL, "Electronic Arts / Vectordean", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_roloRomInfo, g_roloRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Zou! Zou! Zou! Rescue Daisenryaku (J) [!]
static struct BurnRomInfo g_zouzouRomDesc[] = {
	{ "g_zouzou.bin", 0x80000, 0x1a761e67, BRF_PRG },
};

STD_ROM_PICK(g_zouzou);
STD_ROM_FN(g_zouzou);

struct BurnDriverD BurnDrvMDg_zouzou = {
	"g_zouzou", "g_rolo", NULL, "1992",
	"Zou! Zou! Zou! Rescue Daisenryaku (J) [!]\0", NULL, "Electronic Arts / Vectordean", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_zouzouRomInfo, g_zouzouRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sensible Soccer - International Edition (E) (M4) [!]
static struct BurnRomInfo g_seniRomDesc[] = {
	{ "g_seni.bin", 0x80000, 0x04e3bcca, BRF_PRG },
};

STD_ROM_PICK(g_seni);
STD_ROM_FN(g_seni);

struct BurnDriver BurnDrvMDg_seni = {
	"g_seni", NULL, NULL, "1994",
	"Sensible Soccer - International Edition (E) (M4) [!]\0", NULL, "Sensible Software", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_seniRomInfo, g_seniRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sensible Soccer (E) (M4) [!]
static struct BurnRomInfo g_senRomDesc[] = {
	{ "g_sen.bin", 0x80000, 0xf9b396b8, BRF_PRG },
};

STD_ROM_PICK(g_sen);
STD_ROM_FN(g_sen);

struct BurnDriver BurnDrvMDg_sen = {
	"g_sen", "g_seni", NULL, "1993",
	"Sensible Soccer (E) (M4) [!]\0", NULL, "Sensible Software", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_senRomInfo, g_senRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sensible Soccer (Beta)
static struct BurnRomInfo g_sensibRomDesc[] = {
	{ "g_sensib.bin", 0x80000, 0xef52664d, BRF_PRG },
};

STD_ROM_PICK(g_sensib);
STD_ROM_FN(g_sensib);

struct BurnDriverD BurnDrvMDg_sensib = {
	"g_sensib", "g_seni", NULL, "1993",
	"Sensible Soccer (Beta)\0", NULL, "Sensible Software", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sensibRomInfo, g_sensibRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Super Monaco Grand Prix (W) (M2) (REV02) [!]
static struct BurnRomInfo g_smgpRomDesc[] = {
	{ "g_smgp.bin", 0x80000, 0xbe91b28a, BRF_PRG },
};

STD_ROM_PICK(g_smgp);
STD_ROM_FN(g_smgp);

struct BurnDriverD BurnDrvMDg_smgp = {
	"g_smgp", NULL, NULL, "1990",
	"Super Monaco Grand Prix (W) (M2) (REV02) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_smgpRomInfo, g_smgpRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Super Monaco Grand Prix (W) (M2) (REV00) [!]
static struct BurnRomInfo g_smgp00RomDesc[] = {
	{ "g_smgp00.bin", 0x80000, 0x90f9bab3, BRF_PRG },
};

STD_ROM_PICK(g_smgp00);
STD_ROM_FN(g_smgp00);

struct BurnDriverD BurnDrvMDg_smgp00 = {
	"g_smgp00", "g_smgp", NULL, "1990",
	"Super Monaco Grand Prix (W) (M2) (REV00) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_smgp00RomInfo, g_smgp00RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Super Monaco Grand Prix (W) (M2) (REV01) [!]
static struct BurnRomInfo g_smgp01RomDesc[] = {
	{ "g_smgp01.bin", 0x80000, 0xb1823595, BRF_PRG },
};

STD_ROM_PICK(g_smgp01);
STD_ROM_FN(g_smgp01);

struct BurnDriverD BurnDrvMDg_smgp01 = {
	"g_smgp01", "g_smgp", NULL, "1990",
	"Super Monaco Grand Prix (W) (M2) (REV01) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_smgp01RomInfo, g_smgp01RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Super Monaco Grand Prix (U) (M2) (REV03) [!]
static struct BurnRomInfo g_smgp03RomDesc[] = {
	{ "g_smgp03.bin", 0x80000, 0x725018ee, BRF_PRG },
};

STD_ROM_PICK(g_smgp03);
STD_ROM_FN(g_smgp03);

struct BurnDriverD BurnDrvMDg_smgp03 = {
	"g_smgp03", "g_smgp", NULL, "1990",
	"Super Monaco Grand Prix (U) (M2) (REV03) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_smgp03RomInfo, g_smgp03RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sonic The Hedgehog (W) (REV01) [!]
static struct BurnRomInfo g_soniRomDesc[] = {
	{ "g_soni.bin", 0x80000, 0xAFE05EEE, BRF_PRG },
};

STD_ROM_PICK(g_soni);
STD_ROM_FN(g_soni);

struct BurnDriver BurnDrvMDg_soni = {
	"g_soni", NULL, NULL, "1991",
	"Sonic The Hedgehog (W) (REV01) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_soniRomInfo, g_soniRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sonic The Hedgehog (W) (REV00) [!]
static struct BurnRomInfo g_soni00RomDesc[] = {
	{ "g_soni00.bin", 0x80000, 0xf9394e97, BRF_PRG },
};

STD_ROM_PICK(g_soni00);
STD_ROM_FN(g_soni00);

struct BurnDriver BurnDrvMDg_soni00 = {
	"g_soni00", "g_soni", NULL, "1991",
	"Sonic The Hedgehog (W) (REV00) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_soni00RomInfo, g_soni00RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sonic The Hedgehog 2 (W) (REV01) [!]
static struct BurnRomInfo g_son2RomDesc[] = {
	{ "g_son2.bin", 0x100000, 0x7b905383, BRF_PRG },
};

STD_ROM_PICK(g_son2);
STD_ROM_FN(g_son2);

struct BurnDriver BurnDrvMDg_son2 = {
	"g_son2", NULL, NULL, "1992",
	"Sonic The Hedgehog 2 (W) (REV01) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_son2RomInfo, g_son2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sonic The Hedgehog 2 (W) (REVSC02)
static struct BurnRomInfo g_son2aRomDesc[] = {
	{ "g_son2a.bin", 0x100000, 0xf23ad4b3, BRF_PRG },
};

STD_ROM_PICK(g_son2a);
STD_ROM_FN(g_son2a);

struct BurnDriver BurnDrvMDg_son2a = {
	"g_son2a", "g_son2", NULL, "1992",
	"Sonic The Hedgehog 2 (W) (REVSC02)\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_son2aRomInfo, g_son2aRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sonic The Hedgehog 2 (W) (REV00) [!]
static struct BurnRomInfo g_son200RomDesc[] = {
	{ "g_son200.bin", 0x100000, 0x24ab4c3a, BRF_PRG },
};

STD_ROM_PICK(g_son200);
STD_ROM_FN(g_son200);

struct BurnDriver BurnDrvMDg_son200 = {
	"g_son200", "g_son2", NULL, "1992",
	"Sonic The Hedgehog 2 (W) (REV00) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_son200RomInfo, g_son200RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sonic The Hedgehog 2 (Prototype) [!]
static struct BurnRomInfo g_soni2pRomDesc[] = {
	{ "g_soni2p.bin", 0x100000, 0xeea21b5c, BRF_PRG },
};

STD_ROM_PICK(g_soni2p);
STD_ROM_FN(g_soni2p);

struct BurnDriverD BurnDrvMDg_soni2p = {
	"g_soni2p", "g_son2", NULL, "1992",
	"Sonic The Hedgehog 2 (Prototype) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_soni2pRomInfo, g_soni2pRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sonic The Hedgehog 3 (U) [!]
static struct BurnRomInfo g_son3RomDesc[] = {
	{ "g_son3.bin", 0x200000, 0x9bc192ce, BRF_PRG },
};

STD_ROM_PICK(g_son3);
STD_ROM_FN(g_son3);

struct BurnDriver BurnDrvMDg_son3 = {
	"g_son3", NULL, NULL, "1994",
	"Sonic The Hedgehog 3 (U) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_son3RomInfo, g_son3RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sonic The Hedgehog 3 (E) [!]
static struct BurnRomInfo g_son3eRomDesc[] = {
	{ "g_son3e.bin", 0x200000, 0x6a632503, BRF_PRG },
};

STD_ROM_PICK(g_son3e);
STD_ROM_FN(g_son3e);

struct BurnDriver BurnDrvMDg_son3e = {
	"g_son3e", "g_son3", NULL, "1994",
	"Sonic The Hedgehog 3 (E) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_son3eRomInfo, g_son3eRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sonic The Hedgehog 3 (J) [!]
static struct BurnRomInfo g_son3jRomDesc[] = {
	{ "g_son3j.bin", 0x200000, 0xf4951d1f, BRF_PRG },
};

STD_ROM_PICK(g_son3j);
STD_ROM_FN(g_son3j);

struct BurnDriver BurnDrvMDg_son3j = {
	"g_son3j", "g_son3", NULL, "1994",
	"Sonic The Hedgehog 3 (J) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_son3jRomInfo, g_son3jRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sonic 3D Blast (UE) [!]
static struct BurnRomInfo g_s3dRomDesc[] = {
	{ "g_s3d.bin", 0x400000, 0x44a2ca44, BRF_PRG },
};

STD_ROM_PICK(g_s3d);
STD_ROM_FN(g_s3d);

struct BurnDriver BurnDrvMDg_s3d = {
	"g_s3d", NULL, NULL, "1996",
	"Sonic 3D Blast (UE) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_s3dRomInfo, g_s3dRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sonic and Knuckles (W) [!]
static struct BurnRomInfo g_snknRomDesc[] = {
	{ "g_snkn.bin", 0x200000, 0x0658f691, BRF_PRG },
};

STD_ROM_PICK(g_snkn);
STD_ROM_FN(g_snkn);

struct BurnDriver BurnDrvMDg_snkn = {
	"g_snkn", NULL, NULL, "1994",
	"Sonic and Knuckles (W) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_snknRomInfo, g_snknRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sonic and Knuckles & Sonic 1 (W) [!]
static struct BurnRomInfo g_sks1RomDesc[] = {
	{ "g_sks1.bin", 0x280000, 0xe01f6ed5, BRF_PRG },
};

STD_ROM_PICK(g_sks1);
STD_ROM_FN(g_sks1);

struct BurnDriver BurnDrvMDg_sks1 = {
	"g_sks1", "g_snkn", NULL, "1994",
	"Sonic and Knuckles & Sonic 1 (W) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sks1RomInfo, g_sks1RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sonic and Knuckles & Sonic 2 (W) [!]
static struct BurnRomInfo g_sks2RomDesc[] = {
	{ "g_sks2.bin", 0x340000, 0x2ac1e7c6, BRF_PRG },
};

STD_ROM_PICK(g_sks2);
STD_ROM_FN(g_sks2);

struct BurnDriver BurnDrvMDg_sks2 = {
	"g_sks2", "g_snkn", NULL, "1994",
	"Sonic and Knuckles & Sonic 2 (W) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sks2RomInfo, g_sks2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sonic and Knuckles & Sonic 3 (W) [!]
static struct BurnRomInfo g_sks3RomDesc[] = {
	{ "g_sks3.bin", 0x400000, 0x63522553, BRF_PRG },
};

STD_ROM_PICK(g_sks3);
STD_ROM_FN(g_sks3);

struct BurnDriver BurnDrvMDg_sks3 = {
	"g_sks3", "g_snkn", NULL, "1994",
	"Sonic and Knuckles & Sonic 3 (W) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sks3RomInfo, g_sks3RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sonic Spinball (U) [!]
static struct BurnRomInfo g_sonsRomDesc[] = {
	{ "g_sons.bin", 0x100000, 0x677206cb, BRF_PRG },
};

STD_ROM_PICK(g_sons);
STD_ROM_FN(g_sons);

struct BurnDriverD BurnDrvMDg_sons = {
	"g_sons", NULL, NULL, "1993",
	"Sonic Spinball (U) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sonsRomInfo, g_sonsRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sonic Spinball (U) (alt music) [!]
static struct BurnRomInfo g_sspaRomDesc[] = {
	{ "g_sspa.bin", 0x100000, 0xe9960371, BRF_PRG },
};

STD_ROM_PICK(g_sspa);
STD_ROM_FN(g_sspa);

struct BurnDriverD BurnDrvMDg_sspa = {
	"g_sspa", "g_sons", NULL, "1993",
	"Sonic Spinball (U) (alt music) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sspaRomInfo, g_sspaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sonic Spinball (E) [!]
static struct BurnRomInfo g_sonseRomDesc[] = {
	{ "g_sonse.bin", 0x100000, 0xaea0786d, BRF_PRG },
};

STD_ROM_PICK(g_sonse);
STD_ROM_FN(g_sonse);

struct BurnDriverD BurnDrvMDg_sonse = {
	"g_sonse", "g_sons", NULL, "1993",
	"Sonic Spinball (E) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sonseRomInfo, g_sonseRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sonic Spinball (J) [!]
static struct BurnRomInfo g_sonsjRomDesc[] = {
	{ "g_sonsj.bin", 0x100000, 0xacd08ce8, BRF_PRG },
};

STD_ROM_PICK(g_sonsj);
STD_ROM_FN(g_sonsj);

struct BurnDriverD BurnDrvMDg_sonsj = {
	"g_sonsj", "g_sons", NULL, "1993",
	"Sonic Spinball (J) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sonsjRomInfo, g_sonsjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sonic Spinball (Beta)
static struct BurnRomInfo g_sonsbRomDesc[] = {
	{ "g_sonsb.bin", 0x140000, 0xb1524979, BRF_PRG },
};

STD_ROM_PICK(g_sonsb);
STD_ROM_FN(g_sonsb);

struct BurnDriverD BurnDrvMDg_sonsb = {
	"g_sonsb", "g_sons", NULL, "1993",
	"Sonic Spinball (Beta)\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sonsbRomInfo, g_sonsbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Socket (U) [!]
static struct BurnRomInfo g_sockRomDesc[] = {
	{ "g_sock.bin", 0x100000, 0x3c14e15a, BRF_PRG },
};

STD_ROM_PICK(g_sock);
STD_ROM_FN(g_sock);

struct BurnDriverD BurnDrvMDg_sock = {
	"g_sock", NULL, NULL, "1993",
	"Socket (U) [!]\0", NULL, "Vic Tokai", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sockRomInfo, g_sockRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Time Dominator 1st (J) [!]
static struct BurnRomInfo g_tdom1RomDesc[] = {
	{ "g_tdom1.bin", 0x100000, 0x7eba7a5c, BRF_PRG },
};

STD_ROM_PICK(g_tdom1);
STD_ROM_FN(g_tdom1);

struct BurnDriverD BurnDrvMDg_tdom1 = {
	"g_tdom1", "g_sock", NULL, "1993",
	"Time Dominator 1st (J) [!]\0", NULL, "Vic Tokai", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tdom1RomInfo, g_tdom1RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Spider-Man - The Animated Series (W) [!]
static struct BurnRomInfo g_s_asRomDesc[] = {
	{ "g_s_as.bin", 0x200000, 0x11b5b590, BRF_PRG },
};

STD_ROM_PICK(g_s_as);
STD_ROM_FN(g_s_as);

struct BurnDriverD BurnDrvMDg_s_as = {
	"g_s_as", NULL, NULL, "1995",
	"Spider-Man - The Animated Series (W) [!]\0", NULL, "Acclaim", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_s_asRomInfo, g_s_asRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Spider-Man - The Animated Series (Beta 1)
static struct BurnRomInfo g_s_asb1RomDesc[] = {
	{ "g_s_asb1.bin", 0x200000, 0xb88a710d, BRF_PRG },
};

STD_ROM_PICK(g_s_asb1);
STD_ROM_FN(g_s_asb1);

struct BurnDriverD BurnDrvMDg_s_asb1 = {
	"g_s_asb1", "g_s_as", NULL, "1995",
	"Spider-Man - The Animated Series (Beta 1)\0", NULL, "Acclaim", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_s_asb1RomInfo, g_s_asb1RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Spider-Man - The Animated Series (Beta 2)
static struct BurnRomInfo g_s_asb2RomDesc[] = {
	{ "g_s_asb2.bin", 0x200000, 0xb88a710d, BRF_PRG },
};

STD_ROM_PICK(g_s_asb2);
STD_ROM_FN(g_s_asb2);

struct BurnDriverD BurnDrvMDg_s_asb2 = {
	"g_s_asb2", "g_s_as", NULL, "1995",
	"Spider-Man - The Animated Series (Beta 2)\0", NULL, "Acclaim", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_s_asb2RomInfo, g_s_asb2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Spot Goes to Hollywood (U) (REV01) [!]
static struct BurnRomInfo g_spo2RomDesc[] = {
	{ "g_spo2.bin", 0x300000, 0xbdad1cbc, BRF_PRG },
};

STD_ROM_PICK(g_spo2);
STD_ROM_FN(g_spo2);

struct BurnDriverD BurnDrvMDg_spo2 = {
	"g_spo2", NULL, NULL, "199?",
	"Spot Goes to Hollywood (U) (REV01) [!]\0", NULL, "Virgin / Acclaim / Eurocom", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_spo2RomInfo, g_spo2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Spot Goes to Hollywood (E) [!]
static struct BurnRomInfo g_spot2eRomDesc[] = {
	{ "g_spot2e.bin", 0x300000, 0xfbe254ea, BRF_PRG },
};

STD_ROM_PICK(g_spot2e);
STD_ROM_FN(g_spot2e);

struct BurnDriverD BurnDrvMDg_spot2e = {
	"g_spot2e", "g_spo2", NULL, "199?",
	"Spot Goes to Hollywood (E) [!]\0", NULL, "Virgin / Acclaim / Eurocom", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_spot2eRomInfo, g_spot2eRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Steel Empire, The (UE) [!]
static struct BurnRomInfo g_sempRomDesc[] = {
	{ "g_semp.bin", 0x100000, 0xd0e7a0b6, BRF_PRG },
};

STD_ROM_PICK(g_semp);
STD_ROM_FN(g_semp);

struct BurnDriverD BurnDrvMDg_semp = {
	"g_semp", NULL, NULL, "1994",
	"Steel Empire, The (UE) [!]\0", NULL, "Hot B / Acclaim", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sempRomInfo, g_sempRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Koutetsu Teikoku (J) [!]
static struct BurnRomInfo g_koutRomDesc[] = {
	{ "g_kout.bin", 0x100000, 0x755d0b8a, BRF_PRG },
};

STD_ROM_PICK(g_kout);
STD_ROM_FN(g_kout);

struct BurnDriverD BurnDrvMDg_kout = {
	"g_kout", "g_semp", NULL, "1994",
	"Koutetsu Teikoku (J) [!]\0", NULL, "Hot B", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_koutRomInfo, g_koutRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Steel Empire, The (Beta)
static struct BurnRomInfo g_sempbRomDesc[] = {
	{ "g_sempb.bin", 0x100000, 0xe5517b77, BRF_PRG },
};

STD_ROM_PICK(g_sempb);
STD_ROM_FN(g_sempb);

struct BurnDriverD BurnDrvMDg_sempb = {
	"g_sempb", "g_semp", NULL, "1994",
	"Steel Empire, The (Beta)\0", NULL, "Hot B / Acclaim", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sempbRomInfo, g_sempbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Streets of Rage (W) (REV01) [!]
static struct BurnRomInfo g_sorRomDesc[] = {
	{ "g_sor.bin", 0x80000, 0x4052e845, BRF_PRG },
};

STD_ROM_PICK(g_sor);
STD_ROM_FN(g_sor);

struct BurnDriver BurnDrvMDg_sor = {
	"g_sor", NULL, NULL, "1991",
	"Streets of Rage (W) (REV01) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sorRomInfo, g_sorRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Streets of Rage (W) (REV00) [!]
static struct BurnRomInfo g_sor00RomDesc[] = {
	{ "g_sor00.bin", 0x80000, 0xbff227c6, BRF_PRG },
};

STD_ROM_PICK(g_sor00);
STD_ROM_FN(g_sor00);

struct BurnDriver BurnDrvMDg_sor00 = {
	"g_sor00", "g_sor", NULL, "1991",
	"Streets of Rage (W) (REV00) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sor00RomInfo, g_sor00RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Streets of Rage 2 (U) [!]
static struct BurnRomInfo g_sor2RomDesc[] = {
	{ "g_sor2.bin", 0x200000, 0xE01FA526, BRF_PRG },
};

STD_ROM_PICK(g_sor2);
STD_ROM_FN(g_sor2);

struct BurnDriver BurnDrvMDg_sor2 = {
	"g_sor2", NULL, NULL, "1992",
	"Streets of Rage 2 (U) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sor2RomInfo, g_sor2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Streets of Rage 2 (JE) [!]
static struct BurnRomInfo g_sor2jeRomDesc[] = {
	{ "g_sor2je.bin", 0x200000, 0x42e3efdc, BRF_PRG },
};

STD_ROM_PICK(g_sor2je);
STD_ROM_FN(g_sor2je);

struct BurnDriver BurnDrvMDg_sor2je = {
	"g_sor2je", "g_sor2", NULL, "1992",
	"Streets of Rage 2 (JE) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sor2jeRomInfo, g_sor2jeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Bare Knuckle II (Beta)
static struct BurnRomInfo g_bk2bRomDesc[] = {
	{ "g_bk2b.bin", 0x130000, 0x0cf2acbe, BRF_PRG },
};

STD_ROM_PICK(g_bk2b);
STD_ROM_FN(g_bk2b);

struct BurnDriver BurnDrvMDg_bk2b = {
	"g_bk2b", "g_sor2", NULL, "1992",
	"Bare Knuckle II (Beta)\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_bk2bRomInfo, g_bk2bRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Streets of Rage 3 (U) [!]
static struct BurnRomInfo g_sor3RomDesc[] = {
	{ "g_sor3.bin", 0x300000, 0xd5bb15d9, BRF_PRG },
};

STD_ROM_PICK(g_sor3);
STD_ROM_FN(g_sor3);

struct BurnDriver BurnDrvMDg_sor3 = {
	"g_sor3", NULL, NULL, "1994",
	"Streets of Rage 3 (U) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sor3RomInfo, g_sor3RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Streets of Rage 3 (E) (Apr 1994) [!]
static struct BurnRomInfo g_sor3eRomDesc[] = {
	{ "g_sor3e.bin", 0x300000, 0x3b78135f, BRF_PRG },
};

STD_ROM_PICK(g_sor3e);
STD_ROM_FN(g_sor3e);

struct BurnDriver BurnDrvMDg_sor3e = {
	"g_sor3e", "g_sor3", NULL, "1994",
	"Streets of Rage 3 (E) (Apr 1994) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sor3eRomInfo, g_sor3eRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Streets of Rage 3 (E) (May 1994) [!]
static struct BurnRomInfo g_sor3eaRomDesc[] = {
	{ "g_sor3ea.bin", 0x300000, 0x90ef991e, BRF_PRG },
};

STD_ROM_PICK(g_sor3ea);
STD_ROM_FN(g_sor3ea);

struct BurnDriver BurnDrvMDg_sor3ea = {
	"g_sor3ea", "g_sor3", NULL, "1994",
	"Streets of Rage 3 (E) (May 1994) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sor3eaRomInfo, g_sor3eaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Bare Knuckle III (J) [!]
static struct BurnRomInfo g_bar3RomDesc[] = {
	{ "g_bar3.bin", 0x300000, 0x5d09236f, BRF_PRG },
};

STD_ROM_PICK(g_bar3);
STD_ROM_FN(g_bar3);

struct BurnDriver BurnDrvMDg_bar3 = {
	"g_bar3", "g_sor3", NULL, "1994",
	"Bare Knuckle III (J) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_bar3RomInfo, g_bar3RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Bare Knuckle III (Beta)
static struct BurnRomInfo g_bk3bRomDesc[] = {
	{ "g_bk3b.bin", 0x300000, 0xe7ff99db, BRF_PRG },
};

STD_ROM_PICK(g_bk3b);
STD_ROM_FN(g_bk3b);

struct BurnDriver BurnDrvMDg_bk3b = {
	"g_bk3b", "g_sor3", NULL, "1994",
	"Bare Knuckle III (Beta)\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_bk3bRomInfo, g_bk3bRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Journey From Darkness - Strider Returns (U) [c][!]
static struct BurnRomInfo g_str2RomDesc[] = {
	{ "g_str2.bin", 0x100000, 0x42589b79, BRF_PRG },
};

STD_ROM_PICK(g_str2);
STD_ROM_FN(g_str2);

struct BurnDriverD BurnDrvMDg_str2 = {
	"g_str2", NULL, NULL, "1994",
	"Journey From Darkness - Strider Returns (U) [c][!]\0", NULL, "Tiertex / Capcom / US Gold", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_str2RomInfo, g_str2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Strider II (E) [c][!]
static struct BurnRomInfo g_strid2RomDesc[] = {
	{ "g_strid2.bin", 0x100000, 0xe85e5270, BRF_PRG },
};

STD_ROM_PICK(g_strid2);
STD_ROM_FN(g_strid2);

struct BurnDriverD BurnDrvMDg_strid2 = {
	"g_strid2", "g_str2", NULL, "1994",
	"Strider II (E) [c][!]\0", NULL, "Tiertex / Capcom / US Gold", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_strid2RomInfo, g_strid2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sub-Terrania (U) [!]
static struct BurnRomInfo g_subtRomDesc[] = {
	{ "g_subt.bin", 0x200000, 0xdc3c6c45, BRF_PRG },
};

STD_ROM_PICK(g_subt);
STD_ROM_FN(g_subt);

struct BurnDriverD BurnDrvMDg_subt = {
	"g_subt", NULL, NULL, "1993",
	"Sub-Terrania (U) [!]\0", NULL, "Zyrinx", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_subtRomInfo, g_subtRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sub-Terrania (E) [!]
static struct BurnRomInfo g_subteRomDesc[] = {
	{ "g_subte.bin", 0x200000, 0xe8ced28d, BRF_PRG },
};

STD_ROM_PICK(g_subte);
STD_ROM_FN(g_subte);

struct BurnDriverD BurnDrvMDg_subte = {
	"g_subte", "g_subt", NULL, "1993",
	"Sub-Terrania (E) [!]\0", NULL, "Zyrinx", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_subteRomInfo, g_subteRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sub-Terrania (J) [!]
static struct BurnRomInfo g_subtjRomDesc[] = {
	{ "g_subtj.bin", 0x200000, 0x7638ea91, BRF_PRG },
};

STD_ROM_PICK(g_subtj);
STD_ROM_FN(g_subtj);

struct BurnDriverD BurnDrvMDg_subtj = {
	"g_subtj", "g_subt", NULL, "1993",
	"Sub-Terrania (J) [!]\0", NULL, "Zyrinx", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_subtjRomInfo, g_subtjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sub-Terrania (Beta 1)
static struct BurnRomInfo g_subtb1RomDesc[] = {
	{ "g_subtb1.bin", 0x140000, 0x9c13d25c, BRF_PRG },
};

STD_ROM_PICK(g_subtb1);
STD_ROM_FN(g_subtb1);

struct BurnDriverD BurnDrvMDg_subtb1 = {
	"g_subtb1", "g_subt", NULL, "1993",
	"Sub-Terrania (Beta 1)\0", NULL, "Zyrinx", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_subtb1RomInfo, g_subtb1RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sub-Terrania (Beta 2)
static struct BurnRomInfo g_subtb2RomDesc[] = {
	{ "g_subtb2.bin", 0x200000, 0x3a1022d1, BRF_PRG },
};

STD_ROM_PICK(g_subtb2);
STD_ROM_FN(g_subtb2);

struct BurnDriverD BurnDrvMDg_subtb2 = {
	"g_subtb2", "g_subt", NULL, "1993",
	"Sub-Terrania (Beta 2)\0", NULL, "Zyrinx", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_subtb2RomInfo, g_subtb2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Super Street Fighter II - The New Challengers (U) [c][!]
static struct BurnRomInfo g_ssf2RomDesc[] = {
	{ "g_ssf2.bin", 0x500000, 0x165defbf, BRF_PRG },
};

STD_ROM_PICK(g_ssf2);
STD_ROM_FN(g_ssf2);

struct BurnDriverD BurnDrvMDg_ssf2 = {
	"g_ssf2", NULL, NULL, "1994",
	"Super Street Fighter II - The New Challengers (U) [c][!]\0", NULL, "Capcom", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ssf2RomInfo, g_ssf2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Super Street Fighter II - The New Challengers (E) [c][!]
static struct BurnRomInfo g_ssf2eRomDesc[] = {
	{ "g_ssf2e.bin", 0x500000, 0x682c192f, BRF_PRG },
};

STD_ROM_PICK(g_ssf2e);
STD_ROM_FN(g_ssf2e);

struct BurnDriverD BurnDrvMDg_ssf2e = {
	"g_ssf2e", "g_ssf2", NULL, "1994",
	"Super Street Fighter II - The New Challengers (E) [c][!]\0", NULL, "Capcom", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ssf2eRomInfo, g_ssf2eRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Super Street Fighter II - The New Challengers (J) [c][!]
static struct BurnRomInfo g_ssf2jRomDesc[] = {
	{ "g_ssf2j.bin", 0x500000, 0xd8eeb2bd, BRF_PRG },
};

STD_ROM_PICK(g_ssf2j);
STD_ROM_FN(g_ssf2j);

struct BurnDriverD BurnDrvMDg_ssf2j = {
	"g_ssf2j", "g_ssf2", NULL, "1994",
	"Super Street Fighter II - The New Challengers (J) [c][!]\0", NULL, "Capcom", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ssf2jRomInfo, g_ssf2jRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Lightening Force - Quest for the Darkstar (U) [c][!]
static struct BurnRomInfo g_lighfrRomDesc[] = {
	{ "g_lighfr.bin", 0x100000, 0xc8f8c0e0, BRF_PRG },
};

STD_ROM_PICK(g_lighfr);
STD_ROM_FN(g_lighfr);

struct BurnDriverD BurnDrvMDg_lighfr = {
	"g_lighfr", NULL, NULL, "1992",
	"Lightening Force - Quest for the Darkstar (U) [c][!]\0", NULL, "Tecnosoft", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_lighfrRomInfo, g_lighfrRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Thunder Force IV (J) [!]
static struct BurnRomInfo g_tf4RomDesc[] = {
	{ "g_tf4.bin", 0x100000, 0x8d606480, BRF_PRG },
};

STD_ROM_PICK(g_tf4);
STD_ROM_FN(g_tf4);

struct BurnDriverD BurnDrvMDg_tf4 = {
	"g_tf4", "g_lighfr", NULL, "1992",
	"Thunder Force IV (J) [!]\0", NULL, "Tecnosoft", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tf4RomInfo, g_tf4RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Thunder Force IV (E) [c][!]
static struct BurnRomInfo g_tf4eRomDesc[] = {
	{ "g_tf4e.bin", 0x100000, 0xe7e3c05b, BRF_PRG },
};

STD_ROM_PICK(g_tf4e);
STD_ROM_FN(g_tf4e);

struct BurnDriverD BurnDrvMDg_tf4e = {
	"g_tf4e", "g_lighfr", NULL, "1992",
	"Thunder Force IV (E) [c][!]\0", NULL, "Tecnosoft", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tf4eRomInfo, g_tf4eRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Toejam & Earl (U) (REV00) [!]
static struct BurnRomInfo g_tje00RomDesc[] = {
	{ "g_tje00.bin", 0x100000, 0xd1b36786, BRF_PRG },
};

STD_ROM_PICK(g_tje00);
STD_ROM_FN(g_tje00);

struct BurnDriver BurnDrvMDg_tje00 = {
	"g_tje00", NULL, NULL, "1991",
	"Toejam & Earl (U) (REV00) [!]\0", NULL, "Sega / JVP", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tje00RomInfo, g_tje00RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Toejam & Earl (J) (REV02) [!]
static struct BurnRomInfo g_tjeRomDesc[] = {
	{ "g_tje.bin", 0x100000, 0x7a588f4b, BRF_PRG },
};

STD_ROM_PICK(g_tje);
STD_ROM_FN(g_tje);

struct BurnDriver BurnDrvMDg_tje = {
	"g_tje", "g_tje00", NULL, "1991",
	"Toejam & Earl (J) (REV02) [!]\0", NULL, "Sega / JVP", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tjeRomInfo, g_tjeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Toejam & Earl in Panic on Funkotron (U) [!]
static struct BurnRomInfo g_tje2RomDesc[] = {
	{ "g_tje2.bin", 0x200000, 0xaa021bdd, BRF_PRG },
};

STD_ROM_PICK(g_tje2);
STD_ROM_FN(g_tje2);

struct BurnDriverD BurnDrvMDg_tje2 = {
	"g_tje2", NULL, NULL, "1993",
	"Toejam & Earl in Panic on Funkotron (U) [!]\0", NULL, "Sega / JVP", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tje2RomInfo, g_tje2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Toejam & Earl in Panic on Funkotron (E) [!]
static struct BurnRomInfo g_tje2eRomDesc[] = {
	{ "g_tje2e.bin", 0x200000, 0x47b0a871, BRF_PRG },
};

STD_ROM_PICK(g_tje2e);
STD_ROM_FN(g_tje2e);

struct BurnDriverD BurnDrvMDg_tje2e = {
	"g_tje2e", "g_tje2", NULL, "1993",
	"Toejam & Earl in Panic on Funkotron (E) [!]\0", NULL, "Sega / JVP", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tje2eRomInfo, g_tje2eRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Toejam & Earl in Panic on Funkotron (G) [!]
static struct BurnRomInfo g_tje2gRomDesc[] = {
	{ "g_tje2g.bin", 0x200000, 0x4081b9f2, BRF_PRG },
};

STD_ROM_PICK(g_tje2g);
STD_ROM_FN(g_tje2g);

struct BurnDriverD BurnDrvMDg_tje2g = {
	"g_tje2g", "g_tje2", NULL, "1993",
	"Toejam & Earl in Panic on Funkotron (G) [!]\0", NULL, "Sega / JVP", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tje2gRomInfo, g_tje2gRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Toejam & Earl in Panic on Funkotron (J) [!]
static struct BurnRomInfo g_tje2jRomDesc[] = {
	{ "g_tje2j.bin", 0x200000, 0xe1b36850, BRF_PRG },
};

STD_ROM_PICK(g_tje2j);
STD_ROM_FN(g_tje2j);

struct BurnDriverD BurnDrvMDg_tje2j = {
	"g_tje2j", "g_tje2", NULL, "1993",
	"Toejam & Earl in Panic on Funkotron (J) [!]\0", NULL, "Sega / JVP", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tje2jRomInfo, g_tje2jRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Tyrants - Fight Through Time (U) [!]
static struct BurnRomInfo g_tyraRomDesc[] = {
	{ "g_tyra.bin", 0x100000, 0xa744921e, BRF_PRG },
};

STD_ROM_PICK(g_tyra);
STD_ROM_FN(g_tyra);

struct BurnDriverD BurnDrvMDg_tyra = {
	"g_tyra", NULL, NULL, "1994",
	"Tyrants - Fight Through Time (U) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tyraRomInfo, g_tyraRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mega Lo Mania (E) (REV00) [c][!]
static struct BurnRomInfo g_mloRomDesc[] = {
	{ "g_mlo.bin", 0x100000, 0x2148d56d, BRF_PRG },
};

STD_ROM_PICK(g_mlo);
STD_ROM_FN(g_mlo);

struct BurnDriverD BurnDrvMDg_mlo = {
	"g_mlo", "g_tyra", NULL, "1994",
	"Mega Lo Mania (E) (REV00) [c][!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mloRomInfo, g_mloRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mega Lo Mania (E) (REV01)
static struct BurnRomInfo g_mlom01RomDesc[] = {
	{ "g_mlom01.bin", 0x100000, 0xab9fed30, BRF_PRG },
};

STD_ROM_PICK(g_mlom01);
STD_ROM_FN(g_mlom01);

struct BurnDriverD BurnDrvMDg_mlom01 = {
	"g_mlom01", "g_tyra", NULL, "1994",
	"Mega Lo Mania (E) (REV01)\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mlom01RomInfo, g_mlom01RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mega Lo Mania (J) [!]
static struct BurnRomInfo g_mlojRomDesc[] = {
	{ "g_mloj.bin", 0x100000, 0xa60d8619, BRF_PRG },
};

STD_ROM_PICK(g_mloj);
STD_ROM_FN(g_mloj);

struct BurnDriverD BurnDrvMDg_mloj = {
	"g_mloj", "g_tyra", NULL, "1994",
	"Mega Lo Mania (J) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mlojRomInfo, g_mlojRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Vectorman (UE) [!]
static struct BurnRomInfo g_vectRomDesc[] = {
	{ "g_vect.bin", 0x200000, 0xd38b3354, BRF_PRG },
};

STD_ROM_PICK(g_vect);
STD_ROM_FN(g_vect);

struct BurnDriverD BurnDrvMDg_vect = {
	"g_vect", NULL, NULL, "1995",
	"Vectorman (UE) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_vectRomInfo, g_vectRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Vectorman (UE) (Prototype) [!]
static struct BurnRomInfo g_vectpRomDesc[] = {
	{ "g_vectp.bin", 0x200000, 0xa315c8aa, BRF_PRG },
};

STD_ROM_PICK(g_vectp);
STD_ROM_FN(g_vectp);

struct BurnDriverD BurnDrvMDg_vectp = {
	"g_vectp", "g_vect", NULL, "1995",
	"Vectorman (UE) (Prototype) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_vectpRomInfo, g_vectpRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Vectorman (Beta)
static struct BurnRomInfo g_vectbRomDesc[] = {
	{ "g_vectb.bin", 0x200000, 0x2084d3da, BRF_PRG },
};

STD_ROM_PICK(g_vectb);
STD_ROM_FN(g_vectb);

struct BurnDriverD BurnDrvMDg_vectb = {
	"g_vectb", "g_vect", NULL, "1995",
	"Vectorman (Beta)\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_vectbRomInfo, g_vectbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Vectorman 2 (U) [!]
static struct BurnRomInfo g_vec2RomDesc[] = {
	{ "g_vec2.bin", 0x300000, 0xc1a24088, BRF_PRG },
};

STD_ROM_PICK(g_vec2);
STD_ROM_FN(g_vec2);

struct BurnDriverD BurnDrvMDg_vec2 = {
	"g_vec2", NULL, NULL, "1996",
	"Vectorman 2 (U) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_vec2RomInfo, g_vec2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Vectorman 2 (U) (Prototype) [!]
static struct BurnRomInfo g_vect2pRomDesc[] = {
	{ "g_vect2p.bin", 0x200000, 0xada2b0ef, BRF_PRG },
};

STD_ROM_PICK(g_vect2p);
STD_ROM_FN(g_vect2p);

struct BurnDriverD BurnDrvMDg_vect2p = {
	"g_vect2p", "g_vec2", NULL, "1996",
	"Vectorman 2 (U) (Prototype) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_vect2pRomInfo, g_vect2pRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Wani Wani World (J) [c][!]
static struct BurnRomInfo g_waniRomDesc[] = {
	{ "g_wani.bin", 0x80000, 0x56f0dbb2, BRF_PRG },
};

STD_ROM_PICK(g_wani);
STD_ROM_FN(g_wani);

struct BurnDriverD BurnDrvMDg_wani = {
	"g_wani", NULL, NULL, "1992",
	"Wani Wani World (J) [c][!]\0", NULL, "Kaneko", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_waniRomInfo, g_waniRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Warlock (UE) [!]
static struct BurnRomInfo g_wlocRomDesc[] = {
	{ "g_wloc.bin", 0x200000, 0x0a46539b, BRF_PRG },
};

STD_ROM_PICK(g_wloc);
STD_ROM_FN(g_wloc);

struct BurnDriverD BurnDrvMDg_wloc = {
	"g_wloc", NULL, NULL, "1994",
	"Warlock (UE) [!]\0", NULL, "Trimark / Acclaim", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wlocRomInfo, g_wlocRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Warlock (Beta)
static struct BurnRomInfo g_warlobRomDesc[] = {
	{ "g_warlob.bin", 0x200000, 0xc9b6edb3, BRF_PRG },
};

STD_ROM_PICK(g_warlob);
STD_ROM_FN(g_warlob);

struct BurnDriverD BurnDrvMDg_warlob = {
	"g_warlob", "g_wloc", NULL, "1994",
	"Warlock (Beta)\0", NULL, "Trimark / Acclaim", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_warlobRomInfo, g_warlobRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Warrior of Rome (U) [!]
static struct BurnRomInfo g_wromRomDesc[] = {
	{ "g_wrom.bin", 0x100000, 0x5be10c6a, BRF_PRG },
};

STD_ROM_PICK(g_wrom);
STD_ROM_FN(g_wrom);

struct BurnDriverD BurnDrvMDg_wrom = {
	"g_wrom", NULL, NULL, "1991",
	"Warrior of Rome (U) [!]\0", NULL, "Micronet", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wromRomInfo, g_wromRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Caesar no Yabo (J)
static struct BurnRomInfo g_caenoRomDesc[] = {
	{ "g_caeno.bin", 0x100000, 0x69796e93, BRF_PRG },
};

STD_ROM_PICK(g_caeno);
STD_ROM_FN(g_caeno);

struct BurnDriverD BurnDrvMDg_caeno = {
	"g_caeno", "g_wrom", NULL, "1991",
	"Caesar no Yabo (J)\0", NULL, "Micronet", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_caenoRomInfo, g_caenoRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Warrior of Rome II (U) [c][!]
static struct BurnRomInfo g_wrom2RomDesc[] = {
	{ "g_wrom2.bin", 0x100000, 0xcd8c472a, BRF_PRG },
};

STD_ROM_PICK(g_wrom2);
STD_ROM_FN(g_wrom2);

struct BurnDriverD BurnDrvMDg_wrom2 = {
	"g_wrom2", NULL, NULL, "1992",
	"Warrior of Rome II (U) [c][!]\0", NULL, "Micronet", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wrom2RomInfo, g_wrom2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Caesar no Yabo II (J)
static struct BurnRomInfo g_caeno2RomDesc[] = {
	{ "g_caeno2.bin", 0x100000, 0x4f327b3a, BRF_PRG },
};

STD_ROM_PICK(g_caeno2);
STD_ROM_FN(g_caeno2);

struct BurnDriverD BurnDrvMDg_caeno2 = {
	"g_caeno2", "g_wrom2", NULL, "1992",
	"Caesar no Yabo II (J)\0", NULL, "Micronet", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_caeno2RomInfo, g_caeno2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Wings of Wor (U) [!]
static struct BurnRomInfo g_winworRomDesc[] = {
	{ "g_winwor.bin", 0x80000, 0x210a2fcd, BRF_PRG },
};

STD_ROM_PICK(g_winwor);
STD_ROM_FN(g_winwor);

struct BurnDriverD BurnDrvMDg_winwor = {
	"g_winwor", NULL, NULL, "1991",
	"Wings of Wor (U) [!]\0", NULL, "NCS / Dreamworks", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_winworRomInfo, g_winworRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Gynoug (E) [!]
static struct BurnRomInfo g_gynogeRomDesc[] = {
	{ "g_gynoge.bin", 0x80000, 0x03405102, BRF_PRG },
};

STD_ROM_PICK(g_gynoge);
STD_ROM_FN(g_gynoge);

struct BurnDriverD BurnDrvMDg_gynoge = {
	"g_gynoge", "g_winwor", NULL, "1991",
	"Gynoug (E) [!]\0", NULL, "NCS", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_gynogeRomInfo, g_gynogeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Gynoug (J) [!]
static struct BurnRomInfo g_gynoRomDesc[] = {
	{ "g_gyno.bin", 0x80000, 0x1b69241f, BRF_PRG },
};

STD_ROM_PICK(g_gyno);
STD_ROM_FN(g_gyno);

struct BurnDriverD BurnDrvMDg_gyno = {
	"g_gyno", "g_winwor", NULL, "1991",
	"Gynoug (J) [!]\0", NULL, "NCS", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_gynoRomInfo, g_gynoRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// World Championship Soccer (JU) (REV03) [!]
static struct BurnRomInfo g_wcsRomDesc[] = {
	{ "g_wcs.bin", 0x40000, 0xbf272bcb, BRF_PRG },
};

STD_ROM_PICK(g_wcs);
STD_ROM_FN(g_wcs);

struct BurnDriverD BurnDrvMDg_wcs = {
	"g_wcs", NULL, NULL, "1994",
	"World Championship Soccer (JU) (REV03) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wcsRomInfo, g_wcsRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// World Championship Soccer (JU) (REV02)
static struct BurnRomInfo g_wcs02RomDesc[] = {
	{ "g_wcs02.bin", 0x40000, 0xbf84ede6, BRF_PRG },
};

STD_ROM_PICK(g_wcs02);
STD_ROM_FN(g_wcs02);

struct BurnDriverD BurnDrvMDg_wcs02 = {
	"g_wcs02", "g_wcs", NULL, "1994",
	"World Championship Soccer (JU) (REV02)\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wcs02RomInfo, g_wcs02RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// World Championship Soccer (JU) (REV00) [!]
static struct BurnRomInfo g_wcs00RomDesc[] = {
	{ "g_wcs00.bin", 0x40000, 0xb01c3d70, BRF_PRG },
};

STD_ROM_PICK(g_wcs00);
STD_ROM_FN(g_wcs00);

struct BurnDriverD BurnDrvMDg_wcs00 = {
	"g_wcs00", "g_wcs", NULL, "1994",
	"World Championship Soccer (JU) (REV00) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wcs00RomInfo, g_wcs00RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// World Cup Italia 90 (E) [!]
static struct BurnRomInfo g_wc90RomDesc[] = {
	{ "g_wc90.bin", 0x40000, 0xdd95f829, BRF_PRG },
};

STD_ROM_PICK(g_wc90);
STD_ROM_FN(g_wc90);

struct BurnDriverD BurnDrvMDg_wc90 = {
	"g_wc90", "g_wcs", NULL, "1994",
	"World Cup Italia 90 (E) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wc90RomInfo, g_wc90RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Wild Snake (U) (Prototype) [!]
static struct BurnRomInfo g_wildRomDesc[] = {
	{ "g_wild.bin", 0x80000, 0x0c1a49e5, BRF_PRG },
};

STD_ROM_PICK(g_wild);
STD_ROM_FN(g_wild);

struct BurnDriverD BurnDrvMDg_wild = {
	"g_wild", NULL, NULL, "1994",
	"Wild Snake (U) (Prototype) [!]\0", NULL, "Bullet-Proof Software", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wildRomInfo, g_wildRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// World of Illusion Starring Mickey Mouse & Donald Duck (U) [!]
static struct BurnRomInfo g_willRomDesc[] = {
	{ "g_will.bin", 0x100000, 0x921ebd1c, BRF_PRG },
};

STD_ROM_PICK(g_will);
STD_ROM_FN(g_will);

struct BurnDriverD BurnDrvMDg_will = {
	"g_will", NULL, NULL, "1992",
	"World of Illusion Starring Mickey Mouse & Donald Duck (U) [!]\0", NULL, "Disney / Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_willRomInfo, g_willRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// World of Illusion Starring Mickey Mouse & Donald Duck (E) [!]
static struct BurnRomInfo g_willeRomDesc[] = {
	{ "g_wille.bin", 0x100000, 0x121c6a49, BRF_PRG },
};

STD_ROM_PICK(g_wille);
STD_ROM_FN(g_wille);

struct BurnDriverD BurnDrvMDg_wille = {
	"g_wille", "g_will", NULL, "1992",
	"World of Illusion Starring Mickey Mouse & Donald Duck (E) [!]\0", NULL, "Disney / Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_willeRomInfo, g_willeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// World of Illusion Starring Mickey Mouse & Donald Duck (Beta)
static struct BurnRomInfo g_willbRomDesc[] = {
	{ "g_willb.bin", 0x100000, 0x577f680f, BRF_PRG },
};

STD_ROM_PICK(g_willb);
STD_ROM_FN(g_willb);

struct BurnDriverD BurnDrvMDg_willb = {
	"g_willb", "g_will", NULL, "1992",
	"World of Illusion Starring Mickey Mouse & Donald Duck (Beta)\0", NULL, "Disney / Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_willbRomInfo, g_willbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// I Love Mickey & Donald - Fushigi na Magic Box (J)
static struct BurnRomInfo g_micdoRomDesc[] = {
	{ "g_micdo.bin", 0x100000, 0xcb9ee238, BRF_PRG },
};

STD_ROM_PICK(g_micdo);
STD_ROM_FN(g_micdo);

struct BurnDriverD BurnDrvMDg_micdo = {
	"g_micdo", "g_will", NULL, "1992",
	"I Love Mickey & Donald - Fushigi na Magic Box (J)\0", NULL, "Disney / Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_micdoRomInfo, g_micdoRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// X-perts (U) [!]
static struct BurnRomInfo g_xperRomDesc[] = {
	{ "g_xper.bin", 0x400000, 0x57e8abfd, BRF_PRG },
};

STD_ROM_PICK(g_xper);
STD_ROM_FN(g_xper);

struct BurnDriverD BurnDrvMDg_xper = {
	"g_xper", NULL, NULL, "1996",
	"X-perts (U) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_xperRomInfo, g_xperRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Virtua Racing (U) [!]
static struct BurnRomInfo g_virrRomDesc[] = {
	{ "g_virr.bin", 0x200000, 0x7e1a324a, BRF_PRG },
};

STD_ROM_PICK(g_virr);
STD_ROM_FN(g_virr);

struct BurnDriverD BurnDrvMDg_virr = {
	"g_virr", NULL, NULL, "199?",
	"Virtua Racing (U) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_virrRomInfo, g_virrRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Virtua Racing (J) [!]
static struct BurnRomInfo g_virrajRomDesc[] = {
	{ "g_virraj.bin", 0x200000, 0x53a293b5, BRF_PRG },
};

STD_ROM_PICK(g_virraj);
STD_ROM_FN(g_virraj);

struct BurnDriverD BurnDrvMDg_virraj = {
	"g_virraj", "g_virr", NULL, "199?",
	"Virtua Racing (J) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_virrajRomInfo, g_virrajRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Virtua Racing (E) [!]
static struct BurnRomInfo g_virraeRomDesc[] = {
	{ "g_virrae.bin", 0x200000, 0x9624d4ef, BRF_PRG },
};

STD_ROM_PICK(g_virrae);
STD_ROM_FN(g_virrae);

struct BurnDriverD BurnDrvMDg_virrae = {
	"g_virrae", "g_virr", NULL, "199?",
	"Virtua Racing (E) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_virraeRomInfo, g_virraeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Virtua Racing (E) [a1]
static struct BurnRomInfo g_virreaRomDesc[] = {
	{ "g_virrea.bin", 0x200000, 0x5a943df9, BRF_PRG },
};

STD_ROM_PICK(g_virrea);
STD_ROM_FN(g_virrea);

struct BurnDriverD BurnDrvMDg_virrea = {
	"g_virrea", "g_virr", NULL, "199?",
	"Virtua Racing (E) [a1]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_virreaRomInfo, g_virreaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Disney Collection - Castle of Illusion & Quack Shot (E) [!]
static struct BurnRomInfo g_discolRomDesc[] = {
	{ "g_discol.bin", 0x100000, 0xadfde883, BRF_PRG },
};

STD_ROM_PICK(g_discol);
STD_ROM_FN(g_discol);

struct BurnDriverD BurnDrvMDg_discol = {
	"g_discol", NULL, NULL, "1991",
	"Disney Collection - Castle of Illusion & Quack Shot (E) [!]\0", NULL, "Disney / Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_discolRomInfo, g_discolRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Classic Collection (UE) [!]
static struct BurnRomInfo g_clasclRomDesc[] = {
	{ "g_clascl.bin", 0x200000, 0x73f948b1, BRF_PRG },
};

STD_ROM_PICK(g_clascl);
STD_ROM_FN(g_clascl);

struct BurnDriverD BurnDrvMDg_clascl = {
	"g_clascl", NULL, NULL, "199?",
	"Classic Collection (UE) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_clasclRomInfo, g_clasclRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Genesis 6-Pak (U) [!]
static struct BurnRomInfo g_gen6pkRomDesc[] = {
	{ "g_gen6pk.bin", 0x300000, 0x1a6f45dc, BRF_PRG },
};

STD_ROM_PICK(g_gen6pk);
STD_ROM_FN(g_gen6pk);

struct BurnDriverD BurnDrvMDg_gen6pk = {
	"g_gen6pk", NULL, NULL, "199?",
	"Genesis 6-Pak (U) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_gen6pkRomInfo, g_gen6pkRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mega Games I (E) [!]
static struct BurnRomInfo g_mg1eRomDesc[] = {
	{ "g_mg1e.bin", 0x100000, 0xdb753224, BRF_PRG },
};

STD_ROM_PICK(g_mg1e);
STD_ROM_FN(g_mg1e);

struct BurnDriverD BurnDrvMDg_mg1e = {
	"g_mg1e", NULL, NULL, "199?",
	"Mega Games I (E) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mg1eRomInfo, g_mg1eRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mega Games 2 (E) [!]
static struct BurnRomInfo g_mg2eRomDesc[] = {
	{ "g_mg2e.bin", 0x200000, 0x30d59f2f, BRF_PRG },
};

STD_ROM_PICK(g_mg2e);
STD_ROM_FN(g_mg2e);

struct BurnDriverD BurnDrvMDg_mg2e = {
	"g_mg2e", NULL, NULL, "199?",
	"Mega Games 2 (E) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mg2eRomInfo, g_mg2eRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mega Games 3 (E) [!]
static struct BurnRomInfo g_mg3eRomDesc[] = {
	{ "g_mg3e.bin", 0x200000, 0xb4247d98, BRF_PRG },
};

STD_ROM_PICK(g_mg3e);
STD_ROM_FN(g_mg3e);

struct BurnDriverD BurnDrvMDg_mg3e = {
	"g_mg3e", NULL, NULL, "199?",
	"Mega Games 3 (E) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mg3eRomInfo, g_mg3eRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mega Games 6 (Vol 1) (E) [!]
static struct BurnRomInfo g_mg6v1eRomDesc[] = {
	{ "g_mg6v1e.bin", 0x300000, 0xb66fb80d, BRF_PRG },
};

STD_ROM_PICK(g_mg6v1e);
STD_ROM_FN(g_mg6v1e);

struct BurnDriverD BurnDrvMDg_mg6v1e = {
	"g_mg6v1e", NULL, NULL, "199?",
	"Mega Games 6 (Vol 1) (E) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mg6v1eRomInfo, g_mg6v1eRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mega Games 6 (Vol 2) (E) [!]
static struct BurnRomInfo g_mg6v2eRomDesc[] = {
	{ "g_mg6v2e.bin", 0x300000, 0xe8d10db9, BRF_PRG },
};

STD_ROM_PICK(g_mg6v2e);
STD_ROM_FN(g_mg6v2e);

struct BurnDriverD BurnDrvMDg_mg6v2e = {
	"g_mg6v2e", NULL, NULL, "199?",
	"Mega Games 6 (Vol 2) (E) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mg6v2eRomInfo, g_mg6v2eRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mega Games 6 (Vol 3) (E) [!]
static struct BurnRomInfo g_mg6v3eRomDesc[] = {
	{ "g_mg6v3e.bin", 0x300000, 0xfe3e7e4f, BRF_PRG },
};

STD_ROM_PICK(g_mg6v3e);
STD_ROM_FN(g_mg6v3e);

struct BurnDriverD BurnDrvMDg_mg6v3e = {
	"g_mg6v3e", NULL, NULL, "199?",
	"Mega Games 6 (Vol 3) (E) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mg6v3eRomInfo, g_mg6v3eRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mega Games 10 in 1 (EB) [!]
static struct BurnRomInfo g_mg10i1RomDesc[] = {
	{ "g_mg10i1.bin", 0x400000, 0xc19ae368, BRF_PRG },
};

STD_ROM_PICK(g_mg10i1);
STD_ROM_FN(g_mg10i1);

struct BurnDriverD BurnDrvMDg_mg10i1 = {
	"g_mg10i1", NULL, NULL, "199?",
	"Mega Games 10 in 1 (EB) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mg10i1RomInfo, g_mg10i1RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sega Sports 1 (Wimbledon, Ult.Soccer, Super Monaco) (E) [!].
static struct BurnRomInfo g_segsp1RomDesc[] = {
	{ "g_segsp1.bin", 0x280000, 0x07fedaf1, BRF_PRG },
};

STD_ROM_PICK(g_segsp1);
STD_ROM_FN(g_segsp1);

struct BurnDriverD BurnDrvMDg_segsp1 = {
	"g_segsp1", NULL, NULL, "199?",
	"Sega Sports 1 (Wimbledon, Ult.Soccer, Super Monaco) (E) [!].\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_segsp1RomInfo, g_segsp1RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sega Top 5 (B) [!]
static struct BurnRomInfo g_stop5bRomDesc[] = {
	{ "g_stop5b.bin", 0x200000, 0x61069069, BRF_PRG },
};

STD_ROM_PICK(g_stop5b);
STD_ROM_FN(g_stop5b);

struct BurnDriverD BurnDrvMDg_stop5b = {
	"g_stop5b", NULL, NULL, "199?",
	"Sega Top 5 (B) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_stop5bRomInfo, g_stop5bRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sonic Classics (Compilation) (UE) (REV01) [!]
static struct BurnRomInfo g_soncluRomDesc[] = {
	{ "g_sonclu.bin", 0x300000, 0x95b5e8d7, BRF_PRG },
};

STD_ROM_PICK(g_sonclu);
STD_ROM_FN(g_sonclu);

struct BurnDriverD BurnDrvMDg_sonclu = {
	"g_sonclu", NULL, NULL, "199?",
	"Sonic Classics (Compilation) (UE) (REV01) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_soncluRomInfo, g_soncluRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sonic Classics (Compilation) (E) (REV00) [!]
static struct BurnRomInfo g_soncleRomDesc[] = {
	{ "g_soncle.bin", 0x300000, 0x8c70b84e, BRF_PRG },
};

STD_ROM_PICK(g_soncle);
STD_ROM_FN(g_soncle);

struct BurnDriverD BurnDrvMDg_soncle = {
	"g_soncle", "g_sonclu", NULL, "199?",
	"Sonic Classics (Compilation) (E) (REV00) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_soncleRomInfo, g_soncleRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sport Games (B) [c][!]
static struct BurnRomInfo g_sptgbRomDesc[] = {
	{ "g_sptgb.bin", 0x200000, 0x7e3ecabf, BRF_PRG },
};

STD_ROM_PICK(g_sptgb);
STD_ROM_FN(g_sptgb);

struct BurnDriverD BurnDrvMDg_sptgb = {
	"g_sptgb", NULL, NULL, "199?",
	"Sport Games (B) [c][!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sptgbRomInfo, g_sptgbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Menacer 6-in-1 Game Pack (UE) [!]
static struct BurnRomInfo g_menaceRomDesc[] = {
	{ "g_menace.bin", 0x100000, 0x936b85f7, BRF_PRG },
};

STD_ROM_PICK(g_menace);
STD_ROM_FN(g_menace);

struct BurnDriverD BurnDrvMDg_menace = {
	"g_menace", NULL, NULL, "199?",
	"Menacer 6-in-1 Game Pack (UE) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_menaceRomInfo, g_menaceRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Radica: Volume 1 (Sonic the Hedgehog, Altered Beast, Golden Axe, Dr. Robotnik's Mean Bean Machine, Kid Chameleon, Flicky) (US)
static struct BurnRomInfo radicav1RomDesc[] = {
	{ "radicav1.bin", 0x400000, 0x3b4c8438, BRF_PRG },
};

STD_ROM_PICK(radicav1);
STD_ROM_FN(radicav1);

struct BurnDriverD BurnDrvMDradicav1 = {
	"radicav1", NULL, NULL, "2004",
	"Radica: Volume 1 (Sonic the Hedgehog, Altered Beast, Golden Axe, Dr. Robotnik's Mean Bean Machine, Kid Chameleon, Flicky) (US)\0", NULL, "Radica Games / Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, radicav1RomInfo, radicav1RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Radica: Street Fighter Pack (Street Fighter 2' Special Champion Edition, Ghouls and Ghosts) (EURO)
static struct BurnRomInfo radicasfRomDesc[] = {
	{ "radicasf.bin", 0x400000, 0x868afb44, BRF_PRG },
};

STD_ROM_PICK(radicasf);
STD_ROM_FN(radicasf);

struct BurnDriverD BurnDrvMDradicasf = {
	"radicasf", NULL, NULL, "2004",
	"Radica: Street Fighter Pack (Street Fighter 2' Special Champion Edition, Ghouls and Ghosts) (EURO)\0", NULL, "Radica Games / Capcom", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, radicasfRomInfo, radicasfRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Game no Kanzume Otokuyou (J) [!]
static struct BurnRomInfo g_gamenoRomDesc[] = {
	{ "g_gameno.bin", 0x300000, 0xcdad7e6b, BRF_PRG },
};

STD_ROM_PICK(g_gameno);
STD_ROM_FN(g_gameno);

struct BurnDriverD BurnDrvMDg_gameno = {
	"g_gameno", NULL, NULL, "1994",
	"Game no Kanzume Otokuyou (J) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_gamenoRomInfo, g_gamenoRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Doki Doki Penguin Land MD (SN) (J) [!]
static struct BurnRomInfo g_ddplRomDesc[] = {
	{ "g_ddpl.bin", 0x40000, 0x0053bfd6, BRF_PRG },
};

STD_ROM_PICK(g_ddpl);
STD_ROM_FN(g_ddpl);

struct BurnDriverD BurnDrvMDg_ddpl = {
	"g_ddpl", NULL, NULL, "1994",
	"Doki Doki Penguin Land MD (SN) (J) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ddplRomInfo, g_ddplRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// 16 Ton (SN) (J) [!]
static struct BurnRomInfo g_16tonRomDesc[] = {
	{ "g_16ton.bin", 0x40000, 0x537F04B6, BRF_PRG },
};

STD_ROM_PICK(g_16ton);
STD_ROM_FN(g_16ton);

struct BurnDriverD BurnDrvMDg_16ton = {
	"g_16ton", NULL, NULL, "1991",
	"16 Ton (SN) (J) [!]\0", "Sound Problems", "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_16tonRomInfo, g_16tonRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Hyper Marbles (SN) (J) [!]
static struct BurnRomInfo g_hymarRomDesc[] = {
	{ "g_hymar.bin", 0x40000, 0x83bb2799, BRF_PRG },
};

STD_ROM_PICK(g_hymar);
STD_ROM_FN(g_hymar);

struct BurnDriverD BurnDrvMDg_hymar = {
	"g_hymar", NULL, NULL, "1994",
	"Hyper Marbles (SN) (J) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_hymarRomInfo, g_hymarRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Labyrinth of Death (SN) (J) [!]
static struct BurnRomInfo g_labydRomDesc[] = {
	{ "g_labyd.bin", 0x40000, 0xa6d7e02d, BRF_PRG },
};

STD_ROM_PICK(g_labyd);
STD_ROM_FN(g_labyd);

struct BurnDriverD BurnDrvMDg_labyd = {
	"g_labyd", NULL, NULL, "1994",
	"Labyrinth of Death (SN) (J) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_labydRomInfo, g_labydRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Paddle Fighter (SN) (J) [!]
static struct BurnRomInfo g_padfiRomDesc[] = {
	{ "g_padfi.bin", 0x40000, 0x3d8147e6, BRF_PRG },
};

STD_ROM_PICK(g_padfi);
STD_ROM_FN(g_padfi);

struct BurnDriverD BurnDrvMDg_padfi = {
	"g_padfi", NULL, NULL, "1994",
	"Paddle Fighter (SN) (J) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_padfiRomInfo, g_padfiRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Phantasy Star II - Anne's Adventure (SN) (J) [!]
static struct BurnRomInfo g_p2anneRomDesc[] = {
	{ "g_p2anne.bin", 0x40000, 0xfafa5b6f, BRF_PRG },
};

STD_ROM_PICK(g_p2anne);
STD_ROM_FN(g_p2anne);

struct BurnDriverD BurnDrvMDg_p2anne = {
	"g_p2anne", NULL, NULL, "1994",
	"Phantasy Star II - Anne's Adventure (SN) (J) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_p2anneRomInfo, g_p2anneRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Phantasy Star II - Huey's Adventure (SN) (J) [!]
static struct BurnRomInfo g_p2hueyRomDesc[] = {
	{ "g_p2huey.bin", 0x40000, 0x1a076f83, BRF_PRG },
};

STD_ROM_PICK(g_p2huey);
STD_ROM_FN(g_p2huey);

struct BurnDriverD BurnDrvMDg_p2huey = {
	"g_p2huey", NULL, NULL, "1994",
	"Phantasy Star II - Huey's Adventure (SN) (J) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_p2hueyRomInfo, g_p2hueyRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Phantasy Star II - Kinds's Adventure (SN) (J) [!]
static struct BurnRomInfo g_p2kindRomDesc[] = {
	{ "g_p2kind.bin", 0x40000, 0xc334f308, BRF_PRG },
};

STD_ROM_PICK(g_p2kind);
STD_ROM_FN(g_p2kind);

struct BurnDriverD BurnDrvMDg_p2kind = {
	"g_p2kind", NULL, NULL, "1994",
	"Phantasy Star II - Kinds's Adventure (SN) (J) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_p2kindRomInfo, g_p2kindRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Phantasy Star II - Shilka's Adventure (SN) (J) [!]
static struct BurnRomInfo g_p2shilRomDesc[] = {
	{ "g_p2shil.bin", 0x40000, 0x1f83beb2, BRF_PRG },
};

STD_ROM_PICK(g_p2shil);
STD_ROM_FN(g_p2shil);

struct BurnDriverD BurnDrvMDg_p2shil = {
	"g_p2shil", NULL, NULL, "1994",
	"Phantasy Star II - Shilka's Adventure (SN) (J) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_p2shilRomInfo, g_p2shilRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Putter Golf (SN) (J) [!]
static struct BurnRomInfo g_putterRomDesc[] = {
	{ "g_putter.bin", 0x40000, 0x20f168a6, BRF_PRG },
};

STD_ROM_PICK(g_putter);
STD_ROM_FN(g_putter);

struct BurnDriverD BurnDrvMDg_putter = {
	"g_putter", NULL, NULL, "1994",
	"Putter Golf (SN) (J) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_putterRomInfo, g_putterRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Pyramid Magic (SN) (J) [!]
static struct BurnRomInfo g_pymagRomDesc[] = {
	{ "g_pymag.bin", 0x40000, 0x306d839e, BRF_PRG },
};

STD_ROM_PICK(g_pymag);
STD_ROM_FN(g_pymag);

struct BurnDriverD BurnDrvMDg_pymag = {
	"g_pymag", NULL, NULL, "1994",
	"Pyramid Magic (SN) (J) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pymagRomInfo, g_pymagRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Pyramid Magic II (SN) (J) [!]
static struct BurnRomInfo g_pymag2RomDesc[] = {
	{ "g_pymag2.bin", 0x40000, 0xc9ddac72, BRF_PRG },
};

STD_ROM_PICK(g_pymag2);
STD_ROM_FN(g_pymag2);

struct BurnDriverD BurnDrvMDg_pymag2 = {
	"g_pymag2", NULL, NULL, "1994",
	"Pyramid Magic II (SN) (J) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pymag2RomInfo, g_pymag2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Pyramid Magic III (SN) (J) [!]
static struct BurnRomInfo g_pymag3RomDesc[] = {
	{ "g_pymag3.bin", 0x40000, 0x8329820a, BRF_PRG },
};

STD_ROM_PICK(g_pymag3);
STD_ROM_FN(g_pymag3);

struct BurnDriverD BurnDrvMDg_pymag3 = {
	"g_pymag3", NULL, NULL, "1994",
	"Pyramid Magic III (SN) (J) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pymag3RomInfo, g_pymag3RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Pyramid Magic Special (SN) (J) [!]
static struct BurnRomInfo g_pymagsRomDesc[] = {
	{ "g_pymags.bin", 0x40000, 0x153a3afa, BRF_PRG },
};

STD_ROM_PICK(g_pymags);
STD_ROM_FN(g_pymags);

struct BurnDriverD BurnDrvMDg_pymags = {
	"g_pymags", NULL, NULL, "1994",
	"Pyramid Magic Special (SN) (J) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pymagsRomInfo, g_pymagsRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sonic Eraser (SN) (J) [!]
static struct BurnRomInfo g_seraseRomDesc[] = {
	{ "g_serase.bin", 0x40000, 0x62d8a0e7, BRF_PRG },
};

STD_ROM_PICK(g_serase);
STD_ROM_FN(g_serase);

struct BurnDriverD BurnDrvMDg_serase = {
	"g_serase", NULL, NULL, "1994",
	"Sonic Eraser (SN) (J) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_seraseRomInfo, g_seraseRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// MegaMind (SN) (J)
static struct BurnRomInfo g_megamdRomDesc[] = {
	{ "g_megamd.bin", 0x40000, 0x76df2ae2, BRF_PRG },
};

STD_ROM_PICK(g_megamd);
STD_ROM_FN(g_megamd);

struct BurnDriverD BurnDrvMDg_megamd = {
	"g_megamd", NULL, NULL, "1900",
	"MegaMind (SN) (J)\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_megamdRomInfo, g_megamdRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Aworg (SN) (J)
static struct BurnRomInfo g_aworgRomDesc[] = {
	{ "g_aworg.bin", 0x40000, 0x069c27c1, BRF_PRG },
};

STD_ROM_PICK(g_aworg);
STD_ROM_FN(g_aworg);

struct BurnDriver BurnDrvMDg_aworg = {
	"g_aworg", NULL, NULL, "1900",
	"Aworg (SN) (J)\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_aworgRomInfo, g_aworgRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Teddy Boy Blues (SN) (J)
static struct BurnRomInfo g_teddyRomDesc[] = {
	{ "g_teddy.bin", 0x40000, 0x733d2eb3, BRF_PRG },
};

STD_ROM_PICK(g_teddy);
STD_ROM_FN(g_teddy);

struct BurnDriverD BurnDrvMDg_teddy = {
	"g_teddy", NULL, NULL, "1900",
	"Teddy Boy Blues (SN) (J)\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_teddyRomInfo, g_teddyRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Robot Battler (SN) (J)
static struct BurnRomInfo g_robobtRomDesc[] = {
	{ "g_robobt.bin", 0x40000, 0xfdf23eff, BRF_PRG },
};

STD_ROM_PICK(g_robobt);
STD_ROM_FN(g_robobt);

struct BurnDriverD BurnDrvMDg_robobt = {
	"g_robobt", NULL, NULL, "1900",
	"Robot Battler (SN) (J)\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_robobtRomInfo, g_robobtRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Medal City (SN) (J)
static struct BurnRomInfo g_medalcRomDesc[] = {
	{ "g_medalc.bin", 0x40000, 0x3ef4135d, BRF_PRG },
};

STD_ROM_PICK(g_medalc);
STD_ROM_FN(g_medalc);

struct BurnDriverD BurnDrvMDg_medalc = {
	"g_medalc", NULL, NULL, "1900",
	"Medal City (SN) (J)\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_medalcRomInfo, g_medalcRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Riddle Wired (SN) (J)
static struct BurnRomInfo g_riddleRomDesc[] = {
	{ "g_riddle.bin", 0x40000, 0xfae3d720, BRF_PRG },
};

STD_ROM_PICK(g_riddle);
STD_ROM_FN(g_riddle);

struct BurnDriverD BurnDrvMDg_riddle = {
	"g_riddle", NULL, NULL, "1900",
	"Riddle Wired (SN) (J)\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_riddleRomInfo, g_riddleRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Kiss Shot (SN) (J)
static struct BurnRomInfo g_kissshRomDesc[] = {
	{ "g_kisssh.bin", 0x40000, 0xe487088c, BRF_PRG },
};

STD_ROM_PICK(g_kisssh);
STD_ROM_FN(g_kisssh);

struct BurnDriverD BurnDrvMDg_kisssh = {
	"g_kisssh", NULL, NULL, "1900",
	"Kiss Shot (SN) (J)\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_kissshRomInfo, g_kissshRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Super Baseball 2020 (UE) [!]
static struct BurnRomInfo g_2020RomDesc[] = {
	{ "g_2020.bin", 0x200000, 0xc17acee5, BRF_PRG },
};

STD_ROM_PICK(g_2020);
STD_ROM_FN(g_2020);

struct BurnDriverD BurnDrvMDg_2020 = {
	"g_2020", NULL, NULL, "1900",
	"Super Baseball 2020 (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_2020RomInfo, g_2020RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Super Baseball 2020 (J) [!]
static struct BurnRomInfo g_2020jRomDesc[] = {
	{ "g_2020j.bin", 0x200000, 0x2bbee127, BRF_PRG },
};

STD_ROM_PICK(g_2020j);
STD_ROM_FN(g_2020j);

struct BurnDriverD BurnDrvMDg_2020j = {
	"g_2020j", "g_2020", NULL, "1900",
	"Super Baseball 2020 (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_2020jRomInfo, g_2020jRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Andre Agassi Tennis (U) (REV00) [!]
static struct BurnRomInfo g_aateRomDesc[] = {
	{ "g_aate.bin", 0x80000, 0xe755dd51, BRF_PRG },
};

STD_ROM_PICK(g_aate);
STD_ROM_FN(g_aate);

struct BurnDriverD BurnDrvMDg_aate = {
	"g_aate", NULL, NULL, "1900",
	"Andre Agassi Tennis (U) (REV00) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_aateRomInfo, g_aateRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Andre Agassi Tennis (E) (REV01) [c][!]
static struct BurnRomInfo g_aateeRomDesc[] = {
	{ "g_aatee.bin", 0x80000, 0x224256c7, BRF_PRG },
};

STD_ROM_PICK(g_aatee);
STD_ROM_FN(g_aatee);

struct BurnDriverD BurnDrvMDg_aatee = {
	"g_aatee", "g_aate", NULL, "1900",
	"Andre Agassi Tennis (E) (REV01) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_aateeRomInfo, g_aateeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Andre Agassi Tennis (Beta)
static struct BurnRomInfo g_aatebRomDesc[] = {
	{ "g_aateb.bin", 0x80000, 0x3bbf700d, BRF_PRG },
};

STD_ROM_PICK(g_aateb);
STD_ROM_FN(g_aateb);

struct BurnDriverD BurnDrvMDg_aateb = {
	"g_aateb", "g_aate", NULL, "1900",
	"Andre Agassi Tennis (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_aatebRomInfo, g_aatebRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Addams Family, The (UE) [!]
static struct BurnRomInfo g_afamRomDesc[] = {
	{ "g_afam.bin", 0x100000, 0x71f58614, BRF_PRG },
};

STD_ROM_PICK(g_afam);
STD_ROM_FN(g_afam);

struct BurnDriver BurnDrvMDg_afam = {
	"g_afam", NULL, NULL, "1900",
	"Addams Family, The (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_afamRomInfo, g_afamRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Addams Family, The (Beta)
static struct BurnRomInfo g_adambRomDesc[] = {
	{ "g_adamb.bin", 0x100000, 0x2803A5CA, BRF_PRG },
};

STD_ROM_PICK(g_adamb);
STD_ROM_FN(g_adamb);

struct BurnDriver BurnDrvMDg_adamb = {
	"g_adamb", "g_afam", NULL, "1993",
	"Addams Family, The (Beta)\0", NULL, "Acclaim Entertainment, Inc.", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_PLATFORM, 0,
	NULL, g_adambRomInfo, g_adambRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Adventures of Mighty Max, The (U) [!]
static struct BurnRomInfo g_mmaxRomDesc[] = {
	{ "g_mmax.bin", 0x100000, 0x55F13A00, BRF_PRG },
};

STD_ROM_PICK(g_mmax);
STD_ROM_FN(g_mmax);

struct BurnDriver BurnDrvMDg_mmax = {
	"g_mmax", NULL, NULL, "1994",
	"Adventures of Mighty Max, The (U) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_PLATFORM, 0,
	NULL, g_mmaxRomInfo, g_mmaxRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Adventures of Mighty Max, The (E)
static struct BurnRomInfo g_mmaxeRomDesc[] = {
	{ "g_mmaxe.bin", 0x100000, 0x24F1A3BB, BRF_PRG },
};

STD_ROM_PICK(g_mmaxe);
STD_ROM_FN(g_mmaxe);

struct BurnDriver BurnDrvMDg_mmaxe = {
	"g_mmaxe", "g_mmax", NULL, "1994",
	"Adventures of Mighty Max, The (E)\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mmaxeRomInfo, g_mmaxeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Air Buster (U) [c][!]
static struct BurnRomInfo g_abusRomDesc[] = {
	{ "g_abus.bin", 0x80000, 0xf3d65baa, BRF_PRG },
};

STD_ROM_PICK(g_abus);
STD_ROM_FN(g_abus);

struct BurnDriver BurnDrvMDg_abus = {
	"g_abus", NULL, NULL, "1900",
	"Air Buster (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_abusRomInfo, g_abusRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Aero Blasters (J) [!]
static struct BurnRomInfo g_aeroblRomDesc[] = {
	{ "g_aerobl.bin", 0x80000, 0xa00da987, BRF_PRG },
};

STD_ROM_PICK(g_aerobl);
STD_ROM_FN(g_aerobl);

struct BurnDriver BurnDrvMDg_aerobl = {
	"g_aerobl", "g_abus", NULL, "1900",
	"Aero Blasters (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_aeroblRomInfo, g_aeroblRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// After Burner II (UE) [!]
static struct BurnRomInfo g_abu2RomDesc[] = {
	{ "g_abu2.bin", 0x80000, 0xccafe00e, BRF_PRG },
};

STD_ROM_PICK(g_abu2);
STD_ROM_FN(g_abu2);

struct BurnDriverD BurnDrvMDg_abu2 = {
	"g_abu2", NULL, NULL, "1900",
	"After Burner II (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_abu2RomInfo, g_abu2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// After Burner II (J) [!]
static struct BurnRomInfo g_abrn2jRomDesc[] = {
	{ "g_abrn2j.bin", 0x80000, 0x4ff37e66, BRF_PRG },
};

STD_ROM_PICK(g_abrn2j);
STD_ROM_FN(g_abrn2j);

struct BurnDriverD BurnDrvMDg_abrn2j = {
	"g_abrn2j", "g_abu2", NULL, "1900",
	"After Burner II (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_abrn2jRomInfo, g_abrn2jRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Alien 3 (UE) (REV01) [!]
static struct BurnRomInfo g_ali3RomDesc[] = {
	{ "g_ali3.bin", 0x80000, 0xb327fd1b, BRF_PRG },
};

STD_ROM_PICK(g_ali3);
STD_ROM_FN(g_ali3);

struct BurnDriver BurnDrvMDg_ali3 = {
	"g_ali3", NULL, NULL, "1900",
	"Alien 3 (UE) (REV01) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ali3RomInfo, g_ali3RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Alien 3 (UE) (REV00) [!]
static struct BurnRomInfo g_ali300RomDesc[] = {
	{ "g_ali300.bin", 0x80000, 0xa3b00d6e, BRF_PRG },
};

STD_ROM_PICK(g_ali300);
STD_ROM_FN(g_ali300);

struct BurnDriver BurnDrvMDg_ali300 = {
	"g_ali300", "g_ali3", NULL, "1900",
	"Alien 3 (UE) (REV00) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ali300RomInfo, g_ali300RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Arcus Odyssey (U) [!]
static struct BurnRomInfo g_arcuRomDesc[] = {
	{ "g_arcu.bin", 0x100000, 0xbc4d9b20, BRF_PRG },
};

STD_ROM_PICK(g_arcu);
STD_ROM_FN(g_arcu);

struct BurnDriver BurnDrvMDg_arcu = {
	"g_arcu", NULL, NULL, "1900",
	"Arcus Odyssey (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_arcuRomInfo, g_arcuRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Arcus Odyssey (J) [!]
static struct BurnRomInfo g_arcusjRomDesc[] = {
	{ "g_arcusj.bin", 0x100000, 0x41c5fb4f, BRF_PRG },
};

STD_ROM_PICK(g_arcusj);
STD_ROM_FN(g_arcusj);

struct BurnDriver BurnDrvMDg_arcusj = {
	"g_arcusj", "g_arcu", NULL, "1900",
	"Arcus Odyssey (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_arcusjRomInfo, g_arcusjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Alien Soldier (J) [!]
static struct BurnRomInfo g_asolRomDesc[] = {
	{ "g_asol.bin", 0x200000, 0x90fa1539, BRF_PRG },
};

STD_ROM_PICK(g_asol);
STD_ROM_FN(g_asol);

struct BurnDriver BurnDrvMDg_asol = {
	"g_asol", NULL, NULL, "1900",
	"Alien Soldier (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_asolRomInfo, g_asolRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Alien Soldier (E) [!]
static struct BurnRomInfo g_asoldeRomDesc[] = {
	{ "g_asolde.bin", 0x200000, 0x0496e06c, BRF_PRG },
};

STD_ROM_PICK(g_asolde);
STD_ROM_FN(g_asolde);

struct BurnDriverD BurnDrvMDg_asolde = {
	"g_asolde", "g_asol", NULL, "1900",
	"Alien Soldier (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_asoldeRomInfo, g_asoldeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Alisia Dragoon (U) [!]
static struct BurnRomInfo g_alisRomDesc[] = {
	{ "g_alis.bin", 0x100000, 0xd28d5c40, BRF_PRG },
};

STD_ROM_PICK(g_alis);
STD_ROM_FN(g_alis);

struct BurnDriverD BurnDrvMDg_alis = {
	"g_alis", NULL, NULL, "1900",
	"Alisia Dragoon (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_alisRomInfo, g_alisRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Alisia Dragoon (E) [!]
static struct BurnRomInfo g_alisieRomDesc[] = {
	{ "g_alisie.bin", 0x100000, 0x28165bd1, BRF_PRG },
};

STD_ROM_PICK(g_alisie);
STD_ROM_FN(g_alisie);

struct BurnDriverD BurnDrvMDg_alisie = {
	"g_alisie", "g_alis", NULL, "1900",
	"Alisia Dragoon (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_alisieRomInfo, g_alisieRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Alisia Dragoon (J) [!]
static struct BurnRomInfo g_alisijRomDesc[] = {
	{ "g_alisij.bin", 0x100000, 0x4d476722, BRF_PRG },
};

STD_ROM_PICK(g_alisij);
STD_ROM_FN(g_alisij);

struct BurnDriverD BurnDrvMDg_alisij = {
	"g_alisij", "g_alis", NULL, "1900",
	"Alisia Dragoon (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_alisijRomInfo, g_alisijRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Animaniacs (U) [!]
static struct BurnRomInfo g_animRomDesc[] = {
	{ "g_anim.bin", 0x100000, 0x86224d86, BRF_PRG },
};

STD_ROM_PICK(g_anim);
STD_ROM_FN(g_anim);

struct BurnDriverD BurnDrvMDg_anim = {
	"g_anim", NULL, NULL, "1900",
	"Animaniacs (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_animRomInfo, g_animRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Animaniacs (E) [!]
static struct BurnRomInfo g_animeRomDesc[] = {
	{ "g_anime.bin", 0x100000, 0x92b6f255, BRF_PRG },
};

STD_ROM_PICK(g_anime);
STD_ROM_FN(g_anime);

struct BurnDriverD BurnDrvMDg_anime = {
	"g_anime", "g_anim", NULL, "1900",
	"Animaniacs (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_animeRomInfo, g_animeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Asterix and the Great Rescue (E) (M5) [c][!]
static struct BurnRomInfo g_asgrRomDesc[] = {
	{ "g_asgr.bin", 0x200000, 0x4735fee6, BRF_PRG },
};

STD_ROM_PICK(g_asgr);
STD_ROM_FN(g_asgr);

struct BurnDriverD BurnDrvMDg_asgr = {
	"g_asgr", NULL, NULL, "1900",
	"Asterix and the Great Rescue (E) (M5) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_asgrRomInfo, g_asgrRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Asterix and the Great Rescue (U) [!]
static struct BurnRomInfo g_astgruRomDesc[] = {
	{ "g_astgru.bin", 0x200000, 0x7f112cd8, BRF_PRG },
};

STD_ROM_PICK(g_astgru);
STD_ROM_FN(g_astgru);

struct BurnDriverD BurnDrvMDg_astgru = {
	"g_astgru", "g_asgr", NULL, "1900",
	"Asterix and the Great Rescue (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_astgruRomInfo, g_astgruRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Asterix and the Power of The Gods (E) (M4) [!]
static struct BurnRomInfo g_aspgRomDesc[] = {
	{ "g_aspg.bin", 0x200000, 0x4ff1d83f, BRF_PRG },
};

STD_ROM_PICK(g_aspg);
STD_ROM_FN(g_aspg);

struct BurnDriverD BurnDrvMDg_aspg = {
	"g_aspg", NULL, NULL, "1900",
	"Asterix and the Power of The Gods (E) (M4) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_aspgRomInfo, g_aspgRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Asterix and the Power of The Gods (E) (M5) [x] (Beta?)
static struct BurnRomInfo g_astpgxRomDesc[] = {
	{ "g_astpgx.bin", 0x200000, 0x45c8b5b7, BRF_PRG },
};

STD_ROM_PICK(g_astpgx);
STD_ROM_FN(g_astpgx);

struct BurnDriverD BurnDrvMDg_astpgx = {
	"g_astpgx", "g_aspg", NULL, "1900",
	"Asterix and the Power of The Gods (E) (M5) [x] (Beta?)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_astpgxRomInfo, g_astpgxRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Atomic Robo-Kid (U) [c][!]
static struct BurnRomInfo g_akidRomDesc[] = {
	{ "g_akid.bin", 0x80000, 0x7cd8169e, BRF_PRG },
};

STD_ROM_PICK(g_akid);
STD_ROM_FN(g_akid);

struct BurnDriver BurnDrvMDg_akid = {
	"g_akid", NULL, NULL, "1900",
	"Atomic Robo-Kid (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_akidRomInfo, g_akidRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Atomic Robo-Kid (J) [c][!]
static struct BurnRomInfo g_robokjRomDesc[] = {
	{ "g_robokj.bin", 0x80000, 0xe833067e, BRF_PRG },
};

STD_ROM_PICK(g_robokj);
STD_ROM_FN(g_robokj);

struct BurnDriver BurnDrvMDg_robokj = {
	"g_robokj", "g_akid", NULL, "1900",
	"Atomic Robo-Kid (J) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_robokjRomInfo, g_robokjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Awesome Possum Kicks Dr. Machino's Butt! (U) [!]
static struct BurnRomInfo g_awepRomDesc[] = {
	{ "g_awep.bin", 0x200000, 0x1f07577f, BRF_PRG },
};

STD_ROM_PICK(g_awep);
STD_ROM_FN(g_awep);

struct BurnDriverD BurnDrvMDg_awep = {
	"g_awep", NULL, NULL, "1900",
	"Awesome Possum Kicks Dr. Machino's Butt! (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_awepRomInfo, g_awepRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Awesome Possum Kicks Dr. Machino's Butt! (Beta)
static struct BurnRomInfo g_awsepRomDesc[] = {
	{ "g_awsep.bin", 0x200000, 0x0158dc53, BRF_PRG },
};

STD_ROM_PICK(g_awsep);
STD_ROM_FN(g_awsep);

struct BurnDriverD BurnDrvMDg_awsep = {
	"g_awsep", "g_awep", NULL, "1900",
	"Awesome Possum Kicks Dr. Machino's Butt! (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_awsepRomInfo, g_awsepRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Beavis and Butt-head (U) [!]
static struct BurnRomInfo g_beavRomDesc[] = {
	{ "g_beav.bin", 0x200000, 0xf5d7b948, BRF_PRG },
};

STD_ROM_PICK(g_beav);
STD_ROM_FN(g_beav);

struct BurnDriver BurnDrvMDg_beav = {
	"g_beav", NULL, NULL, "1900",
	"Beavis and Butt-head (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_beavRomInfo, g_beavRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Beavis and Butt-head (E) [!]
static struct BurnRomInfo g_beaveRomDesc[] = {
	{ "g_beave.bin", 0x200000, 0xc7b6435e, BRF_PRG },
};

STD_ROM_PICK(g_beave);
STD_ROM_FN(g_beave);

struct BurnDriver BurnDrvMDg_beave = {
	"g_beave", "g_beav", NULL, "1900",
	"Beavis and Butt-head (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_beaveRomInfo, g_beaveRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Beavis and Butt-head (Beta)
static struct BurnRomInfo g_beavibRomDesc[] = {
	{ "g_beavib.bin", 0x200000, 0x81ed5335, BRF_PRG },
};

STD_ROM_PICK(g_beavib);
STD_ROM_FN(g_beavib);

struct BurnDriverD BurnDrvMDg_beavib = {
	"g_beavib", "g_beav", NULL, "1900",
	"Beavis and Butt-head (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_beavibRomInfo, g_beavibRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Best of the Best - Championship Karate (U) [c][!]
static struct BurnRomInfo g_botbRomDesc[] = {
	{ "g_botb.bin", 0x100000, 0xc3d6a5d4, BRF_PRG },
};

STD_ROM_PICK(g_botb);
STD_ROM_FN(g_botb);

struct BurnDriver BurnDrvMDg_botb = {
	"g_botb", NULL, NULL, "1900",
	"Best of the Best - Championship Karate (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_botbRomInfo, g_botbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Best of the Best - Championship Karate (Beta)
static struct BurnRomInfo g_botbbRomDesc[] = {
	{ "g_botbb.bin", 0x100000, 0xf842240b, BRF_PRG },
};

STD_ROM_PICK(g_botbb);
STD_ROM_FN(g_botbb);

struct BurnDriver BurnDrvMDg_botbb = {
	"g_botbb", "g_botb", NULL, "1900",
	"Best of the Best - Championship Karate (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_botbbRomInfo, g_botbbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Kick Boxing, The (J) [!]
static struct BurnRomInfo g_kboxRomDesc[] = {
	{ "g_kbox.bin", 0x100000, 0x9bdc230c, BRF_PRG },
};

STD_ROM_PICK(g_kbox);
STD_ROM_FN(g_kbox);

struct BurnDriver BurnDrvMDg_kbox = {
	"g_kbox", "g_botb", NULL, "1900",
	"Kick Boxing, The (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_kboxRomInfo, g_kboxRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Bubba N Stix (U) [!]
static struct BurnRomInfo g_bubaRomDesc[] = {
	{ "g_buba.bin", 0x100000, 0xd45cb46f, BRF_PRG },
};

STD_ROM_PICK(g_buba);
STD_ROM_FN(g_buba);

struct BurnDriverD BurnDrvMDg_buba = {
	"g_buba", NULL, NULL, "1900",
	"Bubba N Stix (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_bubaRomInfo, g_bubaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Bubba N Stix (E) [c][!]
static struct BurnRomInfo g_bubbaeRomDesc[] = {
	{ "g_bubbae.bin", 0x100000, 0xb467432e, BRF_PRG },
};

STD_ROM_PICK(g_bubbae);
STD_ROM_FN(g_bubbae);

struct BurnDriverD BurnDrvMDg_bubbae = {
	"g_bubbae", "g_buba", NULL, "1900",
	"Bubba N Stix (E) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_bubbaeRomInfo, g_bubbaeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Bubba N Stix (Beta)
static struct BurnRomInfo g_bubbabRomDesc[] = {
	{ "g_bubbab.bin", 0x100000, 0xa8731cb4, BRF_PRG },
};

STD_ROM_PICK(g_bubbab);
STD_ROM_FN(g_bubbab);

struct BurnDriverD BurnDrvMDg_bubbab = {
	"g_bubbab", "g_buba", NULL, "1900",
	"Bubba N Stix (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_bubbabRomInfo, g_bubbabRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Budokan - The Martial Spirit (U) [c][!]
static struct BurnRomInfo g_budoRomDesc[] = {
	{ "g_budo.bin", 0x80000, 0xacd9f5fc, BRF_PRG },
};

STD_ROM_PICK(g_budo);
STD_ROM_FN(g_budo);

struct BurnDriver BurnDrvMDg_budo = {
	"g_budo", NULL, NULL, "1900",
	"Budokan - The Martial Spirit (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_budoRomInfo, g_budoRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Budokan - The Martial Spirit (E) [!]
static struct BurnRomInfo g_budoeRomDesc[] = {
	{ "g_budoe.bin", 0x80000, 0x97add5bd, BRF_PRG },
};

STD_ROM_PICK(g_budoe);
STD_ROM_FN(g_budoe);

struct BurnDriver BurnDrvMDg_budoe = {
	"g_budoe", "g_budo", NULL, "1900",
	"Budokan - The Martial Spirit (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_budoeRomInfo, g_budoeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Bio-Hazard Battle (UE) [!]
static struct BurnRomInfo g_bhbRomDesc[] = {
	{ "g_bhb.bin", 0x100000, 0x95b0ea2b, BRF_PRG },
};

STD_ROM_PICK(g_bhb);
STD_ROM_FN(g_bhb);

struct BurnDriver BurnDrvMDg_bhb = {
	"g_bhb", NULL, NULL, "1900",
	"Bio-Hazard Battle (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_bhbRomInfo, g_bhbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Bio-Hazard Battle (Beta)
static struct BurnRomInfo g_biohzbRomDesc[] = {
	{ "g_biohzb.bin", 0x100000, 0xdd10dd1a, BRF_PRG },
};

STD_ROM_PICK(g_biohzb);
STD_ROM_FN(g_biohzb);

struct BurnDriver BurnDrvMDg_biohzb = {
	"g_biohzb", "g_bhb", NULL, "1900",
	"Bio-Hazard Battle (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_biohzbRomInfo, g_biohzbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Crying - Asia Seimei Sensou (J) [!]
static struct BurnRomInfo g_cryingRomDesc[] = {
	{ "g_crying.bin", 0x100000, 0x4aba1d6a, BRF_PRG },
};

STD_ROM_PICK(g_crying);
STD_ROM_FN(g_crying);

struct BurnDriver BurnDrvMDg_crying = {
	"g_crying", "g_bhb", NULL, "1900",
	"Crying - Asia Seimei Sensou (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_cryingRomInfo, g_cryingRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Body Count (E) (M5) [!]
static struct BurnRomInfo g_bodyRomDesc[] = {
	{ "g_body.bin", 0x100000, 0x3575a030, BRF_PRG },
};

STD_ROM_PICK(g_body);
STD_ROM_FN(g_body);

struct BurnDriverD BurnDrvMDg_body = {
	"g_body", NULL, NULL, "1900",
	"Body Count (E) (M5) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_bodyRomInfo, g_bodyRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Body Count (Beta) (M5)
static struct BurnRomInfo g_bcounbRomDesc[] = {
	{ "g_bcounb.bin", 0x100000, 0xb4ffb6ce, BRF_PRG },
};

STD_ROM_PICK(g_bcounb);
STD_ROM_FN(g_bcounb);

struct BurnDriverD BurnDrvMDg_bcounb = {
	"g_bcounb", "g_body", NULL, "1900",
	"Body Count (Beta) (M5)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_bcounbRomInfo, g_bcounbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Bonanza Bros. (U) [!]
static struct BurnRomInfo g_bnzaRomDesc[] = {
	{ "g_bnza.bin", 0x80000, 0x20d1ad4c, BRF_PRG },
};

STD_ROM_PICK(g_bnza);
STD_ROM_FN(g_bnza);

struct BurnDriverD BurnDrvMDg_bnza = {
	"g_bnza", NULL, NULL, "1900",
	"Bonanza Bros. (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_bnzaRomInfo, g_bnzaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Bonanza Bros. (JE) (REV00) [!]
static struct BurnRomInfo g_bnza00RomDesc[] = {
	{ "g_bnza00.bin", 0x80000, 0xadf6476c, BRF_PRG },
};

STD_ROM_PICK(g_bnza00);
STD_ROM_FN(g_bnza00);

struct BurnDriverD BurnDrvMDg_bnza00 = {
	"g_bnza00", "g_bnza", NULL, "1900",
	"Bonanza Bros. (JE) (REV00) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_bnza00RomInfo, g_bnza00RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Bonanza Bros. (JE) (REV01) [!]
static struct BurnRomInfo g_bnza01RomDesc[] = {
	{ "g_bnza01.bin", 0x80000, 0xc6aac589, BRF_PRG },
};

STD_ROM_PICK(g_bnza01);
STD_ROM_FN(g_bnza01);

struct BurnDriverD BurnDrvMDg_bnza01 = {
	"g_bnza01", "g_bnza", NULL, "1900",
	"Bonanza Bros. (JE) (REV01) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_bnza01RomInfo, g_bnza01RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Boogerman (U) [!]
static struct BurnRomInfo g_boogRomDesc[] = {
	{ "g_boog.bin", 0x300000, 0x1a7a2bec, BRF_PRG },
};

STD_ROM_PICK(g_boog);
STD_ROM_FN(g_boog);

struct BurnDriver BurnDrvMDg_boog = {
	"g_boog", NULL, NULL, "1900",
	"Boogerman (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_boogRomInfo, g_boogRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Boogerman (E) [!]
static struct BurnRomInfo g_boogeRomDesc[] = {
	{ "g_booge.bin", 0x300000, 0xdbc4340c, BRF_PRG },
};

STD_ROM_PICK(g_booge);
STD_ROM_FN(g_booge);

struct BurnDriverD BurnDrvMDg_booge = {
	"g_booge", "g_boog", NULL, "1900",
	"Boogerman (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_boogeRomInfo, g_boogeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Boxing Legends of the Ring (U) [!]
static struct BurnRomInfo g_boxlRomDesc[] = {
	{ "g_boxl.bin", 0x100000, 0x00f225ac, BRF_PRG },
};

STD_ROM_PICK(g_boxl);
STD_ROM_FN(g_boxl);

struct BurnDriverD BurnDrvMDg_boxl = {
	"g_boxl", NULL, NULL, "1900",
	"Boxing Legends of the Ring (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_boxlRomInfo, g_boxlRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Chavez II (U) [!]
static struct BurnRomInfo g_chav2RomDesc[] = {
	{ "g_chav2.bin", 0x100000, 0x5bc0dbb8, BRF_PRG },
};

STD_ROM_PICK(g_chav2);
STD_ROM_FN(g_chav2);

struct BurnDriverD BurnDrvMDg_chav2 = {
	"g_chav2", "g_boxl", NULL, "1900",
	"Chavez II (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_chav2RomInfo, g_chav2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Brutal - Paws of Fury (U) [!]
static struct BurnRomInfo g_brpwRomDesc[] = {
	{ "g_brpw.bin", 0x200000, 0x98d502cd, BRF_PRG },
};

STD_ROM_PICK(g_brpw);
STD_ROM_FN(g_brpw);

struct BurnDriverD BurnDrvMDg_brpw = {
	"g_brpw", NULL, NULL, "1900",
	"Brutal - Paws of Fury (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_brpwRomInfo, g_brpwRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Brutal - Paws of Fury (E)
static struct BurnRomInfo g_brutleRomDesc[] = {
	{ "g_brutle.bin", 0x200000, 0x7e9a8d32, BRF_PRG },
};

STD_ROM_PICK(g_brutle);
STD_ROM_FN(g_brutle);

struct BurnDriverD BurnDrvMDg_brutle = {
	"g_brutle", "g_brpw", NULL, "1900",
	"Brutal - Paws of Fury (E)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_brutleRomInfo, g_brutleRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Burning Force (U) [!]
static struct BurnRomInfo g_burfRomDesc[] = {
	{ "g_burf.bin", 0x80000, 0xbdc8f02c, BRF_PRG },
};

STD_ROM_PICK(g_burf);
STD_ROM_FN(g_burf);

struct BurnDriverD BurnDrvMDg_burf = {
	"g_burf", NULL, NULL, "1900",
	"Burning Force (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_burfRomInfo, g_burfRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Burning Force (J) [!]
static struct BurnRomInfo g_burnfjRomDesc[] = {
	{ "g_burnfj.bin", 0x80000, 0x0c1deb47, BRF_PRG },
};

STD_ROM_PICK(g_burnfj);
STD_ROM_FN(g_burnfj);

struct BurnDriverD BurnDrvMDg_burnfj = {
	"g_burnfj", "g_burf", NULL, "1900",
	"Burning Force (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_burnfjRomInfo, g_burnfjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Burning Force (E)
static struct BurnRomInfo g_burnfeRomDesc[] = {
	{ "g_burnfe.bin", 0x80000, 0x776ff6ff, BRF_PRG },
};

STD_ROM_PICK(g_burnfe);
STD_ROM_FN(g_burnfe);

struct BurnDriverD BurnDrvMDg_burnfe = {
	"g_burnfe", "g_burf", NULL, "1900",
	"Burning Force (E)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_burnfeRomInfo, g_burnfeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Cliffhanger (U) [c][!]
static struct BurnRomInfo g_clifRomDesc[] = {
	{ "g_clif.bin", 0x100000, 0x9cbf44d3, BRF_PRG },
};

STD_ROM_PICK(g_clif);
STD_ROM_FN(g_clif);

struct BurnDriverD BurnDrvMDg_clif = {
	"g_clif", NULL, NULL, "1900",
	"Cliffhanger (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_clifRomInfo, g_clifRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Cliffhanger (E)
static struct BurnRomInfo g_clifheRomDesc[] = {
	{ "g_clifhe.bin", 0x100000, 0x35bff1fd, BRF_PRG },
};

STD_ROM_PICK(g_clifhe);
STD_ROM_FN(g_clifhe);

struct BurnDriverD BurnDrvMDg_clifhe = {
	"g_clifhe", "g_clif", NULL, "1900",
	"Cliffhanger (E)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_clifheRomInfo, g_clifheRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Cliffhanger (Beta)
static struct BurnRomInfo g_clifhbRomDesc[] = {
	{ "g_clifhb.bin", 0x100000, 0x628251fd, BRF_PRG },
};

STD_ROM_PICK(g_clifhb);
STD_ROM_FN(g_clifhb);

struct BurnDriverD BurnDrvMDg_clifhb = {
	"g_clifhb", "g_clif", NULL, "1900",
	"Cliffhanger (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_clifhbRomInfo, g_clifhbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Dahna - Megami Tanjou (J) [!]
static struct BurnRomInfo g_dahnaRomDesc[] = {
	{ "g_dahna.bin", 0x100000, 0x4602584f, BRF_PRG },
};

STD_ROM_PICK(g_dahna);
STD_ROM_FN(g_dahna);

struct BurnDriverD BurnDrvMDg_dahna = {
	"g_dahna", NULL, NULL, "1991",
	"Dahna - Megami Tanjou (J) [!]\0", NULL, "IGS", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_dahnaRomInfo, g_dahnaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Hercules (Unl) [!]
static struct BurnRomInfo g_hercuRomDesc[] = {
	{ "g_hercu.bin", 0x100000, 0xff75d9d0, BRF_PRG },
};

STD_ROM_PICK(g_hercu);
STD_ROM_FN(g_hercu);

struct BurnDriverD BurnDrvMDg_hercu = {
	"g_hercu", "g_dahna", NULL, "1900",
	"Hercules (Unl) [!]\0", NULL, "Unknown (Unlicensed)", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_hercuRomInfo, g_hercuRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Dr. Robotnik's Mean Bean Machine (U) [!]
static struct BurnRomInfo g_beanRomDesc[] = {
	{ "g_bean.bin", 0x100000, 0xc7ca517f, BRF_PRG },
};

STD_ROM_PICK(g_bean);
STD_ROM_FN(g_bean);

struct BurnDriverD BurnDrvMDg_bean = {
	"g_bean", NULL, NULL, "1900",
	"Dr. Robotnik's Mean Bean Machine (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_beanRomInfo, g_beanRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Dr. Robotnik's Mean Bean Machine (E) [!]
static struct BurnRomInfo g_beaneRomDesc[] = {
	{ "g_beane.bin", 0x100000, 0x70680706, BRF_PRG },
};

STD_ROM_PICK(g_beane);
STD_ROM_FN(g_beane);

struct BurnDriverD BurnDrvMDg_beane = {
	"g_beane", "g_bean", NULL, "1900",
	"Dr. Robotnik's Mean Bean Machine (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_beaneRomInfo, g_beaneRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Dr. Robotnik's Mean Bean Machine (Beta)
static struct BurnRomInfo g_beanbRomDesc[] = {
	{ "g_beanb.bin", 0x100000, 0x4d0e5273, BRF_PRG },
};

STD_ROM_PICK(g_beanb);
STD_ROM_FN(g_beanb);

struct BurnDriverD BurnDrvMDg_beanb = {
	"g_beanb", "g_bean", NULL, "1900",
	"Dr. Robotnik's Mean Bean Machine (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_beanbRomInfo, g_beanbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Dashin' Desperadoes (U) [!]
static struct BurnRomInfo g_dashRomDesc[] = {
	{ "g_dash.bin", 0x100000, 0xdcb76fb7, BRF_PRG },
};

STD_ROM_PICK(g_dash);
STD_ROM_FN(g_dash);

struct BurnDriverD BurnDrvMDg_dash = {
	"g_dash", NULL, NULL, "1900",
	"Dashin' Desperadoes (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_dashRomInfo, g_dashRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Dashin' Desperadoes (Beta)
static struct BurnRomInfo g_dashbRomDesc[] = {
	{ "g_dashb.bin", 0x100000, 0xadaffc3f, BRF_PRG },
};

STD_ROM_PICK(g_dashb);
STD_ROM_FN(g_dashb);

struct BurnDriverD BurnDrvMDg_dashb = {
	"g_dashb", "g_dash", NULL, "1900",
	"Dashin' Desperadoes (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_dashbRomInfo, g_dashbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Dragon - The Bruce Lee Story (U) [!]
static struct BurnRomInfo g_bleeRomDesc[] = {
	{ "g_blee.bin", 0x200000, 0xefe850e5, BRF_PRG },
};

STD_ROM_PICK(g_blee);
STD_ROM_FN(g_blee);

struct BurnDriverD BurnDrvMDg_blee = {
	"g_blee", NULL, NULL, "1900",
	"Dragon - The Bruce Lee Story (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_bleeRomInfo, g_bleeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Dragon - The Bruce Lee Story (E)
static struct BurnRomInfo g_drgbleRomDesc[] = {
	{ "g_drgble.bin", 0x200000, 0xfdeed51d, BRF_PRG },
};

STD_ROM_PICK(g_drgble);
STD_ROM_FN(g_drgble);

struct BurnDriverD BurnDrvMDg_drgble = {
	"g_drgble", "g_blee", NULL, "1900",
	"Dragon - The Bruce Lee Story (E)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_drgbleRomInfo, g_drgbleRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Dynamite Duke (W) [!]
static struct BurnRomInfo g_ddukRomDesc[] = {
	{ "g_dduk.bin", 0x80000, 0x39d01c8c, BRF_PRG },
};

STD_ROM_PICK(g_dduk);
STD_ROM_FN(g_dduk);

struct BurnDriverD BurnDrvMDg_dduk = {
	"g_dduk", NULL, NULL, "1900",
	"Dynamite Duke (W) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ddukRomInfo, g_ddukRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Dynamite Duke (W) [a1][!]
static struct BurnRomInfo g_ddukeaRomDesc[] = {
	{ "g_ddukea.bin", 0x80000, 0x246f0bda, BRF_PRG },
};

STD_ROM_PICK(g_ddukea);
STD_ROM_FN(g_ddukea);

struct BurnDriverD BurnDrvMDg_ddukea = {
	"g_ddukea", "g_dduk", NULL, "1900",
	"Dynamite Duke (W) [a1][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ddukeaRomInfo, g_ddukeaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Dynamite Headdy (UE) [c][!]
static struct BurnRomInfo g_dhedRomDesc[] = {
	{ "g_dhed.bin", 0x200000, 0x3dfeeb77, BRF_PRG },
};

STD_ROM_PICK(g_dhed);
STD_ROM_FN(g_dhed);

struct BurnDriverD BurnDrvMDg_dhed = {
	"g_dhed", NULL, NULL, "1900",
	"Dynamite Headdy (UE) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_dhedRomInfo, g_dhedRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Dynamite Headdy (J) [c][!]
static struct BurnRomInfo g_dheadjRomDesc[] = {
	{ "g_dheadj.bin", 0x200000, 0xd03cdb53, BRF_PRG },
};

STD_ROM_PICK(g_dheadj);
STD_ROM_FN(g_dheadj);

struct BurnDriverD BurnDrvMDg_dheadj = {
	"g_dheadj", "g_dhed", NULL, "1900",
	"Dynamite Headdy (J) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_dheadjRomInfo, g_dheadjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Dynamite Headdy (Beta)
static struct BurnRomInfo g_dheadbRomDesc[] = {
	{ "g_dheadb.bin", 0x200000, 0x5c25e934, BRF_PRG },
};

STD_ROM_PICK(g_dheadb);
STD_ROM_FN(g_dheadb);

struct BurnDriverD BurnDrvMDg_dheadb = {
	"g_dheadb", "g_dhed", NULL, "1900",
	"Dynamite Headdy (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_dheadbRomInfo, g_dheadbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Earthworm Jim (U) [!]
static struct BurnRomInfo g_ejimRomDesc[] = {
	{ "g_ejim.bin", 0x300000, 0xdf3acf59, BRF_PRG },
};

STD_ROM_PICK(g_ejim);
STD_ROM_FN(g_ejim);

struct BurnDriverD BurnDrvMDg_ejim = {
	"g_ejim", NULL, NULL, "1900",
	"Earthworm Jim (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ejimRomInfo, g_ejimRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Earthworm Jim (E) [!]
static struct BurnRomInfo g_ejimeRomDesc[] = {
	{ "g_ejime.bin", 0x300000, 0x1c07b337, BRF_PRG },
};

STD_ROM_PICK(g_ejime);
STD_ROM_FN(g_ejime);

struct BurnDriverD BurnDrvMDg_ejime = {
	"g_ejime", "g_ejim", NULL, "1900",
	"Earthworm Jim (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ejimeRomInfo, g_ejimeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Earthworm Jim 2 (U) [!]
static struct BurnRomInfo g_jim2RomDesc[] = {
	{ "g_jim2.bin", 0x300000, 0xd57f8ba7, BRF_PRG },
};

STD_ROM_PICK(g_jim2);
STD_ROM_FN(g_jim2);

struct BurnDriverD BurnDrvMDg_jim2 = {
	"g_jim2", NULL, NULL, "1900",
	"Earthworm Jim 2 (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jim2RomInfo, g_jim2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Earthworm Jim 2 (E) [!]
static struct BurnRomInfo g_ejim2eRomDesc[] = {
	{ "g_ejim2e.bin", 0x300000, 0xaf235fdf, BRF_PRG },
};

STD_ROM_PICK(g_ejim2e);
STD_ROM_FN(g_ejim2e);

struct BurnDriverD BurnDrvMDg_ejim2e = {
	"g_ejim2e", "g_jim2", NULL, "1900",
	"Earthworm Jim 2 (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ejim2eRomInfo, g_ejim2eRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Eternal Champions (U) [!]
static struct BurnRomInfo g_etchRomDesc[] = {
	{ "g_etch.bin", 0x300000, 0x48f1a42e, BRF_PRG },
};

STD_ROM_PICK(g_etch);
STD_ROM_FN(g_etch);

struct BurnDriverD BurnDrvMDg_etch = {
	"g_etch", NULL, NULL, "1900",
	"Eternal Champions (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_etchRomInfo, g_etchRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Eternal Champions (E) [!]
static struct BurnRomInfo g_echmpeRomDesc[] = {
	{ "g_echmpe.bin", 0x300000, 0xb9512f5e, BRF_PRG },
};

STD_ROM_PICK(g_echmpe);
STD_ROM_FN(g_echmpe);

struct BurnDriverD BurnDrvMDg_echmpe = {
	"g_echmpe", "g_etch", NULL, "1900",
	"Eternal Champions (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_echmpeRomInfo, g_echmpeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Eternal Champions (J) [c][!]
static struct BurnRomInfo g_echmpjRomDesc[] = {
	{ "g_echmpj.bin", 0x300000, 0x66aa3c64, BRF_PRG },
};

STD_ROM_PICK(g_echmpj);
STD_ROM_FN(g_echmpj);

struct BurnDriverD BurnDrvMDg_echmpj = {
	"g_echmpj", "g_etch", NULL, "1900",
	"Eternal Champions (J) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_echmpjRomInfo, g_echmpjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Eternal Champions (Beta)
static struct BurnRomInfo g_etchmbRomDesc[] = {
	{ "g_etchmb.bin", 0x300000, 0xe0665f06, BRF_PRG },
};

STD_ROM_PICK(g_etchmb);
STD_ROM_FN(g_etchmb);

struct BurnDriverD BurnDrvMDg_etchmb = {
	"g_etchmb", "g_etch", NULL, "1900",
	"Eternal Champions (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_etchmbRomInfo, g_etchmbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Exile (U) [!]
static struct BurnRomInfo g_exilRomDesc[] = {
	{ "g_exil.bin", 0x100000, 0x1b569dc2, BRF_PRG },
};

STD_ROM_PICK(g_exil);
STD_ROM_FN(g_exil);

struct BurnDriverD BurnDrvMDg_exil = {
	"g_exil", NULL, NULL, "1900",
	"Exile (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_exilRomInfo, g_exilRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// XZR (J) [!]
static struct BurnRomInfo g_xzrRomDesc[] = {
	{ "g_xzr.bin", 0x100000, 0x880bf311, BRF_PRG },
};

STD_ROM_PICK(g_xzr);
STD_ROM_FN(g_xzr);

struct BurnDriverD BurnDrvMDg_xzr = {
	"g_xzr", "g_exil", NULL, "1900",
	"XZR (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_xzrRomInfo, g_xzrRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Exo-Squad (U) [!]
static struct BurnRomInfo g_exosRomDesc[] = {
	{ "g_exos.bin", 0x100000, 0x10ec03f3, BRF_PRG },
};

STD_ROM_PICK(g_exos);
STD_ROM_FN(g_exos);

struct BurnDriverD BurnDrvMDg_exos = {
	"g_exos", NULL, NULL, "1900",
	"Exo-Squad (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_exosRomInfo, g_exosRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Exo-Squad (Beta)
static struct BurnRomInfo g_exosbRomDesc[] = {
	{ "g_exosb.bin", 0x100000, 0x70edf964, BRF_PRG },
};

STD_ROM_PICK(g_exosb);
STD_ROM_FN(g_exosb);

struct BurnDriverD BurnDrvMDg_exosb = {
	"g_exosb", "g_exos", NULL, "1900",
	"Exo-Squad (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_exosbRomInfo, g_exosbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Fantastic Dizzy (UE) (M5) [c][!]
static struct BurnRomInfo g_fandRomDesc[] = {
	{ "g_fand.bin", 0x80000, 0x46447e7a, BRF_PRG },
};

STD_ROM_PICK(g_fand);
STD_ROM_FN(g_fand);

struct BurnDriverD BurnDrvMDg_fand = {
	"g_fand", NULL, NULL, "1900",
	"Fantastic Dizzy (UE) (M5) [c][!]\0", NULL, "Codemasters", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_fandRomInfo, g_fandRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Fantastic Dizzy (UE) (M5) [a1]
static struct BurnRomInfo g_fandaRomDesc[] = {
	{ "g_fanda.bin", 0x80000, 0x86b2a235, BRF_PRG },
};

STD_ROM_PICK(g_fanda);
STD_ROM_FN(g_fanda);

struct BurnDriverD BurnDrvMDg_fanda = {
	"g_fanda", "g_fand", NULL, "1900",
	"Fantastic Dizzy (UE) (M5) [a1]\0", NULL, "Codemasters", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_fandaRomInfo, g_fandaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// FIFA Soccer 95 (UE) [!]
static struct BurnRomInfo g_fi95RomDesc[] = {
	{ "g_fi95.bin", 0x200000, 0xb389d036, BRF_PRG },
};

STD_ROM_PICK(g_fi95);
STD_ROM_FN(g_fi95);

struct BurnDriver BurnDrvMDg_fi95 = {
	"g_fi95", NULL, NULL, "1900",
	"FIFA Soccer 95 (UE) [!]\0", NULL, "Electronic Arts", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_fi95RomInfo, g_fi95RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Futbol Argentino 98 - Pasion de Multitudes (Unl)
static struct BurnRomInfo g_fut98RomDesc[] = {
	{ "g_fut98.bin", 0x200000, 0x5c015888, BRF_PRG },
};

STD_ROM_PICK(g_fut98);
STD_ROM_FN(g_fut98);

struct BurnDriver BurnDrvMDg_fut98 = {
	"g_fut98", "g_fi95", NULL, "1900",
	"Futbol Argentino 98 - Pasion de Multitudes (Unl)\0", NULL, "Unknown (Unlicensed)", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_fut98RomInfo, g_fut98RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// FIFA Soccer 96 (UE) (M6) [!]
static struct BurnRomInfo g_fi96RomDesc[] = {
	{ "g_fi96.bin", 0x200000, 0xbad30ffa, BRF_PRG },
};

STD_ROM_PICK(g_fi96);
STD_ROM_FN(g_fi96);

struct BurnDriver BurnDrvMDg_fi96 = {
	"g_fi96", NULL, NULL, "1900",
	"FIFA Soccer 96 (UE) (M6) [!]\0", NULL, "Electronic Arts", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_fi96RomInfo, g_fi96RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// FIFA Soccer 99 (R) [!]
static struct BurnRomInfo g_fi99rRomDesc[] = {
	{ "g_fi99r.bin", 0x200000, 0xc5c5a4b0, BRF_PRG },
};

STD_ROM_PICK(g_fi99r);
STD_ROM_FN(g_fi99r);

struct BurnDriver BurnDrvMDg_fi99r = {
	"g_fi99r", "g_fi96", NULL, "1900",
	"FIFA Soccer 99 (R) [!]\0", NULL, "Unknown (Unlicensed)", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_fi99rRomInfo, g_fi99rRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// FIFA Soccer 97 Gold Edition (UE) (M6) [!]
static struct BurnRomInfo g_fi97RomDesc[] = {
	{ "g_fi97.bin", 0x200000, 0xa33d5803, BRF_PRG },
};

STD_ROM_PICK(g_fi97);
STD_ROM_FN(g_fi97);

struct BurnDriverD BurnDrvMDg_fi97 = {
	"g_fi97", NULL, NULL, "1996",
	"FIFA Soccer 97 Gold Edition (UE) (M6) [!]\0", NULL, "Electronic Arts", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_fi97RomInfo, g_fi97RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// FIFA Soccer 2000 Gold Edition (Unl) (M6) [!]
static struct BurnRomInfo g_f2000gRomDesc[] = {
	{ "g_f2000g.bin", 0x200000, 0x647df504, BRF_PRG },
};

STD_ROM_PICK(g_f2000g);
STD_ROM_FN(g_f2000g);

struct BurnDriverD BurnDrvMDg_f2000g = {
	"g_f2000g", "g_fi97", NULL, "2000",
	"FIFA Soccer 2000 Gold Edition (Unl) (M6) [!]\0", NULL, "Unknown (Unlicensed)", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_f2000gRomInfo, g_f2000gRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Fire Shark (U) [!]
static struct BurnRomInfo g_fshkRomDesc[] = {
	{ "g_fshk.bin", 0x80000, 0x9c175146, BRF_PRG },
};

STD_ROM_PICK(g_fshk);
STD_ROM_FN(g_fshk);

struct BurnDriverD BurnDrvMDg_fshk = {
	"g_fshk", NULL, NULL, "1900",
	"Fire Shark (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_fshkRomInfo, g_fshkRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Fire Shark (E) [!]
static struct BurnRomInfo g_fshrkeRomDesc[] = {
	{ "g_fshrke.bin", 0x80000, 0x2351ce61, BRF_PRG },
};

STD_ROM_PICK(g_fshrke);
STD_ROM_FN(g_fshrke);

struct BurnDriverD BurnDrvMDg_fshrke = {
	"g_fshrke", "g_fshk", NULL, "1900",
	"Fire Shark (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_fshrkeRomInfo, g_fshrkeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Fire Shark (U) [a1][!]
static struct BurnRomInfo g_fshrkuRomDesc[] = {
	{ "g_fshrku.bin", 0x80000, 0x570b5024, BRF_PRG },
};

STD_ROM_PICK(g_fshrku);
STD_ROM_FN(g_fshrku);

struct BurnDriverD BurnDrvMDg_fshrku = {
	"g_fshrku", "g_fshk", NULL, "1900",
	"Fire Shark (U) [a1][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_fshrkuRomInfo, g_fshrkuRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Same! Same! Same! (J) [!]
static struct BurnRomInfo g_samex3RomDesc[] = {
	{ "g_samex3.bin", 0x80000, 0x77bbd841, BRF_PRG },
};

STD_ROM_PICK(g_samex3);
STD_ROM_FN(g_samex3);

struct BurnDriverD BurnDrvMDg_samex3 = {
	"g_samex3", "g_fshk", NULL, "1900",
	"Same! Same! Same! (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_samex3RomInfo, g_samex3RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Galaxy Force II (W) (REV01) [!]
static struct BurnRomInfo g_gf2RomDesc[] = {
	{ "g_gf2.bin", 0x100000, 0xd15f5c3c, BRF_PRG },
};

STD_ROM_PICK(g_gf2);
STD_ROM_FN(g_gf2);

struct BurnDriverD BurnDrvMDg_gf2 = {
	"g_gf2", NULL, NULL, "1900",
	"Galaxy Force II (W) (REV01) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_gf2RomInfo, g_gf2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Galaxy Force II (W) (REV00)
static struct BurnRomInfo g_glf200RomDesc[] = {
	{ "g_glf200.bin", 0x100000, 0xcae883c5, BRF_PRG },
};

STD_ROM_PICK(g_glf200);
STD_ROM_FN(g_glf200);

struct BurnDriverD BurnDrvMDg_glf200 = {
	"g_glf200", "g_gf2", NULL, "1900",
	"Galaxy Force II (W) (REV00)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_glf200RomInfo, g_glf200RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// General Chaos (UE) [!]
static struct BurnRomInfo g_gencRomDesc[] = {
	{ "g_genc.bin", 0x100000, 0xf1ecc4df, BRF_PRG },
};

STD_ROM_PICK(g_genc);
STD_ROM_FN(g_genc);

struct BurnDriverD BurnDrvMDg_genc = {
	"g_genc", NULL, NULL, "1900",
	"General Chaos (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_gencRomInfo, g_gencRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Dai Konsen (J) [!]
static struct BurnRomInfo g_konsenRomDesc[] = {
	{ "g_konsen.bin", 0x100000, 0x05cc7369, BRF_PRG },
};

STD_ROM_PICK(g_konsen);
STD_ROM_FN(g_konsen);

struct BurnDriverD BurnDrvMDg_konsen = {
	"g_konsen", "g_genc", NULL, "1900",
	"Dai Konsen (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_konsenRomInfo, g_konsenRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Ghostbusters (W) (REV01) [!]
static struct BurnRomInfo g_gbusRomDesc[] = {
	{ "g_gbus.bin", 0x80000, 0x792df93b, BRF_PRG },
};

STD_ROM_PICK(g_gbus);
STD_ROM_FN(g_gbus);

struct BurnDriverD BurnDrvMDg_gbus = {
	"g_gbus", NULL, NULL, "1900",
	"Ghostbusters (W) (REV01) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_gbusRomInfo, g_gbusRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Ghostbusters (W) (REV00) [!]
static struct BurnRomInfo g_gbus00RomDesc[] = {
	{ "g_gbus00.bin", 0x80000, 0x00419da3, BRF_PRG },
};

STD_ROM_PICK(g_gbus00);
STD_ROM_FN(g_gbus00);

struct BurnDriverD BurnDrvMDg_gbus00 = {
	"g_gbus00", "g_gbus", NULL, "1900",
	"Ghostbusters (W) (REV00) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_gbus00RomInfo, g_gbus00RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Gods (U) [!]
static struct BurnRomInfo g_godsRomDesc[] = {
	{ "g_gods.bin", 0x100000, 0xfd234ccd, BRF_PRG },
};

STD_ROM_PICK(g_gods);
STD_ROM_FN(g_gods);

struct BurnDriverD BurnDrvMDg_gods = {
	"g_gods", NULL, NULL, "1900",
	"Gods (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_godsRomInfo, g_godsRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Gods (E) [!]
static struct BurnRomInfo g_godseRomDesc[] = {
	{ "g_godse.bin", 0x100000, 0x6c415016, BRF_PRG },
};

STD_ROM_PICK(g_godse);
STD_ROM_FN(g_godse);

struct BurnDriverD BurnDrvMDg_godse = {
	"g_godse", "g_gods", NULL, "1900",
	"Gods (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_godseRomInfo, g_godseRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Gods (J) [!]
static struct BurnRomInfo g_godsjRomDesc[] = {
	{ "g_godsj.bin", 0x100000, 0xe4f50206, BRF_PRG },
};

STD_ROM_PICK(g_godsj);
STD_ROM_FN(g_godsj);

struct BurnDriverD BurnDrvMDg_godsj = {
	"g_godsj", "g_gods", NULL, "1900",
	"Gods (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_godsjRomInfo, g_godsjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Gods (Beta)
static struct BurnRomInfo g_godsbRomDesc[] = {
	{ "g_godsb.bin", 0x100000, 0x2c06bb64, BRF_PRG },
};

STD_ROM_PICK(g_godsb);
STD_ROM_FN(g_godsb);

struct BurnDriverD BurnDrvMDg_godsb = {
	"g_godsb", "g_gods", NULL, "1900",
	"Gods (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_godsbRomInfo, g_godsbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Golden Axe (W) (REV01) [!]
static struct BurnRomInfo g_gaxRomDesc[] = {
	{ "g_gax.bin", 0x80000, 0x665d7df9, BRF_PRG },
};

STD_ROM_PICK(g_gax);
STD_ROM_FN(g_gax);

struct BurnDriver BurnDrvMDg_gax = {
	"g_gax", NULL, NULL, "1900",
	"Golden Axe (W) (REV01) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_gaxRomInfo, g_gaxRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Golden Axe (W) (REV00) [!]
static struct BurnRomInfo g_gax00RomDesc[] = {
	{ "g_gax00.bin", 0x80000, 0xe8182b90, BRF_PRG },
};

STD_ROM_PICK(g_gax00);
STD_ROM_FN(g_gax00);

struct BurnDriver BurnDrvMDg_gax00 = {
	"g_gax00", "g_gax", NULL, "1900",
	"Golden Axe (W) (REV00) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_gax00RomInfo, g_gax00RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Golden Axe II (W) [!]
static struct BurnRomInfo g_gax2RomDesc[] = {
	{ "g_gax2.bin", 0x80000, 0x725e0a18, BRF_PRG },
};

STD_ROM_PICK(g_gax2);
STD_ROM_FN(g_gax2);

struct BurnDriverD BurnDrvMDg_gax2 = {
	"g_gax2", NULL, NULL, "1900",
	"Golden Axe II (W) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_gax2RomInfo, g_gax2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Golden Axe II (Beta)
static struct BurnRomInfo g_gax2bRomDesc[] = {
	{ "g_gax2b.bin", 0x80000, 0xe62ea1bb, BRF_PRG },
};

STD_ROM_PICK(g_gax2b);
STD_ROM_FN(g_gax2b);

struct BurnDriverD BurnDrvMDg_gax2b = {
	"g_gax2b", "g_gax2", NULL, "1900",
	"Golden Axe II (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_gax2bRomInfo, g_gax2bRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Golden Axe III (J) [!]
static struct BurnRomInfo g_gax3RomDesc[] = {
	{ "g_gax3.bin", 0x100000, 0xc7862ea3, BRF_PRG },
};

STD_ROM_PICK(g_gax3);
STD_ROM_FN(g_gax3);

struct BurnDriverD BurnDrvMDg_gax3 = {
	"g_gax3", NULL, NULL, "1900",
	"Golden Axe III (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_gax3RomInfo, g_gax3RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Granada (JU) (REV01) [!]
static struct BurnRomInfo g_granRomDesc[] = {
	{ "g_gran.bin", 0x80000, 0xe89d1e66, BRF_PRG },
};

STD_ROM_PICK(g_gran);
STD_ROM_FN(g_gran);

struct BurnDriverD BurnDrvMDg_gran = {
	"g_gran", NULL, NULL, "1900",
	"Granada (JU) (REV01) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_granRomInfo, g_granRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Granada (JU) (REV00) [!]
static struct BurnRomInfo g_gran00RomDesc[] = {
	{ "g_gran00.bin", 0x80000, 0x7f45719b, BRF_PRG },
};

STD_ROM_PICK(g_gran00);
STD_ROM_FN(g_gran00);

struct BurnDriverD BurnDrvMDg_gran00 = {
	"g_gran00", "g_gran", NULL, "1900",
	"Granada (JU) (REV00) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_gran00RomInfo, g_gran00RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// GRIND Stormer (U) [!]
static struct BurnRomInfo g_grinRomDesc[] = {
	{ "g_grin.bin", 0x100000, 0x7e6bef15, BRF_PRG },
};

STD_ROM_PICK(g_grin);
STD_ROM_FN(g_grin);

struct BurnDriverD BurnDrvMDg_grin = {
	"g_grin", NULL, NULL, "1900",
	"GRIND Stormer (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_grinRomInfo, g_grinRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// V-Five (J) [!]
static struct BurnRomInfo g_v5RomDesc[] = {
	{ "g_v5.bin", 0x100000, 0xad9d0ec0, BRF_PRG },
};

STD_ROM_PICK(g_v5);
STD_ROM_FN(g_v5);

struct BurnDriverD BurnDrvMDg_v5 = {
	"g_v5", "g_grin", NULL, "1900",
	"V-Five (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_v5RomInfo, g_v5RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Head-On Soccer (U) (M5) [!]
static struct BurnRomInfo g_hosoRomDesc[] = {
	{ "g_hoso.bin", 0x200000, 0xdcffa327, BRF_PRG },
};

STD_ROM_PICK(g_hoso);
STD_ROM_FN(g_hoso);

struct BurnDriverD BurnDrvMDg_hoso = {
	"g_hoso", NULL, NULL, "1900",
	"Head-On Soccer (U) (M5) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_hosoRomInfo, g_hosoRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Fever Pitch Soccer (E) (M5) [!]
static struct BurnRomInfo g_feverRomDesc[] = {
	{ "g_fever.bin", 0x200000, 0xfac29677, BRF_PRG },
};

STD_ROM_PICK(g_fever);
STD_ROM_FN(g_fever);

struct BurnDriverD BurnDrvMDg_fever = {
	"g_fever", "g_hoso", NULL, "1900",
	"Fever Pitch Soccer (E) (M5) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_feverRomInfo, g_feverRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Herzog Zwei (UE) [!]
static struct BurnRomInfo g_herzRomDesc[] = {
	{ "g_herz.bin", 0x80000, 0xa605b65b, BRF_PRG },
};

STD_ROM_PICK(g_herz);
STD_ROM_FN(g_herz);

struct BurnDriverD BurnDrvMDg_herz = {
	"g_herz", NULL, NULL, "1900",
	"Herzog Zwei (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_herzRomInfo, g_herzRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Herzog Zwei (J) [!]
static struct BurnRomInfo g_herzojRomDesc[] = {
	{ "g_herzoj.bin", 0x80000, 0x4cf676b3, BRF_PRG },
};

STD_ROM_PICK(g_herzoj);
STD_ROM_FN(g_herzoj);

struct BurnDriverD BurnDrvMDg_herzoj = {
	"g_herzoj", "g_herz", NULL, "1900",
	"Herzog Zwei (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_herzojRomInfo, g_herzojRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Jurassic Park (U) [!]
static struct BurnRomInfo g_jparRomDesc[] = {
	{ "g_jpar.bin", 0x200000, 0x7b31deef, BRF_PRG },
};

STD_ROM_PICK(g_jpar);
STD_ROM_FN(g_jpar);

struct BurnDriverD BurnDrvMDg_jpar = {
	"g_jpar", NULL, NULL, "1900",
	"Jurassic Park (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jparRomInfo, g_jparRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Jurassic Park (E) [!]
static struct BurnRomInfo g_jparkeRomDesc[] = {
	{ "g_jparke.bin", 0x200000, 0x448341f6, BRF_PRG },
};

STD_ROM_PICK(g_jparke);
STD_ROM_FN(g_jparke);

struct BurnDriverD BurnDrvMDg_jparke = {
	"g_jparke", "g_jpar", NULL, "1900",
	"Jurassic Park (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jparkeRomInfo, g_jparkeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Jurassic Park (J) [!]
static struct BurnRomInfo g_jparkjRomDesc[] = {
	{ "g_jparkj.bin", 0x200000, 0xec8e5783, BRF_PRG },
};

STD_ROM_PICK(g_jparkj);
STD_ROM_FN(g_jparkj);

struct BurnDriverD BurnDrvMDg_jparkj = {
	"g_jparkj", "g_jpar", NULL, "1900",
	"Jurassic Park (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jparkjRomInfo, g_jparkjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Jurassic Park (Beta)
static struct BurnRomInfo g_jparkbRomDesc[] = {
	{ "g_jparkb.bin", 0x200000, 0xcf890eed, BRF_PRG },
};

STD_ROM_PICK(g_jparkb);
STD_ROM_FN(g_jparkb);

struct BurnDriverD BurnDrvMDg_jparkb = {
	"g_jparkb", "g_jpar", NULL, "1900",
	"Jurassic Park (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jparkbRomInfo, g_jparkbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Justice League Task Force (W) [!]
static struct BurnRomInfo g_jltfRomDesc[] = {
	{ "g_jltf.bin", 0x300000, 0x2a60ebe9, BRF_PRG },
};

STD_ROM_PICK(g_jltf);
STD_ROM_FN(g_jltf);

struct BurnDriverD BurnDrvMDg_jltf = {
	"g_jltf", NULL, NULL, "1900",
	"Justice League Task Force (W) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jltfRomInfo, g_jltfRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Kawasaki Superbike Challenge (UE) [!]
static struct BurnRomInfo g_kawaRomDesc[] = {
	{ "g_kawa.bin", 0x100000, 0x631cc8e9, BRF_PRG },
};

STD_ROM_PICK(g_kawa);
STD_ROM_FN(g_kawa);

struct BurnDriverD BurnDrvMDg_kawa = {
	"g_kawa", NULL, NULL, "1900",
	"Kawasaki Superbike Challenge (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_kawaRomInfo, g_kawaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Kawasaki Superbike Challenge (Beta)
static struct BurnRomInfo g_kawabRomDesc[] = {
	{ "g_kawab.bin", 0x100000, 0x55934d1b, BRF_PRG },
};

STD_ROM_PICK(g_kawab);
STD_ROM_FN(g_kawab);

struct BurnDriverD BurnDrvMDg_kawab = {
	"g_kawab", "g_kawa", NULL, "1900",
	"Kawasaki Superbike Challenge (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_kawabRomInfo, g_kawabRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Krusty's Super Funhouse (UE) (REV01) [!]
static struct BurnRomInfo g_ksfhRomDesc[] = {
	{ "g_ksfh.bin", 0x80000, 0x56976261, BRF_PRG },
};

STD_ROM_PICK(g_ksfh);
STD_ROM_FN(g_ksfh);

struct BurnDriverD BurnDrvMDg_ksfh = {
	"g_ksfh", NULL, NULL, "1900",
	"Krusty's Super Funhouse (UE) (REV01) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ksfhRomInfo, g_ksfhRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Krusty's Super Funhouse (UE) (REV00) [!]
static struct BurnRomInfo g_ksfh00RomDesc[] = {
	{ "g_ksfh00.bin", 0x80000, 0xf764005e, BRF_PRG },
};

STD_ROM_PICK(g_ksfh00);
STD_ROM_FN(g_ksfh00);

struct BurnDriverD BurnDrvMDg_ksfh00 = {
	"g_ksfh00", "g_ksfh", NULL, "1900",
	"Krusty's Super Funhouse (UE) (REV00) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ksfh00RomInfo, g_ksfh00RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Last Battle (UE) [!]
static struct BurnRomInfo g_lbatRomDesc[] = {
	{ "g_lbat.bin", 0x80000, 0xbbfaad77, BRF_PRG },
};

STD_ROM_PICK(g_lbat);
STD_ROM_FN(g_lbat);

struct BurnDriverD BurnDrvMDg_lbat = {
	"g_lbat", NULL, NULL, "1900",
	"Last Battle (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_lbatRomInfo, g_lbatRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Hokuto no Ken (J) [!]
static struct BurnRomInfo g_hokutoRomDesc[] = {
	{ "g_hokuto.bin", 0x80000, 0x1b6585e7, BRF_PRG },
};

STD_ROM_PICK(g_hokuto);
STD_ROM_FN(g_hokuto);

struct BurnDriverD BurnDrvMDg_hokuto = {
	"g_hokuto", "g_lbat", NULL, "1900",
	"Hokuto no Ken (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_hokutoRomInfo, g_hokutoRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Lethal Enforcers (U) [!]
static struct BurnRomInfo g_lenRomDesc[] = {
	{ "g_len.bin", 0x200000, 0x51d9a84a, BRF_PRG },
};

STD_ROM_PICK(g_len);
STD_ROM_FN(g_len);

struct BurnDriverD BurnDrvMDg_len = {
	"g_len", NULL, NULL, "1900",
	"Lethal Enforcers (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_lenRomInfo, g_lenRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Lethal Enforcers (E) [!]
static struct BurnRomInfo g_lenfeRomDesc[] = {
	{ "g_lenfe.bin", 0x200000, 0xca2bf99d, BRF_PRG },
};

STD_ROM_PICK(g_lenfe);
STD_ROM_FN(g_lenfe);

struct BurnDriverD BurnDrvMDg_lenfe = {
	"g_lenfe", "g_len", NULL, "1900",
	"Lethal Enforcers (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_lenfeRomInfo, g_lenfeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Lethal Enforcers (J) [!]
static struct BurnRomInfo g_lenfjRomDesc[] = {
	{ "g_lenfj.bin", 0x200000, 0xf25f1e49, BRF_PRG },
};

STD_ROM_PICK(g_lenfj);
STD_ROM_FN(g_lenfj);

struct BurnDriverD BurnDrvMDg_lenfj = {
	"g_lenfj", "g_len", NULL, "1900",
	"Lethal Enforcers (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_lenfjRomInfo, g_lenfjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Lethal Enforcers II - Gun Fighters (U) [!]
static struct BurnRomInfo g_len2RomDesc[] = {
	{ "g_len2.bin", 0x200000, 0xe5fdd28b, BRF_PRG },
};

STD_ROM_PICK(g_len2);
STD_ROM_FN(g_len2);

struct BurnDriverD BurnDrvMDg_len2 = {
	"g_len2", NULL, NULL, "1900",
	"Lethal Enforcers II - Gun Fighters (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_len2RomInfo, g_len2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Lethal Enforcers II - Gun Fighters (E) [!]
static struct BurnRomInfo g_lth2jRomDesc[] = {
	{ "g_lth2j.bin", 0x200000, 0x4bfe045c, BRF_PRG },
};

STD_ROM_PICK(g_lth2j);
STD_ROM_FN(g_lth2j);

struct BurnDriverD BurnDrvMDg_lth2j = {
	"g_lth2j", "g_len2", NULL, "1900",
	"Lethal Enforcers II - Gun Fighters (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_lth2jRomInfo, g_lth2jRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Light Crusader (U) [!]
static struct BurnRomInfo g_lighRomDesc[] = {
	{ "g_ligh.bin", 0x200000, 0xbeb715dc, BRF_PRG },
};

STD_ROM_PICK(g_ligh);
STD_ROM_FN(g_ligh);

struct BurnDriverD BurnDrvMDg_ligh = {
	"g_ligh", NULL, NULL, "1900",
	"Light Crusader (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_lighRomInfo, g_lighRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Light Crusader (E) (M4) [!]
static struct BurnRomInfo g_licrueRomDesc[] = {
	{ "g_licrue.bin", 0x200000, 0x52c7252b, BRF_PRG },
};

STD_ROM_PICK(g_licrue);
STD_ROM_FN(g_licrue);

struct BurnDriverD BurnDrvMDg_licrue = {
	"g_licrue", "g_ligh", NULL, "1900",
	"Light Crusader (E) (M4) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_licrueRomInfo, g_licrueRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Light Crusader (J) [!]
static struct BurnRomInfo g_licrujRomDesc[] = {
	{ "g_licruj.bin", 0x200000, 0x237076a4, BRF_PRG },
};

STD_ROM_PICK(g_licruj);
STD_ROM_FN(g_licruj);

struct BurnDriverD BurnDrvMDg_licruj = {
	"g_licruj", "g_ligh", NULL, "1900",
	"Light Crusader (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_licrujRomInfo, g_licrujRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Light Crusader (K) [!]
static struct BurnRomInfo g_licrukRomDesc[] = {
	{ "g_licruk.bin", 0x200000, 0x6d0cbcb2, BRF_PRG },
};

STD_ROM_PICK(g_licruk);
STD_ROM_FN(g_licruk);

struct BurnDriverD BurnDrvMDg_licruk = {
	"g_licruk", "g_ligh", NULL, "1900",
	"Light Crusader (K) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_licrukRomInfo, g_licrukRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Lost Vikings, The (U) [!]
static struct BurnRomInfo g_lostRomDesc[] = {
	{ "g_lost.bin", 0x100000, 0x7ba49edb, BRF_PRG },
};

STD_ROM_PICK(g_lost);
STD_ROM_FN(g_lost);

struct BurnDriverD BurnDrvMDg_lost = {
	"g_lost", NULL, NULL, "1900",
	"Lost Vikings, The (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_lostRomInfo, g_lostRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Lost Vikings, The (E)
static struct BurnRomInfo g_lostveRomDesc[] = {
	{ "g_lostve.bin", 0x100000, 0x1f14efc6, BRF_PRG },
};

STD_ROM_PICK(g_lostve);
STD_ROM_FN(g_lostve);

struct BurnDriverD BurnDrvMDg_lostve = {
	"g_lostve", "g_lost", NULL, "1900",
	"Lost Vikings, The (E)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_lostveRomInfo, g_lostveRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Lost Vikings, The (Beta)
static struct BurnRomInfo g_lostvbRomDesc[] = {
	{ "g_lostvb.bin", 0x100000, 0x17bed25f, BRF_PRG },
};

STD_ROM_PICK(g_lostvb);
STD_ROM_FN(g_lostvb);

struct BurnDriverD BurnDrvMDg_lostvb = {
	"g_lostvb", "g_lost", NULL, "1900",
	"Lost Vikings, The (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_lostvbRomInfo, g_lostvbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Marko's Magic Football (U) [!]
static struct BurnRomInfo g_mmfRomDesc[] = {
	{ "g_mmf.bin", 0x200000, 0x2b8c8cce, BRF_PRG },
};

STD_ROM_PICK(g_mmf);
STD_ROM_FN(g_mmf);

struct BurnDriverD BurnDrvMDg_mmf = {
	"g_mmf", NULL, NULL, "1900",
	"Marko's Magic Football (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mmfRomInfo, g_mmfRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Marko's Magic Football (E) (M4) [!]
static struct BurnRomInfo g_mmftbeRomDesc[] = {
	{ "g_mmftbe.bin", 0x200000, 0x2307b905, BRF_PRG },
};

STD_ROM_PICK(g_mmftbe);
STD_ROM_FN(g_mmftbe);

struct BurnDriverD BurnDrvMDg_mmftbe = {
	"g_mmftbe", "g_mmf", NULL, "1900",
	"Marko's Magic Football (E) (M4) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mmftbeRomInfo, g_mmftbeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Marko's Magic Football (Beta) (M4)
static struct BurnRomInfo g_mmfbRomDesc[] = {
	{ "g_mmfb.bin", 0x200000, 0x0273e564, BRF_PRG },
};

STD_ROM_PICK(g_mmfb);
STD_ROM_FN(g_mmfb);

struct BurnDriverD BurnDrvMDg_mmfb = {
	"g_mmfb", "g_mmf", NULL, "1900",
	"Marko's Magic Football (Beta) (M4)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mmfbRomInfo, g_mmfbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mortal Kombat (UE) (REV01) [c][!]
static struct BurnRomInfo g_mkRomDesc[] = {
	{ "g_mk.bin", 0x200000, 0x33f19ab6, BRF_PRG },
};

STD_ROM_PICK(g_mk);
STD_ROM_FN(g_mk);

struct BurnDriverD BurnDrvMDg_mk = {
	"g_mk", NULL, NULL, "1900",
	"Mortal Kombat (UE) (REV01) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mkRomInfo, g_mkRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mortal Kombat (W) (REV00) [!]
static struct BurnRomInfo g_mk00RomDesc[] = {
	{ "g_mk00.bin", 0x200000, 0x1aa3a207, BRF_PRG },
};

STD_ROM_PICK(g_mk00);
STD_ROM_FN(g_mk00);

struct BurnDriverD BurnDrvMDg_mk00 = {
	"g_mk00", "g_mk", NULL, "1900",
	"Mortal Kombat (W) (REV00) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mk00RomInfo, g_mk00RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mortal Kombat II (W) [!]
static struct BurnRomInfo g_mk2RomDesc[] = {
	{ "g_mk2.bin", 0x300000, 0xa9e013d8, BRF_PRG },
};

STD_ROM_PICK(g_mk2);
STD_ROM_FN(g_mk2);

struct BurnDriverD BurnDrvMDg_mk2 = {
	"g_mk2", NULL, NULL, "1900",
	"Mortal Kombat II (W) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mk2RomInfo, g_mk2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mortal Kombat 3 (U) [!]
static struct BurnRomInfo g_mk3RomDesc[] = {
	{ "g_mk3.bin", 0x400000, 0xdd638af6, BRF_PRG },
};

STD_ROM_PICK(g_mk3);
STD_ROM_FN(g_mk3);

struct BurnDriverD BurnDrvMDg_mk3 = {
	"g_mk3", NULL, NULL, "1900",
	"Mortal Kombat 3 (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mk3RomInfo, g_mk3RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mortal Kombat 3 (E) [!]
static struct BurnRomInfo g_mk3eRomDesc[] = {
	{ "g_mk3e.bin", 0x400000, 0xaf6de3e8, BRF_PRG },
};

STD_ROM_PICK(g_mk3e);
STD_ROM_FN(g_mk3e);

struct BurnDriverD BurnDrvMDg_mk3e = {
	"g_mk3e", "g_mk3", NULL, "1900",
	"Mortal Kombat 3 (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mk3eRomInfo, g_mk3eRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Ultimate Mortal Kombat 3 (U) [!]
static struct BurnRomInfo g_umk3RomDesc[] = {
	{ "g_umk3.bin", 0x400000, 0x7290770d, BRF_PRG },
};

STD_ROM_PICK(g_umk3);
STD_ROM_FN(g_umk3);

struct BurnDriverD BurnDrvMDg_umk3 = {
	"g_umk3", NULL, NULL, "1900",
	"Ultimate Mortal Kombat 3 (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_umk3RomInfo, g_umk3RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Ultimate Mortal Kombat 3 (E) [!]
static struct BurnRomInfo g_umk3eRomDesc[] = {
	{ "g_umk3e.bin", 0x400000, 0xecfb5cb4, BRF_PRG },
};

STD_ROM_PICK(g_umk3e);
STD_ROM_FN(g_umk3e);

struct BurnDriverD BurnDrvMDg_umk3e = {
	"g_umk3e", "g_umk3", NULL, "1900",
	"Ultimate Mortal Kombat 3 (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_umk3eRomInfo, g_umk3eRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Marvel Land (U) [!]
static struct BurnRomInfo g_marvRomDesc[] = {
	{ "g_marv.bin", 0x100000, 0xcd7eeeb7, BRF_PRG },
};

STD_ROM_PICK(g_marv);
STD_ROM_FN(g_marv);

struct BurnDriverD BurnDrvMDg_marv = {
	"g_marv", NULL, NULL, "1900",
	"Marvel Land (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_marvRomInfo, g_marvRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Marvel Land (J) [!]
static struct BurnRomInfo g_marvjRomDesc[] = {
	{ "g_marvj.bin", 0x100000, 0x5d162d21, BRF_PRG },
};

STD_ROM_PICK(g_marvj);
STD_ROM_FN(g_marvj);

struct BurnDriverD BurnDrvMDg_marvj = {
	"g_marvj", "g_marv", NULL, "1900",
	"Marvel Land (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_marvjRomInfo, g_marvjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Talmit's Adventure (E) [!]
static struct BurnRomInfo g_talmitRomDesc[] = {
	{ "g_talmit.bin", 0x100000, 0x05dc3ffc, BRF_PRG },
};

STD_ROM_PICK(g_talmit);
STD_ROM_FN(g_talmit);

struct BurnDriverD BurnDrvMDg_talmit = {
	"g_talmit", "g_marv", NULL, "1900",
	"Talmit's Adventure (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_talmitRomInfo, g_talmitRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Michael Jackson's Moonwalker (W) (REV01) [!]
static struct BurnRomInfo g_mjmwRomDesc[] = {
	{ "g_mjmw.bin", 0x80000, 0x11ce1f9e, BRF_PRG },
};

STD_ROM_PICK(g_mjmw);
STD_ROM_FN(g_mjmw);

struct BurnDriver BurnDrvMDg_mjmw = {
	"g_mjmw", NULL, NULL, "1900",
	"Michael Jackson's Moonwalker (W) (REV01) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mjmwRomInfo, g_mjmwRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Michael Jackson's Moonwalker (W) (REV00) [!]
static struct BurnRomInfo g_mwlk00RomDesc[] = {
	{ "g_mwlk00.bin", 0x80000, 0x6a70791b, BRF_PRG },
};

STD_ROM_PICK(g_mwlk00);
STD_ROM_FN(g_mwlk00);

struct BurnDriver BurnDrvMDg_mwlk00 = {
	"g_mwlk00", "g_mjmw", NULL, "1900",
	"Michael Jackson's Moonwalker (W) (REV00) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mwlk00RomInfo, g_mwlk00RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Monopoly (U) [c][!]
static struct BurnRomInfo g_monoRomDesc[] = {
	{ "g_mono.bin", 0x80000, 0xc10268da, BRF_PRG },
};

STD_ROM_PICK(g_mono);
STD_ROM_FN(g_mono);

struct BurnDriverD BurnDrvMDg_mono = {
	"g_mono", NULL, NULL, "1900",
	"Monopoly (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_monoRomInfo, g_monoRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Monopoly (Beta)
static struct BurnRomInfo g_monobRomDesc[] = {
	{ "g_monob.bin", 0x80000, 0xdfbcc3fa, BRF_PRG },
};

STD_ROM_PICK(g_monob);
STD_ROM_FN(g_monob);

struct BurnDriverD BurnDrvMDg_monob = {
	"g_monob", "g_mono", NULL, "1900",
	"Monopoly (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_monobRomInfo, g_monobRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// M.U.S.H.A (U) [!]
static struct BurnRomInfo g_mushRomDesc[] = {
	{ "g_mush.bin", 0x80000, 0x58a7f7b4, BRF_PRG },
};

STD_ROM_PICK(g_mush);
STD_ROM_FN(g_mush);

struct BurnDriverD BurnDrvMDg_mush = {
	"g_mush", NULL, NULL, "1900",
	"M.U.S.H.A (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mushRomInfo, g_mushRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Musha Aleste - Full Metal Fighter Ellinor (J) [!]
static struct BurnRomInfo g_mushajRomDesc[] = {
	{ "g_mushaj.bin", 0x80000, 0x8fde18ab, BRF_PRG },
};

STD_ROM_PICK(g_mushaj);
STD_ROM_FN(g_mushaj);

struct BurnDriverD BurnDrvMDg_mushaj = {
	"g_mushaj", "g_mush", NULL, "1900",
	"Musha Aleste - Full Metal Fighter Ellinor (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mushajRomInfo, g_mushajRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mystical Fighter (U) [c][!]
static struct BurnRomInfo g_mysfRomDesc[] = {
	{ "g_mysf.bin", 0x80000, 0xb2f2a69b, BRF_PRG },
};

STD_ROM_PICK(g_mysf);
STD_ROM_FN(g_mysf);

struct BurnDriverD BurnDrvMDg_mysf = {
	"g_mysf", NULL, NULL, "1900",
	"Mystical Fighter (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mysfRomInfo, g_mysfRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Maou Renjishi (J) [c][!]
static struct BurnRomInfo g_maoureRomDesc[] = {
	{ "g_maoure.bin", 0x80000, 0x24a7f28c, BRF_PRG },
};

STD_ROM_PICK(g_maoure);
STD_ROM_FN(g_maoure);

struct BurnDriverD BurnDrvMDg_maoure = {
	"g_maoure", "g_mysf", NULL, "1900",
	"Maou Renjishi (J) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_maoureRomInfo, g_maoureRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// NHL 94 (UE) [!]
static struct BurnRomInfo g_nh94RomDesc[] = {
	{ "nhl94 hl9402", 0x100000, 0xacecd225, BRF_PRG },
};

STD_ROM_PICK(g_nh94);
STD_ROM_FN(g_nh94);

struct BurnDriverD BurnDrvMDg_nh94 = {
	"g_nh94", NULL, NULL, "1900",
	"NHL 94 (UE) [!]\0", NULL, "Electronic Arts", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_nh94RomInfo, g_nh94RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveNoByteswapInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Olympic Gold - Barcelona 92 (U) (M8) [c][!]
static struct BurnRomInfo g_ogolRomDesc[] = {
	{ "g_ogol.bin", 0x80000, 0x339594b4, BRF_PRG },
};

STD_ROM_PICK(g_ogol);
STD_ROM_FN(g_ogol);

struct BurnDriverD BurnDrvMDg_ogol = {
	"g_ogol", NULL, NULL, "1900",
	"Olympic Gold - Barcelona 92 (U) (M8) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ogolRomInfo, g_ogolRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Olympic Gold - Barcelona 92 (E) (M8) [c][!]
static struct BurnRomInfo g_olgoleRomDesc[] = {
	{ "g_olgole.bin", 0x80000, 0x924e57d3, BRF_PRG },
};

STD_ROM_PICK(g_olgole);
STD_ROM_FN(g_olgole);

struct BurnDriverD BurnDrvMDg_olgole = {
	"g_olgole", "g_ogol", NULL, "1900",
	"Olympic Gold - Barcelona 92 (E) (M8) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_olgoleRomInfo, g_olgoleRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Olympic Gold - Barcelona 92 (J) (M8) [c][!]
static struct BurnRomInfo g_olgoljRomDesc[] = {
	{ "g_olgolj.bin", 0x80000, 0xe9c925b8, BRF_PRG },
};

STD_ROM_PICK(g_olgolj);
STD_ROM_FN(g_olgolj);

struct BurnDriverD BurnDrvMDg_olgolj = {
	"g_olgolj", "g_ogol", NULL, "1900",
	"Olympic Gold - Barcelona 92 (J) (M8) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_olgoljRomInfo, g_olgoljRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Olympic Gold - Barcelona 92 (U) (M8) [a1][c][!]
static struct BurnRomInfo g_olgoluRomDesc[] = {
	{ "g_olgolu.bin", 0x80000, 0xaf639376, BRF_PRG },
};

STD_ROM_PICK(g_olgolu);
STD_ROM_FN(g_olgolu);

struct BurnDriverD BurnDrvMDg_olgolu = {
	"g_olgolu", "g_ogol", NULL, "1900",
	"Olympic Gold - Barcelona 92 (U) (M8) [a1][c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_olgoluRomInfo, g_olgoluRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Ottifants, The (E) (M5) [!]
static struct BurnRomInfo g_ottiRomDesc[] = {
	{ "g_otti.bin", 0x100000, 0x41ac8003, BRF_PRG },
};

STD_ROM_PICK(g_otti);
STD_ROM_FN(g_otti);

struct BurnDriverD BurnDrvMDg_otti = {
	"g_otti", NULL, NULL, "1900",
	"Ottifants, The (E) (M5) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ottiRomInfo, g_ottiRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Ottifants, The (Beta)
static struct BurnRomInfo g_ottifbRomDesc[] = {
	{ "g_ottifb.bin", 0x100000, 0xc6e3dd23, BRF_PRG },
};

STD_ROM_PICK(g_ottifb);
STD_ROM_FN(g_ottifb);

struct BurnDriverD BurnDrvMDg_ottifb = {
	"g_ottifb", "g_otti", NULL, "1900",
	"Ottifants, The (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ottifbRomInfo, g_ottifbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// PGA Tour Golf (UE) (REV02) [!]
static struct BurnRomInfo g_pgaRomDesc[] = {
	{ "g_pga.bin", 0x80000, 0xc1f0b4e1, BRF_PRG },
};

STD_ROM_PICK(g_pga);
STD_ROM_FN(g_pga);

struct BurnDriverD BurnDrvMDg_pga = {
	"g_pga", NULL, NULL, "1900",
	"PGA Tour Golf (UE) (REV02) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pgaRomInfo, g_pgaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// PGA Tour Golf (UE) (REV01) [!]
static struct BurnRomInfo g_pgat01RomDesc[] = {
	{ "g_pgat01.bin", 0x80000, 0x0489ff8e, BRF_PRG },
};

STD_ROM_PICK(g_pgat01);
STD_ROM_FN(g_pgat01);

struct BurnDriverD BurnDrvMDg_pgat01 = {
	"g_pgat01", "g_pga", NULL, "1900",
	"PGA Tour Golf (UE) (REV01) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pgat01RomInfo, g_pgat01RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// PGA Tour Golf II (UE) (REV01) [!]
static struct BurnRomInfo g_pga2RomDesc[] = {
	{ "g_pga2.bin", 0x100000, 0xe82b8606, BRF_PRG },
};

STD_ROM_PICK(g_pga2);
STD_ROM_FN(g_pga2);

struct BurnDriverD BurnDrvMDg_pga2 = {
	"g_pga2", NULL, NULL, "1900",
	"PGA Tour Golf II (UE) (REV01) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pga2RomInfo, g_pga2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// PGA Tour Golf II (J) [!]
static struct BurnRomInfo g_pga2jRomDesc[] = {
	{ "g_pga2j.bin", 0x100000, 0xc05b7a4a, BRF_PRG },
};

STD_ROM_PICK(g_pga2j);
STD_ROM_FN(g_pga2j);

struct BurnDriverD BurnDrvMDg_pga2j = {
	"g_pga2j", "g_pga2", NULL, "1900",
	"PGA Tour Golf II (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pga2jRomInfo, g_pga2jRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// PGA Tour Golf II (UE) (REV00) [c][!]
static struct BurnRomInfo g_pga200RomDesc[] = {
	{ "g_pga200.bin", 0x100000, 0x16b2d816, BRF_PRG },
};

STD_ROM_PICK(g_pga200);
STD_ROM_FN(g_pga200);

struct BurnDriverD BurnDrvMDg_pga200 = {
	"g_pga200", "g_pga2", NULL, "1900",
	"PGA Tour Golf II (UE) (REV00) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pga200RomInfo, g_pga200RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// PGA Tour Golf III (UE) [!]
static struct BurnRomInfo g_pga3RomDesc[] = {
	{ "g_pga3.bin", 0x200000, 0xaeb3f65f, BRF_PRG },
};

STD_ROM_PICK(g_pga3);
STD_ROM_FN(g_pga3);

struct BurnDriverD BurnDrvMDg_pga3 = {
	"g_pga3", NULL, NULL, "1900",
	"PGA Tour Golf III (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pga3RomInfo, g_pga3RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// PGA Tour 96 (UE) [!]
static struct BurnRomInfo g_pg96RomDesc[] = {
	{ "g_pg96.bin", 0x200000, 0x9698bbde, BRF_PRG },
};

STD_ROM_PICK(g_pg96);
STD_ROM_FN(g_pg96);

struct BurnDriverD BurnDrvMDg_pg96 = {
	"g_pg96", NULL, NULL, "1900",
	"PGA Tour 96 (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pg96RomInfo, g_pg96RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Phelios (U) [c][!]
static struct BurnRomInfo g_phelRomDesc[] = {
	{ "g_phel.bin", 0x80000, 0x11c79320, BRF_PRG },
};

STD_ROM_PICK(g_phel);
STD_ROM_FN(g_phel);

struct BurnDriverD BurnDrvMDg_phel = {
	"g_phel", NULL, NULL, "1900",
	"Phelios (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_phelRomInfo, g_phelRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Phelios (E) [c][!]
static struct BurnRomInfo g_phelieRomDesc[] = {
	{ "g_phelie.bin", 0x80000, 0x13abc2b2, BRF_PRG },
};

STD_ROM_PICK(g_phelie);
STD_ROM_FN(g_phelie);

struct BurnDriverD BurnDrvMDg_phelie = {
	"g_phelie", "g_phel", NULL, "1900",
	"Phelios (E) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_phelieRomInfo, g_phelieRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Phelios (J) [c][!]
static struct BurnRomInfo g_phelijRomDesc[] = {
	{ "g_phelij.bin", 0x80000, 0x94596174, BRF_PRG },
};

STD_ROM_PICK(g_phelij);
STD_ROM_FN(g_phelij);

struct BurnDriverD BurnDrvMDg_phelij = {
	"g_phelij", "g_phel", NULL, "1900",
	"Phelios (J) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_phelijRomInfo, g_phelijRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Pirates! Gold (U) [!]
static struct BurnRomInfo g_pirgRomDesc[] = {
	{ "g_pirg.bin", 0x100000, 0xed50e75c, BRF_PRG },
};

STD_ROM_PICK(g_pirg);
STD_ROM_FN(g_pirg);

struct BurnDriverD BurnDrvMDg_pirg = {
	"g_pirg", NULL, NULL, "1900",
	"Pirates! Gold (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pirgRomInfo, g_pirgRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Pirates! Gold (Beta)
static struct BurnRomInfo g_pgoldbRomDesc[] = {
	{ "g_pgoldb.bin", 0x100000, 0x0a525641, BRF_PRG },
};

STD_ROM_PICK(g_pgoldb);
STD_ROM_FN(g_pgoldb);

struct BurnDriverD BurnDrvMDg_pgoldb = {
	"g_pgoldb", "g_pirg", NULL, "1900",
	"Pirates! Gold (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pgoldbRomInfo, g_pgoldbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Populous (U) [!]
static struct BurnRomInfo g_popuRomDesc[] = {
	{ "g_popu.bin", 0x80000, 0xbd74b31e, BRF_PRG },
};

STD_ROM_PICK(g_popu);
STD_ROM_FN(g_popu);

struct BurnDriverD BurnDrvMDg_popu = {
	"g_popu", NULL, NULL, "1900",
	"Populous (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_popuRomInfo, g_popuRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Populous (E) [!]
static struct BurnRomInfo g_popueRomDesc[] = {
	{ "g_popue.bin", 0x80000, 0x83d56f64, BRF_PRG },
};

STD_ROM_PICK(g_popue);
STD_ROM_FN(g_popue);

struct BurnDriverD BurnDrvMDg_popue = {
	"g_popue", "g_popu", NULL, "1900",
	"Populous (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_popueRomInfo, g_popueRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Populous (J) [!]
static struct BurnRomInfo g_popujRomDesc[] = {
	{ "g_popuj.bin", 0x80000, 0x97c26818, BRF_PRG },
};

STD_ROM_PICK(g_popuj);
STD_ROM_FN(g_popuj);

struct BurnDriverD BurnDrvMDg_popuj = {
	"g_popuj", "g_popu", NULL, "1900",
	"Populous (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_popujRomInfo, g_popujRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Power Monger (UE) [!]
static struct BurnRomInfo g_pmonRomDesc[] = {
	{ "g_pmon.bin", 0x100000, 0xfb599b86, BRF_PRG },
};

STD_ROM_PICK(g_pmon);
STD_ROM_FN(g_pmon);

struct BurnDriverD BurnDrvMDg_pmon = {
	"g_pmon", NULL, NULL, "1900",
	"Power Monger (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pmonRomInfo, g_pmonRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Power Monger (J) [!]
static struct BurnRomInfo g_pmonjRomDesc[] = {
	{ "g_pmonj.bin", 0x100000, 0x553289b3, BRF_PRG },
};

STD_ROM_PICK(g_pmonj);
STD_ROM_FN(g_pmonj);

struct BurnDriverD BurnDrvMDg_pmonj = {
	"g_pmonj", "g_pmon", NULL, "1900",
	"Power Monger (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pmonjRomInfo, g_pmonjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Psycho Pinball (E) (Oct 1994) (M5) [c][!]
static struct BurnRomInfo g_ppinRomDesc[] = {
	{ "g_ppin.bin", 0x180000, 0xd704784b, BRF_PRG },
};

STD_ROM_PICK(g_ppin);
STD_ROM_FN(g_ppin);

struct BurnDriverD BurnDrvMDg_ppin = {
	"g_ppin", NULL, NULL, "1900",
	"Psycho Pinball (E) (Oct 1994) (M5) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ppinRomInfo, g_ppinRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Psycho Pinball (E) (Sep 1994) (M5) [c][!]
static struct BurnRomInfo g_ppinaRomDesc[] = {
	{ "g_ppina.bin", 0x180000, 0x11e9c3f2, BRF_PRG },
};

STD_ROM_PICK(g_ppina);
STD_ROM_FN(g_ppina);

struct BurnDriverD BurnDrvMDg_ppina = {
	"g_ppina", "g_ppin", NULL, "1900",
	"Psycho Pinball (E) (Sep 1994) (M5) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ppinaRomInfo, g_ppinaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Puggsy (U) [!]
static struct BurnRomInfo g_puggRomDesc[] = {
	{ "g_pugg.bin", 0x100000, 0x70132168, BRF_PRG },
};

STD_ROM_PICK(g_pugg);
STD_ROM_FN(g_pugg);

struct BurnDriverD BurnDrvMDg_pugg = {
	"g_pugg", NULL, NULL, "1900",
	"Puggsy (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_puggRomInfo, g_puggRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Puggsy (E) [!]
static struct BurnRomInfo g_puggseRomDesc[] = {
	{ "g_puggse.bin", 0x100000, 0x5d5c9ade, BRF_PRG },
};

STD_ROM_PICK(g_puggse);
STD_ROM_FN(g_puggse);

struct BurnDriverD BurnDrvMDg_puggse = {
	"g_puggse", "g_pugg", NULL, "1900",
	"Puggsy (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_puggseRomInfo, g_puggseRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Puggsy (Beta)
static struct BurnRomInfo g_puggsbRomDesc[] = {
	{ "g_puggsb.bin", 0x100000, 0x37fa4877, BRF_PRG },
};

STD_ROM_PICK(g_puggsb);
STD_ROM_FN(g_puggsb);

struct BurnDriverD BurnDrvMDg_puggsb = {
	"g_puggsb", "g_pugg", NULL, "1900",
	"Puggsy (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_puggsbRomInfo, g_puggsbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Punisher, The (U) [!]
static struct BurnRomInfo g_puniRomDesc[] = {
	{ "g_puni.bin", 0x200000, 0x695cd8b8, BRF_PRG },
};

STD_ROM_PICK(g_puni);
STD_ROM_FN(g_puni);

struct BurnDriverD BurnDrvMDg_puni = {
	"g_puni", NULL, NULL, "1900",
	"Punisher, The (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_puniRomInfo, g_puniRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Punisher, The (E)
static struct BurnRomInfo g_puniseRomDesc[] = {
	{ "g_punise.bin", 0x200000, 0x729edd17, BRF_PRG },
};

STD_ROM_PICK(g_punise);
STD_ROM_FN(g_punise);

struct BurnDriverD BurnDrvMDg_punise = {
	"g_punise", "g_puni", NULL, "1900",
	"Punisher, The (E)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_puniseRomInfo, g_puniseRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Puyo Puyo Tsu (J) (REV01) [!]
static struct BurnRomInfo g_puy2RomDesc[] = {
	{ "g_puy2.bin", 0x200000, 0x25b7b2aa, BRF_PRG },
};

STD_ROM_PICK(g_puy2);
STD_ROM_FN(g_puy2);

struct BurnDriverD BurnDrvMDg_puy2 = {
	"g_puy2", NULL, NULL, "1900",
	"Puyo Puyo Tsu (J) (REV01) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_puy2RomInfo, g_puy2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Puyo Puyo Tsu (J) (REV00) [!]
static struct BurnRomInfo g_puy200RomDesc[] = {
	{ "g_puy200.bin", 0x200000, 0x51ad7797, BRF_PRG },
};

STD_ROM_PICK(g_puy200);
STD_ROM_FN(g_puy200);

struct BurnDriverD BurnDrvMDg_puy200 = {
	"g_puy200", "g_puy2", NULL, "1900",
	"Puyo Puyo Tsu (J) (REV00) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_puy200RomInfo, g_puy200RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Ristar (UE) [!]
static struct BurnRomInfo g_ristRomDesc[] = {
	{ "g_rist.bin", 0x200000, 0x6511aa61, BRF_PRG },
};

STD_ROM_PICK(g_rist);
STD_ROM_FN(g_rist);

struct BurnDriverD BurnDrvMDg_rist = {
	"g_rist", NULL, NULL, "1900",
	"Ristar (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ristRomInfo, g_ristRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Ristar (UE) (REV00)
static struct BurnRomInfo g_rist00RomDesc[] = {
	{ "g_rist00.bin", 0x200000, 0x9700139b, BRF_PRG },
};

STD_ROM_PICK(g_rist00);
STD_ROM_FN(g_rist00);

struct BurnDriverD BurnDrvMDg_rist00 = {
	"g_rist00", "g_rist", NULL, "1900",
	"Ristar (UE) (REV00)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_rist00RomInfo, g_rist00RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Ristar - The Shooting Star (J) [!]
static struct BurnRomInfo g_ristjRomDesc[] = {
	{ "g_ristj.bin", 0x200000, 0xce464f0e, BRF_PRG },
};

STD_ROM_PICK(g_ristj);
STD_ROM_FN(g_ristj);

struct BurnDriverD BurnDrvMDg_ristj = {
	"g_ristj", "g_rist", NULL, "1900",
	"Ristar - The Shooting Star (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ristjRomInfo, g_ristjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Romance of the Three Kingdoms II (U) [!]
static struct BurnRomInfo g_r3k2RomDesc[] = {
	{ "g_r3k2.bin", 0x100000, 0x3d842478, BRF_PRG },
};

STD_ROM_PICK(g_r3k2);
STD_ROM_FN(g_r3k2);

struct BurnDriverD BurnDrvMDg_r3k2 = {
	"g_r3k2", NULL, NULL, "1900",
	"Romance of the Three Kingdoms II (U) [!]\0", NULL, "Koei", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_r3k2RomInfo, g_r3k2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sangokushi II (J) [!]
static struct BurnRomInfo g_sango2RomDesc[] = {
	{ "g_sango2.bin", 0x100000, 0x437ba326, BRF_PRG },
};

STD_ROM_PICK(g_sango2);
STD_ROM_FN(g_sango2);

struct BurnDriverD BurnDrvMDg_sango2 = {
	"g_sango2", "g_r3k2", NULL, "1900",
	"Sangokushi II (J) [!]\0", NULL, "Koei", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sango2RomInfo, g_sango2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Romance of the Three Kingdoms III (U) [!]
static struct BurnRomInfo g_r3k3RomDesc[] = {
	{ "g_r3k3.bin", 0x140000, 0x7e41c8fe, BRF_PRG },
};

STD_ROM_PICK(g_r3k3);
STD_ROM_FN(g_r3k3);

struct BurnDriverD BurnDrvMDg_r3k3 = {
	"g_r3k3", NULL, NULL, "1900",
	"Romance of the Three Kingdoms III (U) [!]\0", NULL, "Koei", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_r3k3RomInfo, g_r3k3RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sangokushi III (J) [!]
static struct BurnRomInfo g_sango3RomDesc[] = {
	{ "g_sango3.bin", 0x180000, 0xa8de6aea, BRF_PRG },
};

STD_ROM_PICK(g_sango3);
STD_ROM_FN(g_sango3);

struct BurnDriverD BurnDrvMDg_sango3 = {
	"g_sango3", "g_r3k3", NULL, "1900",
	"Sangokushi III (J) [!]\0", NULL, "Koei", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sango3RomInfo, g_sango3RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sagaia (U) [c][!]
static struct BurnRomInfo g_sagiaRomDesc[] = {
	{ "g_sagia.bin", 0x100000, 0xf1e22f43, BRF_PRG },
};

STD_ROM_PICK(g_sagia);
STD_ROM_FN(g_sagia);

struct BurnDriverD BurnDrvMDg_sagia = {
	"g_sagia", NULL, NULL, "1900",
	"Sagaia (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sagiaRomInfo, g_sagiaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Darius II (J) [c][!]
static struct BurnRomInfo g_dar2RomDesc[] = {
	{ "g_dar2.bin", 0x100000, 0x25dfe62a, BRF_PRG },
};

STD_ROM_PICK(g_dar2);
STD_ROM_FN(g_dar2);

struct BurnDriverD BurnDrvMDg_dar2 = {
	"g_dar2", "g_sagia", NULL, "1900",
	"Darius II (J) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_dar2RomInfo, g_dar2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Darius II (J) [a1]
static struct BurnRomInfo g_dari2aRomDesc[] = {
	{ "g_dari2a.bin", 0x100000, 0x0140ab58, BRF_PRG },
};

STD_ROM_PICK(g_dari2a);
STD_ROM_FN(g_dari2a);

struct BurnDriverD BurnDrvMDg_dari2a = {
	"g_dari2a", "g_sagia", NULL, "1900",
	"Darius II (J) [a1]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_dari2aRomInfo, g_dari2aRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Saint Sword (U) [c][!]
static struct BurnRomInfo g_sswoRomDesc[] = {
	{ "g_sswo.bin", 0x80000, 0x44f66bff, BRF_PRG },
};

STD_ROM_PICK(g_sswo);
STD_ROM_FN(g_sswo);

struct BurnDriverD BurnDrvMDg_sswo = {
	"g_sswo", NULL, NULL, "1900",
	"Saint Sword (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sswoRomInfo, g_sswoRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Saint Sword (J) [c][!]
static struct BurnRomInfo g_sswojRomDesc[] = {
	{ "g_sswoj.bin", 0x80000, 0x3960a00f, BRF_PRG },
};

STD_ROM_PICK(g_sswoj);
STD_ROM_FN(g_sswoj);

struct BurnDriverD BurnDrvMDg_sswoj = {
	"g_sswoj", "g_sswo", NULL, "1900",
	"Saint Sword (J) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sswojRomInfo, g_sswojRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Saturday Night Slam Masters (U) [!]
static struct BurnRomInfo g_snsmRomDesc[] = {
	{ "g_snsm.bin", 0x400000, 0x2fb4eaba, BRF_PRG },
};

STD_ROM_PICK(g_snsm);
STD_ROM_FN(g_snsm);

struct BurnDriverD BurnDrvMDg_snsm = {
	"g_snsm", NULL, NULL, "1900",
	"Saturday Night Slam Masters (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_snsmRomInfo, g_snsmRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Saturday Night Slam Masters (E)
static struct BurnRomInfo g_snsmeRomDesc[] = {
	{ "g_snsme.bin", 0x400000, 0x08fa5a3f, BRF_PRG },
};

STD_ROM_PICK(g_snsme);
STD_ROM_FN(g_snsme);

struct BurnDriverD BurnDrvMDg_snsme = {
	"g_snsme", "g_snsm", NULL, "1900",
	"Saturday Night Slam Masters (E)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_snsmeRomInfo, g_snsmeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Shaq Fu (UE) [!]
static struct BurnRomInfo g_shaqRomDesc[] = {
	{ "g_shaq.bin", 0x300000, 0x499955f2, BRF_PRG },
};

STD_ROM_PICK(g_shaq);
STD_ROM_FN(g_shaq);

struct BurnDriverD BurnDrvMDg_shaq = {
	"g_shaq", NULL, NULL, "1900",
	"Shaq Fu (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_shaqRomInfo, g_shaqRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Shadow Blasters (U) [c][!]
static struct BurnRomInfo g_sblsRomDesc[] = {
	{ "g_sbls.bin", 0x80000, 0x713d377b, BRF_PRG },
};

STD_ROM_PICK(g_sbls);
STD_ROM_FN(g_sbls);

struct BurnDriverD BurnDrvMDg_sbls = {
	"g_sbls", NULL, NULL, "1900",
	"Shadow Blasters (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sblsRomInfo, g_sblsRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Shiten Myooh (J) [c][!]
static struct BurnRomInfo g_shitenRomDesc[] = {
	{ "g_shiten.bin", 0x80000, 0x7e729693, BRF_PRG },
};

STD_ROM_PICK(g_shiten);
STD_ROM_FN(g_shiten);

struct BurnDriverD BurnDrvMDg_shiten = {
	"g_shiten", "g_sbls", NULL, "1900",
	"Shiten Myooh (J) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_shitenRomInfo, g_shitenRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Soldiers of Fortune (U) [c][!]
static struct BurnRomInfo g_soldfRomDesc[] = {
	{ "g_soldf.bin", 0x180000, 0xa84d28a1, BRF_PRG },
};

STD_ROM_PICK(g_soldf);
STD_ROM_FN(g_soldf);

struct BurnDriverD BurnDrvMDg_soldf = {
	"g_soldf", NULL, NULL, "1900",
	"Soldiers of Fortune (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_soldfRomInfo, g_soldfRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Chaos Engine, The (E) [!]
static struct BurnRomInfo g_chaoRomDesc[] = {
	{ "g_chao.bin", 0x180000, 0xbd9eecf4, BRF_PRG },
};

STD_ROM_PICK(g_chao);
STD_ROM_FN(g_chao);

struct BurnDriverD BurnDrvMDg_chao = {
	"g_chao", "g_soldf", NULL, "1900",
	"Chaos Engine, The (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_chaoRomInfo, g_chaoRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Chaos Engine, The (Beta)
static struct BurnRomInfo g_cengbRomDesc[] = {
	{ "g_cengb.bin", 0x100000, 0x3fb045c2, BRF_PRG },
};

STD_ROM_PICK(g_cengb);
STD_ROM_FN(g_cengb);

struct BurnDriverD BurnDrvMDg_cengb = {
	"g_cengb", "g_soldf", NULL, "1900",
	"Chaos Engine, The (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_cengbRomInfo, g_cengbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Splatterhouse 2 (U) [c][!]
static struct BurnRomInfo g_spl2RomDesc[] = {
	{ "g_spl2.bin", 0x100000, 0x2D1766E9, BRF_PRG },
};

STD_ROM_PICK(g_spl2);
STD_ROM_FN(g_spl2);

struct BurnDriver BurnDrvMDg_spl2 = {
	"g_spl2", NULL, NULL, "1900",
	"Splatterhouse 2 (U) [c][!]\0", NULL, "Namco", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_spl2RomInfo, g_spl2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Splatterhouse 2 (E) [c][!]
static struct BurnRomInfo g_shou2eRomDesc[] = {
	{ "g_shou2e.bin", 0x100000, 0x2559E03C, BRF_PRG },
};

STD_ROM_PICK(g_shou2e);
STD_ROM_FN(g_shou2e);

struct BurnDriver BurnDrvMDg_shou2e = {
	"g_shou2e", "g_spl2", NULL, "1900",
	"Splatterhouse 2 (E) [c][!]\0", NULL, "Namco", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_shou2eRomInfo, g_shou2eRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Splatterhouse 3 (U) [c][!]
static struct BurnRomInfo g_spl3RomDesc[] = {
	{ "g_spl3.bin", 0x200000, 0x00F05D07, BRF_PRG },
};

STD_ROM_PICK(g_spl3);
STD_ROM_FN(g_spl3);

struct BurnDriver BurnDrvMDg_spl3 = {
	"g_spl3", NULL, NULL, "1900",
	"Splatterhouse 3 (U) [c][!]\0", NULL, "Namco", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_spl3RomInfo, g_spl3RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Splatterhouse Part 3 (J) [c][!]
static struct BurnRomInfo g_shou3jRomDesc[] = {
	{ "g_shou3j.bin", 0x200000, 0x31B83D22, BRF_PRG },
};

STD_ROM_PICK(g_shou3j);
STD_ROM_FN(g_shou3j);

struct BurnDriver BurnDrvMDg_shou3j = {
	"g_shou3j", "g_spl3", NULL, "1900",
	"Splatterhouse Part 3 (J) [c][!]\0", NULL, "Namco", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_shou3jRomInfo, g_shou3jRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Splatterhouse Part 3 (J) [a1]
static struct BurnRomInfo g_sho3jaRomDesc[] = {
	{ "g_sho3ja.bin", 0x200000, 0xED68373A, BRF_PRG },
};

STD_ROM_PICK(g_sho3ja);
STD_ROM_FN(g_sho3ja);

struct BurnDriver BurnDrvMDg_sho3ja = {
	"g_sho3ja", "g_spl3", NULL, "1900",
	"Splatterhouse Part 3 (J) [a1]\0", NULL, "Namco", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sho3jaRomInfo, g_sho3jaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Street Fighter II' - Special Champion Edition (U) [!]
static struct BurnRomInfo g_sf2cRomDesc[] = {
	{ "g_sf2c.bin", 0x300000, 0x13fe08a1, BRF_PRG },
};

STD_ROM_PICK(g_sf2c);
STD_ROM_FN(g_sf2c);

struct BurnDriverD BurnDrvMDg_sf2c = {
	"g_sf2c", NULL, NULL, "1900",
	"Street Fighter II' - Special Champion Edition (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sf2cRomInfo, g_sf2cRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Street Fighter II' - Special Champion Edition (E) [!]
static struct BurnRomInfo g_sf2eRomDesc[] = {
	{ "g_sf2e.bin", 0x300000, 0x56d41136, BRF_PRG },
};

STD_ROM_PICK(g_sf2e);
STD_ROM_FN(g_sf2e);

struct BurnDriverD BurnDrvMDg_sf2e = {
	"g_sf2e", "g_sf2c", NULL, "1900",
	"Street Fighter II' - Special Champion Edition (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sf2eRomInfo, g_sf2eRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Street Fighter II' Plus - Champion Edition (J) [!]
static struct BurnRomInfo g_sf2jRomDesc[] = {
	{ "g_sf2j.bin", 0x300000, 0x2e487ee3, BRF_PRG },
};

STD_ROM_PICK(g_sf2j);
STD_ROM_FN(g_sf2j);

struct BurnDriverD BurnDrvMDg_sf2j = {
	"g_sf2j", "g_sf2c", NULL, "1900",
	"Street Fighter II' Plus - Champion Edition (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sf2jRomInfo, g_sf2jRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Street Fighter II' Turbo (Beta)
static struct BurnRomInfo g_sf2bRomDesc[] = {
	{ "g_sf2b.bin", 0x200000, 0xa85491ae, BRF_PRG },
};

STD_ROM_PICK(g_sf2b);
STD_ROM_FN(g_sf2b);

struct BurnDriverD BurnDrvMDg_sf2b = {
	"g_sf2b", "g_sf2c", NULL, "1900",
	"Street Fighter II' Turbo (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sf2bRomInfo, g_sf2bRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Strider (UE) [!]
static struct BurnRomInfo g_striRomDesc[] = {
	{ "g_stri.bin", 0x100000, 0xb9d099a4, BRF_PRG },
};

STD_ROM_PICK(g_stri);
STD_ROM_FN(g_stri);

struct BurnDriverD BurnDrvMDg_stri = {
	"g_stri", NULL, NULL, "1900",
	"Strider (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_striRomInfo, g_striRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Strider Hiryuu (J) [!]
static struct BurnRomInfo g_stridjRomDesc[] = {
	{ "g_stridj.bin", 0x100000, 0x859173f2, BRF_PRG },
};

STD_ROM_PICK(g_stridj);
STD_ROM_FN(g_stridj);

struct BurnDriverD BurnDrvMDg_stridj = {
	"g_stridj", "g_stri", NULL, "1900",
	"Strider Hiryuu (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_stridjRomInfo, g_stridjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Superman (U) [!]
static struct BurnRomInfo g_supmRomDesc[] = {
	{ "g_supm.bin", 0x100000, 0x543a5869, BRF_PRG },
};

STD_ROM_PICK(g_supm);
STD_ROM_FN(g_supm);

struct BurnDriverD BurnDrvMDg_supm = {
	"g_supm", NULL, NULL, "1900",
	"Superman (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_supmRomInfo, g_supmRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Superman (E) [!]
static struct BurnRomInfo g_supmneRomDesc[] = {
	{ "g_supmne.bin", 0x100000, 0x7db434ba, BRF_PRG },
};

STD_ROM_PICK(g_supmne);
STD_ROM_FN(g_supmne);

struct BurnDriverD BurnDrvMDg_supmne = {
	"g_supmne", "g_supm", NULL, "1900",
	"Superman (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_supmneRomInfo, g_supmneRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Superman (Beta)
static struct BurnRomInfo g_supmnbRomDesc[] = {
	{ "g_supmnb.bin", 0x100000, 0x5cd0e1d4, BRF_PRG },
};

STD_ROM_PICK(g_supmnb);
STD_ROM_FN(g_supmnb);

struct BurnDriverD BurnDrvMDg_supmnb = {
	"g_supmnb", "g_supm", NULL, "1900",
	"Superman (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_supmnbRomInfo, g_supmnbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Super Hydlide (U) [!]
static struct BurnRomInfo g_shydRomDesc[] = {
	{ "g_shyd.bin", 0x80000, 0x1335ddaa, BRF_PRG },
};

STD_ROM_PICK(g_shyd);
STD_ROM_FN(g_shyd);

struct BurnDriverD BurnDrvMDg_shyd = {
	"g_shyd", NULL, NULL, "1900",
	"Super Hydlide (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_shydRomInfo, g_shydRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Super Hydlide (E) [!]
static struct BurnRomInfo g_shydeRomDesc[] = {
	{ "g_shyde.bin", 0x80000, 0x1fe2d90b, BRF_PRG },
};

STD_ROM_PICK(g_shyde);
STD_ROM_FN(g_shyde);

struct BurnDriverD BurnDrvMDg_shyde = {
	"g_shyde", "g_shyd", NULL, "1900",
	"Super Hydlide (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_shydeRomInfo, g_shydeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Super Hydlide (J) [c][!]
static struct BurnRomInfo g_shydjRomDesc[] = {
	{ "g_shydj.bin", 0x80000, 0x599be386, BRF_PRG },
};

STD_ROM_PICK(g_shydj);
STD_ROM_FN(g_shydj);

struct BurnDriverD BurnDrvMDg_shydj = {
	"g_shydj", "g_shyd", NULL, "1900",
	"Super Hydlide (J) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_shydjRomInfo, g_shydjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Syd of Valis (U) [!]
static struct BurnRomInfo g_sydvalRomDesc[] = {
	{ "g_sydval.bin", 0x80000, 0x37dc0108, BRF_PRG },
};

STD_ROM_PICK(g_sydval);
STD_ROM_FN(g_sydval);

struct BurnDriverD BurnDrvMDg_sydval = {
	"g_sydval", NULL, NULL, "1900",
	"Syd of Valis (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sydvalRomInfo, g_sydvalRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Valis SD (J) [!]
static struct BurnRomInfo g_valsdjRomDesc[] = {
	{ "g_valsdj.bin", 0x80000, 0x1aef72ea, BRF_PRG },
};

STD_ROM_PICK(g_valsdj);
STD_ROM_FN(g_valsdj);

struct BurnDriverD BurnDrvMDg_valsdj = {
	"g_valsdj", "g_sydval", NULL, "1900",
	"Valis SD (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_valsdjRomInfo, g_valsdjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sylvester & Tweety in Cagey Capers (UE) [!]
static struct BurnRomInfo g_sylvRomDesc[] = {
	{ "g_sylv.bin", 0x200000, 0x89fc54ce, BRF_PRG },
};

STD_ROM_PICK(g_sylv);
STD_ROM_FN(g_sylv);

struct BurnDriverD BurnDrvMDg_sylv = {
	"g_sylv", NULL, NULL, "1900",
	"Sylvester & Tweety in Cagey Capers (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sylvRomInfo, g_sylvRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sylvester & Tweety in Cagey Capers (Beta)
static struct BurnRomInfo g_sylvbRomDesc[] = {
	{ "g_sylvb.bin", 0x200000, 0x9d9c786b, BRF_PRG },
};

STD_ROM_PICK(g_sylvb);
STD_ROM_FN(g_sylvb);

struct BurnDriverD BurnDrvMDg_sylvb = {
	"g_sylvb", "g_sylv", NULL, "1900",
	"Sylvester & Tweety in Cagey Capers (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sylvbRomInfo, g_sylvbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Target Earth (U) [!]
static struct BurnRomInfo g_targRomDesc[] = {
	{ "g_targ.bin", 0x80000, 0xcddf62d3, BRF_PRG },
};

STD_ROM_PICK(g_targ);
STD_ROM_FN(g_targ);

struct BurnDriverD BurnDrvMDg_targ = {
	"g_targ", NULL, NULL, "1900",
	"Target Earth (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_targRomInfo, g_targRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Assault Suits Leynos (J) [!]
static struct BurnRomInfo g_aslRomDesc[] = {
	{ "g_asl.bin", 0x80000, 0x81a2c800, BRF_PRG },
};

STD_ROM_PICK(g_asl);
STD_ROM_FN(g_asl);

struct BurnDriverD BurnDrvMDg_asl = {
	"g_asl", "g_targ", NULL, "1900",
	"Assault Suits Leynos (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_aslRomInfo, g_aslRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Teenage Mutant Ninja Turtles - The Hyperstone Heist (U) [!]
static struct BurnRomInfo g_turtRomDesc[] = {
	{ "g_turt.bin", 0x100000, 0x679c41de, BRF_PRG },
};

STD_ROM_PICK(g_turt);
STD_ROM_FN(g_turt);

struct BurnDriverD BurnDrvMDg_turt = {
	"g_turt", NULL, NULL, "1900",
	"Teenage Mutant Ninja Turtles - The Hyperstone Heist (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_turtRomInfo, g_turtRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Teenage Mutant Ninja Turtles - Return of the Shredder (J) [!]
static struct BurnRomInfo g_tmntjRomDesc[] = {
	{ "g_tmntj.bin", 0x100000, 0x1b003498, BRF_PRG },
};

STD_ROM_PICK(g_tmntj);
STD_ROM_FN(g_tmntj);

struct BurnDriverD BurnDrvMDg_tmntj = {
	"g_tmntj", "g_turt", NULL, "1900",
	"Teenage Mutant Ninja Turtles - Return of the Shredder (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tmntjRomInfo, g_tmntjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Teenage Mutant Hero Turtles - The Hyperstone Heist (E)
static struct BurnRomInfo g_tmhteRomDesc[] = {
	{ "g_tmhte.bin", 0x100000, 0x966d5286, BRF_PRG },
};

STD_ROM_PICK(g_tmhte);
STD_ROM_FN(g_tmhte);

struct BurnDriverD BurnDrvMDg_tmhte = {
	"g_tmhte", "g_turt", NULL, "1900",
	"Teenage Mutant Hero Turtles - The Hyperstone Heist (E)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tmhteRomInfo, g_tmhteRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Teenage Mutant Ninja Turtles - Tournament Fighters (U) [c][!]
static struct BurnRomInfo g_tutfRomDesc[] = {
	{ "g_tutf.bin", 0x200000, 0x95b5484d, BRF_PRG },
};

STD_ROM_PICK(g_tutf);
STD_ROM_FN(g_tutf);

struct BurnDriverD BurnDrvMDg_tutf = {
	"g_tutf", NULL, NULL, "1900",
	"Teenage Mutant Ninja Turtles - Tournament Fighters (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tutfRomInfo, g_tutfRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Teenage Mutant Ninja Turtles - Tournament Fighters (J) [!]
static struct BurnRomInfo g_tmnttjRomDesc[] = {
	{ "g_tmnttj.bin", 0x200000, 0x8843f2c9, BRF_PRG },
};

STD_ROM_PICK(g_tmnttj);
STD_ROM_FN(g_tmnttj);

struct BurnDriverD BurnDrvMDg_tmnttj = {
	"g_tmnttj", "g_tutf", NULL, "1900",
	"Teenage Mutant Ninja Turtles - Tournament Fighters (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tmnttjRomInfo, g_tmnttjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Teenage Mutant Hero Turtles - Tournament Fighters (E)
static struct BurnRomInfo g_tmhtteRomDesc[] = {
	{ "g_tmhtte.bin", 0x200000, 0x3cd2b7e6, BRF_PRG },
};

STD_ROM_PICK(g_tmhtte);
STD_ROM_FN(g_tmhtte);

struct BurnDriverD BurnDrvMDg_tmhtte = {
	"g_tmhtte", "g_tutf", NULL, "1900",
	"Teenage Mutant Hero Turtles - Tournament Fighters (E)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tmhtteRomInfo, g_tmhtteRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Tiny Toon Adventures - Buster's Hidden Treasure (U) [!]
static struct BurnRomInfo g_ttaRomDesc[] = {
	{ "g_tta.bin", 0x80000, 0xa26d3ae0, BRF_PRG },
};

STD_ROM_PICK(g_tta);
STD_ROM_FN(g_tta);

struct BurnDriverD BurnDrvMDg_tta = {
	"g_tta", NULL, NULL, "1900",
	"Tiny Toon Adventures - Buster's Hidden Treasure (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ttaRomInfo, g_ttaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Tiny Toon Adventures - Buster's Hidden Treasure (E) [!]
static struct BurnRomInfo g_ttabeRomDesc[] = {
	{ "g_ttabe.bin", 0x80000, 0xd10fba51, BRF_PRG },
};

STD_ROM_PICK(g_ttabe);
STD_ROM_FN(g_ttabe);

struct BurnDriverD BurnDrvMDg_ttabe = {
	"g_ttabe", "g_tta", NULL, "1900",
	"Tiny Toon Adventures - Buster's Hidden Treasure (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ttabeRomInfo, g_ttabeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Tiny Toon Adventures - Acme All-Stars (U) [!]
static struct BurnRomInfo g_ttaaRomDesc[] = {
	{ "g_ttaa.bin", 0x100000, 0x2f9faa1d, BRF_PRG },
};

STD_ROM_PICK(g_ttaa);
STD_ROM_FN(g_ttaa);

struct BurnDriverD BurnDrvMDg_ttaa = {
	"g_ttaa", NULL, NULL, "1900",
	"Tiny Toon Adventures - Acme All-Stars (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ttaaRomInfo, g_ttaaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Tiny Toon Adventures - Acme All-Stars (E)
static struct BurnRomInfo g_ttadaeRomDesc[] = {
	{ "g_ttadae.bin", 0x100000, 0x1227b2b2, BRF_PRG },
};

STD_ROM_PICK(g_ttadae);
STD_ROM_FN(g_ttadae);

struct BurnDriverD BurnDrvMDg_ttadae = {
	"g_ttadae", "g_ttaa", NULL, "1900",
	"Tiny Toon Adventures - Acme All-Stars (E)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ttadaeRomInfo, g_ttadaeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Toki - Going Ape Spit (UE) [c][!]
static struct BurnRomInfo g_tokiRomDesc[] = {
	{ "g_toki.bin", 0x80000, 0x7362c3f4, BRF_PRG },
};

STD_ROM_PICK(g_toki);
STD_ROM_FN(g_toki);

struct BurnDriverD BurnDrvMDg_toki = {
	"g_toki", NULL, NULL, "1900",
	"Toki - Going Ape Spit (UE) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tokiRomInfo, g_tokiRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// JuJu Densetsu (J) [!]
static struct BurnRomInfo g_jujuRomDesc[] = {
	{ "g_juju.bin", 0x80000, 0xd09b1ef1, BRF_PRG },
};

STD_ROM_PICK(g_juju);
STD_ROM_FN(g_juju);

struct BurnDriverD BurnDrvMDg_juju = {
	"g_juju", "g_toki", NULL, "1900",
	"JuJu Densetsu (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jujuRomInfo, g_jujuRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Toy Story (U) [!]
static struct BurnRomInfo g_toyRomDesc[] = {
	{ "g_toy.bin", 0x400000, 0x829fe313, BRF_PRG },
};

STD_ROM_PICK(g_toy);
STD_ROM_FN(g_toy);

struct BurnDriverD BurnDrvMDg_toy = {
	"g_toy", NULL, NULL, "1900",
	"Toy Story (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_toyRomInfo, g_toyRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Toy Story (E) [!]
static struct BurnRomInfo g_toysteRomDesc[] = {
	{ "g_toyste.bin", 0x400000, 0x8e89a9f3, BRF_PRG },
};

STD_ROM_PICK(g_toyste);
STD_ROM_FN(g_toyste);

struct BurnDriverD BurnDrvMDg_toyste = {
	"g_toyste", "g_toy", NULL, "1900",
	"Toy Story (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_toysteRomInfo, g_toysteRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Traysia (U) [!]
static struct BurnRomInfo g_traysiRomDesc[] = {
	{ "g_traysi.bin", 0x100000, 0x96184f4f, BRF_PRG },
};

STD_ROM_PICK(g_traysi);
STD_ROM_FN(g_traysi);

struct BurnDriverD BurnDrvMDg_traysi = {
	"g_traysi", NULL, NULL, "1900",
	"Traysia (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_traysiRomInfo, g_traysiRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Minato no Toreijia (J) [!]
static struct BurnRomInfo g_minatoRomDesc[] = {
	{ "g_minato.bin", 0x100000, 0xbd89fd09, BRF_PRG },
};

STD_ROM_PICK(g_minato);
STD_ROM_FN(g_minato);

struct BurnDriverD BurnDrvMDg_minato = {
	"g_minato", "g_traysi", NULL, "1900",
	"Minato no Toreijia (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_minatoRomInfo, g_minatoRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Two Crude Dudes (U) [!]
static struct BurnRomInfo g_2cdRomDesc[] = {
	{ "g_2cd.bin", 0x100000, 0x721b5744, BRF_PRG },
};

STD_ROM_PICK(g_2cd);
STD_ROM_FN(g_2cd);

struct BurnDriverD BurnDrvMDg_2cd = {
	"g_2cd", NULL, NULL, "1900",
	"Two Crude Dudes (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_2cdRomInfo, g_2cdRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Two Crude Dudes (E)
static struct BurnRomInfo g_crudeeRomDesc[] = {
	{ "g_crudee.bin", 0x100000, 0xb6d90a10, BRF_PRG },
};

STD_ROM_PICK(g_crudee);
STD_ROM_FN(g_crudee);

struct BurnDriverD BurnDrvMDg_crudee = {
	"g_crudee", "g_2cd", NULL, "1900",
	"Two Crude Dudes (E)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_crudeeRomInfo, g_crudeeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Crude Buster (J) [!]
static struct BurnRomInfo g_crudebRomDesc[] = {
	{ "g_crudeb.bin", 0x100000, 0xaffb4b00, BRF_PRG },
};

STD_ROM_PICK(g_crudeb);
STD_ROM_FN(g_crudeb);

struct BurnDriverD BurnDrvMDg_crudeb = {
	"g_crudeb", "g_2cd", NULL, "1900",
	"Crude Buster (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_crudebRomInfo, g_crudebRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Ultimate Qix (U) [!]
static struct BurnRomInfo g_uqixRomDesc[] = {
	{ "g_uqix.bin", 0x40000, 0xd83369d9, BRF_PRG },
};

STD_ROM_PICK(g_uqix);
STD_ROM_FN(g_uqix);

struct BurnDriverD BurnDrvMDg_uqix = {
	"g_uqix", NULL, NULL, "1900",
	"Ultimate Qix (U) [!]\0", NULL, "Taito", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_uqixRomInfo, g_uqixRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Volfied (J) [!]
static struct BurnRomInfo g_volfRomDesc[] = {
	{ "g_volf.bin", 0x40000, 0xb0c5e3f7, BRF_PRG },
};

STD_ROM_PICK(g_volf);
STD_ROM_FN(g_volf);

struct BurnDriverD BurnDrvMDg_volf = {
	"g_volf", "g_uqix", NULL, "1900",
	"Volfied (J) [!]\0", NULL, "Taito", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_volfRomInfo, g_volfRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Virtua Fighter 2 (UE) [!]
static struct BurnRomInfo g_vf2RomDesc[] = {
	{ "g_vf2.bin", 0x400000, 0x937380f3, BRF_PRG },
};

STD_ROM_PICK(g_vf2);
STD_ROM_FN(g_vf2);

struct BurnDriverD BurnDrvMDg_vf2 = {
	"g_vf2", NULL, NULL, "1900",
	"Virtua Fighter 2 (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_vf2RomInfo, g_vf2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// VR Troopers (UE) [!]
static struct BurnRomInfo g_vrtrRomDesc[] = {
	{ "g_vrtr.bin", 0x200000, 0x2f35516e, BRF_PRG },
};

STD_ROM_PICK(g_vrtr);
STD_ROM_FN(g_vrtr);

struct BurnDriverD BurnDrvMDg_vrtr = {
	"g_vrtr", NULL, NULL, "1900",
	"VR Troopers (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_vrtrRomInfo, g_vrtrRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Weaponlord (U) [!]
static struct BurnRomInfo g_weapRomDesc[] = {
	{ "g_weap.bin", 0x300000, 0xb9895365, BRF_PRG },
};

STD_ROM_PICK(g_weap);
STD_ROM_FN(g_weap);

struct BurnDriverD BurnDrvMDg_weap = {
	"g_weap", NULL, NULL, "1900",
	"Weaponlord (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_weapRomInfo, g_weapRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Where in the World is Carmen Sandiego (E) (M5)
static struct BurnRomInfo g_wwcseRomDesc[] = {
	{ "g_wwcse.bin", 0x100000, 0xeef372e8, BRF_PRG },
};

STD_ROM_PICK(g_wwcse);
STD_ROM_FN(g_wwcse);

struct BurnDriverD BurnDrvMDg_wwcse = {
	"g_wwcse", NULL, NULL, "1900",
	"Where in the World is Carmen Sandiego (E) (M5)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wwcseRomInfo, g_wwcseRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Where in the World is Carmen Sandiego (B) (M2) [!]
static struct BurnRomInfo g_wwicsRomDesc[] = {
	{ "g_wwics.bin", 0x100000, 0x7d4450ad, BRF_PRG },
};

STD_ROM_PICK(g_wwics);
STD_ROM_FN(g_wwics);

struct BurnDriverD BurnDrvMDg_wwics = {
	"g_wwics", "g_wwcse", NULL, "1900",
	"Where in the World is Carmen Sandiego (B) (M2) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wwicsRomInfo, g_wwicsRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Where in Time is Carmen Sandiego (B) [!]
static struct BurnRomInfo g_wticsRomDesc[] = {
	{ "g_wtics.bin", 0x100000, 0xd523b552, BRF_PRG },
};

STD_ROM_PICK(g_wtics);
STD_ROM_FN(g_wtics);

struct BurnDriverD BurnDrvMDg_wtics = {
	"g_wtics", NULL, NULL, "1900",
	"Where in Time is Carmen Sandiego (B) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wticsRomInfo, g_wticsRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Where in Time is Carmen Sandiego (UE) (M5) [!]
static struct BurnRomInfo g_wticsaRomDesc[] = {
	{ "g_wticsa.bin", 0x100000, 0xea19d4a4, BRF_PRG },
};

STD_ROM_PICK(g_wticsa);
STD_ROM_FN(g_wticsa);

struct BurnDriverD BurnDrvMDg_wticsa = {
	"g_wticsa", "g_wtics", NULL, "1900",
	"Where in Time is Carmen Sandiego (UE) (M5) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wticsaRomInfo, g_wticsaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Warsong (U) [!]
static struct BurnRomInfo g_warsRomDesc[] = {
	{ "g_wars.bin", 0x80000, 0x4b680285, BRF_PRG },
};

STD_ROM_PICK(g_wars);
STD_ROM_FN(g_wars);

struct BurnDriverD BurnDrvMDg_wars = {
	"g_wars", NULL, NULL, "1900",
	"Warsong (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_warsRomInfo, g_warsRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Langrisser (J) [!]
static struct BurnRomInfo g_langrRomDesc[] = {
	{ "g_langr.bin", 0x80000, 0xb6ea5016, BRF_PRG },
};

STD_ROM_PICK(g_langr);
STD_ROM_FN(g_langr);

struct BurnDriverD BurnDrvMDg_langr = {
	"g_langr", "g_wars", NULL, "1900",
	"Langrisser (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_langrRomInfo, g_langrRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Langrisser II (J) (REV00) [!]
static struct BurnRomInfo g_lngr2aRomDesc[] = {
	{ "g_lngr2a.bin", 0x200000, 0x7f891dfc, BRF_PRG },
};

STD_ROM_PICK(g_lngr2a);
STD_ROM_FN(g_lngr2a);

struct BurnDriverD BurnDrvMDg_lngr2a = {
	"g_lngr2a", NULL, NULL, "1900",
	"Langrisser II (J) (REV00) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_lngr2aRomInfo, g_lngr2aRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Langrisser II (J) (REV02) [!]
static struct BurnRomInfo g_lngr2cRomDesc[] = {
	{ "g_lngr2c.bin", 0x200000, 0x4967c9f9, BRF_PRG },
};

STD_ROM_PICK(g_lngr2c);
STD_ROM_FN(g_lngr2c);

struct BurnDriverD BurnDrvMDg_lngr2c = {
	"g_lngr2c", "g_lngr2a", NULL, "1900",
	"Langrisser II (J) (REV02) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_lngr2cRomInfo, g_lngr2cRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Langrisser II (J) (REV01)
static struct BurnRomInfo g_lng201RomDesc[] = {
	{ "g_lng201.bin", 0x200000, 0x0caa0593, BRF_PRG },
};

STD_ROM_PICK(g_lng201);
STD_ROM_FN(g_lng201);

struct BurnDriverD BurnDrvMDg_lng201 = {
	"g_lng201", "g_lngr2a", NULL, "1900",
	"Langrisser II (J) (REV01)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_lng201RomInfo, g_lng201RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Wimbledon Championship Tennis (U) [!]
static struct BurnRomInfo g_wimbRomDesc[] = {
	{ "g_wimb.bin", 0x100000, 0xf9142aee, BRF_PRG },
};

STD_ROM_PICK(g_wimb);
STD_ROM_FN(g_wimb);

struct BurnDriverD BurnDrvMDg_wimb = {
	"g_wimb", NULL, NULL, "1900",
	"Wimbledon Championship Tennis (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wimbRomInfo, g_wimbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Wimbledon Championship Tennis (U) (Prototype) [!]
static struct BurnRomInfo g_wimbpRomDesc[] = {
	{ "g_wimbp.bin", 0x100000, 0x9febc760, BRF_PRG },
};

STD_ROM_PICK(g_wimbp);
STD_ROM_FN(g_wimbp);

struct BurnDriverD BurnDrvMDg_wimbp = {
	"g_wimbp", "g_wimb", NULL, "1900",
	"Wimbledon Championship Tennis (U) (Prototype) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wimbpRomInfo, g_wimbpRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Wimbledon Championship Tennis (E) [!]
static struct BurnRomInfo g_wimbeRomDesc[] = {
	{ "g_wimbe.bin", 0x100000, 0xb791a435, BRF_PRG },
};

STD_ROM_PICK(g_wimbe);
STD_ROM_FN(g_wimbe);

struct BurnDriverD BurnDrvMDg_wimbe = {
	"g_wimbe", "g_wimb", NULL, "1900",
	"Wimbledon Championship Tennis (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wimbeRomInfo, g_wimbeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Wrestle War (JE) [c][!]
static struct BurnRomInfo g_wwarRomDesc[] = {
	{ "g_wwar.bin", 0x80000, 0x2d162a85, BRF_PRG },
};

STD_ROM_PICK(g_wwar);
STD_ROM_FN(g_wwar);

struct BurnDriverD BurnDrvMDg_wwar = {
	"g_wwar", NULL, NULL, "1900",
	"Wrestle War (JE) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wwarRomInfo, g_wwarRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Wrestle War (Beta)
static struct BurnRomInfo g_wwarbRomDesc[] = {
	{ "g_wwarb.bin", 0x80000, 0x1cdee87b, BRF_PRG },
};

STD_ROM_PICK(g_wwarb);
STD_ROM_FN(g_wwarb);

struct BurnDriverD BurnDrvMDg_wwarb = {
	"g_wwarb", "g_wwar", NULL, "1900",
	"Wrestle War (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wwarbRomInfo, g_wwarbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// WWF Wrestlemania Arcade (U) [!]
static struct BurnRomInfo g_wmarRomDesc[] = {
	{ "g_wmar.bin", 0x400000, 0xa5d023f9, BRF_PRG },
};

STD_ROM_PICK(g_wmar);
STD_ROM_FN(g_wmar);

struct BurnDriverD BurnDrvMDg_wmar = {
	"g_wmar", NULL, NULL, "1900",
	"WWF Wrestlemania Arcade (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wmarRomInfo, g_wmarRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// WWF Wrestlemania Arcade (Alpha) [!]
static struct BurnRomInfo g_wfwaalRomDesc[] = {
	{ "g_wfwaal.bin", 0x40000, 0x719d6155, BRF_PRG },
};

STD_ROM_PICK(g_wfwaal);
STD_ROM_FN(g_wfwaal);

struct BurnDriverD BurnDrvMDg_wfwaal = {
	"g_wfwaal", "g_wmar", NULL, "1900",
	"WWF Wrestlemania Arcade (Alpha) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wfwaalRomInfo, g_wfwaalRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// X-Men (U) [!]
static struct BurnRomInfo g_xmenRomDesc[] = {
	{ "g_xmen.bin", 0x100000, 0xf71b21b4, BRF_PRG },
};

STD_ROM_PICK(g_xmen);
STD_ROM_FN(g_xmen);

struct BurnDriverD BurnDrvMDg_xmen = {
	"g_xmen", NULL, NULL, "1900",
	"X-Men (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_xmenRomInfo, g_xmenRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// X-Men (E) [!]
static struct BurnRomInfo g_xmeneRomDesc[] = {
	{ "g_xmene.bin", 0x100000, 0x0b78ca97, BRF_PRG },
};

STD_ROM_PICK(g_xmene);
STD_ROM_FN(g_xmene);

struct BurnDriverD BurnDrvMDg_xmene = {
	"g_xmene", "g_xmen", NULL, "1900",
	"X-Men (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_xmeneRomInfo, g_xmeneRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// X-Men 2 - Clone Wars (UE) [!]
static struct BurnRomInfo g_xme2RomDesc[] = {
	{ "g_xme2.bin", 0x200000, 0x710bc628, BRF_PRG },
};

STD_ROM_PICK(g_xme2);
STD_ROM_FN(g_xme2);

struct BurnDriverD BurnDrvMDg_xme2 = {
	"g_xme2", NULL, NULL, "1900",
	"X-Men 2 - Clone Wars (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_xme2RomInfo, g_xme2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Yuu Yuu Hakusho - Makyou Toitsusen (J) [!]
static struct BurnRomInfo g_yuyuRomDesc[] = {
	{ "g_yuyu.bin", 0x300000, 0x71ceac6f, BRF_PRG },
};

STD_ROM_PICK(g_yuyu);
STD_ROM_FN(g_yuyu);

struct BurnDriverD BurnDrvMDg_yuyu = {
	"g_yuyu", NULL, NULL, "1900",
	"Yuu Yuu Hakusho - Makyou Toitsusen (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_yuyuRomInfo, g_yuyuRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Yuu Yuu Hakusho - Sunset Fighters (B) [!]
static struct BurnRomInfo g_yuyubRomDesc[] = {
	{ "g_yuyub.bin", 0x300000, 0xfe3fb8ee, BRF_PRG },
};

STD_ROM_PICK(g_yuyub);
STD_ROM_FN(g_yuyub);

struct BurnDriverD BurnDrvMDg_yuyub = {
	"g_yuyub", "g_yuyu", NULL, "1900",
	"Yuu Yuu Hakusho - Sunset Fighters (B) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_yuyubRomInfo, g_yuyubRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Ys III - Wanderers from Ys (U) [!]
static struct BurnRomInfo g_ys3RomDesc[] = {
	{ "g_ys3.bin", 0x100000, 0xea27976e, BRF_PRG },
};

STD_ROM_PICK(g_ys3);
STD_ROM_FN(g_ys3);

struct BurnDriverD BurnDrvMDg_ys3 = {
	"g_ys3", NULL, NULL, "1900",
	"Ys III - Wanderers from Ys (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ys3RomInfo, g_ys3RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Ys III - Wanderers from Ys (J) [!]
static struct BurnRomInfo g_ys3jRomDesc[] = {
	{ "g_ys3j.bin", 0x100000, 0x52da4e76, BRF_PRG },
};

STD_ROM_PICK(g_ys3j);
STD_ROM_FN(g_ys3j);

struct BurnDriverD BurnDrvMDg_ys3j = {
	"g_ys3j", "g_ys3", NULL, "1900",
	"Ys III - Wanderers from Ys (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ys3jRomInfo, g_ys3jRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Zany Golf (UE) (REV00) [c][!]
static struct BurnRomInfo g_zanyRomDesc[] = {
	{ "g_zany.bin", 0x80000, 0xed5d12ea, BRF_PRG },
};

STD_ROM_PICK(g_zany);
STD_ROM_FN(g_zany);

struct BurnDriverD BurnDrvMDg_zany = {
	"g_zany", NULL, NULL, "1900",
	"Zany Golf (UE) (REV00) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_zanyRomInfo, g_zanyRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Zany Golf (UE) (REV01))
static struct BurnRomInfo g_zany01RomDesc[] = {
	{ "g_zany01.bin", 0x80000, 0x74ed7607, BRF_PRG },
};

STD_ROM_PICK(g_zany01);
STD_ROM_FN(g_zany01);

struct BurnDriverD BurnDrvMDg_zany01 = {
	"g_zany01", "g_zany", NULL, "1900",
	"Zany Golf (UE) (REV01))\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_zany01RomInfo, g_zany01RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Zero Wing (E) [c][!]
static struct BurnRomInfo g_zwinRomDesc[] = {
	{ "g_zwin.bin", 0x100000, 0x89b744a3, BRF_PRG },
};

STD_ROM_PICK(g_zwin);
STD_ROM_FN(g_zwin);

struct BurnDriverD BurnDrvMDg_zwin = {
	"g_zwin", NULL, NULL, "1900",
	"Zero Wing (E) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_zwinRomInfo, g_zwinRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Zero Wing (J)
static struct BurnRomInfo g_zwingjRomDesc[] = {
	{ "g_zwingj.bin", 0x100000, 0x7e203d2b, BRF_PRG },
};

STD_ROM_PICK(g_zwingj);
STD_ROM_FN(g_zwingj);

struct BurnDriverD BurnDrvMDg_zwingj = {
	"g_zwingj", "g_zwin", NULL, "1900",
	"Zero Wing (J)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_zwingjRomInfo, g_zwingjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Zombies Ate My Neighbors (U) [c][!]
static struct BurnRomInfo g_zombRomDesc[] = {
	{ "g_zomb.bin", 0x100000, 0x2bf3626f, BRF_PRG },
};

STD_ROM_PICK(g_zomb);
STD_ROM_FN(g_zomb);

struct BurnDriverD BurnDrvMDg_zomb = {
	"g_zomb", NULL, NULL, "1900",
	"Zombies Ate My Neighbors (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_zombRomInfo, g_zombRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Zombies (E) [!]
static struct BurnRomInfo g_zombeRomDesc[] = {
	{ "g_zombe.bin", 0x100000, 0x179a1aa2, BRF_PRG },
};

STD_ROM_PICK(g_zombe);
STD_ROM_FN(g_zombe);

struct BurnDriverD BurnDrvMDg_zombe = {
	"g_zombe", "g_zomb", NULL, "1900",
	"Zombies (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_zombeRomInfo, g_zombeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Zool (U) [!]
static struct BurnRomInfo g_zoolRomDesc[] = {
	{ "g_zool.bin", 0x100000, 0xcb2939f1, BRF_PRG },
};

STD_ROM_PICK(g_zool);
STD_ROM_FN(g_zool);

struct BurnDriverD BurnDrvMDg_zool = {
	"g_zool", NULL, NULL, "1900",
	"Zool (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_zoolRomInfo, g_zoolRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Zool (E) [!]
static struct BurnRomInfo g_zooleRomDesc[] = {
	{ "g_zoole.bin", 0x100000, 0x1ee58b03, BRF_PRG },
};

STD_ROM_PICK(g_zoole);
STD_ROM_FN(g_zoole);

struct BurnDriverD BurnDrvMDg_zoole = {
	"g_zoole", "g_zool", NULL, "1900",
	"Zool (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_zooleRomInfo, g_zooleRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Uncharted Waters (U) [!]
static struct BurnRomInfo g_uwRomDesc[] = {
	{ "g_uw.bin", 0x100000, 0x4edaec59, BRF_PRG },
};

STD_ROM_PICK(g_uw);
STD_ROM_FN(g_uw);

struct BurnDriverD BurnDrvMDg_uw = {
	"g_uw", NULL, NULL, "199?",
	"Uncharted Waters (U) [!]\0", NULL, "Koei", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_uwRomInfo, g_uwRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Daikoukai Jidai (J) [c][!]
static struct BurnRomInfo g_daikouRomDesc[] = {
	{ "g_daikou.bin", 0x100000, 0x5a652458, BRF_PRG },
};

STD_ROM_PICK(g_daikou);
STD_ROM_FN(g_daikou);

struct BurnDriverD BurnDrvMDg_daikou = {
	"g_daikou", "g_uw", NULL, "1900",
	"Daikoukai Jidai (J) [c][!]\0", NULL, "Koei", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_daikouRomInfo, g_daikouRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Uncharted Waters - New Horizons (U) [!]
static struct BurnRomInfo g_uwnhRomDesc[] = {
	{ "g_uwnh.bin", 0x200000, 0xead69824, BRF_PRG },
};

STD_ROM_PICK(g_uwnh);
STD_ROM_FN(g_uwnh);

struct BurnDriverD BurnDrvMDg_uwnh = {
	"g_uwnh", NULL, NULL, "1900",
	"Uncharted Waters - New Horizons (U) [!]\0", NULL, "Koei", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_uwnhRomInfo, g_uwnhRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Daikoukai Jidai II (J)
static struct BurnRomInfo g_daik2RomDesc[] = {
	{ "g_daik2.bin", 0x200000, 0xe040f0da, BRF_PRG },
};

STD_ROM_PICK(g_daik2);
STD_ROM_FN(g_daik2);

struct BurnDriverD BurnDrvMDg_daik2 = {
	"g_daik2", "g_uwnh", NULL, "1900",
	"Daikoukai Jidai II (J)\0", NULL, "Koei", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_daik2RomInfo, g_daik2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// GemFire (U) [!]
static struct BurnRomInfo g_gemfiRomDesc[] = {
	{ "g_gemfi.bin", 0x100000, 0x3d36135b, BRF_PRG },
};

STD_ROM_PICK(g_gemfi);
STD_ROM_FN(g_gemfi);

struct BurnDriverD BurnDrvMDg_gemfi = {
	"g_gemfi", NULL, NULL, "1900",
	"GemFire (U) [!]\0", NULL, "Koei", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_gemfiRomInfo, g_gemfiRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Royal Blood (J) [!]
static struct BurnRomInfo g_roybldRomDesc[] = {
	{ "g_roybld.bin", 0x100000, 0x0e0107f1, BRF_PRG },
};

STD_ROM_PICK(g_roybld);
STD_ROM_FN(g_roybld);

struct BurnDriverD BurnDrvMDg_roybld = {
	"g_roybld", "g_gemfi", NULL, "1900",
	"Royal Blood (J) [!]\0", NULL, "Koei", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_roybldRomInfo, g_roybldRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Nobunaga's Ambition (U) [!]
static struct BurnRomInfo g_nobuRomDesc[] = {
	{ "g_nobu.bin", 0x80000, 0xb9bc07bc, BRF_PRG },
};

STD_ROM_PICK(g_nobu);
STD_ROM_FN(g_nobu);

struct BurnDriverD BurnDrvMDg_nobu = {
	"g_nobu", NULL, NULL, "1900",
	"Nobunaga's Ambition (U) [!]\0", NULL, "Koei", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_nobuRomInfo, g_nobuRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Nobunaga no Yabou - Zenkokuban (J) [!]
static struct BurnRomInfo g_nobzenRomDesc[] = {
	{ "g_nobzen.bin", 0x80000, 0x1381b313, BRF_PRG },
};

STD_ROM_PICK(g_nobzen);
STD_ROM_FN(g_nobzen);

struct BurnDriverD BurnDrvMDg_nobzen = {
	"g_nobzen", "g_nobu", NULL, "1900",
	"Nobunaga no Yabou - Zenkokuban (J) [!]\0", NULL, "Koei", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_nobzenRomInfo, g_nobzenRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// P.T.O. Pacific Theater of Operations (U) [!]
static struct BurnRomInfo g_ptoRomDesc[] = {
	{ "g_pto.bin", 0x100000, 0xd9d4c6e2, BRF_PRG },
};

STD_ROM_PICK(g_pto);
STD_ROM_FN(g_pto);

struct BurnDriverD BurnDrvMDg_pto = {
	"g_pto", NULL, NULL, "1900",
	"P.T.O. Pacific Theater of Operations (U) [!]\0", NULL, "Koei", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ptoRomInfo, g_ptoRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Teitoku no Ketsudan (J) [c][!]
static struct BurnRomInfo g_teitokRomDesc[] = {
	{ "g_teitok.bin", 0x100000, 0x9b08e4e4, BRF_PRG },
};

STD_ROM_PICK(g_teitok);
STD_ROM_FN(g_teitok);

struct BurnDriverD BurnDrvMDg_teitok = {
	"g_teitok", "g_pto", NULL, "1900",
	"Teitoku no Ketsudan (J) [c][!]\0", NULL, "Koei", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_teitokRomInfo, g_teitokRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Operation Europe - Path to Victory 1939-1945 (U) [!]
static struct BurnRomInfo g_opeuRomDesc[] = {
	{ "g_opeu.bin", 0x100000, 0xe7cba1d8, BRF_PRG },
};

STD_ROM_PICK(g_opeu);
STD_ROM_FN(g_opeu);

struct BurnDriverD BurnDrvMDg_opeu = {
	"g_opeu", NULL, NULL, "1900",
	"Operation Europe - Path to Victory 1939-1945 (U) [!]\0", NULL, "Koei", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_opeuRomInfo, g_opeuRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Europa Sensen (J) [!]
static struct BurnRomInfo g_europaRomDesc[] = {
	{ "g_europa.bin", 0x100000, 0xb0416c60, BRF_PRG },
};

STD_ROM_PICK(g_europa);
STD_ROM_FN(g_europa);

struct BurnDriverD BurnDrvMDg_europa = {
	"g_europa", "g_opeu", NULL, "1900",
	"Europa Sensen (J) [!]\0", NULL, "Koei", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_europaRomInfo, g_europaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Liberty or Death (U) [!]
static struct BurnRomInfo g_librtyRomDesc[] = {
	{ "g_librty.bin", 0x200000, 0x2adb0364, BRF_PRG },
};

STD_ROM_PICK(g_librty);
STD_ROM_FN(g_librty);

struct BurnDriverD BurnDrvMDg_librty = {
	"g_librty", NULL, NULL, "1993",
	"Liberty or Death (U) [!]\0", NULL, "Koei", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_librtyRomInfo, g_librtyRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Genghis Khan II - Clan of the Gray Wolf (U) [!]
static struct BurnRomInfo g_gengRomDesc[] = {
	{ "g_geng.bin", 0x100000, 0x87a281ae, BRF_PRG },
};

STD_ROM_PICK(g_geng);
STD_ROM_FN(g_geng);

struct BurnDriverD BurnDrvMDg_geng = {
	"g_geng", NULL, NULL, "1900",
	"Genghis Khan II - Clan of the Gray Wolf (U) [!]\0", NULL, "Koei", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_gengRomInfo, g_gengRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Nobunaga no Yabou - Bushou Fuuunsoku (J) [!]
static struct BurnRomInfo g_nobbusRomDesc[] = {
	{ "g_nobbus.bin", 0x100000, 0x30bf8637, BRF_PRG },
};

STD_ROM_PICK(g_nobbus);
STD_ROM_FN(g_nobbus);

struct BurnDriverD BurnDrvMDg_nobbus = {
	"g_nobbus", NULL, NULL, "1900",
	"Nobunaga no Yabou - Bushou Fuuunsoku (J) [!]\0", NULL, "Koei", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_nobbusRomInfo, g_nobbusRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Nobunaga no Yabou Haouden - Lord of Darkness (J) [!]
static struct BurnRomInfo g_noblorRomDesc[] = {
	{ "g_noblor.bin", 0x200000, 0x96c01fc6, BRF_PRG },
};

STD_ROM_PICK(g_noblor);
STD_ROM_FN(g_noblor);

struct BurnDriverD BurnDrvMDg_noblor = {
	"g_noblor", NULL, NULL, "1900",
	"Nobunaga no Yabou Haouden - Lord of Darkness (J) [!]\0", NULL, "Koei", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_noblorRomInfo, g_noblorRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Taikou Risshiden (J) [!]
static struct BurnRomInfo g_taikouRomDesc[] = {
	{ "g_taikou.bin", 0x140000, 0xf96fe15b, BRF_PRG },
};

STD_ROM_PICK(g_taikou);
STD_ROM_FN(g_taikou);

struct BurnDriverD BurnDrvMDg_taikou = {
	"g_taikou", NULL, NULL, "1900",
	"Taikou Risshiden (J) [!]\0", NULL, "Koei", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_taikouRomInfo, g_taikouRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Aoki Ookami to Shiroki Meshika - Genchou Hishi (J) [!]
static struct BurnRomInfo g_aokiRomDesc[] = {
	{ "g_aoki.bin", 0x140000, 0x10be1d93, BRF_PRG },
};

STD_ROM_PICK(g_aoki);
STD_ROM_FN(g_aoki);

struct BurnDriverD BurnDrvMDg_aoki = {
	"g_aoki", NULL, NULL, "1900",
	"Aoki Ookami to Shiroki Meshika - Genchou Hishi (J) [!]\0", NULL, "Koei", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_aokiRomInfo, g_aokiRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Gley Lancer (J)
static struct BurnRomInfo g_gleyRomDesc[] = {
	{ "g_gley.bin", 0x100000, 0x42cf9b5b, BRF_PRG },
};

STD_ROM_PICK(g_gley);
STD_ROM_FN(g_gley);

struct BurnDriverD BurnDrvMDg_gley = {
	"g_gley", NULL, NULL, "1900",
	"Gley Lancer (J)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_gleyRomInfo, g_gleyRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Snake Rattle 'n' Roll (E) [c][!]
static struct BurnRomInfo g_srrRomDesc[] = {
	{ "g_srr.bin", 0x80000, 0x543bed30, BRF_PRG },
};

STD_ROM_PICK(g_srr);
STD_ROM_FN(g_srr);

struct BurnDriverD BurnDrvMDg_srr = {
	"g_srr", NULL, NULL, "1900",
	"Snake Rattle 'n' Roll (E) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_srrRomInfo, g_srrRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Taz-Mania (W) [!]
static struct BurnRomInfo g_tazmRomDesc[] = {
	{ "g_tazm.bin", 0x80000, 0x0e901f45, BRF_PRG },
};

STD_ROM_PICK(g_tazm);
STD_ROM_FN(g_tazm);

struct BurnDriver BurnDrvMDg_tazm = {
	"g_tazm", NULL, NULL, "1900",
	"Taz-Mania (W) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tazmRomInfo, g_tazmRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Cosmic Spacehead (E) (M4) [c][!]
static struct BurnRomInfo g_cossRomDesc[] = {
	{ "g_coss.bin", 0x100000, 0xc593d31c, BRF_PRG },
};

STD_ROM_PICK(g_coss);
STD_ROM_FN(g_coss);

struct BurnDriverD BurnDrvMDg_coss = {
	"g_coss", NULL, NULL, "1900",
	"Cosmic Spacehead (E) (M4) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_cossRomInfo, g_cossRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Pac-Attack (U) [!]
static struct BurnRomInfo g_pacaRomDesc[] = {
	{ "g_paca.bin", 0x40000, 0x5df382f7, BRF_PRG },
};

STD_ROM_PICK(g_paca);
STD_ROM_FN(g_paca);

struct BurnDriverD BurnDrvMDg_paca = {
	"g_paca", NULL, NULL, "1900",
	"Pac-Attack (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pacaRomInfo, g_pacaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Jungle Book, The (U) [!]
static struct BurnRomInfo g_jbokRomDesc[] = {
	{ "g_jbok.bin", 0x200000, 0x3fb6d92e, BRF_PRG },
};

STD_ROM_PICK(g_jbok);
STD_ROM_FN(g_jbok);

struct BurnDriverD BurnDrvMDg_jbok = {
	"g_jbok", NULL, NULL, "1900",
	"Jungle Book, The (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jbokRomInfo, g_jbokRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Jungle Book, The (E) [!]
static struct BurnRomInfo g_jbookeRomDesc[] = {
	{ "g_jbooke.bin", 0x200000, 0xb9709a99, BRF_PRG },
};

STD_ROM_PICK(g_jbooke);
STD_ROM_FN(g_jbooke);

struct BurnDriverD BurnDrvMDg_jbooke = {
	"g_jbooke", "g_jbok", NULL, "1900",
	"Jungle Book, The (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jbookeRomInfo, g_jbookeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Fantasia (JU) (REV01) [!]
static struct BurnRomInfo g_fantRomDesc[] = {
	{ "g_fant.bin", 0x80000, 0xfc43df2d, BRF_PRG },
};

STD_ROM_PICK(g_fant);
STD_ROM_FN(g_fant);

struct BurnDriverD BurnDrvMDg_fant = {
	"g_fant", NULL, NULL, "1900",
	"Fantasia (JU) (REV01) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_fantRomInfo, g_fantRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Fantasia (E) [!]
static struct BurnRomInfo g_fanteRomDesc[] = {
	{ "g_fante.bin", 0x80000, 0xd351b242, BRF_PRG },
};

STD_ROM_PICK(g_fante);
STD_ROM_FN(g_fante);

struct BurnDriverD BurnDrvMDg_fante = {
	"g_fante", "g_fant", NULL, "1900",
	"Fantasia (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_fanteRomInfo, g_fanteRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Fantasia (U) (REV00) [!]
static struct BurnRomInfo g_fant00RomDesc[] = {
	{ "g_fant00.bin", 0x80000, 0x34e04627, BRF_PRG },
};

STD_ROM_PICK(g_fant00);
STD_ROM_FN(g_fant00);

struct BurnDriverD BurnDrvMDg_fant00 = {
	"g_fant00", "g_fant", NULL, "1900",
	"Fantasia (U) (REV00) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_fant00RomInfo, g_fant00RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Decap Attack (UE) [!]
static struct BurnRomInfo g_dcapRomDesc[] = {
	{ "g_dcap.bin", 0x80000, 0x73dc0dd8, BRF_PRG },
};

STD_ROM_PICK(g_dcap);
STD_ROM_FN(g_dcap);

struct BurnDriverD BurnDrvMDg_dcap = {
	"g_dcap", NULL, NULL, "1900",
	"Decap Attack (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_dcapRomInfo, g_dcapRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Jurassic Park 2 - The Lost World (U) [!]
static struct BurnRomInfo g_jpa2RomDesc[] = {
	{ "g_jpa2.bin", 0x400000, 0x140a284c, BRF_PRG },
};

STD_ROM_PICK(g_jpa2);
STD_ROM_FN(g_jpa2);

struct BurnDriverD BurnDrvMDg_jpa2 = {
	"g_jpa2", NULL, NULL, "1998",
	"Jurassic Park 2 - The Lost World (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jpa2RomInfo, g_jpa2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Jim Power - The Arcade Game (Beta)
static struct BurnRomInfo g_jimpRomDesc[] = {
	{ "g_jimp.bin", 0x100000, 0x1cf3238b, BRF_PRG },
};

STD_ROM_PICK(g_jimp);
STD_ROM_FN(g_jimp);

struct BurnDriverD BurnDrvMDg_jimp = {
	"g_jimp", NULL, NULL, "1900",
	"Jim Power - The Arcade Game (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jimpRomInfo, g_jimpRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Charles' Window Bug Example
static struct BurnRomInfo g_wbugRomDesc[] = {
	{ "g_wbug.bin", 0x4000, 0x75582ddb, BRF_PRG },
};

STD_ROM_PICK(g_wbug);
STD_ROM_FN(g_wbug);

struct BurnDriverD BurnDrvMDg_wbug = {
	"g_wbug", NULL, NULL, "1900",
	"Charles' Window Bug Example\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wbugRomInfo, g_wbugRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Raiden Trad (JU) [!]
static struct BurnRomInfo g_raidRomDesc[] = {
	{ "g_raid.bin", 0x100000, 0xf839a811, BRF_PRG },
};

STD_ROM_PICK(g_raid);
STD_ROM_FN(g_raid);

struct BurnDriverD BurnDrvMDg_raid = {
	"g_raid", NULL, NULL, "1900",
	"Raiden Trad (JU) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_raidRomInfo, g_raidRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// FIFA Soccer 98 - Road to the World Cup (E) (M5) [!]
static struct BurnRomInfo g_f98RomDesc[] = {
	{ "g_f98.bin", 0x200000, 0x96947f57, BRF_PRG },
};

STD_ROM_PICK(g_f98);
STD_ROM_FN(g_f98);

struct BurnDriverD BurnDrvMDg_f98 = {
	"g_f98", NULL, NULL, "1900",
	"FIFA Soccer 98 - Road to the World Cup (E) (M5) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_f98RomInfo, g_f98RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Donald in Maui Mallard (E) [!]
static struct BurnRomInfo g_mauiRomDesc[] = {
	{ "g_maui.bin", 0x300000, 0xb2dd857f, BRF_PRG },
};

STD_ROM_PICK(g_maui);
STD_ROM_FN(g_maui);

struct BurnDriverD BurnDrvMDg_maui = {
	"g_maui", NULL, NULL, "1900",
	"Donald in Maui Mallard (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mauiRomInfo, g_mauiRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Ayrton Senna's Super Monaco GP II (U) [!]
static struct BurnRomInfo g_sgp2RomDesc[] = {
	{ "g_sgp2.bin", 0x100000, 0xeac8ded6, BRF_PRG },
};

STD_ROM_PICK(g_sgp2);
STD_ROM_FN(g_sgp2);

struct BurnDriverD BurnDrvMDg_sgp2 = {
	"g_sgp2", NULL, NULL, "1900",
	"Ayrton Senna's Super Monaco GP II (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sgp2RomInfo, g_sgp2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Ayrton Senna's Super Monaco GP II (JE) [!]
static struct BurnRomInfo g_smgp2aRomDesc[] = {
	{ "g_smgp2a.bin", 0x100000, 0x60af0f76, BRF_PRG },
};

STD_ROM_PICK(g_smgp2a);
STD_ROM_FN(g_smgp2a);

struct BurnDriverD BurnDrvMDg_smgp2a = {
	"g_smgp2a", "g_sgp2", NULL, "1900",
	"Ayrton Senna's Super Monaco GP II (JE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_smgp2aRomInfo, g_smgp2aRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Batman Forever (W) [!]
static struct BurnRomInfo g_batfRomDesc[] = {
	{ "g_batf.bin", 0x300000, 0x8b723d01, BRF_PRG },
};

STD_ROM_PICK(g_batf);
STD_ROM_FN(g_batf);

struct BurnDriverD BurnDrvMDg_batf = {
	"g_batf", NULL, NULL, "1900",
	"Batman Forever (W) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_batfRomInfo, g_batfRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Gadget Twins, The (U) [!]
static struct BurnRomInfo g_gtwiRomDesc[] = {
	{ "g_gtwi.bin", 0x100000, 0x7ae5e248, BRF_PRG },
};

STD_ROM_PICK(g_gtwi);
STD_ROM_FN(g_gtwi);

struct BurnDriverD BurnDrvMDg_gtwi = {
	"g_gtwi", NULL, NULL, "1900",
	"Gadget Twins, The (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_gtwiRomInfo, g_gtwiRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Legend of Galahad, The (UE) [!]
static struct BurnRomInfo g_lgalRomDesc[] = {
	{ "g_lgal.bin", 0x100000, 0x679557bc, BRF_PRG },
};

STD_ROM_PICK(g_lgal);
STD_ROM_FN(g_lgal);

struct BurnDriverD BurnDrvMDg_lgal = {
	"g_lgal", NULL, NULL, "1900",
	"Legend of Galahad, The (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_lgalRomInfo, g_lgalRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Lion King, The (W) [!]
static struct BurnRomInfo g_lionRomDesc[] = {
	{ "g_lion.bin", 0x300000, 0x5696a5bc, BRF_PRG },
};

STD_ROM_PICK(g_lion);
STD_ROM_FN(g_lion);

struct BurnDriverD BurnDrvMDg_lion = {
	"g_lion", NULL, NULL, "1900",
	"Lion King, The (W) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_lionRomInfo, g_lionRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Panorama Cotton (J) [c][!]
static struct BurnRomInfo g_panoRomDesc[] = {
	{ "g_pano.bin", 0x280000, 0x9e57d92e, BRF_PRG },
};

STD_ROM_PICK(g_pano);
STD_ROM_FN(g_pano);

struct BurnDriverD BurnDrvMDg_pano = {
	"g_pano", NULL, NULL, "1900",
	"Panorama Cotton (J) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_panoRomInfo, g_panoRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Pulseman (J) [c][!]
static struct BurnRomInfo g_pulsRomDesc[] = {
	{ "g_puls.bin", 0x200000, 0x138a104e, BRF_PRG },
};

STD_ROM_PICK(g_puls);
STD_ROM_FN(g_puls);

struct BurnDriverD BurnDrvMDg_puls = {
	"g_puls", NULL, NULL, "1900",
	"Pulseman (J) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pulsRomInfo, g_pulsRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Red Zone (UE) [!]
static struct BurnRomInfo g_redzRomDesc[] = {
	{ "g_redz.bin", 0x200000, 0x56512ee5, BRF_PRG },
};

STD_ROM_PICK(g_redz);
STD_ROM_FN(g_redz);

struct BurnDriverD BurnDrvMDg_redz = {
	"g_redz", NULL, NULL, "1900",
	"Red Zone (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_redzRomInfo, g_redzRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Skitchin' (UE) [!]
static struct BurnRomInfo g_skitRomDesc[] = {
	{ "g_skit.bin", 0x200000, 0xf785f9d7, BRF_PRG },
};

STD_ROM_PICK(g_skit);
STD_ROM_FN(g_skit);

struct BurnDriverD BurnDrvMDg_skit = {
	"g_skit", NULL, NULL, "1900",
	"Skitchin' (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_skitRomInfo, g_skitRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Super Fantasy Zone (E) [!]
static struct BurnRomInfo g_sfzeRomDesc[] = {
	{ "g_sfze.bin", 0x100000, 0x927975be, BRF_PRG },
};

STD_ROM_PICK(g_sfze);
STD_ROM_FN(g_sfze);

struct BurnDriverD BurnDrvMDg_sfze = {
	"g_sfze", NULL, NULL, "1900",
	"Super Fantasy Zone (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sfzeRomInfo, g_sfzeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Super Fantasy Zone (J) [!]
static struct BurnRomInfo g_sfzRomDesc[] = {
	{ "g_sfz.bin", 0x100000, 0x767780d7, BRF_PRG },
};

STD_ROM_PICK(g_sfz);
STD_ROM_FN(g_sfz);

struct BurnDriverD BurnDrvMDg_sfz = {
	"g_sfz", "g_sfze", NULL, "1900",
	"Super Fantasy Zone (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sfzRomInfo, g_sfzRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Super Hang-On (W) (REV01) [!]
static struct BurnRomInfo g_shoRomDesc[] = {
	{ "g_sho.bin", 0x80000, 0x3877d107, BRF_PRG },
};

STD_ROM_PICK(g_sho);
STD_ROM_FN(g_sho);

struct BurnDriverD BurnDrvMDg_sho = {
	"g_sho", NULL, NULL, "1900",
	"Super Hang-On (W) (REV01) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_shoRomInfo, g_shoRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Super Hang-On (W) (REV00) [!]
static struct BurnRomInfo g_sho00RomDesc[] = {
	{ "g_sho00.bin", 0x80000, 0xcb2201a3, BRF_PRG },
};

STD_ROM_PICK(g_sho00);
STD_ROM_FN(g_sho00);

struct BurnDriverD BurnDrvMDg_sho00 = {
	"g_sho00", "g_sho", NULL, "1900",
	"Super Hang-On (W) (REV00) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sho00RomInfo, g_sho00RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Super Thunder Blade (W) (REV01) [!]
static struct BurnRomInfo g_stbRomDesc[] = {
	{ "g_stb.bin", 0x80000, 0xb13087ee, BRF_PRG },
};

STD_ROM_PICK(g_stb);
STD_ROM_FN(g_stb);

struct BurnDriverD BurnDrvMDg_stb = {
	"g_stb", NULL, NULL, "1900",
	"Super Thunder Blade (W) (REV01) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_stbRomInfo, g_stbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Super Thunder Blade (W) (REV00) [!]
static struct BurnRomInfo g_stb00RomDesc[] = {
	{ "g_stb00.bin", 0x80000, 0x8bd77836, BRF_PRG },
};

STD_ROM_PICK(g_stb00);
STD_ROM_FN(g_stb00);

struct BurnDriverD BurnDrvMDg_stb00 = {
	"g_stb00", "g_stb", NULL, "1900",
	"Super Thunder Blade (W) (REV00) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_stb00RomInfo, g_stb00RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Pinocchio (U) [!]
static struct BurnRomInfo g_pinoRomDesc[] = {
	{ "g_pino.bin", 0x300000, 0xcd4128d8, BRF_PRG },
};

STD_ROM_PICK(g_pino);
STD_ROM_FN(g_pino);

struct BurnDriverD BurnDrvMDg_pino = {
	"g_pino", NULL, NULL, "1900",
	"Pinocchio (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pinoRomInfo, g_pinoRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Pinocchio (E)
static struct BurnRomInfo g_pinnoeRomDesc[] = {
	{ "g_pinnoe.bin", 0x300000, 0x28014bdc, BRF_PRG },
};

STD_ROM_PICK(g_pinnoe);
STD_ROM_FN(g_pinnoe);

struct BurnDriverD BurnDrvMDg_pinnoe = {
	"g_pinnoe", "g_pino", NULL, "1900",
	"Pinocchio (E)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pinnoeRomInfo, g_pinnoeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Puyo Puyo (J) [!]
static struct BurnRomInfo g_puyoRomDesc[] = {
	{ "g_puyo.bin", 0x80000, 0x7f26614e, BRF_PRG },
};

STD_ROM_PICK(g_puyo);
STD_ROM_FN(g_puyo);

struct BurnDriverD BurnDrvMDg_puyo = {
	"g_puyo", NULL, NULL, "1900",
	"Puyo Puyo (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_puyoRomInfo, g_puyoRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// SeaQuest DSV (U) [!]
static struct BurnRomInfo g_seaqRomDesc[] = {
	{ "g_seaq.bin", 0x200000, 0x25b05480, BRF_PRG },
};

STD_ROM_PICK(g_seaq);
STD_ROM_FN(g_seaq);

struct BurnDriverD BurnDrvMDg_seaq = {
	"g_seaq", NULL, NULL, "1900",
	"SeaQuest DSV (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_seaqRomInfo, g_seaqRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// SeaQuest DSV (E)
static struct BurnRomInfo g_seaqeRomDesc[] = {
	{ "g_seaqe.bin", 0x200000, 0x0bb511bd, BRF_PRG },
};

STD_ROM_PICK(g_seaqe);
STD_ROM_FN(g_seaqe);

struct BurnDriverD BurnDrvMDg_seaqe = {
	"g_seaqe", "g_seaq", NULL, "1900",
	"SeaQuest DSV (E)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_seaqeRomInfo, g_seaqeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Tale Spin (UE) [!]
static struct BurnRomInfo g_taleRomDesc[] = {
	{ "g_tale.bin", 0x80000, 0xf5c0c8d0, BRF_PRG },
};

STD_ROM_PICK(g_tale);
STD_ROM_FN(g_tale);

struct BurnDriverD BurnDrvMDg_tale = {
	"g_tale", NULL, NULL, "1900",
	"Tale Spin (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_taleRomInfo, g_taleRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Ballz (UE) [!]
static struct BurnRomInfo g_balzRomDesc[] = {
	{ "g_balz.bin", 0x200000, 0xb362b705, BRF_PRG },
};

STD_ROM_PICK(g_balz);
STD_ROM_FN(g_balz);

struct BurnDriverD BurnDrvMDg_balz = {
	"g_balz", NULL, NULL, "1900",
	"Ballz (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_balzRomInfo, g_balzRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Bonkers (UE) [!]
static struct BurnRomInfo g_bonkRomDesc[] = {
	{ "g_bonk.bin", 0x100000, 0xd1e66017, BRF_PRG },
};

STD_ROM_PICK(g_bonk);
STD_ROM_FN(g_bonk);

struct BurnDriverD BurnDrvMDg_bonk = {
	"g_bonk", NULL, NULL, "1900",
	"Bonkers (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_bonkRomInfo, g_bonkRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Marsupilami (U) (M5) [!]
static struct BurnRomInfo g_marsRomDesc[] = {
	{ "g_mars.bin", 0x200000, 0xc76558df, BRF_PRG },
};

STD_ROM_PICK(g_mars);
STD_ROM_FN(g_mars);

struct BurnDriverD BurnDrvMDg_mars = {
	"g_mars", NULL, NULL, "1900",
	"Marsupilami (U) (M5) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_marsRomInfo, g_marsRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Marsupilami (E) (M5) [!]
static struct BurnRomInfo g_marsueRomDesc[] = {
	{ "g_marsue.bin", 0x200000, 0xe09bbd70, BRF_PRG },
};

STD_ROM_PICK(g_marsue);
STD_ROM_FN(g_marsue);

struct BurnDriverD BurnDrvMDg_marsue = {
	"g_marsue", "g_mars", NULL, "1900",
	"Marsupilami (E) (M5) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_marsueRomInfo, g_marsueRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Flavio's DMA Test (PD)
static struct BurnRomInfo g_fdmaRomDesc[] = {
	{ "g_fdma.bin", 0x4000, 0x549cebf4, BRF_PRG },
};

STD_ROM_PICK(g_fdma);
STD_ROM_FN(g_fdma);

struct BurnDriverD BurnDrvMDg_fdma = {
	"g_fdma", NULL, NULL, "1900",
	"Flavio's DMA Test (PD)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_fdmaRomInfo, g_fdmaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// 3 Ninjas Kick Back (U) [!]
static struct BurnRomInfo g_3ninRomDesc[] = {
	{ "g_3nin.bin", 0x200000, 0xE5A24999, BRF_PRG },
};

STD_ROM_PICK(g_3nin);
STD_ROM_FN(g_3nin);

struct BurnDriver BurnDrvMDg_3nin = {
	"g_3nin", NULL, NULL, "1994",
	"3 Ninjas Kick Back (U) [!]\0", NULL, "Sony Electronic", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_SCRFIGHT, 0,
	NULL, g_3ninRomInfo, g_3ninRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Haunting Starring Polterguy (UE) [!]
static struct BurnRomInfo g_haunRomDesc[] = {
	{ "g_haun.bin", 0x200000, 0xc9fc876d, BRF_PRG },
};

STD_ROM_PICK(g_haun);
STD_ROM_FN(g_haun);

struct BurnDriverD BurnDrvMDg_haun = {
	"g_haun", NULL, NULL, "1900",
	"Haunting Starring Polterguy (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_haunRomInfo, g_haunRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Misadventures of Flink, The (E) [!]
static struct BurnRomInfo g_mfliRomDesc[] = {
	{ "g_mfli.bin", 0x100000, 0xbef9a4f4, BRF_PRG },
};

STD_ROM_PICK(g_mfli);
STD_ROM_FN(g_mfli);

struct BurnDriverD BurnDrvMDg_mfli = {
	"g_mfli", NULL, NULL, "1900",
	"Misadventures of Flink, The (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mfliRomInfo, g_mfliRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Risky Woods (UE) [!]
static struct BurnRomInfo g_rwooRomDesc[] = {
	{ "g_rwoo.bin", 0x100000, 0xd975e93c, BRF_PRG },
};

STD_ROM_PICK(g_rwoo);
STD_ROM_FN(g_rwoo);

struct BurnDriverD BurnDrvMDg_rwoo = {
	"g_rwoo", NULL, NULL, "1900",
	"Risky Woods (UE) [!]\0", NULL, "Electronic Arts", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_rwooRomInfo, g_rwooRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Draxos (J) [!]
static struct BurnRomInfo g_draxosRomDesc[] = {
	{ "g_draxos.bin", 0x100000, 0x1ea07af2, BRF_PRG },
};

STD_ROM_PICK(g_draxos);
STD_ROM_FN(g_draxos);

struct BurnDriverD BurnDrvMDg_draxos = {
	"g_draxos", "g_rwoo", NULL, "1900",
	"Draxos (J) [!]\0", NULL, "Electronic Arts", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_draxosRomInfo, g_draxosRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Lotus Turbo Challenge (UE) [!]
static struct BurnRomInfo g_lotRomDesc[] = {
	{ "g_lot.bin", 0x100000, 0xa3cf6e9c, BRF_PRG },
};

STD_ROM_PICK(g_lot);
STD_ROM_FN(g_lot);

struct BurnDriverD BurnDrvMDg_lot = {
	"g_lot", NULL, NULL, "1900",
	"Lotus Turbo Challenge (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_lotRomInfo, g_lotRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Lotus II RECS (UE) [!]
static struct BurnRomInfo g_lot2RomDesc[] = {
	{ "g_lot2.bin", 0x100000, 0x1d8ee010, BRF_PRG },
};

STD_ROM_PICK(g_lot2);
STD_ROM_FN(g_lot2);

struct BurnDriverD BurnDrvMDg_lot2 = {
	"g_lot2", NULL, NULL, "1900",
	"Lotus II RECS (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_lot2RomInfo, g_lot2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Lotus II RECS (Beta)
static struct BurnRomInfo g_lotu2bRomDesc[] = {
	{ "g_lotu2b.bin", 0x100000, 0x2997b7d4, BRF_PRG },
};

STD_ROM_PICK(g_lotu2b);
STD_ROM_FN(g_lotu2b);

struct BurnDriverD BurnDrvMDg_lotu2b = {
	"g_lotu2b", "g_lot2", NULL, "1900",
	"Lotus II RECS (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_lotu2bRomInfo, g_lotu2bRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Street Racer (E) [c][!]
static struct BurnRomInfo g_straRomDesc[] = {
	{ "g_stra.bin", 0x100000, 0x1a58d5fe, BRF_PRG },
};

STD_ROM_PICK(g_stra);
STD_ROM_FN(g_stra);

struct BurnDriverD BurnDrvMDg_stra = {
	"g_stra", NULL, NULL, "1900",
	"Street Racer (E) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_straRomInfo, g_straRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Tinhead (U) [!]
static struct BurnRomInfo g_tinhRomDesc[] = {
	{ "g_tinh.bin", 0x100000, 0xd6724b84, BRF_PRG },
};

STD_ROM_PICK(g_tinh);
STD_ROM_FN(g_tinh);

struct BurnDriverD BurnDrvMDg_tinh = {
	"g_tinh", NULL, NULL, "1900",
	"Tinhead (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tinhRomInfo, g_tinhRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Top Gear 2 (U) [!]
static struct BurnRomInfo g_tg2RomDesc[] = {
	{ "g_tg2.bin", 0x100000, 0xbd3074d2, BRF_PRG },
};

STD_ROM_PICK(g_tg2);
STD_ROM_FN(g_tg2);

struct BurnDriverD BurnDrvMDg_tg2 = {
	"g_tg2", NULL, NULL, "1900",
	"Top Gear 2 (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tg2RomInfo, g_tg2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Greendog - The Beached Surfer Dude (UE) [!]
static struct BurnRomInfo g_gdogRomDesc[] = {
	{ "g_gdog.bin", 0x80000, 0xc4820a03, BRF_PRG },
};

STD_ROM_PICK(g_gdog);
STD_ROM_FN(g_gdog);

struct BurnDriverD BurnDrvMDg_gdog = {
	"g_gdog", NULL, NULL, "1900",
	"Greendog - The Beached Surfer Dude (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_gdogRomInfo, g_gdogRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Gaiares (JU) [!]
static struct BurnRomInfo g_gaiaRomDesc[] = {
	{ "g_gaia.bin", 0x100000, 0x5d8bf68b, BRF_PRG },
};

STD_ROM_PICK(g_gaia);
STD_ROM_FN(g_gaia);

struct BurnDriverD BurnDrvMDg_gaia = {
	"g_gaia", NULL, NULL, "1900",
	"Gaiares (JU) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_gaiaRomInfo, g_gaiaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Elemental Master (U) [!]
static struct BurnRomInfo g_elemRomDesc[] = {
	{ "g_elem.bin", 0x80000, 0x390918c6, BRF_PRG },
};

STD_ROM_PICK(g_elem);
STD_ROM_FN(g_elem);

struct BurnDriverD BurnDrvMDg_elem = {
	"g_elem", NULL, NULL, "1900",
	"Elemental Master (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_elemRomInfo, g_elemRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Elemental Master (J) [!]
static struct BurnRomInfo g_elemjRomDesc[] = {
	{ "g_elemj.bin", 0x80000, 0x5f553e29, BRF_PRG },
};

STD_ROM_PICK(g_elemj);
STD_ROM_FN(g_elemj);

struct BurnDriverD BurnDrvMDg_elemj = {
	"g_elemj", "g_elem", NULL, "1900",
	"Elemental Master (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_elemjRomInfo, g_elemjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Man Overboard! - S.S. Lucifer (E) [c][!]
static struct BurnRomInfo g_manoRomDesc[] = {
	{ "g_mano.bin", 0x100000, 0xcae0e3a6, BRF_PRG },
};

STD_ROM_PICK(g_mano);
STD_ROM_FN(g_mano);

struct BurnDriverD BurnDrvMDg_mano = {
	"g_mano", NULL, NULL, "1900",
	"Man Overboard! - S.S. Lucifer (E) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_manoRomInfo, g_manoRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Daze Before Christmas, The (E) [!]
static struct BurnRomInfo g_dazeRomDesc[] = {
	{ "g_daze.bin", 0x200000, 0xb95e25c9, BRF_PRG },
};

STD_ROM_PICK(g_daze);
STD_ROM_FN(g_daze);

struct BurnDriverD BurnDrvMDg_daze = {
	"g_daze", NULL, NULL, "1900",
	"Daze Before Christmas, The (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_dazeRomInfo, g_dazeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Daze Before Christmas, The (Beta)
static struct BurnRomInfo g_dazebRomDesc[] = {
	{ "g_dazeb.bin", 0x200000, 0x317c9491, BRF_PRG },
};

STD_ROM_PICK(g_dazeb);
STD_ROM_FN(g_dazeb);

struct BurnDriverD BurnDrvMDg_dazeb = {
	"g_dazeb", "g_daze", NULL, "1900",
	"Daze Before Christmas, The (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_dazebRomInfo, g_dazebRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Jimmy White's Whirlwind Snooker (E) [c][!]
static struct BurnRomInfo g_jwwsRomDesc[] = {
	{ "g_jwws.bin", 0x80000, 0x0aef5b1f, BRF_PRG },
};

STD_ROM_PICK(g_jwws);
STD_ROM_FN(g_jwws);

struct BurnDriverD BurnDrvMDg_jwws = {
	"g_jwws", NULL, NULL, "1900",
	"Jimmy White's Whirlwind Snooker (E) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jwwsRomInfo, g_jwwsRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Bubble and Squeak (U) [!].zip
static struct BurnRomInfo g_busqRomDesc[] = {
	{ "g_busq.bin", 0x80000, 0x28c4a006, BRF_PRG },
};

STD_ROM_PICK(g_busq);
STD_ROM_FN(g_busq);

struct BurnDriver BurnDrvMDg_busq = {
	"g_busq", NULL, NULL, "1900",
	"Bubble and Squeak (U) [!].zip\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_busqRomInfo, g_busqRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Outlander (U) [!]
static struct BurnRomInfo g_olanRomDesc[] = {
	{ "g_olan.bin", 0x100000, 0xfe6f2350, BRF_PRG },
};

STD_ROM_PICK(g_olan);
STD_ROM_FN(g_olan);

struct BurnDriverD BurnDrvMDg_olan = {
	"g_olan", NULL, NULL, "1900",
	"Outlander (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_olanRomInfo, g_olanRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Outlander (Beta)
static struct BurnRomInfo g_outlnbRomDesc[] = {
	{ "g_outlnb.bin", 0x100000, 0xc5ba7bbf, BRF_PRG },
};

STD_ROM_PICK(g_outlnb);
STD_ROM_FN(g_outlnb);

struct BurnDriverD BurnDrvMDg_outlnb = {
	"g_outlnb", "g_olan", NULL, "1900",
	"Outlander (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_outlnbRomInfo, g_outlnbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Pirates of Dark Water, The (U) [!]
static struct BurnRomInfo g_pidwRomDesc[] = {
	{ "g_pidw.bin", 0x200000, 0x0c45b9f7, BRF_PRG },
};

STD_ROM_PICK(g_pidw);
STD_ROM_FN(g_pidw);

struct BurnDriverD BurnDrvMDg_pidw = {
	"g_pidw", NULL, NULL, "1900",
	"Pirates of Dark Water, The (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pidwRomInfo, g_pidwRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Pirates of Dark Water, The (UE)
static struct BurnRomInfo g_pirdwuRomDesc[] = {
	{ "g_pirdwu.bin", 0x200000, 0x0a62de34, BRF_PRG },
};

STD_ROM_PICK(g_pirdwu);
STD_ROM_FN(g_pirdwu);

struct BurnDriverD BurnDrvMDg_pirdwu = {
	"g_pirdwu", "g_pidw", NULL, "1900",
	"Pirates of Dark Water, The (UE)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pirdwuRomInfo, g_pirdwuRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Beauty and the Beast - Roar of the Beast (U) [!]
static struct BurnRomInfo g_bbrbRomDesc[] = {
	{ "g_bbrb.bin", 0x100000, 0x13e7b519, BRF_PRG },
};

STD_ROM_PICK(g_bbrb);
STD_ROM_FN(g_bbrb);

struct BurnDriverD BurnDrvMDg_bbrb = {
	"g_bbrb", NULL, NULL, "1900",
	"Beauty and the Beast - Roar of the Beast (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_bbrbRomInfo, g_bbrbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Beauty and the Beast - Belle's Quest (U) [!]
static struct BurnRomInfo g_bbbqRomDesc[] = {
	{ "g_bbbq.bin", 0x100000, 0xbefb6fae, BRF_PRG },
};

STD_ROM_PICK(g_bbbq);
STD_ROM_FN(g_bbbq);

struct BurnDriverD BurnDrvMDg_bbbq = {
	"g_bbbq", NULL, NULL, "1900",
	"Beauty and the Beast - Belle's Quest (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_bbbqRomInfo, g_bbbqRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Tanto R (J) [!]
static struct BurnRomInfo g_tanrRomDesc[] = {
	{ "g_tanr.bin", 0x200000, 0xd2d2d437, BRF_PRG },
};

STD_ROM_PICK(g_tanr);
STD_ROM_FN(g_tanr);

struct BurnDriverD BurnDrvMDg_tanr = {
	"g_tanr", NULL, NULL, "1900",
	"Tanto R (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tanrRomInfo, g_tanrRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Rings of Power (UE) [!]
static struct BurnRomInfo g_ropRomDesc[] = {
	{ "g_rop.bin", 0x100000, 0x41fcc497, BRF_PRG },
};

STD_ROM_PICK(g_rop);
STD_ROM_FN(g_rop);

struct BurnDriverD BurnDrvMDg_rop = {
	"g_rop", NULL, NULL, "1900",
	"Rings of Power (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ropRomInfo, g_ropRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Pete Sampras Tennis 96 (E) [c][!]
static struct BurnRomInfo g_ps96RomDesc[] = {
	{ "g_ps96.bin", 0x200000, 0x14e3fb7b, BRF_PRG },
};

STD_ROM_PICK(g_ps96);
STD_ROM_FN(g_ps96);

struct BurnDriverD BurnDrvMDg_ps96 = {
	"g_ps96", NULL, NULL, "1900",
	"Pete Sampras Tennis 96 (E) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ps96RomInfo, g_ps96RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Toughman Contest (UE) [!]
static struct BurnRomInfo g_tougRomDesc[] = {
	{ "g_toug.bin", 0x400000, 0xe19fbc93, BRF_PRG },
};

STD_ROM_PICK(g_toug);
STD_ROM_FN(g_toug);

struct BurnDriverD BurnDrvMDg_toug = {
	"g_toug", NULL, NULL, "1900",
	"Toughman Contest (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tougRomInfo, g_tougRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Spirou (E) [!]
static struct BurnRomInfo g_spirRomDesc[] = {
	{ "g_spir.bin", 0x100000, 0x6634b130, BRF_PRG },
};

STD_ROM_PICK(g_spir);
STD_ROM_FN(g_spir);

struct BurnDriverD BurnDrvMDg_spir = {
	"g_spir", NULL, NULL, "1900",
	"Spirou (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_spirRomInfo, g_spirRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Striker (E) (M5) [!]
static struct BurnRomInfo g_strkRomDesc[] = {
	{ "g_strk.bin", 0x200000, 0xcc5d7ab2, BRF_PRG },
};

STD_ROM_PICK(g_strk);
STD_ROM_FN(g_strk);

struct BurnDriverD BurnDrvMDg_strk = {
	"g_strk", NULL, NULL, "1900",
	"Striker (E) (M5) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_strkRomInfo, g_strkRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Striker (Beta)
static struct BurnRomInfo g_strikbRomDesc[] = {
	{ "g_strikb.bin", 0x200000, 0xc10b270e, BRF_PRG },
};

STD_ROM_PICK(g_strikb);
STD_ROM_FN(g_strikb);

struct BurnDriverD BurnDrvMDg_strikb = {
	"g_strikb", "g_strk", NULL, "1900",
	"Striker (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_strikbRomInfo, g_strikbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Cheese Cat-Astrophe Starring Speedy Gonzales (E) (M4) [!]
static struct BurnRomInfo g_cheeRomDesc[] = {
	{ "g_chee.bin", 0x200000, 0xff634b28, BRF_PRG },
};

STD_ROM_PICK(g_chee);
STD_ROM_FN(g_chee);

struct BurnDriverD BurnDrvMDg_chee = {
	"g_chee", NULL, NULL, "1900",
	"Cheese Cat-Astrophe Starring Speedy Gonzales (E) (M4) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_cheeRomInfo, g_cheeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Test Drive II - The Duel (U) [!]
static struct BurnRomInfo g_td2RomDesc[] = {
	{ "g_td2.bin", 0x100000, 0xf9bdf8c5, BRF_PRG },
};

STD_ROM_PICK(g_td2);
STD_ROM_FN(g_td2);

struct BurnDriverD BurnDrvMDg_td2 = {
	"g_td2", NULL, NULL, "1900",
	"Test Drive II - The Duel (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_td2RomInfo, g_td2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Turbo Outrun (JE) [c][!]
static struct BurnRomInfo g_toutRomDesc[] = {
	{ "g_tout.bin", 0x80000, 0x0c661369, BRF_PRG },
};

STD_ROM_PICK(g_tout);
STD_ROM_FN(g_tout);

struct BurnDriverD BurnDrvMDg_tout = {
	"g_tout", NULL, NULL, "1900",
	"Turbo Outrun (JE) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_toutRomInfo, g_toutRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Garfield - Caught in the Act (UE) [!]
static struct BurnRomInfo g_garfRomDesc[] = {
	{ "g_garf.bin", 0x200000, 0xf0ff078e, BRF_PRG },
};

STD_ROM_PICK(g_garf);
STD_ROM_FN(g_garf);

struct BurnDriverD BurnDrvMDg_garf = {
	"g_garf", NULL, NULL, "1900",
	"Garfield - Caught in the Act (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_garfRomInfo, g_garfRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Dragon's Revenge (UE) [!]
static struct BurnRomInfo g_drevRomDesc[] = {
	{ "g_drev.bin", 0x100000, 0x841edbc0, BRF_PRG },
};

STD_ROM_PICK(g_drev);
STD_ROM_FN(g_drev);

struct BurnDriverD BurnDrvMDg_drev = {
	"g_drev", NULL, NULL, "1900",
	"Dragon's Revenge (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_drevRomInfo, g_drevRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Addams Family Values (E) (M3) [!]
static struct BurnRomInfo g_afavRomDesc[] = {
	{ "g_afav.bin", 0x200000, 0xB906B992, BRF_PRG },
};

STD_ROM_PICK(g_afav);
STD_ROM_FN(g_afav);

struct BurnDriver BurnDrvMDg_afav = {
	"g_afav", NULL, NULL, "1994",
	"Addams Family Values (E) (M3) [!]\0", NULL, "Ocean Software Ltd.", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_afavRomInfo, g_afavRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Ariel - The Little Mermaid (UE) [!]
static struct BurnRomInfo g_arieRomDesc[] = {
	{ "g_arie.bin", 0x80000, 0x58e297df, BRF_PRG },
};

STD_ROM_PICK(g_arie);
STD_ROM_FN(g_arie);

struct BurnDriver BurnDrvMDg_arie = {
	"g_arie", NULL, NULL, "1900",
	"Ariel - The Little Mermaid (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_arieRomInfo, g_arieRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Arrow Flash (UE) [!]
static struct BurnRomInfo g_arroRomDesc[] = {
	{ "g_arro.bin", 0x80000, 0x4d89e66b, BRF_PRG },
};

STD_ROM_PICK(g_arro);
STD_ROM_FN(g_arro);

struct BurnDriverD BurnDrvMDg_arro = {
	"g_arro", NULL, NULL, "1900",
	"Arrow Flash (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_arroRomInfo, g_arroRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Arrow Flash (J) [!]
static struct BurnRomInfo g_arrowjRomDesc[] = {
	{ "g_arrowj.bin", 0x80000, 0xd49f8444, BRF_PRG },
};

STD_ROM_PICK(g_arrowj);
STD_ROM_FN(g_arrowj);

struct BurnDriverD BurnDrvMDg_arrowj = {
	"g_arrowj", "g_arro", NULL, "1900",
	"Arrow Flash (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_arrowjRomInfo, g_arrowjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Blaster Master 2 (U) [c][!]
static struct BurnRomInfo g_blmaRomDesc[] = {
	{ "g_blma.bin", 0x100000, 0xc11e4ba1, BRF_PRG },
};

STD_ROM_PICK(g_blma);
STD_ROM_FN(g_blma);

struct BurnDriverD BurnDrvMDg_blma = {
	"g_blma", NULL, NULL, "1900",
	"Blaster Master 2 (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_blmaRomInfo, g_blmaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Blaster Master 2 (Beta)
static struct BurnRomInfo g_blam2bRomDesc[] = {
	{ "g_blam2b.bin", 0x100000, 0x08f78c70, BRF_PRG },
};

STD_ROM_PICK(g_blam2b);
STD_ROM_FN(g_blam2b);

struct BurnDriverD BurnDrvMDg_blam2b = {
	"g_blam2b", "g_blma", NULL, "1900",
	"Blaster Master 2 (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_blam2bRomInfo, g_blam2bRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Bubsy in Claws Encounters of the Furred Kind (UE) [!]
static struct BurnRomInfo g_bubsRomDesc[] = {
	{ "g_bubs.bin", 0x200000, 0x3e30d365, BRF_PRG },
};

STD_ROM_PICK(g_bubs);
STD_ROM_FN(g_bubs);

struct BurnDriver BurnDrvMDg_bubs = {
	"g_bubs", NULL, NULL, "1900",
	"Bubsy in Claws Encounters of the Furred Kind (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_bubsRomInfo, g_bubsRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Bubsy II (UE) [!]
static struct BurnRomInfo g_bub2RomDesc[] = {
	{ "g_bub2.bin", 0x200000, 0xf8beff56, BRF_PRG },
};

STD_ROM_PICK(g_bub2);
STD_ROM_FN(g_bub2);

struct BurnDriver BurnDrvMDg_bub2 = {
	"g_bub2", NULL, NULL, "1900",
	"Bubsy II (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_bub2RomInfo, g_bub2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Bugs Bunny in Double Trouble (U) [!]
static struct BurnRomInfo g_bbnyRomDesc[] = {
	{ "g_bbny.bin", 0x200000, 0x365305a2, BRF_PRG },
};

STD_ROM_PICK(g_bbny);
STD_ROM_FN(g_bbny);

struct BurnDriver BurnDrvMDg_bbny = {
	"g_bbny", NULL, NULL, "1900",
	"Bugs Bunny in Double Trouble (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_bbnyRomInfo, g_bbnyRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Cannon Fodder (E) [!]
static struct BurnRomInfo g_canoRomDesc[] = {
	{ "g_cano.bin", 0x180000, 0xad217654, BRF_PRG },
};

STD_ROM_PICK(g_cano);
STD_ROM_FN(g_cano);

struct BurnDriverD BurnDrvMDg_cano = {
	"g_cano", NULL, NULL, "1900",
	"Cannon Fodder (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_canoRomInfo, g_canoRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Championship Pool (U) [!]
static struct BurnRomInfo g_cpooRomDesc[] = {
	{ "g_cpoo.bin", 0x100000, 0x253512cf, BRF_PRG },
};

STD_ROM_PICK(g_cpoo);
STD_ROM_FN(g_cpoo);

struct BurnDriverD BurnDrvMDg_cpoo = {
	"g_cpoo", NULL, NULL, "1900",
	"Championship Pool (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_cpooRomInfo, g_cpooRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Chase HQ II (U) [!]
static struct BurnRomInfo g_chq2RomDesc[] = {
	{ "g_chq2.bin", 0x80000, 0xf39e4bf2, BRF_PRG },
};

STD_ROM_PICK(g_chq2);
STD_ROM_FN(g_chq2);

struct BurnDriverD BurnDrvMDg_chq2 = {
	"g_chq2", NULL, NULL, "1900",
	"Chase HQ II (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_chq2RomInfo, g_chq2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Super HQ (J) [!]
static struct BurnRomInfo g_suphqRomDesc[] = {
	{ "g_suphq.bin", 0x80000, 0xab2c52b0, BRF_PRG },
};

STD_ROM_PICK(g_suphq);
STD_ROM_FN(g_suphq);

struct BurnDriverD BurnDrvMDg_suphq = {
	"g_suphq", "g_chq2", NULL, "1900",
	"Super HQ (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_suphqRomInfo, g_suphqRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Cool Spot (U) [!]
static struct BurnRomInfo g_coolRomDesc[] = {
	{ "g_cool.bin", 0x100000, 0xf024c1a1, BRF_PRG },
};

STD_ROM_PICK(g_cool);
STD_ROM_FN(g_cool);

struct BurnDriverD BurnDrvMDg_cool = {
	"g_cool", NULL, NULL, "1900",
	"Cool Spot (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_coolRomInfo, g_coolRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Cool Spot (E) [!]
static struct BurnRomInfo g_cooleRomDesc[] = {
	{ "g_coole.bin", 0x100000, 0x5f09fa41, BRF_PRG },
};

STD_ROM_PICK(g_coole);
STD_ROM_FN(g_coole);

struct BurnDriverD BurnDrvMDg_coole = {
	"g_coole", "g_cool", NULL, "1900",
	"Cool Spot (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_cooleRomInfo, g_cooleRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Cool Spot (Beta)
static struct BurnRomInfo g_spotbRomDesc[] = {
	{ "g_spotb.bin", 0x100000, 0x0ebaa4a8, BRF_PRG },
};

STD_ROM_PICK(g_spotb);
STD_ROM_FN(g_spotb);

struct BurnDriverD BurnDrvMDg_spotb = {
	"g_spotb", "g_cool", NULL, "1900",
	"Cool Spot (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_spotbRomInfo, g_spotbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Dick Vitale's Awesome Baby! College Hoops (U) [!]
static struct BurnRomInfo g_dicvRomDesc[] = {
	{ "g_dicv.bin", 0x200000, 0x1312cf22, BRF_PRG },
};

STD_ROM_PICK(g_dicv);
STD_ROM_FN(g_dicv);

struct BurnDriverD BurnDrvMDg_dicv = {
	"g_dicv", NULL, NULL, "1900",
	"Dick Vitale's Awesome Baby! College Hoops (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_dicvRomInfo, g_dicvRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Escape From Mars Starring Taz (U) [!]
static struct BurnRomInfo g_taz2RomDesc[] = {
	{ "g_taz2.bin", 0x200000, 0x62009f8c, BRF_PRG },
};

STD_ROM_PICK(g_taz2);
STD_ROM_FN(g_taz2);

struct BurnDriverD BurnDrvMDg_taz2 = {
	"g_taz2", NULL, NULL, "1900",
	"Escape From Mars Starring Taz (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_taz2RomInfo, g_taz2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Escape From Mars Starring Taz (E) [!]
static struct BurnRomInfo g_taz2eRomDesc[] = {
	{ "g_taz2e.bin", 0x200000, 0x62100099, BRF_PRG },
};

STD_ROM_PICK(g_taz2e);
STD_ROM_FN(g_taz2e);

struct BurnDriverD BurnDrvMDg_taz2e = {
	"g_taz2e", "g_taz2", NULL, "1900",
	"Escape From Mars Starring Taz (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_taz2eRomInfo, g_taz2eRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Heavy Unit (J) [!]
static struct BurnRomInfo g_huniRomDesc[] = {
	{ "g_huni.bin", 0x80000, 0x1acbe608, BRF_PRG },
};

STD_ROM_PICK(g_huni);
STD_ROM_FN(g_huni);

struct BurnDriverD BurnDrvMDg_huni = {
	"g_huni", NULL, NULL, "1900",
	"Heavy Unit (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_huniRomInfo, g_huniRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Hook (U) [c][!]
static struct BurnRomInfo g_hookRomDesc[] = {
	{ "g_hook.bin", 0x100000, 0x2c48e712, BRF_PRG },
};

STD_ROM_PICK(g_hook);
STD_ROM_FN(g_hook);

struct BurnDriverD BurnDrvMDg_hook = {
	"g_hook", NULL, NULL, "1900",
	"Hook (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_hookRomInfo, g_hookRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Humans, The (U) [!]
static struct BurnRomInfo g_humaRomDesc[] = {
	{ "g_huma.bin", 0x100000, 0xa0cf4366, BRF_PRG },
};

STD_ROM_PICK(g_huma);
STD_ROM_FN(g_huma);

struct BurnDriverD BurnDrvMDg_huma = {
	"g_huma", NULL, NULL, "1900",
	"Humans, The (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_humaRomInfo, g_humaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Jurassic Park - Rampage Edition (UE) [!]
static struct BurnRomInfo g_jpraRomDesc[] = {
	{ "g_jpra.bin", 0x200000, 0x98b4aa1b, BRF_PRG },
};

STD_ROM_PICK(g_jpra);
STD_ROM_FN(g_jpra);

struct BurnDriverD BurnDrvMDg_jpra = {
	"g_jpra", NULL, NULL, "1900",
	"Jurassic Park - Rampage Edition (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jpraRomInfo, g_jpraRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Magical Hat no Buttobi Turbo! Daibouken (J) [!]
static struct BurnRomInfo g_mhatRomDesc[] = {
	{ "g_mhat.bin", 0x80000, 0xe43e853d, BRF_PRG },
};

STD_ROM_PICK(g_mhat);
STD_ROM_FN(g_mhat);

struct BurnDriverD BurnDrvMDg_mhat = {
	"g_mhat", NULL, NULL, "1900",
	"Magical Hat no Buttobi Turbo! Daibouken (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mhatRomInfo, g_mhatRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mr. Nutz (E) [!]
static struct BurnRomInfo g_nutzRomDesc[] = {
	{ "g_nutz.bin", 0x100000, 0x0786ea0b, BRF_PRG },
};

STD_ROM_PICK(g_nutz);
STD_ROM_FN(g_nutz);

struct BurnDriverD BurnDrvMDg_nutz = {
	"g_nutz", NULL, NULL, "1900",
	"Mr. Nutz (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_nutzRomInfo, g_nutzRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Ooze, The (JU) [!]
static struct BurnRomInfo g_oozeRomDesc[] = {
	{ "g_ooze.bin", 0x100000, 0x1c0dd42f, BRF_PRG },
};

STD_ROM_PICK(g_ooze);
STD_ROM_FN(g_ooze);

struct BurnDriverD BurnDrvMDg_ooze = {
	"g_ooze", NULL, NULL, "1900",
	"Ooze, The (JU) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_oozeRomInfo, g_oozeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Ooze, The (E) [!]
static struct BurnRomInfo g_oozeeRomDesc[] = {
	{ "g_oozee.bin", 0x100000, 0xe16b102c, BRF_PRG },
};

STD_ROM_PICK(g_oozee);
STD_ROM_FN(g_oozee);

struct BurnDriverD BurnDrvMDg_oozee = {
	"g_oozee", "g_ooze", NULL, "1900",
	"Ooze, The (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_oozeeRomInfo, g_oozeeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Phantom 2040 (U) [!]
static struct BurnRomInfo g_2040RomDesc[] = {
	{ "g_2040.bin", 0x200000, 0xfb36e1f3, BRF_PRG },
};

STD_ROM_PICK(g_2040);
STD_ROM_FN(g_2040);

struct BurnDriverD BurnDrvMDg_2040 = {
	"g_2040", NULL, NULL, "1900",
	"Phantom 2040 (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_2040RomInfo, g_2040RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Phantom 2040 (E) [!]
static struct BurnRomInfo g_2040eRomDesc[] = {
	{ "g_2040e.bin", 0x200000, 0xb024882e, BRF_PRG },
};

STD_ROM_PICK(g_2040e);
STD_ROM_FN(g_2040e);

struct BurnDriverD BurnDrvMDg_2040e = {
	"g_2040e", "g_2040", NULL, "1900",
	"Phantom 2040 (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_2040eRomInfo, g_2040eRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Skeleton Krew (U) [!]
static struct BurnRomInfo g_krewRomDesc[] = {
	{ "g_krew.bin", 0x200000, 0xc2e05acb, BRF_PRG },
};

STD_ROM_PICK(g_krew);
STD_ROM_FN(g_krew);

struct BurnDriverD BurnDrvMDg_krew = {
	"g_krew", NULL, NULL, "1900",
	"Skeleton Krew (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_krewRomInfo, g_krewRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Skeleton Krew (E)
static struct BurnRomInfo g_skreweRomDesc[] = {
	{ "g_skrewe.bin", 0x200000, 0x5f872737, BRF_PRG },
};

STD_ROM_PICK(g_skrewe);
STD_ROM_FN(g_skrewe);

struct BurnDriverD BurnDrvMDg_skrewe = {
	"g_skrewe", "g_krew", NULL, "1900",
	"Skeleton Krew (E)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_skreweRomInfo, g_skreweRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Super Kick Off (E) [c][!]
static struct BurnRomInfo g_skikRomDesc[] = {
	{ "g_skik.bin", 0x80000, 0xf43793ff, BRF_PRG },
};

STD_ROM_PICK(g_skik);
STD_ROM_FN(g_skik);

struct BurnDriverD BurnDrvMDg_skik = {
	"g_skik", NULL, NULL, "1900",
	"Super Kick Off (E) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_skikRomInfo, g_skikRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Terminator, The (U) [!]
static struct BurnRomInfo g_termRomDesc[] = {
	{ "g_term.bin", 0x100000, 0x31a629be, BRF_PRG },
};

STD_ROM_PICK(g_term);
STD_ROM_FN(g_term);

struct BurnDriverD BurnDrvMDg_term = {
	"g_term", NULL, NULL, "1900",
	"Terminator, The (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_termRomInfo, g_termRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Terminator, The (E) [c][!]
static struct BurnRomInfo g_termeRomDesc[] = {
	{ "g_terme.bin", 0x100000, 0x15f4d302, BRF_PRG },
};

STD_ROM_PICK(g_terme);
STD_ROM_FN(g_terme);

struct BurnDriverD BurnDrvMDg_terme = {
	"g_terme", "g_term", NULL, "1900",
	"Terminator, The (E) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_termeRomInfo, g_termeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Wolverine Adamantium Rage (UE) [!]
static struct BurnRomInfo g_wolvRomDesc[] = {
	{ "g_wolv.bin", 0x200000, 0xd2437bb7, BRF_PRG },
};

STD_ROM_PICK(g_wolv);
STD_ROM_FN(g_wolv);

struct BurnDriverD BurnDrvMDg_wolv = {
	"g_wolv", NULL, NULL, "1900",
	"Wolverine Adamantium Rage (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wolvRomInfo, g_wolvRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// AAAHH!!! Real Monsters (UE) [!]
static struct BurnRomInfo g_realRomDesc[] = {
	{ "g_real.bin", 0x200000, 0xFDC80BFC, BRF_PRG },
};

STD_ROM_PICK(g_real);
STD_ROM_FN(g_real);

struct BurnDriverD BurnDrvMDg_real = {
	"g_real", NULL, NULL, "1995",
	"AAAHH!!! Real Monsters (UE) [!]\0", "Sound Problems", "Viacom International Inc.", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_realRomInfo, g_realRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Barkley Shut Up and Jam! 2 (U) [!]
static struct BurnRomInfo g_suj2RomDesc[] = {
	{ "g_suj2.bin", 0x200000, 0x321bb6bd, BRF_PRG },
};

STD_ROM_PICK(g_suj2);
STD_ROM_FN(g_suj2);

struct BurnDriverD BurnDrvMDg_suj2 = {
	"g_suj2", NULL, NULL, "1900",
	"Barkley Shut Up and Jam! 2 (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_suj2RomInfo, g_suj2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Batman Returns (W) [!]
static struct BurnRomInfo g_batrRomDesc[] = {
	{ "g_batr.bin", 0x100000, 0x4a3225c0, BRF_PRG },
};

STD_ROM_PICK(g_batr);
STD_ROM_FN(g_batr);

struct BurnDriverD BurnDrvMDg_batr = {
	"g_batr", NULL, NULL, "1900",
	"Batman Returns (W) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_batrRomInfo, g_batrRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Battletoads (W) [!]
static struct BurnRomInfo g_btoaRomDesc[] = {
	{ "g_btoa.bin", 0x80000, 0xd10e103a, BRF_PRG },
};

STD_ROM_PICK(g_btoa);
STD_ROM_FN(g_btoa);

struct BurnDriverD BurnDrvMDg_btoa = {
	"g_btoa", NULL, NULL, "1900",
	"Battletoads (W) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_btoaRomInfo, g_btoaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Chester Cheetah 2 - Wild Wild Quest (U) [!]
static struct BurnRomInfo g_che2RomDesc[] = {
	{ "g_che2.bin", 0x100000, 0xb97b735d, BRF_PRG },
};

STD_ROM_PICK(g_che2);
STD_ROM_FN(g_che2);

struct BurnDriverD BurnDrvMDg_che2 = {
	"g_che2", NULL, NULL, "1900",
	"Chester Cheetah 2 - Wild Wild Quest (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_che2RomInfo, g_che2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Chester Cheetah (U) [c][!]
static struct BurnRomInfo g_cheRomDesc[] = {
	{ "g_che.bin", 0x100000, 0x250e3ec3, BRF_PRG },
};

STD_ROM_PICK(g_che);
STD_ROM_FN(g_che);

struct BurnDriverD BurnDrvMDg_che = {
	"g_che", NULL, NULL, "1900",
	"Chester Cheetah (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_cheRomInfo, g_cheRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Clay Fighter (U) [!]
static struct BurnRomInfo g_clayRomDesc[] = {
	{ "g_clay.bin", 0x200000, 0xb12c1bc1, BRF_PRG },
};

STD_ROM_PICK(g_clay);
STD_ROM_FN(g_clay);

struct BurnDriverD BurnDrvMDg_clay = {
	"g_clay", NULL, NULL, "1900",
	"Clay Fighter (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_clayRomInfo, g_clayRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Daffy Duck in Hollywood (E) (M5) [!]
static struct BurnRomInfo g_daffRomDesc[] = {
	{ "g_daff.bin", 0x200000, 0x1fdc66b0, BRF_PRG },
};

STD_ROM_PICK(g_daff);
STD_ROM_FN(g_daff);

struct BurnDriverD BurnDrvMDg_daff = {
	"g_daff", NULL, NULL, "1900",
	"Daffy Duck in Hollywood (E) (M5) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_daffRomInfo, g_daffRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Daffy Duck in Hollywood (Beta)
static struct BurnRomInfo g_daffybRomDesc[] = {
	{ "g_daffyb.bin", 0x200000, 0x0eaa4740, BRF_PRG },
};

STD_ROM_PICK(g_daffyb);
STD_ROM_FN(g_daffyb);

struct BurnDriverD BurnDrvMDg_daffyb = {
	"g_daffyb", "g_daff", NULL, "1900",
	"Daffy Duck in Hollywood (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_daffybRomInfo, g_daffybRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Davis Cup World Tour Tennis (UE) [!]
static struct BurnRomInfo g_daviRomDesc[] = {
	{ "g_davi.bin", 0x100000, 0x894686f0, BRF_PRG },
};

STD_ROM_PICK(g_davi);
STD_ROM_FN(g_davi);

struct BurnDriverD BurnDrvMDg_davi = {
	"g_davi", NULL, NULL, "1900",
	"Davis Cup World Tour Tennis (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_daviRomInfo, g_daviRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Desert Demolition (UE) [!]
static struct BurnRomInfo g_desdRomDesc[] = {
	{ "g_desd.bin", 0x100000, 0xc287343d, BRF_PRG },
};

STD_ROM_PICK(g_desd);
STD_ROM_FN(g_desd);

struct BurnDriverD BurnDrvMDg_desd = {
	"g_desd", NULL, NULL, "1900",
	"Desert Demolition (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_desdRomInfo, g_desdRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Desert Strike - Return to the Gulf (UE) [!]
static struct BurnRomInfo g_dstrRomDesc[] = {
	{ "g_dstr.bin", 0x100000, 0x67a9860b, BRF_PRG },
};

STD_ROM_PICK(g_dstr);
STD_ROM_FN(g_dstr);

struct BurnDriver BurnDrvMDg_dstr = {
	"g_dstr", NULL, NULL, "1900",
	"Desert Strike - Return to the Gulf (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_dstrRomInfo, g_dstrRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Doom Troopers - The Mutant Chronicles (U) [c][!]
static struct BurnRomInfo g_dtroRomDesc[] = {
	{ "g_dtro.bin", 0x200000, 0x11194414, BRF_PRG },
};

STD_ROM_PICK(g_dtro);
STD_ROM_FN(g_dtro);

struct BurnDriverD BurnDrvMDg_dtro = {
	"g_dtro", NULL, NULL, "1900",
	"Doom Troopers - The Mutant Chronicles (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_dtroRomInfo, g_dtroRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// ESPN SpeedWorld (U) [c][!]
static struct BurnRomInfo g_e_swRomDesc[] = {
	{ "g_e-sw.bin", 0x200000, 0xf50be478, BRF_PRG },
};

STD_ROM_PICK(g_e_sw);
STD_ROM_FN(g_e_sw);

struct BurnDriverD BurnDrvMDg_e_sw = {
	"g_e_sw", NULL, NULL, "1900",
	"ESPN SpeedWorld (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_e_swRomInfo, g_e_swRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// F1 World Championship Edition (E) [!]
static struct BurnRomInfo g_f1ceRomDesc[] = {
	{ "g_f1ce.bin", 0x200000, 0x74cee0a7, BRF_PRG },
};

STD_ROM_PICK(g_f1ce);
STD_ROM_FN(g_f1ce);

struct BurnDriverD BurnDrvMDg_f1ce = {
	"g_f1ce", NULL, NULL, "1900",
	"F1 World Championship Edition (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_f1ceRomInfo, g_f1ceRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// F1 World Championship Edition (Beta)
static struct BurnRomInfo g_f1wcebRomDesc[] = {
	{ "g_f1wceb.bin", 0x200000, 0x2269ed6b, BRF_PRG },
};

STD_ROM_PICK(g_f1wceb);
STD_ROM_FN(g_f1wceb);

struct BurnDriverD BurnDrvMDg_f1wceb = {
	"g_f1wceb", "g_f1ce", NULL, "1900",
	"F1 World Championship Edition (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_f1wcebRomInfo, g_f1wcebRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Flicky (UE) [!]
static struct BurnRomInfo g_flicRomDesc[] = {
	{ "g_flic.bin", 0x20000, 0x4291c8ab, BRF_PRG },
};

STD_ROM_PICK(g_flic);
STD_ROM_FN(g_flic);

struct BurnDriverD BurnDrvMDg_flic = {
	"g_flic", NULL, NULL, "1900",
	"Flicky (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_flicRomInfo, g_flicRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Hurricanes (E) [!]
static struct BurnRomInfo g_hurrRomDesc[] = {
	{ "g_hurr.bin", 0x200000, 0xdeccc874, BRF_PRG },
};

STD_ROM_PICK(g_hurr);
STD_ROM_FN(g_hurr);

struct BurnDriverD BurnDrvMDg_hurr = {
	"g_hurr", NULL, NULL, "1900",
	"Hurricanes (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_hurrRomInfo, g_hurrRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Izzy's Quest for the Olympic Rings (UE) [!]
static struct BurnRomInfo g_izzyRomDesc[] = {
	{ "g_izzy.bin", 0x200000, 0x77b416e4, BRF_PRG },
};

STD_ROM_PICK(g_izzy);
STD_ROM_FN(g_izzy);

struct BurnDriverD BurnDrvMDg_izzy = {
	"g_izzy", NULL, NULL, "1900",
	"Izzy's Quest for the Olympic Rings (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_izzyRomInfo, g_izzyRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Jungle Strike (UE) [!]
static struct BurnRomInfo g_jstrRomDesc[] = {
	{ "g_jstr.bin", 0x200000, 0xa5d29735, BRF_PRG },
};

STD_ROM_PICK(g_jstr);
STD_ROM_FN(g_jstr);

struct BurnDriverD BurnDrvMDg_jstr = {
	"g_jstr", NULL, NULL, "1900",
	"Jungle Strike (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jstrRomInfo, g_jstrRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Jungle Strike (J) [!]
static struct BurnRomInfo g_jstrjRomDesc[] = {
	{ "g_jstrj.bin", 0x200000, 0xba7a870b, BRF_PRG },
};

STD_ROM_PICK(g_jstrj);
STD_ROM_FN(g_jstrj);

struct BurnDriverD BurnDrvMDg_jstrj = {
	"g_jstrj", "g_jstr", NULL, "1900",
	"Jungle Strike (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jstrjRomInfo, g_jstrjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Jungle Strike (Beta)
static struct BurnRomInfo g_jstrkbRomDesc[] = {
	{ "g_jstrkb.bin", 0x200000, 0x0cd540d4, BRF_PRG },
};

STD_ROM_PICK(g_jstrkb);
STD_ROM_FN(g_jstrkb);

struct BurnDriverD BurnDrvMDg_jstrkb = {
	"g_jstrkb", "g_jstr", NULL, "1900",
	"Jungle Strike (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jstrkbRomInfo, g_jstrkbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Lawnmower Man, The (UE) [!]
static struct BurnRomInfo g_lawnRomDesc[] = {
	{ "g_lawn.bin", 0x100000, 0xa7cacd59, BRF_PRG },
};

STD_ROM_PICK(g_lawn);
STD_ROM_FN(g_lawn);

struct BurnDriverD BurnDrvMDg_lawn = {
	"g_lawn", NULL, NULL, "1900",
	"Lawnmower Man, The (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_lawnRomInfo, g_lawnRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Lemmings 2 - The Tribes (U) [!]
static struct BurnRomInfo g_lem2RomDesc[] = {
	{ "g_lem2.bin", 0x200000, 0xde59a3a3, BRF_PRG },
};

STD_ROM_PICK(g_lem2);
STD_ROM_FN(g_lem2);

struct BurnDriverD BurnDrvMDg_lem2 = {
	"g_lem2", NULL, NULL, "1900",
	"Lemmings 2 - The Tribes (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_lem2RomInfo, g_lem2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Lemmings 2 - The Tribes (E) [!]
static struct BurnRomInfo g_lem2eRomDesc[] = {
	{ "g_lem2e.bin", 0x200000, 0x741eb624, BRF_PRG },
};

STD_ROM_PICK(g_lem2e);
STD_ROM_FN(g_lem2e);

struct BurnDriverD BurnDrvMDg_lem2e = {
	"g_lem2e", "g_lem2", NULL, "1900",
	"Lemmings 2 - The Tribes (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_lem2eRomInfo, g_lem2eRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Megapanel (J) [c][!]
static struct BurnRomInfo g_megpRomDesc[] = {
	{ "g_megp.bin", 0x40000, 0x6240f579, BRF_PRG },
};

STD_ROM_PICK(g_megp);
STD_ROM_FN(g_megp);

struct BurnDriverD BurnDrvMDg_megp = {
	"g_megp", NULL, NULL, "1900",
	"Megapanel (J) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_megpRomInfo, g_megpRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mighty Morphin Power Rangers (U) [!]
static struct BurnRomInfo g_mmprRomDesc[] = {
	{ "g_mmpr.bin", 0x200000, 0x715158a9, BRF_PRG },
};

STD_ROM_PICK(g_mmpr);
STD_ROM_FN(g_mmpr);

struct BurnDriverD BurnDrvMDg_mmpr = {
	"g_mmpr", NULL, NULL, "1900",
	"Mighty Morphin Power Rangers (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mmprRomInfo, g_mmprRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mighty Morphin Power Rangers (E) [!]
static struct BurnRomInfo g_mmpreRomDesc[] = {
	{ "g_mmpre.bin", 0x200000, 0x7f96e663, BRF_PRG },
};

STD_ROM_PICK(g_mmpre);
STD_ROM_FN(g_mmpre);

struct BurnDriverD BurnDrvMDg_mmpre = {
	"g_mmpre", "g_mmpr", NULL, "1900",
	"Mighty Morphin Power Rangers (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mmpreRomInfo, g_mmpreRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mighty Morphin Power Rangers - The Movie (U) [!]
static struct BurnRomInfo g_mmpmRomDesc[] = {
	{ "g_mmpm.bin", 0x200000, 0xaa941cbc, BRF_PRG },
};

STD_ROM_PICK(g_mmpm);
STD_ROM_FN(g_mmpm);

struct BurnDriverD BurnDrvMDg_mmpm = {
	"g_mmpm", NULL, NULL, "1900",
	"Mighty Morphin Power Rangers - The Movie (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mmpmRomInfo, g_mmpmRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mighty Morphin Power Rangers - The Movie (E) [!]
static struct BurnRomInfo g_mmprmeRomDesc[] = {
	{ "g_mmprme.bin", 0x200000, 0x254a4972, BRF_PRG },
};

STD_ROM_PICK(g_mmprme);
STD_ROM_FN(g_mmprme);

struct BurnDriverD BurnDrvMDg_mmprme = {
	"g_mmprme", "g_mmpm", NULL, "1900",
	"Mighty Morphin Power Rangers - The Movie (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mmprmeRomInfo, g_mmprmeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// NHL 98 (U) [c][!]
static struct BurnRomInfo g_nh98RomDesc[] = {
	{ "g_nh98.bin", 0x200000, 0x7b64cd98, BRF_PRG },
};

STD_ROM_PICK(g_nh98);
STD_ROM_FN(g_nh98);

struct BurnDriverD BurnDrvMDg_nh98 = {
	"g_nh98", NULL, NULL, "1900",
	"NHL 98 (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_nh98RomInfo, g_nh98RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Spider-Man and Venom - Separation Anxiety (U) [!]
static struct BurnRomInfo g_s_saRomDesc[] = {
	{ "g_s-sa.bin", 0x300000, 0x512ade32, BRF_PRG },
};

STD_ROM_PICK(g_s_sa);
STD_ROM_FN(g_s_sa);

struct BurnDriverD BurnDrvMDg_s_sa = {
	"g_s_sa", NULL, NULL, "1900",
	"Spider-Man and Venom - Separation Anxiety (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_s_saRomInfo, g_s_saRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Tick, The (U) [!]
static struct BurnRomInfo g_tickRomDesc[] = {
	{ "g_tick.bin", 0x200000, 0x425132f0, BRF_PRG },
};

STD_ROM_PICK(g_tick);
STD_ROM_FN(g_tick);

struct BurnDriverD BurnDrvMDg_tick = {
	"g_tick", NULL, NULL, "1900",
	"Tick, The (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tickRomInfo, g_tickRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Undead Line (J) [!]
static struct BurnRomInfo g_udedRomDesc[] = {
	{ "g_uded.bin", 0x100000, 0xfb3ca1e6, BRF_PRG },
};

STD_ROM_PICK(g_uded);
STD_ROM_FN(g_uded);

struct BurnDriverD BurnDrvMDg_uded = {
	"g_uded", NULL, NULL, "1900",
	"Undead Line (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_udedRomInfo, g_udedRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Urban Strike (UE) [!]
static struct BurnRomInfo g_ustrRomDesc[] = {
	{ "g_ustr.bin", 0x200000, 0xcf690a75, BRF_PRG },
};

STD_ROM_PICK(g_ustr);
STD_ROM_FN(g_ustr);

struct BurnDriverD BurnDrvMDg_ustr = {
	"g_ustr", NULL, NULL, "1900",
	"Urban Strike (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ustrRomInfo, g_ustrRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mutant League Hockey (UE) [!]
static struct BurnRomInfo g_muthRomDesc[] = {
	{ "g_muth.bin", 0x200000, 0x3529180f, BRF_PRG },
};

STD_ROM_PICK(g_muth);
STD_ROM_FN(g_muth);

struct BurnDriverD BurnDrvMDg_muth = {
	"g_muth", NULL, NULL, "1900",
	"Mutant League Hockey (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_muthRomInfo, g_muthRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Battletoads and Double Dragon (U) [c][!]
static struct BurnRomInfo g_btddRomDesc[] = {
	{ "g_btdd.bin", 0x100000, 0x8239dd17, BRF_PRG },
};

STD_ROM_PICK(g_btdd);
STD_ROM_FN(g_btdd);

struct BurnDriver BurnDrvMDg_btdd = {
	"g_btdd", NULL, NULL, "1900",
	"Battletoads and Double Dragon (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_btddRomInfo, g_btddRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Chakan - The Forever Man (UE) [!]
static struct BurnRomInfo g_chakRomDesc[] = {
	{ "g_chak.bin", 0x100000, 0x046a48de, BRF_PRG },
};

STD_ROM_PICK(g_chak);
STD_ROM_FN(g_chak);

struct BurnDriverD BurnDrvMDg_chak = {
	"g_chak", NULL, NULL, "1900",
	"Chakan - The Forever Man (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_chakRomInfo, g_chakRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Cutthroat Island (UE) [!]
static struct BurnRomInfo g_cuttRomDesc[] = {
	{ "g_cutt.bin", 0x200000, 0xebabbc70, BRF_PRG },
};

STD_ROM_PICK(g_cutt);
STD_ROM_FN(g_cutt);

struct BurnDriverD BurnDrvMDg_cutt = {
	"g_cutt", NULL, NULL, "1900",
	"Cutthroat Island (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_cuttRomInfo, g_cuttRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Dangerous Seed (J) [!]
static struct BurnRomInfo g_dangRomDesc[] = {
	{ "g_dang.bin", 0x80000, 0xa2990031, BRF_PRG },
};

STD_ROM_PICK(g_dang);
STD_ROM_FN(g_dang);

struct BurnDriverD BurnDrvMDg_dang = {
	"g_dang", NULL, NULL, "1900",
	"Dangerous Seed (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_dangRomInfo, g_dangRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Blades of Vengeance (UE) [!]
static struct BurnRomInfo g_blavRomDesc[] = {
	{ "g_blav.bin", 0x100000, 0x74c65a49, BRF_PRG },
};

STD_ROM_PICK(g_blav);
STD_ROM_FN(g_blav);

struct BurnDriverD BurnDrvMDg_blav = {
	"g_blav", NULL, NULL, "1900",
	"Blades of Vengeance (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_blavRomInfo, g_blavRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Death Duel (U) [!]
static struct BurnRomInfo g_duelRomDesc[] = {
	{ "g_duel.bin", 0x100000, 0xa9804dcc, BRF_PRG },
};

STD_ROM_PICK(g_duel);
STD_ROM_FN(g_duel);

struct BurnDriverD BurnDrvMDg_duel = {
	"g_duel", NULL, NULL, "1900",
	"Death Duel (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_duelRomInfo, g_duelRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Demolition Man (UE) [!]
static struct BurnRomInfo g_demoRomDesc[] = {
	{ "g_demo.bin", 0x200000, 0x5ff71877, BRF_PRG },
};

STD_ROM_PICK(g_demo);
STD_ROM_FN(g_demo);

struct BurnDriverD BurnDrvMDg_demo = {
	"g_demo", NULL, NULL, "1900",
	"Demolition Man (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_demoRomInfo, g_demoRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Dick Tracy (W) [c][!]
static struct BurnRomInfo g_dickRomDesc[] = {
	{ "g_dick.bin", 0x80000, 0xef887533, BRF_PRG },
};

STD_ROM_PICK(g_dick);
STD_ROM_FN(g_dick);

struct BurnDriverD BurnDrvMDg_dick = {
	"g_dick", NULL, NULL, "1900",
	"Dick Tracy (W) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_dickRomInfo, g_dickRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Doraemon - Yume Dorobouto 7 Nin No Gozansu (J) [!]
static struct BurnRomInfo g_doraRomDesc[] = {
	{ "g_dora.bin", 0x80000, 0xeeed1130, BRF_PRG },
};

STD_ROM_PICK(g_dora);
STD_ROM_FN(g_dora);

struct BurnDriverD BurnDrvMDg_dora = {
	"g_dora", NULL, NULL, "1900",
	"Doraemon - Yume Dorobouto 7 Nin No Gozansu (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_doraRomInfo, g_doraRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Duke Nukem 3D (B) [!]
static struct BurnRomInfo g_dn3dRomDesc[] = {
	{ "g_dn3d.bin", 0x400000, 0x6bd2accb, BRF_PRG },
};

STD_ROM_PICK(g_dn3d);
STD_ROM_FN(g_dn3d);

struct BurnDriverD BurnDrvMDg_dn3d = {
	"g_dn3d", NULL, NULL, "1900",
	"Duke Nukem 3D (B) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_dn3dRomInfo, g_dn3dRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// El Viento (U) [!]
static struct BurnRomInfo g_elviRomDesc[] = {
	{ "g_elvi.bin", 0x100000, 0x070a1ceb, BRF_PRG },
};

STD_ROM_PICK(g_elvi);
STD_ROM_FN(g_elvi);

struct BurnDriverD BurnDrvMDg_elvi = {
	"g_elvi", NULL, NULL, "1900",
	"El Viento (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_elviRomInfo, g_elviRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// El Viento (J) [!]
static struct BurnRomInfo g_elvinjRomDesc[] = {
	{ "g_elvinj.bin", 0x100000, 0x6091c36e, BRF_PRG },
};

STD_ROM_PICK(g_elvinj);
STD_ROM_FN(g_elvinj);

struct BurnDriverD BurnDrvMDg_elvinj = {
	"g_elvinj", "g_elvi", NULL, "1900",
	"El Viento (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_elvinjRomInfo, g_elvinjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Final Zone (JU) [!]
static struct BurnRomInfo g_fzonRomDesc[] = {
	{ "g_fzon.bin", 0x80000, 0x731fa4a1, BRF_PRG },
};

STD_ROM_PICK(g_fzon);
STD_ROM_FN(g_fzon);

struct BurnDriverD BurnDrvMDg_fzon = {
	"g_fzon", NULL, NULL, "1900",
	"Final Zone (JU) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_fzonRomInfo, g_fzonRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Generations Lost (UE) [!]
static struct BurnRomInfo g_glosRomDesc[] = {
	{ "g_glos.bin", 0x100000, 0x131f36a6, BRF_PRG },
};

STD_ROM_PICK(g_glos);
STD_ROM_FN(g_glos);

struct BurnDriverD BurnDrvMDg_glos = {
	"g_glos", NULL, NULL, "1900",
	"Generations Lost (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_glosRomInfo, g_glosRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Home Alone (U) [!]
static struct BurnRomInfo g_homeRomDesc[] = {
	{ "g_home.bin", 0x80000, 0xaa0d4387, BRF_PRG },
};

STD_ROM_PICK(g_home);
STD_ROM_FN(g_home);

struct BurnDriverD BurnDrvMDg_home = {
	"g_home", NULL, NULL, "1900",
	"Home Alone (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_homeRomInfo, g_homeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Home Alone (Beta)
static struct BurnRomInfo g_homeabRomDesc[] = {
	{ "g_homeab.bin", 0x100000, 0x3a235fb9, BRF_PRG },
};

STD_ROM_PICK(g_homeab);
STD_ROM_FN(g_homeab);

struct BurnDriverD BurnDrvMDg_homeab = {
	"g_homeab", "g_home", NULL, "1900",
	"Home Alone (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_homeabRomInfo, g_homeabRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Home Alone 2 - Lost in New York (U) [!]
static struct BurnRomInfo g_hom2RomDesc[] = {
	{ "g_hom2.bin", 0x80000, 0xcbf87c14, BRF_PRG },
};

STD_ROM_PICK(g_hom2);
STD_ROM_FN(g_hom2);

struct BurnDriverD BurnDrvMDg_hom2 = {
	"g_hom2", NULL, NULL, "1900",
	"Home Alone 2 - Lost in New York (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_hom2RomInfo, g_hom2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Immortal, The (UE) [!]
static struct BurnRomInfo g_immoRomDesc[] = {
	{ "g_immo.bin", 0x100000, 0xf653c508, BRF_PRG },
};

STD_ROM_PICK(g_immo);
STD_ROM_FN(g_immo);

struct BurnDriverD BurnDrvMDg_immo = {
	"g_immo", NULL, NULL, "1900",
	"Immortal, The (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_immoRomInfo, g_immoRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Immortal, The (J) [!]
static struct BurnRomInfo g_immorjRomDesc[] = {
	{ "g_immorj.bin", 0x200000, 0xc99fad92, BRF_PRG },
};

STD_ROM_PICK(g_immorj);
STD_ROM_FN(g_immorj);

struct BurnDriverD BurnDrvMDg_immorj = {
	"g_immorj", "g_immo", NULL, "1900",
	"Immortal, The (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_immorjRomInfo, g_immorjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Incredible Hulk, The (UE) [!]
static struct BurnRomInfo g_hulkRomDesc[] = {
	{ "g_hulk.bin", 0x200000, 0x84a5a2dc, BRF_PRG },
};

STD_ROM_PICK(g_hulk);
STD_ROM_FN(g_hulk);

struct BurnDriverD BurnDrvMDg_hulk = {
	"g_hulk", NULL, NULL, "1900",
	"Incredible Hulk, The (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_hulkRomInfo, g_hulkRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Junction (JU) [!]
static struct BurnRomInfo g_juncRomDesc[] = {
	{ "g_junc.bin", 0x80000, 0x94cdce8d, BRF_PRG },
};

STD_ROM_PICK(g_junc);
STD_ROM_FN(g_junc);

struct BurnDriverD BurnDrvMDg_junc = {
	"g_junc", NULL, NULL, "1900",
	"Junction (JU) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_juncRomInfo, g_juncRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Incredible Crash Dummies, The (UE) [c][!]
static struct BurnRomInfo g_icdRomDesc[] = {
	{ "g_icd.bin", 0x100000, 0x1f6e574a, BRF_PRG },
};

STD_ROM_PICK(g_icd);
STD_ROM_FN(g_icd);

struct BurnDriverD BurnDrvMDg_icd = {
	"g_icd", NULL, NULL, "1900",
	"Incredible Crash Dummies, The (UE) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_icdRomInfo, g_icdRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Incredible Crash Dummies, The (Beta)
static struct BurnRomInfo g_icdbRomDesc[] = {
	{ "g_icdb.bin", 0x100000, 0x623a920f, BRF_PRG },
};

STD_ROM_PICK(g_icdb);
STD_ROM_FN(g_icdb);

struct BurnDriverD BurnDrvMDg_icdb = {
	"g_icdb", "g_icd", NULL, "1900",
	"Incredible Crash Dummies, The (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_icdbRomInfo, g_icdbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mary Shelley's Frankenstein (U) [!]
static struct BurnRomInfo g_franRomDesc[] = {
	{ "g_fran.bin", 0x200000, 0x48993dc3, BRF_PRG },
};

STD_ROM_PICK(g_fran);
STD_ROM_FN(g_fran);

struct BurnDriverD BurnDrvMDg_fran = {
	"g_fran", NULL, NULL, "1900",
	"Mary Shelley's Frankenstein (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_franRomInfo, g_franRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Onslaught (U) [c][!]
static struct BurnRomInfo g_onslRomDesc[] = {
	{ "g_onsl.bin", 0x80000, 0x9f19d6df, BRF_PRG },
};

STD_ROM_PICK(g_onsl);
STD_ROM_FN(g_onsl);

struct BurnDriverD BurnDrvMDg_onsl = {
	"g_onsl", NULL, NULL, "1900",
	"Onslaught (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_onslRomInfo, g_onslRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Pengo (J) [!]
static struct BurnRomInfo g_pengRomDesc[] = {
	{ "g_peng.bin", 0x100000, 0xd1e2324b, BRF_PRG },
};

STD_ROM_PICK(g_peng);
STD_ROM_FN(g_peng);

struct BurnDriverD BurnDrvMDg_peng = {
	"g_peng", NULL, NULL, "1900",
	"Pengo (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pengRomInfo, g_pengRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Rainbow Islands - The Story of Bubble Bobble 2 (J) [c][!]
static struct BurnRomInfo g_rainRomDesc[] = {
	{ "g_rain.bin", 0x80000, 0xc74dcb35, BRF_PRG },
};

STD_ROM_PICK(g_rain);
STD_ROM_FN(g_rain);

struct BurnDriverD BurnDrvMDg_rain = {
	"g_rain", NULL, NULL, "1900",
	"Rainbow Islands - The Story of Bubble Bobble 2 (J) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_rainRomInfo, g_rainRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Simpsons, The - Bart's Nightmare (UE) (REV02) [!]
static struct BurnRomInfo g_btnmRomDesc[] = {
	{ "g_btnm.bin", 0x100000, 0x24d7507c, BRF_PRG },
};

STD_ROM_PICK(g_btnm);
STD_ROM_FN(g_btnm);

struct BurnDriverD BurnDrvMDg_btnm = {
	"g_btnm", NULL, NULL, "1900",
	"Simpsons, The - Bart's Nightmare (UE) (REV02) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_btnmRomInfo, g_btnmRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Smurfs, The (E) (M5) [!]
static struct BurnRomInfo g_smfRomDesc[] = {
	{ "g_smf.bin", 0x100000, 0x88b30eff, BRF_PRG },
};

STD_ROM_PICK(g_smf);
STD_ROM_FN(g_smf);

struct BurnDriverD BurnDrvMDg_smf = {
	"g_smf", NULL, NULL, "1900",
	"Smurfs, The (E) (M5) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_smfRomInfo, g_smfRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Smurfs 2, The (E) (M4) [!]
static struct BurnRomInfo g_smf2RomDesc[] = {
	{ "g_smf2.bin", 0x100000, 0xb28bdd69, BRF_PRG },
};

STD_ROM_PICK(g_smf2);
STD_ROM_FN(g_smf2);

struct BurnDriverD BurnDrvMDg_smf2 = {
	"g_smf2", NULL, NULL, "1900",
	"Smurfs 2, The (E) (M4) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_smf2RomInfo, g_smf2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Spider-Man and Venom - Maximum Carnage (W) [!]
static struct BurnRomInfo g_s_mcRomDesc[] = {
	{ "g_s_mc.bin", 0x200000, 0x8FA0B6E6, BRF_PRG },
};

STD_ROM_PICK(g_s_mc);
STD_ROM_FN(g_s_mc);

struct BurnDriverD BurnDrvMDg_s_mc = {
	"g_s_mc", NULL, NULL, "1900",
	"Spider-Man and Venom - Maximum Carnage (W) [!]\0", "Sound Problems", "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_s_mcRomInfo, g_s_mcRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Stargate (UE) [!]
static struct BurnRomInfo g_sgatRomDesc[] = {
	{ "g_sgat.bin", 0x200000, 0xe587069e, BRF_PRG },
};

STD_ROM_PICK(g_sgat);
STD_ROM_FN(g_sgat);

struct BurnDriverD BurnDrvMDg_sgat = {
	"g_sgat", NULL, NULL, "1900",
	"Stargate (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sgatRomInfo, g_sgatRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Stargate (Beta)
static struct BurnRomInfo g_sgatbRomDesc[] = {
	{ "g_sgatb.bin", 0x200000, 0x8dc8ab23, BRF_PRG },
};

STD_ROM_PICK(g_sgatb);
STD_ROM_FN(g_sgatb);

struct BurnDriverD BurnDrvMDg_sgatb = {
	"g_sgatb", "g_sgat", NULL, "1900",
	"Stargate (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sgatbRomInfo, g_sgatbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Task Force Harrier EX (U) [!]
static struct BurnRomInfo g_tfhRomDesc[] = {
	{ "g_tfh.bin", 0x100000, 0xc8bb0257, BRF_PRG },
};

STD_ROM_PICK(g_tfh);
STD_ROM_FN(g_tfh);

struct BurnDriverD BurnDrvMDg_tfh = {
	"g_tfh", NULL, NULL, "1900",
	"Task Force Harrier EX (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tfhRomInfo, g_tfhRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Task Force Harrier EX (J) [!]
static struct BurnRomInfo g_tfhxjRomDesc[] = {
	{ "g_tfhxj.bin", 0x100000, 0xe9a54eed, BRF_PRG },
};

STD_ROM_PICK(g_tfhxj);
STD_ROM_FN(g_tfhxj);

struct BurnDriverD BurnDrvMDg_tfhxj = {
	"g_tfhxj", "g_tfh", NULL, "1900",
	"Task Force Harrier EX (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tfhxjRomInfo, g_tfhxjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Thunder Force III (JU) [!]
static struct BurnRomInfo g_tf3RomDesc[] = {
	{ "g_tf3.bin", 0x80000, 0x1b3f399a, BRF_PRG },
};

STD_ROM_PICK(g_tf3);
STD_ROM_FN(g_tf3);

struct BurnDriverD BurnDrvMDg_tf3 = {
	"g_tf3", NULL, NULL, "1900",
	"Thunder Force III (JU) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tf3RomInfo, g_tf3RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Trampoline Terror! (U) [!]
static struct BurnRomInfo g_tterRomDesc[] = {
	{ "g_tter.bin", 0x40000, 0xaabb349f, BRF_PRG },
};

STD_ROM_PICK(g_tter);
STD_ROM_FN(g_tter);

struct BurnDriverD BurnDrvMDg_tter = {
	"g_tter", NULL, NULL, "1900",
	"Trampoline Terror! (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tterRomInfo, g_tterRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Universal Soldier (U) [!]
static struct BurnRomInfo g_unisRomDesc[] = {
	{ "g_unis.bin", 0x100000, 0x352ebd49, BRF_PRG },
};

STD_ROM_PICK(g_unis);
STD_ROM_FN(g_unis);

struct BurnDriverD BurnDrvMDg_unis = {
	"g_unis", NULL, NULL, "1900",
	"Universal Soldier (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_unisRomInfo, g_unisRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Wayne's World (U) [c][!]
static struct BurnRomInfo g_waynRomDesc[] = {
	{ "g_wayn.bin", 0x100000, 0xd2cf6ebe, BRF_PRG },
};

STD_ROM_PICK(g_wayn);
STD_ROM_FN(g_wayn);

struct BurnDriverD BurnDrvMDg_wayn = {
	"g_wayn", NULL, NULL, "1900",
	"Wayne's World (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_waynRomInfo, g_waynRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Yogi Bear's Cartoon Capers (E) [!]
static struct BurnRomInfo g_yogiRomDesc[] = {
	{ "g_yogi.bin", 0x100000, 0x204f97d8, BRF_PRG },
};

STD_ROM_PICK(g_yogi);
STD_ROM_FN(g_yogi);

struct BurnDriverD BurnDrvMDg_yogi = {
	"g_yogi", NULL, NULL, "1900",
	"Yogi Bear's Cartoon Capers (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_yogiRomInfo, g_yogiRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Super Off Road (U) [!]
static struct BurnRomInfo g_soffRomDesc[] = {
	{ "g_soff.bin", 0x80000, 0x8f2fdada, BRF_PRG },
};

STD_ROM_PICK(g_soff);
STD_ROM_FN(g_soff);

struct BurnDriverD BurnDrvMDg_soff = {
	"g_soff", NULL, NULL, "1900",
	"Super Off Road (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_soffRomInfo, g_soffRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Double Dragon (U) [!]
static struct BurnRomInfo g_ddrRomDesc[] = {
	{ "g_ddr.bin", 0x80000, 0x054f5d53, BRF_PRG },
};

STD_ROM_PICK(g_ddr);
STD_ROM_FN(g_ddr);

struct BurnDriverD BurnDrvMDg_ddr = {
	"g_ddr", NULL, NULL, "1900",
	"Double Dragon (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ddrRomInfo, g_ddrRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Double Dragon 2 - The Revenge (J) [!]
static struct BurnRomInfo g_ddr2RomDesc[] = {
	{ "g_ddr2.bin", 0x80000, 0xa8bfdbd6, BRF_PRG },
};

STD_ROM_PICK(g_ddr2);
STD_ROM_FN(g_ddr2);

struct BurnDriverD BurnDrvMDg_ddr2 = {
	"g_ddr2", NULL, NULL, "1900",
	"Double Dragon 2 - The Revenge (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ddr2RomInfo, g_ddr2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Double Dragon 3 - The Rosetta Stone (UE) [!]
static struct BurnRomInfo g_ddr3RomDesc[] = {
	{ "g_ddr3.bin", 0x100000, 0xb36ab75c, BRF_PRG },
};

STD_ROM_PICK(g_ddr3);
STD_ROM_FN(g_ddr3);

struct BurnDriverD BurnDrvMDg_ddr3 = {
	"g_ddr3", NULL, NULL, "1900",
	"Double Dragon 3 - The Rosetta Stone (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ddr3RomInfo, g_ddr3RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Double Dragon V - The Shadow Falls (U) [!]
static struct BurnRomInfo g_ddrvRomDesc[] = {
	{ "g_ddrv.bin", 0x300000, 0x27e59e35, BRF_PRG },
};

STD_ROM_PICK(g_ddrv);
STD_ROM_FN(g_ddrv);

struct BurnDriverD BurnDrvMDg_ddrv = {
	"g_ddrv", NULL, NULL, "1900",
	"Double Dragon V - The Shadow Falls (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ddrvRomInfo, g_ddrvRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Fido Dido (Prototype)
static struct BurnRomInfo g_fidoRomDesc[] = {
	{ "g_fido.bin", 0x100000, 0xc6d4a240, BRF_PRG },
};

STD_ROM_PICK(g_fido);
STD_ROM_FN(g_fido);

struct BurnDriverD BurnDrvMDg_fido = {
	"g_fido", NULL, NULL, "1900",
	"Fido Dido (Prototype)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_fidoRomInfo, g_fidoRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// TNN Bass Tournament of Champions (U) (REV01) [!]
static struct BurnRomInfo g_tnnbRomDesc[] = {
	{ "g_tnnb.bin", 0x100000, 0xc83ffa1b, BRF_PRG },
};

STD_ROM_PICK(g_tnnb);
STD_ROM_FN(g_tnnb);

struct BurnDriverD BurnDrvMDg_tnnb = {
	"g_tnnb", NULL, NULL, "1900",
	"TNN Bass Tournament of Champions (U) (REV01) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tnnbRomInfo, g_tnnbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// TNN Outdoors Bass Tournament '96 (U) [!]
static struct BurnRomInfo g_tnnoRomDesc[] = {
	{ "g_tnno.bin", 0x200000, 0x5c523c0b, BRF_PRG },
};

STD_ROM_PICK(g_tnno);
STD_ROM_FN(g_tnno);

struct BurnDriverD BurnDrvMDg_tnno = {
	"g_tnno", NULL, NULL, "1900",
	"TNN Outdoors Bass Tournament '96 (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tnnoRomInfo, g_tnnoRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sesame Street Counting Cafe (U) [!]
static struct BurnRomInfo g_ssccRomDesc[] = {
	{ "g_sscc.bin", 0x100000, 0x0a4f48c3, BRF_PRG },
};

STD_ROM_PICK(g_sscc);
STD_ROM_FN(g_sscc);

struct BurnDriverD BurnDrvMDg_sscc = {
	"g_sscc", NULL, NULL, "1900",
	"Sesame Street Counting Cafe (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ssccRomInfo, g_ssccRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Batman - Revenge of the Joker (U) [!]
static struct BurnRomInfo g_batjRomDesc[] = {
	{ "g_batj.bin", 0x100000, 0xcaa044a1, BRF_PRG },
};

STD_ROM_PICK(g_batj);
STD_ROM_FN(g_batj);

struct BurnDriverD BurnDrvMDg_batj = {
	"g_batj", NULL, NULL, "1900",
	"Batman - Revenge of the Joker (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_batjRomInfo, g_batjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Battle Squadron (UE) [!]
static struct BurnRomInfo g_bsquRomDesc[] = {
	{ "g_bsqu.bin", 0x80000, 0x0feaa8bf, BRF_PRG },
};

STD_ROM_PICK(g_bsqu);
STD_ROM_FN(g_bsqu);

struct BurnDriver BurnDrvMDg_bsqu = {
	"g_bsqu", NULL, NULL, "1900",
	"Battle Squadron (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_bsquRomInfo, g_bsquRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Earnest Evans (U) [!]
static struct BurnRomInfo g_earnRomDesc[] = {
	{ "g_earn.bin", 0x100000, 0xa243816d, BRF_PRG },
};

STD_ROM_PICK(g_earn);
STD_ROM_FN(g_earn);

struct BurnDriverD BurnDrvMDg_earn = {
	"g_earn", NULL, NULL, "1900",
	"Earnest Evans (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_earnRomInfo, g_earnRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// F-117 Night Storm (UE) [!]
static struct BurnRomInfo g_f117RomDesc[] = {
	{ "g_f117.bin", 0x200000, 0x1bf67a07, BRF_PRG },
};

STD_ROM_PICK(g_f117);
STD_ROM_FN(g_f117);

struct BurnDriverD BurnDrvMDg_f117 = {
	"g_f117", NULL, NULL, "1900",
	"F-117 Night Storm (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_f117RomInfo, g_f117RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// F-117 Stealth - Operation Night Storm (J) [!]
static struct BurnRomInfo g_f117jRomDesc[] = {
	{ "g_f117j.bin", 0x200000, 0xea6e421a, BRF_PRG },
};

STD_ROM_PICK(g_f117j);
STD_ROM_FN(g_f117j);

struct BurnDriverD BurnDrvMDg_f117j = {
	"g_f117j", "g_f117", NULL, "1900",
	"F-117 Stealth - Operation Night Storm (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_f117jRomInfo, g_f117jRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Frogger (U) [!]
static struct BurnRomInfo g_frogRomDesc[] = {
	{ "g_frog.bin", 0x80000, 0xea2e48c0, BRF_PRG },
};

STD_ROM_PICK(g_frog);
STD_ROM_FN(g_frog);

struct BurnDriverD BurnDrvMDg_frog = {
	"g_frog", NULL, NULL, "1900",
	"Frogger (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_frogRomInfo, g_frogRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// G-LOC Air Battle (W) [c][!]
static struct BurnRomInfo g_glocRomDesc[] = {
	{ "g_gloc.bin", 0x100000, 0xf2af886e, BRF_PRG },
};

STD_ROM_PICK(g_gloc);
STD_ROM_FN(g_gloc);

struct BurnDriverD BurnDrvMDg_gloc = {
	"g_gloc", NULL, NULL, "1900",
	"G-LOC Air Battle (W) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_glocRomInfo, g_glocRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// G-LOC Air Battle (Beta)
static struct BurnRomInfo g_glocbRomDesc[] = {
	{ "g_glocb.bin", 0x100000, 0x175c7e63, BRF_PRG },
};

STD_ROM_PICK(g_glocb);
STD_ROM_FN(g_glocb);

struct BurnDriverD BurnDrvMDg_glocb = {
	"g_glocb", "g_gloc", NULL, "1900",
	"G-LOC Air Battle (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_glocbRomInfo, g_glocbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Gunship (E) [c][!]
static struct BurnRomInfo g_gshiRomDesc[] = {
	{ "g_gshi.bin", 0x100000, 0xda1440c9, BRF_PRG },
};

STD_ROM_PICK(g_gshi);
STD_ROM_FN(g_gshi);

struct BurnDriverD BurnDrvMDg_gshi = {
	"g_gshi", NULL, NULL, "1900",
	"Gunship (E) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_gshiRomInfo, g_gshiRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Last Action Hero (U) [!]
static struct BurnRomInfo g_lastRomDesc[] = {
	{ "g_last.bin", 0x100000, 0x15357dde, BRF_PRG },
};

STD_ROM_PICK(g_last);
STD_ROM_FN(g_last);

struct BurnDriverD BurnDrvMDg_last = {
	"g_last", NULL, NULL, "1900",
	"Last Action Hero (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_lastRomInfo, g_lastRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Ms. Pac-Man (U) [!]
static struct BurnRomInfo g_mpacRomDesc[] = {
	{ "g_mpac.bin", 0x20000, 0xaf041be6, BRF_PRG },
};

STD_ROM_PICK(g_mpac);
STD_ROM_FN(g_mpac);

struct BurnDriverD BurnDrvMDg_mpac = {
	"g_mpac", NULL, NULL, "1900",
	"Ms. Pac-Man (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mpacRomInfo, g_mpacRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Nightmare Circus (B) [!]
static struct BurnRomInfo g_ncirRomDesc[] = {
	{ "g_ncir.bin", 0x200000, 0x06da3217, BRF_PRG },
};

STD_ROM_PICK(g_ncir);
STD_ROM_FN(g_ncir);

struct BurnDriverD BurnDrvMDg_ncir = {
	"g_ncir", NULL, NULL, "1900",
	"Nightmare Circus (B) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ncirRomInfo, g_ncirRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Nightmare Circus (Beta)
static struct BurnRomInfo g_ncircbRomDesc[] = {
	{ "g_ncircb.bin", 0x200000, 0x31de5a94, BRF_PRG },
};

STD_ROM_PICK(g_ncircb);
STD_ROM_FN(g_ncircb);

struct BurnDriverD BurnDrvMDg_ncircb = {
	"g_ncircb", "g_ncir", NULL, "1900",
	"Nightmare Circus (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ncircbRomInfo, g_ncircbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Pac-Mania (U) [!]
static struct BurnRomInfo g_pacmRomDesc[] = {
	{ "g_pacm.bin", 0x40000, 0x74bba09b, BRF_PRG },
};

STD_ROM_PICK(g_pacm);
STD_ROM_FN(g_pacm);

struct BurnDriverD BurnDrvMDg_pacm = {
	"g_pacm", NULL, NULL, "1900",
	"Pac-Mania (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pacmRomInfo, g_pacmRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Power Drive (E) (M5) [!]
static struct BurnRomInfo g_pdriRomDesc[] = {
	{ "g_pdri.bin", 0x100000, 0x8c00ad61, BRF_PRG },
};

STD_ROM_PICK(g_pdri);
STD_ROM_FN(g_pdri);

struct BurnDriverD BurnDrvMDg_pdri = {
	"g_pdri", NULL, NULL, "1900",
	"Power Drive (E) (M5) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pdriRomInfo, g_pdriRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Shadow of the Beast 2 (UE) [!]
static struct BurnRomInfo g_sbe2RomDesc[] = {
	{ "g_sbe2.bin", 0x100000, 0x2dede3db, BRF_PRG },
};

STD_ROM_PICK(g_sbe2);
STD_ROM_FN(g_sbe2);

struct BurnDriverD BurnDrvMDg_sbe2 = {
	"g_sbe2", NULL, NULL, "1900",
	"Shadow of the Beast 2 (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sbe2RomInfo, g_sbe2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Space Invaders 91 (U) [c][!]
static struct BurnRomInfo g_si91RomDesc[] = {
	{ "g_si91.bin", 0x40000, 0xbb83b528, BRF_PRG },
};

STD_ROM_PICK(g_si91);
STD_ROM_FN(g_si91);

struct BurnDriverD BurnDrvMDg_si91 = {
	"g_si91", NULL, NULL, "1900",
	"Space Invaders 91 (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_si91RomInfo, g_si91RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Space Invaders 90 (J) [c][!]
static struct BurnRomInfo g_si90RomDesc[] = {
	{ "g_si90.bin", 0x40000, 0x22adbd66, BRF_PRG },
};

STD_ROM_PICK(g_si90);
STD_ROM_FN(g_si90);

struct BurnDriverD BurnDrvMDg_si90 = {
	"g_si90", "g_si91", NULL, "1900",
	"Space Invaders 90 (J) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_si90RomInfo, g_si90RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Spider-Man and X-Men - Arcade's Revenge (UE) [!]
static struct BurnRomInfo g_s_arRomDesc[] = {
	{ "g_s_ar.bin", 0x100000, 0x4a4414ea, BRF_PRG },
};

STD_ROM_PICK(g_s_ar);
STD_ROM_FN(g_s_ar);

struct BurnDriverD BurnDrvMDg_s_ar = {
	"g_s_ar", NULL, NULL, "1900",
	"Spider-Man and X-Men - Arcade's Revenge (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_s_arRomInfo, g_s_arRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Super Smash TV (UE) [!]
static struct BurnRomInfo g_sstvRomDesc[] = {
	{ "g_sstv.bin", 0x80000, 0xf22412b6, BRF_PRG },
};

STD_ROM_PICK(g_sstv);
STD_ROM_FN(g_sstv);

struct BurnDriverD BurnDrvMDg_sstv = {
	"g_sstv", NULL, NULL, "1900",
	"Super Smash TV (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sstvRomInfo, g_sstvRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sol-Deace (U) [!]
static struct BurnRomInfo g_soldRomDesc[] = {
	{ "g_sold.bin", 0x100000, 0xa77e4e9f, BRF_PRG },
};

STD_ROM_PICK(g_sold);
STD_ROM_FN(g_sold);

struct BurnDriverD BurnDrvMDg_sold = {
	"g_sold", NULL, NULL, "1900",
	"Sol-Deace (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_soldRomInfo, g_soldRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Terminator 2 - Judgment Day (UE) [!]
static struct BurnRomInfo g_ter2RomDesc[] = {
	{ "g_ter2.bin", 0x100000, 0x2f75e896, BRF_PRG },
};

STD_ROM_PICK(g_ter2);
STD_ROM_FN(g_ter2);

struct BurnDriverD BurnDrvMDg_ter2 = {
	"g_ter2", NULL, NULL, "1900",
	"Terminator 2 - Judgment Day (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ter2RomInfo, g_ter2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Thunder Force II (U) [!]
static struct BurnRomInfo g_tf2RomDesc[] = {
	{ "g_tf2.bin", 0x80000, 0x9b1561b3, BRF_PRG },
};

STD_ROM_PICK(g_tf2);
STD_ROM_FN(g_tf2);

struct BurnDriverD BurnDrvMDg_tf2 = {
	"g_tf2", NULL, NULL, "1900",
	"Thunder Force II (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tf2RomInfo, g_tf2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Thunder Force II MD (J) [!]
static struct BurnRomInfo g_tf2mdRomDesc[] = {
	{ "g_tf2md.bin", 0x80000, 0xe75ec3e0, BRF_PRG },
};

STD_ROM_PICK(g_tf2md);
STD_ROM_FN(g_tf2md);

struct BurnDriverD BurnDrvMDg_tf2md = {
	"g_tf2md", "g_tf2", NULL, "1900",
	"Thunder Force II MD (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tf2mdRomInfo, g_tf2mdRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Toys (U) [!]
static struct BurnRomInfo g_toysRomDesc[] = {
	{ "g_toys.bin", 0x100000, 0xcbc9951b, BRF_PRG },
};

STD_ROM_PICK(g_toys);
STD_ROM_FN(g_toys);

struct BurnDriverD BurnDrvMDg_toys = {
	"g_toys", NULL, NULL, "1900",
	"Toys (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_toysRomInfo, g_toysRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// True Lies (W) [!]
static struct BurnRomInfo g_trueRomDesc[] = {
	{ "g_true.bin", 0x200000, 0x18c09468, BRF_PRG },
};

STD_ROM_PICK(g_true);
STD_ROM_FN(g_true);

struct BurnDriverD BurnDrvMDg_true = {
	"g_true", NULL, NULL, "1900",
	"True Lies (W) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_trueRomInfo, g_trueRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Viewpoint (U) [!]
static struct BurnRomInfo g_viewRomDesc[] = {
	{ "g_view.bin", 0x200000, 0x59c71866, BRF_PRG },
};

STD_ROM_PICK(g_view);
STD_ROM_FN(g_view);

struct BurnDriverD BurnDrvMDg_view = {
	"g_view", NULL, NULL, "1900",
	"Viewpoint (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_viewRomInfo, g_viewRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Viewpoint (Beta)
static struct BurnRomInfo g_viewpbRomDesc[] = {
	{ "g_viewpb.bin", 0x180000, 0xf2e69ce7, BRF_PRG },
};

STD_ROM_PICK(g_viewpb);
STD_ROM_FN(g_viewpb);

struct BurnDriverD BurnDrvMDg_viewpb = {
	"g_viewpb", "g_view", NULL, "1900",
	"Viewpoint (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_viewpbRomInfo, g_viewpbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Worms (E) [!]
static struct BurnRomInfo g_wormRomDesc[] = {
	{ "g_worm.bin", 0x200000, 0xb9a8b299, BRF_PRG },
};

STD_ROM_PICK(g_worm);
STD_ROM_FN(g_worm);

struct BurnDriverD BurnDrvMDg_worm = {
	"g_worm", NULL, NULL, "1900",
	"Worms (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wormRomInfo, g_wormRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Worms (E) (Prototype) [c][!]
static struct BurnRomInfo g_wormpRomDesc[] = {
	{ "g_wormp.bin", 0x200000, 0x1d191694, BRF_PRG },
};

STD_ROM_PICK(g_wormp);
STD_ROM_FN(g_wormp);

struct BurnDriverD BurnDrvMDg_wormp = {
	"g_wormp", "g_worm", NULL, "1900",
	"Worms (E) (Prototype) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wormpRomInfo, g_wormpRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Xenon 2 - Megablast (E) [c][!]
static struct BurnRomInfo g_xen2RomDesc[] = {
	{ "g_xen2.bin", 0x80000, 0x59abe7f9, BRF_PRG },
};

STD_ROM_PICK(g_xen2);
STD_ROM_FN(g_xen2);

struct BurnDriverD BurnDrvMDg_xen2 = {
	"g_xen2", NULL, NULL, "1900",
	"Xenon 2 - Megablast (E) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_xen2RomInfo, g_xen2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Zoop (U) [!]
static struct BurnRomInfo g_zoopRomDesc[] = {
	{ "g_zoop.bin", 0x80000, 0xa899befa, BRF_PRG },
};

STD_ROM_PICK(g_zoop);
STD_ROM_FN(g_zoop);

struct BurnDriverD BurnDrvMDg_zoop = {
	"g_zoop", NULL, NULL, "1900",
	"Zoop (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_zoopRomInfo, g_zoopRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Zoop (E) [!]
static struct BurnRomInfo g_zoopeRomDesc[] = {
	{ "g_zoope.bin", 0x80000, 0x2fdac6ab, BRF_PRG },
};

STD_ROM_PICK(g_zoope);
STD_ROM_FN(g_zoope);

struct BurnDriverD BurnDrvMDg_zoope = {
	"g_zoope", "g_zoop", NULL, "1900",
	"Zoop (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_zoopeRomInfo, g_zoopeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Art Alive! (W) [!]
static struct BurnRomInfo g_artaRomDesc[] = {
	{ "g_arta.bin", 0x20000, 0xf1b72cdd, BRF_PRG },
};

STD_ROM_PICK(g_arta);
STD_ROM_FN(g_arta);

struct BurnDriver BurnDrvMDg_arta = {
	"g_arta", NULL, NULL, "1900",
	"Art Alive! (W) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_artaRomInfo, g_artaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Arcade Classics (U) [!]
static struct BurnRomInfo g_arcaRomDesc[] = {
	{ "g_arca.bin", 0x80000, 0x8aed2090, BRF_PRG },
};

STD_ROM_PICK(g_arca);
STD_ROM_FN(g_arca);

struct BurnDriverD BurnDrvMDg_arca = {
	"g_arca", NULL, NULL, "1900",
	"Arcade Classics (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_arcaRomInfo, g_arcaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Aah! Harimanada (J) [!]
static struct BurnRomInfo g_aahhRomDesc[] = {
	{ "g_aahh.bin", 0x200000, 0x065F6021, BRF_PRG },
};

STD_ROM_PICK(g_aahh);
STD_ROM_FN(g_aahh);

struct BurnDriver BurnDrvMDg_aahh = {
	"g_aahh", NULL, NULL, "1993",
	"Aah! Harimanada (J) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_VSFIGHT, 0,
	NULL, g_aahhRomInfo, g_aahhRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// AWS Pro Moves Soccer (U) [!]
static struct BurnRomInfo g_awspRomDesc[] = {
	{ "g_awsp.bin", 0x80000, 0x707017e5, BRF_PRG },
};

STD_ROM_PICK(g_awsp);
STD_ROM_FN(g_awsp);

struct BurnDriver BurnDrvMDg_awsp = {
	"g_awsp", NULL, NULL, "1900",
	"AWS Pro Moves Soccer (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_awspRomInfo, g_awspRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Barkley Shut Up and Jam! (UE) [!]
static struct BurnRomInfo g_sujRomDesc[] = {
	{ "g_suj.bin", 0x100000, 0x63fbf497, BRF_PRG },
};

STD_ROM_PICK(g_suj);
STD_ROM_FN(g_suj);

struct BurnDriverD BurnDrvMDg_suj = {
	"g_suj", NULL, NULL, "1900",
	"Barkley Shut Up and Jam! (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sujRomInfo, g_sujRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Bishoujo Senshi Sailor Moon (J) [!]
static struct BurnRomInfo g_sailRomDesc[] = {
	{ "g_sail.bin", 0x200000, 0x5e246938, BRF_PRG },
};

STD_ROM_PICK(g_sail);
STD_ROM_FN(g_sail);

struct BurnDriverD BurnDrvMDg_sail = {
	"g_sail", NULL, NULL, "1900",
	"Bishoujo Senshi Sailor Moon (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sailRomInfo, g_sailRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Brian Lara Cricket 96 (E) [c][!]
static struct BurnRomInfo g_bl96RomDesc[] = {
	{ "g_bl96.bin", 0x100000, 0xfa3024af, BRF_PRG },
};

STD_ROM_PICK(g_bl96);
STD_ROM_FN(g_bl96);

struct BurnDriverD BurnDrvMDg_bl96 = {
	"g_bl96", NULL, NULL, "1900",
	"Brian Lara Cricket 96 (E) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_bl96RomInfo, g_bl96RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Brian Lara Cricket 96 (E) [a1][x]
static struct BurnRomInfo g_blc96xRomDesc[] = {
	{ "g_blc96x.bin", 0x100000, 0xfe52f7e1, BRF_PRG },
};

STD_ROM_PICK(g_blc96x);
STD_ROM_FN(g_blc96x);

struct BurnDriverD BurnDrvMDg_blc96x = {
	"g_blc96x", "g_bl96", NULL, "1900",
	"Brian Lara Cricket 96 (E) [a1][x]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_blc96xRomInfo, g_blc96xRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Buck Rogers - Countdown to Doomsday (UE) [!]
static struct BurnRomInfo g_buckRomDesc[] = {
	{ "g_buck.bin", 0x100000, 0x44e3bfff, BRF_PRG },
};

STD_ROM_PICK(g_buck);
STD_ROM_FN(g_buck);

struct BurnDriverD BurnDrvMDg_buck = {
	"g_buck", NULL, NULL, "1900",
	"Buck Rogers - Countdown to Doomsday (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_buckRomInfo, g_buckRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Curse (J) [!]
static struct BurnRomInfo g_cursRomDesc[] = {
	{ "g_curs.bin", 0x80000, 0xa4fbf9a9, BRF_PRG },
};

STD_ROM_PICK(g_curs);
STD_ROM_FN(g_curs);

struct BurnDriverD BurnDrvMDg_curs = {
	"g_curs", NULL, NULL, "1900",
	"Curse (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_cursRomInfo, g_cursRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// HardBall '95 (U) [!]
static struct BurnRomInfo g_hb95RomDesc[] = {
	{ "g_hb95.bin", 0x300000, 0xed10bc9e, BRF_PRG },
};

STD_ROM_PICK(g_hb95);
STD_ROM_FN(g_hb95);

struct BurnDriverD BurnDrvMDg_hb95 = {
	"g_hb95", NULL, NULL, "1900",
	"HardBall '95 (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_hb95RomInfo, g_hb95RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// World Series Baseball '98 (U) [!]
static struct BurnRomInfo g_ws98RomDesc[] = {
	{ "g_ws98.bin", 0x300000, 0x05b1ab53, BRF_PRG },
};

STD_ROM_PICK(g_ws98);
STD_ROM_FN(g_ws98);

struct BurnDriverD BurnDrvMDg_ws98 = {
	"g_ws98", NULL, NULL, "1900",
	"World Series Baseball '98 (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ws98RomInfo, g_ws98RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Dark Castle (UE) [!]
static struct BurnRomInfo g_darkRomDesc[] = {
	{ "g_dark.bin", 0x80000, 0x0464aca4, BRF_PRG },
};

STD_ROM_PICK(g_dark);
STD_ROM_FN(g_dark);

struct BurnDriverD BurnDrvMDg_dark = {
	"g_dark", NULL, NULL, "1900",
	"Dark Castle (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_darkRomInfo, g_darkRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Darwin 4081 (J) [!]
static struct BurnRomInfo g_4081RomDesc[] = {
	{ "g_4081.bin", 0x80000, 0x7a33b0cb, BRF_PRG },
};

STD_ROM_PICK(g_4081);
STD_ROM_FN(g_4081);

struct BurnDriver BurnDrvMDg_4081 = {
	"g_4081", NULL, NULL, "1900",
	"Darwin 4081 (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_4081RomInfo, g_4081RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Death and Return of Superman, The (U) [!]
static struct BurnRomInfo g_drsRomDesc[] = {
	{ "g_drs.bin", 0x200000, 0x982242d3, BRF_PRG },
};

STD_ROM_PICK(g_drs);
STD_ROM_FN(g_drs);

struct BurnDriverD BurnDrvMDg_drs = {
	"g_drs", NULL, NULL, "1900",
	"Death and Return of Superman, The (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_drsRomInfo, g_drsRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Dinosaurs for Hire (U) [!]
static struct BurnRomInfo g_hireRomDesc[] = {
	{ "g_hire.bin", 0x100000, 0x39351146, BRF_PRG },
};

STD_ROM_PICK(g_hire);
STD_ROM_FN(g_hire);

struct BurnDriverD BurnDrvMDg_hire = {
	"g_hire", NULL, NULL, "1900",
	"Dinosaurs for Hire (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_hireRomInfo, g_hireRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Double Clutch (E) [c][!]
static struct BurnRomInfo g_dblcRomDesc[] = {
	{ "g_dblc.bin", 0x40000, 0xd98c623c, BRF_PRG },
};

STD_ROM_PICK(g_dblc);
STD_ROM_FN(g_dblc);

struct BurnDriverD BurnDrvMDg_dblc = {
	"g_dblc", NULL, NULL, "1900",
	"Double Clutch (E) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_dblcRomInfo, g_dblcRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Eliminate Down (J) [!]
static struct BurnRomInfo g_elimRomDesc[] = {
	{ "g_elim.bin", 0x100000, 0x48467542, BRF_PRG },
};

STD_ROM_PICK(g_elim);
STD_ROM_FN(g_elim);

struct BurnDriverD BurnDrvMDg_elim = {
	"g_elim", NULL, NULL, "1900",
	"Eliminate Down (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_elimRomInfo, g_elimRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Fire Mustang (J) [c][!]
static struct BurnRomInfo g_mustRomDesc[] = {
	{ "g_must.bin", 0x80000, 0xeb7e36c3, BRF_PRG },
};

STD_ROM_PICK(g_must);
STD_ROM_FN(g_must);

struct BurnDriverD BurnDrvMDg_must = {
	"g_must", NULL, NULL, "1900",
	"Fire Mustang (J) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mustRomInfo, g_mustRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Goofy's Hysterical History Tour (U) [!]
static struct BurnRomInfo g_goofRomDesc[] = {
	{ "g_goof.bin", 0x100000, 0x4e1cc833, BRF_PRG },
};

STD_ROM_PICK(g_goof);
STD_ROM_FN(g_goof);

struct BurnDriverD BurnDrvMDg_goof = {
	"g_goof", NULL, NULL, "1900",
	"Goofy's Hysterical History Tour (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_goofRomInfo, g_goofRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Honoo no Toukyuuji Dodge Danpei (J) [!]
static struct BurnRomInfo g_dodgRomDesc[] = {
	{ "g_dodg.bin", 0x80000, 0x630f07c6, BRF_PRG },
};

STD_ROM_PICK(g_dodg);
STD_ROM_FN(g_dodg);

struct BurnDriverD BurnDrvMDg_dodg = {
	"g_dodg", NULL, NULL, "1900",
	"Honoo no Toukyuuji Dodge Danpei (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_dodgRomInfo, g_dodgRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Jennifer Capriati Tennis (U) [!]
static struct BurnRomInfo g_jcteRomDesc[] = {
	{ "g_jcte.bin", 0x80000, 0xab2abc8e, BRF_PRG },
};

STD_ROM_PICK(g_jcte);
STD_ROM_FN(g_jcte);

struct BurnDriverD BurnDrvMDg_jcte = {
	"g_jcte", NULL, NULL, "1900",
	"Jennifer Capriati Tennis (U) [!]\0", NULL, "Sega / System Sacom", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jcteRomInfo, g_jcteRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Grand Slam Tennis (E) [!]
static struct BurnRomInfo g_gsteRomDesc[] = {
	{ "g_gste.bin", 0x80000, 0x8c2670de, BRF_PRG },
};

STD_ROM_PICK(g_gste);
STD_ROM_FN(g_gste);

struct BurnDriverD BurnDrvMDg_gste = {
	"g_gste", "g_jcte", NULL, "1900",
	"Grand Slam Tennis (E) [!]\0", NULL, "Sega / System Sacom", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_gsteRomInfo, g_gsteRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Grand Slam Tennis (J) [!]
static struct BurnRomInfo g_gstenjRomDesc[] = {
	{ "g_gstenj.bin", 0x80000, 0x30cf37d0, BRF_PRG },
};

STD_ROM_PICK(g_gstenj);
STD_ROM_FN(g_gstenj);

struct BurnDriverD BurnDrvMDg_gstenj = {
	"g_gstenj", "g_jcte", NULL, "1900",
	"Grand Slam Tennis (J) [!]\0", NULL, "Sega / System Sacom", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_gstenjRomInfo, g_gstenjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Magical Taruruuto-Kun (J) [!]
static struct BurnRomInfo g_taruRomDesc[] = {
	{ "g_taru.bin", 0x80000, 0xf11060a5, BRF_PRG },
};

STD_ROM_PICK(g_taru);
STD_ROM_FN(g_taru);

struct BurnDriverD BurnDrvMDg_taru = {
	"g_taru", NULL, NULL, "1900",
	"Magical Taruruuto-Kun (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_taruRomInfo, g_taruRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Osomatsu-kun - Hachamecha Gekijou (J) [c][!]
static struct BurnRomInfo g_osomRomDesc[] = {
	{ "g_osom.bin", 0x40000, 0x2453350c, BRF_PRG },
};

STD_ROM_PICK(g_osom);
STD_ROM_FN(g_osom);

struct BurnDriverD BurnDrvMDg_osom = {
	"g_osom", NULL, NULL, "1900",
	"Osomatsu-kun - Hachamecha Gekijou (J) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_osomRomInfo, g_osomRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Scooby Doo Mystery (U) [!]
static struct BurnRomInfo g_scobRomDesc[] = {
	{ "g_scob.bin", 0x200000, 0x7bb9dd9b, BRF_PRG },
};

STD_ROM_PICK(g_scob);
STD_ROM_FN(g_scob);

struct BurnDriverD BurnDrvMDg_scob = {
	"g_scob", NULL, NULL, "1900",
	"Scooby Doo Mystery (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_scobRomInfo, g_scobRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Shadowrun (U) [!]
static struct BurnRomInfo g_srunRomDesc[] = {
	{ "g_srun.bin", 0x200000, 0xfbb92909, BRF_PRG },
};

STD_ROM_PICK(g_srun);
STD_ROM_FN(g_srun);

struct BurnDriverD BurnDrvMDg_srun = {
	"g_srun", NULL, NULL, "1900",
	"Shadowrun (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_srunRomInfo, g_srunRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Shadowrun (J) [!]
static struct BurnRomInfo g_shrunjRomDesc[] = {
	{ "g_shrunj.bin", 0x200000, 0xd32199f7, BRF_PRG },
};

STD_ROM_PICK(g_shrunj);
STD_ROM_FN(g_shrunj);

struct BurnDriverD BurnDrvMDg_shrunj = {
	"g_shrunj", "g_srun", NULL, "1900",
	"Shadowrun (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_shrunjRomInfo, g_shrunjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Snow Bros. - Nick & Tom (J) [c][!]
static struct BurnRomInfo g_snowRomDesc[] = {
	{ "g_snow.bin", 0x100000, 0x11b56228, BRF_PRG },
};

STD_ROM_PICK(g_snow);
STD_ROM_FN(g_snow);

struct BurnDriverD BurnDrvMDg_snow = {
	"g_snow", NULL, NULL, "1900",
	"Snow Bros. - Nick & Tom (J) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_snowRomInfo, g_snowRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Spider-Man vs The Kingpin (W) [!]
static struct BurnRomInfo g_s_kpRomDesc[] = {
	{ "g_s_kp.bin", 0x80000, 0x70ab775f, BRF_PRG },
};

STD_ROM_PICK(g_s_kp);
STD_ROM_FN(g_s_kp);

struct BurnDriverD BurnDrvMDg_s_kp = {
	"g_s_kp", NULL, NULL, "1900",
	"Spider-Man vs The Kingpin (W) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_s_kpRomInfo, g_s_kpRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Super Battleship (U) [!]
static struct BurnRomInfo g_sbatRomDesc[] = {
	{ "g_sbat.bin", 0x80000, 0x99ca1bfb, BRF_PRG },
};

STD_ROM_PICK(g_sbat);
STD_ROM_FN(g_sbat);

struct BurnDriverD BurnDrvMDg_sbat = {
	"g_sbat", NULL, NULL, "1900",
	"Super Battleship (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sbatRomInfo, g_sbatRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Theme Park (UE) [!]
static struct BurnRomInfo g_tprkRomDesc[] = {
	{ "g_tprk.bin", 0x200000, 0x289da2c5, BRF_PRG },
};

STD_ROM_PICK(g_tprk);
STD_ROM_FN(g_tprk);

struct BurnDriverD BurnDrvMDg_tprk = {
	"g_tprk", NULL, NULL, "1900",
	"Theme Park (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tprkRomInfo, g_tprkRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Tintin Au Tibet (E) (M6) [!]
static struct BurnRomInfo g_tintRomDesc[] = {
	{ "g_tint.bin", 0x200000, 0x4243caf3, BRF_PRG },
};

STD_ROM_PICK(g_tint);
STD_ROM_FN(g_tint);

struct BurnDriverD BurnDrvMDg_tint = {
	"g_tint", NULL, NULL, "1900",
	"Tintin Au Tibet (E) (M6) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tintRomInfo, g_tintRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Uchu Senkan Gomora (J) [c][!]
static struct BurnRomInfo g_gomoRomDesc[] = {
	{ "g_gomo.bin", 0x100000, 0xc511e8d2, BRF_PRG },
};

STD_ROM_PICK(g_gomo);
STD_ROM_FN(g_gomo);

struct BurnDriverD BurnDrvMDg_gomo = {
	"g_gomo", NULL, NULL, "1900",
	"Uchu Senkan Gomora (J) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_gomoRomInfo, g_gomoRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Ultimate Soccer (E) [!]
static struct BurnRomInfo g_usocRomDesc[] = {
	{ "g_usoc.bin", 0x100000, 0x83db6e58, BRF_PRG },
};

STD_ROM_PICK(g_usoc);
STD_ROM_FN(g_usoc);

struct BurnDriverD BurnDrvMDg_usoc = {
	"g_usoc", NULL, NULL, "1900",
	"Ultimate Soccer (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_usocRomInfo, g_usocRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// WWF Royal Rumble (W) [!]
static struct BurnRomInfo g_wfrrRomDesc[] = {
	{ "g_wfrr.bin", 0x200000, 0xb69dc53e, BRF_PRG },
};

STD_ROM_PICK(g_wfrr);
STD_ROM_FN(g_wfrr);

struct BurnDriverD BurnDrvMDg_wfrr = {
	"g_wfrr", NULL, NULL, "1900",
	"WWF Royal Rumble (W) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wfrrRomInfo, g_wfrrRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// WWF RAW (W) [!]
static struct BurnRomInfo g_wfraRomDesc[] = {
	{ "g_wfra.bin", 0x300000, 0x4ef5d411, BRF_PRG },
};

STD_ROM_PICK(g_wfra);
STD_ROM_FN(g_wfra);

struct BurnDriverD BurnDrvMDg_wfra = {
	"g_wfra", NULL, NULL, "1900",
	"WWF RAW (W) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wfraRomInfo, g_wfraRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Wolf Child (U) [c][!]
static struct BurnRomInfo g_wolfRomDesc[] = {
	{ "g_wolf.bin", 0x100000, 0xeb5b1cbf, BRF_PRG },
};

STD_ROM_PICK(g_wolf);
STD_ROM_FN(g_wolf);

struct BurnDriverD BurnDrvMDg_wolf = {
	"g_wolf", NULL, NULL, "1900",
	"Wolf Child (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wolfRomInfo, g_wolfRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Zoom! (JU) [!]
static struct BurnRomInfo g_zoomRomDesc[] = {
	{ "g_zoom.bin", 0x40000, 0x724d6965, BRF_PRG },
};

STD_ROM_PICK(g_zoom);
STD_ROM_FN(g_zoom);

struct BurnDriverD BurnDrvMDg_zoom = {
	"g_zoom", NULL, NULL, "1900",
	"Zoom! (JU) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_zoomRomInfo, g_zoomRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Resq (Beta)
static struct BurnRomInfo g_resqRomDesc[] = {
	{ "g_resq.bin", 0x100000, 0x558e35e0, BRF_PRG },
};

STD_ROM_PICK(g_resq);
STD_ROM_FN(g_resq);

struct BurnDriverD BurnDrvMDg_resq = {
	"g_resq", NULL, NULL, "1900",
	"Resq (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_resqRomInfo, g_resqRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Jelly Boy (Beta)
static struct BurnRomInfo g_jelyRomDesc[] = {
	{ "g_jely.bin", 0x100000, 0x7cfadc16, BRF_PRG },
};

STD_ROM_PICK(g_jely);
STD_ROM_FN(g_jely);

struct BurnDriverD BurnDrvMDg_jely = {
	"g_jely", NULL, NULL, "1900",
	"Jelly Boy (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jelyRomInfo, g_jelyRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Itchy and Scratchy Game, The (Beta)
static struct BurnRomInfo g_itchRomDesc[] = {
	{ "g_itch.bin", 0x100000, 0x81b7725d, BRF_PRG },
};

STD_ROM_PICK(g_itch);
STD_ROM_FN(g_itch);

struct BurnDriverD BurnDrvMDg_itch = {
	"g_itch", NULL, NULL, "1900",
	"Itchy and Scratchy Game, The (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_itchRomInfo, g_itchRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Second Samurai, The (E)
static struct BurnRomInfo g_2samRomDesc[] = {
	{ "g_2sam.bin", 0x100000, 0x78e92143, BRF_PRG },
};

STD_ROM_PICK(g_2sam);
STD_ROM_FN(g_2sam);

struct BurnDriver BurnDrvMDg_2sam = {
	"g_2sam", NULL, NULL, "1900",
	"Second Samurai, The (E)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_2samRomInfo, g_2samRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Fun Car Rally (Unl)
static struct BurnRomInfo g_fcrRomDesc[] = {
	{ "g_fcr.bin", 0x100000, 0x42e27845, BRF_PRG },
};

STD_ROM_PICK(g_fcr);
STD_ROM_FN(g_fcr);

struct BurnDriverD BurnDrvMDg_fcr = {
	"g_fcr", NULL, NULL, "1900",
	"Fun Car Rally (Unl)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_fcrRomInfo, g_fcrRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Predator 2 (UE) [!]
static struct BurnRomInfo g_pre2RomDesc[] = {
	{ "g_pre2.bin", 0x100000, 0xbdba113e, BRF_PRG },
};

STD_ROM_PICK(g_pre2);
STD_ROM_FN(g_pre2);

struct BurnDriverD BurnDrvMDg_pre2 = {
	"g_pre2", NULL, NULL, "1900",
	"Predator 2 (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pre2RomInfo, g_pre2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Premier Manager 97 (E) [!]
static struct BurnRomInfo g_pm97RomDesc[] = {
	{ "g_pm97.bin", 0x100000, 0xfccbf69b, BRF_PRG },
};

STD_ROM_PICK(g_pm97);
STD_ROM_FN(g_pm97);

struct BurnDriverD BurnDrvMDg_pm97 = {
	"g_pm97", NULL, NULL, "1900",
	"Premier Manager 97 (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pm97RomInfo, g_pm97RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Premier Manager (E) [!]
static struct BurnRomInfo g_pmanRomDesc[] = {
	{ "g_pman.bin", 0x100000, 0x303b889f, BRF_PRG },
};

STD_ROM_PICK(g_pman);
STD_ROM_FN(g_pman);

struct BurnDriverD BurnDrvMDg_pman = {
	"g_pman", NULL, NULL, "1900",
	"Premier Manager (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pmanRomInfo, g_pmanRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Primal Rage (UE) [!]
static struct BurnRomInfo g_primRomDesc[] = {
	{ "g_prim.bin", 0x300000, 0x2884c6d1, BRF_PRG },
};

STD_ROM_PICK(g_prim);
STD_ROM_FN(g_prim);

struct BurnDriverD BurnDrvMDg_prim = {
	"g_prim", NULL, NULL, "1900",
	"Primal Rage (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_primRomInfo, g_primRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Powerball (U) [c][!]
static struct BurnRomInfo g_pwblRomDesc[] = {
	{ "g_pwbl.bin", 0x80000, 0x7adf232f, BRF_PRG },
};

STD_ROM_PICK(g_pwbl);
STD_ROM_FN(g_pwbl);

struct BurnDriverD BurnDrvMDg_pwbl = {
	"g_pwbl", NULL, NULL, "1900",
	"Powerball (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pwblRomInfo, g_pwblRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Wrestleball (J) [c][!]
static struct BurnRomInfo g_wresblRomDesc[] = {
	{ "g_wresbl.bin", 0x80000, 0xd563e07f, BRF_PRG },
};

STD_ROM_PICK(g_wresbl);
STD_ROM_FN(g_wresbl);

struct BurnDriverD BurnDrvMDg_wresbl = {
	"g_wresbl", "g_pwbl", NULL, "1900",
	"Wrestleball (J) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wresblRomInfo, g_wresblRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Rambo III (W) (REV01) [!]
static struct BurnRomInfo g_ram3RomDesc[] = {
	{ "g_ram3.bin", 0x40000, 0x4d47a647, BRF_PRG },
};

STD_ROM_PICK(g_ram3);
STD_ROM_FN(g_ram3);

struct BurnDriverD BurnDrvMDg_ram3 = {
	"g_ram3", NULL, NULL, "1900",
	"Rambo III (W) (REV01) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ram3RomInfo, g_ram3RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Rambo III (W) (REV00) [!]
static struct BurnRomInfo g_ram300RomDesc[] = {
	{ "g_ram300.bin", 0x40000, 0x2232f03d, BRF_PRG },
};

STD_ROM_PICK(g_ram300);
STD_ROM_FN(g_ram300);

struct BurnDriverD BurnDrvMDg_ram300 = {
	"g_ram300", "g_ram3", NULL, "1900",
	"Rambo III (W) (REV00) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ram300RomInfo, g_ram300RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Revolution X (UE) [!]
static struct BurnRomInfo g_revxRomDesc[] = {
	{ "g_revx.bin", 0x400000, 0x5fb0c5d4, BRF_PRG },
};

STD_ROM_PICK(g_revx);
STD_ROM_FN(g_revx);

struct BurnDriverD BurnDrvMDg_revx = {
	"g_revx", NULL, NULL, "1900",
	"Revolution X (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_revxRomInfo, g_revxRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Shadow Dancer - The Secret of Shinobi (W) [c][!]
static struct BurnRomInfo g_sdanRomDesc[] = {
	{ "g_sdan.bin", 0x80000, 0xebe9ad10, BRF_PRG },
};

STD_ROM_PICK(g_sdan);
STD_ROM_FN(g_sdan);

struct BurnDriverD BurnDrvMDg_sdan = {
	"g_sdan", NULL, NULL, "1900",
	"Shadow Dancer - The Secret of Shinobi (W) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sdanRomInfo, g_sdanRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Super Battle Tank - War in the Gulf (U) [!]
static struct BurnRomInfo g_sbtRomDesc[] = {
	{ "g_sbt.bin", 0x80000, 0xb0b5e3c9, BRF_PRG },
};

STD_ROM_PICK(g_sbt);
STD_ROM_FN(g_sbt);

struct BurnDriverD BurnDrvMDg_sbt = {
	"g_sbt", NULL, NULL, "1900",
	"Super Battle Tank - War in the Gulf (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sbtRomInfo, g_sbtRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Faery Tale Adventure, The (UE) [!]
static struct BurnRomInfo g_faerRomDesc[] = {
	{ "g_faer.bin", 0x80000, 0x963f4969, BRF_PRG },
};

STD_ROM_PICK(g_faer);
STD_ROM_FN(g_faer);

struct BurnDriverD BurnDrvMDg_faer = {
	"g_faer", NULL, NULL, "1900",
	"Faery Tale Adventure, The (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_faerRomInfo, g_faerRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Gain Ground (UE) [c][!]
static struct BurnRomInfo g_gainRomDesc[] = {
	{ "g_gain.bin", 0x80000, 0x83e7b8ae, BRF_PRG },
};

STD_ROM_PICK(g_gain);
STD_ROM_FN(g_gain);

struct BurnDriverD BurnDrvMDg_gain = {
	"g_gain", NULL, NULL, "1900",
	"Gain Ground (UE) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_gainRomInfo, g_gainRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Gain Ground (J) [!]
static struct BurnRomInfo g_ggroujRomDesc[] = {
	{ "g_ggrouj.bin", 0x80000, 0x8641a2ab, BRF_PRG },
};

STD_ROM_PICK(g_ggrouj);
STD_ROM_FN(g_ggrouj);

struct BurnDriverD BurnDrvMDg_ggrouj = {
	"g_ggrouj", "g_gain", NULL, "1900",
	"Gain Ground (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ggroujRomInfo, g_ggroujRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Zero Tolerance (UE) [!]
static struct BurnRomInfo g_0tolRomDesc[] = {
	{ "g_0tol.bin", 0x200000, 0x23f603f5, BRF_PRG },
};

STD_ROM_PICK(g_0tol);
STD_ROM_FN(g_0tol);

struct BurnDriverD BurnDrvMDg_0tol = {
	"g_0tol", NULL, NULL, "1900",
	"Zero Tolerance (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_0tolRomInfo, g_0tolRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Whip Rush 2222 AD (U) [!]
static struct BurnRomInfo g_whipRomDesc[] = {
	{ "g_whip.bin", 0x80000, 0x7eb6b86b, BRF_PRG },
};

STD_ROM_PICK(g_whip);
STD_ROM_FN(g_whip);

struct BurnDriverD BurnDrvMDg_whip = {
	"g_whip", NULL, NULL, "1900",
	"Whip Rush 2222 AD (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_whipRomInfo, g_whipRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Whip Rush 2222 AD (J) [!]
static struct BurnRomInfo g_whipjRomDesc[] = {
	{ "g_whipj.bin", 0x80000, 0x8084b4d1, BRF_PRG },
};

STD_ROM_PICK(g_whipj);
STD_ROM_FN(g_whipj);

struct BurnDriverD BurnDrvMDg_whipj = {
	"g_whipj", "g_whip", NULL, "1900",
	"Whip Rush 2222 AD (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_whipjRomInfo, g_whipjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Virtual Bart (W) [!]
static struct BurnRomInfo g_vbarRomDesc[] = {
	{ "g_vbar.bin", 0x200000, 0x8db9f378, BRF_PRG },
};

STD_ROM_PICK(g_vbar);
STD_ROM_FN(g_vbar);

struct BurnDriverD BurnDrvMDg_vbar = {
	"g_vbar", NULL, NULL, "1900",
	"Virtual Bart (W) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_vbarRomInfo, g_vbarRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Twin Hawk (JE) [!]
static struct BurnRomInfo g_twihRomDesc[] = {
	{ "g_twih.bin", 0x80000, 0xa2ec8c67, BRF_PRG },
};

STD_ROM_PICK(g_twih);
STD_ROM_FN(g_twih);

struct BurnDriverD BurnDrvMDg_twih = {
	"g_twih", NULL, NULL, "1900",
	"Twin Hawk (JE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_twihRomInfo, g_twihRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Twin Cobra (U) [!]
static struct BurnRomInfo g_twicRomDesc[] = {
	{ "g_twic.bin", 0xa0000, 0x2c708248, BRF_PRG },
};

STD_ROM_PICK(g_twic);
STD_ROM_FN(g_twic);

struct BurnDriverD BurnDrvMDg_twic = {
	"g_twic", NULL, NULL, "1900",
	"Twin Cobra (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_twicRomInfo, g_twicRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Kyuukyou Tiger (J) [!]
static struct BurnRomInfo g_kyuukyRomDesc[] = {
	{ "g_kyuuky.bin", 0xa0000, 0x61276d21, BRF_PRG },
};

STD_ROM_PICK(g_kyuuky);
STD_ROM_FN(g_kyuuky);

struct BurnDriverD BurnDrvMDg_kyuuky = {
	"g_kyuuky", "g_twic", NULL, "1900",
	"Kyuukyou Tiger (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_kyuukyRomInfo, g_kyuukyRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Turrican (U) [c][!]
static struct BurnRomInfo g_turrRomDesc[] = {
	{ "g_turr.bin", 0x80000, 0x634d67a6, BRF_PRG },
};

STD_ROM_PICK(g_turr);
STD_ROM_FN(g_turr);

struct BurnDriverD BurnDrvMDg_turr = {
	"g_turr", NULL, NULL, "1900",
	"Turrican (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_turrRomInfo, g_turrRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Truxton (W) [!]
static struct BurnRomInfo g_truxRomDesc[] = {
	{ "g_trux.bin", 0x80000, 0x5bd0882d, BRF_PRG },
};

STD_ROM_PICK(g_trux);
STD_ROM_FN(g_trux);

struct BurnDriverD BurnDrvMDg_trux = {
	"g_trux", NULL, NULL, "1900",
	"Truxton (W) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_truxRomInfo, g_truxRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Triple Play '96 (U) [c][!]
static struct BurnRomInfo g_tp96RomDesc[] = {
	{ "g_tp96.bin", 0x400000, 0xf1748e91, BRF_PRG },
};

STD_ROM_PICK(g_tp96);
STD_ROM_FN(g_tp96);

struct BurnDriverD BurnDrvMDg_tp96 = {
	"g_tp96", NULL, NULL, "1900",
	"Triple Play '96 (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tp96RomInfo, g_tp96RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Triple Play Gold (U) [c][!]
static struct BurnRomInfo g_tpgoRomDesc[] = {
	{ "g_tpgo.bin", 0x400000, 0xbbe69017, BRF_PRG },
};

STD_ROM_PICK(g_tpgo);
STD_ROM_FN(g_tpgo);

struct BurnDriverD BurnDrvMDg_tpgo = {
	"g_tpgo", NULL, NULL, "1900",
	"Triple Play Gold (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tpgoRomInfo, g_tpgoRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Triple Play Gold (U) [a1]
static struct BurnRomInfo g_tpgolaRomDesc[] = {
	{ "g_tpgola.bin", 0x400000, 0xa89638a0, BRF_PRG },
};

STD_ROM_PICK(g_tpgola);
STD_ROM_FN(g_tpgola);

struct BurnDriverD BurnDrvMDg_tpgola = {
	"g_tpgola", "g_tpgo", NULL, "1900",
	"Triple Play Gold (U) [a1]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tpgolaRomInfo, g_tpgolaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Time Killers (U) [!]
static struct BurnRomInfo g_tkilRomDesc[] = {
	{ "g_tkil.bin", 0x200000, 0x4b5f52ac, BRF_PRG },
};

STD_ROM_PICK(g_tkil);
STD_ROM_FN(g_tkil);

struct BurnDriverD BurnDrvMDg_tkil = {
	"g_tkil", NULL, NULL, "1900",
	"Time Killers (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tkilRomInfo, g_tkilRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Time Killers (E) [!]
static struct BurnRomInfo g_tkilleRomDesc[] = {
	{ "g_tkille.bin", 0x200000, 0xa4f48a1a, BRF_PRG },
};

STD_ROM_PICK(g_tkille);
STD_ROM_FN(g_tkille);

struct BurnDriverD BurnDrvMDg_tkille = {
	"g_tkille", "g_tkil", NULL, "1900",
	"Time Killers (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tkilleRomInfo, g_tkilleRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Tetris (J) [!]
static struct BurnRomInfo g_tetrRomDesc[] = {
	{ "g_tetr.bin", 0x40000, 0x4ce90db0, BRF_PRG },
};

STD_ROM_PICK(g_tetr);
STD_ROM_FN(g_tetr);

struct BurnDriverD BurnDrvMDg_tetr = {
	"g_tetr", NULL, NULL, "1900",
	"Tetris (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tetrRomInfo, g_tetrRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Syndicate (UE) [!]
static struct BurnRomInfo g_syndRomDesc[] = {
	{ "g_synd.bin", 0x200000, 0x95bbf87b, BRF_PRG },
};

STD_ROM_PICK(g_synd);
STD_ROM_FN(g_synd);

struct BurnDriverD BurnDrvMDg_synd = {
	"g_synd", NULL, NULL, "1900",
	"Syndicate (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_syndRomInfo, g_syndRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Power Instinct (J) [!]
static struct BurnRomInfo g_pinsRomDesc[] = {
	{ "g_pins.bin", 0x300000, 0xabe9c415, BRF_PRG },
};

STD_ROM_PICK(g_pins);
STD_ROM_FN(g_pins);

struct BurnDriverD BurnDrvMDg_pins = {
	"g_pins", NULL, NULL, "1900",
	"Power Instinct (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pinsRomInfo, g_pinsRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Pac-Man 2 - The New Adventures (U) [!]
static struct BurnRomInfo g_pac2RomDesc[] = {
	{ "g_pac2.bin", 0x200000, 0xfe7a7ed1, BRF_PRG },
};

STD_ROM_PICK(g_pac2);
STD_ROM_FN(g_pac2);

struct BurnDriverD BurnDrvMDg_pac2 = {
	"g_pac2", NULL, NULL, "1900",
	"Pac-Man 2 - The New Adventures (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pac2RomInfo, g_pac2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Ninja Burai Densetsu (J) [!]
static struct BurnRomInfo g_nbdRomDesc[] = {
	{ "g_nbd.bin", 0x100000, 0xa8d828a0, BRF_PRG },
};

STD_ROM_PICK(g_nbd);
STD_ROM_FN(g_nbd);

struct BurnDriverD BurnDrvMDg_nbd = {
	"g_nbd", NULL, NULL, "1900",
	"Ninja Burai Densetsu (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_nbdRomInfo, g_nbdRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Kick Off 3 - European Challenge (E) [!]
static struct BurnRomInfo g_ko3RomDesc[] = {
	{ "g_ko3.bin", 0x100000, 0xbc37401a, BRF_PRG },
};

STD_ROM_PICK(g_ko3);
STD_ROM_FN(g_ko3);

struct BurnDriverD BurnDrvMDg_ko3 = {
	"g_ko3", NULL, NULL, "1900",
	"Kick Off 3 - European Challenge (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ko3RomInfo, g_ko3RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Ka-Ge-Ki - Fists of Steel (U) [!]
static struct BurnRomInfo g_kgkRomDesc[] = {
	{ "g_kgk.bin", 0x100000, 0xeffc0fa6, BRF_PRG },
};

STD_ROM_PICK(g_kgk);
STD_ROM_FN(g_kgk);

struct BurnDriverD BurnDrvMDg_kgk = {
	"g_kgk", NULL, NULL, "1900",
	"Ka-Ge-Ki - Fists of Steel (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_kgkRomInfo, g_kgkRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Ka-Ge-Ki (J) [!]
static struct BurnRomInfo g_kagekiRomDesc[] = {
	{ "g_kageki.bin", 0x100000, 0x391866a1, BRF_PRG },
};

STD_ROM_PICK(g_kageki);
STD_ROM_FN(g_kageki);

struct BurnDriverD BurnDrvMDg_kageki = {
	"g_kageki", "g_kgk", NULL, "1900",
	"Ka-Ge-Ki (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_kagekiRomInfo, g_kagekiRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// We're Back! - A Dinosaur's Tale (U) [!]
static struct BurnRomInfo g_wbdtRomDesc[] = {
	{ "g_wbdt.bin", 0x100000, 0x70155b5b, BRF_PRG },
};

STD_ROM_PICK(g_wbdt);
STD_ROM_FN(g_wbdt);

struct BurnDriverD BurnDrvMDg_wbdt = {
	"g_wbdt", NULL, NULL, "1900",
	"We're Back! - A Dinosaur's Tale (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wbdtRomInfo, g_wbdtRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Combat Cars (UE) [!]
static struct BurnRomInfo g_comcRomDesc[] = {
	{ "g_comc.bin", 0x100000, 0xe439b101, BRF_PRG },
};

STD_ROM_PICK(g_comc);
STD_ROM_FN(g_comc);

struct BurnDriverD BurnDrvMDg_comc = {
	"g_comc", NULL, NULL, "1900",
	"Combat Cars (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_comcRomInfo, g_comcRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// 688 Attack Sub (UE) [!]
static struct BurnRomInfo g_688aRomDesc[] = {
	{ "g_688a.bin", 0x100000, 0xF2C58BF7, BRF_PRG },
};

STD_ROM_PICK(g_688a);
STD_ROM_FN(g_688a);

struct BurnDriverD BurnDrvMDg_688a = {
	"g_688a", NULL, NULL, "1991",
	"688 Attack Sub (UE) [!]\0", "Sound Problems", "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_688aRomInfo, g_688aRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Air Diver (U) [!]
static struct BurnRomInfo g_adivRomDesc[] = {
	{ "g_adiv.bin", 0x80000, 0x2041885e, BRF_PRG },
};

STD_ROM_PICK(g_adiv);
STD_ROM_FN(g_adiv);

struct BurnDriver BurnDrvMDg_adiv = {
	"g_adiv", NULL, NULL, "1900",
	"Air Diver (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_adivRomInfo, g_adivRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Air Diver (J) [!]
static struct BurnRomInfo g_adivejRomDesc[] = {
	{ "g_adivej.bin", 0x80000, 0x9e2d5b91, BRF_PRG },
};

STD_ROM_PICK(g_adivej);
STD_ROM_FN(g_adivej);

struct BurnDriverD BurnDrvMDg_adivej = {
	"g_adivej", "g_adiv", NULL, "1900",
	"Air Diver (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_adivejRomInfo, g_adivejRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Alien Storm (W) [!]
static struct BurnRomInfo g_astoRomDesc[] = {
	{ "g_asto.bin", 0x80000, 0xf5ac8de5, BRF_PRG },
};

STD_ROM_PICK(g_asto);
STD_ROM_FN(g_asto);

struct BurnDriverD BurnDrvMDg_asto = {
	"g_asto", NULL, NULL, "1900",
	"Alien Storm (W) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_astoRomInfo, g_astoRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// American Gladiators (U) [!]
static struct BurnRomInfo g_aglaRomDesc[] = {
	{ "g_agla.bin", 0x100000, 0x9952fa85, BRF_PRG },
};

STD_ROM_PICK(g_agla);
STD_ROM_FN(g_agla);

struct BurnDriverD BurnDrvMDg_agla = {
	"g_agla", NULL, NULL, "1900",
	"American Gladiators (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_aglaRomInfo, g_aglaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Arch Rivals (UE) [!]
static struct BurnRomInfo g_archRomDesc[] = {
	{ "g_arch.bin", 0x80000, 0xe389d7e7, BRF_PRG },
};

STD_ROM_PICK(g_arch);
STD_ROM_FN(g_arch);

struct BurnDriver BurnDrvMDg_arch = {
	"g_arch", NULL, NULL, "1900",
	"Arch Rivals (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_archRomInfo, g_archRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Arnold Palmer Tournament Golf (UE) [!]
static struct BurnRomInfo g_aptgRomDesc[] = {
	{ "g_aptg.bin", 0x80000, 0x35b995ef, BRF_PRG },
};

STD_ROM_PICK(g_aptg);
STD_ROM_FN(g_aptg);

struct BurnDriverD BurnDrvMDg_aptg = {
	"g_aptg", NULL, NULL, "1900",
	"Arnold Palmer Tournament Golf (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_aptgRomInfo, g_aptgRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// ATP Tour Championship Tennis (U) [!]
static struct BurnRomInfo g_atptRomDesc[] = {
	{ "g_atpt.bin", 0x200000, 0x8c822884, BRF_PRG },
};

STD_ROM_PICK(g_atpt);
STD_ROM_FN(g_atpt);

struct BurnDriver BurnDrvMDg_atpt = {
	"g_atpt", NULL, NULL, "1900",
	"ATP Tour Championship Tennis (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_atptRomInfo, g_atptRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// ATP Tour Championship Tennis (E)
static struct BurnRomInfo g_atpteRomDesc[] = {
	{ "g_atpte.bin", 0x200000, 0x1a3da8c5, BRF_PRG },
};

STD_ROM_PICK(g_atpte);
STD_ROM_FN(g_atpte);

struct BurnDriverD BurnDrvMDg_atpte = {
	"g_atpte", "g_atpt", NULL, "1900",
	"ATP Tour Championship Tennis (E)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_atpteRomInfo, g_atpteRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Australian Rugby League (E) [!]
static struct BurnRomInfo g_arugRomDesc[] = {
	{ "g_arug.bin", 0x200000, 0xac5bc26a, BRF_PRG },
};

STD_ROM_PICK(g_arug);
STD_ROM_FN(g_arug);

struct BurnDriver BurnDrvMDg_arug = {
	"g_arug", NULL, NULL, "1900",
	"Australian Rugby League (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_arugRomInfo, g_arugRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Ball Jacks (JE) [c][!]
static struct BurnRomInfo g_bjakRomDesc[] = {
	{ "g_bjak.bin", 0x40000, 0xf5c3c54f, BRF_PRG },
};

STD_ROM_PICK(g_bjak);
STD_ROM_FN(g_bjak);

struct BurnDriver BurnDrvMDg_bjak = {
	"g_bjak", NULL, NULL, "1900",
	"Ball Jacks (JE) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_bjakRomInfo, g_bjakRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Barbie Super Model (U) [!]
static struct BurnRomInfo g_barbRomDesc[] = {
	{ "g_barb.bin", 0x100000, 0x81c9662b, BRF_PRG },
};

STD_ROM_PICK(g_barb);
STD_ROM_FN(g_barb);

struct BurnDriver BurnDrvMDg_barb = {
	"g_barb", NULL, NULL, "1900",
	"Barbie Super Model (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_barbRomInfo, g_barbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Bass Masters Classics (U) [!]
static struct BurnRomInfo g_bassRomDesc[] = {
	{ "g_bass.bin", 0x200000, 0xcf1ff00a, BRF_PRG },
};

STD_ROM_PICK(g_bass);
STD_ROM_FN(g_bass);

struct BurnDriver BurnDrvMDg_bass = {
	"g_bass", NULL, NULL, "1900",
	"Bass Masters Classics (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_bassRomInfo, g_bassRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Bass Masters Classic Pro Edition (U) [!]
static struct BurnRomInfo g_baspRomDesc[] = {
	{ "g_basp.bin", 0x200000, 0x9eddeb3d, BRF_PRG },
};

STD_ROM_PICK(g_basp);
STD_ROM_FN(g_basp);

struct BurnDriver BurnDrvMDg_basp = {
	"g_basp", NULL, NULL, "1900",
	"Bass Masters Classic Pro Edition (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_baspRomInfo, g_baspRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Barney's Hide & Seek Game (U) [!]
static struct BurnRomInfo g_barnRomDesc[] = {
	{ "g_barn.bin", 0x100000, 0x1efa9d53, BRF_PRG },
};

STD_ROM_PICK(g_barn);
STD_ROM_FN(g_barn);

struct BurnDriverD BurnDrvMDg_barn = {
	"g_barn", NULL, NULL, "1900",
	"Barney's Hide & Seek Game (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_barnRomInfo, g_barnRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Battle Mania - Dai Gin Jou (J) [c][!]
static struct BurnRomInfo g_btl2RomDesc[] = {
	{ "g_btl2.bin", 0x100000, 0x312fa0f2, BRF_PRG },
};

STD_ROM_PICK(g_btl2);
STD_ROM_FN(g_btl2);

struct BurnDriver BurnDrvMDg_btl2 = {
	"g_btl2", NULL, NULL, "1900",
	"Battle Mania - Dai Gin Jou (J) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_btl2RomInfo, g_btl2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Trouble Shooter (U) [!]
static struct BurnRomInfo g_tshtRomDesc[] = {
	{ "g_tsht.bin", 0x80000, 0xbecfc39b, BRF_PRG },
};

STD_ROM_PICK(g_tsht);
STD_ROM_FN(g_tsht);

struct BurnDriver BurnDrvMDg_tsht = {
	"g_tsht", NULL, NULL, "1900",
	"Trouble Shooter (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tshtRomInfo, g_tshtRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Battle Mania (J) [!]
static struct BurnRomInfo g_btlmRomDesc[] = {
	{ "g_btlm.bin", 0x80000, 0xa76c4a29, BRF_PRG },
};

STD_ROM_PICK(g_btlm);
STD_ROM_FN(g_btlm);

struct BurnDriver BurnDrvMDg_btlm = {
	"g_btlm", "g_tsht", NULL, "1900",
	"Battle Mania (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_btlmRomInfo, g_btlmRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Battlemaster (U) [c][!]
static struct BurnRomInfo g_btmsRomDesc[] = {
	{ "g_btms.bin", 0x80000, 0xfd2b35e3, BRF_PRG },
};

STD_ROM_PICK(g_btms);
STD_ROM_FN(g_btms);

struct BurnDriver BurnDrvMDg_btms = {
	"g_btms", NULL, NULL, "1900",
	"Battlemaster (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_btmsRomInfo, g_btmsRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Battletech (U) [!]
static struct BurnRomInfo g_btecRomDesc[] = {
	{ "g_btec.bin", 0x200000, 0x409e5d14, BRF_PRG },
};

STD_ROM_PICK(g_btec);
STD_ROM_FN(g_btec);

struct BurnDriverD BurnDrvMDg_btec = {
	"g_btec", NULL, NULL, "1900",
	"Battletech (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_btecRomInfo, g_btecRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Berenstain Bears', The - Camping Adventure (U) [!]
static struct BurnRomInfo g_bearRomDesc[] = {
	{ "g_bear.bin", 0x100000, 0x1f86237b, BRF_PRG },
};

STD_ROM_PICK(g_bear);
STD_ROM_FN(g_bear);

struct BurnDriver BurnDrvMDg_bear = {
	"g_bear", NULL, NULL, "1900",
	"Berenstain Bears', The - Camping Adventure (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_bearRomInfo, g_bearRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Bill Walsh College Football '95 (U) [!]
static struct BurnRomInfo g_bw95RomDesc[] = {
	{ "g_bw95.bin", 0x200000, 0xa582f45a, BRF_PRG },
};

STD_ROM_PICK(g_bw95);
STD_ROM_FN(g_bw95);

struct BurnDriver BurnDrvMDg_bw95 = {
	"g_bw95", NULL, NULL, "1900",
	"Bill Walsh College Football '95 (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_bw95RomInfo, g_bw95RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Bill Walsh College Football (UE) [c][!]
static struct BurnRomInfo g_bwcfRomDesc[] = {
	{ "g_bwcf.bin", 0x100000, 0x3ed83362, BRF_PRG },
};

STD_ROM_PICK(g_bwcf);
STD_ROM_FN(g_bwcf);

struct BurnDriver BurnDrvMDg_bwcf = {
	"g_bwcf", NULL, NULL, "1900",
	"Bill Walsh College Football (UE) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_bwcfRomInfo, g_bwcfRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Bimini Run (U) (REV02) [!]
static struct BurnRomInfo g_bimiRomDesc[] = {
	{ "g_bimi.bin", 0x80000, 0xd4dc5188, BRF_PRG },
};

STD_ROM_PICK(g_bimi);
STD_ROM_FN(g_bimi);

struct BurnDriver BurnDrvMDg_bimi = {
	"g_bimi", NULL, NULL, "1900",
	"Bimini Run (U) (REV02) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_bimiRomInfo, g_bimiRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Championship Bowling (U) [!]
static struct BurnRomInfo g_cbwlRomDesc[] = {
	{ "g_cbwl.bin", 0x80000, 0x1bf92520, BRF_PRG },
};

STD_ROM_PICK(g_cbwl);
STD_ROM_FN(g_cbwl);

struct BurnDriverD BurnDrvMDg_cbwl = {
	"g_cbwl", NULL, NULL, "1900",
	"Championship Bowling (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_cbwlRomInfo, g_cbwlRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Boogie Woogie Bowling (J) [!]
static struct BurnRomInfo g_bwbwRomDesc[] = {
	{ "g_bwbw.bin", 0x80000, 0xccf52828, BRF_PRG },
};

STD_ROM_PICK(g_bwbw);
STD_ROM_FN(g_bwbw);

struct BurnDriverD BurnDrvMDg_bwbw = {
	"g_bwbw", "g_cbwl", NULL, "1900",
	"Boogie Woogie Bowling (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_bwbwRomInfo, g_bwbwRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Brett Hull Hockey '95 (U) [!]
static struct BurnRomInfo g_bh95RomDesc[] = {
	{ "g_bh95.bin", 0x200000, 0xf7775a09, BRF_PRG },
};

STD_ROM_PICK(g_bh95);
STD_ROM_FN(g_bh95);

struct BurnDriverD BurnDrvMDg_bh95 = {
	"g_bh95", NULL, NULL, "1900",
	"Brett Hull Hockey '95 (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_bh95RomInfo, g_bh95RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Lakers vs Celtics and the NBA Playoffs (UE) [!]
static struct BurnRomInfo g_lvscRomDesc[] = {
	{ "g_lvsc.bin", 0x80000, 0x0e33fc75, BRF_PRG },
};

STD_ROM_PICK(g_lvsc);
STD_ROM_FN(g_lvsc);

struct BurnDriverD BurnDrvMDg_lvsc = {
	"g_lvsc", NULL, NULL, "1990",
	"Lakers vs Celtics and the NBA Playoffs (UE) [!]\0", NULL, "Electronic Arts", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_lvscRomInfo, g_lvscRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Bulls vs Blazers and the NBA Playoffs (UE) [!]
static struct BurnRomInfo g_bvsbRomDesc[] = {
	{ "g_bvsb.bin", 0x100000, 0xd4e4b4e8, BRF_PRG },
};

STD_ROM_PICK(g_bvsb);
STD_ROM_FN(g_bvsb);

struct BurnDriver BurnDrvMDg_bvsb = {
	"g_bvsb", NULL, NULL, "1993",
	"Bulls vs Blazers and the NBA Playoffs (UE) [!]\0", NULL, "Electronic Arts", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_bvsbRomInfo, g_bvsbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// NBA Playoff - Bulls vs Blazers (J) [!]
static struct BurnRomInfo g_npbbRomDesc[] = {
	{ "g_npbb.bin", 0x100000, 0x4565ce1f, BRF_PRG },
};

STD_ROM_PICK(g_npbb);
STD_ROM_FN(g_npbb);

struct BurnDriverD BurnDrvMDg_npbb = {
	"g_npbb", NULL, NULL, "1993",
	"NBA Playoff - Bulls vs Blazers (J) [!]\0", NULL, "Electronic Arts", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_npbbRomInfo, g_npbbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Bulls vs Lakers and the NBA Playoffs (UE) [!]
static struct BurnRomInfo g_bvslRomDesc[] = {
	{ "g_bvsl.bin", 0x100000, 0xe56023a0, BRF_PRG },
};

STD_ROM_PICK(g_bvsl);
STD_ROM_FN(g_bvsl);

struct BurnDriver BurnDrvMDg_bvsl = {
	"g_bvsl", NULL, NULL, "1991",
	"Bulls vs Lakers and the NBA Playoffs (UE) [!]\0", NULL, "Electronic Arts", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_bvslRomInfo, g_bvslRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// NBA Pro Basketball - Bulls vs Lakers (J) [!]
static struct BurnRomInfo g_npblRomDesc[] = {
	{ "g_npbl.bin", 0x100000, 0x4416ce39, BRF_PRG },
};

STD_ROM_PICK(g_npbl);
STD_ROM_FN(g_npbl);

struct BurnDriverD BurnDrvMDg_npbl = {
	"g_npbl", NULL, NULL, "1993",
	"NBA Pro Basketball - Bulls vs Lakers (J) [!]\0", NULL, "Electronic Arts", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_npblRomInfo, g_npblRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Cadash (JU) [c][!]
static struct BurnRomInfo g_cadaRomDesc[] = {
	{ "g_cada.bin", 0x80000, 0x13bdf374, BRF_PRG },
};

STD_ROM_PICK(g_cada);
STD_ROM_FN(g_cada);

struct BurnDriverD BurnDrvMDg_cada = {
	"g_cada", NULL, NULL, "1900",
	"Cadash (JU) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_cadaRomInfo, g_cadaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Caesars Palace (U) [!]
static struct BurnRomInfo g_caesRomDesc[] = {
	{ "g_caes.bin", 0x80000, 0x8fdaa9bb, BRF_PRG },
};

STD_ROM_PICK(g_caes);
STD_ROM_FN(g_caes);

struct BurnDriverD BurnDrvMDg_caes = {
	"g_caes", NULL, NULL, "1900",
	"Caesars Palace (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_caesRomInfo, g_caesRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Cal Ripken Jr. Baseball (U) [c][!]
static struct BurnRomInfo g_crjbRomDesc[] = {
	{ "g_crjb.bin", 0x100000, 0x9b1c96c0, BRF_PRG },
};

STD_ROM_PICK(g_crjb);
STD_ROM_FN(g_crjb);

struct BurnDriverD BurnDrvMDg_crjb = {
	"g_crjb", NULL, NULL, "1900",
	"Cal Ripken Jr. Baseball (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_crjbRomInfo, g_crjbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Caliber Fifty (U) [!]
static struct BurnRomInfo g_c50RomDesc[] = {
	{ "g_c50.bin", 0x100000, 0x44f4fa05, BRF_PRG },
};

STD_ROM_PICK(g_c50);
STD_ROM_FN(g_c50);

struct BurnDriverD BurnDrvMDg_c50 = {
	"g_c50", NULL, NULL, "1900",
	"Caliber Fifty (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_c50RomInfo, g_c50RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// California Games (UE) [!]
static struct BurnRomInfo g_cgamRomDesc[] = {
	{ "g_cgam.bin", 0x80000, 0x43b1b672, BRF_PRG },
};

STD_ROM_PICK(g_cgam);
STD_ROM_FN(g_cgam);

struct BurnDriver BurnDrvMDg_cgam = {
	"g_cgam", NULL, NULL, "1900",
	"California Games (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_cgamRomInfo, g_cgamRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Captain America and the Avengers (U) [!]
static struct BurnRomInfo g_capaRomDesc[] = {
	{ "g_capa.bin", 0x100000, 0xe0639ca2, BRF_PRG },
};

STD_ROM_PICK(g_capa);
STD_ROM_FN(g_capa);

struct BurnDriverD BurnDrvMDg_capa = {
	"g_capa", NULL, NULL, "1900",
	"Captain America and the Avengers (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_capaRomInfo, g_capaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Captain America and the Avengers (Beta)
static struct BurnRomInfo g_capambRomDesc[] = {
	{ "g_capamb.bin", 0x100000, 0xbaac59c0, BRF_PRG },
};

STD_ROM_PICK(g_capamb);
STD_ROM_FN(g_capamb);

struct BurnDriverD BurnDrvMDg_capamb = {
	"g_capamb", "g_capa", NULL, "1900",
	"Captain America and the Avengers (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_capambRomInfo, g_capambRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Captain America and the Avengers (E)
static struct BurnRomInfo g_capameRomDesc[] = {
	{ "g_capame.bin", 0x100000, 0x43225612, BRF_PRG },
};

STD_ROM_PICK(g_capame);
STD_ROM_FN(g_capame);

struct BurnDriverD BurnDrvMDg_capame = {
	"g_capame", "g_capa", NULL, "1900",
	"Captain America and the Avengers (E)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_capameRomInfo, g_capameRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Chiki Chiki Boys (UE) [!]
static struct BurnRomInfo g_chikRomDesc[] = {
	{ "g_chik.bin", 0x100000, 0x813a7d62, BRF_PRG },
};

STD_ROM_PICK(g_chik);
STD_ROM_FN(g_chik);

struct BurnDriverD BurnDrvMDg_chik = {
	"g_chik", NULL, NULL, "1900",
	"Chiki Chiki Boys (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_chikRomInfo, g_chikRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Chiki Chiki Boys (J) [!]
static struct BurnRomInfo g_chikijRomDesc[] = {
	{ "g_chikij.bin", 0x100000, 0x06918c17, BRF_PRG },
};

STD_ROM_PICK(g_chikij);
STD_ROM_FN(g_chikij);

struct BurnDriverD BurnDrvMDg_chikij = {
	"g_chikij", "g_chik", NULL, "1900",
	"Chiki Chiki Boys (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_chikijRomInfo, g_chikijRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Clue (U) [!]
static struct BurnRomInfo g_clueRomDesc[] = {
	{ "g_clue.bin", 0x80000, 0x7753a296, BRF_PRG },
};

STD_ROM_PICK(g_clue);
STD_ROM_FN(g_clue);

struct BurnDriverD BurnDrvMDg_clue = {
	"g_clue", NULL, NULL, "1900",
	"Clue (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_clueRomInfo, g_clueRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Cuty Suzuki no Ringside Angel (J) [!]
static struct BurnRomInfo g_cutyRomDesc[] = {
	{ "g_cuty.bin", 0x80000, 0x13795dca, BRF_PRG },
};

STD_ROM_PICK(g_cuty);
STD_ROM_FN(g_cuty);

struct BurnDriverD BurnDrvMDg_cuty = {
	"g_cuty", NULL, NULL, "1900",
	"Cuty Suzuki no Ringside Angel (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_cutyRomInfo, g_cutyRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// D&D - Warriors of the Eternal Sun (UE) [!]
static struct BurnRomInfo g_ddweRomDesc[] = {
	{ "g_ddwe.bin", 0x100000, 0xaf4a9cd7, BRF_PRG },
};

STD_ROM_PICK(g_ddwe);
STD_ROM_FN(g_ddwe);

struct BurnDriverD BurnDrvMDg_ddwe = {
	"g_ddwe", NULL, NULL, "1900",
	"D&D - Warriors of the Eternal Sun (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ddweRomInfo, g_ddweRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Dino Land (U) [!]
static struct BurnRomInfo g_dlndRomDesc[] = {
	{ "g_dlnd.bin", 0x80000, 0x5fe351b8, BRF_PRG },
};

STD_ROM_PICK(g_dlnd);
STD_ROM_FN(g_dlnd);

struct BurnDriverD BurnDrvMDg_dlnd = {
	"g_dlnd", NULL, NULL, "1900",
	"Dino Land (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_dlndRomInfo, g_dlndRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Dino Land (J) [!]
static struct BurnRomInfo g_dinoljRomDesc[] = {
	{ "g_dinolj.bin", 0x80000, 0x81f939de, BRF_PRG },
};

STD_ROM_PICK(g_dinolj);
STD_ROM_FN(g_dinolj);

struct BurnDriverD BurnDrvMDg_dinolj = {
	"g_dinolj", "g_dlnd", NULL, "1900",
	"Dino Land (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_dinoljRomInfo, g_dinoljRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// ESPN Baseball Tonight (U) [!]
static struct BurnRomInfo g_e_btRomDesc[] = {
	{ "g_e_bt.bin", 0x200000, 0x96d8440c, BRF_PRG },
};

STD_ROM_PICK(g_e_bt);
STD_ROM_FN(g_e_bt);

struct BurnDriverD BurnDrvMDg_e_bt = {
	"g_e_bt", NULL, NULL, "1900",
	"ESPN Baseball Tonight (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_e_btRomInfo, g_e_btRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// ESPN National Hockey Night (U) [c][!]
static struct BurnRomInfo g_e_hnRomDesc[] = {
	{ "g_e_hn.bin", 0x200000, 0x1d08828c, BRF_PRG },
};

STD_ROM_PICK(g_e_hn);
STD_ROM_FN(g_e_hn);

struct BurnDriverD BurnDrvMDg_e_hn = {
	"g_e_hn", NULL, NULL, "1900",
	"ESPN National Hockey Night (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_e_hnRomInfo, g_e_hnRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// ESPN National Hockey Night (Beta)
static struct BurnRomInfo g_e_nhbRomDesc[] = {
	{ "g_e_nhb.bin", 0x200000, 0xa427814a, BRF_PRG },
};

STD_ROM_PICK(g_e_nhb);
STD_ROM_FN(g_e_nhb);

struct BurnDriverD BurnDrvMDg_e_nhb = {
	"g_e_nhb", "g_e_hn", NULL, "1900",
	"ESPN National Hockey Night (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_e_nhbRomInfo, g_e_nhbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// ESPN Sunday Night NFL (U) [!]
static struct BurnRomInfo g_e_snRomDesc[] = {
	{ "g_e_sn.bin", 0x200000, 0x61e9c309, BRF_PRG },
};

STD_ROM_PICK(g_e_sn);
STD_ROM_FN(g_e_sn);

struct BurnDriverD BurnDrvMDg_e_sn = {
	"g_e_sn", NULL, NULL, "1900",
	"ESPN Sunday Night NFL (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_e_snRomInfo, g_e_snRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Evander Holyfield's Real Deal Boxing (JU) [!]
static struct BurnRomInfo g_ehrdRomDesc[] = {
	{ "g_ehrd.bin", 0x80000, 0x4fef37c8, BRF_PRG },
};

STD_ROM_PICK(g_ehrd);
STD_ROM_FN(g_ehrd);

struct BurnDriverD BurnDrvMDg_ehrd = {
	"g_ehrd", NULL, NULL, "1900",
	"Evander Holyfield's Real Deal Boxing (JU) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ehrdRomInfo, g_ehrdRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Family Feud (U) [!]
static struct BurnRomInfo g_feudRomDesc[] = {
	{ "g_feud.bin", 0x80000, 0x1aa628b0, BRF_PRG },
};

STD_ROM_PICK(g_feud);
STD_ROM_FN(g_feud);

struct BurnDriverD BurnDrvMDg_feud = {
	"g_feud", NULL, NULL, "1900",
	"Family Feud (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_feudRomInfo, g_feudRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Fatal Labyrinth (JU) [!]
static struct BurnRomInfo g_fatlRomDesc[] = {
	{ "g_fatl.bin", 0x20000, 0x5f0bd984, BRF_PRG },
};

STD_ROM_PICK(g_fatl);
STD_ROM_FN(g_fatl);

struct BurnDriverD BurnDrvMDg_fatl = {
	"g_fatl", NULL, NULL, "1900",
	"Fatal Labyrinth (JU) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_fatlRomInfo, g_fatlRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// FIFA International Soccer (UE) (M4) [!]
static struct BurnRomInfo g_fifaRomDesc[] = {
	{ "g_fifa.bin", 0x200000, 0xbddbb763, BRF_PRG },
};

STD_ROM_PICK(g_fifa);
STD_ROM_FN(g_fifa);

struct BurnDriver BurnDrvMDg_fifa = {
	"g_fifa", NULL, NULL, "1993",
	"FIFA International Soccer (UE) (M4) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_fifaRomInfo, g_fifaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Growl (U) [!]
static struct BurnRomInfo g_grwlRomDesc[] = {
	{ "g_grwl.bin", 0x80000, 0xf60ef143, BRF_PRG },
};

STD_ROM_PICK(g_grwl);
STD_ROM_FN(g_grwl);

struct BurnDriverD BurnDrvMDg_grwl = {
	"g_grwl", NULL, NULL, "1900",
	"Growl (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_grwlRomInfo, g_grwlRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Runark (J) [!]
static struct BurnRomInfo g_runarkRomDesc[] = {
	{ "g_runark.bin", 0x80000, 0x0894d8fb, BRF_PRG },
};

STD_ROM_PICK(g_runark);
STD_ROM_FN(g_runark);

struct BurnDriverD BurnDrvMDg_runark = {
	"g_runark", "g_grwl", NULL, "1900",
	"Runark (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_runarkRomInfo, g_runarkRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Heavy Nova (U) [!]
static struct BurnRomInfo g_hnovRomDesc[] = {
	{ "g_hnov.bin", 0x100000, 0xf6b6a9d8, BRF_PRG },
};

STD_ROM_PICK(g_hnov);
STD_ROM_FN(g_hnov);

struct BurnDriverD BurnDrvMDg_hnov = {
	"g_hnov", NULL, NULL, "1900",
	"Heavy Nova (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_hnovRomInfo, g_hnovRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Hit The Ice (U) [c][!]
static struct BurnRomInfo g_hiceRomDesc[] = {
	{ "g_hice.bin", 0x80000, 0x85b23606, BRF_PRG },
};

STD_ROM_PICK(g_hice);
STD_ROM_FN(g_hice);

struct BurnDriverD BurnDrvMDg_hice = {
	"g_hice", NULL, NULL, "1900",
	"Hit The Ice (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_hiceRomInfo, g_hiceRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Jack Nicklaus' Power Challenge Golf (UE) [!]
static struct BurnRomInfo g_jnpgRomDesc[] = {
	{ "g_jnpg.bin", 0x100000, 0x5545e909, BRF_PRG },
};

STD_ROM_PICK(g_jnpg);
STD_ROM_FN(g_jnpg);

struct BurnDriverD BurnDrvMDg_jnpg = {
	"g_jnpg", NULL, NULL, "1900",
	"Jack Nicklaus' Power Challenge Golf (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jnpgRomInfo, g_jnpgRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// King of the Monsters (U) [!]
static struct BurnRomInfo g_ktmRomDesc[] = {
	{ "g_ktm.bin", 0x100000, 0xf390d406, BRF_PRG },
};

STD_ROM_PICK(g_ktm);
STD_ROM_FN(g_ktm);

struct BurnDriverD BurnDrvMDg_ktm = {
	"g_ktm", NULL, NULL, "1900",
	"King of the Monsters (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ktmRomInfo, g_ktmRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// King of the Monsters (E) [!]
static struct BurnRomInfo g_kotmeRomDesc[] = {
	{ "g_kotme.bin", 0x100000, 0x7a94fd49, BRF_PRG },
};

STD_ROM_PICK(g_kotme);
STD_ROM_FN(g_kotme);

struct BurnDriverD BurnDrvMDg_kotme = {
	"g_kotme", "g_ktm", NULL, "1900",
	"King of the Monsters (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_kotmeRomInfo, g_kotmeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Klax (UE) [!]
static struct BurnRomInfo g_klaxRomDesc[] = {
	{ "g_klax.bin", 0x40000, 0x248cd09e, BRF_PRG },
};

STD_ROM_PICK(g_klax);
STD_ROM_FN(g_klax);

struct BurnDriverD BurnDrvMDg_klax = {
	"g_klax", NULL, NULL, "1900",
	"Klax (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_klaxRomInfo, g_klaxRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Klax (J) [c][!]
static struct BurnRomInfo g_klaxjRomDesc[] = {
	{ "g_klaxj.bin", 0x40000, 0x1afcc1da, BRF_PRG },
};

STD_ROM_PICK(g_klaxj);
STD_ROM_FN(g_klaxj);

struct BurnDriverD BurnDrvMDg_klaxj = {
	"g_klaxj", "g_klax", NULL, "1900",
	"Klax (J) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_klaxjRomInfo, g_klaxjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// King of the Monsters 2 (U) [!]
static struct BurnRomInfo g_ktm2RomDesc[] = {
	{ "g_ktm2.bin", 0x200000, 0xee1638ac, BRF_PRG },
};

STD_ROM_PICK(g_ktm2);
STD_ROM_FN(g_ktm2);

struct BurnDriverD BurnDrvMDg_ktm2 = {
	"g_ktm2", NULL, NULL, "1900",
	"King of the Monsters 2 (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ktm2RomInfo, g_ktm2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mario Andretti Racing (UE) [!]
static struct BurnRomInfo g_mamrRomDesc[] = {
	{ "g_mamr.bin", 0x200000, 0x7f1dc0aa, BRF_PRG },
};

STD_ROM_PICK(g_mamr);
STD_ROM_FN(g_mamr);

struct BurnDriverD BurnDrvMDg_mamr = {
	"g_mamr", NULL, NULL, "1900",
	"Mario Andretti Racing (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mamrRomInfo, g_mamrRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Master of Monsters (U) [!]
static struct BurnRomInfo g_mamoRomDesc[] = {
	{ "g_mamo.bin", 0x80000, 0x91354820, BRF_PRG },
};

STD_ROM_PICK(g_mamo);
STD_ROM_FN(g_mamo);

struct BurnDriverD BurnDrvMDg_mamo = {
	"g_mamo", NULL, NULL, "1900",
	"Master of Monsters (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mamoRomInfo, g_mamoRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Master of Monsters (J) [!]
static struct BurnRomInfo g_momonjRomDesc[] = {
	{ "g_momonj.bin", 0x80000, 0xd51ee8c2, BRF_PRG },
};

STD_ROM_PICK(g_momonj);
STD_ROM_FN(g_momonj);

struct BurnDriverD BurnDrvMDg_momonj = {
	"g_momonj", "g_mamo", NULL, "1900",
	"Master of Monsters (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_momonjRomInfo, g_momonjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mega Bomberman - 8 Player Demo (Unl)
static struct BurnRomInfo g_mb8pRomDesc[] = {
	{ "g_mb8p.bin", 0x100000, 0xd41c0d81, BRF_PRG },
};

STD_ROM_PICK(g_mb8p);
STD_ROM_FN(g_mb8p);

struct BurnDriverD BurnDrvMDg_mb8p = {
	"g_mb8p", NULL, NULL, "1900",
	"Mega Bomberman - 8 Player Demo (Unl)\0", NULL, "Factor 5", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mb8pRomInfo, g_mb8pRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// New Zealand Story, The (J) [!]
static struct BurnRomInfo g_tnzsRomDesc[] = {
	{ "g_tnzs.bin", 0x80000, 0x1c77ad21, BRF_PRG },
};

STD_ROM_PICK(g_tnzs);
STD_ROM_FN(g_tnzs);

struct BurnDriverD BurnDrvMDg_tnzs = {
	"g_tnzs", NULL, NULL, "1900",
	"New Zealand Story, The (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tnzsRomInfo, g_tnzsRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// No Escape (U) [!]
static struct BurnRomInfo g_noesRomDesc[] = {
	{ "g_noes.bin", 0x200000, 0x44ee5f20, BRF_PRG },
};

STD_ROM_PICK(g_noes);
STD_ROM_FN(g_noes);

struct BurnDriverD BurnDrvMDg_noes = {
	"g_noes", NULL, NULL, "1900",
	"No Escape (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_noesRomInfo, g_noesRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Normy's Beach Babe-O-Rama (UE) [!]
static struct BurnRomInfo g_normRomDesc[] = {
	{ "g_norm.bin", 0x100000, 0xb56a8220, BRF_PRG },
};

STD_ROM_PICK(g_norm);
STD_ROM_FN(g_norm);

struct BurnDriverD BurnDrvMDg_norm = {
	"g_norm", NULL, NULL, "1900",
	"Normy's Beach Babe-O-Rama (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_normRomInfo, g_normRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Paperboy (UE) [!]
static struct BurnRomInfo g_papbRomDesc[] = {
	{ "g_papb.bin", 0x80000, 0x0a44819b, BRF_PRG },
};

STD_ROM_PICK(g_papb);
STD_ROM_FN(g_papb);

struct BurnDriverD BurnDrvMDg_papb = {
	"g_papb", NULL, NULL, "1900",
	"Paperboy (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_papbRomInfo, g_papbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Paperboy (J) [!]
static struct BurnRomInfo g_pboyjRomDesc[] = {
	{ "g_pboyj.bin", 0x80000, 0xe14250ae, BRF_PRG },
};

STD_ROM_PICK(g_pboyj);
STD_ROM_FN(g_pboyj);

struct BurnDriverD BurnDrvMDg_pboyj = {
	"g_pboyj", "g_papb", NULL, "1900",
	"Paperboy (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pboyjRomInfo, g_pboyjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Paperboy II (U) [!]
static struct BurnRomInfo g_pap2RomDesc[] = {
	{ "g_pap2.bin", 0x100000, 0x1de28bb1, BRF_PRG },
};

STD_ROM_PICK(g_pap2);
STD_ROM_FN(g_pap2);

struct BurnDriverD BurnDrvMDg_pap2 = {
	"g_pap2", NULL, NULL, "1900",
	"Paperboy II (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pap2RomInfo, g_pap2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Populous II - Two Tribes (E) [!]
static struct BurnRomInfo g_pop2RomDesc[] = {
	{ "g_pop2.bin", 0x100000, 0xee988bd9, BRF_PRG },
};

STD_ROM_PICK(g_pop2);
STD_ROM_FN(g_pop2);

struct BurnDriverD BurnDrvMDg_pop2 = {
	"g_pop2", NULL, NULL, "1900",
	"Populous II - Two Tribes (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pop2RomInfo, g_pop2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Quad Challenge (U) [c][!]
static struct BurnRomInfo g_quadRomDesc[] = {
	{ "g_quad.bin", 0x80000, 0x74736a80, BRF_PRG },
};

STD_ROM_PICK(g_quad);
STD_ROM_FN(g_quad);

struct BurnDriverD BurnDrvMDg_quad = {
	"g_quad", NULL, NULL, "1900",
	"Quad Challenge (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_quadRomInfo, g_quadRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// MegaTraX (J) [c][!]
static struct BurnRomInfo g_mtrxRomDesc[] = {
	{ "g_mtrx.bin", 0x80000, 0xa0837741, BRF_PRG },
};

STD_ROM_PICK(g_mtrx);
STD_ROM_FN(g_mtrx);

struct BurnDriverD BurnDrvMDg_mtrx = {
	"g_mtrx", "g_quad", NULL, "1900",
	"MegaTraX (J) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mtrxRomInfo, g_mtrxRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Risk (U) [!]
static struct BurnRomInfo g_riskRomDesc[] = {
	{ "g_risk.bin", 0x80000, 0x80416d0d, BRF_PRG },
};

STD_ROM_PICK(g_risk);
STD_ROM_FN(g_risk);

struct BurnDriverD BurnDrvMDg_risk = {
	"g_risk", NULL, NULL, "1900",
	"Risk (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_riskRomInfo, g_riskRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Rise of the Robots (E) [!]
static struct BurnRomInfo g_riseRomDesc[] = {
	{ "g_rise.bin", 0x300000, 0x5650780b, BRF_PRG },
};

STD_ROM_PICK(g_rise);
STD_ROM_FN(g_rise);

struct BurnDriverD BurnDrvMDg_rise = {
	"g_rise", NULL, NULL, "1900",
	"Rise of the Robots (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_riseRomInfo, g_riseRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Shove It! - The Warehouse Game (U) [!]
static struct BurnRomInfo g_shovRomDesc[] = {
	{ "g_shov.bin", 0x20000, 0xc51f40cb, BRF_PRG },
};

STD_ROM_PICK(g_shov);
STD_ROM_FN(g_shov);

struct BurnDriverD BurnDrvMDg_shov = {
	"g_shov", NULL, NULL, "1900",
	"Shove It! - The Warehouse Game (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_shovRomInfo, g_shovRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sokoban (J) [!]
static struct BurnRomInfo g_sokobaRomDesc[] = {
	{ "g_sokoba.bin", 0x40000, 0x40f4aacc, BRF_PRG },
};

STD_ROM_PICK(g_sokoba);
STD_ROM_FN(g_sokoba);

struct BurnDriverD BurnDrvMDg_sokoba = {
	"g_sokoba", "g_shov", NULL, "1900",
	"Sokoban (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sokobaRomInfo, g_sokobaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Star Trek - Deep Space 9 - Crossroads of Time (U) [!]
static struct BurnRomInfo g_std9RomDesc[] = {
	{ "g_std9.bin", 0x100000, 0xa771e1a4, BRF_PRG },
};

STD_ROM_PICK(g_std9);
STD_ROM_FN(g_std9);

struct BurnDriverD BurnDrvMDg_std9 = {
	"g_std9", NULL, NULL, "1900",
	"Star Trek - Deep Space 9 - Crossroads of Time (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_std9RomInfo, g_std9RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Star Trek - Deep Space 9 - Crossroads of Time (E)
static struct BurnRomInfo g_stds9eRomDesc[] = {
	{ "g_stds9e.bin", 0x100000, 0xd4b122f9, BRF_PRG },
};

STD_ROM_PICK(g_stds9e);
STD_ROM_FN(g_stds9e);

struct BurnDriverD BurnDrvMDg_stds9e = {
	"g_stds9e", "g_std9", NULL, "1900",
	"Star Trek - Deep Space 9 - Crossroads of Time (E)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_stds9eRomInfo, g_stds9eRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Star Trek - The Next Generation (U) (REV01) [!]
static struct BurnRomInfo g_stngRomDesc[] = {
	{ "g_stng.bin", 0x200000, 0xef840ef2, BRF_PRG },
};

STD_ROM_PICK(g_stng);
STD_ROM_FN(g_stng);

struct BurnDriverD BurnDrvMDg_stng = {
	"g_stng", NULL, NULL, "1900",
	"Star Trek - The Next Generation (U) (REV01) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_stngRomInfo, g_stngRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Star Trek - The Next Generation (U) (REV00) [!]
static struct BurnRomInfo g_stng00RomDesc[] = {
	{ "g_stng00.bin", 0x200000, 0x272153fb, BRF_PRG },
};

STD_ROM_PICK(g_stng00);
STD_ROM_FN(g_stng00);

struct BurnDriverD BurnDrvMDg_stng00 = {
	"g_stng00", "g_stng", NULL, "1900",
	"Star Trek - The Next Generation (U) (REV00) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_stng00RomInfo, g_stng00RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// T2 - The Arcade Game (UE) (REV01) [!]
static struct BurnRomInfo g_t2arRomDesc[] = {
	{ "g_t2ar.bin", 0x100000, 0xa1264f17, BRF_PRG },
};

STD_ROM_PICK(g_t2ar);
STD_ROM_FN(g_t2ar);

struct BurnDriverD BurnDrvMDg_t2ar = {
	"g_t2ar", NULL, NULL, "1900",
	"T2 - The Arcade Game (UE) (REV01) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_t2arRomInfo, g_t2arRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// T2 - The Arcade Game (Beta)
static struct BurnRomInfo g_t2arcbRomDesc[] = {
	{ "g_t2arcb.bin", 0x100000, 0x94255703, BRF_PRG },
};

STD_ROM_PICK(g_t2arcb);
STD_ROM_FN(g_t2arcb);

struct BurnDriverD BurnDrvMDg_t2arcb = {
	"g_t2arcb", "g_t2ar", NULL, "1900",
	"T2 - The Arcade Game (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_t2arcbRomInfo, g_t2arcbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Valis (U) [!]
static struct BurnRomInfo g_valiRomDesc[] = {
	{ "g_vali.bin", 0x100000, 0x13bc5b72, BRF_PRG },
};

STD_ROM_PICK(g_vali);
STD_ROM_FN(g_vali);

struct BurnDriverD BurnDrvMDg_vali = {
	"g_vali", NULL, NULL, "1900",
	"Valis (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_valiRomInfo, g_valiRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Valis (J) [!]
static struct BurnRomInfo g_valjRomDesc[] = {
	{ "g_valj.bin", 0x100000, 0x24431625, BRF_PRG },
};

STD_ROM_PICK(g_valj);
STD_ROM_FN(g_valj);

struct BurnDriverD BurnDrvMDg_valj = {
	"g_valj", "g_vali", NULL, "1900",
	"Valis (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_valjRomInfo, g_valjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Valis III (U) [!]
static struct BurnRomInfo g_val3RomDesc[] = {
	{ "g_val3.bin", 0x100000, 0x59a2a368, BRF_PRG },
};

STD_ROM_PICK(g_val3);
STD_ROM_FN(g_val3);

struct BurnDriverD BurnDrvMDg_val3 = {
	"g_val3", NULL, NULL, "1900",
	"Valis III (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_val3RomInfo, g_val3RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Valis III (J) [!]
static struct BurnRomInfo g_val3jRomDesc[] = {
	{ "g_val3j.bin", 0x100000, 0x4d49a166, BRF_PRG },
};

STD_ROM_PICK(g_val3j);
STD_ROM_FN(g_val3j);

struct BurnDriverD BurnDrvMDg_val3j = {
	"g_val3j", "g_val3", NULL, "1900",
	"Valis III (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_val3jRomInfo, g_val3jRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Virtual Pinball (UE) [!]
static struct BurnRomInfo g_vpinRomDesc[] = {
	{ "g_vpin.bin", 0x100000, 0xd63473aa, BRF_PRG },
};

STD_ROM_PICK(g_vpin);
STD_ROM_FN(g_vpin);

struct BurnDriverD BurnDrvMDg_vpin = {
	"g_vpin", NULL, NULL, "1900",
	"Virtual Pinball (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_vpinRomInfo, g_vpinRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// NBA Live 98 (U) [!]
static struct BurnRomInfo g_nl98RomDesc[] = {
	{ "g_nl98.bin", 0x200000, 0x23473a8a, BRF_PRG },
};

STD_ROM_PICK(g_nl98);
STD_ROM_FN(g_nl98);

struct BurnDriverD BurnDrvMDg_nl98 = {
	"g_nl98", NULL, NULL, "1900",
	"NBA Live 98 (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_nl98RomInfo, g_nl98RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// NBA Live 97 (UE) [!]
static struct BurnRomInfo g_nl97RomDesc[] = {
	{ "g_nl97.bin", 0x200000, 0x7024843a, BRF_PRG },
};

STD_ROM_PICK(g_nl97);
STD_ROM_FN(g_nl97);

struct BurnDriverD BurnDrvMDg_nl97 = {
	"g_nl97", NULL, NULL, "1900",
	"NBA Live 97 (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_nl97RomInfo, g_nl97RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// NBA Live 96 (UE) [!]
static struct BurnRomInfo g_nl96RomDesc[] = {
	{ "g_nl96.bin", 0x200000, 0x49de0062, BRF_PRG },
};

STD_ROM_PICK(g_nl96);
STD_ROM_FN(g_nl96);

struct BurnDriverD BurnDrvMDg_nl96 = {
	"g_nl96", NULL, NULL, "1900",
	"NBA Live 96 (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_nl96RomInfo, g_nl96RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// NBA Live 95 (UE) [!]
static struct BurnRomInfo g_nl95RomDesc[] = {
	{ "g_nl95.bin", 0x200000, 0x66018abc, BRF_PRG },
};

STD_ROM_PICK(g_nl95);
STD_ROM_FN(g_nl95);

struct BurnDriverD BurnDrvMDg_nl95 = {
	"g_nl95", NULL, NULL, "1900",
	"NBA Live 95 (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_nl95RomInfo, g_nl95RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// NBA All-Star Challenge (UE) [!]
static struct BurnRomInfo g_nascRomDesc[] = {
	{ "g_nasc.bin", 0x100000, 0xc4674adf, BRF_PRG },
};

STD_ROM_PICK(g_nasc);
STD_ROM_FN(g_nasc);

struct BurnDriverD BurnDrvMDg_nasc = {
	"g_nasc", NULL, NULL, "1900",
	"NBA All-Star Challenge (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_nascRomInfo, g_nascRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// NBA Action (U) [!]
static struct BurnRomInfo g_nactRomDesc[] = {
	{ "g_nact.bin", 0x200000, 0x99c348ba, BRF_PRG },
};

STD_ROM_PICK(g_nact);
STD_ROM_FN(g_nact);

struct BurnDriverD BurnDrvMDg_nact = {
	"g_nact", NULL, NULL, "1994",
	"NBA Action (U) [!]\0", NULL, "Sega Sports", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_nactRomInfo, g_nactRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// NBA Action '95 (UE) [!]
static struct BurnRomInfo g_na95RomDesc[] = {
	{ "g_na95.bin", 0x200000, 0xaa7006d6, BRF_PRG },
};

STD_ROM_PICK(g_na95);
STD_ROM_FN(g_na95);

struct BurnDriverD BurnDrvMDg_na95 = {
	"g_na95", NULL, NULL, "1900",
	"NBA Action '95 (UE) [!]\0", NULL, "Sega Sports", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_na95RomInfo, g_na95RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// NBA Showdown 94 (UE) [!]
static struct BurnRomInfo g_nbs9RomDesc[] = {
	{ "g_nbs9.bin", 0x200000, 0x160b7090, BRF_PRG },
};

STD_ROM_PICK(g_nbs9);
STD_ROM_FN(g_nbs9);

struct BurnDriverD BurnDrvMDg_nbs9 = {
	"g_nbs9", NULL, NULL, "1900",
	"NBA Showdown 94 (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_nbs9RomInfo, g_nbs9RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// NBA Showdown 94 (Beta)
static struct BurnRomInfo g_nba94bRomDesc[] = {
	{ "g_nba94b.bin", 0x200000, 0x6643a308, BRF_PRG },
};

STD_ROM_PICK(g_nba94b);
STD_ROM_FN(g_nba94b);

struct BurnDriverD BurnDrvMDg_nba94b = {
	"g_nba94b", "g_nbs9", NULL, "1900",
	"NBA Showdown 94 (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_nba94bRomInfo, g_nba94bRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// NBA Pro Basketball '94 (J) [!]
static struct BurnRomInfo g_nba9RomDesc[] = {
	{ "g_nba9.bin", 0x200000, 0xeea19bce, BRF_PRG },
};

STD_ROM_PICK(g_nba9);
STD_ROM_FN(g_nba9);

struct BurnDriverD BurnDrvMDg_nba9 = {
	"g_nba9", "g_nbs9", NULL, "1900",
	"NBA Pro Basketball '94 (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_nba9RomInfo, g_nba9RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// NCAA College Football (U) [!]
static struct BurnRomInfo g_nccfRomDesc[] = {
	{ "g_nccf.bin", 0x100000, 0x081012f0, BRF_PRG },
};

STD_ROM_PICK(g_nccf);
STD_ROM_FN(g_nccf);

struct BurnDriverD BurnDrvMDg_nccf = {
	"g_nccf", NULL, NULL, "1900",
	"NCAA College Football (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_nccfRomInfo, g_nccfRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// NCAA Final Four College Basketball (U) [!]
static struct BurnRomInfo g_ncffRomDesc[] = {
	{ "g_ncff.bin", 0x180000, 0xed0c1303, BRF_PRG },
};

STD_ROM_PICK(g_ncff);
STD_ROM_FN(g_ncff);

struct BurnDriverD BurnDrvMDg_ncff = {
	"g_ncff", NULL, NULL, "1900",
	"NCAA Final Four College Basketball (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ncffRomInfo, g_ncffRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// NFL 98 (U) [!]
static struct BurnRomInfo g_nfl8RomDesc[] = {
	{ "g_nfl8.bin", 0x200000, 0xf73ec54c, BRF_PRG },
};

STD_ROM_PICK(g_nfl8);
STD_ROM_FN(g_nfl8);

struct BurnDriverD BurnDrvMDg_nfl8 = {
	"g_nfl8", NULL, NULL, "1997",
	"NFL 98 (U) [!]\0", NULL, "Sega Sports", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_nfl8RomInfo, g_nfl8RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// NFL '95 (U) [!]
static struct BurnRomInfo g_nfl5RomDesc[] = {
	{ "g_nfl5.bin", 0x200000, 0xb58e4a81, BRF_PRG },
};

STD_ROM_PICK(g_nfl5);
STD_ROM_FN(g_nfl5);

struct BurnDriverD BurnDrvMDg_nfl5 = {
	"g_nfl5", NULL, NULL, "1994",
	"NFL '95 (U) [!]\0", NULL, "Sega Sports", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_nfl5RomInfo, g_nfl5RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// NFL Football '94 Starring Joe Montana (U) [!]
static struct BurnRomInfo g_nfl4RomDesc[] = {
	{ "g_nfl4.bin", 0x200000, 0x0d486ed5, BRF_PRG },
};

STD_ROM_PICK(g_nfl4);
STD_ROM_FN(g_nfl4);

struct BurnDriverD BurnDrvMDg_nfl4 = {
	"g_nfl4", NULL, NULL, "1900",
	"NFL Football '94 Starring Joe Montana (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_nfl4RomInfo, g_nfl4RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// NFL Football '94 Starring Joe Montana (J) [!]
static struct BurnRomInfo g_nfl94jRomDesc[] = {
	{ "g_nfl94j.bin", 0x200000, 0xe490dc4a, BRF_PRG },
};

STD_ROM_PICK(g_nfl94j);
STD_ROM_FN(g_nfl94j);

struct BurnDriverD BurnDrvMDg_nfl94j = {
	"g_nfl94j", "g_nfl4", NULL, "1900",
	"NFL Football '94 Starring Joe Montana (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_nfl94jRomInfo, g_nfl94jRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// NFL Prime Time (U) [!]
static struct BurnRomInfo g_nfptRomDesc[] = {
	{ "g_nfpt.bin", 0x200000, 0x5aa53cbc, BRF_PRG },
};

STD_ROM_PICK(g_nfpt);
STD_ROM_FN(g_nfpt);

struct BurnDriverD BurnDrvMDg_nfpt = {
	"g_nfpt", NULL, NULL, "1995",
	"NFL Prime Time (U) [!]\0", NULL, "Sega Sports", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_nfptRomInfo, g_nfptRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// NFL Quarterback Club 96 (UE) [!]
static struct BurnRomInfo g_nqc6RomDesc[] = {
	{ "g_nqc6.bin", 0x400000, 0xd5a37cab, BRF_PRG },
};

STD_ROM_PICK(g_nqc6);
STD_ROM_FN(g_nqc6);

struct BurnDriverD BurnDrvMDg_nqc6 = {
	"g_nqc6", NULL, NULL, "1900",
	"NFL Quarterback Club 96 (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_nqc6RomInfo, g_nqc6RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// NFL Quarterback Club (W) [!]
static struct BurnRomInfo g_nqcRomDesc[] = {
	{ "g_nqc.bin", 0x300000, 0x94542eaf, BRF_PRG },
};

STD_ROM_PICK(g_nqc);
STD_ROM_FN(g_nqc);

struct BurnDriverD BurnDrvMDg_nqc = {
	"g_nqc", NULL, NULL, "1900",
	"NFL Quarterback Club (W) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_nqcRomInfo, g_nqcRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// NHL 95 (UE) [!]
static struct BurnRomInfo g_nh95RomDesc[] = {
	{ "g_nh95.bin", 0x200000, 0xe8ee917e, BRF_PRG },
};

STD_ROM_PICK(g_nh95);
STD_ROM_FN(g_nh95);

struct BurnDriverD BurnDrvMDg_nh95 = {
	"g_nh95", NULL, NULL, "1900",
	"NHL 95 (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_nh95RomInfo, g_nh95RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// NHL 95 Elitserien (E) [c][!]
static struct BurnRomInfo g_nh9eRomDesc[] = {
	{ "g_nh9e.bin", 0x200000, 0xe10a25c0, BRF_PRG },
};

STD_ROM_PICK(g_nh9e);
STD_ROM_FN(g_nh9e);

struct BurnDriverD BurnDrvMDg_nh9e = {
	"g_nh9e", "g_nh95", NULL, "1900",
	"NHL 95 Elitserien (E) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_nh9eRomInfo, g_nh9eRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// NHL 96 (UE) [!]
static struct BurnRomInfo g_nh96RomDesc[] = {
	{ "g_nh96.bin", 0x200000, 0x8135702c, BRF_PRG },
};

STD_ROM_PICK(g_nh96);
STD_ROM_FN(g_nh96);

struct BurnDriverD BurnDrvMDg_nh96 = {
	"g_nh96", NULL, NULL, "1900",
	"NHL 96 (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_nh96RomInfo, g_nh96RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// NHL 96 Elitserien (E)
static struct BurnRomInfo g_nhl96eRomDesc[] = {
	{ "g_nhl96e.bin", 0x200000, 0x9821d0a3, BRF_PRG },
};

STD_ROM_PICK(g_nhl96e);
STD_ROM_FN(g_nhl96e);

struct BurnDriverD BurnDrvMDg_nhl96e = {
	"g_nhl96e", "g_nh96", NULL, "1900",
	"NHL 96 Elitserien (E)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_nhl96eRomInfo, g_nhl96eRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// NHL 97 (UE) [!]
static struct BurnRomInfo g_nh97RomDesc[] = {
	{ "g_nh97.bin", 0x200000, 0xf067c103, BRF_PRG },
};

STD_ROM_PICK(g_nh97);
STD_ROM_FN(g_nh97);

struct BurnDriverD BurnDrvMDg_nh97 = {
	"g_nh97", NULL, NULL, "1900",
	"NHL 97 (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_nh97RomInfo, g_nh97RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// NHL All-Star Hockey '95 (U) [!]
static struct BurnRomInfo g_nashRomDesc[] = {
	{ "g_nash.bin", 0x200000, 0xe6c0218b, BRF_PRG },
};

STD_ROM_PICK(g_nash);
STD_ROM_FN(g_nash);

struct BurnDriverD BurnDrvMDg_nash = {
	"g_nash", NULL, NULL, "1900",
	"NHL All-Star Hockey '95 (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_nashRomInfo, g_nashRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// NHLPA Hockey '93 (UE) (REV01) [!]
static struct BurnRomInfo g_nhlpRomDesc[] = {
	{ "g_nhlp.bin", 0x80000, 0xf361d0bf, BRF_PRG },
};

STD_ROM_PICK(g_nhlp);
STD_ROM_FN(g_nhlp);

struct BurnDriverD BurnDrvMDg_nhlp = {
	"g_nhlp", NULL, NULL, "1900",
	"NHLPA Hockey '93 (UE) (REV01) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_nhlpRomInfo, g_nhlpRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// NHLPA Hockey '93 (UE) (REV00)
static struct BurnRomInfo g_nhlp00RomDesc[] = {
	{ "g_nhlp00.bin", 0x80000, 0xcbbf4262, BRF_PRG },
};

STD_ROM_PICK(g_nhlp00);
STD_ROM_FN(g_nhlp00);

struct BurnDriverD BurnDrvMDg_nhlp00 = {
	"g_nhlp00", "g_nhlp", NULL, "1900",
	"NHLPA Hockey '93 (UE) (REV00)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_nhlp00RomInfo, g_nhlp00RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Olympic Summer Games Atlanta 96 (UE) [!]
static struct BurnRomInfo g_olsgRomDesc[] = {
	{ "g_olsg.bin", 0x200000, 0x9e470fb9, BRF_PRG },
};

STD_ROM_PICK(g_olsg);
STD_ROM_FN(g_olsg);

struct BurnDriverD BurnDrvMDg_olsg = {
	"g_olsg", NULL, NULL, "1900",
	"Olympic Summer Games Atlanta 96 (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_olsgRomInfo, g_olsgRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Pat Riley Basketball (U) [!]
static struct BurnRomInfo g_patbRomDesc[] = {
	{ "g_patb.bin", 0x80000, 0x3d9318a7, BRF_PRG },
};

STD_ROM_PICK(g_patb);
STD_ROM_FN(g_patb);

struct BurnDriverD BurnDrvMDg_patb = {
	"g_patb", NULL, NULL, "1990",
	"Pat Riley Basketball (U) [!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_patbRomInfo, g_patbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Super Real Basketball (E) [c][!]
static struct BurnRomInfo g_srbaRomDesc[] = {
	{ "g_srba.bin", 0x80000, 0xf04765ba, BRF_PRG },
};

STD_ROM_PICK(g_srba);
STD_ROM_FN(g_srba);

struct BurnDriverD BurnDrvMDg_srba = {
	"g_srba", "g_patb", NULL, "1900",
	"Super Real Basketball (E) [c][!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_srbaRomInfo, g_srbaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Super Real Basketball (J) [c][!]
static struct BurnRomInfo g_srealjRomDesc[] = {
	{ "g_srealj.bin", 0x80000, 0x4346e11a, BRF_PRG },
};

STD_ROM_PICK(g_srealj);
STD_ROM_FN(g_srealj);

struct BurnDriverD BurnDrvMDg_srealj = {
	"g_srealj", "g_patb", NULL, "1900",
	"Super Real Basketball (J) [c][!]\0", NULL, "Sega", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_srealjRomInfo, g_srealjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Pele! (UE) [!]
static struct BurnRomInfo g_peleRomDesc[] = {
	{ "g_pele.bin", 0x100000, 0x5a8abe51, BRF_PRG },
};

STD_ROM_PICK(g_pele);
STD_ROM_FN(g_pele);

struct BurnDriverD BurnDrvMDg_pele = {
	"g_pele", NULL, NULL, "1900",
	"Pele! (UE) [!]\0", NULL, "Accolate", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_peleRomInfo, g_peleRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Pele's World Tournament Soccer (UE) [!]
static struct BurnRomInfo g_pelwRomDesc[] = {
	{ "g_pelw.bin", 0x200000, 0x05a486e9, BRF_PRG },
};

STD_ROM_PICK(g_pelw);
STD_ROM_FN(g_pelw);

struct BurnDriverD BurnDrvMDg_pelw = {
	"g_pelw", NULL, NULL, "1900",
	"Pele's World Tournament Soccer (UE) [!]\0", NULL, "Sport Accolate", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pelwRomInfo, g_pelwRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// PGA European Tour (UE) [!]
static struct BurnRomInfo g_pgaeRomDesc[] = {
	{ "g_pgae.bin", 0x100000, 0x8ca45acd, BRF_PRG },
};

STD_ROM_PICK(g_pgae);
STD_ROM_FN(g_pgae);

struct BurnDriverD BurnDrvMDg_pgae = {
	"g_pgae", NULL, NULL, "1900",
	"PGA European Tour (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pgaeRomInfo, g_pgaeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// RBI Baseball 3 (U) [c][!]
static struct BurnRomInfo g_rb3RomDesc[] = {
	{ "g_rb3.bin", 0x80000, 0x4840348c, BRF_PRG },
};

STD_ROM_PICK(g_rb3);
STD_ROM_FN(g_rb3);

struct BurnDriverD BurnDrvMDg_rb3 = {
	"g_rb3", NULL, NULL, "1900",
	"RBI Baseball 3 (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_rb3RomInfo, g_rb3RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// RBI Baseball 4 (U) [!]
static struct BurnRomInfo g_rb4RomDesc[] = {
	{ "g_rb4.bin", 0x100000, 0xfecf9b94, BRF_PRG },
};

STD_ROM_PICK(g_rb4);
STD_ROM_FN(g_rb4);

struct BurnDriverD BurnDrvMDg_rb4 = {
	"g_rb4", NULL, NULL, "1900",
	"RBI Baseball 4 (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_rb4RomInfo, g_rb4RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// RBI Baseball 4 (Beta)
static struct BurnRomInfo g_rbi4bRomDesc[] = {
	{ "g_rbi4b.bin", 0x100000, 0xf7420278, BRF_PRG },
};

STD_ROM_PICK(g_rbi4b);
STD_ROM_FN(g_rbi4b);

struct BurnDriverD BurnDrvMDg_rbi4b = {
	"g_rbi4b", "g_rb4", NULL, "1900",
	"RBI Baseball 4 (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_rbi4bRomInfo, g_rbi4bRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// RBI Baseball 93 (U) [!]
static struct BurnRomInfo g_rb93RomDesc[] = {
	{ "g_rb93.bin", 0x100000, 0xbeafce84, BRF_PRG },
};

STD_ROM_PICK(g_rb93);
STD_ROM_FN(g_rb93);

struct BurnDriverD BurnDrvMDg_rb93 = {
	"g_rb93", NULL, NULL, "1900",
	"RBI Baseball 93 (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_rb93RomInfo, g_rb93RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// RBI Baseball 94 (UE) [!]
static struct BurnRomInfo g_rb94RomDesc[] = {
	{ "g_rb94.bin", 0x200000, 0x4eb4d5e4, BRF_PRG },
};

STD_ROM_PICK(g_rb94);
STD_ROM_FN(g_rb94);

struct BurnDriverD BurnDrvMDg_rb94 = {
	"g_rb94", NULL, NULL, "1900",
	"RBI Baseball 94 (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_rb94RomInfo, g_rb94RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Roger Clemens' MVP Baseball (U) [!]
static struct BurnRomInfo g_rcvmRomDesc[] = {
	{ "g_rcvm.bin", 0x100000, 0x83699e34, BRF_PRG },
};

STD_ROM_PICK(g_rcvm);
STD_ROM_FN(g_rcvm);

struct BurnDriverD BurnDrvMDg_rcvm = {
	"g_rcvm", NULL, NULL, "1900",
	"Roger Clemens' MVP Baseball (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_rcvmRomInfo, g_rcvmRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Rugby World Cup 1995 (UE) (M3) [!]
static struct BurnRomInfo g_rw93RomDesc[] = {
	{ "g_rw93.bin", 0x200000, 0x61f90a8a, BRF_PRG },
};

STD_ROM_PICK(g_rw93);
STD_ROM_FN(g_rw93);

struct BurnDriverD BurnDrvMDg_rw93 = {
	"g_rw93", NULL, NULL, "1900",
	"Rugby World Cup 1995 (UE) (M3) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_rw93RomInfo, g_rw93RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Shane Warne Cricket (E) [c][!]
static struct BurnRomInfo g_swcrRomDesc[] = {
	{ "g_swcr.bin", 0x100000, 0x68865f6f, BRF_PRG },
};

STD_ROM_PICK(g_swcr);
STD_ROM_FN(g_swcr);

struct BurnDriverD BurnDrvMDg_swcr = {
	"g_swcr", NULL, NULL, "1900",
	"Shane Warne Cricket (E) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_swcrRomInfo, g_swcrRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Super High Impact (UE) [!]
static struct BurnRomInfo g_shiRomDesc[] = {
	{ "g_shi.bin", 0x100000, 0xb870c2f7, BRF_PRG },
};

STD_ROM_PICK(g_shi);
STD_ROM_FN(g_shi);

struct BurnDriverD BurnDrvMDg_shi = {
	"g_shi", NULL, NULL, "1900",
	"Super High Impact (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_shiRomInfo, g_shiRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Super Volleyball (U) [c][!]
static struct BurnRomInfo g_svolRomDesc[] = {
	{ "g_svol.bin", 0x40000, 0xa88fee44, BRF_PRG },
};

STD_ROM_PICK(g_svol);
STD_ROM_FN(g_svol);

struct BurnDriverD BurnDrvMDg_svol = {
	"g_svol", NULL, NULL, "1900",
	"Super Volleyball (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_svolRomInfo, g_svolRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Super Volleyball (U) [a1][!]
static struct BurnRomInfo g_svoluaRomDesc[] = {
	{ "g_svolua.bin", 0x40000, 0x85102799, BRF_PRG },
};

STD_ROM_PICK(g_svolua);
STD_ROM_FN(g_svolua);

struct BurnDriverD BurnDrvMDg_svolua = {
	"g_svolua", "g_svol", NULL, "1900",
	"Super Volleyball (U) [a1][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_svoluaRomInfo, g_svoluaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Super Volleyball (J) [x]
static struct BurnRomInfo g_svolxRomDesc[] = {
	{ "g_svolx.bin", 0x40000, 0xe2ee8ad2, BRF_PRG },
};

STD_ROM_PICK(g_svolx);
STD_ROM_FN(g_svolx);

struct BurnDriverD BurnDrvMDg_svolx = {
	"g_svolx", "g_svol", NULL, "1900",
	"Super Volleyball (J) [x]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_svolxRomInfo, g_svolxRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// World Series Baseball '96 (U) [!]
static struct BurnRomInfo g_ws96RomDesc[] = {
	{ "g_ws96.bin", 0x300000, 0x04ee8272, BRF_PRG },
};

STD_ROM_PICK(g_ws96);
STD_ROM_FN(g_ws96);

struct BurnDriverD BurnDrvMDg_ws96 = {
	"g_ws96", NULL, NULL, "1900",
	"World Series Baseball '96 (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ws96RomInfo, g_ws96RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// World Series Baseball '95 (U) [!]
static struct BurnRomInfo g_ws95RomDesc[] = {
	{ "g_ws95.bin", 0x300000, 0x25130077, BRF_PRG },
};

STD_ROM_PICK(g_ws95);
STD_ROM_FN(g_ws95);

struct BurnDriverD BurnDrvMDg_ws95 = {
	"g_ws95", NULL, NULL, "1900",
	"World Series Baseball '95 (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ws95RomInfo, g_ws95RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// World Series Baseball (U) [!]
static struct BurnRomInfo g_wsbRomDesc[] = {
	{ "g_wsb.bin", 0x200000, 0x57c1d5ec, BRF_PRG },
};

STD_ROM_PICK(g_wsb);
STD_ROM_FN(g_wsb);

struct BurnDriverD BurnDrvMDg_wsb = {
	"g_wsb", NULL, NULL, "1900",
	"World Series Baseball (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wsbRomInfo, g_wsbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// World Trophy Soccer (U) [!]
static struct BurnRomInfo g_wtsRomDesc[] = {
	{ "g_wts.bin", 0x80000, 0x6e3edc7c, BRF_PRG },
};

STD_ROM_PICK(g_wts);
STD_ROM_FN(g_wts);

struct BurnDriver BurnDrvMDg_wts = {
	"g_wts", NULL, NULL, "1900",
	"World Trophy Soccer (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wtsRomInfo, g_wtsRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// European Club Soccer (E) [!]
static struct BurnRomInfo g_ecsRomDesc[] = {
	{ "g_ecs.bin", 0x80000, 0x6a5cf104, BRF_PRG },
};

STD_ROM_PICK(g_ecs);
STD_ROM_FN(g_ecs);

struct BurnDriver BurnDrvMDg_ecs = {
	"g_ecs", "g_wts", NULL, "1900",
	"European Club Soccer (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ecsRomInfo, g_ecsRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// J. League Champion Soccer (J) [!]
static struct BurnRomInfo g_jlcsRomDesc[] = {
	{ "g_jlcs.bin", 0x80000, 0x453c405e, BRF_PRG },
};

STD_ROM_PICK(g_jlcs);
STD_ROM_FN(g_jlcs);

struct BurnDriverD BurnDrvMDg_jlcs = {
	"g_jlcs", "g_wts", NULL, "1900",
	"J. League Champion Soccer (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jlcsRomInfo, g_jlcsRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// WWF Super Wrestlemania (UE) [!]
static struct BurnRomInfo g_wfswRomDesc[] = {
	{ "g_wfsw.bin", 0x100000, 0xb929d6c5, BRF_PRG },
};

STD_ROM_PICK(g_wfsw);
STD_ROM_FN(g_wfsw);

struct BurnDriverD BurnDrvMDg_wfsw = {
	"g_wfsw", NULL, NULL, "1900",
	"WWF Super Wrestlemania (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wfswRomInfo, g_wfswRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// John Madden NFL 94 (UE) [c][!]
static struct BurnRomInfo g_ma94RomDesc[] = {
	{ "g_ma94.bin", 0x200000, 0xd14b811b, BRF_PRG },
};

STD_ROM_PICK(g_ma94);
STD_ROM_FN(g_ma94);

struct BurnDriverD BurnDrvMDg_ma94 = {
	"g_ma94", NULL, NULL, "1900",
	"John Madden NFL 94 (UE) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ma94RomInfo, g_ma94RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Madden NFL 95 (UE) [!]
static struct BurnRomInfo g_ma95RomDesc[] = {
	{ "g_ma95.bin", 0x200000, 0xdb0be0c2, BRF_PRG },
};

STD_ROM_PICK(g_ma95);
STD_ROM_FN(g_ma95);

struct BurnDriverD BurnDrvMDg_ma95 = {
	"g_ma95", NULL, NULL, "1900",
	"Madden NFL 95 (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ma95RomInfo, g_ma95RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Madden NFL 96 (UE) [!]
static struct BurnRomInfo g_ma96RomDesc[] = {
	{ "g_ma96.bin", 0x200000, 0xf126918b, BRF_PRG },
};

STD_ROM_PICK(g_ma96);
STD_ROM_FN(g_ma96);

struct BurnDriverD BurnDrvMDg_ma96 = {
	"g_ma96", NULL, NULL, "1900",
	"Madden NFL 96 (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ma96RomInfo, g_ma96RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Madden NFL 97 (UE) [!]
static struct BurnRomInfo g_ma97RomDesc[] = {
	{ "g_ma97.bin", 0x200000, 0xc4b4e112, BRF_PRG },
};

STD_ROM_PICK(g_ma97);
STD_ROM_FN(g_ma97);

struct BurnDriverD BurnDrvMDg_ma97 = {
	"g_ma97", NULL, NULL, "1900",
	"Madden NFL 97 (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ma97RomInfo, g_ma97RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Madden NFL 98 (U) [c][!]
static struct BurnRomInfo g_ma98RomDesc[] = {
	{ "g_ma98.bin", 0x200000, 0xe051ea62, BRF_PRG },
};

STD_ROM_PICK(g_ma98);
STD_ROM_FN(g_ma98);

struct BurnDriverD BurnDrvMDg_ma98 = {
	"g_ma98", NULL, NULL, "1900",
	"Madden NFL 98 (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ma98RomInfo, g_ma98RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// John Madden Football '93 (UE) [c][!]
static struct BurnRomInfo g_ma93RomDesc[] = {
	{ "g_ma93.bin", 0x100000, 0xca323b3e, BRF_PRG },
};

STD_ROM_PICK(g_ma93);
STD_ROM_FN(g_ma93);

struct BurnDriverD BurnDrvMDg_ma93 = {
	"g_ma93", NULL, NULL, "1900",
	"John Madden Football '93 (UE) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ma93RomInfo, g_ma93RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// John Madden Football '92 (UE) [!]
static struct BurnRomInfo g_ma92RomDesc[] = {
	{ "g_ma92.bin", 0x80000, 0x046e3945, BRF_PRG },
};

STD_ROM_PICK(g_ma92);
STD_ROM_FN(g_ma92);

struct BurnDriver BurnDrvMDg_ma92 = {
	"g_ma92", NULL, NULL, "1900",
	"John Madden Football '92 (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ma92RomInfo, g_ma92RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// John Madden Football '93 - Championship Edition (U) [!]
static struct BurnRomInfo g_ma3cRomDesc[] = {
	{ "g_ma3c.bin", 0x100000, 0xca534b1a, BRF_PRG },
};

STD_ROM_PICK(g_ma3c);
STD_ROM_FN(g_ma3c);

struct BurnDriverD BurnDrvMDg_ma3c = {
	"g_ma3c", NULL, NULL, "1900",
	"John Madden Football '93 - Championship Edition (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ma3cRomInfo, g_ma3cRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// John Madden Football (UE) [!]
static struct BurnRomInfo g_maRomDesc[] = {
	{ "g_ma.bin", 0x80000, 0x90fb8818, BRF_PRG },
};

STD_ROM_PICK(g_ma);
STD_ROM_FN(g_ma);

struct BurnDriver BurnDrvMDg_ma = {
	"g_ma", NULL, NULL, "1900",
	"John Madden Football (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_maRomInfo, g_maRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// John Madden Football (J) [!]
static struct BurnRomInfo g_madjRomDesc[] = {
	{ "g_madj.bin", 0x80000, 0x0460611c, BRF_PRG },
};

STD_ROM_PICK(g_madj);
STD_ROM_FN(g_madj);

struct BurnDriver BurnDrvMDg_madj = {
	"g_madj", "g_ma", NULL, "1900",
	"John Madden Football (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_madjRomInfo, g_madjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Joe Montana Football (JU) [!]
static struct BurnRomInfo g_jmofRomDesc[] = {
	{ "g_jmof.bin", 0x80000, 0x8aa6a1dd, BRF_PRG },
};

STD_ROM_PICK(g_jmof);
STD_ROM_FN(g_jmof);

struct BurnDriverD BurnDrvMDg_jmof = {
	"g_jmof", NULL, NULL, "1900",
	"Joe Montana Football (JU) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jmofRomInfo, g_jmofRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Joe Montana Sports Talk Football 2 (W) (REV01) [!]
static struct BurnRomInfo g_jms2RomDesc[] = {
	{ "g_jms2.bin", 0x100000, 0xa45da893, BRF_PRG },
};

STD_ROM_PICK(g_jms2);
STD_ROM_FN(g_jms2);

struct BurnDriver BurnDrvMDg_jms2 = {
	"g_jms2", NULL, NULL, "1900",
	"Joe Montana Sports Talk Football 2 (W) (REV01) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jms2RomInfo, g_jms2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Joe Montana Sports Talk Football 2 (W) (REV00) [!]
static struct BurnRomInfo g_jmons2RomDesc[] = {
	{ "g_jmons2.bin", 0x100000, 0xf2363a4a, BRF_PRG },
};

STD_ROM_PICK(g_jmons2);
STD_ROM_FN(g_jmons2);

struct BurnDriver BurnDrvMDg_jmons2 = {
	"g_jmons2", "g_jms2", NULL, "1900",
	"Joe Montana Sports Talk Football 2 (W) (REV00) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jmons2RomInfo, g_jmons2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Joe Montana Sports Talk Football (UE) [!]
static struct BurnRomInfo g_jmsRomDesc[] = {
	{ "g_jms.bin", 0x180000, 0xce0b1fe1, BRF_PRG },
};

STD_ROM_PICK(g_jms);
STD_ROM_FN(g_jms);

struct BurnDriverD BurnDrvMDg_jms = {
	"g_jms", NULL, NULL, "1900",
	"Joe Montana Sports Talk Football (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jmsRomInfo, g_jmsRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// HardBall '94 (UE) [!]
static struct BurnRomInfo g_hb94RomDesc[] = {
	{ "g_hb94.bin", 0x200000, 0xea9c4878, BRF_PRG },
};

STD_ROM_PICK(g_hb94);
STD_ROM_FN(g_hb94);

struct BurnDriverD BurnDrvMDg_hb94 = {
	"g_hb94", NULL, NULL, "1900",
	"HardBall '94 (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_hb94RomInfo, g_hb94RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// HardBall III (U) [!]
static struct BurnRomInfo g_hb3RomDesc[] = {
	{ "g_hb3.bin", 0x200000, 0xa4f2f011, BRF_PRG },
};

STD_ROM_PICK(g_hb3);
STD_ROM_FN(g_hb3);

struct BurnDriverD BurnDrvMDg_hb3 = {
	"g_hb3", NULL, NULL, "1900",
	"HardBall III (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_hb3RomInfo, g_hb3RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// HardBall! (U) [!]
static struct BurnRomInfo g_hbRomDesc[] = {
	{ "g_hb.bin", 0x100000, 0xbd1b9a04, BRF_PRG },
};

STD_ROM_PICK(g_hb);
STD_ROM_FN(g_hb);

struct BurnDriverD BurnDrvMDg_hb = {
	"g_hb", NULL, NULL, "1900",
	"HardBall! (U) [!]\0", NULL, "Accolate / Ballistic", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_hbRomInfo, g_hbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Coach K College Basketball (U) [!]
static struct BurnRomInfo g_coakRomDesc[] = {
	{ "g_coak.bin", 0x200000, 0x67c309c6, BRF_PRG },
};

STD_ROM_PICK(g_coak);
STD_ROM_FN(g_coak);

struct BurnDriverD BurnDrvMDg_coak = {
	"g_coak", NULL, NULL, "1900",
	"Coach K College Basketball (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_coakRomInfo, g_coakRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// College Football USA 96 (U) [!]
static struct BurnRomInfo g_cf96RomDesc[] = {
	{ "g_cf96.bin", 0x200000, 0xb9075385, BRF_PRG },
};

STD_ROM_PICK(g_cf96);
STD_ROM_FN(g_cf96);

struct BurnDriverD BurnDrvMDg_cf96 = {
	"g_cf96", NULL, NULL, "1900",
	"College Football USA 96 (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_cf96RomInfo, g_cf96RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// College Football USA 97 - The Road to New Orleans (U) [!]
static struct BurnRomInfo g_cf97RomDesc[] = {
	{ "g_cf97.bin", 0x200000, 0x2ebb90a3, BRF_PRG },
};

STD_ROM_PICK(g_cf97);
STD_ROM_FN(g_cf97);

struct BurnDriverD BurnDrvMDg_cf97 = {
	"g_cf97", NULL, NULL, "1900",
	"College Football USA 97 - The Road to New Orleans (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_cf97RomInfo, g_cf97RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// College Football's National Championship (U) [!]
static struct BurnRomInfo g_cfnRomDesc[] = {
	{ "g_cfn.bin", 0x200000, 0x172c5dbb, BRF_PRG },
};

STD_ROM_PICK(g_cfn);
STD_ROM_FN(g_cfn);

struct BurnDriverD BurnDrvMDg_cfn = {
	"g_cfn", NULL, NULL, "1994",
	"College Football's National Championship (U) [!]\0", NULL, "Sega Sports", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_cfnRomInfo, g_cfnRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// College Football's National Championship II (U) [!]
static struct BurnRomInfo g_cfn2RomDesc[] = {
	{ "g_cfn2.bin", 0x200000, 0x65b64413, BRF_PRG },
};

STD_ROM_PICK(g_cfn2);
STD_ROM_FN(g_cfn2);

struct BurnDriverD BurnDrvMDg_cfn2 = {
	"g_cfn2", NULL, NULL, "1995",
	"College Football's National Championship II (U) [!]\0", NULL, "Sega Sports", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_cfn2RomInfo, g_cfn2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// College Slam (U) [!]
static struct BurnRomInfo g_cslaRomDesc[] = {
	{ "g_csla.bin", 0x400000, 0x96a42431, BRF_PRG },
};

STD_ROM_PICK(g_csla);
STD_ROM_FN(g_csla);

struct BurnDriverD BurnDrvMDg_csla = {
	"g_csla", NULL, NULL, "1900",
	"College Slam (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_cslaRomInfo, g_cslaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Crystal's Pony Tale (U) [!]
static struct BurnRomInfo g_crysRomDesc[] = {
	{ "g_crys.bin", 0x100000, 0x6cf7a4df, BRF_PRG },
};

STD_ROM_PICK(g_crys);
STD_ROM_FN(g_crys);

struct BurnDriverD BurnDrvMDg_crys = {
	"g_crys", NULL, NULL, "1900",
	"Crystal's Pony Tale (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_crysRomInfo, g_crysRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// CyberBall (W) [!]
static struct BurnRomInfo g_cybbRomDesc[] = {
	{ "g_cybb.bin", 0x80000, 0x76120e96, BRF_PRG },
};

STD_ROM_PICK(g_cybb);
STD_ROM_FN(g_cybb);

struct BurnDriverD BurnDrvMDg_cybb = {
	"g_cybb", NULL, NULL, "1900",
	"CyberBall (W) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_cybbRomInfo, g_cybbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// David Crane's Amazing Tennis (U) [!]
static struct BurnRomInfo g_dcatRomDesc[] = {
	{ "g_dcat.bin", 0x100000, 0x9177088c, BRF_PRG },
};

STD_ROM_PICK(g_dcat);
STD_ROM_FN(g_dcat);

struct BurnDriverD BurnDrvMDg_dcat = {
	"g_dcat", NULL, NULL, "1900",
	"David Crane's Amazing Tennis (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_dcatRomInfo, g_dcatRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// David Robinson Basketball (J) [!]
static struct BurnRomInfo g_drbbRomDesc[] = {
	{ "g_drbb.bin", 0x80000, 0x56164b00, BRF_PRG },
};

STD_ROM_PICK(g_drbb);
STD_ROM_FN(g_drbb);

struct BurnDriverD BurnDrvMDg_drbb = {
	"g_drbb", NULL, NULL, "1900",
	"David Robinson Basketball (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_drbbRomInfo, g_drbbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// David Robinson's Supreme Court (U) [!]
static struct BurnRomInfo g_drscRomDesc[] = {
	{ "g_drsc.bin", 0x80000, 0x512b7599, BRF_PRG },
};

STD_ROM_PICK(g_drsc);
STD_ROM_FN(g_drsc);

struct BurnDriverD BurnDrvMDg_drsc = {
	"g_drsc", NULL, NULL, "1900",
	"David Robinson's Supreme Court (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_drscRomInfo, g_drscRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Devil's Course 3-D Golf (J) [!]
static struct BurnRomInfo g_dc3dRomDesc[] = {
	{ "g_dc3d.bin", 0x180000, 0xbd090c67, BRF_PRG },
};

STD_ROM_PICK(g_dc3d);
STD_ROM_FN(g_dc3d);

struct BurnDriverD BurnDrvMDg_dc3d = {
	"g_dc3d", NULL, NULL, "1900",
	"Devil's Course 3-D Golf (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_dc3dRomInfo, g_dc3dRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Double Dribble - The Playoff Edition (U) [!]
static struct BurnRomInfo g_dribRomDesc[] = {
	{ "g_drib.bin", 0x200000, 0x8352b1d0, BRF_PRG },
};

STD_ROM_PICK(g_drib);
STD_ROM_FN(g_drib);

struct BurnDriverD BurnDrvMDg_drib = {
	"g_drib", NULL, NULL, "1900",
	"Double Dribble - The Playoff Edition (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_dribRomInfo, g_dribRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Dragon Ball Z - Buyuu Retsuden (J) [!]
static struct BurnRomInfo g_dbzjRomDesc[] = {
	{ "g_dbzj.bin", 0x200000, 0xaf8f3371, BRF_PRG },
};

STD_ROM_PICK(g_dbzj);
STD_ROM_FN(g_dbzj);

struct BurnDriverD BurnDrvMDg_dbzj = {
	"g_dbzj", NULL, NULL, "1900",
	"Dragon Ball Z - Buyuu Retsuden (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_dbzjRomInfo, g_dbzjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Dragon Ball Z - L'Appel du Destin (F) [!]
static struct BurnRomInfo g_dbzfRomDesc[] = {
	{ "g_dbzf.bin", 0x200000, 0xf035c737, BRF_PRG },
};

STD_ROM_PICK(g_dbzf);
STD_ROM_FN(g_dbzf);

struct BurnDriverD BurnDrvMDg_dbzf = {
	"g_dbzf", "g_dbzj", NULL, "1900",
	"Dragon Ball Z - L'Appel du Destin (F) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_dbzfRomInfo, g_dbzfRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Dragon's Eye - Shanghai 3 (J) [!]
static struct BurnRomInfo g_deyeRomDesc[] = {
	{ "g_deye.bin", 0x40000, 0x81f0c3cf, BRF_PRG },
};

STD_ROM_PICK(g_deye);
STD_ROM_FN(g_deye);

struct BurnDriverD BurnDrvMDg_deye = {
	"g_deye", NULL, NULL, "1900",
	"Dragon's Eye - Shanghai 3 (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_deyeRomInfo, g_deyeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// NHL Hockey (U) [!]
static struct BurnRomInfo g_nhlhRomDesc[] = {
	{ "g_nhlh.bin", 0x80000, 0x2641653f, BRF_PRG },
};

STD_ROM_PICK(g_nhlh);
STD_ROM_FN(g_nhlh);

struct BurnDriver BurnDrvMDg_nhlh = {
	"g_nhlh", NULL, NULL, "1900",
	"NHL Hockey (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_nhlhRomInfo, g_nhlhRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// EA Hockey (E) [!]
static struct BurnRomInfo g_eahoRomDesc[] = {
	{ "g_eaho.bin", 0x80000, 0x9bfc279c, BRF_PRG },
};

STD_ROM_PICK(g_eaho);
STD_ROM_FN(g_eaho);

struct BurnDriver BurnDrvMDg_eaho = {
	"g_eaho", "g_nhlh", NULL, "1900",
	"EA Hockey (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_eahoRomInfo, g_eahoRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Pro Hockey (J) [!]
static struct BurnRomInfo g_prhoRomDesc[] = {
	{ "g_prho.bin", 0x80000, 0x9dcdc894, BRF_PRG },
};

STD_ROM_PICK(g_prho);
STD_ROM_FN(g_prho);

struct BurnDriver BurnDrvMDg_prho = {
	"g_prho", "g_nhlh", NULL, "1900",
	"Pro Hockey (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_prhoRomInfo, g_prhoRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// F1 Circus MD (J) [!]
static struct BurnRomInfo g_f1cRomDesc[] = {
	{ "g_f1c.bin", 0x80000, 0x5d30befb, BRF_PRG },
};

STD_ROM_PICK(g_f1c);
STD_ROM_FN(g_f1c);

struct BurnDriverD BurnDrvMDg_f1c = {
	"g_f1c", NULL, NULL, "1900",
	"F1 Circus MD (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_f1cRomInfo, g_f1cRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// F1 Grand Prix - Nakajima Satoru (J) [!]
static struct BurnRomInfo g_f1gpRomDesc[] = {
	{ "g_f1gp.bin", 0x100000, 0x93be47cf, BRF_PRG },
};

STD_ROM_PICK(g_f1gp);
STD_ROM_FN(g_f1gp);

struct BurnDriverD BurnDrvMDg_f1gp = {
	"g_f1gp", NULL, NULL, "1900",
	"F1 Grand Prix - Nakajima Satoru (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_f1gpRomInfo, g_f1gpRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// F1 Hero MD (J) [c][!]
static struct BurnRomInfo g_f1hRomDesc[] = {
	{ "g_f1h.bin", 0x100000, 0x24f87987, BRF_PRG },
};

STD_ROM_PICK(g_f1h);
STD_ROM_FN(g_f1h);

struct BurnDriverD BurnDrvMDg_f1h = {
	"g_f1h", NULL, NULL, "1900",
	"F1 Hero MD (J) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_f1hRomInfo, g_f1hRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// F1 Super License - Nakajima Satoru (J) [!]
static struct BurnRomInfo g_f1slRomDesc[] = {
	{ "g_f1sl.bin", 0x100000, 0x8774bc79, BRF_PRG },
};

STD_ROM_PICK(g_f1sl);
STD_ROM_FN(g_f1sl);

struct BurnDriverD BurnDrvMDg_f1sl = {
	"g_f1sl", NULL, NULL, "1900",
	"F1 Super License - Nakajima Satoru (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_f1slRomInfo, g_f1slRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Fastest 1 (J) [!]
static struct BurnRomInfo g_fas1RomDesc[] = {
	{ "g_fas1.bin", 0x80000, 0xbb43f0de, BRF_PRG },
};

STD_ROM_PICK(g_fas1);
STD_ROM_FN(g_fas1);

struct BurnDriverD BurnDrvMDg_fas1 = {
	"g_fas1", NULL, NULL, "1900",
	"Fastest 1 (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_fas1RomInfo, g_fas1RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Fire Pro Wrestling Gaiden (J) [c][!]
static struct BurnRomInfo g_fproRomDesc[] = {
	{ "g_fpro.bin", 0x80000, 0x24408c73, BRF_PRG },
};

STD_ROM_PICK(g_fpro);
STD_ROM_FN(g_fpro);

struct BurnDriverD BurnDrvMDg_fpro = {
	"g_fpro", NULL, NULL, "1900",
	"Fire Pro Wrestling Gaiden (J) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_fproRomInfo, g_fproRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Foreman For Real (W) [!]
static struct BurnRomInfo g_foreRomDesc[] = {
	{ "g_fore.bin", 0x300000, 0x36248f90, BRF_PRG },
};

STD_ROM_PICK(g_fore);
STD_ROM_FN(g_fore);

struct BurnDriverD BurnDrvMDg_fore = {
	"g_fore", NULL, NULL, "1900",
	"Foreman For Real (W) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_foreRomInfo, g_foreRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Fun-N-Games (U) [c][!]
static struct BurnRomInfo g_fungRomDesc[] = {
	{ "g_fung.bin", 0x100000, 0xb5ae351d, BRF_PRG },
};

STD_ROM_PICK(g_fung);
STD_ROM_FN(g_fung);

struct BurnDriverD BurnDrvMDg_fung = {
	"g_fung", NULL, NULL, "1900",
	"Fun-N-Games (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_fungRomInfo, g_fungRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Fun-N-Games (E)
static struct BurnRomInfo g_funngeRomDesc[] = {
	{ "g_funnge.bin", 0x100000, 0xda4ab3cd, BRF_PRG },
};

STD_ROM_PICK(g_funnge);
STD_ROM_FN(g_funnge);

struct BurnDriverD BurnDrvMDg_funnge = {
	"g_funnge", "g_fung", NULL, "1900",
	"Fun-N-Games (E)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_funngeRomInfo, g_funngeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Great Waldo Search, The (U) [!]
static struct BurnRomInfo g_gwsRomDesc[] = {
	{ "g_gws.bin", 0x100000, 0x8c5c93b8, BRF_PRG },
};

STD_ROM_PICK(g_gws);
STD_ROM_FN(g_gws);

struct BurnDriverD BurnDrvMDg_gws = {
	"g_gws", NULL, NULL, "1900",
	"Great Waldo Search, The (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_gwsRomInfo, g_gwsRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// IMG International Tour Tennis (UE) [!]
static struct BurnRomInfo g_imgiRomDesc[] = {
	{ "g_imgi.bin", 0x200000, 0xe04ffc2b, BRF_PRG },
};

STD_ROM_PICK(g_imgi);
STD_ROM_FN(g_imgi);

struct BurnDriverD BurnDrvMDg_imgi = {
	"g_imgi", NULL, NULL, "1900",
	"IMG International Tour Tennis (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_imgiRomInfo, g_imgiRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// International Rugby (E) [c][!]
static struct BurnRomInfo g_intrRomDesc[] = {
	{ "g_intr.bin", 0x80000, 0xd97d1699, BRF_PRG },
};

STD_ROM_PICK(g_intr);
STD_ROM_FN(g_intr);

struct BurnDriverD BurnDrvMDg_intr = {
	"g_intr", NULL, NULL, "1900",
	"International Rugby (E) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_intrRomInfo, g_intrRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Ishido - The Way of the Stones (U) [c][!]
static struct BurnRomInfo g_ishiRomDesc[] = {
	{ "g_ishi.bin", 0x20000, 0xb1de7d5e, BRF_PRG },
};

STD_ROM_PICK(g_ishi);
STD_ROM_FN(g_ishi);

struct BurnDriverD BurnDrvMDg_ishi = {
	"g_ishi", NULL, NULL, "1900",
	"Ishido - The Way of the Stones (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ishiRomInfo, g_ishiRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// J. League Pro Striker 2 (J) [!]
static struct BurnRomInfo g_jlp2RomDesc[] = {
	{ "g_jlp2.bin", 0x200000, 0x9fe71002, BRF_PRG },
};

STD_ROM_PICK(g_jlp2);
STD_ROM_FN(g_jlp2);

struct BurnDriverD BurnDrvMDg_jlp2 = {
	"g_jlp2", NULL, NULL, "1900",
	"J. League Pro Striker 2 (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jlp2RomInfo, g_jlp2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// J. League Pro Striker - Perfect Edition (J) [c][!]
static struct BurnRomInfo g_jlppRomDesc[] = {
	{ "g_jlpp.bin", 0x100000, 0x0abed379, BRF_PRG },
};

STD_ROM_PICK(g_jlpp);
STD_ROM_FN(g_jlpp);

struct BurnDriverD BurnDrvMDg_jlpp = {
	"g_jlpp", NULL, NULL, "1900",
	"J. League Pro Striker - Perfect Edition (J) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jlppRomInfo, g_jlppRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// J. League Pro Striker Final Stage (J) [c][!]
static struct BurnRomInfo g_jlpfRomDesc[] = {
	{ "g_jlpf.bin", 0x200000, 0xe35e25fb, BRF_PRG },
};

STD_ROM_PICK(g_jlpf);
STD_ROM_FN(g_jlpf);

struct BurnDriverD BurnDrvMDg_jlpf = {
	"g_jlpf", NULL, NULL, "1900",
	"J. League Pro Striker Final Stage (J) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jlpfRomInfo, g_jlpfRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// James Buster Douglas Knock Out Boxing (UE) [c][!]
static struct BurnRomInfo g_jbdbRomDesc[] = {
	{ "g_jbdb.bin", 0x80000, 0x87bbcf2a, BRF_PRG },
};

STD_ROM_PICK(g_jbdb);
STD_ROM_FN(g_jbdb);

struct BurnDriverD BurnDrvMDg_jbdb = {
	"g_jbdb", NULL, NULL, "1900",
	"James Buster Douglas Knock Out Boxing (UE) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jbdbRomInfo, g_jbdbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Final Blow (J) [c][!]
static struct BurnRomInfo g_fbloRomDesc[] = {
	{ "g_fblo.bin", 0x80000, 0x48ad505d, BRF_PRG },
};

STD_ROM_PICK(g_fblo);
STD_ROM_FN(g_fblo);

struct BurnDriverD BurnDrvMDg_fblo = {
	"g_fblo", "g_jbdb", NULL, "1900",
	"Final Blow (J) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_fbloRomInfo, g_fbloRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Jammit (U) [!]
static struct BurnRomInfo g_jammRomDesc[] = {
	{ "g_jamm.bin", 0x200000, 0xd91b52b8, BRF_PRG },
};

STD_ROM_PICK(g_jamm);
STD_ROM_FN(g_jamm);

struct BurnDriverD BurnDrvMDg_jamm = {
	"g_jamm", NULL, NULL, "1900",
	"Jammit (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jammRomInfo, g_jammRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Janou Touryumon (J) [!]
static struct BurnRomInfo g_janoRomDesc[] = {
	{ "g_jano.bin", 0x100000, 0x7011e8eb, BRF_PRG },
};

STD_ROM_PICK(g_jano);
STD_ROM_FN(g_jano);

struct BurnDriverD BurnDrvMDg_jano = {
	"g_jano", NULL, NULL, "1900",
	"Janou Touryumon (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_janoRomInfo, g_janoRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Janou Touryumon (J) [a1][!]
static struct BurnRomInfo g_janoutRomDesc[] = {
	{ "g_janout.bin", 0x100000, 0xb5ef97c6, BRF_PRG },
};

STD_ROM_PICK(g_janout);
STD_ROM_FN(g_janout);

struct BurnDriverD BurnDrvMDg_janout = {
	"g_janout", "g_jano", NULL, "1900",
	"Janou Touryumon (J) [a1][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_janoutRomInfo, g_janoutRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Jantei Monogatari (J) [!]
static struct BurnRomInfo g_jantRomDesc[] = {
	{ "g_jant.bin", 0x100000, 0x8a1b19ad, BRF_PRG },
};

STD_ROM_PICK(g_jant);
STD_ROM_FN(g_jant);

struct BurnDriverD BurnDrvMDg_jant = {
	"g_jant", NULL, NULL, "1900",
	"Jantei Monogatari (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jantRomInfo, g_jantRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Jeopardy! (U) [c][!]
static struct BurnRomInfo g_jeopRomDesc[] = {
	{ "g_jeop.bin", 0x80000, 0x56cff3f1, BRF_PRG },
};

STD_ROM_PICK(g_jeop);
STD_ROM_FN(g_jeop);

struct BurnDriverD BurnDrvMDg_jeop = {
	"g_jeop", NULL, NULL, "1900",
	"Jeopardy! (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jeopRomInfo, g_jeopRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Jeopardy! Deluxe (U) [c][!]
static struct BurnRomInfo g_jeodRomDesc[] = {
	{ "g_jeod.bin", 0x80000, 0x25e2f9d2, BRF_PRG },
};

STD_ROM_PICK(g_jeod);
STD_ROM_FN(g_jeod);

struct BurnDriverD BurnDrvMDg_jeod = {
	"g_jeod", NULL, NULL, "1900",
	"Jeopardy! Deluxe (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jeodRomInfo, g_jeodRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Jeopardy! Sports Edition (U) [!]
static struct BurnRomInfo g_jeosRomDesc[] = {
	{ "g_jeos.bin", 0x80000, 0x13f924d2, BRF_PRG },
};

STD_ROM_PICK(g_jeos);
STD_ROM_FN(g_jeos);

struct BurnDriverD BurnDrvMDg_jeos = {
	"g_jeos", NULL, NULL, "1900",
	"Jeopardy! Sports Edition (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jeosRomInfo, g_jeosRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Jerry Glanville's Pigskin Footbrawl (U) [!]
static struct BurnRomInfo g_jgpfRomDesc[] = {
	{ "g_jgpf.bin", 0x100000, 0xe7f48d30, BRF_PRG },
};

STD_ROM_PICK(g_jgpf);
STD_ROM_FN(g_jgpf);

struct BurnDriverD BurnDrvMDg_jgpf = {
	"g_jgpf", NULL, NULL, "1900",
	"Jerry Glanville's Pigskin Footbrawl (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jgpfRomInfo, g_jgpfRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Joe & Mac Caveman Ninja (U) [!]
static struct BurnRomInfo g_jmacRomDesc[] = {
	{ "g_jmac.bin", 0x100000, 0x85bcc1c7, BRF_PRG },
};

STD_ROM_PICK(g_jmac);
STD_ROM_FN(g_jmac);

struct BurnDriverD BurnDrvMDg_jmac = {
	"g_jmac", NULL, NULL, "1900",
	"Joe & Mac Caveman Ninja (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jmacRomInfo, g_jmacRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// King's Bounty (UE) [!]
static struct BurnRomInfo g_kbouRomDesc[] = {
	{ "g_kbou.bin", 0x80000, 0xaa68a92e, BRF_PRG },
};

STD_ROM_PICK(g_kbou);
STD_ROM_FN(g_kbou);

struct BurnDriverD BurnDrvMDg_kbou = {
	"g_kbou", NULL, NULL, "1900",
	"King's Bounty (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_kbouRomInfo, g_kbouRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Magic School Bus, The (U) [!]
static struct BurnRomInfo g_msbuRomDesc[] = {
	{ "g_msbu.bin", 0x100000, 0x1a5d4412, BRF_PRG },
};

STD_ROM_PICK(g_msbu);
STD_ROM_FN(g_msbu);

struct BurnDriverD BurnDrvMDg_msbu = {
	"g_msbu", NULL, NULL, "1900",
	"Magic School Bus, The (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_msbuRomInfo, g_msbuRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mamono Hunter Yohko - Makai Kara no Tenkosei (J) [!]
static struct BurnRomInfo g_mhyRomDesc[] = {
	{ "g_mhy.bin", 0x80000, 0x10bb359b, BRF_PRG },
};

STD_ROM_PICK(g_mhy);
STD_ROM_FN(g_mhy);

struct BurnDriverD BurnDrvMDg_mhy = {
	"g_mhy", NULL, NULL, "1900",
	"Mamono Hunter Yohko - Makai Kara no Tenkosei (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mhyRomInfo, g_mhyRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Marble Madness (UE) [!]
static struct BurnRomInfo g_mmadRomDesc[] = {
	{ "g_mmad.bin", 0x80000, 0x79eba28a, BRF_PRG },
};

STD_ROM_PICK(g_mmad);
STD_ROM_FN(g_mmad);

struct BurnDriverD BurnDrvMDg_mmad = {
	"g_mmad", NULL, NULL, "1900",
	"Marble Madness (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mmadRomInfo, g_mmadRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mario Lemieux Hockey (UE) [!]
static struct BurnRomInfo g_mlemRomDesc[] = {
	{ "g_mlem.bin", 0x80000, 0xf664eb6c, BRF_PRG },
};

STD_ROM_PICK(g_mlem);
STD_ROM_FN(g_mlem);

struct BurnDriverD BurnDrvMDg_mlem = {
	"g_mlem", NULL, NULL, "1900",
	"Mario Lemieux Hockey (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mlemRomInfo, g_mlemRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Master of Weapon (J) [!]
static struct BurnRomInfo g_moweRomDesc[] = {
	{ "g_mowe.bin", 0x80000, 0x12ad6178, BRF_PRG },
};

STD_ROM_PICK(g_mowe);
STD_ROM_FN(g_mowe);

struct BurnDriverD BurnDrvMDg_mowe = {
	"g_mowe", NULL, NULL, "1900",
	"Master of Weapon (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_moweRomInfo, g_moweRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Math Blaster - Episode 1 (U) [!]
static struct BurnRomInfo g_mathRomDesc[] = {
	{ "g_math.bin", 0x100000, 0xd055a462, BRF_PRG },
};

STD_ROM_PICK(g_math);
STD_ROM_FN(g_math);

struct BurnDriverD BurnDrvMDg_math = {
	"g_math", NULL, NULL, "1900",
	"Math Blaster - Episode 1 (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mathRomInfo, g_mathRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mega SWIV (E)
static struct BurnRomInfo g_mswiveRomDesc[] = {
	{ "g_mswive.bin", 0x100000, 0x78c2f046, BRF_PRG },
};

STD_ROM_PICK(g_mswive);
STD_ROM_FN(g_mswive);

struct BurnDriverD BurnDrvMDg_mswive = {
	"g_mswive", NULL, NULL, "1900",
	"Mega SWIV (E)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mswiveRomInfo, g_mswiveRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mega SWIV (E) [p1][!]
static struct BurnRomInfo g_mswiRomDesc[] = {
	{ "g_mswi.bin", 0x100000, 0x1ec66bf7, BRF_PRG },
};

STD_ROM_PICK(g_mswi);
STD_ROM_FN(g_mswi);

struct BurnDriverD BurnDrvMDg_mswi = {
	"g_mswi", "g_mswive", NULL, "1900",
	"Mega SWIV (E) [p1][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mswiRomInfo, g_mswiRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mercs (W) [!]
static struct BurnRomInfo g_mercRomDesc[] = {
	{ "g_merc.bin", 0x100000, 0x16113a72, BRF_PRG },
};

STD_ROM_PICK(g_merc);
STD_ROM_FN(g_merc);

struct BurnDriverD BurnDrvMDg_merc = {
	"g_merc", NULL, NULL, "1900",
	"Mercs (W) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mercRomInfo, g_mercRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mickey's Ultimate Challenge (U) [!]
static struct BurnRomInfo g_multRomDesc[] = {
	{ "g_mult.bin", 0x100000, 0x30b512ee, BRF_PRG },
};

STD_ROM_PICK(g_mult);
STD_ROM_FN(g_mult);

struct BurnDriverD BurnDrvMDg_mult = {
	"g_mult", NULL, NULL, "1900",
	"Mickey's Ultimate Challenge (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_multRomInfo, g_multRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Might and Magic - Gates to Another World (UE) (REV01) [!]
static struct BurnRomInfo g_mmgRomDesc[] = {
	{ "g_mmg.bin", 0xc0000, 0xf509145f, BRF_PRG },
};

STD_ROM_PICK(g_mmg);
STD_ROM_FN(g_mmg);

struct BurnDriverD BurnDrvMDg_mmg = {
	"g_mmg", NULL, NULL, "1900",
	"Might and Magic - Gates to Another World (UE) (REV01) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mmgRomInfo, g_mmgRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// MLBPA Baseball (U) [!]
static struct BurnRomInfo g_mlbbRomDesc[] = {
	{ "g_mlbb.bin", 0x200000, 0x14a8064d, BRF_PRG },
};

STD_ROM_PICK(g_mlbb);
STD_ROM_FN(g_mlbb);

struct BurnDriverD BurnDrvMDg_mlbb = {
	"g_mlbb", NULL, NULL, "1900",
	"MLBPA Baseball (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mlbbRomInfo, g_mlbbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// MLBPA Sports Talk Baseball (U) [!]
static struct BurnRomInfo g_mlbsRomDesc[] = {
	{ "g_mlbs.bin", 0x100000, 0x0deb79c2, BRF_PRG },
};

STD_ROM_PICK(g_mlbs);
STD_ROM_FN(g_mlbs);

struct BurnDriverD BurnDrvMDg_mlbs = {
	"g_mlbs", NULL, NULL, "1900",
	"MLBPA Sports Talk Baseball (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mlbsRomInfo, g_mlbsRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Muhammad Ali Heavyweight Boxing (U) [c][!]
static struct BurnRomInfo g_mahbRomDesc[] = {
	{ "g_mahb.bin", 0x100000, 0xb638b6a3, BRF_PRG },
};

STD_ROM_PICK(g_mahb);
STD_ROM_FN(g_mahb);

struct BurnDriverD BurnDrvMDg_mahb = {
	"g_mahb", NULL, NULL, "1900",
	"Muhammad Ali Heavyweight Boxing (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mahbRomInfo, g_mahbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Muhammad Ali Heavyweight Boxing (Beta)
static struct BurnRomInfo g_mahbbRomDesc[] = {
	{ "g_mahbb.bin", 0x100000, 0x7b852653, BRF_PRG },
};

STD_ROM_PICK(g_mahbb);
STD_ROM_FN(g_mahbb);

struct BurnDriverD BurnDrvMDg_mahbb = {
	"g_mahbb", "g_mahb", NULL, "1900",
	"Muhammad Ali Heavyweight Boxing (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mahbbRomInfo, g_mahbbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Muhammad Ali Heavyweight Boxing (E)
static struct BurnRomInfo g_mahbeRomDesc[] = {
	{ "g_mahbe.bin", 0x100000, 0x8ea4717b, BRF_PRG },
};

STD_ROM_PICK(g_mahbe);
STD_ROM_FN(g_mahbe);

struct BurnDriverD BurnDrvMDg_mahbe = {
	"g_mahbe", "g_mahb", NULL, "1900",
	"Muhammad Ali Heavyweight Boxing (E)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mahbeRomInfo, g_mahbeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Newman-Haas IndyCar Racing (W) [!]
static struct BurnRomInfo g_nhirRomDesc[] = {
	{ "g_nhir.bin", 0x200000, 0x1233a229, BRF_PRG },
};

STD_ROM_PICK(g_nhir);
STD_ROM_FN(g_nhir);

struct BurnDriverD BurnDrvMDg_nhir = {
	"g_nhir", NULL, NULL, "1900",
	"Newman-Haas IndyCar Racing (W) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_nhirRomInfo, g_nhirRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Deadly Moves (U) [!]
static struct BurnRomInfo g_dmovRomDesc[] = {
	{ "g_dmov.bin", 0x100000, 0x35cbd237, BRF_PRG },
};

STD_ROM_PICK(g_dmov);
STD_ROM_FN(g_dmov);

struct BurnDriverD BurnDrvMDg_dmov = {
	"g_dmov", NULL, NULL, "1900",
	"Deadly Moves (U) [!]\0", NULL, "Kaneko", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_dmovRomInfo, g_dmovRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Power Athlete (J) [!]
static struct BurnRomInfo g_pathRomDesc[] = {
	{ "g_path.bin", 0x100000, 0xb41b77cf, BRF_PRG },
};

STD_ROM_PICK(g_path);
STD_ROM_FN(g_path);

struct BurnDriverD BurnDrvMDg_path = {
	"g_path", "g_dmov", NULL, "1900",
	"Power Athlete (J) [!]\0", NULL, "Kaneko", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pathRomInfo, g_pathRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Pro Quarterback (U) [c][!]
static struct BurnRomInfo g_prquRomDesc[] = {
	{ "g_prqu.bin", 0x100000, 0xcc8b2b69, BRF_PRG },
};

STD_ROM_PICK(g_prqu);
STD_ROM_FN(g_prqu);

struct BurnDriverD BurnDrvMDg_prqu = {
	"g_prqu", NULL, NULL, "1900",
	"Pro Quarterback (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_prquRomInfo, g_prquRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Rampart (U) [!]
static struct BurnRomInfo g_rampRomDesc[] = {
	{ "g_ramp.bin", 0x80000, 0x9c4dd057, BRF_PRG },
};

STD_ROM_PICK(g_ramp);
STD_ROM_FN(g_ramp);

struct BurnDriverD BurnDrvMDg_ramp = {
	"g_ramp", NULL, NULL, "1900",
	"Rampart (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_rampRomInfo, g_rampRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Richard Scarry's Busytown (U) [!]
static struct BurnRomInfo g_rsbtRomDesc[] = {
	{ "g_rsbt.bin", 0x200000, 0x7bb60c3f, BRF_PRG },
};

STD_ROM_PICK(g_rsbt);
STD_ROM_FN(g_rsbt);

struct BurnDriverD BurnDrvMDg_rsbt = {
	"g_rsbt", NULL, NULL, "1900",
	"Richard Scarry's Busytown (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_rsbtRomInfo, g_rsbtRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Rockman Megaworld (J) [!]
static struct BurnRomInfo g_rmmwRomDesc[] = {
	{ "g_rmmw.bin", 0x200000, 0x85c956ef, BRF_PRG },
};

STD_ROM_PICK(g_rmmw);
STD_ROM_FN(g_rmmw);

struct BurnDriverD BurnDrvMDg_rmmw = {
	"g_rmmw", NULL, NULL, "1900",
	"Rockman Megaworld (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_rmmwRomInfo, g_rmmwRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Rockman Megaworld (J) [a1][!]
static struct BurnRomInfo g_rkmnjaRomDesc[] = {
	{ "g_rkmnja.bin", 0x200000, 0x4d87235e, BRF_PRG },
};

STD_ROM_PICK(g_rkmnja);
STD_ROM_FN(g_rkmnja);

struct BurnDriverD BurnDrvMDg_rkmnja = {
	"g_rkmnja", "g_rmmw", NULL, "1900",
	"Rockman Megaworld (J) [a1][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_rkmnjaRomInfo, g_rkmnjaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Megaman - The Wily Wars (E)
static struct BurnRomInfo g_megmeRomDesc[] = {
	{ "g_megme.bin", 0x200000, 0xdcf6e8b2, BRF_PRG },
};

STD_ROM_PICK(g_megme);
STD_ROM_FN(g_megme);

struct BurnDriverD BurnDrvMDg_megme = {
	"g_megme", "g_rmmw", NULL, "1900",
	"Megaman - The Wily Wars (E)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_megmeRomInfo, g_megmeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Rocky and Bullwinkle, The Adventures of (U) [!]
static struct BurnRomInfo g_robwRomDesc[] = {
	{ "g_robw.bin", 0x100000, 0xef02d57b, BRF_PRG },
};

STD_ROM_PICK(g_robw);
STD_ROM_FN(g_robw);

struct BurnDriverD BurnDrvMDg_robw = {
	"g_robw", NULL, NULL, "1900",
	"Rocky and Bullwinkle, The Adventures of (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_robwRomInfo, g_robwRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Slam Dunk - Shikyou Gekitotsu! (J) [!]
static struct BurnRomInfo g_sdnkRomDesc[] = {
	{ "g_sdnk.bin", 0x200000, 0xcdf5678f, BRF_PRG },
};

STD_ROM_PICK(g_sdnk);
STD_ROM_FN(g_sdnk);

struct BurnDriverD BurnDrvMDg_sdnk = {
	"g_sdnk", NULL, NULL, "1900",
	"Slam Dunk - Shikyou Gekitotsu! (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sdnkRomInfo, g_sdnkRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Slaughter Sport (U) [c][!]
static struct BurnRomInfo g_sspoRomDesc[] = {
	{ "g_sspo.bin", 0xa0000, 0xaf9f9d9c, BRF_PRG },
};

STD_ROM_PICK(g_sspo);
STD_ROM_FN(g_sspo);

struct BurnDriverD BurnDrvMDg_sspo = {
	"g_sspo", NULL, NULL, "1900",
	"Slaughter Sport (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sspoRomInfo, g_sspoRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Fatman (J) [c][!]
static struct BurnRomInfo g_fatmRomDesc[] = {
	{ "g_fatm.bin", 0xa0000, 0x7867da3a, BRF_PRG },
};

STD_ROM_PICK(g_fatm);
STD_ROM_FN(g_fatm);

struct BurnDriverD BurnDrvMDg_fatm = {
	"g_fatm", "g_sspo", NULL, "1900",
	"Fatman (J) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_fatmRomInfo, g_fatmRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Star Control (U) [c][!]
static struct BurnRomInfo g_sconRomDesc[] = {
	{ "g_scon.bin", 0x180000, 0x8e2bceaf, BRF_PRG },
};

STD_ROM_PICK(g_scon);
STD_ROM_FN(g_scon);

struct BurnDriverD BurnDrvMDg_scon = {
	"g_scon", NULL, NULL, "1900",
	"Star Control (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sconRomInfo, g_sconRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Street Smart (JU) [!]
static struct BurnRomInfo g_ssmaRomDesc[] = {
	{ "g_ssma.bin", 0x80000, 0xb1dedfad, BRF_PRG },
};

STD_ROM_PICK(g_ssma);
STD_ROM_FN(g_ssma);

struct BurnDriverD BurnDrvMDg_ssma = {
	"g_ssma", NULL, NULL, "1900",
	"Street Smart (JU) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ssmaRomInfo, g_ssmaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Summer Challenge (U) [!]
static struct BurnRomInfo g_sumcRomDesc[] = {
	{ "g_sumc.bin", 0x200000, 0xd7d53dc1, BRF_PRG },
};

STD_ROM_PICK(g_sumc);
STD_ROM_FN(g_sumc);

struct BurnDriverD BurnDrvMDg_sumc = {
	"g_sumc", NULL, NULL, "1900",
	"Summer Challenge (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sumcRomInfo, g_sumcRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Super League (E)
static struct BurnRomInfo g_sleageRomDesc[] = {
	{ "g_sleage.bin", 0x80000, 0x55baec6e, BRF_PRG },
};

STD_ROM_PICK(g_sleage);
STD_ROM_FN(g_sleage);

struct BurnDriverD BurnDrvMDg_sleage = {
	"g_sleage", NULL, NULL, "1900",
	"Super League (E)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sleageRomInfo, g_sleageRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Super League (J) [!]
static struct BurnRomInfo g_slRomDesc[] = {
	{ "g_sl.bin", 0x80000, 0xea13cb1d, BRF_PRG },
};

STD_ROM_PICK(g_sl);
STD_ROM_FN(g_sl);

struct BurnDriverD BurnDrvMDg_sl = {
	"g_sl", "g_sleage", NULL, "1900",
	"Super League (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_slRomInfo, g_slRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Super League 91 (J) [!]
static struct BurnRomInfo g_sl91RomDesc[] = {
	{ "g_sl91.bin", 0x80000, 0xa948ab7e, BRF_PRG },
};

STD_ROM_PICK(g_sl91);
STD_ROM_FN(g_sl91);

struct BurnDriverD BurnDrvMDg_sl91 = {
	"g_sl91", NULL, NULL, "1900",
	"Super League 91 (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sl91RomInfo, g_sl91RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Super Masters Golf (J) [!]
static struct BurnRomInfo g_smasRomDesc[] = {
	{ "g_smas.bin", 0x80000, 0x088ba825, BRF_PRG },
};

STD_ROM_PICK(g_smas);
STD_ROM_FN(g_smas);

struct BurnDriverD BurnDrvMDg_smas = {
	"g_smas", NULL, NULL, "1900",
	"Super Masters Golf (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_smasRomInfo, g_smasRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sword of Sodan (UE) [!]
static struct BurnRomInfo g_swsoRomDesc[] = {
	{ "g_swso.bin", 0x80000, 0x9cb8468f, BRF_PRG },
};

STD_ROM_PICK(g_swso);
STD_ROM_FN(g_swso);

struct BurnDriverD BurnDrvMDg_swso = {
	"g_swso", NULL, NULL, "1900",
	"Sword of Sodan (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_swsoRomInfo, g_swsoRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sword of Sodan (J) [!]
static struct BurnRomInfo g_swsojRomDesc[] = {
	{ "g_swsoj.bin", 0x80000, 0x58edb3f3, BRF_PRG },
};

STD_ROM_PICK(g_swsoj);
STD_ROM_FN(g_swsoj);

struct BurnDriverD BurnDrvMDg_swsoj = {
	"g_swsoj", "g_swso", NULL, "1900",
	"Sword of Sodan (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_swsojRomInfo, g_swsojRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sword of Vermilion (UE) [!]
static struct BurnRomInfo g_swveRomDesc[] = {
	{ "g_swve.bin", 0xa0000, 0xea1bc9ab, BRF_PRG },
};

STD_ROM_PICK(g_swve);
STD_ROM_FN(g_swve);

struct BurnDriverD BurnDrvMDg_swve = {
	"g_swve", NULL, NULL, "1900",
	"Sword of Vermilion (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_swveRomInfo, g_swveRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sword of Vermilion (J) [c][!]
static struct BurnRomInfo g_swvejRomDesc[] = {
	{ "g_swvej.bin", 0xa0000, 0xe400dfc3, BRF_PRG },
};

STD_ROM_PICK(g_swvej);
STD_ROM_FN(g_swvej);

struct BurnDriverD BurnDrvMDg_swvej = {
	"g_swvej", "g_swve", NULL, "1900",
	"Sword of Vermilion (J) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_swvejRomInfo, g_swvejRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Team USA Basketball (UE) [c][!]
static struct BurnRomInfo g_tusaRomDesc[] = {
	{ "g_tusa.bin", 0x100000, 0xa0caf97e, BRF_PRG },
};

STD_ROM_PICK(g_tusa);
STD_ROM_FN(g_tusa);

struct BurnDriverD BurnDrvMDg_tusa = {
	"g_tusa", NULL, NULL, "1900",
	"Team USA Basketball (UE) [c][!]\0", NULL, "Electronic Arts", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tusaRomInfo, g_tusaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Dream Team USA (J) [!]
static struct BurnRomInfo g_dtusaRomDesc[] = {
	{ "g_dtusa.bin", 0x100000, 0xe2e21b72, BRF_PRG },
};

STD_ROM_PICK(g_dtusa);
STD_ROM_FN(g_dtusa);

struct BurnDriverD BurnDrvMDg_dtusa = {
	"g_dtusa", "g_tusa", NULL, "1900",
	"Dream Team USA (J) [!]\0", NULL, "Electronic Arts", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_dtusaRomInfo, g_dtusaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Techno Clash (UE) [!]
static struct BurnRomInfo g_tclsRomDesc[] = {
	{ "g_tcls.bin", 0x100000, 0x4e65e483, BRF_PRG },
};

STD_ROM_PICK(g_tcls);
STD_ROM_FN(g_tcls);

struct BurnDriverD BurnDrvMDg_tcls = {
	"g_tcls", NULL, NULL, "1900",
	"Techno Clash (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tclsRomInfo, g_tclsRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Technocop (U) [!]
static struct BurnRomInfo g_tcopRomDesc[] = {
	{ "g_tcop.bin", 0x80000, 0x7459ad06, BRF_PRG },
};

STD_ROM_PICK(g_tcop);
STD_ROM_FN(g_tcop);

struct BurnDriverD BurnDrvMDg_tcop = {
	"g_tcop", NULL, NULL, "1900",
	"Technocop (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tcopRomInfo, g_tcopRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Tecmo Cup (J) [x]
static struct BurnRomInfo g_tcupxRomDesc[] = {
	{ "g_tcupx.bin", 0x80000, 0x88fdd060, BRF_PRG },
};

STD_ROM_PICK(g_tcupx);
STD_ROM_FN(g_tcupx);

struct BurnDriverD BurnDrvMDg_tcupx = {
	"g_tcupx", NULL, NULL, "1900",
	"Tecmo Cup (J) [x]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tcupxRomInfo, g_tcupxRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Tecmo Cup (J) [p1][!]
static struct BurnRomInfo g_tcRomDesc[] = {
	{ "g_tc.bin", 0x80000, 0xe889e218, BRF_PRG },
};

STD_ROM_PICK(g_tc);
STD_ROM_FN(g_tc);

struct BurnDriverD BurnDrvMDg_tc = {
	"g_tc", "g_tcupx", NULL, "1900",
	"Tecmo Cup (J) [p1][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tcRomInfo, g_tcRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Tecmo Super Baseball (U) [!]
static struct BurnRomInfo g_tsbRomDesc[] = {
	{ "g_tsb.bin", 0x100000, 0x227a1178, BRF_PRG },
};

STD_ROM_PICK(g_tsb);
STD_ROM_FN(g_tsb);

struct BurnDriverD BurnDrvMDg_tsb = {
	"g_tsb", NULL, NULL, "1900",
	"Tecmo Super Baseball (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tsbRomInfo, g_tsbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Tecmo Super Bowl (U) (Oct 1993) [!]
static struct BurnRomInfo g_tsbwRomDesc[] = {
	{ "g_tsbw.bin", 0x100000, 0x21f27d34, BRF_PRG },
};

STD_ROM_PICK(g_tsbw);
STD_ROM_FN(g_tsbw);

struct BurnDriverD BurnDrvMDg_tsbw = {
	"g_tsbw", NULL, NULL, "1900",
	"Tecmo Super Bowl (U) (Oct 1993) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tsbwRomInfo, g_tsbwRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Tecmo Super Bowl (J) [!]
static struct BurnRomInfo g_tsbwljRomDesc[] = {
	{ "g_tsbwlj.bin", 0x100000, 0x90c6e20c, BRF_PRG },
};

STD_ROM_PICK(g_tsbwlj);
STD_ROM_FN(g_tsbwlj);

struct BurnDriverD BurnDrvMDg_tsbwlj = {
	"g_tsbwlj", "g_tsbw", NULL, "1900",
	"Tecmo Super Bowl (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tsbwljRomInfo, g_tsbwljRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Tecmo Super Bowl (U) (Sep 1993) [!]
static struct BurnRomInfo g_tsbwluRomDesc[] = {
	{ "g_tsbwlu.bin", 0x100000, 0xbd5933ee, BRF_PRG },
};

STD_ROM_PICK(g_tsbwlu);
STD_ROM_FN(g_tsbwlu);

struct BurnDriverD BurnDrvMDg_tsbwlu = {
	"g_tsbwlu", "g_tsbw", NULL, "1900",
	"Tecmo Super Bowl (U) (Sep 1993) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tsbwluRomInfo, g_tsbwluRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Tecmo Super Bowl II SE (U) [!]
static struct BurnRomInfo g_tbw2RomDesc[] = {
	{ "g_tbw2.bin", 0x200000, 0x0a0e67d8, BRF_PRG },
};

STD_ROM_PICK(g_tbw2);
STD_ROM_FN(g_tbw2);

struct BurnDriverD BurnDrvMDg_tbw2 = {
	"g_tbw2", NULL, NULL, "1900",
	"Tecmo Super Bowl II SE (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tbw2RomInfo, g_tbw2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Tecmo Super Bowl II SE (J) [!]
static struct BurnRomInfo g_tsbw2jRomDesc[] = {
	{ "g_tsbw2j.bin", 0x200000, 0x32fb633d, BRF_PRG },
};

STD_ROM_PICK(g_tsbw2j);
STD_ROM_FN(g_tsbw2j);

struct BurnDriverD BurnDrvMDg_tsbw2j = {
	"g_tsbw2j", "g_tbw2", NULL, "1900",
	"Tecmo Super Bowl II SE (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tsbw2jRomInfo, g_tsbw2jRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Tecmo Super Bowl III Final Edition (U) [!]
static struct BurnRomInfo g_tbw3RomDesc[] = {
	{ "g_tbw3.bin", 0x200000, 0xaae4089f, BRF_PRG },
};

STD_ROM_PICK(g_tbw3);
STD_ROM_FN(g_tbw3);

struct BurnDriverD BurnDrvMDg_tbw3 = {
	"g_tbw3", NULL, NULL, "1900",
	"Tecmo Super Bowl III Final Edition (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tbw3RomInfo, g_tbw3RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Tecmo Super Hockey (U) [!]
static struct BurnRomInfo g_tshRomDesc[] = {
	{ "g_tsh.bin", 0x100000, 0x5f86ddc9, BRF_PRG },
};

STD_ROM_PICK(g_tsh);
STD_ROM_FN(g_tsh);

struct BurnDriverD BurnDrvMDg_tsh = {
	"g_tsh", NULL, NULL, "1900",
	"Tecmo Super Hockey (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tshRomInfo, g_tshRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Tecmo Super NBA Basketball (U) [!]
static struct BurnRomInfo g_tsnbRomDesc[] = {
	{ "g_tsnb.bin", 0x100000, 0x53913991, BRF_PRG },
};

STD_ROM_PICK(g_tsnb);
STD_ROM_FN(g_tsnb);

struct BurnDriverD BurnDrvMDg_tsnb = {
	"g_tsnb", NULL, NULL, "1900",
	"Tecmo Super NBA Basketball (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tsnbRomInfo, g_tsnbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Tecmo Super NBA Basketball (J) [!]
static struct BurnRomInfo g_tsnbajRomDesc[] = {
	{ "g_tsnbaj.bin", 0x100000, 0x79f33eb6, BRF_PRG },
};

STD_ROM_PICK(g_tsnbaj);
STD_ROM_FN(g_tsnbaj);

struct BurnDriverD BurnDrvMDg_tsnbaj = {
	"g_tsnbaj", "g_tsnb", NULL, "1900",
	"Tecmo Super NBA Basketball (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tsnbajRomInfo, g_tsnbajRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Tecmo World Cup '93 (U) [!]
static struct BurnRomInfo g_tw93RomDesc[] = {
	{ "g_tw93.bin", 0x40000, 0xcaf8eb2c, BRF_PRG },
};

STD_ROM_PICK(g_tw93);
STD_ROM_FN(g_tw93);

struct BurnDriverD BurnDrvMDg_tw93 = {
	"g_tw93", NULL, NULL, "1900",
	"Tecmo World Cup '93 (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tw93RomInfo, g_tw93RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Tecmo World Cup '92 (J) [!]
static struct BurnRomInfo g_tw92RomDesc[] = {
	{ "g_tw92.bin", 0x40000, 0x5e93c8b0, BRF_PRG },
};

STD_ROM_PICK(g_tw92);
STD_ROM_FN(g_tw92);

struct BurnDriverD BurnDrvMDg_tw92 = {
	"g_tw92", "g_tw93", NULL, "1900",
	"Tecmo World Cup '92 (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tw92RomInfo, g_tw92RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Thomas the Tank Engine & Friends (U) [c][!]
static struct BurnRomInfo g_ttnkRomDesc[] = {
	{ "g_ttnk.bin", 0x100000, 0x1a406299, BRF_PRG },
};

STD_ROM_PICK(g_ttnk);
STD_ROM_FN(g_ttnk);

struct BurnDriverD BurnDrvMDg_ttnk = {
	"g_ttnk", NULL, NULL, "1900",
	"Thomas the Tank Engine & Friends (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ttnkRomInfo, g_ttnkRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Tommy Lasorda Baseball (U) [!]
static struct BurnRomInfo g_tlbbRomDesc[] = {
	{ "g_tlbb.bin", 0x80000, 0x4fb50304, BRF_PRG },
};

STD_ROM_PICK(g_tlbb);
STD_ROM_FN(g_tlbb);

struct BurnDriverD BurnDrvMDg_tlbb = {
	"g_tlbb", NULL, NULL, "1900",
	"Tommy Lasorda Baseball (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tlbbRomInfo, g_tlbbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Tony La Russa Baseball 95 (U) [!]
static struct BurnRomInfo g_tr95RomDesc[] = {
	{ "g_tr95.bin", 0x200000, 0x3f848a92, BRF_PRG },
};

STD_ROM_PICK(g_tr95);
STD_ROM_FN(g_tr95);

struct BurnDriverD BurnDrvMDg_tr95 = {
	"g_tr95", NULL, NULL, "1900",
	"Tony La Russa Baseball 95 (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tr95RomInfo, g_tr95RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Tony La Russa Baseball (U) [!]
static struct BurnRomInfo g_trbbRomDesc[] = {
	{ "g_trbb.bin", 0x100000, 0x24629c78, BRF_PRG },
};

STD_ROM_PICK(g_trbb);
STD_ROM_FN(g_trbb);

struct BurnDriverD BurnDrvMDg_trbb = {
	"g_trbb", NULL, NULL, "1900",
	"Tony La Russa Baseball (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_trbbRomInfo, g_trbbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Total Football (E) [!]
static struct BurnRomInfo g_totfRomDesc[] = {
	{ "g_totf.bin", 0x200000, 0x8360b66a, BRF_PRG },
};

STD_ROM_PICK(g_totf);
STD_ROM_FN(g_totf);

struct BurnDriverD BurnDrvMDg_totf = {
	"g_totf", NULL, NULL, "1900",
	"Total Football (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_totfRomInfo, g_totfRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Toxic Crusaders (U) [!]
static struct BurnRomInfo g_toxiRomDesc[] = {
	{ "g_toxi.bin", 0x80000, 0x11fd46ce, BRF_PRG },
};

STD_ROM_PICK(g_toxi);
STD_ROM_FN(g_toxi);

struct BurnDriverD BurnDrvMDg_toxi = {
	"g_toxi", NULL, NULL, "1900",
	"Toxic Crusaders (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_toxiRomInfo, g_toxiRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Troy Aikman NFL Football (U) [!]
static struct BurnRomInfo g_tanfRomDesc[] = {
	{ "g_tanf.bin", 0x200000, 0x015f2713, BRF_PRG },
};

STD_ROM_PICK(g_tanf);
STD_ROM_FN(g_tanf);

struct BurnDriverD BurnDrvMDg_tanf = {
	"g_tanf", NULL, NULL, "1900",
	"Troy Aikman NFL Football (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tanfRomInfo, g_tanfRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Ultraman (J) [!]
static struct BurnRomInfo g_umanRomDesc[] = {
	{ "g_uman.bin", 0x80000, 0x83b4d5fb, BRF_PRG },
};

STD_ROM_PICK(g_uman);
STD_ROM_FN(g_uman);

struct BurnDriverD BurnDrvMDg_uman = {
	"g_uman", NULL, NULL, "1900",
	"Ultraman (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_umanRomInfo, g_umanRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Unnecessary Roughness 95 (U) [!]
static struct BurnRomInfo g_ur95RomDesc[] = {
	{ "g_ur95.bin", 0x200000, 0x9920e7b7, BRF_PRG },
};

STD_ROM_PICK(g_ur95);
STD_ROM_FN(g_ur95);

struct BurnDriverD BurnDrvMDg_ur95 = {
	"g_ur95", NULL, NULL, "1900",
	"Unnecessary Roughness 95 (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ur95RomInfo, g_ur95RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Vapor Trail (U) [!]
static struct BurnRomInfo g_vaptRomDesc[] = {
	{ "g_vapt.bin", 0x100000, 0xc49e3a0a, BRF_PRG },
};

STD_ROM_PICK(g_vapt);
STD_ROM_FN(g_vapt);

struct BurnDriverD BurnDrvMDg_vapt = {
	"g_vapt", NULL, NULL, "1900",
	"Vapor Trail (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_vaptRomInfo, g_vaptRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Kuuga - Operation Vapor Trail (J) [!]
static struct BurnRomInfo g_kuugaRomDesc[] = {
	{ "g_kuuga.bin", 0x100000, 0x83b6b6ba, BRF_PRG },
};

STD_ROM_PICK(g_kuuga);
STD_ROM_FN(g_kuuga);

struct BurnDriverD BurnDrvMDg_kuuga = {
	"g_kuuga", "g_vapt", NULL, "1900",
	"Kuuga - Operation Vapor Trail (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_kuugaRomInfo, g_kuugaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Wacky Worlds (U) [!]
static struct BurnRomInfo g_wackRomDesc[] = {
	{ "g_wack.bin", 0x100000, 0x8af4552d, BRF_PRG },
};

STD_ROM_PICK(g_wack);
STD_ROM_FN(g_wack);

struct BurnDriverD BurnDrvMDg_wack = {
	"g_wack", NULL, NULL, "1900",
	"Wacky Worlds (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wackRomInfo, g_wackRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Wayne Gretzsky NHLPA All-Stars (UE) [!]
static struct BurnRomInfo g_wgasRomDesc[] = {
	{ "g_wgas.bin", 0x200000, 0xc2c13b81, BRF_PRG },
};

STD_ROM_PICK(g_wgas);
STD_ROM_FN(g_wgas);

struct BurnDriverD BurnDrvMDg_wgas = {
	"g_wgas", NULL, NULL, "1900",
	"Wayne Gretzsky NHLPA All-Stars (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wgasRomInfo, g_wgasRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Wheel of Fortune (U) [!]
static struct BurnRomInfo g_wforRomDesc[] = {
	{ "g_wfor.bin", 0x80000, 0xc8d8efc3, BRF_PRG },
};

STD_ROM_PICK(g_wfor);
STD_ROM_FN(g_wfor);

struct BurnDriverD BurnDrvMDg_wfor = {
	"g_wfor", NULL, NULL, "1900",
	"Wheel of Fortune (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wforRomInfo, g_wforRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Winter Challenge (UE) [!]
static struct BurnRomInfo g_wincRomDesc[] = {
	{ "g_winc.bin", 0x100000, 0xf57c7068, BRF_PRG },
};

STD_ROM_PICK(g_winc);
STD_ROM_FN(g_winc);

struct BurnDriverD BurnDrvMDg_winc = {
	"g_winc", NULL, NULL, "1900",
	"Winter Challenge (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wincRomInfo, g_wincRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Winter Challenge (Beta)
static struct BurnRomInfo g_wchalbRomDesc[] = {
	{ "g_wchalb.bin", 0x100000, 0x60d2a8c4, BRF_PRG },
};

STD_ROM_PICK(g_wchalb);
STD_ROM_FN(g_wchalb);

struct BurnDriverD BurnDrvMDg_wchalb = {
	"g_wchalb", "g_winc", NULL, "1900",
	"Winter Challenge (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wchalbRomInfo, g_wchalbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Wonder Boy III - Monster Lair (JE) [c][!]
static struct BurnRomInfo g_wb3RomDesc[] = {
	{ "g_wb3.bin", 0x80000, 0xc24bc5e4, BRF_PRG },
};

STD_ROM_PICK(g_wb3);
STD_ROM_FN(g_wb3);

struct BurnDriverD BurnDrvMDg_wb3 = {
	"g_wb3", NULL, NULL, "1900",
	"Wonder Boy III - Monster Lair (JE) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wb3RomInfo, g_wb3RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Wonder Boy in Monster World (UE) [!]
static struct BurnRomInfo g_wbmwRomDesc[] = {
	{ "g_wbmw.bin", 0xc0000, 0x1592f5b0, BRF_PRG },
};

STD_ROM_PICK(g_wbmw);
STD_ROM_FN(g_wbmw);

struct BurnDriverD BurnDrvMDg_wbmw = {
	"g_wbmw", NULL, NULL, "1900",
	"Wonder Boy in Monster World (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wbmwRomInfo, g_wbmwRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Turma da Monica na Terra Dos Monstros (B) [!]
static struct BurnRomInfo g_turmaRomDesc[] = {
	{ "g_turma.bin", 0x100000, 0xf8288de1, BRF_PRG },
};

STD_ROM_PICK(g_turma);
STD_ROM_FN(g_turma);

struct BurnDriverD BurnDrvMDg_turma = {
	"g_turma", "g_wbmw", NULL, "1900",
	"Turma da Monica na Terra Dos Monstros (B) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_turmaRomInfo, g_turmaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// World Class Leaderboard Golf (U) [c][!]
static struct BurnRomInfo g_wclRomDesc[] = {
	{ "g_wcl.bin", 0x80000, 0x53434bab, BRF_PRG },
};

STD_ROM_PICK(g_wcl);
STD_ROM_FN(g_wcl);

struct BurnDriverD BurnDrvMDg_wcl = {
	"g_wcl", NULL, NULL, "1900",
	"World Class Leaderboard Golf (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wclRomInfo, g_wclRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// World Class Leaderboard Golf (E) [x]
static struct BurnRomInfo g_wclbxRomDesc[] = {
	{ "g_wclbx.bin", 0x80000, 0xdaca01c3, BRF_PRG },
};

STD_ROM_PICK(g_wclbx);
STD_ROM_FN(g_wclbx);

struct BurnDriverD BurnDrvMDg_wclbx = {
	"g_wclbx", "g_wcl", NULL, "1900",
	"World Class Leaderboard Golf (E) [x]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wclbxRomInfo, g_wclbxRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// World Cup USA 94 (UE) [!]
static struct BurnRomInfo g_wc94RomDesc[] = {
	{ "g_wc94.bin", 0x100000, 0x0171b47f, BRF_PRG },
};

STD_ROM_PICK(g_wc94);
STD_ROM_FN(g_wc94);

struct BurnDriverD BurnDrvMDg_wc94 = {
	"g_wc94", NULL, NULL, "1900",
	"World Cup USA 94 (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wc94RomInfo, g_wc94RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Champions World Class Soccer (W) (M4) [!]
static struct BurnRomInfo g_cwcsRomDesc[] = {
	{ "g_cwcs.bin", 0x100000, 0x883e33db, BRF_PRG },
};

STD_ROM_PICK(g_cwcs);
STD_ROM_FN(g_cwcs);

struct BurnDriverD BurnDrvMDg_cwcs = {
	"g_cwcs", NULL, NULL, "1900",
	"Champions World Class Soccer (W) (M4) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_cwcsRomInfo, g_cwcsRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Championship Pro-Am (U) [c][!]
static struct BurnRomInfo g_cpamRomDesc[] = {
	{ "g_cpam.bin", 0x40000, 0xb496de28, BRF_PRG },
};

STD_ROM_PICK(g_cpam);
STD_ROM_FN(g_cpam);

struct BurnDriverD BurnDrvMDg_cpam = {
	"g_cpam", NULL, NULL, "1900",
	"Championship Pro-Am (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_cpamRomInfo, g_cpamRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Centurion - Defender of Rome (UE) [!]
static struct BurnRomInfo g_cdorRomDesc[] = {
	{ "g_cdor.bin", 0xc0000, 0x21283b14, BRF_PRG },
};

STD_ROM_PICK(g_cdor);
STD_ROM_FN(g_cdor);

struct BurnDriverD BurnDrvMDg_cdor = {
	"g_cdor", NULL, NULL, "1900",
	"Centurion - Defender of Rome (UE) [!]\0", NULL, "Electronic Arts", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_cdorRomInfo, g_cdorRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Chi Chi's Pro Challenge (U) [!]
static struct BurnRomInfo g_chiRomDesc[] = {
	{ "g_chi.bin", 0x100000, 0x9c3973a4, BRF_PRG },
};

STD_ROM_PICK(g_chi);
STD_ROM_FN(g_chi);

struct BurnDriverD BurnDrvMDg_chi = {
	"g_chi", NULL, NULL, "1900",
	"Chi Chi's Pro Challenge (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_chiRomInfo, g_chiRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Battle Golfer Yui (J) [c][!]
static struct BurnRomInfo g_bglfRomDesc[] = {
	{ "g_bglf.bin", 0x80000, 0x4aa03e4e, BRF_PRG },
};

STD_ROM_PICK(g_bglf);
STD_ROM_FN(g_bglf);

struct BurnDriverD BurnDrvMDg_bglf = {
	"g_bglf", NULL, NULL, "1900",
	"Battle Golfer Yui (J) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_bglfRomInfo, g_bglfRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Bahamut Senki (J) [c][!]
static struct BurnRomInfo g_bahaRomDesc[] = {
	{ "g_baha.bin", 0x80000, 0xb1e268da, BRF_PRG },
};

STD_ROM_PICK(g_baha);
STD_ROM_FN(g_baha);

struct BurnDriverD BurnDrvMDg_baha = {
	"g_baha", NULL, NULL, "1900",
	"Bahamut Senki (J) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_bahaRomInfo, g_bahaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Advanced Daisenryaku (J) (REV01) [!]
static struct BurnRomInfo g_advdaiRomDesc[] = {
	{ "g_advdai.bin", 0x100000, 0xE0D5E18A, BRF_PRG },
};

STD_ROM_PICK(g_advdai);
STD_ROM_FN(g_advdai);

struct BurnDriverD BurnDrvMDg_advdai = {
	"g_advdai", NULL, NULL, "1988",
	"Advanced Daisenryaku (J) (REV01) [!]\0", "Sound Problems", "System Soft Corp.", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_advdaiRomInfo, g_advdaiRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Blue Almanac (J) [!]
static struct BurnRomInfo g_bluealRomDesc[] = {
	{ "g_blueal.bin", 0x100000, 0x7222ebb3, BRF_PRG },
};

STD_ROM_PICK(g_blueal);
STD_ROM_FN(g_blueal);

struct BurnDriverD BurnDrvMDg_blueal = {
	"g_blueal", NULL, NULL, "1900",
	"Blue Almanac (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_bluealRomInfo, g_bluealRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Chibi Maruko-Chan - Wakuwaku Shopping (J) [c][!]
static struct BurnRomInfo g_chibiRomDesc[] = {
	{ "g_chibi.bin", 0x80000, 0x91a144b8, BRF_PRG },
};

STD_ROM_PICK(g_chibi);
STD_ROM_FN(g_chibi);

struct BurnDriverD BurnDrvMDg_chibi = {
	"g_chibi", NULL, NULL, "1900",
	"Chibi Maruko-Chan - Wakuwaku Shopping (J) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_chibiRomInfo, g_chibiRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Crayon Shin-Chan - Arashi o Yobu Sono Ko (J) [!]
static struct BurnRomInfo g_crayonRomDesc[] = {
	{ "g_crayon.bin", 0x200000, 0x97fc42d2, BRF_PRG },
};

STD_ROM_PICK(g_crayon);
STD_ROM_FN(g_crayon);

struct BurnDriverD BurnDrvMDg_crayon = {
	"g_crayon", NULL, NULL, "1900",
	"Crayon Shin-Chan - Arashi o Yobu Sono Ko (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_crayonRomInfo, g_crayonRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// EA Sports Double Header (E) [!]
static struct BurnRomInfo g_eadoubRomDesc[] = {
	{ "g_eadoub.bin", 0x100000, 0xa0b54cbc, BRF_PRG },
};

STD_ROM_PICK(g_eadoub);
STD_ROM_FN(g_eadoub);

struct BurnDriverD BurnDrvMDg_eadoub = {
	"g_eadoub", NULL, NULL, "1900",
	"EA Sports Double Header (E) [!]\0", NULL, "Electronic Arts", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_eadoubRomInfo, g_eadoubRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Fushigi no Umi no Nadia (J) [c][!]
static struct BurnRomInfo g_fushigRomDesc[] = {
	{ "g_fushig.bin", 0x100000, 0x4762062a, BRF_PRG },
};

STD_ROM_PICK(g_fushig);
STD_ROM_FN(g_fushig);

struct BurnDriverD BurnDrvMDg_fushig = {
	"g_fushig", NULL, NULL, "1900",
	"Fushigi no Umi no Nadia (J) [c][!]\0", NULL, "Namco", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_fushigRomInfo, g_fushigRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Frank Thomas Big Hurt Baseball (UE) [!]
static struct BurnRomInfo g_ftbhbRomDesc[] = {
	{ "g_ftbhb.bin", 0x400000, 0x863e0950, BRF_PRG },
};

STD_ROM_PICK(g_ftbhb);
STD_ROM_FN(g_ftbhb);

struct BurnDriverD BurnDrvMDg_ftbhb = {
	"g_ftbhb", NULL, NULL, "1900",
	"Frank Thomas Big Hurt Baseball (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ftbhbRomInfo, g_ftbhbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Greatest Heavyweights of the Ring (U) [!]
static struct BurnRomInfo g_ghworRomDesc[] = {
	{ "g_ghwor.bin", 0x200000, 0x6e3621d5, BRF_PRG },
};

STD_ROM_PICK(g_ghwor);
STD_ROM_FN(g_ghwor);

struct BurnDriverD BurnDrvMDg_ghwor = {
	"g_ghwor", NULL, NULL, "1900",
	"Greatest Heavyweights of the Ring (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ghworRomInfo, g_ghworRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Greatest Heavyweights of the Ring (J) [c][!]
static struct BurnRomInfo g_ghwjRomDesc[] = {
	{ "g_ghwj.bin", 0x200000, 0x7ef8b162, BRF_PRG },
};

STD_ROM_PICK(g_ghwj);
STD_ROM_FN(g_ghwj);

struct BurnDriverD BurnDrvMDg_ghwj = {
	"g_ghwj", "g_ghwor", NULL, "1900",
	"Greatest Heavyweights of the Ring (J) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ghwjRomInfo, g_ghwjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// High School Soccer - Kunio Kun (J) [!]
static struct BurnRomInfo g_hssocRomDesc[] = {
	{ "g_hssoc.bin", 0x80000, 0xf49c3a86, BRF_PRG },
};

STD_ROM_PICK(g_hssoc);
STD_ROM_FN(g_hssoc);

struct BurnDriverD BurnDrvMDg_hssoc = {
	"g_hssoc", NULL, NULL, "1900",
	"High School Soccer - Kunio Kun (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_hssocRomInfo, g_hssocRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Ichidant R (J) [!]
static struct BurnRomInfo g_ichirRomDesc[] = {
	{ "g_ichir.bin", 0x200000, 0x7bdec762, BRF_PRG },
};

STD_ROM_PICK(g_ichir);
STD_ROM_FN(g_ichir);

struct BurnDriverD BurnDrvMDg_ichir = {
	"g_ichir", NULL, NULL, "1900",
	"Ichidant R (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ichirRomInfo, g_ichirRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Hyokkori Hyoutanjima (J) [c][!]
static struct BurnRomInfo g_hyokkRomDesc[] = {
	{ "g_hyokk.bin", 0x80000, 0x72253bdb, BRF_PRG },
};

STD_ROM_PICK(g_hyokk);
STD_ROM_FN(g_hyokk);

struct BurnDriverD BurnDrvMDg_hyokk = {
	"g_hyokk", NULL, NULL, "1900",
	"Hyokkori Hyoutanjima (J) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_hyokkRomInfo, g_hyokkRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Kishi Densetsu (J) [!]
static struct BurnRomInfo g_kishiRomDesc[] = {
	{ "g_kishi.bin", 0x180000, 0x22e1f04a, BRF_PRG },
};

STD_ROM_PICK(g_kishi);
STD_ROM_FN(g_kishi);

struct BurnDriverD BurnDrvMDg_kishi = {
	"g_kishi", NULL, NULL, "1900",
	"Kishi Densetsu (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_kishiRomInfo, g_kishiRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Kyuukai Douchuuki (J) [c][!]
static struct BurnRomInfo g_kyuukRomDesc[] = {
	{ "g_kyuuk.bin", 0x80000, 0xde48dce3, BRF_PRG },
};

STD_ROM_PICK(g_kyuuk);
STD_ROM_FN(g_kyuuk);

struct BurnDriverD BurnDrvMDg_kyuuk = {
	"g_kyuuk", NULL, NULL, "1900",
	"Kyuukai Douchuuki (J) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_kyuukRomInfo, g_kyuukRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// M-1 Abrams Battle Tank (UE) (REV01) [!]
static struct BurnRomInfo g_m1tankRomDesc[] = {
	{ "g_m1tank.bin", 0x80000, 0x1e2f74cf, BRF_PRG },
};

STD_ROM_PICK(g_m1tank);
STD_ROM_FN(g_m1tank);

struct BurnDriverD BurnDrvMDg_m1tank = {
	"g_m1tank", NULL, NULL, "1900",
	"M-1 Abrams Battle Tank (UE) (REV01) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_m1tankRomInfo, g_m1tankRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Madou Monogatari I (J) [!]
static struct BurnRomInfo g_madouRomDesc[] = {
	{ "g_madou.bin", 0x200000, 0xdd82c401, BRF_PRG },
};

STD_ROM_PICK(g_madou);
STD_ROM_FN(g_madou);

struct BurnDriverD BurnDrvMDg_madou = {
	"g_madou", NULL, NULL, "1900",
	"Madou Monogatari I (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_madouRomInfo, g_madouRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mahjong Cop Ryuu - Shiro Ookami no Yabou (J) [!]
static struct BurnRomInfo g_mahcopRomDesc[] = {
	{ "g_mahcop.bin", 0x40000, 0x1ccbc782, BRF_PRG },
};

STD_ROM_PICK(g_mahcop);
STD_ROM_FN(g_mahcop);

struct BurnDriverD BurnDrvMDg_mahcop = {
	"g_mahcop", NULL, NULL, "1900",
	"Mahjong Cop Ryuu - Shiro Ookami no Yabou (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mahcopRomInfo, g_mahcopRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Maten no Soumetsu / Maten Densetsu (J) [!]
static struct BurnRomInfo g_matenRomDesc[] = {
	{ "g_maten.bin", 0x100000, 0xb804a105, BRF_PRG },
};

STD_ROM_PICK(g_maten);
STD_ROM_FN(g_maten);

struct BurnDriverD BurnDrvMDg_maten = {
	"g_maten", NULL, NULL, "1900",
	"Maten no Soumetsu / Maten Densetsu (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_matenRomInfo, g_matenRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mega Anser (J) (REV01) [!]
static struct BurnRomInfo g_manserRomDesc[] = {
	{ "g_manser.bin", 0x80000, 0x08ece367, BRF_PRG },
};

STD_ROM_PICK(g_manser);
STD_ROM_FN(g_manser);

struct BurnDriverD BurnDrvMDg_manser = {
	"g_manser", NULL, NULL, "1900",
	"Mega Anser (J) (REV01) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_manserRomInfo, g_manserRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mega Q - The Party Quiz Game (J) [!]
static struct BurnRomInfo g_megaqRomDesc[] = {
	{ "g_megaq.bin", 0x100000, 0x9d4b447a, BRF_PRG },
};

STD_ROM_PICK(g_megaq);
STD_ROM_FN(g_megaq);

struct BurnDriverD BurnDrvMDg_megaq = {
	"g_megaq", NULL, NULL, "1900",
	"Mega Q - The Party Quiz Game (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_megaqRomInfo, g_megaqRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Metal Fangs (J) [c][!]
static struct BurnRomInfo g_mfangRomDesc[] = {
	{ "g_mfang.bin", 0x80000, 0xa8df1c4c, BRF_PRG },
};

STD_ROM_PICK(g_mfang);
STD_ROM_FN(g_mfang);

struct BurnDriverD BurnDrvMDg_mfang = {
	"g_mfang", NULL, NULL, "1900",
	"Metal Fangs (J) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mfangRomInfo, g_mfangRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Miracle Piano Teaching System (U) [!]
static struct BurnRomInfo g_mpianoRomDesc[] = {
	{ "g_mpiano.bin", 0x80000, 0xa719542e, BRF_PRG },
};

STD_ROM_PICK(g_mpiano);
STD_ROM_FN(g_mpiano);

struct BurnDriverD BurnDrvMDg_mpiano = {
	"g_mpiano", NULL, NULL, "1900",
	"Miracle Piano Teaching System (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mpianoRomInfo, g_mpianoRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Monster World IV (J) [!]
static struct BurnRomInfo g_monwr4RomDesc[] = {
	{ "g_monwr4.bin", 0x200000, 0x36a3aaa4, BRF_PRG },
};

STD_ROM_PICK(g_monwr4);
STD_ROM_FN(g_monwr4);

struct BurnDriverD BurnDrvMDg_monwr4 = {
	"g_monwr4", NULL, NULL, "1900",
	"Monster World IV (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_monwr4RomInfo, g_monwr4RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// New 3D Golf Simulation Harukanaru Augusta (J) [!]
static struct BurnRomInfo g_new3dgRomDesc[] = {
	{ "g_new3dg.bin", 0x180000, 0xd2a9bf92, BRF_PRG },
};

STD_ROM_PICK(g_new3dg);
STD_ROM_FN(g_new3dg);

struct BurnDriverD BurnDrvMDg_new3dg = {
	"g_new3dg", NULL, NULL, "1900",
	"New 3D Golf Simulation Harukanaru Augusta (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_new3dgRomInfo, g_new3dgRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Nikkan Sports Pro Yakyuu Van (J) [!]
static struct BurnRomInfo g_nikkanRomDesc[] = {
	{ "g_nikkan.bin", 0x80000, 0xc3655a59, BRF_PRG },
};

STD_ROM_PICK(g_nikkan);
STD_ROM_FN(g_nikkan);

struct BurnDriverD BurnDrvMDg_nikkan = {
	"g_nikkan", NULL, NULL, "1900",
	"Nikkan Sports Pro Yakyuu Van (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_nikkanRomInfo, g_nikkanRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Patlabor (J) [c][!]
static struct BurnRomInfo g_patlabRomDesc[] = {
	{ "g_patlab.bin", 0x80000, 0x21a0e749, BRF_PRG },
};

STD_ROM_PICK(g_patlab);
STD_ROM_FN(g_patlab);

struct BurnDriverD BurnDrvMDg_patlab = {
	"g_patlab", NULL, NULL, "1900",
	"Patlabor (J) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_patlabRomInfo, g_patlabRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Psy-O-Blade Moving Adventure (J) [!]
static struct BurnRomInfo g_psyoblRomDesc[] = {
	{ "g_psyobl.bin", 0xa0000, 0x8ba7e6c5, BRF_PRG },
};

STD_ROM_PICK(g_psyobl);
STD_ROM_FN(g_psyobl);

struct BurnDriverD BurnDrvMDg_psyobl = {
	"g_psyobl", NULL, NULL, "1900",
	"Psy-O-Blade Moving Adventure (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_psyoblRomInfo, g_psyoblRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Ransei no Hasha (J) [!]
static struct BurnRomInfo g_ranseiRomDesc[] = {
	{ "g_ransei.bin", 0x100000, 0xa9a0083d, BRF_PRG },
};

STD_ROM_PICK(g_ransei);
STD_ROM_FN(g_ransei);

struct BurnDriverD BurnDrvMDg_ransei = {
	"g_ransei", NULL, NULL, "1900",
	"Ransei no Hasha (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ranseiRomInfo, g_ranseiRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Rent A Hero (J) [!]
static struct BurnRomInfo g_rentheRomDesc[] = {
	{ "g_renthe.bin", 0x100000, 0x2e515f82, BRF_PRG },
};

STD_ROM_PICK(g_renthe);
STD_ROM_FN(g_renthe);

struct BurnDriverD BurnDrvMDg_renthe = {
	"g_renthe", NULL, NULL, "1900",
	"Rent A Hero (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_rentheRomInfo, g_rentheRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sangokushi Retsuden - Ransei no Eiyuu Tachi (J) [!]
static struct BurnRomInfo g_sangorRomDesc[] = {
	{ "g_sangor.bin", 0x100000, 0x0f56785a, BRF_PRG },
};

STD_ROM_PICK(g_sangor);
STD_ROM_FN(g_sangor);

struct BurnDriverD BurnDrvMDg_sangor = {
	"g_sangor", NULL, NULL, "1900",
	"Sangokushi Retsuden - Ransei no Eiyuu Tachi (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sangorRomInfo, g_sangorRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sangokushi Retsuden (Ch)
static struct BurnRomInfo g_sanretRomDesc[] = {
	{ "g_sanret.bin", 0x100000, 0x3b5cc398, BRF_PRG },
};

STD_ROM_PICK(g_sanret);
STD_ROM_FN(g_sanret);

struct BurnDriverD BurnDrvMDg_sanret = {
	"g_sanret", "g_sangor", NULL, "1900",
	"Sangokushi Retsuden (Ch)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sanretRomInfo, g_sanretRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Shi Kin Joh (J) [!]
static struct BurnRomInfo g_shikinRomDesc[] = {
	{ "g_shikin.bin", 0x80000, 0x5ea0c97d, BRF_PRG },
};

STD_ROM_PICK(g_shikin);
STD_ROM_FN(g_shikin);

struct BurnDriverD BurnDrvMDg_shikin = {
	"g_shikin", NULL, NULL, "1900",
	"Shi Kin Joh (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_shikinRomInfo, g_shikinRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Shogi no Hoshi (J) [!]
static struct BurnRomInfo g_shogiRomDesc[] = {
	{ "g_shogi.bin", 0x40000, 0x4148f816, BRF_PRG },
};

STD_ROM_PICK(g_shogi);
STD_ROM_FN(g_shogi);

struct BurnDriverD BurnDrvMDg_shogi = {
	"g_shogi", NULL, NULL, "1900",
	"Shogi no Hoshi (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_shogiRomInfo, g_shogiRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Shura no Mon (J) [!]
static struct BurnRomInfo g_shuraRomDesc[] = {
	{ "g_shura.bin", 0x100000, 0xe19da6e5, BRF_PRG },
};

STD_ROM_PICK(g_shura);
STD_ROM_FN(g_shura);

struct BurnDriverD BurnDrvMDg_shura = {
	"g_shura", NULL, NULL, "1900",
	"Shura no Mon (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_shuraRomInfo, g_shuraRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Slap Fight (J) [c][!]
static struct BurnRomInfo g_slapfRomDesc[] = {
	{ "g_slapf.bin", 0x100000, 0xd6695695, BRF_PRG },
};

STD_ROM_PICK(g_slapf);
STD_ROM_FN(g_slapf);

struct BurnDriverD BurnDrvMDg_slapf = {
	"g_slapf", NULL, NULL, "1900",
	"Slap Fight (J) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_slapfRomInfo, g_slapfRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Super Daisenryaku (J) (REV00) [!]
static struct BurnRomInfo g_supdaiRomDesc[] = {
	{ "g_supdai.bin", 0x80000, 0xd50a166f, BRF_PRG },
};

STD_ROM_PICK(g_supdai);
STD_ROM_FN(g_supdai);

struct BurnDriverD BurnDrvMDg_supdai = {
	"g_supdai", NULL, NULL, "1900",
	"Super Daisenryaku (J) (REV00) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_supdaiRomInfo, g_supdaiRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Surging Aura (J) [!]
static struct BurnRomInfo g_surginRomDesc[] = {
	{ "g_surgin.bin", 0x200000, 0x65ac1d2b, BRF_PRG },
};

STD_ROM_PICK(g_surgin);
STD_ROM_FN(g_surgin);

struct BurnDriverD BurnDrvMDg_surgin = {
	"g_surgin", NULL, NULL, "1900",
	"Surging Aura (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_surginRomInfo, g_surginRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Taiga Drama Taiheiki (J) [!]
static struct BurnRomInfo g_taigaRomDesc[] = {
	{ "g_taiga.bin", 0x100000, 0x09fbb30e, BRF_PRG },
};

STD_ROM_PICK(g_taiga);
STD_ROM_FN(g_taiga);

struct BurnDriverD BurnDrvMDg_taiga = {
	"g_taiga", NULL, NULL, "1900",
	"Taiga Drama Taiheiki (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_taigaRomInfo, g_taigaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Tel Tel Mahjong (J) [c][!]
static struct BurnRomInfo g_telmjRomDesc[] = {
	{ "g_telmj.bin", 0x40000, 0x44817e92, BRF_PRG },
};

STD_ROM_PICK(g_telmj);
STD_ROM_FN(g_telmj);

struct BurnDriverD BurnDrvMDg_telmj = {
	"g_telmj", NULL, NULL, "1900",
	"Tel Tel Mahjong (J) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_telmjRomInfo, g_telmjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Tel Tel Stadium (J) [!]
static struct BurnRomInfo g_telstdRomDesc[] = {
	{ "g_telstd.bin", 0x80000, 0x54cf8c29, BRF_PRG },
};

STD_ROM_PICK(g_telstd);
STD_ROM_FN(g_telstd);

struct BurnDriverD BurnDrvMDg_telstd = {
	"g_telstd", NULL, NULL, "1900",
	"Tel Tel Stadium (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_telstdRomInfo, g_telstdRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Top Pro Golf 2 (J) [!]
static struct BurnRomInfo g_tpglf2RomDesc[] = {
	{ "g_tpglf2.bin", 0x100000, 0xb8ce98b3, BRF_PRG },
};

STD_ROM_PICK(g_tpglf2);
STD_ROM_FN(g_tpglf2);

struct BurnDriverD BurnDrvMDg_tpglf2 = {
	"g_tpglf2", NULL, NULL, "1900",
	"Top Pro Golf 2 (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tpglf2RomInfo, g_tpglf2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Top Pro Golf (J) [!]
static struct BurnRomInfo g_tpglfRomDesc[] = {
	{ "g_tpglf.bin", 0x100000, 0x62bad606, BRF_PRG },
};

STD_ROM_PICK(g_tpglf);
STD_ROM_FN(g_tpglf);

struct BurnDriverD BurnDrvMDg_tpglf = {
	"g_tpglf", NULL, NULL, "1900",
	"Top Pro Golf (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tpglfRomInfo, g_tpglfRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Twinkle Tale (J) [!]
static struct BurnRomInfo g_twintlRomDesc[] = {
	{ "g_twintl.bin", 0x100000, 0xd757f924, BRF_PRG },
};

STD_ROM_PICK(g_twintl);
STD_ROM_FN(g_twintl);

struct BurnDriverD BurnDrvMDg_twintl = {
	"g_twintl", NULL, NULL, "1900",
	"Twinkle Tale (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_twintlRomInfo, g_twintlRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Verytex (J) [c][!]
static struct BurnRomInfo g_verytxRomDesc[] = {
	{ "g_verytx.bin", 0x80000, 0xbafc375f, BRF_PRG },
};

STD_ROM_PICK(g_verytx);
STD_ROM_FN(g_verytx);

struct BurnDriverD BurnDrvMDg_verytx = {
	"g_verytx", NULL, NULL, "1900",
	"Verytex (J) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_verytxRomInfo, g_verytxRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Vixen 357 (J) [!]
static struct BurnRomInfo g_vix357RomDesc[] = {
	{ "g_vix357.bin", 0x100000, 0x3afa2d7b, BRF_PRG },
};

STD_ROM_PICK(g_vix357);
STD_ROM_FN(g_vix357);

struct BurnDriverD BurnDrvMDg_vix357 = {
	"g_vix357", NULL, NULL, "1900",
	"Vixen 357 (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_vix357RomInfo, g_vix357RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Waialae no Kiseki (J) [!]
static struct BurnRomInfo g_waialaRomDesc[] = {
	{ "g_waiala.bin", 0x180000, 0xcbe2c1f6, BRF_PRG },
};

STD_ROM_PICK(g_waiala);
STD_ROM_FN(g_waiala);

struct BurnDriverD BurnDrvMDg_waiala = {
	"g_waiala", NULL, NULL, "1900",
	"Waialae no Kiseki (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_waialaRomInfo, g_waialaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Warpspeed (U) [!]
static struct BurnRomInfo g_warpsRomDesc[] = {
	{ "g_warps.bin", 0x200000, 0x143697ed, BRF_PRG },
};

STD_ROM_PICK(g_warps);
STD_ROM_FN(g_warps);

struct BurnDriverD BurnDrvMDg_warps = {
	"g_warps", NULL, NULL, "1900",
	"Warpspeed (U) [!]\0", NULL, "Accolade", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_warpsRomInfo, g_warpsRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Wonder Boy V - Monster World III (J) [!]
static struct BurnRomInfo g_wboy5RomDesc[] = {
	{ "g_wboy5.bin", 0xa0000, 0x45a50f96, BRF_PRG },
};

STD_ROM_PICK(g_wboy5);
STD_ROM_FN(g_wboy5);

struct BurnDriverD BurnDrvMDg_wboy5 = {
	"g_wboy5", NULL, NULL, "1900",
	"Wonder Boy V - Monster World III (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wboy5RomInfo, g_wboy5RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Wonder Library (J) [!]
static struct BurnRomInfo g_wonlibRomDesc[] = {
	{ "g_wonlib.bin", 0x80000, 0x9350e754, BRF_PRG },
};

STD_ROM_PICK(g_wonlib);
STD_ROM_FN(g_wonlib);

struct BurnDriverD BurnDrvMDg_wonlib = {
	"g_wonlib", NULL, NULL, "1900",
	"Wonder Library (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wonlibRomInfo, g_wonlibRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// X Dazedly Ray (J) [!]
static struct BurnRomInfo g_xdazeRomDesc[] = {
	{ "g_xdaze.bin", 0x80000, 0xab22d002, BRF_PRG },
};

STD_ROM_PICK(g_xdaze);
STD_ROM_FN(g_xdaze);

struct BurnDriverD BurnDrvMDg_xdaze = {
	"g_xdaze", NULL, NULL, "1900",
	"X Dazedly Ray (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_xdazeRomInfo, g_xdazeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Yuu Yuu Hakusho Gaiden (J) [!]
static struct BurnRomInfo g_yuyugaRomDesc[] = {
	{ "g_yuyuga.bin", 0x200000, 0x7dc98176, BRF_PRG },
};

STD_ROM_PICK(g_yuyuga);
STD_ROM_FN(g_yuyuga);

struct BurnDriverD BurnDrvMDg_yuyuga = {
	"g_yuyuga", NULL, NULL, "1900",
	"Yuu Yuu Hakusho Gaiden (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_yuyugaRomInfo, g_yuyugaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Zan - Yasha Enbukyoku / Zan Yasha Enbuden (J) [!]
static struct BurnRomInfo g_zanyaRomDesc[] = {
	{ "g_zanya.bin", 0x80000, 0x637fe8f3, BRF_PRG },
};

STD_ROM_PICK(g_zanya);
STD_ROM_FN(g_zanya);

struct BurnDriverD BurnDrvMDg_zanya = {
	"g_zanya", NULL, NULL, "1991",
	"Zan - Yasha Enbukyoku / Zan Yasha Enbuden (J) [!]\0", NULL, "Wolfteam", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_zanyaRomInfo, g_zanyaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// A Ressha de Gyoukou MD (J) [!]
static struct BurnRomInfo g_areshaRomDesc[] = {
	{ "g_aresha.bin", 0x80000, 0x3D45DE4F, BRF_PRG },
};

STD_ROM_PICK(g_aresha);
STD_ROM_FN(g_aresha);

struct BurnDriver BurnDrvMDg_aresha = {
	"g_aresha", NULL, NULL, "1991",
	"A Ressha de Gyoukou MD (J) [!]\0", NULL, "Artdink", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_areshaRomInfo, g_areshaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Chou Yakyuu Miracle Nine (J) [!]
static struct BurnRomInfo g_chouyaRomDesc[] = {
	{ "g_chouya.bin", 0x200000, 0x6d8c2206, BRF_PRG },
};

STD_ROM_PICK(g_chouya);
STD_ROM_FN(g_chouya);

struct BurnDriverD BurnDrvMDg_chouya = {
	"g_chouya", NULL, NULL, "1900",
	"Chou Yakyuu Miracle Nine (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_chouyaRomInfo, g_chouyaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Dyna Brothers 2 (J) [!]
static struct BurnRomInfo g_dynab2RomDesc[] = {
	{ "g_dynab2.bin", 0x200000, 0x47e0a64e, BRF_PRG },
};

STD_ROM_PICK(g_dynab2);
STD_ROM_FN(g_dynab2);

struct BurnDriverD BurnDrvMDg_dynab2 = {
	"g_dynab2", NULL, NULL, "1900",
	"Dyna Brothers 2 (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_dynab2RomInfo, g_dynab2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// F-15 Strike Eagle II (U) [!]
static struct BurnRomInfo g_f15sRomDesc[] = {
	{ "g_f15s.bin", 0x100000, 0x412c4d60, BRF_PRG },
};

STD_ROM_PICK(g_f15s);
STD_ROM_FN(g_f15s);

struct BurnDriverD BurnDrvMDg_f15s = {
	"g_f15s", NULL, NULL, "1900",
	"F-15 Strike Eagle II (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_f15sRomInfo, g_f15sRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// F-15 Strike Eagle II (E) [!]
static struct BurnRomInfo g_f15eRomDesc[] = {
	{ "g_f15e.bin", 0x100000, 0xe98ee370, BRF_PRG },
};

STD_ROM_PICK(g_f15e);
STD_ROM_FN(g_f15e);

struct BurnDriverD BurnDrvMDg_f15e = {
	"g_f15e", "g_f15s", NULL, "1900",
	"F-15 Strike Eagle II (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_f15eRomInfo, g_f15eRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// F-15 Strike Eagle II (Beta)
static struct BurnRomInfo g_f15bRomDesc[] = {
	{ "g_f15b.bin", 0x100000, 0xfd4f5a01, BRF_PRG },
};

STD_ROM_PICK(g_f15b);
STD_ROM_FN(g_f15b);

struct BurnDriverD BurnDrvMDg_f15b = {
	"g_f15b", "g_f15s", NULL, "1900",
	"F-15 Strike Eagle II (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_f15bRomInfo, g_f15bRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Gambler Jiko Chuushinha - Katayama Masayuki no Mahjong Doujou (J) [!]
static struct BurnRomInfo g_gamblrRomDesc[] = {
	{ "g_gamblr.bin", 0x80000, 0x05650b7a, BRF_PRG },
};

STD_ROM_PICK(g_gamblr);
STD_ROM_FN(g_gamblr);

struct BurnDriverD BurnDrvMDg_gamblr = {
	"g_gamblr", NULL, NULL, "1900",
	"Gambler Jiko Chuushinha - Katayama Masayuki no Mahjong Doujou (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_gamblrRomInfo, g_gamblrRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// King Colossus (J) [!]
static struct BurnRomInfo g_kingcjRomDesc[] = {
	{ "g_kingcj.bin", 0x100000, 0xffe7b3c7, BRF_PRG },
};

STD_ROM_PICK(g_kingcj);
STD_ROM_FN(g_kingcj);

struct BurnDriverD BurnDrvMDg_kingcj = {
	"g_kingcj", NULL, NULL, "1900",
	"King Colossus (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_kingcjRomInfo, g_kingcjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Lord Monarch (J) [!]
static struct BurnRomInfo g_lordmoRomDesc[] = {
	{ "g_lordmo.bin", 0x200000, 0x238bf5db, BRF_PRG },
};

STD_ROM_PICK(g_lordmo);
STD_ROM_FN(g_lordmo);

struct BurnDriverD BurnDrvMDg_lordmo = {
	"g_lordmo", NULL, NULL, "1900",
	"Lord Monarch (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_lordmoRomInfo, g_lordmoRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mega Bomberman (UE) [!]
static struct BurnRomInfo g_mbmbRomDesc[] = {
	{ "g_mbmb.bin", 0x100000, 0x4bd6667d, BRF_PRG },
};

STD_ROM_PICK(g_mbmb);
STD_ROM_FN(g_mbmb);

struct BurnDriverD BurnDrvMDg_mbmb = {
	"g_mbmb", NULL, NULL, "1900",
	"Mega Bomberman (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mbmbRomInfo, g_mbmbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mega Bomberman (UE) [a1][!]
static struct BurnRomInfo g_mbmbaRomDesc[] = {
	{ "g_mbmba.bin", 0x100000, 0x54ab3beb, BRF_PRG },
};

STD_ROM_PICK(g_mbmba);
STD_ROM_FN(g_mbmba);

struct BurnDriverD BurnDrvMDg_mbmba = {
	"g_mbmba", "g_mbmb", NULL, "1900",
	"Mega Bomberman (UE) [a1][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mbmbaRomInfo, g_mbmbaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// NBA Jam (UE) (REV01) [!]
static struct BurnRomInfo g_njamRomDesc[] = {
	{ "g_njam.bin", 0x200000, 0xeb8360e6, BRF_PRG },
};

STD_ROM_PICK(g_njam);
STD_ROM_FN(g_njam);

struct BurnDriverD BurnDrvMDg_njam = {
	"g_njam", NULL, NULL, "1900",
	"NBA Jam (UE) (REV01) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_njamRomInfo, g_njamRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// NBA Jam (UE) (REV00) [!]
static struct BurnRomInfo g_nbaj00RomDesc[] = {
	{ "g_nbaj00.bin", 0x200000, 0x10fa248f, BRF_PRG },
};

STD_ROM_PICK(g_nbaj00);
STD_ROM_FN(g_nbaj00);

struct BurnDriverD BurnDrvMDg_nbaj00 = {
	"g_nbaj00", "g_njam", NULL, "1900",
	"NBA Jam (UE) (REV00) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_nbaj00RomInfo, g_nbaj00RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// NBA Jam (J) [!]
static struct BurnRomInfo g_nbajjRomDesc[] = {
	{ "g_nbajj.bin", 0x200000, 0xa6c6305a, BRF_PRG },
};

STD_ROM_PICK(g_nbajj);
STD_ROM_FN(g_nbajj);

struct BurnDriverD BurnDrvMDg_nbajj = {
	"g_nbajj", "g_njam", NULL, "1900",
	"NBA Jam (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_nbajjRomInfo, g_nbajjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// NBA Jam Tournament Edition (W) (REV01) [!]
static struct BurnRomInfo g_njteRomDesc[] = {
	{ "g_njte.bin", 0x300000, 0x6e25ebf0, BRF_PRG },
};

STD_ROM_PICK(g_njte);
STD_ROM_FN(g_njte);

struct BurnDriverD BurnDrvMDg_njte = {
	"g_njte", NULL, NULL, "1900",
	"NBA Jam Tournament Edition (W) (REV01) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_njteRomInfo, g_njteRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// NBA Jam Tournament Edition (W) (REV00) [!]
static struct BurnRomInfo g_nbajt0RomDesc[] = {
	{ "g_nbajt0.bin", 0x300000, 0xe9ffcb37, BRF_PRG },
};

STD_ROM_PICK(g_nbajt0);
STD_ROM_FN(g_nbajt0);

struct BurnDriverD BurnDrvMDg_nbajt0 = {
	"g_nbajt0", "g_njte", NULL, "1900",
	"NBA Jam Tournament Edition (W) (REV00) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_nbajt0RomInfo, g_nbajt0RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// NBA Hang Time (U) [!]
static struct BurnRomInfo g_nbahRomDesc[] = {
	{ "g_nbah.bin", 0x300000, 0x176b0338, BRF_PRG },
};

STD_ROM_PICK(g_nbah);
STD_ROM_FN(g_nbah);

struct BurnDriverD BurnDrvMDg_nbah = {
	"g_nbah", NULL, NULL, "1900",
	"NBA Hang Time (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_nbahRomInfo, g_nbahRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// NBA Hang Time (E) [!]
static struct BurnRomInfo g_nbahteRomDesc[] = {
	{ "g_nbahte.bin", 0x300000, 0xedb4d4aa, BRF_PRG },
};

STD_ROM_PICK(g_nbahte);
STD_ROM_FN(g_nbahte);

struct BurnDriverD BurnDrvMDg_nbahte = {
	"g_nbahte", "g_nbah", NULL, "1900",
	"NBA Hang Time (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_nbahteRomInfo, g_nbahteRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Nigel Mansell's World Championship (U) [!]
static struct BurnRomInfo g_nigelRomDesc[] = {
	{ "g_nigel.bin", 0x100000, 0x6bc57b2c, BRF_PRG },
};

STD_ROM_PICK(g_nigel);
STD_ROM_FN(g_nigel);

struct BurnDriverD BurnDrvMDg_nigel = {
	"g_nigel", NULL, NULL, "1900",
	"Nigel Mansell's World Championship (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_nigelRomInfo, g_nigelRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Nigel Mansell's World Championship (E)
static struct BurnRomInfo g_nigwceRomDesc[] = {
	{ "g_nigwce.bin", 0x100000, 0x3fe3d63b, BRF_PRG },
};

STD_ROM_PICK(g_nigwce);
STD_ROM_FN(g_nigwce);

struct BurnDriverD BurnDrvMDg_nigwce = {
	"g_nigwce", "g_nigel", NULL, "1900",
	"Nigel Mansell's World Championship (E)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_nigwceRomInfo, g_nigwceRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// OutRun (W) [!]
static struct BurnRomInfo g_orunRomDesc[] = {
	{ "g_orun.bin", 0x100000, 0xfdd9a8d2, BRF_PRG },
};

STD_ROM_PICK(g_orun);
STD_ROM_FN(g_orun);

struct BurnDriverD BurnDrvMDg_orun = {
	"g_orun", NULL, NULL, "1900",
	"OutRun (W) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_orunRomInfo, g_orunRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// OutRun (J) [!]
static struct BurnRomInfo g_orunjRomDesc[] = {
	{ "g_orunj.bin", 0x100000, 0xee7d9f4a, BRF_PRG },
};

STD_ROM_PICK(g_orunj);
STD_ROM_FN(g_orunj);

struct BurnDriverD BurnDrvMDg_orunj = {
	"g_orunj", "g_orun", NULL, "1900",
	"OutRun (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_orunjRomInfo, g_orunjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Pitfall - The Mayan Adventure (U) [!]
static struct BurnRomInfo g_pitfRomDesc[] = {
	{ "g_pitf.bin", 0x200000, 0xf917e34f, BRF_PRG },
};

STD_ROM_PICK(g_pitf);
STD_ROM_FN(g_pitf);

struct BurnDriverD BurnDrvMDg_pitf = {
	"g_pitf", NULL, NULL, "1900",
	"Pitfall - The Mayan Adventure (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pitfRomInfo, g_pitfRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Pitfall - The Mayan Adventure (E) [!]
static struct BurnRomInfo g_pitfeRomDesc[] = {
	{ "g_pitfe.bin", 0x200000, 0xc9198e19, BRF_PRG },
};

STD_ROM_PICK(g_pitfe);
STD_ROM_FN(g_pitfe);

struct BurnDriverD BurnDrvMDg_pitfe = {
	"g_pitfe", "g_pitf", NULL, "1900",
	"Pitfall - The Mayan Adventure (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pitfeRomInfo, g_pitfeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Pit Fighter (W) (Oct 1991) [!]
static struct BurnRomInfo g_pifgRomDesc[] = {
	{ "g_pifg.bin", 0x100000, 0xd48a8b02, BRF_PRG },
};

STD_ROM_PICK(g_pifg);
STD_ROM_FN(g_pifg);

struct BurnDriverD BurnDrvMDg_pifg = {
	"g_pifg", NULL, NULL, "1900",
	"Pit Fighter (W) (Oct 1991) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pifgRomInfo, g_pifgRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Pit Fighter (UE) (Jun 1991) [!]
static struct BurnRomInfo g_pifiaRomDesc[] = {
	{ "g_pifia.bin", 0x100000, 0x1e0e0831, BRF_PRG },
};

STD_ROM_PICK(g_pifia);
STD_ROM_FN(g_pifia);

struct BurnDriverD BurnDrvMDg_pifia = {
	"g_pifia", "g_pifg", NULL, "1900",
	"Pit Fighter (UE) (Jun 1991) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pifiaRomInfo, g_pifiaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Pocahontas (U) [!]
static struct BurnRomInfo g_pocaRomDesc[] = {
	{ "g_poca.bin", 0x400000, 0x6ddd1c6d, BRF_PRG },
};

STD_ROM_PICK(g_poca);
STD_ROM_FN(g_poca);

struct BurnDriverD BurnDrvMDg_poca = {
	"g_poca", NULL, NULL, "1900",
	"Pocahontas (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pocaRomInfo, g_pocaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Pocahontas (E) [!]
static struct BurnRomInfo g_pocaeRomDesc[] = {
	{ "g_pocae.bin", 0x400000, 0x165e7987, BRF_PRG },
};

STD_ROM_PICK(g_pocae);
STD_ROM_FN(g_pocae);

struct BurnDriverD BurnDrvMDg_pocae = {
	"g_pocae", "g_poca", NULL, "1900",
	"Pocahontas (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pocaeRomInfo, g_pocaeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Ren and Stimpy Show, The - Stimpy's Invention (U) [!]
static struct BurnRomInfo g_rensRomDesc[] = {
	{ "g_rens.bin", 0x100000, 0xd9503ba5, BRF_PRG },
};

STD_ROM_PICK(g_rens);
STD_ROM_FN(g_rens);

struct BurnDriverD BurnDrvMDg_rens = {
	"g_rens", NULL, NULL, "1900",
	"Ren and Stimpy Show, The - Stimpy's Invention (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_rensRomInfo, g_rensRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Ren and Stimpy Show, The - Stimpy's Invention (Beta))
static struct BurnRomInfo g_renstbRomDesc[] = {
	{ "g_renstb.bin", 0x100000, 0xfcb86336, BRF_PRG },
};

STD_ROM_PICK(g_renstb);
STD_ROM_FN(g_renstb);

struct BurnDriverD BurnDrvMDg_renstb = {
	"g_renstb", "g_rens", NULL, "1900",
	"Ren and Stimpy Show, The - Stimpy's Invention (Beta))\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_renstbRomInfo, g_renstbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Ren and Stimpy Show, The - Stimpy's Invention (E) [!]
static struct BurnRomInfo g_rensteRomDesc[] = {
	{ "g_renste.bin", 0x100000, 0xc276c220, BRF_PRG },
};

STD_ROM_PICK(g_renste);
STD_ROM_FN(g_renste);

struct BurnDriverD BurnDrvMDg_renste = {
	"g_renste", "g_rens", NULL, "1900",
	"Ren and Stimpy Show, The - Stimpy's Invention (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_rensteRomInfo, g_rensteRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Side Pocket (JU) [!]
static struct BurnRomInfo g_sideRomDesc[] = {
	{ "g_side.bin", 0x100000, 0xaf9f275d, BRF_PRG },
};

STD_ROM_PICK(g_side);
STD_ROM_FN(g_side);

struct BurnDriverD BurnDrvMDg_side = {
	"g_side", NULL, NULL, "1900",
	"Side Pocket (JU) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sideRomInfo, g_sideRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Side Pocket (E) [!]
static struct BurnRomInfo g_sidepeRomDesc[] = {
	{ "g_sidepe.bin", 0x100000, 0x36e08145, BRF_PRG },
};

STD_ROM_PICK(g_sidepe);
STD_ROM_FN(g_sidepe);

struct BurnDriverD BurnDrvMDg_sidepe = {
	"g_sidepe", "g_side", NULL, "1900",
	"Side Pocket (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sidepeRomInfo, g_sidepeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Minnesota Fats Pool Legend (U) [!]
static struct BurnRomInfo g_mfplRomDesc[] = {
	{ "g_mfpl.bin", 0x100000, 0x38174f40, BRF_PRG },
};

STD_ROM_PICK(g_mfpl);
STD_ROM_FN(g_mfpl);

struct BurnDriverD BurnDrvMDg_mfpl = {
	"g_mfpl", NULL, NULL, "1900",
	"Minnesota Fats Pool Legend (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mfplRomInfo, g_mfplRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Simpsons, The - Bart vs The Space Mutants (UE) (REV01) [!]
static struct BurnRomInfo g_bartRomDesc[] = {
	{ "g_bart.bin", 0x80000, 0xdb70e8ca, BRF_PRG },
};

STD_ROM_PICK(g_bart);
STD_ROM_FN(g_bart);

struct BurnDriver BurnDrvMDg_bart = {
	"g_bart", NULL, NULL, "1900",
	"Simpsons, The - Bart vs The Space Mutants (UE) (REV01) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_bartRomInfo, g_bartRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Simpsons, The - Bart vs The Space Mutants (UE) (REV00) [!]
static struct BurnRomInfo g_bart00RomDesc[] = {
	{ "g_bart00.bin", 0x80000, 0xc8620574, BRF_PRG },
};

STD_ROM_PICK(g_bart00);
STD_ROM_FN(g_bart00);

struct BurnDriver BurnDrvMDg_bart00 = {
	"g_bart00", "g_bart", NULL, "1900",
	"Simpsons, The - Bart vs The Space Mutants (UE) (REV00) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_bart00RomInfo, g_bart00RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sorcerian (J) [!]
static struct BurnRomInfo g_sorcerRomDesc[] = {
	{ "g_sorcer.bin", 0x80000, 0xa143a8c5, BRF_PRG },
};

STD_ROM_PICK(g_sorcer);
STD_ROM_FN(g_sorcer);

struct BurnDriverD BurnDrvMDg_sorcer = {
	"g_sorcer", NULL, NULL, "1900",
	"Sorcerian (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sorcerRomInfo, g_sorcerRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Wardner (U) [!]
static struct BurnRomInfo g_wardRomDesc[] = {
	{ "g_ward.bin", 0x80000, 0x1e369ae2, BRF_PRG },
};

STD_ROM_PICK(g_ward);
STD_ROM_FN(g_ward);

struct BurnDriverD BurnDrvMDg_ward = {
	"g_ward", NULL, NULL, "1900",
	"Wardner (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wardRomInfo, g_wardRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Wardner no Mori Special (J) [!]
static struct BurnRomInfo g_wardjRomDesc[] = {
	{ "g_wardj.bin", 0x80000, 0x80f1035c, BRF_PRG },
};

STD_ROM_PICK(g_wardj);
STD_ROM_FN(g_wardj);

struct BurnDriverD BurnDrvMDg_wardj = {
	"g_wardj", "g_ward", NULL, "1900",
	"Wardner no Mori Special (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wardjRomInfo, g_wardjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Wiz 'n' Liz - The Frantic Wabbit Wescue (U) [!]
static struct BurnRomInfo g_wizlRomDesc[] = {
	{ "g_wizl.bin", 0x100000, 0xdf036b62, BRF_PRG },
};

STD_ROM_PICK(g_wizl);
STD_ROM_FN(g_wizl);

struct BurnDriverD BurnDrvMDg_wizl = {
	"g_wizl", NULL, NULL, "1900",
	"Wiz 'n' Liz - The Frantic Wabbit Wescue (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wizlRomInfo, g_wizlRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Wiz 'n' Liz - The Frantic Wabbit Wescue (E) [!]
static struct BurnRomInfo g_wizlieRomDesc[] = {
	{ "g_wizlie.bin", 0x100000, 0xf09353b4, BRF_PRG },
};

STD_ROM_PICK(g_wizlie);
STD_ROM_FN(g_wizlie);

struct BurnDriverD BurnDrvMDg_wizlie = {
	"g_wizlie", "g_wizl", NULL, "1900",
	"Wiz 'n' Liz - The Frantic Wabbit Wescue (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wizlieRomInfo, g_wizlieRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Zero the Kamikaze Squirrel (U) [!]
static struct BurnRomInfo g_ztksRomDesc[] = {
	{ "g_ztks.bin", 0x200000, 0x423968df, BRF_PRG },
};

STD_ROM_PICK(g_ztks);
STD_ROM_FN(g_ztks);

struct BurnDriverD BurnDrvMDg_ztks = {
	"g_ztks", NULL, NULL, "1900",
	"Zero the Kamikaze Squirrel (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ztksRomInfo, g_ztksRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Zero the Kamikaze Squirrel (E) [!]
static struct BurnRomInfo g_ztkseRomDesc[] = {
	{ "g_ztkse.bin", 0x200000, 0x45ff0b4b, BRF_PRG },
};

STD_ROM_PICK(g_ztkse);
STD_ROM_FN(g_ztkse);

struct BurnDriverD BurnDrvMDg_ztkse = {
	"g_ztkse", "g_ztks", NULL, "1900",
	"Zero the Kamikaze Squirrel (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ztkseRomInfo, g_ztkseRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Back to the Future Part III (U) [!]
static struct BurnRomInfo g_bttfRomDesc[] = {
	{ "g_bttf.bin", 0x80000, 0x66a388c3, BRF_PRG },
};

STD_ROM_PICK(g_bttf);
STD_ROM_FN(g_bttf);

struct BurnDriverD BurnDrvMDg_bttf = {
	"g_bttf", NULL, NULL, "1900",
	"Back to the Future Part III (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_bttfRomInfo, g_bttfRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Back to the Future Part III (E) [!]
static struct BurnRomInfo g_bttfeRomDesc[] = {
	{ "g_bttfe.bin", 0x80000, 0x2737f92e, BRF_PRG },
};

STD_ROM_PICK(g_bttfe);
STD_ROM_FN(g_bttfe);

struct BurnDriverD BurnDrvMDg_bttfe = {
	"g_bttfe", "g_bttf", NULL, "1900",
	"Back to the Future Part III (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_bttfeRomInfo, g_bttfeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Beast Wrestler (U) [!]
static struct BurnRomInfo g_bwreRomDesc[] = {
	{ "g_bwre.bin", 0x100000, 0x0ca5bb64, BRF_PRG },
};

STD_ROM_PICK(g_bwre);
STD_ROM_FN(g_bwre);

struct BurnDriverD BurnDrvMDg_bwre = {
	"g_bwre", NULL, NULL, "1900",
	"Beast Wrestler (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_bwreRomInfo, g_bwreRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Beast Warriors (J) [!]
static struct BurnRomInfo g_beaswrRomDesc[] = {
	{ "g_beaswr.bin", 0x100000, 0x4646c694, BRF_PRG },
};

STD_ROM_PICK(g_beaswr);
STD_ROM_FN(g_beaswr);

struct BurnDriverD BurnDrvMDg_beaswr = {
	"g_beaswr", "g_bwre", NULL, "1900",
	"Beast Warriors (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_beaswrRomInfo, g_beaswrRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Chuck Rock (U) [c][!]
static struct BurnRomInfo g_chukRomDesc[] = {
	{ "g_chuk.bin", 0x100000, 0xf8ac454a, BRF_PRG },
};

STD_ROM_PICK(g_chuk);
STD_ROM_FN(g_chuk);

struct BurnDriverD BurnDrvMDg_chuk = {
	"g_chuk", NULL, NULL, "1900",
	"Chuck Rock (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_chukRomInfo, g_chukRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Chuck Rock (E) [c][!]
static struct BurnRomInfo g_chkrkeRomDesc[] = {
	{ "g_chkrke.bin", 0x100000, 0x7cd40bea, BRF_PRG },
};

STD_ROM_PICK(g_chkrke);
STD_ROM_FN(g_chkrke);

struct BurnDriverD BurnDrvMDg_chkrke = {
	"g_chkrke", "g_chuk", NULL, "1900",
	"Chuck Rock (E) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_chkrkeRomInfo, g_chkrkeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Chuck II - Son of Chuck (U) [!]
static struct BurnRomInfo g_chk2RomDesc[] = {
	{ "g_chk2.bin", 0x100000, 0x408b1cdb, BRF_PRG },
};

STD_ROM_PICK(g_chk2);
STD_ROM_FN(g_chk2);

struct BurnDriverD BurnDrvMDg_chk2 = {
	"g_chk2", NULL, NULL, "1900",
	"Chuck II - Son of Chuck (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_chk2RomInfo, g_chk2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Chuck II - Son of Chuck (Beta)
static struct BurnRomInfo g_chk2bRomDesc[] = {
	{ "g_chk2b.bin", 0x100000, 0xd6a3b324, BRF_PRG },
};

STD_ROM_PICK(g_chk2b);
STD_ROM_FN(g_chk2b);

struct BurnDriverD BurnDrvMDg_chk2b = {
	"g_chk2b", "g_chk2", NULL, "1900",
	"Chuck II - Son of Chuck (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_chk2bRomInfo, g_chk2bRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Chuck II - Son of Chuck (E) [c][!]
static struct BurnRomInfo g_chk2eRomDesc[] = {
	{ "g_chk2e.bin", 0x100000, 0x1ade9488, BRF_PRG },
};

STD_ROM_PICK(g_chk2e);
STD_ROM_FN(g_chk2e);

struct BurnDriverD BurnDrvMDg_chk2e = {
	"g_chk2e", "g_chk2", NULL, "1900",
	"Chuck II - Son of Chuck (E) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_chk2eRomInfo, g_chk2eRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Chuck II - Son of Chuck (J) [!]
static struct BurnRomInfo g_chk2jRomDesc[] = {
	{ "g_chk2j.bin", 0x100000, 0xbfd24be8, BRF_PRG },
};

STD_ROM_PICK(g_chk2j);
STD_ROM_FN(g_chk2j);

struct BurnDriverD BurnDrvMDg_chk2j = {
	"g_chk2j", "g_chk2", NULL, "1900",
	"Chuck II - Son of Chuck (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_chk2jRomInfo, g_chk2jRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Crack Down (U) [!]
static struct BurnRomInfo g_crkdRomDesc[] = {
	{ "g_crkd.bin", 0x80000, 0xb9ce9051, BRF_PRG },
};

STD_ROM_PICK(g_crkd);
STD_ROM_FN(g_crkd);

struct BurnDriverD BurnDrvMDg_crkd = {
	"g_crkd", NULL, NULL, "1900",
	"Crack Down (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_crkdRomInfo, g_crkdRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Crack Down (E) [!]
static struct BurnRomInfo g_crkdeRomDesc[] = {
	{ "g_crkde.bin", 0x80000, 0xd012a47a, BRF_PRG },
};

STD_ROM_PICK(g_crkde);
STD_ROM_FN(g_crkde);

struct BurnDriverD BurnDrvMDg_crkde = {
	"g_crkde", "g_crkd", NULL, "1900",
	"Crack Down (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_crkdeRomInfo, g_crkdeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Crack Down (J) [!]
static struct BurnRomInfo g_crkdjRomDesc[] = {
	{ "g_crkdj.bin", 0x80000, 0x538aaa5d, BRF_PRG },
};

STD_ROM_PICK(g_crkdj);
STD_ROM_FN(g_crkdj);

struct BurnDriverD BurnDrvMDg_crkdj = {
	"g_crkdj", "g_crkd", NULL, "1900",
	"Crack Down (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_crkdjRomInfo, g_crkdjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// DJ Boy (U) [!]
static struct BurnRomInfo g_djbyRomDesc[] = {
	{ "g_djby.bin", 0x80000, 0xdc9f02db, BRF_PRG },
};

STD_ROM_PICK(g_djby);
STD_ROM_FN(g_djby);

struct BurnDriverD BurnDrvMDg_djby = {
	"g_djby", NULL, NULL, "1900",
	"DJ Boy (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_djbyRomInfo, g_djbyRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// DJ Boy (E) [!]
static struct BurnRomInfo g_djboyeRomDesc[] = {
	{ "g_djboye.bin", 0x80000, 0x860e749a, BRF_PRG },
};

STD_ROM_PICK(g_djboye);
STD_ROM_FN(g_djboye);

struct BurnDriverD BurnDrvMDg_djboye = {
	"g_djboye", "g_djby", NULL, "1900",
	"DJ Boy (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_djboyeRomInfo, g_djboyeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// DJ Boy (J) [!]
static struct BurnRomInfo g_djboyjRomDesc[] = {
	{ "g_djboyj.bin", 0x80000, 0x202abaa8, BRF_PRG },
};

STD_ROM_PICK(g_djboyj);
STD_ROM_FN(g_djboyj);

struct BurnDriverD BurnDrvMDg_djboyj = {
	"g_djboyj", "g_djby", NULL, "1900",
	"DJ Boy (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_djboyjRomInfo, g_djboyjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// ESWAT Cyber Police - City Under Siege (U) [!]
static struct BurnRomInfo g_eswaRomDesc[] = {
	{ "g_eswa.bin", 0x80000, 0xe72f8a36, BRF_PRG },
};

STD_ROM_PICK(g_eswa);
STD_ROM_FN(g_eswa);

struct BurnDriverD BurnDrvMDg_eswa = {
	"g_eswa", NULL, NULL, "1900",
	"ESWAT Cyber Police - City Under Siege (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_eswaRomInfo, g_eswaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// ESWAT Cyber Police (J) [!]
static struct BurnRomInfo g_eswatjRomDesc[] = {
	{ "g_eswatj.bin", 0x80000, 0x87b636a2, BRF_PRG },
};

STD_ROM_PICK(g_eswatj);
STD_ROM_FN(g_eswatj);

struct BurnDriverD BurnDrvMDg_eswatj = {
	"g_eswatj", "g_eswa", NULL, "1900",
	"ESWAT Cyber Police (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_eswatjRomInfo, g_eswatjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// World Heroes (U) [!]
static struct BurnRomInfo g_wherRomDesc[] = {
	{ "g_wher.bin", 0x200000, 0x0f4d22ec, BRF_PRG },
};

STD_ROM_PICK(g_wher);
STD_ROM_FN(g_wher);

struct BurnDriverD BurnDrvMDg_wher = {
	"g_wher", NULL, NULL, "1900",
	"World Heroes (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wherRomInfo, g_wherRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// World Heroes (J) [!]
static struct BurnRomInfo g_wherojRomDesc[] = {
	{ "g_wheroj.bin", 0x200000, 0x56e3ceff, BRF_PRG },
};

STD_ROM_PICK(g_wheroj);
STD_ROM_FN(g_wheroj);

struct BurnDriverD BurnDrvMDg_wheroj = {
	"g_wheroj", "g_wher", NULL, "1900",
	"World Heroes (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wherojRomInfo, g_wherojRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Uzu Keobukseon (K) [!]
static struct BurnRomInfo g_uzukeRomDesc[] = {
	{ "g_uzuke.bin", 0x80000, 0xa7255ba8, BRF_PRG },
};

STD_ROM_PICK(g_uzuke);
STD_ROM_FN(g_uzuke);

struct BurnDriverD BurnDrvMDg_uzuke = {
	"g_uzuke", NULL, NULL, "1900",
	"Uzu Keobukseon (K) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_uzukeRomInfo, g_uzukeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// F1 World Championship (U) [!]
static struct BurnRomInfo g_f1wcRomDesc[] = {
	{ "g_f1wc.bin", 0x100000, 0xccd73738, BRF_PRG },
};

STD_ROM_PICK(g_f1wc);
STD_ROM_FN(g_f1wc);

struct BurnDriverD BurnDrvMDg_f1wc = {
	"g_f1wc", NULL, NULL, "1900",
	"F1 World Championship (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_f1wcRomInfo, g_f1wcRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// F1 World Championship (E) [!]
static struct BurnRomInfo g_f1wlceRomDesc[] = {
	{ "g_f1wlce.bin", 0x100000, 0xfbdd4520, BRF_PRG },
};

STD_ROM_PICK(g_f1wlce);
STD_ROM_FN(g_f1wlce);

struct BurnDriverD BurnDrvMDg_f1wlce = {
	"g_f1wlce", "g_f1wc", NULL, "1900",
	"F1 World Championship (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_f1wlceRomInfo, g_f1wlceRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Ferias Frustradas do Pica-Pau (B) [!]
static struct BurnRomInfo g_feriasRomDesc[] = {
	{ "g_ferias.bin", 0x100000, 0x7b2e416d, BRF_PRG },
};

STD_ROM_PICK(g_ferias);
STD_ROM_FN(g_ferias);

struct BurnDriverD BurnDrvMDg_ferias = {
	"g_ferias", NULL, NULL, "1900",
	"Ferias Frustradas do Pica-Pau (B) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_feriasRomInfo, g_feriasRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Fighting Masters (U) [!]
static struct BurnRomInfo g_fmasRomDesc[] = {
	{ "g_fmas.bin", 0x80000, 0x5f51983b, BRF_PRG },
};

STD_ROM_PICK(g_fmas);
STD_ROM_FN(g_fmas);

struct BurnDriverD BurnDrvMDg_fmas = {
	"g_fmas", NULL, NULL, "1900",
	"Fighting Masters (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_fmasRomInfo, g_fmasRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Fighting Masters (J) [!]
static struct BurnRomInfo g_fghmsjRomDesc[] = {
	{ "g_fghmsj.bin", 0x80000, 0x39be80ec, BRF_PRG },
};

STD_ROM_PICK(g_fghmsj);
STD_ROM_FN(g_fghmsj);

struct BurnDriverD BurnDrvMDg_fghmsj = {
	"g_fghmsj", "g_fmas", NULL, "1900",
	"Fighting Masters (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_fghmsjRomInfo, g_fghmsjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Flintstones, The (U) [c][!]
static struct BurnRomInfo g_flinRomDesc[] = {
	{ "g_flin.bin", 0x80000, 0x7c982c59, BRF_PRG },
};

STD_ROM_PICK(g_flin);
STD_ROM_FN(g_flin);

struct BurnDriverD BurnDrvMDg_flin = {
	"g_flin", NULL, NULL, "1900",
	"Flintstones, The (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_flinRomInfo, g_flinRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Flintstones, The (E) [c][!]
static struct BurnRomInfo g_flinteRomDesc[] = {
	{ "g_flinte.bin", 0x80000, 0x21845d61, BRF_PRG },
};

STD_ROM_PICK(g_flinte);
STD_ROM_FN(g_flinte);

struct BurnDriverD BurnDrvMDg_flinte = {
	"g_flinte", "g_flin", NULL, "1900",
	"Flintstones, The (E) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_flinteRomInfo, g_flinteRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Flintstones, The (J) [c][!]
static struct BurnRomInfo g_flintjRomDesc[] = {
	{ "g_flintj.bin", 0x80000, 0x920a3031, BRF_PRG },
};

STD_ROM_PICK(g_flintj);
STD_ROM_FN(g_flintj);

struct BurnDriverD BurnDrvMDg_flintj = {
	"g_flintj", "g_flin", NULL, "1900",
	"Flintstones, The (J) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_flintjRomInfo, g_flintjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Forgotten Worlds (W) (REV01) [!]
static struct BurnRomInfo g_fwRomDesc[] = {
	{ "g_fw.bin", 0x80000, 0x95513985, BRF_PRG },
};

STD_ROM_PICK(g_fw);
STD_ROM_FN(g_fw);

struct BurnDriverD BurnDrvMDg_fw = {
	"g_fw", NULL, NULL, "1900",
	"Forgotten Worlds (W) (REV01) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_fwRomInfo, g_fwRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Forgotten Worlds (W) (REV00) [!]
static struct BurnRomInfo g_fw00RomDesc[] = {
	{ "g_fw00.bin", 0x80000, 0xd0ee6434, BRF_PRG },
};

STD_ROM_PICK(g_fw00);
STD_ROM_FN(g_fw00);

struct BurnDriverD BurnDrvMDg_fw00 = {
	"g_fw00", "g_fw", NULL, "1900",
	"Forgotten Worlds (W) (REV00) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_fw00RomInfo, g_fw00RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// George Foreman's KO Boxing (U) [!]
static struct BurnRomInfo g_gfkoRomDesc[] = {
	{ "g_gfko.bin", 0x100000, 0xe1fdc787, BRF_PRG },
};

STD_ROM_PICK(g_gfko);
STD_ROM_FN(g_gfko);

struct BurnDriverD BurnDrvMDg_gfko = {
	"g_gfko", NULL, NULL, "1900",
	"George Foreman's KO Boxing (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_gfkoRomInfo, g_gfkoRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// George Foreman's KO Boxing (E) [!]
static struct BurnRomInfo g_gfkobeRomDesc[] = {
	{ "g_gfkobe.bin", 0x100000, 0xbd556381, BRF_PRG },
};

STD_ROM_PICK(g_gfkobe);
STD_ROM_FN(g_gfkobe);

struct BurnDriverD BurnDrvMDg_gfkobe = {
	"g_gfkobe", "g_gfko", NULL, "1900",
	"George Foreman's KO Boxing (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_gfkobeRomInfo, g_gfkobeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Hellfire (U) [!]
static struct BurnRomInfo g_hellRomDesc[] = {
	{ "g_hell.bin", 0x80000, 0x184018f9, BRF_PRG },
};

STD_ROM_PICK(g_hell);
STD_ROM_FN(g_hell);

struct BurnDriverD BurnDrvMDg_hell = {
	"g_hell", NULL, NULL, "1900",
	"Hellfire (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_hellRomInfo, g_hellRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Hellfire (J) [!]
static struct BurnRomInfo g_helfijRomDesc[] = {
	{ "g_helfij.bin", 0x80000, 0x8e5e13ba, BRF_PRG },
};

STD_ROM_PICK(g_helfij);
STD_ROM_FN(g_helfij);

struct BurnDriverD BurnDrvMDg_helfij = {
	"g_helfij", "g_hell", NULL, "1900",
	"Hellfire (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_helfijRomInfo, g_helfijRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Hellfire (E)
static struct BurnRomInfo g_helfieRomDesc[] = {
	{ "g_helfie.bin", 0x80000, 0xcf30acec, BRF_PRG },
};

STD_ROM_PICK(g_helfie);
STD_ROM_FN(g_helfie);

struct BurnDriverD BurnDrvMDg_helfie = {
	"g_helfie", "g_hell", NULL, "1900",
	"Hellfire (E)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_helfieRomInfo, g_helfieRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Hybrid Front, The (J) [!]
static struct BurnRomInfo g_hybridRomDesc[] = {
	{ "g_hybrid.bin", 0x200000, 0xa1f1cfe7, BRF_PRG },
};

STD_ROM_PICK(g_hybrid);
STD_ROM_FN(g_hybrid);

struct BurnDriverD BurnDrvMDg_hybrid = {
	"g_hybrid", NULL, NULL, "1900",
	"Hybrid Front, The (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_hybridRomInfo, g_hybridRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Hybrid Front, The (J) (Prototype) [!]
static struct BurnRomInfo g_hybripRomDesc[] = {
	{ "g_hybrip.bin", 0x300000, 0x04f02687, BRF_PRG },
};

STD_ROM_PICK(g_hybrip);
STD_ROM_FN(g_hybrip);

struct BurnDriverD BurnDrvMDg_hybrip = {
	"g_hybrip", "g_hybrid", NULL, "1900",
	"Hybrid Front, The (J) (Prototype) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_hybripRomInfo, g_hybripRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Hyper Dunk - The Playoff Edition (E) [!]
static struct BurnRomInfo g_hyduRomDesc[] = {
	{ "g_hydu.bin", 0x200000, 0xf27c576a, BRF_PRG },
};

STD_ROM_PICK(g_hydu);
STD_ROM_FN(g_hydu);

struct BurnDriverD BurnDrvMDg_hydu = {
	"g_hydu", NULL, NULL, "1900",
	"Hyper Dunk - The Playoff Edition (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_hyduRomInfo, g_hyduRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Hyper Dunk - The Playoff Edition (Beta)
static struct BurnRomInfo g_hdunkbRomDesc[] = {
	{ "g_hdunkb.bin", 0x200000, 0xdb124bbb, BRF_PRG },
};

STD_ROM_PICK(g_hdunkb);
STD_ROM_FN(g_hdunkb);

struct BurnDriverD BurnDrvMDg_hdunkb = {
	"g_hdunkb", "g_hydu", NULL, "1900",
	"Hyper Dunk - The Playoff Edition (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_hdunkbRomInfo, g_hdunkbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Hyper Dunk - The Playoff Edition (J) [!]
static struct BurnRomInfo g_hdunkjRomDesc[] = {
	{ "g_hdunkj.bin", 0x200000, 0x5baf53d7, BRF_PRG },
};

STD_ROM_PICK(g_hdunkj);
STD_ROM_FN(g_hdunkj);

struct BurnDriverD BurnDrvMDg_hdunkj = {
	"g_hdunkj", "g_hydu", NULL, "1900",
	"Hyper Dunk - The Playoff Edition (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_hdunkjRomInfo, g_hdunkjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Indiana Jones and the Last Crusade (U) [c][!]
static struct BurnRomInfo g_indlRomDesc[] = {
	{ "g_indl.bin", 0x100000, 0x3599a3fd, BRF_PRG },
};

STD_ROM_PICK(g_indl);
STD_ROM_FN(g_indl);

struct BurnDriverD BurnDrvMDg_indl = {
	"g_indl", NULL, NULL, "1900",
	"Indiana Jones and the Last Crusade (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_indlRomInfo, g_indlRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Indiana Jones and the Last Crusade (E) [c][!]
static struct BurnRomInfo g_indlceRomDesc[] = {
	{ "g_indlce.bin", 0x100000, 0xeb8f4374, BRF_PRG },
};

STD_ROM_PICK(g_indlce);
STD_ROM_FN(g_indlce);

struct BurnDriverD BurnDrvMDg_indlce = {
	"g_indlce", "g_indl", NULL, "1900",
	"Indiana Jones and the Last Crusade (E) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_indlceRomInfo, g_indlceRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Insector X (U) [!]
static struct BurnRomInfo g_insxRomDesc[] = {
	{ "g_insx.bin", 0x80000, 0x70626304, BRF_PRG },
};

STD_ROM_PICK(g_insx);
STD_ROM_FN(g_insx);

struct BurnDriverD BurnDrvMDg_insx = {
	"g_insx", NULL, NULL, "1900",
	"Insector X (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_insxRomInfo, g_insxRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Insector X (J) [!]
static struct BurnRomInfo g_insxjRomDesc[] = {
	{ "g_insxj.bin", 0x80000, 0x9625c434, BRF_PRG },
};

STD_ROM_PICK(g_insxj);
STD_ROM_FN(g_insxj);

struct BurnDriverD BurnDrvMDg_insxj = {
	"g_insxj", "g_insx", NULL, "1900",
	"Insector X (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_insxjRomInfo, g_insxjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// J. League Pro Striker (J) (REV03) [c][!]
static struct BurnRomInfo g_jlpRomDesc[] = {
	{ "g_jlp.bin", 0x100000, 0x2d5b7a11, BRF_PRG },
};

STD_ROM_PICK(g_jlp);
STD_ROM_FN(g_jlp);

struct BurnDriverD BurnDrvMDg_jlp = {
	"g_jlp", NULL, NULL, "1900",
	"J. League Pro Striker (J) (REV03) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jlpRomInfo, g_jlpRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// J. League Pro Striker (J) (REV00) [c][!]
static struct BurnRomInfo g_jlps00RomDesc[] = {
	{ "g_jlps00.bin", 0x100000, 0xec229156, BRF_PRG },
};

STD_ROM_PICK(g_jlps00);
STD_ROM_FN(g_jlps00);

struct BurnDriverD BurnDrvMDg_jlps00 = {
	"g_jlps00", "g_jlp", NULL, "1900",
	"J. League Pro Striker (J) (REV00) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jlps00RomInfo, g_jlps00RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Jewel Master (UE) [!]
static struct BurnRomInfo g_jewlRomDesc[] = {
	{ "g_jewl.bin", 0x80000, 0xcee98813, BRF_PRG },
};

STD_ROM_PICK(g_jewl);
STD_ROM_FN(g_jewl);

struct BurnDriverD BurnDrvMDg_jewl = {
	"g_jewl", NULL, NULL, "1900",
	"Jewel Master (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jewlRomInfo, g_jewlRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Jewel Master (J) [c][!]
static struct BurnRomInfo g_jewljRomDesc[] = {
	{ "g_jewlj.bin", 0x80000, 0x2cf6926c, BRF_PRG },
};

STD_ROM_PICK(g_jewlj);
STD_ROM_FN(g_jewlj);

struct BurnDriverD BurnDrvMDg_jewlj = {
	"g_jewlj", "g_jewl", NULL, "1900",
	"Jewel Master (J) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jewljRomInfo, g_jewljRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Jordan vs Bird - Super One-on-One (UE) (REV01) [!]
static struct BurnRomInfo g_jb11RomDesc[] = {
	{ "g_jb11.bin", 0x80000, 0x4d3ddd7c, BRF_PRG },
};

STD_ROM_PICK(g_jb11);
STD_ROM_FN(g_jb11);

struct BurnDriverD BurnDrvMDg_jb11 = {
	"g_jb11", NULL, NULL, "1900",
	"Jordan vs Bird - Super One-on-One (UE) (REV01) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jb11RomInfo, g_jb11RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Jordan vs Bird - Super One-on-One (J) [!]
static struct BurnRomInfo g_jb11jRomDesc[] = {
	{ "g_jb11j.bin", 0x80000, 0x8837e896, BRF_PRG },
};

STD_ROM_PICK(g_jb11j);
STD_ROM_FN(g_jb11j);

struct BurnDriverD BurnDrvMDg_jb11j = {
	"g_jb11j", "g_jb11", NULL, "1900",
	"Jordan vs Bird - Super One-on-One (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jb11jRomInfo, g_jb11jRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Jordan vs Bird - Super One-on-One (UE) (REV00) [!]
static struct BurnRomInfo g_jb1100RomDesc[] = {
	{ "g_jb1100.bin", 0x80000, 0x22d77e6d, BRF_PRG },
};

STD_ROM_PICK(g_jb1100);
STD_ROM_FN(g_jb1100);

struct BurnDriverD BurnDrvMDg_jb1100 = {
	"g_jb1100", "g_jb11", NULL, "1900",
	"Jordan vs Bird - Super One-on-One (UE) (REV00) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_jb1100RomInfo, g_jb1100RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// King Salmon - The Big Catch (U) [c][!]
static struct BurnRomInfo g_ksalRomDesc[] = {
	{ "g_ksal.bin", 0x80000, 0xf516e7d9, BRF_PRG },
};

STD_ROM_PICK(g_ksal);
STD_ROM_FN(g_ksal);

struct BurnDriverD BurnDrvMDg_ksal = {
	"g_ksal", NULL, NULL, "1900",
	"King Salmon - The Big Catch (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ksalRomInfo, g_ksalRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// King Salmon - The Big Catch (J) [c][!]
static struct BurnRomInfo g_ksalmjRomDesc[] = {
	{ "g_ksalmj.bin", 0x80000, 0x2cfc9f61, BRF_PRG },
};

STD_ROM_PICK(g_ksalmj);
STD_ROM_FN(g_ksalmj);

struct BurnDriverD BurnDrvMDg_ksalmj = {
	"g_ksalmj", "g_ksal", NULL, "1900",
	"King Salmon - The Big Catch (J) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ksalmjRomInfo, g_ksalmjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// LHX Attack Chopper (UE) [!]
static struct BurnRomInfo g_lhxRomDesc[] = {
	{ "g_lhx.bin", 0x100000, 0x70c3428d, BRF_PRG },
};

STD_ROM_PICK(g_lhx);
STD_ROM_FN(g_lhx);

struct BurnDriverD BurnDrvMDg_lhx = {
	"g_lhx", NULL, NULL, "1900",
	"LHX Attack Chopper (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_lhxRomInfo, g_lhxRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// LHX Attack Chopper (J) [!]
static struct BurnRomInfo g_lhxjRomDesc[] = {
	{ "g_lhxj.bin", 0x100000, 0x224ff103, BRF_PRG },
};

STD_ROM_PICK(g_lhxj);
STD_ROM_FN(g_lhxj);

struct BurnDriverD BurnDrvMDg_lhxj = {
	"g_lhxj", "g_lhx", NULL, "1900",
	"LHX Attack Chopper (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_lhxjRomInfo, g_lhxjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Midnight Resistance (U) [!]
static struct BurnRomInfo g_midrRomDesc[] = {
	{ "g_midr.bin", 0x100000, 0x187c6af6, BRF_PRG },
};

STD_ROM_PICK(g_midr);
STD_ROM_FN(g_midr);

struct BurnDriverD BurnDrvMDg_midr = {
	"g_midr", NULL, NULL, "1900",
	"Midnight Resistance (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_midrRomInfo, g_midrRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Midnight Resistance (J) [!]
static struct BurnRomInfo g_midrejRomDesc[] = {
	{ "g_midrej.bin", 0x100000, 0x8f3f6e4d, BRF_PRG },
};

STD_ROM_PICK(g_midrej);
STD_ROM_FN(g_midrej);

struct BurnDriverD BurnDrvMDg_midrej = {
	"g_midrej", "g_midr", NULL, "1900",
	"Midnight Resistance (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_midrejRomInfo, g_midrejRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mig-29 Fighter Pilot (U) [!]
static struct BurnRomInfo g_m29RomDesc[] = {
	{ "g_m29.bin", 0x100000, 0x59ccabb2, BRF_PRG },
};

STD_ROM_PICK(g_m29);
STD_ROM_FN(g_m29);

struct BurnDriverD BurnDrvMDg_m29 = {
	"g_m29", NULL, NULL, "1900",
	"Mig-29 Fighter Pilot (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_m29RomInfo, g_m29RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mig-29 Fighter Pilot (E) [!]
static struct BurnRomInfo g_mig29eRomDesc[] = {
	{ "g_mig29e.bin", 0x100000, 0x70b0a5d7, BRF_PRG },
};

STD_ROM_PICK(g_mig29e);
STD_ROM_FN(g_mig29e);

struct BurnDriverD BurnDrvMDg_mig29e = {
	"g_mig29e", "g_m29", NULL, "1900",
	"Mig-29 Fighter Pilot (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mig29eRomInfo, g_mig29eRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mike Ditka Power Football (U) [!]
static struct BurnRomInfo g_mikeRomDesc[] = {
	{ "g_mike.bin", 0x100000, 0x6078b310, BRF_PRG },
};

STD_ROM_PICK(g_mike);
STD_ROM_FN(g_mike);

struct BurnDriverD BurnDrvMDg_mike = {
	"g_mike", NULL, NULL, "1900",
	"Mike Ditka Power Football (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mikeRomInfo, g_mikeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mike Ditka Power Football (U) [a1][!]
static struct BurnRomInfo g_mdpfuaRomDesc[] = {
	{ "g_mdpfua.bin", 0x100000, 0xde50ca8e, BRF_PRG },
};

STD_ROM_PICK(g_mdpfua);
STD_ROM_FN(g_mdpfua);

struct BurnDriverD BurnDrvMDg_mdpfua = {
	"g_mdpfua", "g_mike", NULL, "1900",
	"Mike Ditka Power Football (U) [a1][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mdpfuaRomInfo, g_mdpfuaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mutant League Football (UE) [!]
static struct BurnRomInfo g_mlfootRomDesc[] = {
	{ "g_mlfoot.bin", 0x100000, 0xdce29c9d, BRF_PRG },
};

STD_ROM_PICK(g_mlfoot);
STD_ROM_FN(g_mlfoot);

struct BurnDriverD BurnDrvMDg_mlfoot = {
	"g_mlfoot", NULL, NULL, "1900",
	"Mutant League Football (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mlfootRomInfo, g_mlfootRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mutant League Football (J) [!]
static struct BurnRomInfo g_mutlfjRomDesc[] = {
	{ "g_mutlfj.bin", 0x100000, 0x2a97e6af, BRF_PRG },
};

STD_ROM_PICK(g_mutlfj);
STD_ROM_FN(g_mutlfj);

struct BurnDriverD BurnDrvMDg_mutlfj = {
	"g_mutlfj", "g_mlfoot", NULL, "1900",
	"Mutant League Football (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mutlfjRomInfo, g_mutlfjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mystic Defender (W) (REV01) [!]
static struct BurnRomInfo g_mysdRomDesc[] = {
	{ "g_mysd.bin", 0x80000, 0x50fd5d93, BRF_PRG },
};

STD_ROM_PICK(g_mysd);
STD_ROM_FN(g_mysd);

struct BurnDriverD BurnDrvMDg_mysd = {
	"g_mysd", NULL, NULL, "1900",
	"Mystic Defender (W) (REV01) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mysdRomInfo, g_mysdRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Mystic Defender (W) (REV00) [!]
static struct BurnRomInfo g_mysd00RomDesc[] = {
	{ "g_mysd00.bin", 0x80000, 0xf9ce1ab8, BRF_PRG },
};

STD_ROM_PICK(g_mysd00);
STD_ROM_FN(g_mysd00);

struct BurnDriverD BurnDrvMDg_mysd00 = {
	"g_mysd00", "g_mysd", NULL, "1900",
	"Mystic Defender (W) (REV00) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mysd00RomInfo, g_mysd00RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Kujaku Ou 2 (J) [!]
static struct BurnRomInfo g_kujakuRomDesc[] = {
	{ "g_kujaku.bin", 0x60000, 0xaffd56bc, BRF_PRG },
};

STD_ROM_PICK(g_kujaku);
STD_ROM_FN(g_kujaku);

struct BurnDriverD BurnDrvMDg_kujaku = {
	"g_kujaku", "g_mysd", NULL, "1900",
	"Kujaku Ou 2 (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_kujakuRomInfo, g_kujakuRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Olympic Winter Games - Lillehammer 94 (U) [c][!]
static struct BurnRomInfo g_olwgRomDesc[] = {
	{ "g_olwg.bin", 0x200000, 0xc5834437, BRF_PRG },
};

STD_ROM_PICK(g_olwg);
STD_ROM_FN(g_olwg);

struct BurnDriverD BurnDrvMDg_olwg = {
	"g_olwg", NULL, NULL, "1900",
	"Olympic Winter Games - Lillehammer 94 (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_olwgRomInfo, g_olwgRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Olympic Winter Games - Lillehammer 94 (E) [c][!]
static struct BurnRomInfo g_olwgeRomDesc[] = {
	{ "g_olwge.bin", 0x200000, 0xfa537a45, BRF_PRG },
};

STD_ROM_PICK(g_olwge);
STD_ROM_FN(g_olwge);

struct BurnDriverD BurnDrvMDg_olwge = {
	"g_olwge", "g_olwg", NULL, "1900",
	"Olympic Winter Games - Lillehammer 94 (E) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_olwgeRomInfo, g_olwgeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Olympic Winter Games - Lillehammer 94 (J) [c][!]
static struct BurnRomInfo g_olwgjRomDesc[] = {
	{ "g_olwgj.bin", 0x200000, 0x654a4684, BRF_PRG },
};

STD_ROM_PICK(g_olwgj);
STD_ROM_FN(g_olwgj);

struct BurnDriverD BurnDrvMDg_olwgj = {
	"g_olwgj", "g_olwg", NULL, "1900",
	"Olympic Winter Games - Lillehammer 94 (J) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_olwgjRomInfo, g_olwgjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Shadow of the Beast (UE) [!]
static struct BurnRomInfo g_sbeaRomDesc[] = {
	{ "g_sbea.bin", 0x100000, 0xbd385c27, BRF_PRG },
};

STD_ROM_PICK(g_sbea);
STD_ROM_FN(g_sbea);

struct BurnDriverD BurnDrvMDg_sbea = {
	"g_sbea", NULL, NULL, "1900",
	"Shadow of the Beast (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sbeaRomInfo, g_sbeaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Shadow of the Beast (J) [!]
static struct BurnRomInfo g_shbeajRomDesc[] = {
	{ "g_shbeaj.bin", 0x100000, 0x0cd09d31, BRF_PRG },
};

STD_ROM_PICK(g_shbeaj);
STD_ROM_FN(g_shbeaj);

struct BurnDriverD BurnDrvMDg_shbeaj = {
	"g_shbeaj", "g_sbea", NULL, "1900",
	"Shadow of the Beast (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_shbeajRomInfo, g_shbeajRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Show do Milhao (B) [!]
static struct BurnRomInfo g_showdRomDesc[] = {
	{ "g_showd.bin", 0x200000, 0x0a22df04, BRF_PRG },
};

STD_ROM_PICK(g_showd);
STD_ROM_FN(g_showd);

struct BurnDriverD BurnDrvMDg_showd = {
	"g_showd", NULL, NULL, "1900",
	"Show do Milhao (B) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_showdRomInfo, g_showdRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Show do Milhao Volume 2 (B) [!]
static struct BurnRomInfo g_showd2RomDesc[] = {
	{ "g_showd2.bin", 0x200000, 0x48ee66cb, BRF_PRG },
};

STD_ROM_PICK(g_showd2);
STD_ROM_FN(g_showd2);

struct BurnDriverD BurnDrvMDg_showd2 = {
	"g_showd2", NULL, NULL, "1900",
	"Show do Milhao Volume 2 (B) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_showd2RomInfo, g_showd2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Show do Milhao Volume 2 (B) [a1][!]
static struct BurnRomInfo g_shwd2aRomDesc[] = {
	{ "g_shwd2a.bin", 0x200000, 0xd8c9ac6d, BRF_PRG },
};

STD_ROM_PICK(g_shwd2a);
STD_ROM_FN(g_shwd2a);

struct BurnDriverD BurnDrvMDg_shwd2a = {
	"g_shwd2a", "g_showd2", NULL, "1900",
	"Show do Milhao Volume 2 (B) [a1][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_shwd2aRomInfo, g_shwd2aRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sorcerer's Kingdom (U) (REV01) [!]
static struct BurnRomInfo g_sorckdRomDesc[] = {
	{ "g_sorckd.bin", 0x100000, 0xbb1fc9ce, BRF_PRG },
};

STD_ROM_PICK(g_sorckd);
STD_ROM_FN(g_sorckd);

struct BurnDriverD BurnDrvMDg_sorckd = {
	"g_sorckd", NULL, NULL, "1900",
	"Sorcerer's Kingdom (U) (REV01) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sorckdRomInfo, g_sorckdRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sorcerer's Kingdom (U) (REV00)
static struct BurnRomInfo g_sork00RomDesc[] = {
	{ "g_sork00.bin", 0x100000, 0xcbe6c1ea, BRF_PRG },
};

STD_ROM_PICK(g_sork00);
STD_ROM_FN(g_sork00);

struct BurnDriverD BurnDrvMDg_sork00 = {
	"g_sork00", "g_sorckd", NULL, "1900",
	"Sorcerer's Kingdom (U) (REV00)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sork00RomInfo, g_sork00RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sorcer Kingdom (J) [c][!]
static struct BurnRomInfo g_sorkinRomDesc[] = {
	{ "g_sorkin.bin", 0x100000, 0x944135ca, BRF_PRG },
};

STD_ROM_PICK(g_sorkin);
STD_ROM_FN(g_sorkin);

struct BurnDriverD BurnDrvMDg_sorkin = {
	"g_sorkin", "g_sorckd", NULL, "1900",
	"Sorcer Kingdom (J) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sorkinRomInfo, g_sorkinRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Space Harrier II (UE) [!]
static struct BurnRomInfo g_sharRomDesc[] = {
	{ "g_shar.bin", 0x80000, 0xe5c9cbb0, BRF_PRG },
};

STD_ROM_PICK(g_shar);
STD_ROM_FN(g_shar);

struct BurnDriverD BurnDrvMDg_shar = {
	"g_shar", NULL, NULL, "1900",
	"Space Harrier II (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sharRomInfo, g_sharRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Space Harrier II (J) [!]
static struct BurnRomInfo g_shar2jRomDesc[] = {
	{ "g_shar2j.bin", 0x80000, 0xedc0fb28, BRF_PRG },
};

STD_ROM_PICK(g_shar2j);
STD_ROM_FN(g_shar2j);

struct BurnDriverD BurnDrvMDg_shar2j = {
	"g_shar2j", "g_shar", NULL, "1900",
	"Space Harrier II (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_shar2jRomInfo, g_shar2jRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sparkster (U) [!]
static struct BurnRomInfo g_sprkRomDesc[] = {
	{ "g_sprk.bin", 0x100000, 0x6bdb14ed, BRF_PRG },
};

STD_ROM_PICK(g_sprk);
STD_ROM_FN(g_sprk);

struct BurnDriverD BurnDrvMDg_sprk = {
	"g_sprk", NULL, NULL, "1900",
	"Sparkster (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sprkRomInfo, g_sprkRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sparkster (E) [!]
static struct BurnRomInfo g_sparkeRomDesc[] = {
	{ "g_sparke.bin", 0x100000, 0xd63e9f2d, BRF_PRG },
};

STD_ROM_PICK(g_sparke);
STD_ROM_FN(g_sparke);

struct BurnDriverD BurnDrvMDg_sparke = {
	"g_sparke", "g_sprk", NULL, "1900",
	"Sparkster (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sparkeRomInfo, g_sparkeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sparkster (J) [!]
static struct BurnRomInfo g_sparkjRomDesc[] = {
	{ "g_sparkj.bin", 0x100000, 0x914ec662, BRF_PRG },
};

STD_ROM_PICK(g_sparkj);
STD_ROM_FN(g_sparkj);

struct BurnDriverD BurnDrvMDg_sparkj = {
	"g_sparkj", "g_sprk", NULL, "1900",
	"Sparkster (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sparkjRomInfo, g_sparkjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Star Cruiser (J) [!]
static struct BurnRomInfo g_starcjRomDesc[] = {
	{ "g_starcj.bin", 0x80000, 0x2b75b52f, BRF_PRG },
};

STD_ROM_PICK(g_starcj);
STD_ROM_FN(g_starcj);

struct BurnDriverD BurnDrvMDg_starcj = {
	"g_starcj", NULL, NULL, "1900",
	"Star Cruiser (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_starcjRomInfo, g_starcjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Speed Ball 2 - Brutal Deluxe (U) [c][!]
static struct BurnRomInfo g_spb2RomDesc[] = {
	{ "g_spb2.bin", 0x80000, 0x9fc340a7, BRF_PRG },
};

STD_ROM_PICK(g_spb2);
STD_ROM_FN(g_spb2);

struct BurnDriverD BurnDrvMDg_spb2 = {
	"g_spb2", NULL, NULL, "1900",
	"Speed Ball 2 - Brutal Deluxe (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_spb2RomInfo, g_spb2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Speed Ball 2 - Brutal Deluxe (E) [!]
static struct BurnRomInfo g_sbal2eRomDesc[] = {
	{ "g_sbal2e.bin", 0x80000, 0x056a6e03, BRF_PRG },
};

STD_ROM_PICK(g_sbal2e);
STD_ROM_FN(g_sbal2e);

struct BurnDriverD BurnDrvMDg_sbal2e = {
	"g_sbal2e", "g_spb2", NULL, "1900",
	"Speed Ball 2 - Brutal Deluxe (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sbal2eRomInfo, g_sbal2eRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Steel Talons (UE) [!]
static struct BurnRomInfo g_stalonRomDesc[] = {
	{ "g_stalon.bin", 0x80000, 0x10e4ec63, BRF_PRG },
};

STD_ROM_PICK(g_stalon);
STD_ROM_FN(g_stalon);

struct BurnDriverD BurnDrvMDg_stalon = {
	"g_stalon", NULL, NULL, "1900",
	"Steel Talons (UE) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_stalonRomInfo, g_stalonRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Steel Talons (J) [!]
static struct BurnRomInfo g_staljRomDesc[] = {
	{ "g_stalj.bin", 0x80000, 0x04f388e6, BRF_PRG },
};

STD_ROM_PICK(g_stalj);
STD_ROM_FN(g_stalj);

struct BurnDriverD BurnDrvMDg_stalj = {
	"g_stalj", "g_stalon", NULL, "1900",
	"Steel Talons (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_staljRomInfo, g_staljRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Stormlord (U) [!]
static struct BurnRomInfo g_stolRomDesc[] = {
	{ "g_stol.bin", 0x80000, 0x39ab50a5, BRF_PRG },
};

STD_ROM_PICK(g_stol);
STD_ROM_FN(g_stol);

struct BurnDriverD BurnDrvMDg_stol = {
	"g_stol", NULL, NULL, "1900",
	"Stormlord (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_stolRomInfo, g_stolRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Stormlord (J) [!]
static struct BurnRomInfo g_slordjRomDesc[] = {
	{ "g_slordj.bin", 0x80000, 0x0b440fed, BRF_PRG },
};

STD_ROM_PICK(g_slordj);
STD_ROM_FN(g_slordj);

struct BurnDriverD BurnDrvMDg_slordj = {
	"g_slordj", "g_stol", NULL, "1900",
	"Stormlord (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_slordjRomInfo, g_slordjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Starflight (UE) (REV01) [!]
static struct BurnRomInfo g_sfliRomDesc[] = {
	{ "g_sfli.bin", 0x100000, 0x1217dbea, BRF_PRG },
};

STD_ROM_PICK(g_sfli);
STD_ROM_FN(g_sfli);

struct BurnDriverD BurnDrvMDg_sfli = {
	"g_sfli", NULL, NULL, "1900",
	"Starflight (UE) (REV01) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_sfliRomInfo, g_sfliRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Starflight (UE) (REV00) [!]
static struct BurnRomInfo g_strf00RomDesc[] = {
	{ "g_strf00.bin", 0x100000, 0xd550c928, BRF_PRG },
};

STD_ROM_PICK(g_strf00);
STD_ROM_FN(g_strf00);

struct BurnDriverD BurnDrvMDg_strf00 = {
	"g_strf00", "g_sfli", NULL, "1900",
	"Starflight (UE) (REV00) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_strf00RomInfo, g_strf00RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sunset Riders (U) [!]
static struct BurnRomInfo g_ssriRomDesc[] = {
	{ "g_ssri.bin", 0x80000, 0xac30c297, BRF_PRG },
};

STD_ROM_PICK(g_ssri);
STD_ROM_FN(g_ssri);

struct BurnDriverD BurnDrvMDg_ssri = {
	"g_ssri", NULL, NULL, "1900",
	"Sunset Riders (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ssriRomInfo, g_ssriRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sunset Riders (E) [!]
static struct BurnRomInfo g_ssrideRomDesc[] = {
	{ "g_ssride.bin", 0x80000, 0x0ff33054, BRF_PRG },
};

STD_ROM_PICK(g_ssride);
STD_ROM_FN(g_ssride);

struct BurnDriverD BurnDrvMDg_ssride = {
	"g_ssride", "g_ssri", NULL, "1900",
	"Sunset Riders (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ssrideRomInfo, g_ssrideRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Pachinko Canyon (J) [!]
static struct BurnRomInfo g_pachinRomDesc[] = {
	{ "g_pachin.bin", 0x100000, 0x9d137e7f, BRF_PRG },
};

STD_ROM_PICK(g_pachin);
STD_ROM_FN(g_pachin);

struct BurnDriverD BurnDrvMDg_pachin = {
	"g_pachin", NULL, NULL, "1900",
	"Pachinko Canyon (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pachinRomInfo, g_pachinRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Pete Sampras Tennis (UE) (REV00) (J-Cart) [c][!]
static struct BurnRomInfo g_pstRomDesc[] = {
	{ "g_pst.bin", 0x100000, 0x9ef5bbd1, BRF_PRG },
};

STD_ROM_PICK(g_pst);
STD_ROM_FN(g_pst);

struct BurnDriverD BurnDrvMDg_pst = {
	"g_pst", NULL, NULL, "1900",
	"Pete Sampras Tennis (UE) (REV00) (J-Cart) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pstRomInfo, g_pstRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Pete Sampras Tennis (E) (REV03) (J-Cart) [!]
static struct BurnRomInfo g_pste03RomDesc[] = {
	{ "g_pste03.bin", 0x100000, 0xaa8b19bc, BRF_PRG },
};

STD_ROM_PICK(g_pste03);
STD_ROM_FN(g_pste03);

struct BurnDriverD BurnDrvMDg_pste03 = {
	"g_pste03", "g_pst", NULL, "1900",
	"Pete Sampras Tennis (E) (REV03) (J-Cart) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pste03RomInfo, g_pste03RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Rastan Saga II (U) [!]
static struct BurnRomInfo g_rastRomDesc[] = {
	{ "g_rast.bin", 0x80000, 0xc7ee8965, BRF_PRG },
};

STD_ROM_PICK(g_rast);
STD_ROM_FN(g_rast);

struct BurnDriverD BurnDrvMDg_rast = {
	"g_rast", NULL, NULL, "1900",
	"Rastan Saga II (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_rastRomInfo, g_rastRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Rastan Saga II (J) [!]
static struct BurnRomInfo g_rastjRomDesc[] = {
	{ "g_rastj.bin", 0x80000, 0xebacfb5a, BRF_PRG },
};

STD_ROM_PICK(g_rastj);
STD_ROM_FN(g_rastj);

struct BurnDriverD BurnDrvMDg_rastj = {
	"g_rastj", "g_rast", NULL, "1900",
	"Rastan Saga II (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_rastjRomInfo, g_rastjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Road Blasters (U) [!]
static struct BurnRomInfo g_rblsRomDesc[] = {
	{ "g_rbls.bin", 0x80000, 0xec6cd5f0, BRF_PRG },
};

STD_ROM_PICK(g_rbls);
STD_ROM_FN(g_rbls);

struct BurnDriverD BurnDrvMDg_rbls = {
	"g_rbls", NULL, NULL, "1900",
	"Road Blasters (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_rblsRomInfo, g_rblsRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Road Blasters (J) [!]
static struct BurnRomInfo g_roadbjRomDesc[] = {
	{ "g_roadbj.bin", 0x80000, 0xa0015440, BRF_PRG },
};

STD_ROM_PICK(g_roadbj);
STD_ROM_FN(g_roadbj);

struct BurnDriverD BurnDrvMDg_roadbj = {
	"g_roadbj", "g_rbls", NULL, "1900",
	"Road Blasters (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_roadbjRomInfo, g_roadbjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Rock n' Roll Racing (U) [!]
static struct BurnRomInfo g_rrrRomDesc[] = {
	{ "g_rrr.bin", 0x100000, 0x6abab577, BRF_PRG },
};

STD_ROM_PICK(g_rrr);
STD_ROM_FN(g_rrr);

struct BurnDriverD BurnDrvMDg_rrr = {
	"g_rrr", NULL, NULL, "1900",
	"Rock n' Roll Racing (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_rrrRomInfo, g_rrrRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Rock n' Roll Racing (E) [!]
static struct BurnRomInfo g_rnrreRomDesc[] = {
	{ "g_rnrre.bin", 0x100000, 0xbc5a0562, BRF_PRG },
};

STD_ROM_PICK(g_rnrre);
STD_ROM_FN(g_rnrre);

struct BurnDriverD BurnDrvMDg_rnrre = {
	"g_rnrre", "g_rrr", NULL, "1900",
	"Rock n' Roll Racing (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_rnrreRomInfo, g_rnrreRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Rocket Knight Adventures (U) [!]
static struct BurnRomInfo g_rkniRomDesc[] = {
	{ "g_rkni.bin", 0x100000, 0xa6efec47, BRF_PRG },
};

STD_ROM_PICK(g_rkni);
STD_ROM_FN(g_rkni);

struct BurnDriverD BurnDrvMDg_rkni = {
	"g_rkni", NULL, NULL, "1900",
	"Rocket Knight Adventures (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_rkniRomInfo, g_rkniRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Rocket Knight Adventures (E) [!]
static struct BurnRomInfo g_rkadveRomDesc[] = {
	{ "g_rkadve.bin", 0x100000, 0x8eedfd51, BRF_PRG },
};

STD_ROM_PICK(g_rkadve);
STD_ROM_FN(g_rkadve);

struct BurnDriverD BurnDrvMDg_rkadve = {
	"g_rkadve", "g_rkni", NULL, "1900",
	"Rocket Knight Adventures (E) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_rkadveRomInfo, g_rkadveRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Rocket Knight Adventures (J) [!]
static struct BurnRomInfo g_rkadvjRomDesc[] = {
	{ "g_rkadvj.bin", 0x100000, 0xd1c8c1c5, BRF_PRG },
};

STD_ROM_PICK(g_rkadvj);
STD_ROM_FN(g_rkadvj);

struct BurnDriverD BurnDrvMDg_rkadvj = {
	"g_rkadvj", "g_rkni", NULL, "1900",
	"Rocket Knight Adventures (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_rkadvjRomInfo, g_rkadvjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Rolling Thunder 2 (U) [c][!]
static struct BurnRomInfo g_rth2RomDesc[] = {
	{ "g_rth2.bin", 0x100000, 0x3ace429b, BRF_PRG },
};

STD_ROM_PICK(g_rth2);
STD_ROM_FN(g_rth2);

struct BurnDriverD BurnDrvMDg_rth2 = {
	"g_rth2", NULL, NULL, "1900",
	"Rolling Thunder 2 (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_rth2RomInfo, g_rth2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Rolling Thunder 2 (E) [c][!]
static struct BurnRomInfo g_rthn2eRomDesc[] = {
	{ "g_rthn2e.bin", 0x100000, 0xc440f292, BRF_PRG },
};

STD_ROM_PICK(g_rthn2e);
STD_ROM_FN(g_rthn2e);

struct BurnDriverD BurnDrvMDg_rthn2e = {
	"g_rthn2e", "g_rth2", NULL, "1900",
	"Rolling Thunder 2 (E) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_rthn2eRomInfo, g_rthn2eRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Rolling Thunder 2 (J) [c][!]
static struct BurnRomInfo g_rthn2jRomDesc[] = {
	{ "g_rthn2j.bin", 0x100000, 0x965b2628, BRF_PRG },
};

STD_ROM_PICK(g_rthn2j);
STD_ROM_FN(g_rthn2j);

struct BurnDriverD BurnDrvMDg_rthn2j = {
	"g_rthn2j", "g_rth2", NULL, "1900",
	"Rolling Thunder 2 (J) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_rthn2jRomInfo, g_rthn2jRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Rolling Thunder 3 (U) [c][!]
static struct BurnRomInfo g_rth3RomDesc[] = {
	{ "g_rth3.bin", 0x180000, 0x64fb13aa, BRF_PRG },
};

STD_ROM_PICK(g_rth3);
STD_ROM_FN(g_rth3);

struct BurnDriverD BurnDrvMDg_rth3 = {
	"g_rth3", NULL, NULL, "1900",
	"Rolling Thunder 3 (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_rth3RomInfo, g_rth3RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Telebradesco Residencia (B) [!]
static struct BurnRomInfo g_telerbRomDesc[] = {
	{ "g_telerb.bin", 0x40000, 0x1db99045, BRF_PRG },
};

STD_ROM_PICK(g_telerb);
STD_ROM_FN(g_telerb);

struct BurnDriverD BurnDrvMDg_telerb = {
	"g_telerb", NULL, NULL, "1900",
	"Telebradesco Residencia (B) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_telerbRomInfo, g_telerbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Thunder Fox (U) [c][!]
static struct BurnRomInfo g_tfoxRomDesc[] = {
	{ "g_tfox.bin", 0x100000, 0x5463f50f, BRF_PRG },
};

STD_ROM_PICK(g_tfox);
STD_ROM_FN(g_tfox);

struct BurnDriverD BurnDrvMDg_tfox = {
	"g_tfox", NULL, NULL, "1900",
	"Thunder Fox (U) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tfoxRomInfo, g_tfoxRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Thunder Fox (J) [c][!]
static struct BurnRomInfo g_tfoxjRomDesc[] = {
	{ "g_tfoxj.bin", 0x100000, 0xeca6cffa, BRF_PRG },
};

STD_ROM_PICK(g_tfoxj);
STD_ROM_FN(g_tfoxj);

struct BurnDriverD BurnDrvMDg_tfoxj = {
	"g_tfoxj", "g_tfox", NULL, "1900",
	"Thunder Fox (J) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tfoxjRomInfo, g_tfoxjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Todd's Adventures in Slime World (U) [!]
static struct BurnRomInfo g_toddRomDesc[] = {
	{ "g_todd.bin", 0x80000, 0x652e8b7d, BRF_PRG },
};

STD_ROM_PICK(g_todd);
STD_ROM_FN(g_todd);

struct BurnDriverD BurnDrvMDg_todd = {
	"g_todd", NULL, NULL, "1900",
	"Todd's Adventures in Slime World (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_toddRomInfo, g_toddRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Todd's Adventures in Slime World (J) [!]
static struct BurnRomInfo g_toddjRomDesc[] = {
	{ "g_toddj.bin", 0x80000, 0x7ff5529f, BRF_PRG },
};

STD_ROM_PICK(g_toddj);
STD_ROM_FN(g_toddj);

struct BurnDriverD BurnDrvMDg_toddj = {
	"g_toddj", "g_todd", NULL, "1900",
	"Todd's Adventures in Slime World (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_toddjRomInfo, g_toddjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Tom and Jerry - Frantic Antics (U) (1994) [!]
static struct BurnRomInfo g_tomjRomDesc[] = {
	{ "g_tomj.bin", 0x100000, 0x3044460c, BRF_PRG },
};

STD_ROM_PICK(g_tomj);
STD_ROM_FN(g_tomj);

struct BurnDriverD BurnDrvMDg_tomj = {
	"g_tomj", NULL, NULL, "1900",
	"Tom and Jerry - Frantic Antics (U) (1994) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tomjRomInfo, g_tomjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Tom and Jerry - Frantic Antics (U) (1993) [!]
static struct BurnRomInfo g_tajuaRomDesc[] = {
	{ "g_tajua.bin", 0x100000, 0xb9992e1c, BRF_PRG },
};

STD_ROM_PICK(g_tajua);
STD_ROM_FN(g_tajua);

struct BurnDriverD BurnDrvMDg_tajua = {
	"g_tajua", "g_tomj", NULL, "1900",
	"Tom and Jerry - Frantic Antics (U) (1993) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_tajuaRomInfo, g_tajuaRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Baby's Day Out (Beta 2)
static struct BurnRomInfo g_babyb2RomDesc[] = {
	{ "g_babyb2.bin", 0x100000, 0xb2e7cc49, BRF_PRG },
};

STD_ROM_PICK(g_babyb2);
STD_ROM_FN(g_babyb2);

struct BurnDriver BurnDrvMDg_babyb2 = {
	"g_babyb2", NULL, NULL, "1900",
	"Baby's Day Out (Beta 2)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_babyb2RomInfo, g_babyb2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Baby's Day Out (Beta 1)
static struct BurnRomInfo g_babyb1RomDesc[] = {
	{ "g_babyb1.bin", 0x100000, 0x459b891c, BRF_PRG },
};

STD_ROM_PICK(g_babyb1);
STD_ROM_FN(g_babyb1);

struct BurnDriverD BurnDrvMDg_babyb1 = {
	"g_babyb1", "g_babyb2", NULL, "1900",
	"Baby's Day Out (Beta 1)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_babyb1RomInfo, g_babyb1RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Barbie Vacation Adventure (Beta)
static struct BurnRomInfo g_barbvbRomDesc[] = {
	{ "g_barbvb.bin", 0x100000, 0x10e0ba69, BRF_PRG },
};

STD_ROM_PICK(g_barbvb);
STD_ROM_FN(g_barbvb);

struct BurnDriverD BurnDrvMDg_barbvb = {
	"g_barbvb", NULL, NULL, "1900",
	"Barbie Vacation Adventure (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_barbvbRomInfo, g_barbvbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Batman (U) [!]
static struct BurnRomInfo g_batRomDesc[] = {
	{ "g_bat.bin", 0x80000, 0x017410ae, BRF_PRG },
};

STD_ROM_PICK(g_bat);
STD_ROM_FN(g_bat);

struct BurnDriver BurnDrvMDg_bat = {
	"g_bat", NULL, NULL, "1900",
	"Batman (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_batRomInfo, g_batRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Batman (J) [!]
static struct BurnRomInfo g_batmnjRomDesc[] = {
	{ "g_batmnj.bin", 0x80000, 0xd7b4febf, BRF_PRG },
};

STD_ROM_PICK(g_batmnj);
STD_ROM_FN(g_batmnj);

struct BurnDriver BurnDrvMDg_batmnj = {
	"g_batmnj", "g_bat", NULL, "1900",
	"Batman (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_batmnjRomInfo, g_batmnjRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Batman (E)
static struct BurnRomInfo g_batmeRomDesc[] = {
	{ "g_batme.bin", 0x80000, 0x61c60c35, BRF_PRG },
};

STD_ROM_PICK(g_batme);
STD_ROM_FN(g_batme);

struct BurnDriverD BurnDrvMDg_batme = {
	"g_batme", "g_bat", NULL, "1900",
	"Batman (E)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_batmeRomInfo, g_batmeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Beyond Zero Tolerance (Beta)
static struct BurnRomInfo g_bzerotRomDesc[] = {
	{ "g_bzerot.bin", 0x200000, 0xc61ed2ed, BRF_PRG },
};

STD_ROM_PICK(g_bzerot);
STD_ROM_FN(g_bzerot);

struct BurnDriver BurnDrvMDg_bzerot = {
	"g_bzerot", NULL, NULL, "1900",
	"Beyond Zero Tolerance (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_bzerotRomInfo, g_bzerotRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Blockbuster World Video Game Championship II (U)
static struct BurnRomInfo g_blocb2RomDesc[] = {
	{ "g_blocb2.bin", 0x400000, 0x4385e366, BRF_PRG },
};

STD_ROM_PICK(g_blocb2);
STD_ROM_FN(g_blocb2);

struct BurnDriverD BurnDrvMDg_blocb2 = {
	"g_blocb2", NULL, NULL, "1900",
	"Blockbuster World Video Game Championship II (U)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_blocb2RomInfo, g_blocb2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Chuck's Excellent Art Tool Animator (U)
static struct BurnRomInfo g_chuckRomDesc[] = {
	{ "g_chuck.bin", 0x60000, 0x6360ee58, BRF_PRG },
};

STD_ROM_PICK(g_chuck);
STD_ROM_FN(g_chuck);

struct BurnDriverD BurnDrvMDg_chuck = {
	"g_chuck", NULL, NULL, "1900",
	"Chuck's Excellent Art Tool Animator (U)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_chuckRomInfo, g_chuckRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Combat Aces (Beta)
static struct BurnRomInfo g_comacbRomDesc[] = {
	{ "g_comacb.bin", 0x80000, 0x84560d5a, BRF_PRG },
};

STD_ROM_PICK(g_comacb);
STD_ROM_FN(g_comacb);

struct BurnDriverD BurnDrvMDg_comacb = {
	"g_comacb", NULL, NULL, "1900",
	"Combat Aces (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_comacbRomInfo, g_comacbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Congo - The Game (Beta)
static struct BurnRomInfo g_congobRomDesc[] = {
	{ "g_congob.bin", 0xf7d36, 0x13746716, BRF_PRG },
};

STD_ROM_PICK(g_congob);
STD_ROM_FN(g_congob);

struct BurnDriverD BurnDrvMDg_congob = {
	"g_congob", NULL, NULL, "1900",
	"Congo - The Game (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_congobRomInfo, g_congobRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Davis Cup World Tour Tennis 2 (Beta 1)
static struct BurnRomInfo g_dwctb1RomDesc[] = {
	{ "g_dwctb1.bin", 0x200000, 0x6f4183c0, BRF_PRG },
};

STD_ROM_PICK(g_dwctb1);
STD_ROM_FN(g_dwctb1);

struct BurnDriverD BurnDrvMDg_dwctb1 = {
	"g_dwctb1", NULL, NULL, "1900",
	"Davis Cup World Tour Tennis 2 (Beta 1)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_dwctb1RomInfo, g_dwctb1RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Davis Cup World Tour Tennis 2 (Beta 2)
static struct BurnRomInfo g_dwctb2RomDesc[] = {
	{ "g_dwctb2.bin", 0x200000, 0x76f2bed8, BRF_PRG },
};

STD_ROM_PICK(g_dwctb2);
STD_ROM_FN(g_dwctb2);

struct BurnDriverD BurnDrvMDg_dwctb2 = {
	"g_dwctb2", NULL, NULL, "1900",
	"Davis Cup World Tour Tennis 2 (Beta 2)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_dwctb2RomInfo, g_dwctb2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Davis Cup World Tour Tennis (Beta)
static struct BurnRomInfo g_dwctbRomDesc[] = {
	{ "g_dwctb.bin", 0x100000, 0x7c6b0646, BRF_PRG },
};

STD_ROM_PICK(g_dwctb);
STD_ROM_FN(g_dwctb);

struct BurnDriverD BurnDrvMDg_dwctb = {
	"g_dwctb", NULL, NULL, "1900",
	"Davis Cup World Tour Tennis (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_dwctbRomInfo, g_dwctbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Demolition Man (Beta)
static struct BurnRomInfo g_demombRomDesc[] = {
	{ "g_demomb.bin", 0x200000, 0x57ffad7a, BRF_PRG },
};

STD_ROM_PICK(g_demomb);
STD_ROM_FN(g_demomb);

struct BurnDriverD BurnDrvMDg_demomb = {
	"g_demomb", NULL, NULL, "1900",
	"Demolition Man (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_demombRomInfo, g_demombRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Dominus (Beta)
static struct BurnRomInfo g_dominuRomDesc[] = {
	{ "g_dominu.bin", 0xc0000, 0x30006ebd, BRF_PRG },
};

STD_ROM_PICK(g_dominu);
STD_ROM_FN(g_dominu);

struct BurnDriverD BurnDrvMDg_dominu = {
	"g_dominu", NULL, NULL, "1900",
	"Dominus (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_dominuRomInfo, g_dominuRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Dune - The Battle for Arrakis (U) [!]
static struct BurnRomInfo g_duneRomDesc[] = {
	{ "g_dune.bin", 0x100000, 0x4dea40ba, BRF_PRG },
};

STD_ROM_PICK(g_dune);
STD_ROM_FN(g_dune);

struct BurnDriverD BurnDrvMDg_dune = {
	"g_dune", NULL, NULL, "1900",
	"Dune - The Battle for Arrakis (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_duneRomInfo, g_duneRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Dune - Der Kampf um den Wuestenplaneten (G)
static struct BurnRomInfo g_dunegRomDesc[] = {
	{ "g_duneg.bin", 0x100000, 0x39790728, BRF_PRG },
};

STD_ROM_PICK(g_duneg);
STD_ROM_FN(g_duneg);

struct BurnDriverD BurnDrvMDg_duneg = {
	"g_duneg", "g_dune", NULL, "1900",
	"Dune - Der Kampf um den Wuestenplaneten (G)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_dunegRomInfo, g_dunegRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Dune - The Battle for Arrakis (E)
static struct BurnRomInfo g_duneeRomDesc[] = {
	{ "g_dunee.bin", 0x100000, 0xb58ae71d, BRF_PRG },
};

STD_ROM_PICK(g_dunee);
STD_ROM_FN(g_dunee);

struct BurnDriverD BurnDrvMDg_dunee = {
	"g_dunee", "g_dune", NULL, "1900",
	"Dune - The Battle for Arrakis (E)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_duneeRomInfo, g_duneeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Dyna Brothers (J) (REV01)
static struct BurnRomInfo g_dynabrRomDesc[] = {
	{ "g_dynabr.bin", 0x100000, 0x360c1b20, BRF_PRG },
};

STD_ROM_PICK(g_dynabr);
STD_ROM_FN(g_dynabr);

struct BurnDriverD BurnDrvMDg_dynabr = {
	"g_dynabr", NULL, NULL, "1900",
	"Dyna Brothers (J) (REV01)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_dynabrRomInfo, g_dynabrRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Ferrari Grand Prix Challenge (U) [!]
static struct BurnRomInfo g_frgpRomDesc[] = {
	{ "g_frgp.bin", 0x100000, 0xf73f6bec, BRF_PRG },
};

STD_ROM_PICK(g_frgp);
STD_ROM_FN(g_frgp);

struct BurnDriverD BurnDrvMDg_frgp = {
	"g_frgp", NULL, NULL, "1900",
	"Ferrari Grand Prix Challenge (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_frgpRomInfo, g_frgpRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Ferrari Grand Prix Challenge (Beta)
static struct BurnRomInfo g_fergpbRomDesc[] = {
	{ "g_fergpb.bin", 0x100000, 0xd48d02d4, BRF_PRG },
};

STD_ROM_PICK(g_fergpb);
STD_ROM_FN(g_fergpb);

struct BurnDriverD BurnDrvMDg_fergpb = {
	"g_fergpb", "g_frgp", NULL, "1900",
	"Ferrari Grand Prix Challenge (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_fergpbRomInfo, g_fergpbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// It Came From The Desert (Beta)
static struct BurnRomInfo g_icftdRomDesc[] = {
	{ "g_icftd.bin", 0x80000, 0x25afb4f7, BRF_PRG },
};

STD_ROM_PICK(g_icftd);
STD_ROM_FN(g_icftd);

struct BurnDriverD BurnDrvMDg_icftd = {
	"g_icftd", NULL, NULL, "1900",
	"It Came From The Desert (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_icftdRomInfo, g_icftdRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Might and Magic III - Isles of Terra (Beta)
static struct BurnRomInfo g_mima3bRomDesc[] = {
	{ "g_mima3b.bin", 0x200000, 0x6ef7104a, BRF_PRG },
};

STD_ROM_PICK(g_mima3b);
STD_ROM_FN(g_mima3b);

struct BurnDriverD BurnDrvMDg_mima3b = {
	"g_mima3b", NULL, NULL, "1900",
	"Might and Magic III - Isles of Terra (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_mima3bRomInfo, g_mima3bRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Ninja Gaiden (Beta)
static struct BurnRomInfo g_ngaidbRomDesc[] = {
	{ "g_ngaidb.bin", 0x100000, 0x0d7f74ba, BRF_PRG },
};

STD_ROM_PICK(g_ngaidb);
STD_ROM_FN(g_ngaidb);

struct BurnDriverD BurnDrvMDg_ngaidb = {
	"g_ngaidb", NULL, NULL, "1900",
	"Ninja Gaiden (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_ngaidbRomInfo, g_ngaidbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Pebble Beach Golf Links (U) [!]
static struct BurnRomInfo g_pbglRomDesc[] = {
	{ "g_pbgl.bin", 0x200000, 0x95823c43, BRF_PRG },
};

STD_ROM_PICK(g_pbgl);
STD_ROM_FN(g_pbgl);

struct BurnDriverD BurnDrvMDg_pbgl = {
	"g_pbgl", NULL, NULL, "1900",
	"Pebble Beach Golf Links (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pbglRomInfo, g_pbglRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Pebble Beach Golf Links (E)
static struct BurnRomInfo g_pblbeeRomDesc[] = {
	{ "g_pblbee.bin", 0x200000, 0x6cfc7297, BRF_PRG },
};

STD_ROM_PICK(g_pblbee);
STD_ROM_FN(g_pblbee);

struct BurnDriverD BurnDrvMDg_pblbee = {
	"g_pblbee", "g_pbgl", NULL, "1900",
	"Pebble Beach Golf Links (E)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pblbeeRomInfo, g_pblbeeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Pebble Beach no Hatou (J) [!]
static struct BurnRomInfo g_pblbejRomDesc[] = {
	{ "g_pblbej.bin", 0x180000, 0x96ed2e5d, BRF_PRG },
};

STD_ROM_PICK(g_pblbej);
STD_ROM_FN(g_pblbej);

struct BurnDriverD BurnDrvMDg_pblbej = {
	"g_pblbej", "g_pbgl", NULL, "1900",
	"Pebble Beach no Hatou (J) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pblbejRomInfo, g_pblbejRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Pink Goes to Hollywood (U) [!]
static struct BurnRomInfo g_pinkRomDesc[] = {
	{ "g_pink.bin", 0x100000, 0xb5804771, BRF_PRG },
};

STD_ROM_PICK(g_pink);
STD_ROM_FN(g_pink);

struct BurnDriverD BurnDrvMDg_pink = {
	"g_pink", NULL, NULL, "1900",
	"Pink Goes to Hollywood (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pinkRomInfo, g_pinkRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Pink Goes to Hollywood (Beta)
static struct BurnRomInfo g_pinkbRomDesc[] = {
	{ "g_pinkb.bin", 0x100000, 0x56087cff, BRF_PRG },
};

STD_ROM_PICK(g_pinkb);
STD_ROM_FN(g_pinkb);

struct BurnDriverD BurnDrvMDg_pinkb = {
	"g_pinkb", "g_pink", NULL, "1900",
	"Pink Goes to Hollywood (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_pinkbRomInfo, g_pinkbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Radical Rex (U) [!]
static struct BurnRomInfo g_radrRomDesc[] = {
	{ "g_radr.bin", 0x100000, 0x2e6eec7e, BRF_PRG },
};

STD_ROM_PICK(g_radr);
STD_ROM_FN(g_radr);

struct BurnDriverD BurnDrvMDg_radr = {
	"g_radr", NULL, NULL, "1900",
	"Radical Rex (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_radrRomInfo, g_radrRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Radical Rex (E)
static struct BurnRomInfo g_rrexeRomDesc[] = {
	{ "g_rrexe.bin", 0x100000, 0xd02d3282, BRF_PRG },
};

STD_ROM_PICK(g_rrexe);
STD_ROM_FN(g_rrexe);

struct BurnDriverD BurnDrvMDg_rrexe = {
	"g_rrexe", "g_radr", NULL, "1900",
	"Radical Rex (E)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_rrexeRomInfo, g_rrexeRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Scrabble (Beta)
static struct BurnRomInfo g_scrabbRomDesc[] = {
	{ "g_scrabb.bin", 0x100000, 0x360b2610, BRF_PRG },
};

STD_ROM_PICK(g_scrabb);
STD_ROM_FN(g_scrabb);

struct BurnDriverD BurnDrvMDg_scrabb = {
	"g_scrabb", NULL, NULL, "1900",
	"Scrabble (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_scrabbRomInfo, g_scrabbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Shanghai 2 - Dragon's Eye (U) [!]
static struct BurnRomInfo g_s2deRomDesc[] = {
	{ "g_s2de.bin", 0x100000, 0xebe9e840, BRF_PRG },
};

STD_ROM_PICK(g_s2de);
STD_ROM_FN(g_s2de);

struct BurnDriverD BurnDrvMDg_s2de = {
	"g_s2de", NULL, NULL, "1900",
	"Shanghai 2 - Dragon's Eye (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_s2deRomInfo, g_s2deRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Shanghai 2 - Dragon's Eye (Beta)
static struct BurnRomInfo g_shan2bRomDesc[] = {
	{ "g_shan2b.bin", 0x100000, 0x5e33867b, BRF_PRG },
};

STD_ROM_PICK(g_shan2b);
STD_ROM_FN(g_shan2b);

struct BurnDriverD BurnDrvMDg_shan2b = {
	"g_shan2b", "g_s2de", NULL, "1900",
	"Shanghai 2 - Dragon's Eye (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_shan2bRomInfo, g_shan2bRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Sonic and Crackers (Beta)
static struct BurnRomInfo g_soncrkRomDesc[] = {
	{ "g_soncrk.bin", 0x100000, 0x7fada88d, BRF_PRG },
};

STD_ROM_PICK(g_soncrk);
STD_ROM_FN(g_soncrk);

struct BurnDriver BurnDrvMDg_soncrk = {
	"g_soncrk", NULL, NULL, "1900",
	"Sonic and Crackers (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_GAME_WORKING | BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_soncrkRomInfo, g_soncrkRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Wacky Races (Beta)
static struct BurnRomInfo g_wacracRomDesc[] = {
	{ "g_wacrac.bin", 0x200000, 0x1b173f09, BRF_PRG },
};

STD_ROM_PICK(g_wacrac);
STD_ROM_FN(g_wacrac);

struct BurnDriverD BurnDrvMDg_wacrac = {
	"g_wacrac", NULL, NULL, "1900",
	"Wacky Races (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wacracRomInfo, g_wacracRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Waterworld (Beta)
static struct BurnRomInfo g_watrbRomDesc[] = {
	{ "g_watrb.bin", 0x200000, 0x51c80498, BRF_PRG },
};

STD_ROM_PICK(g_watrb);
STD_ROM_FN(g_watrb);

struct BurnDriverD BurnDrvMDg_watrb = {
	"g_watrb", NULL, NULL, "1900",
	"Waterworld (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_watrbRomInfo, g_watrbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Williams Arcade's Greatest Hits (U) [!]
static struct BurnRomInfo g_waghRomDesc[] = {
	{ "g_wagh.bin", 0x80000, 0xd68e9c00, BRF_PRG },
};

STD_ROM_PICK(g_wagh);
STD_ROM_FN(g_wagh);

struct BurnDriverD BurnDrvMDg_wagh = {
	"g_wagh", NULL, NULL, "1900",
	"Williams Arcade's Greatest Hits (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_waghRomInfo, g_waghRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Williams Arcade's Greatest Hits (E)
static struct BurnRomInfo g_wagheRomDesc[] = {
	{ "g_waghe.bin", 0x80000, 0xc0dce0e5, BRF_PRG },
};

STD_ROM_PICK(g_waghe);
STD_ROM_FN(g_waghe);

struct BurnDriverD BurnDrvMDg_waghe = {
	"g_waghe", "g_wagh", NULL, "1900",
	"Williams Arcade's Greatest Hits (E)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wagheRomInfo, g_wagheRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// World Championship Soccer II (U) [!]
static struct BurnRomInfo g_wcs2RomDesc[] = {
	{ "g_wcs2.bin", 0x100000, 0xc1dd1c8e, BRF_PRG },
};

STD_ROM_PICK(g_wcs2);
STD_ROM_FN(g_wcs2);

struct BurnDriverD BurnDrvMDg_wcs2 = {
	"g_wcs2", NULL, NULL, "1900",
	"World Championship Soccer II (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wcs2RomInfo, g_wcs2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// World Championship Soccer II (Beta)
static struct BurnRomInfo g_wcs2bRomDesc[] = {
	{ "g_wcs2b.bin", 0x100000, 0xc1e21c1a, BRF_PRG },
};

STD_ROM_PICK(g_wcs2b);
STD_ROM_FN(g_wcs2b);

struct BurnDriverD BurnDrvMDg_wcs2b = {
	"g_wcs2b", "g_wcs2", NULL, "1900",
	"World Championship Soccer II (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_wcs2bRomInfo, g_wcs2bRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Young Indiana Jones - Instrument of Chaos (U) [!]
static struct BurnRomInfo g_yidyRomDesc[] = {
	{ "g_yidy.bin", 0x100000, 0x4e384ef0, BRF_PRG },
};

STD_ROM_PICK(g_yidy);
STD_ROM_FN(g_yidy);

struct BurnDriverD BurnDrvMDg_yidy = {
	"g_yidy", NULL, NULL, "1900",
	"Young Indiana Jones - Instrument of Chaos (U) [!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_yidyRomInfo, g_yidyRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Young Indiana Jones - Instrument of Chaos (Beta)
static struct BurnRomInfo g_yindybRomDesc[] = {
	{ "g_yindyb.bin", 0x180000, 0xad6c2050, BRF_PRG },
};

STD_ROM_PICK(g_yindyb);
STD_ROM_FN(g_yindyb);

struct BurnDriverD BurnDrvMDg_yindyb = {
	"g_yindyb", "g_yidy", NULL, "1900",
	"Young Indiana Jones - Instrument of Chaos (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_yindybRomInfo, g_yindybRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Young Indiana Jones Chronicles (Beta)
static struct BurnRomInfo g_yindcbRomDesc[] = {
	{ "g_yindcb.bin", 0x80000, 0x44f6be35, BRF_PRG },
};

STD_ROM_PICK(g_yindcb);
STD_ROM_FN(g_yindcb);

struct BurnDriverD BurnDrvMDg_yindcb = {
	"g_yindcb", NULL, NULL, "1900",
	"Young Indiana Jones Chronicles (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_yindcbRomInfo, g_yindcbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Zombie High (U) (Prototype)
static struct BurnRomInfo g_zombhbRomDesc[] = {
	{ "g_zombhb.bin", 0xfa0ef, 0x7bea6194, BRF_PRG },
};

STD_ROM_PICK(g_zombhb);
STD_ROM_FN(g_zombhb);

struct BurnDriverD BurnDrvMDg_zombhb = {
	"g_zombhb", NULL, NULL, "1900",
	"Zombie High (U) (Prototype)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_zombhbRomInfo, g_zombhbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Brian Lara Cricket (E) [c][!]
static struct BurnRomInfo g_blcrRomDesc[] = {
	{ "g_blcr.bin", 0x100000, 0x408cf5c3, BRF_PRG },
};

STD_ROM_PICK(g_blcr);
STD_ROM_FN(g_blcr);

struct BurnDriverD BurnDrvMDg_blcr = {
	"g_blcr", NULL, NULL, "1900",
	"Brian Lara Cricket (E) [c][!]\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_blcrRomInfo, g_blcrRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Brian Lara Cricket (Beta)
static struct BurnRomInfo g_blcrbRomDesc[] = {
	{ "g_blcrb.bin", 0x100000, 0x90f5c2b7, BRF_PRG },
};

STD_ROM_PICK(g_blcrb);
STD_ROM_FN(g_blcrb);

struct BurnDriverD BurnDrvMDg_blcrb = {
	"g_blcrb", "g_blcr", NULL, "1900",
	"Brian Lara Cricket (Beta)\0", NULL, "Unsorted", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY | BDF_CLONE, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_blcrbRomInfo, g_blcrbRomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

// Prince of Persia 2 (Prototype)
static struct BurnRomInfo g_princ2RomDesc[] = {
	{ "g_princ2.bin", 0x200000, 0x3ab44d46, BRF_PRG },
};

STD_ROM_PICK(g_princ2);
STD_ROM_FN(g_princ2);

struct BurnDriverD BurnDrvMDg_princ2 = {
	"g_princ2", NULL, NULL, "1900",
	"Prince of Persia 2 (Prototype)\0", NULL, "Psygnosis", "Megadrive",
	NULL, NULL, NULL, NULL,
	BDF_16BIT_ONLY, 2, HARDWARE_SEGA_MEGADRIVE, //GBF_MISC, 0,
	NULL, g_princ2RomInfo, g_princ2RomName, MegadriveInputInfo, MegadriveDIPInfo,
	MegadriveInit, MegadriveExit, MegadriveFrame, NULL, MegadriveScan,
	0, NULL, NULL, NULL, &bMegadriveRecalcPalette, 320, 224, 4, 3
};

