 # MIPS Make68K - V0.30 - Copyright 2005, Manuel Geran (bdiamond@free.fr)
 #   based on Make68K - Copyright 1998, Mike Coates (mame@btinternet.com)
 #                                    & Darren Olafson (deo@mail.island.net)
 #   OpenDingux adaptation - Copyright 2012-2013 Dmitry Smagin

		 .set mips32

		 .set noreorder
		 .set nobopt
		 .globl M68000_RUN
		 .globl M68000_RESET
		 .globl M68000_regs
		 .globl M68000_OPCODETABLE
		 .globl a68k_memory_intf

# most macros are fixed by Nebuleon
# m_j macro is taken from here http://gcc.gnu.org/ml/gcc-patches/2002-12/msg00603.html

.macro m_seb dest,src
    sll   \dest,\src,24
    sra   \dest,\dest,24
.endm

.macro m_seh dest,src
    sll   \dest,\src,16
    sra   \dest,\dest,16
.endm

.macro m_ror dest,src,shift
    sw    $t0,-4($sp)
    srl   $t0,\dest,\shift
    sll   \dest,32-\shift
    or    \dest,$t0
    lw    $t0,-4($sp)
.endm

.macro m_rorv dest,src,shift
    sw    $t1,-8($sp)
    sw    $t0,-4($sp)
    srlv  $t0,\dest,\shift
    li    $t1,32
    sub   $t1,\shift
    sllv  \dest,\dest,$t1
    or    \dest,$t0
    lw    $t1,-8($sp)
    lw    $t0,-4($sp)
.endm

.macro m_j dest
    .set  noat
    la    $at,\dest
    jr    $at
    .set  at
.endm

.macro m_jal address
    .set  noat
    la    $at,\address
    jalr  $at
    .set  at
.endm


 # Vars Mame declares / needs access to

		 .extern m68k_ICount,4
		 .extern OP_ROM,4
		 .extern OP_RAM,4
		 .extern mem_amask,4
		 .extern mame_debug,4
		 .extern illegal_op,4
		 .extern illegal_pc,4
		 .extern opcode_entry,4
		 .extern cur_mrhard,4



		 .text
		 .align 4

	 .ent   M68000_RESET
M68000_RESET:
		 .frame $sp, 0, $ra
		 jr    $ra
		 nop			 # Delay slot

	 .end   M68000_RESET


	 .ent   M68000_RUN
M68000_RUN:
		 .frame $sp, 112, $ra
		 addiu $sp,$sp,-112

		 sw    $gp,0x44($sp)
		 sw    $s0,0x48($sp)
		 sw    $s1,0x4C($sp)
		 sw    $s2,0x50($sp)
		 sw    $s3,0x54($sp)
		 sw    $s4,0x58($sp)
		 sw    $s5,0x5C($sp)
		 sw    $s6,0x60($sp)
		 sw    $s7,0x64($sp)
		 sw    $fp,0x68($sp)
		 sw    $ra,0x6C($sp)
		 .cpload $gp
		 la    $s5,M68000_regs
		 lbu   $t0,0x54($s5)
		 xori  $t1,$t0,0x80
		 lw    $t7,m68k_ICount
		 beq   $t1,$0,MC68Kexit
		 movz  $t7,$0,$t1		 # Delay slot
		 la    $fp,M68000_OPCODETABLE
		 lw    $s6,OP_ROM
		 lw    $s7,0x50($s5)
		 lw    $v0,0x48($s5)
		 addu  $s7,$s6,$s7
		 or    $s4,$0,$v0
		 or    $s3,$0,$v0
		 or    $s2,$0,$v0
		 or    $s1,$0,$v0
		 or    $s0,$0,$v0
		 andi  $s4,$s4,0x10
		 andi  $s3,$s3,0x08
		 andi  $s2,$s2,0x04
		 andi  $s1,$s1,0x02
		 andi  $s0,$s0,0x01
		 srl   $s4,$s4,4
		 srl   $s3,$s3,3
		 srl   $s2,$s2,2
		 srl   $s1,$s1,1
 # Check for Interrupt waiting

		 andi  $t0,$t0,0x07
		 bne   $t0,$0,interrupt

		 nop			 # Delay slot

IntCont:
		 bltz  $t7,MainExit
		 nop  		 # Delay slot

		 lhu   $t8,0x00($s7)
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp

		 lw    $a3,0x00($a3)

		 jr    $a3
		 nop			 # Delay slot

MainExit:
		 subu  $s7,$s7,$s6
		 sw    $s7,0x50($s5)		 # Save PC
		 or    $v0,$0,$s4
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s3
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s2
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s1
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s0
		 lw    $t0,0x44($s5)
		 sw    $v0,0x48($s5)
		 andi  $t0,$t0,0x20
		 bne   $t0,$0,ME1		 # Mode ?
		 lw    $v0,0x3C($s5)		 # Delay slot - Get A7
		 bgez  $0,MC68Kexit
		 sw    $v0,0x78($s5)		 # Delay slot - Save in USP
ME1:
		 sw    $v0,0x40($s5)		 # Save in ISP
MC68Kexit:
		 sw    $t7,m68k_ICount
		 lw    $gp,0x44($sp)
		 lw    $s0,0x48($sp)
		 lw    $s1,0x4C($sp)
		 lw    $s2,0x50($sp)
		 lw    $s3,0x54($sp)
		 lw    $s4,0x58($sp)
		 lw    $s5,0x5C($sp)
		 lw    $s6,0x60($sp)
		 lw    $s7,0x64($sp)
		 lw    $fp,0x68($sp)
		 lw    $ra,0x6C($sp)
		 jr    $ra
		 addiu $sp,$sp,112		 # Delay slot
	 .end   M68000_RUN


 # Interrupt check

interrupt:
		 lbu   $t1,0x54($s5)
		 andi  $v0,$t1,0x07
		 xori  $t0,$v0,0x07		 # Always take 7
		 beq   $t0,$0,procint
		 nop			 # Delay slot

		 lbu   $v1,0x44($s5)		 # int mask
		 andi  $v1,$v1,0x07
		 subu  $t0,$v0,$v1
		 blez  $t0,IntCont
		 nop			 # Delay slot

procint:
		 andi  $t1,$t1,0x7f		 # remove stop
		 sb    $t1,0x54($s5)

		 lw    $t9,0x5C($s5)
		 sw    $v0,0x40($sp)		 # save level

		 sw    $v0,0x3C($sp)
		 sw    $t7,m68k_ICount
		 jalr  $t9	 # get the IRQ level
		 or    $a0,$0,$v0		 # irq line #		    Delay slot
		 lw    $t7,m68k_ICount
		 bgez  $v0,AUTOVECTOR
		 lw    $v1,0x3C($sp)		 # Delay slot
		 addiu $v0,$v1,24		 # Vector

AUTOVECTOR:

		 m_jal   Exception
		 nop                    	 # Delay slot

		 lbu   $v1,0x44($s5)		 # set Int mask
		 lw    $v0,0x40($sp)
		 andi  $v1,$v1,0xF8
		 or    $v1,$v1,$v0
		 bgez  $0,IntCont
		 sb    $v1,0x44($s5)		 # Delay slot

	 .ent   Exception
Exception:
		 addiu $sp,$sp,-48
		 .frame $sp, 48, $ra

		 sw    $ra,0x2C($sp)
		 sb    $v0,0x28($sp)		 # Save IRQ Vector for Later
		 la    $t0,exception_cycles
		 addu  $v0,$v0,$t0
		 lbu   $v0,0x00($v0)		 # Get Cycles
		 subu  $t7,$t7,$v0		 # Decrement ICount
		 lbu   $v0,0x44($s5)
		 sll   $v0,$v0,4
		 or    $v0,$v0,$s4
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s3
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s2
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s1
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s0
		 andi  $t9,$v0,0x2000			 # Which Mode ?
		 bne   $t9,$0,ExSuperMode		 # Supervisor
		 lw    $a0,0x3C($s5)			 # Delay slot - Get A7
		 srl   $t9,$v0,8
		 ori   $t9,$t9,0x20
		 sb    $t9,0x44($s5)			 # Set Supervisor Mode
		 sw    $a0,0x78($s5)			 # Save in USP
		 lw    $a0,0x40($s5)			 # Get ISP
ExSuperMode:
		 addiu $a0,$a0,-6
		 sw    $a0,0x3C($s5)			 # Put in A7
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $a0,0x24($sp)
		 sw    $gp,0x20($sp)
		 or    $a1,$0,$v0
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x20($sp)
		 lw    $a0,0x24($sp)
		 lw    $t7,m68k_ICount
		 subu  $a1,$s7,$s6
		 addiu $a0,$a0,2
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x24($sp)
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x24($sp)
		 lw    $t7,m68k_ICount
		 lbu   $v0,0x28($sp)		 # Level
		 lw    $t9,0x7C($s5)
		 sll   $v0,$v0,2
		 addu  $a0,$v0,$t9
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x28($sp)
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x28($sp)
		 lw    $t7,m68k_ICount
		 andi  $t9,$v0,0x01
		 beq   $t9,$0,2f
		 addu  $s7,$s6,$v0     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $v0,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$v0,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 lw    $ra,0x2C($sp)
		 jr    $ra
		 addiu $sp,$sp,48		 # Delay slot

	 .end   Exception




OP0_1000:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x0f
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lbu   $v0,0x00($t0)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 sb    $v0,0x00($t8)
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1010:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 sb    $v0,0x00($t8)
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1018:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 sb    $v0,0x00($t8)
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_101f:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 sb    $v0,0x00($t8)
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1020:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 sb    $v0,0x00($t8)
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1027:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 sb    $v0,0x00($t8)
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1028:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 sb    $v0,0x00($t8)
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1030:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 sb    $v0,0x00($t8)
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1038:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 sb    $v0,0x00($t8)
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1039:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 sb    $v0,0x00($t8)
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_103a:				#:
		 addiu $s7,$s7,2

		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xAC($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 sb    $v0,0x00($t8)
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_103b:				#:
		 addiu $s7,$s7,2

		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xAC($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 sb    $v0,0x00($t8)
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_103c:				#:
		 addiu $s7,$s7,2

		 lbu   $v0,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 sb    $v0,0x00($t8)
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1080:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x0f
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lbu   $v0,0x00($t0)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1090:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1098:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_109f:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_10a0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_10a7:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_10a8:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_10b0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_10b8:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_10b9:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_10ba:				#:
		 addiu $s7,$s7,2

		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xAC($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_10bb:				#:
		 addiu $s7,$s7,2

		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xAC($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_10bc:				#:
		 addiu $s7,$s7,2

		 lbu   $v0,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_10c0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x0f
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lbu   $v0,0x00($t0)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t8)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_10d0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t8)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_10d8:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t8)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_10df:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t8)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_10e0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t8)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_10e7:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t8)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_10e8:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t8)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_10f0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t8)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_10f8:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t8)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_10f9:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t8)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_10fa:				#:
		 addiu $s7,$s7,2

		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xAC($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t8)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_10fb:				#:
		 addiu $s7,$s7,2

		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xAC($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t8)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_10fc:				#:
		 addiu $s7,$s7,2

		 lbu   $v0,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t8)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1100:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x0f
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lbu   $v0,0x00($t0)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1110:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1118:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_111f:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1120:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1127:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1128:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1130:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1138:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1139:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_113a:				#:
		 addiu $s7,$s7,2

		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xAC($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_113b:				#:
		 addiu $s7,$s7,2

		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xAC($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_113c:				#:
		 addiu $s7,$s7,2

		 lbu   $v0,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1140:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x0f
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lbu   $v0,0x00($t0)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lh    $a3,0x00($s7)
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1150:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lh    $a3,0x00($s7)
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1158:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lh    $a3,0x00($s7)
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_115f:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lh    $a3,0x00($s7)
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1160:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lh    $a3,0x00($s7)
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1167:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lh    $a3,0x00($s7)
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1168:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lh    $a3,0x00($s7)
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1170:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lh    $a3,0x00($s7)
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1178:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lh    $a3,0x00($s7)
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1179:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lh    $a3,0x00($s7)
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_117a:				#:
		 addiu $s7,$s7,2

		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xAC($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lh    $a3,0x00($s7)
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_117b:				#:
		 addiu $s7,$s7,2

		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xAC($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lh    $a3,0x00($s7)
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_117c:				#:
		 addiu $s7,$s7,2

		 lbu   $v0,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lh    $a3,0x00($s7)
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1180:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x0f
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lbu   $v0,0x00($t0)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1190:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1198:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_119f:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_11a0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_11a7:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_11a8:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_11b0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_11b8:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_11b9:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-26
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_11ba:				#:
		 addiu $s7,$s7,2

		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xAC($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_11bb:				#:
		 addiu $s7,$s7,2

		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xAC($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_11bc:				#:
		 addiu $s7,$s7,2

		 lbu   $v0,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_11c0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x0f
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lbu   $v0,0x00($t0)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_11d0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_11d8:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_11df:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_11e0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_11e7:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_11e8:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_11f0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_11f8:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_11f9:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_11fa:				#:
		 addiu $s7,$s7,2

		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xAC($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_11fb:				#:
		 addiu $s7,$s7,2

		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xAC($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_11fc:				#:
		 addiu $s7,$s7,2

		 lbu   $v0,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_13c0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x0f
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lbu   $v0,0x00($t0)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_13d0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_13d8:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_13df:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_13e0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_13e7:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_13e8:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_13f0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-26
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_13f8:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_13f9:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-28
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_13fa:				#:
		 addiu $s7,$s7,2

		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xAC($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_13fb:				#:
		 addiu $s7,$s7,2

		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xAC($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-26
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_13fc:				#:
		 addiu $s7,$s7,2

		 lbu   $v0,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1ec0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x0f
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lbu   $v0,0x00($t0)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1ed0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1ed8:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1edf:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1ee0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1ee7:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1ee8:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1ef0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1ef8:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1ef9:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1efa:				#:
		 addiu $s7,$s7,2

		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xAC($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1efb:				#:
		 addiu $s7,$s7,2

		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xAC($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1efc:				#:
		 addiu $s7,$s7,2

		 lbu   $v0,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1f00:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x0f
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lbu   $v0,0x00($t0)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1f10:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1f18:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1f1f:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1f20:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1f27:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1f28:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1f30:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1f38:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1f39:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1f3a:				#:
		 addiu $s7,$s7,2

		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xAC($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1f3b:				#:
		 addiu $s7,$s7,2

		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xAC($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_1f3c:				#:
		 addiu $s7,$s7,2

		 lbu   $v0,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_2000:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x0f
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $v0,0x00($t0)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 sw    $v0,0x00($t8)
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_2010:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 sw    $v0,0x00($t8)
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_2018:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 sw    $v0,0x00($t8)
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_2020:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 sw    $v0,0x00($t8)
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_2028:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 sw    $v0,0x00($t8)
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_2030:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 sw    $v0,0x00($t8)
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_2038:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 sw    $v0,0x00($t8)
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_2039:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 sw    $v0,0x00($t8)
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_203a:				#:
		 addiu $s7,$s7,2

		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xB4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 sw    $v0,0x00($t8)
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_203b:				#:
		 addiu $s7,$s7,2

		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xB4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 sw    $v0,0x00($t8)
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_203c:				#:
		 addiu $s7,$s7,2

		 lhu   $v0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $v0,$v0,16
		 or    $v0,$v0,$t9
		 addiu $s7,$s7,4
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 sw    $v0,0x00($t8)
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_2040:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x0f
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $v0,0x00($t0)
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 sw    $v0,0x20($t8)
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_2050:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 sw    $v0,0x20($t8)
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_2058:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 sw    $v0,0x20($t8)
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_2060:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 sw    $v0,0x20($t8)
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_2068:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 sw    $v0,0x20($t8)
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_2070:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 sw    $v0,0x20($t8)
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_2078:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 sw    $v0,0x20($t8)
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_2079:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 sw    $v0,0x20($t8)
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_207a:				#:
		 addiu $s7,$s7,2

		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xB4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 sw    $v0,0x20($t8)
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_207b:				#:
		 addiu $s7,$s7,2

		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xB4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 sw    $v0,0x20($t8)
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_207c:				#:
		 addiu $s7,$s7,2

		 lhu   $v0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $v0,$v0,16
		 or    $v0,$v0,$t9
		 addiu $s7,$s7,4
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 sw    $v0,0x20($t8)
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_2080:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x0f
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $v0,0x00($t0)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_2090:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_2098:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_20a0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_20a8:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_20b0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-26
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_20b8:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_20b9:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-28
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_20ba:				#:
		 addiu $s7,$s7,2

		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xB4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_20bb:				#:
		 addiu $s7,$s7,2

		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xB4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-26
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_20bc:				#:
		 addiu $s7,$s7,2

		 lhu   $v0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $v0,$v0,16
		 or    $v0,$v0,$t9
		 addiu $s7,$s7,4
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_20c0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x0f
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $v0,0x00($t0)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($t8)
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_20d0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($t8)
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_20d8:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($t8)
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_20e0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($t8)
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_20e8:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($t8)
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_20f0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($t8)
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-26
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_20f8:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($t8)
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_20f9:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($t8)
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-28
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_20fa:				#:
		 addiu $s7,$s7,2

		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xB4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($t8)
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_20fb:				#:
		 addiu $s7,$s7,2

		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xB4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($t8)
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-26
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_20fc:				#:
		 addiu $s7,$s7,2

		 lhu   $v0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $v0,$v0,16
		 or    $v0,$v0,$t9
		 addiu $s7,$s7,4
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($t8)
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_2100:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x0f
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $v0,0x00($t0)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 sw    $v0,0x3C($sp)
		 sw    $t6,0x38($sp)
		 addiu $t6,$t6,2
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $t9,0xA0($s5)
		 lw    $a0,0x38($sp)
		 lw    $a1,0x3C($sp)
		 jalr  $t9
		 srl   $a1,$a1,16         	 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_2110:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 sw    $v0,0x3C($sp)
		 sw    $t6,0x38($sp)
		 addiu $t6,$t6,2
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $t9,0xA0($s5)
		 lw    $a0,0x38($sp)
		 lw    $a1,0x3C($sp)
		 jalr  $t9
		 srl   $a1,$a1,16         	 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_2118:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 sw    $v0,0x3C($sp)
		 sw    $t6,0x38($sp)
		 addiu $t6,$t6,2
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $t9,0xA0($s5)
		 lw    $a0,0x38($sp)
		 lw    $a1,0x3C($sp)
		 jalr  $t9
		 srl   $a1,$a1,16         	 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_2120:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 sw    $v0,0x3C($sp)
		 sw    $t6,0x38($sp)
		 addiu $t6,$t6,2
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $t9,0xA0($s5)
		 lw    $a0,0x38($sp)
		 lw    $a1,0x3C($sp)
		 jalr  $t9
		 srl   $a1,$a1,16         	 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_2128:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 sw    $v0,0x3C($sp)
		 sw    $t6,0x38($sp)
		 addiu $t6,$t6,2
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $t9,0xA0($s5)
		 lw    $a0,0x38($sp)
		 lw    $a1,0x3C($sp)
		 jalr  $t9
		 srl   $a1,$a1,16         	 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-26
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_2130:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 sw    $v0,0x3C($sp)
		 sw    $t6,0x38($sp)
		 addiu $t6,$t6,2
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $t9,0xA0($s5)
		 lw    $a0,0x38($sp)
		 lw    $a1,0x3C($sp)
		 jalr  $t9
		 srl   $a1,$a1,16         	 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-28
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_2138:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 sw    $v0,0x3C($sp)
		 sw    $t6,0x38($sp)
		 addiu $t6,$t6,2
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $t9,0xA0($s5)
		 lw    $a0,0x38($sp)
		 lw    $a1,0x3C($sp)
		 jalr  $t9
		 srl   $a1,$a1,16         	 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-26
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_2139:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 sw    $v0,0x3C($sp)
		 sw    $t6,0x38($sp)
		 addiu $t6,$t6,2
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $t9,0xA0($s5)
		 lw    $a0,0x38($sp)
		 lw    $a1,0x3C($sp)
		 jalr  $t9
		 srl   $a1,$a1,16         	 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-30
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_213a:				#:
		 addiu $s7,$s7,2

		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xB4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 sw    $v0,0x3C($sp)
		 sw    $t6,0x38($sp)
		 addiu $t6,$t6,2
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $t9,0xA0($s5)
		 lw    $a0,0x38($sp)
		 lw    $a1,0x3C($sp)
		 jalr  $t9
		 srl   $a1,$a1,16         	 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-26
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_213b:				#:
		 addiu $s7,$s7,2

		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xB4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 sw    $v0,0x3C($sp)
		 sw    $t6,0x38($sp)
		 addiu $t6,$t6,2
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $t9,0xA0($s5)
		 lw    $a0,0x38($sp)
		 lw    $a1,0x3C($sp)
		 jalr  $t9
		 srl   $a1,$a1,16         	 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-28
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_213c:				#:
		 addiu $s7,$s7,2

		 lhu   $v0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $v0,$v0,16
		 or    $v0,$v0,$t9
		 addiu $s7,$s7,4
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 sw    $v0,0x3C($sp)
		 sw    $t6,0x38($sp)
		 addiu $t6,$t6,2
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $t9,0xA0($s5)
		 lw    $a0,0x38($sp)
		 lw    $a1,0x3C($sp)
		 jalr  $t9
		 srl   $a1,$a1,16         	 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_2140:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x0f
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $v0,0x00($t0)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lh    $a3,0x00($s7)
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_2150:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lh    $a3,0x00($s7)
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_2158:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lh    $a3,0x00($s7)
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_2160:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lh    $a3,0x00($s7)
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-26
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_2168:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lh    $a3,0x00($s7)
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-28
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_2170:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lh    $a3,0x00($s7)
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-30
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_2178:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lh    $a3,0x00($s7)
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-28
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_2179:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lh    $a3,0x00($s7)
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-32
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_217a:				#:
		 addiu $s7,$s7,2

		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xB4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lh    $a3,0x00($s7)
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-28
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_217b:				#:
		 addiu $s7,$s7,2

		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xB4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lh    $a3,0x00($s7)
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-30
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_217c:				#:
		 addiu $s7,$s7,2

		 lhu   $v0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $v0,$v0,16
		 or    $v0,$v0,$t9
		 addiu $s7,$s7,4
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lh    $a3,0x00($s7)
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_2180:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x0f
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $v0,0x00($t0)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_2190:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-26
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_2198:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-26
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_21a0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-28
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_21a8:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-30
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_21b0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-32
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_21b8:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-30
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_21b9:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-34
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_21ba:				#:
		 addiu $s7,$s7,2

		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xB4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-30
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_21bb:				#:
		 addiu $s7,$s7,2

		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xB4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-32
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_21bc:				#:
		 addiu $s7,$s7,2

		 lhu   $v0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $v0,$v0,16
		 or    $v0,$v0,$t9
		 addiu $s7,$s7,4
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_21c0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x0f
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $v0,0x00($t0)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_21d0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_21d8:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_21e0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-26
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_21e8:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-28
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_21f0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-30
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_21f8:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-28
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_21f9:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-32
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_21fa:				#:
		 addiu $s7,$s7,2

		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xB4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-28
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_21fb:				#:
		 addiu $s7,$s7,2

		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xB4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-30
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_21fc:				#:
		 addiu $s7,$s7,2

		 lhu   $v0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $v0,$v0,16
		 or    $v0,$v0,$t9
		 addiu $s7,$s7,4
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_23c0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x0f
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $v0,0x00($t0)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_23d0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-28
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_23d8:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-28
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_23e0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-30
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_23e8:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-32
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_23f0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-34
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_23f8:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-32
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_23f9:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-36
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_23fa:				#:
		 addiu $s7,$s7,2

		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xB4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-32
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_23fb:				#:
		 addiu $s7,$s7,2

		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xB4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-34
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_23fc:				#:
		 addiu $s7,$s7,2

		 lhu   $v0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $v0,$v0,16
		 or    $v0,$v0,$t9
		 addiu $s7,$s7,4
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_3000:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x0f
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lhu   $v0,0x00($t0)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 sh    $v0,0x00($t8)
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_3010:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 sh    $v0,0x00($t8)
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_3018:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 sh    $v0,0x00($t8)
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_3020:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 sh    $v0,0x00($t8)
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_3028:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 sh    $v0,0x00($t8)
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_3030:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 sh    $v0,0x00($t8)
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_3038:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 sh    $v0,0x00($t8)
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_3039:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 sh    $v0,0x00($t8)
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_303a:				#:
		 addiu $s7,$s7,2

		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 sh    $v0,0x00($t8)
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_303b:				#:
		 addiu $s7,$s7,2

		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 sh    $v0,0x00($t8)
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_303c:				#:
		 addiu $s7,$s7,2

		 lhu   $v0,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 sh    $v0,0x00($t8)
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_3040:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x0f
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lhu   $v0,0x00($t0)
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 m_seh   $v0,$v0
		 addu  $t8,$t8,$s5
		 sw    $v0,0x20($t8)
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_3050:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 m_seh   $v0,$v0
		 addu  $t8,$t8,$s5
		 sw    $v0,0x20($t8)
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_3058:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 m_seh   $v0,$v0
		 addu  $t8,$t8,$s5
		 sw    $v0,0x20($t8)
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_3060:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 m_seh   $v0,$v0
		 addu  $t8,$t8,$s5
		 sw    $v0,0x20($t8)
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_3068:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 m_seh   $v0,$v0
		 addu  $t8,$t8,$s5
		 sw    $v0,0x20($t8)
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_3070:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 m_seh   $v0,$v0
		 addu  $t8,$t8,$s5
		 sw    $v0,0x20($t8)
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_3078:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 m_seh   $v0,$v0
		 addu  $t8,$t8,$s5
		 sw    $v0,0x20($t8)
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_3079:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 m_seh   $v0,$v0
		 addu  $t8,$t8,$s5
		 sw    $v0,0x20($t8)
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_307a:				#:
		 addiu $s7,$s7,2

		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 m_seh   $v0,$v0
		 addu  $t8,$t8,$s5
		 sw    $v0,0x20($t8)
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_307b:				#:
		 addiu $s7,$s7,2

		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 m_seh   $v0,$v0
		 addu  $t8,$t8,$s5
		 sw    $v0,0x20($t8)
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_307c:				#:
		 addiu $s7,$s7,2

		 lhu   $v0,0x00($s7)
		 addiu $s7,$s7,2
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 m_seh   $v0,$v0
		 addu  $t8,$t8,$s5
		 sw    $v0,0x20($t8)
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_3080:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x0f
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lhu   $v0,0x00($t0)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_3090:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_3098:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_30a0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_30a8:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_30b0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_30b8:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_30b9:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_30ba:				#:
		 addiu $s7,$s7,2

		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_30bb:				#:
		 addiu $s7,$s7,2

		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_30bc:				#:
		 addiu $s7,$s7,2

		 lhu   $v0,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_30c0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x0f
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lhu   $v0,0x00($t0)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_30d0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_30d8:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_30e0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_30e8:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_30f0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_30f8:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_30f9:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_30fa:				#:
		 addiu $s7,$s7,2

		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_30fb:				#:
		 addiu $s7,$s7,2

		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_30fc:				#:
		 addiu $s7,$s7,2

		 lhu   $v0,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_3100:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x0f
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lhu   $v0,0x00($t0)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_3110:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_3118:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_3120:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_3128:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_3130:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_3138:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_3139:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_313a:				#:
		 addiu $s7,$s7,2

		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_313b:				#:
		 addiu $s7,$s7,2

		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_313c:				#:
		 addiu $s7,$s7,2

		 lhu   $v0,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_3140:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x0f
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lhu   $v0,0x00($t0)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lh    $a3,0x00($s7)
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_3150:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lh    $a3,0x00($s7)
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_3158:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lh    $a3,0x00($s7)
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_3160:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lh    $a3,0x00($s7)
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_3168:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lh    $a3,0x00($s7)
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_3170:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lh    $a3,0x00($s7)
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_3178:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lh    $a3,0x00($s7)
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_3179:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lh    $a3,0x00($s7)
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_317a:				#:
		 addiu $s7,$s7,2

		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lh    $a3,0x00($s7)
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_317b:				#:
		 addiu $s7,$s7,2

		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lh    $a3,0x00($s7)
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_317c:				#:
		 addiu $s7,$s7,2

		 lhu   $v0,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lh    $a3,0x00($s7)
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_3180:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x0f
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lhu   $v0,0x00($t0)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_3190:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_3198:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_31a0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_31a8:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_31b0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_31b8:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_31b9:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-26
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_31ba:				#:
		 addiu $s7,$s7,2

		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_31bb:				#:
		 addiu $s7,$s7,2

		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_31bc:				#:
		 addiu $s7,$s7,2

		 lhu   $v0,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_31c0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x0f
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lhu   $v0,0x00($t0)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_31d0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_31d8:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_31e0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_31e8:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_31f0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_31f8:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_31f9:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_31fa:				#:
		 addiu $s7,$s7,2

		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_31fb:				#:
		 addiu $s7,$s7,2

		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_31fc:				#:
		 addiu $s7,$s7,2

		 lhu   $v0,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_33c0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x0f
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lhu   $v0,0x00($t0)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_33d0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_33d8:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_33e0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_33e8:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_33f0:				#:
		 addiu $s7,$s7,2

		 and   $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-26
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_33f8:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_33f9:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-28
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_33fa:				#:
		 addiu $s7,$s7,2

		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_33fb:				#:
		 addiu $s7,$s7,2

		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-26
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_33fc:				#:
		 addiu $s7,$s7,2

		 lhu   $v0,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0000:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lb    $v0,0x00($t8)
		 or    $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 sb    $v0,0x00($t8)
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0010:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 or    $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0018:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t8)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 or    $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_001f:				#:
		 addiu $s7,$s7,2

		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 or    $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0020:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 or    $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0027:				#:
		 addiu $s7,$s7,2

		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 or    $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0028:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 or    $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0030:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 or    $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0038:				#:
		 addiu $s7,$s7,2

		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 or    $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0039:				#:
		 addiu $s7,$s7,2

		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 or    $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_003c:				#:
		 addiu $s7,$s7,2

		 lbu   $t0,0x00($s7)
		 addiu $s7,$s7,2
		 or    $v0,$0,$s4
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s3
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s2
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s1
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s0
		 or    $v0,$v0,$t0
		 or    $s4,$0,$v0
		 or    $s3,$0,$v0
		 or    $s2,$0,$v0
		 or    $s1,$0,$v0
		 or    $s0,$0,$v0
		 andi  $s4,$s4,0x10
		 andi  $s3,$s3,0x08
		 andi  $s2,$s2,0x04
		 andi  $s1,$s1,0x02
		 andi  $s0,$s0,0x01
		 srl   $s4,$s4,4
		 srl   $s3,$s3,3
		 srl   $s2,$s2,2
		 srl   $s1,$s1,1
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0040:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lh    $v0,0x00($t8)
		 or    $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 sh    $v0,0x00($t8)
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0050:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 or    $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0058:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 or    $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0060:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 or    $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0068:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 or    $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0070:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 or    $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0078:				#:
		 addiu $s7,$s7,2

		 lh    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 or    $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0079:				#:
		 addiu $s7,$s7,2

		 lh    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 or    $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_007c:				#:
		 lbu   $t0,0x44($s5)
		 andi  $t0,$t0,0x20       	 # Supervisor Mode ?
		 bne   $t0,$0,9f
		 addiu $s7,$s7,2        	 # Delay slot

		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,8

		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	9:
		 lhu   $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lbu   $v0,0x44($s5)
		 sll   $v0,$v0,4
		 or    $v0,$v0,$s4
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s3
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s2
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s1
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s0
		 or    $v0,$v0,$t0
		 andi  $t9,$v0,0x2000    	 # User Mode ?
		 bne   $t9,$0,0f
		 or    $s2,$0,$v0       	 # Delay slot
		 lw    $s0,0x3C($s5)
		 lw    $s1,0x78($s5)
		 sw    $s0,0x40($s5)
		 sw    $s1,0x3C($s5)
	0:
		 srl   $s2,$s2,8
		 sb    $s2,0x44($s5)    	 # T, S & I
		 or    $s4,$0,$v0
		 or    $s3,$0,$v0
		 or    $s2,$0,$v0
		 or    $s1,$0,$v0
		 or    $s0,$0,$v0
		 andi  $s4,$s4,0x10
		 andi  $s3,$s3,0x08
		 andi  $s2,$s2,0x04
		 andi  $s1,$s1,0x02
		 andi  $s0,$s0,0x01
		 srl   $s4,$s4,4
		 srl   $s3,$s3,3
		 srl   $s2,$s2,2
		 srl   $s1,$s1,1
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lbu   $a3,0x54($s5)     	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
 # Check for Interrupt waiting

		 andi  $a3,$a3,0x07
		 beq   $a3,$0,3f
		 nop                    	 # Delay slot
		 m_j   interrupt
		 nop                    	 # Delay slot

	3:
		 lhu   $t8,0x00($s7)
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0080:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lhu   $t0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t0,$t0,16
		 or    $t0,$t0,$t9
		 addiu $s7,$s7,4
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $v0,0x00($t8)
		 or    $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 sw    $v0,0x00($t8)
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0090:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lhu   $t0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t0,$t0,16
		 or    $t0,$t0,$t9
		 addiu $s7,$s7,4
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-28
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0098:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lhu   $t0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t0,$t0,16
		 or    $t0,$t0,$t9
		 addiu $s7,$s7,4
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($t8)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-28
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_00a0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lhu   $t0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t0,$t0,16
		 or    $t0,$t0,$t9
		 addiu $s7,$s7,4
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($t8)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-30
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_00a8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lhu   $t0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t0,$t0,16
		 or    $t0,$t0,$t9
		 addiu $s7,$s7,4
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-32
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_00b0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lhu   $t0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t0,$t0,16
		 or    $t0,$t0,$t9
		 addiu $s7,$s7,4
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-34
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_00b8:				#:
		 addiu $s7,$s7,2

		 lhu   $t0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t0,$t0,16
		 or    $t0,$t0,$t9
		 addiu $s7,$s7,4
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-32
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_00b9:				#:
		 addiu $s7,$s7,2

		 lhu   $t0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t0,$t0,16
		 or    $t0,$t0,$t9
		 addiu $s7,$s7,4
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-36
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0200:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lb    $v0,0x00($t8)
		 and   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 sb    $v0,0x00($t8)
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0210:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 and   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0218:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t8)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 and   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_021f:				#:
		 addiu $s7,$s7,2

		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 and   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0220:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 and   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0227:				#:
		 addiu $s7,$s7,2

		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 and   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0228:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 and   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0230:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 and   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0238:				#:
		 addiu $s7,$s7,2

		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 and   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0239:				#:
		 addiu $s7,$s7,2

		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 and   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_023c:				#:
		 addiu $s7,$s7,2

		 lbu   $t0,0x00($s7)
		 addiu $s7,$s7,2
		 or    $v0,$0,$s4
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s3
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s2
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s1
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s0
		 and   $v0,$v0,$t0
		 or    $s4,$0,$v0
		 or    $s3,$0,$v0
		 or    $s2,$0,$v0
		 or    $s1,$0,$v0
		 or    $s0,$0,$v0
		 andi  $s4,$s4,0x10
		 andi  $s3,$s3,0x08
		 andi  $s2,$s2,0x04
		 andi  $s1,$s1,0x02
		 andi  $s0,$s0,0x01
		 srl   $s4,$s4,4
		 srl   $s3,$s3,3
		 srl   $s2,$s2,2
		 srl   $s1,$s1,1
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0240:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lh    $v0,0x00($t8)
		 and   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 sh    $v0,0x00($t8)
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0250:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 and   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0258:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 and   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0260:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 and   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0268:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 and   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0270:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 and   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0278:				#:
		 addiu $s7,$s7,2

		 lh    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 and   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0279:				#:
		 addiu $s7,$s7,2

		 lh    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 and   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_027c:				#:
		 lbu   $t0,0x44($s5)
		 andi  $t0,$t0,0x20       	 # Supervisor Mode ?
		 bne   $t0,$0,9f
		 addiu $s7,$s7,2        	 # Delay slot

		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,8

		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	9:
		 lhu   $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lbu   $v0,0x44($s5)
		 sll   $v0,$v0,4
		 or    $v0,$v0,$s4
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s3
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s2
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s1
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s0
		 and   $v0,$v0,$t0
		 andi  $t9,$v0,0x2000    	 # User Mode ?
		 bne   $t9,$0,0f
		 or    $s2,$0,$v0       	 # Delay slot
		 lw    $s0,0x3C($s5)
		 lw    $s1,0x78($s5)
		 sw    $s0,0x40($s5)
		 sw    $s1,0x3C($s5)
	0:
		 srl   $s2,$s2,8
		 sb    $s2,0x44($s5)    	 # T, S & I
		 or    $s4,$0,$v0
		 or    $s3,$0,$v0
		 or    $s2,$0,$v0
		 or    $s1,$0,$v0
		 or    $s0,$0,$v0
		 andi  $s4,$s4,0x10
		 andi  $s3,$s3,0x08
		 andi  $s2,$s2,0x04
		 andi  $s1,$s1,0x02
		 andi  $s0,$s0,0x01
		 srl   $s4,$s4,4
		 srl   $s3,$s3,3
		 srl   $s2,$s2,2
		 srl   $s1,$s1,1
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lbu   $a3,0x54($s5)     	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
 # Check for Interrupt waiting

		 andi  $a3,$a3,0x07
		 beq   $a3,$0,3f
		 nop                    	 # Delay slot
		 m_j   interrupt
		 nop                    	 # Delay slot

	3:
		 lhu   $t8,0x00($s7)
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0280:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lhu   $t0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t0,$t0,16
		 or    $t0,$t0,$t9
		 addiu $s7,$s7,4
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $v0,0x00($t8)
		 and   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 sw    $v0,0x00($t8)
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0290:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lhu   $t0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t0,$t0,16
		 or    $t0,$t0,$t9
		 addiu $s7,$s7,4
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-28
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0298:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lhu   $t0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t0,$t0,16
		 or    $t0,$t0,$t9
		 addiu $s7,$s7,4
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($t8)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-28
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_02a0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lhu   $t0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t0,$t0,16
		 or    $t0,$t0,$t9
		 addiu $s7,$s7,4
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($t8)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-30
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_02a8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lhu   $t0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t0,$t0,16
		 or    $t0,$t0,$t9
		 addiu $s7,$s7,4
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-32
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_02b0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lhu   $t0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t0,$t0,16
		 or    $t0,$t0,$t9
		 addiu $s7,$s7,4
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-34
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_02b8:				#:
		 addiu $s7,$s7,2

		 lhu   $t0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t0,$t0,16
		 or    $t0,$t0,$t9
		 addiu $s7,$s7,4
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-32
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_02b9:				#:
		 addiu $s7,$s7,2

		 lhu   $t0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t0,$t0,16
		 or    $t0,$t0,$t9
		 addiu $s7,$s7,4
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-36
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0400:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 subu  $v0,$t1,$t0
		 sltu  $s0,$t1,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 sb    $v0,0x00($t8)
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0410:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t1,$v0
		 subu  $v0,$t1,$t0
		 sltu  $s0,$t1,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0418:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t8)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t1,$v0
		 subu  $v0,$t1,$t0
		 sltu  $s0,$t1,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_041f:				#:
		 addiu $s7,$s7,2

		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t1,$v0
		 subu  $v0,$t1,$t0
		 sltu  $s0,$t1,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0420:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t1,$v0
		 subu  $v0,$t1,$t0
		 sltu  $s0,$t1,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0427:				#:
		 addiu $s7,$s7,2

		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t1,$v0
		 subu  $v0,$t1,$t0
		 sltu  $s0,$t1,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0428:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t1,$v0
		 subu  $v0,$t1,$t0
		 sltu  $s0,$t1,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0430:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t1,$v0
		 subu  $v0,$t1,$t0
		 sltu  $s0,$t1,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0438:				#:
		 addiu $s7,$s7,2

		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t1,$v0
		 subu  $v0,$t1,$t0
		 sltu  $s0,$t1,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0439:				#:
		 addiu $s7,$s7,2

		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t1,$v0
		 subu  $v0,$t1,$t0
		 sltu  $s0,$t1,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0440:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 subu  $v0,$t1,$t0
		 sltu  $s0,$t1,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 sh    $v0,0x00($t8)
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0450:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t1,$v0
		 subu  $v0,$t1,$t0
		 sltu  $s0,$t1,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0458:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t1,$v0
		 subu  $v0,$t1,$t0
		 sltu  $s0,$t1,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0460:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t1,$v0
		 subu  $v0,$t1,$t0
		 sltu  $s0,$t1,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0468:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t1,$v0
		 subu  $v0,$t1,$t0
		 sltu  $s0,$t1,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0470:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t1,$v0
		 subu  $v0,$t1,$t0
		 sltu  $s0,$t1,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0478:				#:
		 addiu $s7,$s7,2

		 lh    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t1,$v0
		 subu  $v0,$t1,$t0
		 sltu  $s0,$t1,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0479:				#:
		 addiu $s7,$s7,2

		 lh    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t1,$v0
		 subu  $v0,$t1,$t0
		 sltu  $s0,$t1,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0480:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lhu   $t0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t0,$t0,16
		 or    $t0,$t0,$t9
		 addiu $s7,$s7,4
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 subu  $v0,$t1,$t0
		 sltu  $s0,$t1,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 sw    $v0,0x00($t8)
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0490:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lhu   $t0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t0,$t0,16
		 or    $t0,$t0,$t9
		 addiu $s7,$s7,4
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t1,$0,$v0
		 subu  $v0,$t1,$t0
		 sltu  $s0,$t1,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-28
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0498:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lhu   $t0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t0,$t0,16
		 or    $t0,$t0,$t9
		 addiu $s7,$s7,4
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($t8)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t1,$0,$v0
		 subu  $v0,$t1,$t0
		 sltu  $s0,$t1,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-28
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_04a0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lhu   $t0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t0,$t0,16
		 or    $t0,$t0,$t9
		 addiu $s7,$s7,4
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($t8)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t1,$0,$v0
		 subu  $v0,$t1,$t0
		 sltu  $s0,$t1,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-30
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_04a8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lhu   $t0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t0,$t0,16
		 or    $t0,$t0,$t9
		 addiu $s7,$s7,4
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t1,$0,$v0
		 subu  $v0,$t1,$t0
		 sltu  $s0,$t1,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-32
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_04b0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lhu   $t0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t0,$t0,16
		 or    $t0,$t0,$t9
		 addiu $s7,$s7,4
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t1,$0,$v0
		 subu  $v0,$t1,$t0
		 sltu  $s0,$t1,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-34
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_04b8:				#:
		 addiu $s7,$s7,2

		 lhu   $t0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t0,$t0,16
		 or    $t0,$t0,$t9
		 addiu $s7,$s7,4
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t1,$0,$v0
		 subu  $v0,$t1,$t0
		 sltu  $s0,$t1,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-32
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_04b9:				#:
		 addiu $s7,$s7,2

		 lhu   $t0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t0,$t0,16
		 or    $t0,$t0,$t9
		 addiu $s7,$s7,4
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t1,$0,$v0
		 subu  $v0,$t1,$t0
		 sltu  $s0,$t1,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-36
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0600:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 addu  $v0,$t1,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 sb    $v0,0x00($t8)
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0610:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t1,$v0
		 addu  $v0,$t1,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0618:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t8)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t1,$v0
		 addu  $v0,$t1,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_061f:				#:
		 addiu $s7,$s7,2

		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t1,$v0
		 addu  $v0,$t1,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0620:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t1,$v0
		 addu  $v0,$t1,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0627:				#:
		 addiu $s7,$s7,2

		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t1,$v0
		 addu  $v0,$t1,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0628:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t1,$v0
		 addu  $v0,$t1,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0630:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t1,$v0
		 addu  $v0,$t1,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0638:				#:
		 addiu $s7,$s7,2

		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t1,$v0
		 addu  $v0,$t1,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0639:				#:
		 addiu $s7,$s7,2

		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t1,$v0
		 addu  $v0,$t1,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0640:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 addu  $v0,$t1,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 sh    $v0,0x00($t8)
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0650:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t1,$v0
		 addu  $v0,$t1,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0658:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t1,$v0
		 addu  $v0,$t1,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0660:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t1,$v0
		 addu  $v0,$t1,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0668:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t1,$v0
		 addu  $v0,$t1,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0670:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t1,$v0
		 addu  $v0,$t1,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0678:				#:
		 addiu $s7,$s7,2

		 lh    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t1,$v0
		 addu  $v0,$t1,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0679:				#:
		 addiu $s7,$s7,2

		 lh    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t1,$v0
		 addu  $v0,$t1,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0680:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lhu   $t0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t0,$t0,16
		 or    $t0,$t0,$t9
		 addiu $s7,$s7,4
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 addu  $v0,$t1,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 sw    $v0,0x00($t8)
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0690:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lhu   $t0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t0,$t0,16
		 or    $t0,$t0,$t9
		 addiu $s7,$s7,4
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t1,$0,$v0
		 addu  $v0,$t1,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-28
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0698:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lhu   $t0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t0,$t0,16
		 or    $t0,$t0,$t9
		 addiu $s7,$s7,4
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($t8)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t1,$0,$v0
		 addu  $v0,$t1,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-28
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_06a0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lhu   $t0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t0,$t0,16
		 or    $t0,$t0,$t9
		 addiu $s7,$s7,4
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($t8)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t1,$0,$v0
		 addu  $v0,$t1,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-30
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_06a8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lhu   $t0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t0,$t0,16
		 or    $t0,$t0,$t9
		 addiu $s7,$s7,4
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t1,$0,$v0
		 addu  $v0,$t1,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-32
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_06b0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lhu   $t0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t0,$t0,16
		 or    $t0,$t0,$t9
		 addiu $s7,$s7,4
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t1,$0,$v0
		 addu  $v0,$t1,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-34
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_06b8:				#:
		 addiu $s7,$s7,2

		 lhu   $t0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t0,$t0,16
		 or    $t0,$t0,$t9
		 addiu $s7,$s7,4
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t1,$0,$v0
		 addu  $v0,$t1,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-32
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_06b9:				#:
		 addiu $s7,$s7,2

		 lhu   $t0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t0,$t0,16
		 or    $t0,$t0,$t9
		 addiu $s7,$s7,4
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t1,$0,$v0
		 addu  $v0,$t1,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$t1,$t0
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-36
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0a00:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lb    $v0,0x00($t8)
		 xor   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 sb    $v0,0x00($t8)
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0a10:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 xor   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0a18:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t8)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 xor   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0a1f:				#:
		 addiu $s7,$s7,2

		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 xor   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0a20:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 xor   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0a27:				#:
		 addiu $s7,$s7,2

		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 xor   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0a28:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 xor   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0a30:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 xor   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0a38:				#:
		 addiu $s7,$s7,2

		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 xor   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0a39:				#:
		 addiu $s7,$s7,2

		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 xor   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0a3c:				#:
		 addiu $s7,$s7,2

		 lbu   $t0,0x00($s7)
		 addiu $s7,$s7,2
		 or    $v0,$0,$s4
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s3
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s2
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s1
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s0
		 xor   $v0,$v0,$t0
		 or    $s4,$0,$v0
		 or    $s3,$0,$v0
		 or    $s2,$0,$v0
		 or    $s1,$0,$v0
		 or    $s0,$0,$v0
		 andi  $s4,$s4,0x10
		 andi  $s3,$s3,0x08
		 andi  $s2,$s2,0x04
		 andi  $s1,$s1,0x02
		 andi  $s0,$s0,0x01
		 srl   $s4,$s4,4
		 srl   $s3,$s3,3
		 srl   $s2,$s2,2
		 srl   $s1,$s1,1
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0a40:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lh    $v0,0x00($t8)
		 xor   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 sh    $v0,0x00($t8)
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0a50:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 xor   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0a58:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 xor   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0a60:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 xor   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0a68:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 xor   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0a70:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 xor   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0a78:				#:
		 addiu $s7,$s7,2

		 lh    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 xor   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0a79:				#:
		 addiu $s7,$s7,2

		 lh    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 xor   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0a7c:				#:
		 lbu   $t0,0x44($s5)
		 andi  $t0,$t0,0x20       	 # Supervisor Mode ?
		 bne   $t0,$0,9f
		 addiu $s7,$s7,2        	 # Delay slot

		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,8

		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	9:
		 lhu   $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lbu   $v0,0x44($s5)
		 sll   $v0,$v0,4
		 or    $v0,$v0,$s4
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s3
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s2
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s1
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s0
		 xor   $v0,$v0,$t0
		 andi  $t9,$v0,0x2000    	 # User Mode ?
		 bne   $t9,$0,0f
		 or    $s2,$0,$v0       	 # Delay slot
		 lw    $s0,0x3C($s5)
		 lw    $s1,0x78($s5)
		 sw    $s0,0x40($s5)
		 sw    $s1,0x3C($s5)
	0:
		 srl   $s2,$s2,8
		 sb    $s2,0x44($s5)    	 # T, S & I
		 or    $s4,$0,$v0
		 or    $s3,$0,$v0
		 or    $s2,$0,$v0
		 or    $s1,$0,$v0
		 or    $s0,$0,$v0
		 andi  $s4,$s4,0x10
		 andi  $s3,$s3,0x08
		 andi  $s2,$s2,0x04
		 andi  $s1,$s1,0x02
		 andi  $s0,$s0,0x01
		 srl   $s4,$s4,4
		 srl   $s3,$s3,3
		 srl   $s2,$s2,2
		 srl   $s1,$s1,1
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lbu   $a3,0x54($s5)     	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
 # Check for Interrupt waiting

		 andi  $a3,$a3,0x07
		 beq   $a3,$0,3f
		 nop                    	 # Delay slot
		 m_j   interrupt
		 nop                    	 # Delay slot

	3:
		 lhu   $t8,0x00($s7)
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0a80:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lhu   $t0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t0,$t0,16
		 or    $t0,$t0,$t9
		 addiu $s7,$s7,4
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $v0,0x00($t8)
		 xor   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 sw    $v0,0x00($t8)
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0a90:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lhu   $t0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t0,$t0,16
		 or    $t0,$t0,$t9
		 addiu $s7,$s7,4
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 xor   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-28
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0a98:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lhu   $t0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t0,$t0,16
		 or    $t0,$t0,$t9
		 addiu $s7,$s7,4
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($t8)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 xor   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-28
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0aa0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lhu   $t0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t0,$t0,16
		 or    $t0,$t0,$t9
		 addiu $s7,$s7,4
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($t8)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 xor   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-30
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0aa8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lhu   $t0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t0,$t0,16
		 or    $t0,$t0,$t9
		 addiu $s7,$s7,4
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 xor   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-32
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0ab0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lhu   $t0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t0,$t0,16
		 or    $t0,$t0,$t9
		 addiu $s7,$s7,4
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 xor   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-34
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0ab8:				#:
		 addiu $s7,$s7,2

		 lhu   $t0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t0,$t0,16
		 or    $t0,$t0,$t9
		 addiu $s7,$s7,4
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 xor   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-32
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0ab9:				#:
		 addiu $s7,$s7,2

		 lhu   $t0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t0,$t0,16
		 or    $t0,$t0,$t9
		 addiu $s7,$s7,4
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 xor   $v0,$v0,$t0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-36
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0c00:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lb    $v0,0x00($t8)
		 subu  $t1,$v0,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$v0,$t0
		 xor   $t9,$t1,$v0
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t1
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0c10:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 subu  $t1,$v0,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$v0,$t0
		 xor   $t9,$t1,$v0
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t1
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0c18:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t8)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 subu  $t1,$v0,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$v0,$t0
		 xor   $t9,$t1,$v0
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t1
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0c1f:				#:
		 addiu $s7,$s7,2

		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 subu  $t1,$v0,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$v0,$t0
		 xor   $t9,$t1,$v0
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t1
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0c20:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 subu  $t1,$v0,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$v0,$t0
		 xor   $t9,$t1,$v0
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t1
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0c27:				#:
		 addiu $s7,$s7,2

		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 subu  $t1,$v0,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$v0,$t0
		 xor   $t9,$t1,$v0
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t1
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0c28:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 subu  $t1,$v0,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$v0,$t0
		 xor   $t9,$t1,$v0
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t1
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0c30:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 subu  $t1,$v0,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$v0,$t0
		 xor   $t9,$t1,$v0
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t1
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0c38:				#:
		 addiu $s7,$s7,2

		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 subu  $t1,$v0,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$v0,$t0
		 xor   $t9,$t1,$v0
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t1
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0c39:				#:
		 addiu $s7,$s7,2

		 lb    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 subu  $t1,$v0,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$v0,$t0
		 xor   $t9,$t1,$v0
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t1
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0c40:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lh    $v0,0x00($t8)
		 subu  $t1,$v0,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$v0,$t0
		 xor   $t9,$t1,$v0
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$t1
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0c50:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 subu  $t1,$v0,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$v0,$t0
		 xor   $t9,$t1,$v0
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$t1
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0c58:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 subu  $t1,$v0,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$v0,$t0
		 xor   $t9,$t1,$v0
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$t1
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0c60:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 subu  $t1,$v0,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$v0,$t0
		 xor   $t9,$t1,$v0
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$t1
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0c68:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 subu  $t1,$v0,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$v0,$t0
		 xor   $t9,$t1,$v0
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$t1
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0c70:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 subu  $t1,$v0,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$v0,$t0
		 xor   $t9,$t1,$v0
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$t1
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0c78:				#:
		 addiu $s7,$s7,2

		 lh    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 subu  $t1,$v0,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$v0,$t0
		 xor   $t9,$t1,$v0
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$t1
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0c79:				#:
		 addiu $s7,$s7,2

		 lh    $t0,0x00($s7)
		 addiu $s7,$s7,2
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 subu  $t1,$v0,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$v0,$t0
		 xor   $t9,$t1,$v0
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$t1
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0c80:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lhu   $t0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t0,$t0,16
		 or    $t0,$t0,$t9
		 addiu $s7,$s7,4
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $v0,0x00($t8)
		 subu  $t1,$v0,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$v0,$t0
		 xor   $t9,$t1,$v0
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t1,$0        	 # Set Sign
		 sltiu $s2,$t1,1         	 # Set Zero
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0c90:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lhu   $t0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t0,$t0,16
		 or    $t0,$t0,$t9
		 addiu $s7,$s7,4
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $t1,$v0,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$v0,$t0
		 xor   $t9,$t1,$v0
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t1,$0        	 # Set Sign
		 sltiu $s2,$t1,1         	 # Set Zero
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0c98:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lhu   $t0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t0,$t0,16
		 or    $t0,$t0,$t9
		 addiu $s7,$s7,4
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($t8)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $t1,$v0,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$v0,$t0
		 xor   $t9,$t1,$v0
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t1,$0        	 # Set Sign
		 sltiu $s2,$t1,1         	 # Set Zero
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0ca0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lhu   $t0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t0,$t0,16
		 or    $t0,$t0,$t9
		 addiu $s7,$s7,4
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($t8)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $t1,$v0,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$v0,$t0
		 xor   $t9,$t1,$v0
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t1,$0        	 # Set Sign
		 sltiu $s2,$t1,1         	 # Set Zero
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0ca8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lhu   $t0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t0,$t0,16
		 or    $t0,$t0,$t9
		 addiu $s7,$s7,4
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $t1,$v0,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$v0,$t0
		 xor   $t9,$t1,$v0
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t1,$0        	 # Set Sign
		 sltiu $s2,$t1,1         	 # Set Zero
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0cb0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lhu   $t0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t0,$t0,16
		 or    $t0,$t0,$t9
		 addiu $s7,$s7,4
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $t1,$v0,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$v0,$t0
		 xor   $t9,$t1,$v0
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t1,$0        	 # Set Sign
		 sltiu $s2,$t1,1         	 # Set Zero
		 addiu $t7,$t7,-26
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0cb8:				#:
		 addiu $s7,$s7,2

		 lhu   $t0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t0,$t0,16
		 or    $t0,$t0,$t9
		 addiu $s7,$s7,4
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $t1,$v0,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$v0,$t0
		 xor   $t9,$t1,$v0
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t1,$0        	 # Set Sign
		 sltiu $s2,$t1,1         	 # Set Zero
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0cb9:				#:
		 addiu $s7,$s7,2

		 lhu   $t0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t0,$t0,16
		 or    $t0,$t0,$t9
		 addiu $s7,$s7,4
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $t1,$v0,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$v0,$t0
		 xor   $t9,$t1,$v0
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t1,$0        	 # Set Sign
		 sltiu $s2,$t1,1         	 # Set Zero
		 addiu $t7,$t7,-28
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0100:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $s2,0x00($t8)
		 ori   $t2,$0,1
		 sllv  $t1,$t2,$s2
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $v0,0x00($t0)
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0110:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $s2,0x00($t8)
		 ori   $t2,$0,1
		 andi  $s2,$s2,0x07
		 sllv  $t1,$t2,$s2
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0118:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $s2,0x00($t8)
		 ori   $t2,$0,1
		 andi  $s2,$s2,0x07
		 sllv  $t1,$t2,$s2
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_011f:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $s2,0x00($t8)
		 ori   $t2,$0,1
		 andi  $s2,$s2,0x07
		 sllv  $t1,$t2,$s2
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0120:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $s2,0x00($t8)
		 ori   $t2,$0,1
		 andi  $s2,$s2,0x07
		 sllv  $t1,$t2,$s2
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0127:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $s2,0x00($t8)
		 ori   $t2,$0,1
		 andi  $s2,$s2,0x07
		 sllv  $t1,$t2,$s2
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0128:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $s2,0x00($t8)
		 ori   $t2,$0,1
		 andi  $s2,$s2,0x07
		 sllv  $t1,$t2,$s2
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0130:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $s2,0x00($t8)
		 ori   $t2,$0,1
		 andi  $s2,$s2,0x07
		 sllv  $t1,$t2,$s2
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0138:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $s2,0x00($t8)
		 ori   $t2,$0,1
		 andi  $s2,$s2,0x07
		 sllv  $t1,$t2,$s2
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0139:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $s2,0x00($t8)
		 ori   $t2,$0,1
		 andi  $s2,$s2,0x07
		 sllv  $t1,$t2,$s2
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_013a:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $s2,0x00($t8)
		 ori   $t2,$0,1
		 andi  $s2,$s2,0x07
		 sllv  $t1,$t2,$s2
		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xAC($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_013b:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $s2,0x00($t8)
		 ori   $t2,$0,1
		 andi  $s2,$s2,0x07
		 sllv  $t1,$t2,$s2
		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xAC($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_013c:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $s2,0x00($t8)
		 ori   $t2,$0,1
		 andi  $s2,$s2,0x07
		 sllv  $t1,$t2,$s2
		 lbu   $v0,0x00($s7)
		 addiu $s7,$s7,2
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0140:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $s2,0x00($t8)
		 ori   $t2,$0,1
		 sllv  $t1,$t2,$s2
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $v0,0x00($t0)
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 xor   $v0,$v0,$t1
		 sw    $v0,0x00($t0)
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0150:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $s2,0x00($t8)
		 ori   $t2,$0,1
		 andi  $s2,$s2,0x07
		 sllv  $t1,$t2,$s2
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 xor   $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0158:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $s2,0x00($t8)
		 ori   $t2,$0,1
		 andi  $s2,$s2,0x07
		 sllv  $t1,$t2,$s2
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 xor   $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_015f:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $s2,0x00($t8)
		 ori   $t2,$0,1
		 andi  $s2,$s2,0x07
		 sllv  $t1,$t2,$s2
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 xor   $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0160:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $s2,0x00($t8)
		 ori   $t2,$0,1
		 andi  $s2,$s2,0x07
		 sllv  $t1,$t2,$s2
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 xor   $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0167:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $s2,0x00($t8)
		 ori   $t2,$0,1
		 andi  $s2,$s2,0x07
		 sllv  $t1,$t2,$s2
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 xor   $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0168:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $s2,0x00($t8)
		 ori   $t2,$0,1
		 andi  $s2,$s2,0x07
		 sllv  $t1,$t2,$s2
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 xor   $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0170:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $s2,0x00($t8)
		 ori   $t2,$0,1
		 andi  $s2,$s2,0x07
		 sllv  $t1,$t2,$s2
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 xor   $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0178:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $s2,0x00($t8)
		 ori   $t2,$0,1
		 andi  $s2,$s2,0x07
		 sllv  $t1,$t2,$s2
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 xor   $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0179:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $s2,0x00($t8)
		 ori   $t2,$0,1
		 andi  $s2,$s2,0x07
		 sllv  $t1,$t2,$s2
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 xor   $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0180:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $s2,0x00($t8)
		 ori   $t2,$0,1
		 sllv  $t1,$t2,$s2
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $v0,0x00($t0)
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 nor   $t1,$0,$t1
		 and   $v0,$v0,$t1
		 sw    $v0,0x00($t0)
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0190:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $s2,0x00($t8)
		 ori   $t2,$0,1
		 andi  $s2,$s2,0x07
		 sllv  $t1,$t2,$s2
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 nor   $t1,$0,$t1
		 and   $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0198:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $s2,0x00($t8)
		 ori   $t2,$0,1
		 andi  $s2,$s2,0x07
		 sllv  $t1,$t2,$s2
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 nor   $t1,$0,$t1
		 and   $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_019f:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $s2,0x00($t8)
		 ori   $t2,$0,1
		 andi  $s2,$s2,0x07
		 sllv  $t1,$t2,$s2
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 nor   $t1,$0,$t1
		 and   $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_01a0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $s2,0x00($t8)
		 ori   $t2,$0,1
		 andi  $s2,$s2,0x07
		 sllv  $t1,$t2,$s2
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 nor   $t1,$0,$t1
		 and   $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_01a7:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $s2,0x00($t8)
		 ori   $t2,$0,1
		 andi  $s2,$s2,0x07
		 sllv  $t1,$t2,$s2
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 nor   $t1,$0,$t1
		 and   $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_01a8:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $s2,0x00($t8)
		 ori   $t2,$0,1
		 andi  $s2,$s2,0x07
		 sllv  $t1,$t2,$s2
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 nor   $t1,$0,$t1
		 and   $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_01b0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $s2,0x00($t8)
		 ori   $t2,$0,1
		 andi  $s2,$s2,0x07
		 sllv  $t1,$t2,$s2
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 nor   $t1,$0,$t1
		 and   $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_01b8:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $s2,0x00($t8)
		 ori   $t2,$0,1
		 andi  $s2,$s2,0x07
		 sllv  $t1,$t2,$s2
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 nor   $t1,$0,$t1
		 and   $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_01b9:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $s2,0x00($t8)
		 ori   $t2,$0,1
		 andi  $s2,$s2,0x07
		 sllv  $t1,$t2,$s2
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 nor   $t1,$0,$t1
		 and   $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_01c0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $s2,0x00($t8)
		 ori   $t2,$0,1
		 sllv  $t1,$t2,$s2
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $v0,0x00($t0)
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 or    $v0,$v0,$t1
		 sw    $v0,0x00($t0)
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_01d0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $s2,0x00($t8)
		 ori   $t2,$0,1
		 andi  $s2,$s2,0x07
		 sllv  $t1,$t2,$s2
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 or    $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_01d8:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $s2,0x00($t8)
		 ori   $t2,$0,1
		 andi  $s2,$s2,0x07
		 sllv  $t1,$t2,$s2
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 or    $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_01df:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $s2,0x00($t8)
		 ori   $t2,$0,1
		 andi  $s2,$s2,0x07
		 sllv  $t1,$t2,$s2
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 or    $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_01e0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $s2,0x00($t8)
		 ori   $t2,$0,1
		 andi  $s2,$s2,0x07
		 sllv  $t1,$t2,$s2
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 or    $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_01e7:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $s2,0x00($t8)
		 ori   $t2,$0,1
		 andi  $s2,$s2,0x07
		 sllv  $t1,$t2,$s2
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 or    $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_01e8:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $s2,0x00($t8)
		 ori   $t2,$0,1
		 andi  $s2,$s2,0x07
		 sllv  $t1,$t2,$s2
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 or    $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_01f0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $s2,0x00($t8)
		 ori   $t2,$0,1
		 andi  $s2,$s2,0x07
		 sllv  $t1,$t2,$s2
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 or    $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_01f8:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $s2,0x00($t8)
		 ori   $t2,$0,1
		 andi  $s2,$s2,0x07
		 sllv  $t1,$t2,$s2
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 or    $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_01f9:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $s2,0x00($t8)
		 ori   $t2,$0,1
		 andi  $s2,$s2,0x07
		 sllv  $t1,$t2,$s2
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 or    $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0108:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 sll   $t0,$v0,8
		 addiu $t6,$t6,2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t0,$t0,$v0
		 sh    $t0,0x00($t8)
		 addiu $t7,$t7,-28
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0148:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 sll   $t0,$v0,8
		 addiu $t6,$t6,2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t0,$t0,$v0
		 addiu $t6,$t6,2
		 sll   $t0,$v0,16
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 sll   $t1,$v0,8
		 addiu $t6,$t6,2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t0,$t0,$t1
		 or    $t0,$t0,$v0
		 sw    $t0,0x00($t8)
		 addiu $t7,$t7,-36
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0188:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t8,0x00($t8)
		 sll   $a1,$t8,8
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t6,$t6,2
		 or    $a1,$0,$t8
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-28
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_01c8:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t8,0x00($t8)
		 sll   $a1,$t8,24
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t6,$t6,2
		 sll   $a1,$t8,16
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t6,$t6,2
		 sll   $a1,$t8,8
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t6,$t6,2
		 or    $a1,$0,$t8
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-36
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0800:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 lhu   $s2,0x00($s7)
		 addiu $s7,$s7,2
		 ori   $t2,$0,1
		 sllv  $t1,$t2,$s2
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $v0,0x00($t0)
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0810:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 lhu   $s2,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s2,$s2,0x07
		 ori   $t2,$0,1
		 sllv  $t1,$t2,$s2
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0818:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 lhu   $s2,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s2,$s2,0x07
		 ori   $t2,$0,1
		 sllv  $t1,$t2,$s2
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_081f:				#:
		 addiu $s7,$s7,2

		 lhu   $s2,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s2,$s2,0x07
		 ori   $t2,$0,1
		 sllv  $t1,$t2,$s2
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0820:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 lhu   $s2,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s2,$s2,0x07
		 ori   $t2,$0,1
		 sllv  $t1,$t2,$s2
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0827:				#:
		 addiu $s7,$s7,2

		 lhu   $s2,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s2,$s2,0x07
		 ori   $t2,$0,1
		 sllv  $t1,$t2,$s2
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0828:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 lhu   $s2,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s2,$s2,0x07
		 ori   $t2,$0,1
		 sllv  $t1,$t2,$s2
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0830:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 lhu   $s2,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s2,$s2,0x07
		 ori   $t2,$0,1
		 sllv  $t1,$t2,$s2
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0838:				#:
		 addiu $s7,$s7,2

		 lhu   $s2,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s2,$s2,0x07
		 ori   $t2,$0,1
		 sllv  $t1,$t2,$s2
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0839:				#:
		 addiu $s7,$s7,2

		 lhu   $s2,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s2,$s2,0x07
		 ori   $t2,$0,1
		 sllv  $t1,$t2,$s2
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_083a:				#:
		 addiu $s7,$s7,2

		 lhu   $s2,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s2,$s2,0x07
		 ori   $t2,$0,1
		 sllv  $t1,$t2,$s2
		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xAC($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_083b:				#:
		 addiu $s7,$s7,2

		 lhu   $s2,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s2,$s2,0x07
		 ori   $t2,$0,1
		 sllv  $t1,$t2,$s2
		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xAC($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0840:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 lhu   $s2,0x00($s7)
		 addiu $s7,$s7,2
		 ori   $t2,$0,1
		 sllv  $t1,$t2,$s2
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $v0,0x00($t0)
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 xor   $v0,$v0,$t1
		 sw    $v0,0x00($t0)
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0850:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 lhu   $s2,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s2,$s2,0x07
		 ori   $t2,$0,1
		 sllv  $t1,$t2,$s2
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 xor   $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0858:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 lhu   $s2,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s2,$s2,0x07
		 ori   $t2,$0,1
		 sllv  $t1,$t2,$s2
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 xor   $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_085f:				#:
		 addiu $s7,$s7,2

		 lhu   $s2,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s2,$s2,0x07
		 ori   $t2,$0,1
		 sllv  $t1,$t2,$s2
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 xor   $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0860:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 lhu   $s2,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s2,$s2,0x07
		 ori   $t2,$0,1
		 sllv  $t1,$t2,$s2
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 xor   $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0867:				#:
		 addiu $s7,$s7,2

		 lhu   $s2,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s2,$s2,0x07
		 ori   $t2,$0,1
		 sllv  $t1,$t2,$s2
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 xor   $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0868:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 lhu   $s2,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s2,$s2,0x07
		 ori   $t2,$0,1
		 sllv  $t1,$t2,$s2
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 xor   $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0870:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 lhu   $s2,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s2,$s2,0x07
		 ori   $t2,$0,1
		 sllv  $t1,$t2,$s2
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 xor   $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0878:				#:
		 addiu $s7,$s7,2

		 lhu   $s2,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s2,$s2,0x07
		 ori   $t2,$0,1
		 sllv  $t1,$t2,$s2
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 xor   $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0879:				#:
		 addiu $s7,$s7,2

		 lhu   $s2,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s2,$s2,0x07
		 ori   $t2,$0,1
		 sllv  $t1,$t2,$s2
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 xor   $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0880:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 lhu   $s2,0x00($s7)
		 addiu $s7,$s7,2
		 ori   $t2,$0,1
		 sllv  $t1,$t2,$s2
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $v0,0x00($t0)
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 nor   $t1,$0,$t1
		 and   $v0,$v0,$t1
		 sw    $v0,0x00($t0)
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0890:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 lhu   $s2,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s2,$s2,0x07
		 ori   $t2,$0,1
		 sllv  $t1,$t2,$s2
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 nor   $t1,$0,$t1
		 and   $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_0898:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 lhu   $s2,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s2,$s2,0x07
		 ori   $t2,$0,1
		 sllv  $t1,$t2,$s2
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 nor   $t1,$0,$t1
		 and   $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_089f:				#:
		 addiu $s7,$s7,2

		 lhu   $s2,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s2,$s2,0x07
		 ori   $t2,$0,1
		 sllv  $t1,$t2,$s2
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 nor   $t1,$0,$t1
		 and   $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_08a0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 lhu   $s2,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s2,$s2,0x07
		 ori   $t2,$0,1
		 sllv  $t1,$t2,$s2
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 nor   $t1,$0,$t1
		 and   $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_08a7:				#:
		 addiu $s7,$s7,2

		 lhu   $s2,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s2,$s2,0x07
		 ori   $t2,$0,1
		 sllv  $t1,$t2,$s2
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 nor   $t1,$0,$t1
		 and   $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_08a8:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 lhu   $s2,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s2,$s2,0x07
		 ori   $t2,$0,1
		 sllv  $t1,$t2,$s2
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 nor   $t1,$0,$t1
		 and   $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_08b0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 lhu   $s2,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s2,$s2,0x07
		 ori   $t2,$0,1
		 sllv  $t1,$t2,$s2
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 nor   $t1,$0,$t1
		 and   $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_08b8:				#:
		 addiu $s7,$s7,2

		 lhu   $s2,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s2,$s2,0x07
		 ori   $t2,$0,1
		 sllv  $t1,$t2,$s2
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 nor   $t1,$0,$t1
		 and   $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_08b9:				#:
		 addiu $s7,$s7,2

		 lhu   $s2,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s2,$s2,0x07
		 ori   $t2,$0,1
		 sllv  $t1,$t2,$s2
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 nor   $t1,$0,$t1
		 and   $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_08c0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 lhu   $s2,0x00($s7)
		 addiu $s7,$s7,2
		 ori   $t2,$0,1
		 sllv  $t1,$t2,$s2
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $v0,0x00($t0)
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 or    $v0,$v0,$t1
		 sw    $v0,0x00($t0)
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_08d0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 lhu   $s2,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s2,$s2,0x07
		 ori   $t2,$0,1
		 sllv  $t1,$t2,$s2
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 or    $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_08d8:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 lhu   $s2,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s2,$s2,0x07
		 ori   $t2,$0,1
		 sllv  $t1,$t2,$s2
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 or    $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_08df:				#:
		 addiu $s7,$s7,2

		 lhu   $s2,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s2,$s2,0x07
		 ori   $t2,$0,1
		 sllv  $t1,$t2,$s2
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 or    $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_08e0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 lhu   $s2,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s2,$s2,0x07
		 ori   $t2,$0,1
		 sllv  $t1,$t2,$s2
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 or    $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_08e7:				#:
		 addiu $s7,$s7,2

		 lhu   $s2,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s2,$s2,0x07
		 ori   $t2,$0,1
		 sllv  $t1,$t2,$s2
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 or    $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_08e8:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 lhu   $s2,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s2,$s2,0x07
		 ori   $t2,$0,1
		 sllv  $t1,$t2,$s2
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 or    $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_08f0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 lhu   $s2,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s2,$s2,0x07
		 ori   $t2,$0,1
		 sllv  $t1,$t2,$s2
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 or    $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_08f8:				#:
		 addiu $s7,$s7,2

		 lhu   $s2,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s2,$s2,0x07
		 ori   $t2,$0,1
		 sllv  $t1,$t2,$s2
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 or    $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_08f9:				#:
		 addiu $s7,$s7,2

		 lhu   $s2,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s2,$s2,0x07
		 ori   $t2,$0,1
		 sllv  $t1,$t2,$s2
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 srlv  $s2,$t2,$s2
		 xori  $s2,$s2,1        	 # Set Zero Flag
		 or    $v0,$v0,$t1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_41d0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 sw    $t6,0x20($t8)
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_41e8:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 sw    $t6,0x20($t8)
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_41f0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 sw    $t6,0x20($t8)
		 addiu $t7,$t7,-26
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_41f8:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 sw    $t6,0x20($t8)
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_41f9:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 sw    $t6,0x20($t8)
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_41fa:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 sw    $t6,0x20($t8)
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_41fb:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 sw    $t6,0x20($t8)
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4850:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $a0,0x3C($s5)     	 # Push onto Stack
		 addiu $a0,$a0,-4
		 sw    $a0,0x3C($s5)
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4868:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $a0,0x3C($s5)     	 # Push onto Stack
		 addiu $a0,$a0,-4
		 sw    $a0,0x3C($s5)
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-28
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4870:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $a0,0x3C($s5)     	 # Push onto Stack
		 addiu $a0,$a0,-4
		 sw    $a0,0x3C($s5)
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-34
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4878:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $a0,0x3C($s5)     	 # Push onto Stack
		 addiu $a0,$a0,-4
		 sw    $a0,0x3C($s5)
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-28
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4879:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $a0,0x3C($s5)     	 # Push onto Stack
		 addiu $a0,$a0,-4
		 sw    $a0,0x3C($s5)
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-32
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_487a:				#:
		 addiu $s7,$s7,2

		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $a0,0x3C($s5)     	 # Push onto Stack
		 addiu $a0,$a0,-4
		 sw    $a0,0x3C($s5)
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-28
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_487b:				#:
		 addiu $s7,$s7,2

		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $a0,0x3C($s5)     	 # Push onto Stack
		 addiu $a0,$a0,-4
		 sw    $a0,0x3C($s5)
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-30
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_40c0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lbu   $v0,0x44($s5)
		 sll   $v0,$v0,4
		 or    $v0,$v0,$s4
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s3
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s2
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s1
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s0
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 sh    $v0,0x00($t8)
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_40d0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lbu   $v0,0x44($s5)
		 sll   $v0,$v0,4
		 or    $v0,$v0,$s4
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s3
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s2
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s1
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s0
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_40d8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lbu   $v0,0x44($s5)
		 sll   $v0,$v0,4
		 or    $v0,$v0,$s4
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s3
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s2
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s1
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s0
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_40e0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lbu   $v0,0x44($s5)
		 sll   $v0,$v0,4
		 or    $v0,$v0,$s4
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s3
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s2
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s1
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s0
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_40e8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lbu   $v0,0x44($s5)
		 sll   $v0,$v0,4
		 or    $v0,$v0,$s4
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s3
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s2
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s1
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s0
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_40f0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lbu   $v0,0x44($s5)
		 sll   $v0,$v0,4
		 or    $v0,$v0,$s4
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s3
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s2
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s1
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s0
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_40f8:				#:
		 addiu $s7,$s7,2

		 lbu   $v0,0x44($s5)
		 sll   $v0,$v0,4
		 or    $v0,$v0,$s4
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s3
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s2
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s1
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s0
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_40f9:				#:
		 addiu $s7,$s7,2

		 lbu   $v0,0x44($s5)
		 sll   $v0,$v0,4
		 or    $v0,$v0,$s4
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s3
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s2
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s1
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s0
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_42c0:				#:
		 addiu $s7,$s7,2

		 lw    $t9,0x84($s5)
		 bne   $t9,$0,0f
		 andi  $t8,$t8,0x07     	 # Delay slot
		 m_j   ILLEGAL
	0:
		 nop                    	 # Delay slot

		 or    $v0,$0,$s4
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s3
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s2
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s1
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s0
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 sh    $v0,0x00($t8)
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_42d0:				#:
		 addiu $s7,$s7,2

		 lw    $t9,0x84($s5)
		 bne   $t9,$0,0f
		 andi  $t8,$t8,0x07     	 # Delay slot
		 m_j   ILLEGAL
	0:
		 nop                    	 # Delay slot

		 or    $v0,$0,$s4
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s3
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s2
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s1
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s0
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_42d8:				#:
		 addiu $s7,$s7,2

		 lw    $t9,0x84($s5)
		 bne   $t9,$0,0f
		 andi  $t8,$t8,0x07     	 # Delay slot
		 m_j   ILLEGAL
	0:
		 nop                    	 # Delay slot

		 or    $v0,$0,$s4
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s3
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s2
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s1
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s0
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_42e0:				#:
		 addiu $s7,$s7,2

		 lw    $t9,0x84($s5)
		 bne   $t9,$0,0f
		 andi  $t8,$t8,0x07     	 # Delay slot
		 m_j   ILLEGAL
	0:
		 nop                    	 # Delay slot

		 or    $v0,$0,$s4
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s3
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s2
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s1
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s0
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_42e8:				#:
		 addiu $s7,$s7,2

		 lw    $t9,0x84($s5)
		 bne   $t9,$0,0f
		 andi  $t8,$t8,0x07     	 # Delay slot
		 m_j   ILLEGAL
	0:
		 nop                    	 # Delay slot

		 or    $v0,$0,$s4
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s3
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s2
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s1
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s0
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_42f0:				#:
		 addiu $s7,$s7,2

		 lw    $t9,0x84($s5)
		 bne   $t9,$0,0f
		 andi  $t8,$t8,0x07     	 # Delay slot
		 m_j   ILLEGAL
	0:
		 nop                    	 # Delay slot

		 or    $v0,$0,$s4
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s3
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s2
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s1
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s0
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_42f8:				#:
		 addiu $s7,$s7,2

		 lw    $t9,0x84($s5)
		 bne   $t9,$0,0f
		 nop                    	 # Delay slot
		 m_j   ILLEGAL
	0:
		 nop                    	 # Delay slot

		 or    $v0,$0,$s4
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s3
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s2
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s1
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s0
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_42f9:				#:
		 addiu $s7,$s7,2

		 lw    $t9,0x84($s5)
		 bne   $t9,$0,0f
		 nop                    	 # Delay slot
		 m_j   ILLEGAL
	0:
		 nop                    	 # Delay slot

		 or    $v0,$0,$s4
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s3
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s2
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s1
		 sll   $v0,$v0,1
		 or    $v0,$v0,$s0
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_44c0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lhu   $v0,0x00($t8)
		 or    $s4,$0,$v0
		 or    $s3,$0,$v0
		 or    $s2,$0,$v0
		 or    $s1,$0,$v0
		 or    $s0,$0,$v0
		 andi  $s4,$s4,0x10
		 andi  $s3,$s3,0x08
		 andi  $s2,$s2,0x04
		 andi  $s1,$s1,0x02
		 andi  $s0,$s0,0x01
		 srl   $s4,$s4,4
		 srl   $s3,$s3,3
		 srl   $s2,$s2,2
		 srl   $s1,$s1,1
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_44d0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $s4,$0,$v0
		 or    $s3,$0,$v0
		 or    $s2,$0,$v0
		 or    $s1,$0,$v0
		 or    $s0,$0,$v0
		 andi  $s4,$s4,0x10
		 andi  $s3,$s3,0x08
		 andi  $s2,$s2,0x04
		 andi  $s1,$s1,0x02
		 andi  $s0,$s0,0x01
		 srl   $s4,$s4,4
		 srl   $s3,$s3,3
		 srl   $s2,$s2,2
		 srl   $s1,$s1,1
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_44d8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $s4,$0,$v0
		 or    $s3,$0,$v0
		 or    $s2,$0,$v0
		 or    $s1,$0,$v0
		 or    $s0,$0,$v0
		 andi  $s4,$s4,0x10
		 andi  $s3,$s3,0x08
		 andi  $s2,$s2,0x04
		 andi  $s1,$s1,0x02
		 andi  $s0,$s0,0x01
		 srl   $s4,$s4,4
		 srl   $s3,$s3,3
		 srl   $s2,$s2,2
		 srl   $s1,$s1,1
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_44e0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $s4,$0,$v0
		 or    $s3,$0,$v0
		 or    $s2,$0,$v0
		 or    $s1,$0,$v0
		 or    $s0,$0,$v0
		 andi  $s4,$s4,0x10
		 andi  $s3,$s3,0x08
		 andi  $s2,$s2,0x04
		 andi  $s1,$s1,0x02
		 andi  $s0,$s0,0x01
		 srl   $s4,$s4,4
		 srl   $s3,$s3,3
		 srl   $s2,$s2,2
		 srl   $s1,$s1,1
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_44e8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $s4,$0,$v0
		 or    $s3,$0,$v0
		 or    $s2,$0,$v0
		 or    $s1,$0,$v0
		 or    $s0,$0,$v0
		 andi  $s4,$s4,0x10
		 andi  $s3,$s3,0x08
		 andi  $s2,$s2,0x04
		 andi  $s1,$s1,0x02
		 andi  $s0,$s0,0x01
		 srl   $s4,$s4,4
		 srl   $s3,$s3,3
		 srl   $s2,$s2,2
		 srl   $s1,$s1,1
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_44f0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $s4,$0,$v0
		 or    $s3,$0,$v0
		 or    $s2,$0,$v0
		 or    $s1,$0,$v0
		 or    $s0,$0,$v0
		 andi  $s4,$s4,0x10
		 andi  $s3,$s3,0x08
		 andi  $s2,$s2,0x04
		 andi  $s1,$s1,0x02
		 andi  $s0,$s0,0x01
		 srl   $s4,$s4,4
		 srl   $s3,$s3,3
		 srl   $s2,$s2,2
		 srl   $s1,$s1,1
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_44f8:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $s4,$0,$v0
		 or    $s3,$0,$v0
		 or    $s2,$0,$v0
		 or    $s1,$0,$v0
		 or    $s0,$0,$v0
		 andi  $s4,$s4,0x10
		 andi  $s3,$s3,0x08
		 andi  $s2,$s2,0x04
		 andi  $s1,$s1,0x02
		 andi  $s0,$s0,0x01
		 srl   $s4,$s4,4
		 srl   $s3,$s3,3
		 srl   $s2,$s2,2
		 srl   $s1,$s1,1
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_44f9:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $s4,$0,$v0
		 or    $s3,$0,$v0
		 or    $s2,$0,$v0
		 or    $s1,$0,$v0
		 or    $s0,$0,$v0
		 andi  $s4,$s4,0x10
		 andi  $s3,$s3,0x08
		 andi  $s2,$s2,0x04
		 andi  $s1,$s1,0x02
		 andi  $s0,$s0,0x01
		 srl   $s4,$s4,4
		 srl   $s3,$s3,3
		 srl   $s2,$s2,2
		 srl   $s1,$s1,1
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_44fa:				#:
		 addiu $s7,$s7,2

		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $s4,$0,$v0
		 or    $s3,$0,$v0
		 or    $s2,$0,$v0
		 or    $s1,$0,$v0
		 or    $s0,$0,$v0
		 andi  $s4,$s4,0x10
		 andi  $s3,$s3,0x08
		 andi  $s2,$s2,0x04
		 andi  $s1,$s1,0x02
		 andi  $s0,$s0,0x01
		 srl   $s4,$s4,4
		 srl   $s3,$s3,3
		 srl   $s2,$s2,2
		 srl   $s1,$s1,1
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_44fb:				#:
		 addiu $s7,$s7,2

		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $s4,$0,$v0
		 or    $s3,$0,$v0
		 or    $s2,$0,$v0
		 or    $s1,$0,$v0
		 or    $s0,$0,$v0
		 andi  $s4,$s4,0x10
		 andi  $s3,$s3,0x08
		 andi  $s2,$s2,0x04
		 andi  $s1,$s1,0x02
		 andi  $s0,$s0,0x01
		 srl   $s4,$s4,4
		 srl   $s3,$s3,3
		 srl   $s2,$s2,2
		 srl   $s1,$s1,1
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_44fc:				#:
		 addiu $s7,$s7,2

		 lhu   $v0,0x00($s7)
		 addiu $s7,$s7,2
		 or    $s4,$0,$v0
		 or    $s3,$0,$v0
		 or    $s2,$0,$v0
		 or    $s1,$0,$v0
		 or    $s0,$0,$v0
		 andi  $s4,$s4,0x10
		 andi  $s3,$s3,0x08
		 andi  $s2,$s2,0x04
		 andi  $s1,$s1,0x02
		 andi  $s0,$s0,0x01
		 srl   $s4,$s4,4
		 srl   $s3,$s3,3
		 srl   $s2,$s2,2
		 srl   $s1,$s1,1
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_46c0:				#:
		 lbu   $t0,0x44($s5)
		 andi  $t0,$t0,0x20       	 # Supervisor Mode ?
		 beq   $t0,$0,9f
		 addiu $s7,$s7,2        	 # Delay slot

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lhu   $v0,0x00($t8)
		 andi  $t9,$v0,0x2000    	 # User Mode ?
		 bne   $t9,$0,0f
		 or    $s2,$0,$v0       	 # Delay slot
		 lw    $s0,0x3C($s5)
		 lw    $s1,0x78($s5)
		 sw    $s0,0x40($s5)
		 sw    $s1,0x3C($s5)
	0:
		 srl   $s2,$s2,8
		 sb    $s2,0x44($s5)    	 # T, S & I
		 or    $s4,$0,$v0
		 or    $s3,$0,$v0
		 or    $s2,$0,$v0
		 or    $s1,$0,$v0
		 or    $s0,$0,$v0
		 andi  $s4,$s4,0x10
		 andi  $s3,$s3,0x08
		 andi  $s2,$s2,0x04
		 andi  $s1,$s1,0x02
		 andi  $s0,$s0,0x01
		 srl   $s4,$s4,4
		 srl   $s3,$s3,3
		 srl   $s2,$s2,2
		 srl   $s1,$s1,1
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lbu   $a3,0x54($s5)     	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
 # Check for Interrupt waiting

		 andi  $a3,$a3,0x07
		 beq   $a3,$0,3f
		 nop                    	 # Delay slot
		 m_j   interrupt
		 nop                    	 # Delay slot

	3:
		 lhu   $t8,0x00($s7)
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	9:
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,8

		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lbu   $a3,0x54($s5)     	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
 # Check for Interrupt waiting

		 andi  $a3,$a3,0x07
		 beq   $a3,$0,3f
		 nop                    	 # Delay slot
		 m_j   interrupt
		 nop                    	 # Delay slot

	3:
		 lhu   $t8,0x00($s7)
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_46d0:				#:
		 lbu   $t0,0x44($s5)
		 andi  $t0,$t0,0x20       	 # Supervisor Mode ?
		 beq   $t0,$0,9f
		 addiu $s7,$s7,2        	 # Delay slot

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 andi  $t9,$v0,0x2000    	 # User Mode ?
		 bne   $t9,$0,0f
		 or    $s2,$0,$v0       	 # Delay slot
		 lw    $s0,0x3C($s5)
		 lw    $s1,0x78($s5)
		 sw    $s0,0x40($s5)
		 sw    $s1,0x3C($s5)
	0:
		 srl   $s2,$s2,8
		 sb    $s2,0x44($s5)    	 # T, S & I
		 or    $s4,$0,$v0
		 or    $s3,$0,$v0
		 or    $s2,$0,$v0
		 or    $s1,$0,$v0
		 or    $s0,$0,$v0
		 andi  $s4,$s4,0x10
		 andi  $s3,$s3,0x08
		 andi  $s2,$s2,0x04
		 andi  $s1,$s1,0x02
		 andi  $s0,$s0,0x01
		 srl   $s4,$s4,4
		 srl   $s3,$s3,3
		 srl   $s2,$s2,2
		 srl   $s1,$s1,1
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lbu   $a3,0x54($s5)     	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
 # Check for Interrupt waiting

		 andi  $a3,$a3,0x07
		 beq   $a3,$0,3f
		 nop                    	 # Delay slot
		 m_j   interrupt
		 nop                    	 # Delay slot

	3:
		 lhu   $t8,0x00($s7)
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	9:
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,8

		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lbu   $a3,0x54($s5)     	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
 # Check for Interrupt waiting

		 andi  $a3,$a3,0x07
		 beq   $a3,$0,3f
		 nop                    	 # Delay slot
		 m_j   interrupt
		 nop                    	 # Delay slot

	3:
		 lhu   $t8,0x00($s7)
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_46d8:				#:
		 lbu   $t0,0x44($s5)
		 andi  $t0,$t0,0x20       	 # Supervisor Mode ?
		 beq   $t0,$0,9f
		 addiu $s7,$s7,2        	 # Delay slot

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 andi  $t9,$v0,0x2000    	 # User Mode ?
		 bne   $t9,$0,0f
		 or    $s2,$0,$v0       	 # Delay slot
		 lw    $s0,0x3C($s5)
		 lw    $s1,0x78($s5)
		 sw    $s0,0x40($s5)
		 sw    $s1,0x3C($s5)
	0:
		 srl   $s2,$s2,8
		 sb    $s2,0x44($s5)    	 # T, S & I
		 or    $s4,$0,$v0
		 or    $s3,$0,$v0
		 or    $s2,$0,$v0
		 or    $s1,$0,$v0
		 or    $s0,$0,$v0
		 andi  $s4,$s4,0x10
		 andi  $s3,$s3,0x08
		 andi  $s2,$s2,0x04
		 andi  $s1,$s1,0x02
		 andi  $s0,$s0,0x01
		 srl   $s4,$s4,4
		 srl   $s3,$s3,3
		 srl   $s2,$s2,2
		 srl   $s1,$s1,1
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lbu   $a3,0x54($s5)     	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
 # Check for Interrupt waiting

		 andi  $a3,$a3,0x07
		 beq   $a3,$0,3f
		 nop                    	 # Delay slot
		 m_j   interrupt
		 nop                    	 # Delay slot

	3:
		 lhu   $t8,0x00($s7)
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	9:
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,8

		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lbu   $a3,0x54($s5)     	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
 # Check for Interrupt waiting

		 andi  $a3,$a3,0x07
		 beq   $a3,$0,3f
		 nop                    	 # Delay slot
		 m_j   interrupt
		 nop                    	 # Delay slot

	3:
		 lhu   $t8,0x00($s7)
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_46e0:				#:
		 lbu   $t0,0x44($s5)
		 andi  $t0,$t0,0x20       	 # Supervisor Mode ?
		 beq   $t0,$0,9f
		 addiu $s7,$s7,2        	 # Delay slot

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 andi  $t9,$v0,0x2000    	 # User Mode ?
		 bne   $t9,$0,0f
		 or    $s2,$0,$v0       	 # Delay slot
		 lw    $s0,0x3C($s5)
		 lw    $s1,0x78($s5)
		 sw    $s0,0x40($s5)
		 sw    $s1,0x3C($s5)
	0:
		 srl   $s2,$s2,8
		 sb    $s2,0x44($s5)    	 # T, S & I
		 or    $s4,$0,$v0
		 or    $s3,$0,$v0
		 or    $s2,$0,$v0
		 or    $s1,$0,$v0
		 or    $s0,$0,$v0
		 andi  $s4,$s4,0x10
		 andi  $s3,$s3,0x08
		 andi  $s2,$s2,0x04
		 andi  $s1,$s1,0x02
		 andi  $s0,$s0,0x01
		 srl   $s4,$s4,4
		 srl   $s3,$s3,3
		 srl   $s2,$s2,2
		 srl   $s1,$s1,1
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lbu   $a3,0x54($s5)     	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
 # Check for Interrupt waiting

		 andi  $a3,$a3,0x07
		 beq   $a3,$0,3f
		 nop                    	 # Delay slot
		 m_j   interrupt
		 nop                    	 # Delay slot

	3:
		 lhu   $t8,0x00($s7)
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	9:
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,8

		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lbu   $a3,0x54($s5)     	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
 # Check for Interrupt waiting

		 andi  $a3,$a3,0x07
		 beq   $a3,$0,3f
		 nop                    	 # Delay slot
		 m_j   interrupt
		 nop                    	 # Delay slot

	3:
		 lhu   $t8,0x00($s7)
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_46e8:				#:
		 lbu   $t0,0x44($s5)
		 andi  $t0,$t0,0x20       	 # Supervisor Mode ?
		 beq   $t0,$0,9f
		 addiu $s7,$s7,2        	 # Delay slot

		 andi  $t8,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 andi  $t9,$v0,0x2000    	 # User Mode ?
		 bne   $t9,$0,0f
		 or    $s2,$0,$v0       	 # Delay slot
		 lw    $s0,0x3C($s5)
		 lw    $s1,0x78($s5)
		 sw    $s0,0x40($s5)
		 sw    $s1,0x3C($s5)
	0:
		 srl   $s2,$s2,8
		 sb    $s2,0x44($s5)    	 # T, S & I
		 or    $s4,$0,$v0
		 or    $s3,$0,$v0
		 or    $s2,$0,$v0
		 or    $s1,$0,$v0
		 or    $s0,$0,$v0
		 andi  $s4,$s4,0x10
		 andi  $s3,$s3,0x08
		 andi  $s2,$s2,0x04
		 andi  $s1,$s1,0x02
		 andi  $s0,$s0,0x01
		 srl   $s4,$s4,4
		 srl   $s3,$s3,3
		 srl   $s2,$s2,2
		 srl   $s1,$s1,1
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lbu   $a3,0x54($s5)     	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
 # Check for Interrupt waiting

		 andi  $a3,$a3,0x07
		 beq   $a3,$0,3f
		 nop                    	 # Delay slot
		 m_j   interrupt
		 nop                    	 # Delay slot

	3:
		 lhu   $t8,0x00($s7)
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	9:
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,8

		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lbu   $a3,0x54($s5)     	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
 # Check for Interrupt waiting

		 andi  $a3,$a3,0x07
		 beq   $a3,$0,3f
		 nop                    	 # Delay slot
		 m_j   interrupt
		 nop                    	 # Delay slot

	3:
		 lhu   $t8,0x00($s7)
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_46f0:				#:
		 lbu   $t0,0x44($s5)
		 andi  $t0,$t0,0x20       	 # Supervisor Mode ?
		 beq   $t0,$0,9f
		 addiu $s7,$s7,2        	 # Delay slot

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 andi  $t9,$v0,0x2000    	 # User Mode ?
		 bne   $t9,$0,0f
		 or    $s2,$0,$v0       	 # Delay slot
		 lw    $s0,0x3C($s5)
		 lw    $s1,0x78($s5)
		 sw    $s0,0x40($s5)
		 sw    $s1,0x3C($s5)
	0:
		 srl   $s2,$s2,8
		 sb    $s2,0x44($s5)    	 # T, S & I
		 or    $s4,$0,$v0
		 or    $s3,$0,$v0
		 or    $s2,$0,$v0
		 or    $s1,$0,$v0
		 or    $s0,$0,$v0
		 andi  $s4,$s4,0x10
		 andi  $s3,$s3,0x08
		 andi  $s2,$s2,0x04
		 andi  $s1,$s1,0x02
		 andi  $s0,$s0,0x01
		 srl   $s4,$s4,4
		 srl   $s3,$s3,3
		 srl   $s2,$s2,2
		 srl   $s1,$s1,1
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lbu   $a3,0x54($s5)     	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
 # Check for Interrupt waiting

		 andi  $a3,$a3,0x07
		 beq   $a3,$0,3f
		 nop                    	 # Delay slot
		 m_j   interrupt
		 nop                    	 # Delay slot

	3:
		 lhu   $t8,0x00($s7)
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	9:
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,8

		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lbu   $a3,0x54($s5)     	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
 # Check for Interrupt waiting

		 andi  $a3,$a3,0x07
		 beq   $a3,$0,3f
		 nop                    	 # Delay slot
		 m_j   interrupt
		 nop                    	 # Delay slot

	3:
		 lhu   $t8,0x00($s7)
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_46f8:				#:
		 lbu   $t0,0x44($s5)
		 andi  $t0,$t0,0x20       	 # Supervisor Mode ?
		 beq   $t0,$0,9f
		 addiu $s7,$s7,2        	 # Delay slot

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 andi  $t9,$v0,0x2000    	 # User Mode ?
		 bne   $t9,$0,0f
		 or    $s2,$0,$v0       	 # Delay slot
		 lw    $s0,0x3C($s5)
		 lw    $s1,0x78($s5)
		 sw    $s0,0x40($s5)
		 sw    $s1,0x3C($s5)
	0:
		 srl   $s2,$s2,8
		 sb    $s2,0x44($s5)    	 # T, S & I
		 or    $s4,$0,$v0
		 or    $s3,$0,$v0
		 or    $s2,$0,$v0
		 or    $s1,$0,$v0
		 or    $s0,$0,$v0
		 andi  $s4,$s4,0x10
		 andi  $s3,$s3,0x08
		 andi  $s2,$s2,0x04
		 andi  $s1,$s1,0x02
		 andi  $s0,$s0,0x01
		 srl   $s4,$s4,4
		 srl   $s3,$s3,3
		 srl   $s2,$s2,2
		 srl   $s1,$s1,1
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lbu   $a3,0x54($s5)     	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
 # Check for Interrupt waiting

		 andi  $a3,$a3,0x07
		 beq   $a3,$0,3f
		 nop                    	 # Delay slot
		 m_j   interrupt
		 nop                    	 # Delay slot

	3:
		 lhu   $t8,0x00($s7)
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	9:
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,8

		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lbu   $a3,0x54($s5)     	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
 # Check for Interrupt waiting

		 andi  $a3,$a3,0x07
		 beq   $a3,$0,3f
		 nop                    	 # Delay slot
		 m_j   interrupt
		 nop                    	 # Delay slot

	3:
		 lhu   $t8,0x00($s7)
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_46f9:				#:
		 lbu   $t0,0x44($s5)
		 andi  $t0,$t0,0x20       	 # Supervisor Mode ?
		 beq   $t0,$0,9f
		 addiu $s7,$s7,2        	 # Delay slot

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 andi  $t9,$v0,0x2000    	 # User Mode ?
		 bne   $t9,$0,0f
		 or    $s2,$0,$v0       	 # Delay slot
		 lw    $s0,0x3C($s5)
		 lw    $s1,0x78($s5)
		 sw    $s0,0x40($s5)
		 sw    $s1,0x3C($s5)
	0:
		 srl   $s2,$s2,8
		 sb    $s2,0x44($s5)    	 # T, S & I
		 or    $s4,$0,$v0
		 or    $s3,$0,$v0
		 or    $s2,$0,$v0
		 or    $s1,$0,$v0
		 or    $s0,$0,$v0
		 andi  $s4,$s4,0x10
		 andi  $s3,$s3,0x08
		 andi  $s2,$s2,0x04
		 andi  $s1,$s1,0x02
		 andi  $s0,$s0,0x01
		 srl   $s4,$s4,4
		 srl   $s3,$s3,3
		 srl   $s2,$s2,2
		 srl   $s1,$s1,1
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lbu   $a3,0x54($s5)     	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
 # Check for Interrupt waiting

		 andi  $a3,$a3,0x07
		 beq   $a3,$0,3f
		 nop                    	 # Delay slot
		 m_j   interrupt
		 nop                    	 # Delay slot

	3:
		 lhu   $t8,0x00($s7)
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	9:
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,8

		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lbu   $a3,0x54($s5)     	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
 # Check for Interrupt waiting

		 andi  $a3,$a3,0x07
		 beq   $a3,$0,3f
		 nop                    	 # Delay slot
		 m_j   interrupt
		 nop                    	 # Delay slot

	3:
		 lhu   $t8,0x00($s7)
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_46fa:				#:
		 lbu   $t0,0x44($s5)
		 andi  $t0,$t0,0x20       	 # Supervisor Mode ?
		 beq   $t0,$0,9f
		 addiu $s7,$s7,2        	 # Delay slot

		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 andi  $t9,$v0,0x2000    	 # User Mode ?
		 bne   $t9,$0,0f
		 or    $s2,$0,$v0       	 # Delay slot
		 lw    $s0,0x3C($s5)
		 lw    $s1,0x78($s5)
		 sw    $s0,0x40($s5)
		 sw    $s1,0x3C($s5)
	0:
		 srl   $s2,$s2,8
		 sb    $s2,0x44($s5)    	 # T, S & I
		 or    $s4,$0,$v0
		 or    $s3,$0,$v0
		 or    $s2,$0,$v0
		 or    $s1,$0,$v0
		 or    $s0,$0,$v0
		 andi  $s4,$s4,0x10
		 andi  $s3,$s3,0x08
		 andi  $s2,$s2,0x04
		 andi  $s1,$s1,0x02
		 andi  $s0,$s0,0x01
		 srl   $s4,$s4,4
		 srl   $s3,$s3,3
		 srl   $s2,$s2,2
		 srl   $s1,$s1,1
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lbu   $a3,0x54($s5)     	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
 # Check for Interrupt waiting

		 andi  $a3,$a3,0x07
		 beq   $a3,$0,3f
		 nop                    	 # Delay slot
		 m_j   interrupt
		 nop                    	 # Delay slot

	3:
		 lhu   $t8,0x00($s7)
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	9:
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,8

		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lbu   $a3,0x54($s5)     	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
 # Check for Interrupt waiting

		 andi  $a3,$a3,0x07
		 beq   $a3,$0,3f
		 nop                    	 # Delay slot
		 m_j   interrupt
		 nop                    	 # Delay slot

	3:
		 lhu   $t8,0x00($s7)
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_46fb:				#:
		 lbu   $t0,0x44($s5)
		 andi  $t0,$t0,0x20       	 # Supervisor Mode ?
		 beq   $t0,$0,9f
		 addiu $s7,$s7,2        	 # Delay slot

		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 andi  $t9,$v0,0x2000    	 # User Mode ?
		 bne   $t9,$0,0f
		 or    $s2,$0,$v0       	 # Delay slot
		 lw    $s0,0x3C($s5)
		 lw    $s1,0x78($s5)
		 sw    $s0,0x40($s5)
		 sw    $s1,0x3C($s5)
	0:
		 srl   $s2,$s2,8
		 sb    $s2,0x44($s5)    	 # T, S & I
		 or    $s4,$0,$v0
		 or    $s3,$0,$v0
		 or    $s2,$0,$v0
		 or    $s1,$0,$v0
		 or    $s0,$0,$v0
		 andi  $s4,$s4,0x10
		 andi  $s3,$s3,0x08
		 andi  $s2,$s2,0x04
		 andi  $s1,$s1,0x02
		 andi  $s0,$s0,0x01
		 srl   $s4,$s4,4
		 srl   $s3,$s3,3
		 srl   $s2,$s2,2
		 srl   $s1,$s1,1
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lbu   $a3,0x54($s5)     	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
 # Check for Interrupt waiting

		 andi  $a3,$a3,0x07
		 beq   $a3,$0,3f
		 nop                    	 # Delay slot
		 m_j   interrupt
		 nop                    	 # Delay slot

	3:
		 lhu   $t8,0x00($s7)
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	9:
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,8

		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lbu   $a3,0x54($s5)     	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
 # Check for Interrupt waiting

		 andi  $a3,$a3,0x07
		 beq   $a3,$0,3f
		 nop                    	 # Delay slot
		 m_j   interrupt
		 nop                    	 # Delay slot

	3:
		 lhu   $t8,0x00($s7)
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_46fc:				#:
		 lbu   $t0,0x44($s5)
		 andi  $t0,$t0,0x20       	 # Supervisor Mode ?
		 beq   $t0,$0,9f
		 addiu $s7,$s7,2        	 # Delay slot

		 lhu   $v0,0x00($s7)
		 addiu $s7,$s7,2
		 andi  $t9,$v0,0x2000    	 # User Mode ?
		 bne   $t9,$0,0f
		 or    $s2,$0,$v0       	 # Delay slot
		 lw    $s0,0x3C($s5)
		 lw    $s1,0x78($s5)
		 sw    $s0,0x40($s5)
		 sw    $s1,0x3C($s5)
	0:
		 srl   $s2,$s2,8
		 sb    $s2,0x44($s5)    	 # T, S & I
		 or    $s4,$0,$v0
		 or    $s3,$0,$v0
		 or    $s2,$0,$v0
		 or    $s1,$0,$v0
		 or    $s0,$0,$v0
		 andi  $s4,$s4,0x10
		 andi  $s3,$s3,0x08
		 andi  $s2,$s2,0x04
		 andi  $s1,$s1,0x02
		 andi  $s0,$s0,0x01
		 srl   $s4,$s4,4
		 srl   $s3,$s3,3
		 srl   $s2,$s2,2
		 srl   $s1,$s1,1
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lbu   $a3,0x54($s5)     	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
 # Check for Interrupt waiting

		 andi  $a3,$a3,0x07
		 beq   $a3,$0,3f
		 nop                    	 # Delay slot
		 m_j   interrupt
		 nop                    	 # Delay slot

	3:
		 lhu   $t8,0x00($s7)
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	9:
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,8

		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lbu   $a3,0x54($s5)     	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
 # Check for Interrupt waiting

		 andi  $a3,$a3,0x07
		 beq   $a3,$0,3f
		 nop                    	 # Delay slot
		 m_j   interrupt
		 nop                    	 # Delay slot

	3:
		 lhu   $t8,0x00($s7)
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5000:				#:
		 addiu $s7,$s7,2

		 andi  $s0,$t8,0x07
		 sll   $s0,$s0,2
		 addu  $s0,$s0,$s5
		 lb    $t1,0x00($s0)
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 addu  $v0,$t1,$t8
		 sb    $v0,0x00($s0)
		 sltu  $s0,$v0,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5088:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t1,0x20($t0)
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 addu  $v0,$t1,$t8
		 sw    $v0,0x20($t0)
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5010:				#:
		 addiu $s7,$s7,2

		 andi  $s0,$t8,0x07
		 sll   $s0,$s0,2
		 addu  $s0,$s0,$s5
		 lw    $t6,0x20($s0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t1,$v0
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 addu  $v0,$t1,$t8
		 sltu  $s0,$v0,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5018:				#:
		 addiu $s7,$s7,2

		 andi  $s0,$t8,0x07
		 sll   $s0,$s0,2
		 addu  $s0,$s0,$s5
		 lw    $t6,0x20($s0)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($s0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t1,$v0
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 addu  $v0,$t1,$t8
		 sltu  $s0,$v0,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_501f:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t1,$v0
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 addu  $v0,$t1,$t8
		 sltu  $s0,$v0,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5020:				#:
		 addiu $s7,$s7,2

		 andi  $s0,$t8,0x07
		 sll   $s0,$s0,2
		 addu  $s0,$s0,$s5
		 lw    $t6,0x20($s0)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($s0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t1,$v0
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 addu  $v0,$t1,$t8
		 sltu  $s0,$v0,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5027:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t1,$v0
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 addu  $v0,$t1,$t8
		 sltu  $s0,$v0,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5028:				#:
		 addiu $s7,$s7,2

		 andi  $s0,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $s0,$s0,2
		 addu  $s0,$s0,$s5
		 lw    $t6,0x20($s0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t1,$v0
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 addu  $v0,$t1,$t8
		 sltu  $s0,$v0,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5030:				#:
		 addiu $s7,$s7,2

		 andi  $s0,$t8,0x07
		 sll   $s0,$s0,2
		 addu  $s0,$s0,$s5
		 lw    $t6,0x20($s0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t1,$v0
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 addu  $v0,$t1,$t8
		 sltu  $s0,$v0,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5038:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t1,$v0
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 addu  $v0,$t1,$t8
		 sltu  $s0,$v0,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5039:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t1,$v0
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 addu  $v0,$t1,$t8
		 sltu  $s0,$v0,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5040:				#:
		 addiu $s7,$s7,2

		 andi  $s0,$t8,0x07
		 sll   $s0,$s0,2
		 addu  $s0,$s0,$s5
		 lh    $t1,0x00($s0)
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 addu  $v0,$t1,$t8
		 sh    $v0,0x00($s0)
		 sltu  $s0,$v0,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5050:				#:
		 addiu $s7,$s7,2

		 andi  $s0,$t8,0x07
		 sll   $s0,$s0,2
		 addu  $s0,$s0,$s5
		 lw    $t6,0x20($s0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t1,$v0
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 addu  $v0,$t1,$t8
		 sltu  $s0,$v0,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5058:				#:
		 addiu $s7,$s7,2

		 andi  $s0,$t8,0x07
		 sll   $s0,$s0,2
		 addu  $s0,$s0,$s5
		 lw    $t6,0x20($s0)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($s0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t1,$v0
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 addu  $v0,$t1,$t8
		 sltu  $s0,$v0,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5060:				#:
		 addiu $s7,$s7,2

		 andi  $s0,$t8,0x07
		 sll   $s0,$s0,2
		 addu  $s0,$s0,$s5
		 lw    $t6,0x20($s0)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($s0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t1,$v0
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 addu  $v0,$t1,$t8
		 sltu  $s0,$v0,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5068:				#:
		 addiu $s7,$s7,2

		 andi  $s0,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $s0,$s0,2
		 addu  $s0,$s0,$s5
		 lw    $t6,0x20($s0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t1,$v0
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 addu  $v0,$t1,$t8
		 sltu  $s0,$v0,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5070:				#:
		 addiu $s7,$s7,2

		 andi  $s0,$t8,0x07
		 sll   $s0,$s0,2
		 addu  $s0,$s0,$s5
		 lw    $t6,0x20($s0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t1,$v0
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 addu  $v0,$t1,$t8
		 sltu  $s0,$v0,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5078:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t1,$v0
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 addu  $v0,$t1,$t8
		 sltu  $s0,$v0,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5079:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t1,$v0
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 addu  $v0,$t1,$t8
		 sltu  $s0,$v0,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5080:				#:
		 addiu $s7,$s7,2

		 andi  $s0,$t8,0x07
		 sll   $s0,$s0,2
		 addu  $s0,$s0,$s5
		 lw    $t1,0x00($s0)
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 addu  $v0,$t1,$t8
		 sw    $v0,0x00($s0)
		 sltu  $s0,$v0,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5090:				#:
		 addiu $s7,$s7,2

		 andi  $s0,$t8,0x07
		 sll   $s0,$s0,2
		 addu  $s0,$s0,$s5
		 lw    $t6,0x20($s0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t1,$0,$v0
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 addu  $v0,$t1,$t8
		 sltu  $s0,$v0,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5098:				#:
		 addiu $s7,$s7,2

		 andi  $s0,$t8,0x07
		 sll   $s0,$s0,2
		 addu  $s0,$s0,$s5
		 lw    $t6,0x20($s0)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($s0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t1,$0,$v0
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 addu  $v0,$t1,$t8
		 sltu  $s0,$v0,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_50a0:				#:
		 addiu $s7,$s7,2

		 andi  $s0,$t8,0x07
		 sll   $s0,$s0,2
		 addu  $s0,$s0,$s5
		 lw    $t6,0x20($s0)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($s0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t1,$0,$v0
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 addu  $v0,$t1,$t8
		 sltu  $s0,$v0,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_50a8:				#:
		 addiu $s7,$s7,2

		 andi  $s0,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $s0,$s0,2
		 addu  $s0,$s0,$s5
		 lw    $t6,0x20($s0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t1,$0,$v0
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 addu  $v0,$t1,$t8
		 sltu  $s0,$v0,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_50b0:				#:
		 addiu $s7,$s7,2

		 andi  $s0,$t8,0x07
		 sll   $s0,$s0,2
		 addu  $s0,$s0,$s5
		 lw    $t6,0x20($s0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t1,$0,$v0
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 addu  $v0,$t1,$t8
		 sltu  $s0,$v0,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-26
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_50b8:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t1,$0,$v0
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 addu  $v0,$t1,$t8
		 sltu  $s0,$v0,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_50b9:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t1,$0,$v0
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 addu  $v0,$t1,$t8
		 sltu  $s0,$v0,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-28
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_50c0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 addiu $v0,$0,-1
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 sb    $v0,0x00($t8)
		 andi  $v0,$v0,0x02
		 addiu $t7,$t7,-4
		 subu  $t7,$t7,$v0
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_50c8:				#:
		 bgez  $0,1f
		 andi  $t8,$t8,0x07     	 # Delay slot
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lhu   $t0,0x00($t8)
		 addiu $t1,$t0,-1
		 beq   $t0,$0,9f         	 # Is it -1
		 sh    $t1,0x00($t8)     	 # Delay slot

		 addiu $s7,$s7,2

		 lh    $v0,0x00($s7)
		 addu  $s7,$s7,$v0
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	9:
	1:
		 addiu $s7,$s7,4

		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_50d0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $v0,$0,-1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_50d8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t8)
		 addiu $v0,$0,-1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_50df:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 addiu $v0,$0,-1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_50e0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 addiu $v0,$0,-1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_50e7:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 addiu $v0,$0,-1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_50e8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 addiu $v0,$0,-1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_50f0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 addiu $v0,$0,-1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_50f8:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 addiu $v0,$0,-1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_50f9:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 addiu $v0,$0,-1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5100:				#:
		 addiu $s7,$s7,2

		 andi  $s0,$t8,0x07
		 sll   $s0,$s0,2
		 addu  $s0,$s0,$s5
		 lb    $t1,0x00($s0)
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 subu  $v0,$t1,$t8
		 sb    $v0,0x00($s0)
		 sltu  $s0,$t1,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5188:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t1,0x20($t0)
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 subu  $v0,$t1,$t8
		 sw    $v0,0x20($t0)
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5110:				#:
		 addiu $s7,$s7,2

		 andi  $s0,$t8,0x07
		 sll   $s0,$s0,2
		 addu  $s0,$s0,$s5
		 lw    $t6,0x20($s0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t1,$v0
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 subu  $v0,$t1,$t8
		 sltu  $s0,$t1,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5118:				#:
		 addiu $s7,$s7,2

		 andi  $s0,$t8,0x07
		 sll   $s0,$s0,2
		 addu  $s0,$s0,$s5
		 lw    $t6,0x20($s0)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($s0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t1,$v0
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 subu  $v0,$t1,$t8
		 sltu  $s0,$t1,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_511f:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t1,$v0
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 subu  $v0,$t1,$t8
		 sltu  $s0,$t1,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5120:				#:
		 addiu $s7,$s7,2

		 andi  $s0,$t8,0x07
		 sll   $s0,$s0,2
		 addu  $s0,$s0,$s5
		 lw    $t6,0x20($s0)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($s0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t1,$v0
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 subu  $v0,$t1,$t8
		 sltu  $s0,$t1,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5127:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t1,$v0
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 subu  $v0,$t1,$t8
		 sltu  $s0,$t1,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5128:				#:
		 addiu $s7,$s7,2

		 andi  $s0,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $s0,$s0,2
		 addu  $s0,$s0,$s5
		 lw    $t6,0x20($s0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t1,$v0
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 subu  $v0,$t1,$t8
		 sltu  $s0,$t1,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5130:				#:
		 addiu $s7,$s7,2

		 andi  $s0,$t8,0x07
		 sll   $s0,$s0,2
		 addu  $s0,$s0,$s5
		 lw    $t6,0x20($s0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t1,$v0
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 subu  $v0,$t1,$t8
		 sltu  $s0,$t1,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5138:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t1,$v0
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 subu  $v0,$t1,$t8
		 sltu  $s0,$t1,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5139:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t1,$v0
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 subu  $v0,$t1,$t8
		 sltu  $s0,$t1,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5140:				#:
		 addiu $s7,$s7,2

		 andi  $s0,$t8,0x07
		 sll   $s0,$s0,2
		 addu  $s0,$s0,$s5
		 lh    $t1,0x00($s0)
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 subu  $v0,$t1,$t8
		 sh    $v0,0x00($s0)
		 sltu  $s0,$t1,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5150:				#:
		 addiu $s7,$s7,2

		 andi  $s0,$t8,0x07
		 sll   $s0,$s0,2
		 addu  $s0,$s0,$s5
		 lw    $t6,0x20($s0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t1,$v0
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 subu  $v0,$t1,$t8
		 sltu  $s0,$t1,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5158:				#:
		 addiu $s7,$s7,2

		 andi  $s0,$t8,0x07
		 sll   $s0,$s0,2
		 addu  $s0,$s0,$s5
		 lw    $t6,0x20($s0)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($s0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t1,$v0
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 subu  $v0,$t1,$t8
		 sltu  $s0,$t1,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5160:				#:
		 addiu $s7,$s7,2

		 andi  $s0,$t8,0x07
		 sll   $s0,$s0,2
		 addu  $s0,$s0,$s5
		 lw    $t6,0x20($s0)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($s0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t1,$v0
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 subu  $v0,$t1,$t8
		 sltu  $s0,$t1,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5168:				#:
		 addiu $s7,$s7,2

		 andi  $s0,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $s0,$s0,2
		 addu  $s0,$s0,$s5
		 lw    $t6,0x20($s0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t1,$v0
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 subu  $v0,$t1,$t8
		 sltu  $s0,$t1,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5170:				#:
		 addiu $s7,$s7,2

		 andi  $s0,$t8,0x07
		 sll   $s0,$s0,2
		 addu  $s0,$s0,$s5
		 lw    $t6,0x20($s0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t1,$v0
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 subu  $v0,$t1,$t8
		 sltu  $s0,$t1,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5178:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t1,$v0
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 subu  $v0,$t1,$t8
		 sltu  $s0,$t1,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5179:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t1,$v0
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 subu  $v0,$t1,$t8
		 sltu  $s0,$t1,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5180:				#:
		 addiu $s7,$s7,2

		 andi  $s0,$t8,0x07
		 sll   $s0,$s0,2
		 addu  $s0,$s0,$s5
		 lw    $t1,0x00($s0)
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 subu  $v0,$t1,$t8
		 sw    $v0,0x00($s0)
		 sltu  $s0,$t1,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5190:				#:
		 addiu $s7,$s7,2

		 andi  $s0,$t8,0x07
		 sll   $s0,$s0,2
		 addu  $s0,$s0,$s5
		 lw    $t6,0x20($s0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t1,$0,$v0
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 subu  $v0,$t1,$t8
		 sltu  $s0,$t1,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5198:				#:
		 addiu $s7,$s7,2

		 andi  $s0,$t8,0x07
		 sll   $s0,$s0,2
		 addu  $s0,$s0,$s5
		 lw    $t6,0x20($s0)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($s0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t1,$0,$v0
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 subu  $v0,$t1,$t8
		 sltu  $s0,$t1,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_51a0:				#:
		 addiu $s7,$s7,2

		 andi  $s0,$t8,0x07
		 sll   $s0,$s0,2
		 addu  $s0,$s0,$s5
		 lw    $t6,0x20($s0)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($s0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t1,$0,$v0
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 subu  $v0,$t1,$t8
		 sltu  $s0,$t1,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_51a8:				#:
		 addiu $s7,$s7,2

		 andi  $s0,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $s0,$s0,2
		 addu  $s0,$s0,$s5
		 lw    $t6,0x20($s0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t1,$0,$v0
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 subu  $v0,$t1,$t8
		 sltu  $s0,$t1,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_51b0:				#:
		 addiu $s7,$s7,2

		 andi  $s0,$t8,0x07
		 sll   $s0,$s0,2
		 addu  $s0,$s0,$s5
		 lw    $t6,0x20($s0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t1,$0,$v0
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 subu  $v0,$t1,$t8
		 sltu  $s0,$t1,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-26
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_51b8:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t1,$0,$v0
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 subu  $v0,$t1,$t8
		 sltu  $s0,$t1,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_51b9:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t1,$0,$v0
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 subu  $v0,$t1,$t8
		 sltu  $s0,$t1,$t8       	 # Set Carry
		 xor   $s1,$t1,$t8
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-28
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_51c0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 and   $v0,$0,$0
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 sb    $v0,0x00($t8)
		 andi  $v0,$v0,0x02
		 addiu $t7,$t7,-4
		 subu  $t7,$t7,$v0
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_51c8:				#:
		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lhu   $t0,0x00($t8)
		 addiu $t1,$t0,-1
		 beq   $t0,$0,9f         	 # Is it -1
		 sh    $t1,0x00($t8)     	 # Delay slot

		 addiu $s7,$s7,2

		 lh    $v0,0x00($s7)
		 addu  $s7,$s7,$v0
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	9:
	1:
		 addiu $s7,$s7,4

		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_51d0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 and   $v0,$0,$0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_51d8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t8)
		 and   $v0,$0,$0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_51df:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 and   $v0,$0,$0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_51e0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 and   $v0,$0,$0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_51e7:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 and   $v0,$0,$0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_51e8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 and   $v0,$0,$0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_51f0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 and   $v0,$0,$0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_51f8:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 and   $v0,$0,$0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_51f9:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 and   $v0,$0,$0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_52c0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 nor   $v0,$s0,$s2
		 andi  $v0,$v0,1
		 subu  $v0,$0,$v0
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 sb    $v0,0x00($t8)
		 andi  $v0,$v0,0x02
		 addiu $t7,$t7,-4
		 subu  $t7,$t7,$v0
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_52c8:				#:
		 nor   $t9,$s0,$s2
		 andi  $t9,$t9,1
		 bne   $t9,$0,1f
		 andi  $t8,$t8,0x07     	 # Delay slot
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lhu   $t0,0x00($t8)
		 addiu $t1,$t0,-1
		 beq   $t0,$0,9f         	 # Is it -1
		 sh    $t1,0x00($t8)     	 # Delay slot

		 addiu $s7,$s7,2

		 lh    $v0,0x00($s7)
		 addu  $s7,$s7,$v0
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	9:
	1:
		 addiu $s7,$s7,4

		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_52d0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 nor   $v0,$s0,$s2
		 andi  $v0,$v0,1
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_52d8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t8)
		 nor   $v0,$s0,$s2
		 andi  $v0,$v0,1
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_52df:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 nor   $v0,$s0,$s2
		 andi  $v0,$v0,1
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_52e0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 nor   $v0,$s0,$s2
		 andi  $v0,$v0,1
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_52e7:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 nor   $v0,$s0,$s2
		 andi  $v0,$v0,1
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_52e8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 nor   $v0,$s0,$s2
		 andi  $v0,$v0,1
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_52f0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 nor   $v0,$s0,$s2
		 andi  $v0,$v0,1
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_52f8:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 nor   $v0,$s0,$s2
		 andi  $v0,$v0,1
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_52f9:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 nor   $v0,$s0,$s2
		 andi  $v0,$v0,1
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_53c0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 or    $v0,$s0,$s2
		 subu  $v0,$0,$v0
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 sb    $v0,0x00($t8)
		 andi  $v0,$v0,0x02
		 addiu $t7,$t7,-4
		 subu  $t7,$t7,$v0
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_53c8:				#:
		 or   $t9,$s0,$s2
		 bne   $t9,$0,1f
		 andi  $t8,$t8,0x07     	 # Delay slot
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lhu   $t0,0x00($t8)
		 addiu $t1,$t0,-1
		 beq   $t0,$0,9f         	 # Is it -1
		 sh    $t1,0x00($t8)     	 # Delay slot

		 addiu $s7,$s7,2

		 lh    $v0,0x00($s7)
		 addu  $s7,$s7,$v0
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	9:
	1:
		 addiu $s7,$s7,4

		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_53d0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 or    $v0,$s0,$s2
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_53d8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t8)
		 or    $v0,$s0,$s2
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_53df:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 or    $v0,$s0,$s2
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_53e0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 or    $v0,$s0,$s2
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_53e7:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 or    $v0,$s0,$s2
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_53e8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 or    $v0,$s0,$s2
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_53f0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 or    $v0,$s0,$s2
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_53f8:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 or    $v0,$s0,$s2
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_53f9:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 or    $v0,$s0,$s2
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_54c0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 xori  $v0,$s0,1         	 # Check Carry
		 subu  $v0,$0,$v0
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 sb    $v0,0x00($t8)
		 andi  $v0,$v0,0x02
		 addiu $t7,$t7,-4
		 subu  $t7,$t7,$v0
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_54c8:				#:
		 beq   $s0,$0,1f        	 # check carry
		 andi  $t8,$t8,0x07     	 # Delay slot
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lhu   $t0,0x00($t8)
		 addiu $t1,$t0,-1
		 beq   $t0,$0,9f         	 # Is it -1
		 sh    $t1,0x00($t8)     	 # Delay slot

		 addiu $s7,$s7,2

		 lh    $v0,0x00($s7)
		 addu  $s7,$s7,$v0
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	9:
	1:
		 addiu $s7,$s7,4

		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_54d0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 xori  $v0,$s0,1         	 # Check Carry
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_54d8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t8)
		 xori  $v0,$s0,1         	 # Check Carry
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_54df:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 xori  $v0,$s0,1         	 # Check Carry
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_54e0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 xori  $v0,$s0,1         	 # Check Carry
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_54e7:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 xori  $v0,$s0,1         	 # Check Carry
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_54e8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 xori  $v0,$s0,1         	 # Check Carry
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_54f0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 xori  $v0,$s0,1         	 # Check Carry
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_54f8:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 xori  $v0,$s0,1         	 # Check Carry
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_54f9:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 xori  $v0,$s0,1         	 # Check Carry
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_55c0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 or    $v0,$0,$s0        	 # Check Carry
		 subu  $v0,$0,$v0
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 sb    $v0,0x00($t8)
		 andi  $v0,$v0,0x02
		 addiu $t7,$t7,-4
		 subu  $t7,$t7,$v0
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_55c8:				#:
		 bne   $s0,$0,1f        	 # check carry
		 andi  $t8,$t8,0x07     	 # Delay slot
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lhu   $t0,0x00($t8)
		 addiu $t1,$t0,-1
		 beq   $t0,$0,9f         	 # Is it -1
		 sh    $t1,0x00($t8)     	 # Delay slot

		 addiu $s7,$s7,2

		 lh    $v0,0x00($s7)
		 addu  $s7,$s7,$v0
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	9:
	1:
		 addiu $s7,$s7,4

		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_55d0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 or    $v0,$0,$s0        	 # Check Carry
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_55d8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t8)
		 or    $v0,$0,$s0        	 # Check Carry
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_55df:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 or    $v0,$0,$s0        	 # Check Carry
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_55e0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 or    $v0,$0,$s0        	 # Check Carry
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_55e7:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 or    $v0,$0,$s0        	 # Check Carry
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_55e8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 or    $v0,$0,$s0        	 # Check Carry
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_55f0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 or    $v0,$0,$s0        	 # Check Carry
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_55f8:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 or    $v0,$0,$s0        	 # Check Carry
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_55f9:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 or    $v0,$0,$s0        	 # Check Carry
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_56c0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 xori  $v0,$s2,1         	 # Check Zero
		 subu  $v0,$0,$v0
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 sb    $v0,0x00($t8)
		 andi  $v0,$v0,0x02
		 addiu $t7,$t7,-4
		 subu  $t7,$t7,$v0
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_56c8:				#:
		 beq   $s2,$0,1f        	 # Check zero
		 andi  $t8,$t8,0x07     	 # Delay slot
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lhu   $t0,0x00($t8)
		 addiu $t1,$t0,-1
		 beq   $t0,$0,9f         	 # Is it -1
		 sh    $t1,0x00($t8)     	 # Delay slot

		 addiu $s7,$s7,2

		 lh    $v0,0x00($s7)
		 addu  $s7,$s7,$v0
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	9:
	1:
		 addiu $s7,$s7,4

		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_56d0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 xori  $v0,$s2,1         	 # Check Zero
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_56d8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t8)
		 xori  $v0,$s2,1         	 # Check Zero
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_56df:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 xori  $v0,$s2,1         	 # Check Zero
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_56e0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 xori  $v0,$s2,1         	 # Check Zero
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_56e7:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 xori  $v0,$s2,1         	 # Check Zero
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_56e8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 xori  $v0,$s2,1         	 # Check Zero
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_56f0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 xori  $v0,$s2,1         	 # Check Zero
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_56f8:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 xori  $v0,$s2,1         	 # Check Zero
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_56f9:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 xori  $v0,$s2,1         	 # Check Zero
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_57c0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 or    $v0,$0,$s2        	 # Check Zero
		 subu  $v0,$0,$v0
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 sb    $v0,0x00($t8)
		 andi  $v0,$v0,0x02
		 addiu $t7,$t7,-4
		 subu  $t7,$t7,$v0
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_57c8:				#:
		 bne   $s2,$0,1f        	 # Check zero
		 andi  $t8,$t8,0x07     	 # Delay slot
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lhu   $t0,0x00($t8)
		 addiu $t1,$t0,-1
		 beq   $t0,$0,9f         	 # Is it -1
		 sh    $t1,0x00($t8)     	 # Delay slot

		 addiu $s7,$s7,2

		 lh    $v0,0x00($s7)
		 addu  $s7,$s7,$v0
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	9:
	1:
		 addiu $s7,$s7,4

		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_57d0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 or    $v0,$0,$s2        	 # Check Zero
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_57d8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t8)
		 or    $v0,$0,$s2        	 # Check Zero
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_57df:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 or    $v0,$0,$s2        	 # Check Zero
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_57e0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 or    $v0,$0,$s2        	 # Check Zero
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_57e7:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 or    $v0,$0,$s2        	 # Check Zero
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_57e8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 or    $v0,$0,$s2        	 # Check Zero
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_57f0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 or    $v0,$0,$s2        	 # Check Zero
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_57f8:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 or    $v0,$0,$s2        	 # Check Zero
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_57f9:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 or    $v0,$0,$s2        	 # Check Zero
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_58c0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 xori  $v0,$s1,1         	 # Check Overflow
		 subu  $v0,$0,$v0
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 sb    $v0,0x00($t8)
		 andi  $v0,$v0,0x02
		 addiu $t7,$t7,-4
		 subu  $t7,$t7,$v0
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_58c8:				#:
		 beq   $s1,$0,1f        	 # Check Overflow
		 andi  $t8,$t8,0x07     	 # Delay slot
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lhu   $t0,0x00($t8)
		 addiu $t1,$t0,-1
		 beq   $t0,$0,9f         	 # Is it -1
		 sh    $t1,0x00($t8)     	 # Delay slot

		 addiu $s7,$s7,2

		 lh    $v0,0x00($s7)
		 addu  $s7,$s7,$v0
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	9:
	1:
		 addiu $s7,$s7,4

		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_58d0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 xori  $v0,$s1,1         	 # Check Overflow
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_58d8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t8)
		 xori  $v0,$s1,1         	 # Check Overflow
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_58df:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 xori  $v0,$s1,1         	 # Check Overflow
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_58e0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 xori  $v0,$s1,1         	 # Check Overflow
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_58e7:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 xori  $v0,$s1,1         	 # Check Overflow
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_58e8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 xori  $v0,$s1,1         	 # Check Overflow
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_58f0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 xori  $v0,$s1,1         	 # Check Overflow
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_58f8:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 xori  $v0,$s1,1         	 # Check Overflow
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_58f9:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 xori  $v0,$s1,1         	 # Check Overflow
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_59c0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 or    $v0,$0,$s1        	 # Check Overflow
		 subu  $v0,$0,$v0
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 sb    $v0,0x00($t8)
		 andi  $v0,$v0,0x02
		 addiu $t7,$t7,-4
		 subu  $t7,$t7,$v0
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_59c8:				#:
		 bne   $s1,$0,1f        	 # Check Overflow
		 andi  $t8,$t8,0x07     	 # Delay slot
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lhu   $t0,0x00($t8)
		 addiu $t1,$t0,-1
		 beq   $t0,$0,9f         	 # Is it -1
		 sh    $t1,0x00($t8)     	 # Delay slot

		 addiu $s7,$s7,2

		 lh    $v0,0x00($s7)
		 addu  $s7,$s7,$v0
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	9:
	1:
		 addiu $s7,$s7,4

		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_59d0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 or    $v0,$0,$s1        	 # Check Overflow
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_59d8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t8)
		 or    $v0,$0,$s1        	 # Check Overflow
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_59df:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 or    $v0,$0,$s1        	 # Check Overflow
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_59e0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 or    $v0,$0,$s1        	 # Check Overflow
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_59e7:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 or    $v0,$0,$s1        	 # Check Overflow
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_59e8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 or    $v0,$0,$s1        	 # Check Overflow
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_59f0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 or    $v0,$0,$s1        	 # Check Overflow
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_59f8:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 or    $v0,$0,$s1        	 # Check Overflow
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_59f9:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 or    $v0,$0,$s1        	 # Check Overflow
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5ac0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 xori  $v0,$s3,1         	 # Check Sign
		 subu  $v0,$0,$v0
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 sb    $v0,0x00($t8)
		 andi  $v0,$v0,0x02
		 addiu $t7,$t7,-4
		 subu  $t7,$t7,$v0
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5ac8:				#:
		 beq   $s3,$0,1f        	 # Check Sign
		 andi  $t8,$t8,0x07     	 # Delay slot
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lhu   $t0,0x00($t8)
		 addiu $t1,$t0,-1
		 beq   $t0,$0,9f         	 # Is it -1
		 sh    $t1,0x00($t8)     	 # Delay slot

		 addiu $s7,$s7,2

		 lh    $v0,0x00($s7)
		 addu  $s7,$s7,$v0
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	9:
	1:
		 addiu $s7,$s7,4

		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5ad0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 xori  $v0,$s3,1         	 # Check Sign
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5ad8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t8)
		 xori  $v0,$s3,1         	 # Check Sign
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5adf:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 xori  $v0,$s3,1         	 # Check Sign
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5ae0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 xori  $v0,$s3,1         	 # Check Sign
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5ae7:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 xori  $v0,$s3,1         	 # Check Sign
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5ae8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 xori  $v0,$s3,1         	 # Check Sign
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5af0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 xori  $v0,$s3,1         	 # Check Sign
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5af8:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 xori  $v0,$s3,1         	 # Check Sign
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5af9:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 xori  $v0,$s3,1         	 # Check Sign
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5bc0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 or    $v0,$0,$s3        	 # Check Sign
		 subu  $v0,$0,$v0
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 sb    $v0,0x00($t8)
		 andi  $v0,$v0,0x02
		 addiu $t7,$t7,-4
		 subu  $t7,$t7,$v0
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5bc8:				#:
		 bne   $s3,$0,1f        	 # Check Sign
		 andi  $t8,$t8,0x07     	 # Delay slot
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lhu   $t0,0x00($t8)
		 addiu $t1,$t0,-1
		 beq   $t0,$0,9f         	 # Is it -1
		 sh    $t1,0x00($t8)     	 # Delay slot

		 addiu $s7,$s7,2

		 lh    $v0,0x00($s7)
		 addu  $s7,$s7,$v0
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	9:
	1:
		 addiu $s7,$s7,4

		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5bd0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 or    $v0,$0,$s3        	 # Check Sign
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5bd8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t8)
		 or    $v0,$0,$s3        	 # Check Sign
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5bdf:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 or    $v0,$0,$s3        	 # Check Sign
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5be0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 or    $v0,$0,$s3        	 # Check Sign
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5be7:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 or    $v0,$0,$s3        	 # Check Sign
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5be8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 or    $v0,$0,$s3        	 # Check Sign
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5bf0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 or    $v0,$0,$s3        	 # Check Sign
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5bf8:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 or    $v0,$0,$s3        	 # Check Sign
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5bf9:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 or    $v0,$0,$s3        	 # Check Sign
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5cc0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 xor   $v0,$s3,$s1
		 addiu $v0,$v0,-1
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 sb    $v0,0x00($t8)
		 andi  $v0,$v0,0x02
		 addiu $t7,$t7,-4
		 subu  $t7,$t7,$v0
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5cc8:				#:
		 beq   $s3,$s1,1f
		 andi  $t8,$t8,0x07     	 # Delay slot
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lhu   $t0,0x00($t8)
		 addiu $t1,$t0,-1
		 beq   $t0,$0,9f         	 # Is it -1
		 sh    $t1,0x00($t8)     	 # Delay slot

		 addiu $s7,$s7,2

		 lh    $v0,0x00($s7)
		 addu  $s7,$s7,$v0
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	9:
	1:
		 addiu $s7,$s7,4

		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5cd0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 xor   $v0,$s3,$s1
		 addiu $v0,$v0,-1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5cd8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t8)
		 xor   $v0,$s3,$s1
		 addiu $v0,$v0,-1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5cdf:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 xor   $v0,$s3,$s1
		 addiu $v0,$v0,-1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5ce0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 xor   $v0,$s3,$s1
		 addiu $v0,$v0,-1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5ce7:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 xor   $v0,$s3,$s1
		 addiu $v0,$v0,-1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5ce8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 xor   $v0,$s3,$s1
		 addiu $v0,$v0,-1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5cf0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 xor   $v0,$s3,$s1
		 addiu $v0,$v0,-1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5cf8:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 xor   $v0,$s3,$s1
		 addiu $v0,$v0,-1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5cf9:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 xor   $v0,$s3,$s1
		 addiu $v0,$v0,-1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5dc0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 xor   $v0,$s3,$s1
		 subu  $v0,$0,$v0
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 sb    $v0,0x00($t8)
		 andi  $v0,$v0,0x02
		 addiu $t7,$t7,-4
		 subu  $t7,$t7,$v0
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5dc8:				#:
		 bne   $s3,$s1,1f
		 andi  $t8,$t8,0x07     	 # Delay slot
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lhu   $t0,0x00($t8)
		 addiu $t1,$t0,-1
		 beq   $t0,$0,9f         	 # Is it -1
		 sh    $t1,0x00($t8)     	 # Delay slot

		 addiu $s7,$s7,2

		 lh    $v0,0x00($s7)
		 addu  $s7,$s7,$v0
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	9:
	1:
		 addiu $s7,$s7,4

		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5dd0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 xor   $v0,$s3,$s1
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5dd8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t8)
		 xor   $v0,$s3,$s1
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5ddf:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 xor   $v0,$s3,$s1
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5de0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 xor   $v0,$s3,$s1
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5de7:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 xor   $v0,$s3,$s1
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5de8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 xor   $v0,$s3,$s1
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5df0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 xor   $v0,$s3,$s1
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5df8:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 xor   $v0,$s3,$s1
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5df9:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 xor   $v0,$s3,$s1
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5ec0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 xor   $v0,$s3,$s1
		 or    $v0,$v0,$s2
		 addiu $v0,$v0,-1
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 sb    $v0,0x00($t8)
		 andi  $v0,$v0,0x02
		 addiu $t7,$t7,-4
		 subu  $t7,$t7,$v0
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5ec8:				#:
		 xor    $t9,$s3,$s1
		 or     $t9,$t9,$s2
		 beq   $t9,$0,1f
		 andi  $t8,$t8,0x07     	 # Delay slot
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lhu   $t0,0x00($t8)
		 addiu $t1,$t0,-1
		 beq   $t0,$0,9f         	 # Is it -1
		 sh    $t1,0x00($t8)     	 # Delay slot

		 addiu $s7,$s7,2

		 lh    $v0,0x00($s7)
		 addu  $s7,$s7,$v0
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	9:
	1:
		 addiu $s7,$s7,4

		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5ed0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 xor   $v0,$s3,$s1
		 or    $v0,$v0,$s2
		 addiu $v0,$v0,-1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5ed8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t8)
		 xor   $v0,$s3,$s1
		 or    $v0,$v0,$s2
		 addiu $v0,$v0,-1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5edf:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 xor   $v0,$s3,$s1
		 or    $v0,$v0,$s2
		 addiu $v0,$v0,-1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5ee0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 xor   $v0,$s3,$s1
		 or    $v0,$v0,$s2
		 addiu $v0,$v0,-1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5ee7:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 xor   $v0,$s3,$s1
		 or    $v0,$v0,$s2
		 addiu $v0,$v0,-1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5ee8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 xor   $v0,$s3,$s1
		 or    $v0,$v0,$s2
		 addiu $v0,$v0,-1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5ef0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 xor   $v0,$s3,$s1
		 or    $v0,$v0,$s2
		 addiu $v0,$v0,-1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5ef8:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 xor   $v0,$s3,$s1
		 or    $v0,$v0,$s2
		 addiu $v0,$v0,-1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5ef9:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 xor   $v0,$s3,$s1
		 or    $v0,$v0,$s2
		 addiu $v0,$v0,-1
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5fc0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 xor   $v0,$s3,$s1
		 or    $v0,$v0,$s2
		 subu  $v0,$0,$v0
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 sb    $v0,0x00($t8)
		 andi  $v0,$v0,0x02
		 addiu $t7,$t7,-4
		 subu  $t7,$t7,$v0
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5fc8:				#:
		 xor    $t9,$s3,$s1
		 or     $t9,$t9,$s2
		 bne   $t9,$0,1f
		 andi  $t8,$t8,0x07     	 # Delay slot
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lhu   $t0,0x00($t8)
		 addiu $t1,$t0,-1
		 beq   $t0,$0,9f         	 # Is it -1
		 sh    $t1,0x00($t8)     	 # Delay slot

		 addiu $s7,$s7,2

		 lh    $v0,0x00($s7)
		 addu  $s7,$s7,$v0
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	9:
	1:
		 addiu $s7,$s7,4

		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5fd0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 xor   $v0,$s3,$s1
		 or    $v0,$v0,$s2
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5fd8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t8)
		 xor   $v0,$s3,$s1
		 or    $v0,$v0,$s2
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5fdf:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 xor   $v0,$s3,$s1
		 or    $v0,$v0,$s2
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5fe0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 xor   $v0,$s3,$s1
		 or    $v0,$v0,$s2
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5fe7:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 xor   $v0,$s3,$s1
		 or    $v0,$v0,$s2
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5fe8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 xor   $v0,$s3,$s1
		 or    $v0,$v0,$s2
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5ff0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 xor   $v0,$s3,$s1
		 or    $v0,$v0,$s2
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5ff8:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 xor   $v0,$s3,$s1
		 or    $v0,$v0,$s2
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_5ff9:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 xor   $v0,$s3,$s1
		 or    $v0,$v0,$s2
		 subu  $v0,$0,$v0
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_6000:				#:
		 addiu $s7,$s7,2

		 lh    $v0,0x00($s7)
		 addu  $s7,$s7,$v0
		 andi  $t9,$s7,0x01
		 beq   $t9,$0,2f
		 subu  $a1,$s7,$s6     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $a1,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$a1,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_6001:				#:
		 addiu $s7,$s7,2

		 m_seb   $t8,$t8
		 addu  $s7,$s7,$t8
		 andi  $t9,$s7,0x01
		 beq   $t9,$0,2f
		 subu  $a1,$s7,$s6     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $a1,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$a1,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_6100:				#:
		 addiu $s7,$s7,2

		 lh    $t8,0x00($s7)
		 addu  $t8,$s7,$t8
		 addiu $s7,$s7,2
		 lw    $a1,0x88($s5)
		 lui   $t9,0xFF00
		 lw    $a0,0x3C($s5)     	 # Push onto Stack
		 and   $a1,$a1,$t9
		 subu  $t9,$s7,$s6
		 addiu $a0,$a0,-4
		 sw    $a0,0x3C($s5)
		 or    $a1,$a1,$t9
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $s7,$0,$t8

		 andi  $t9,$s7,0x01
		 beq   $t9,$0,2f
		 subu  $a1,$s7,$s6     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $a1,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$a1,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_6101:				#:
		 addiu $s7,$s7,2

		 lw    $a1,0x88($s5)
		 lui   $t9,0xFF00
		 lw    $a0,0x3C($s5)     	 # Push onto Stack
		 and   $a1,$a1,$t9
		 subu  $t9,$s7,$s6
		 addiu $a0,$a0,-4
		 sw    $a0,0x3C($s5)
		 or    $a1,$a1,$t9
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t8,$t8
		 addu  $s7,$s7,$t8
		 andi  $t9,$s7,0x01
		 beq   $t9,$0,2f
		 subu  $a1,$s7,$s6     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $a1,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$a1,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_6200:				#:
		 nor   $t9,$s0,$s2
		 andi  $t9,$t9,1
		 bne   $t9,$0,1f
		 addiu $s7,$s7,2        	 # Delay slot

		 addiu $s7,$s7,2
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	1:
		 lh    $v0,0x00($s7)
		 addu  $s7,$s7,$v0
		 andi  $t9,$s7,0x01
		 beq   $t9,$0,2f
		 subu  $a1,$s7,$s6     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $a1,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$a1,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_6201:				#:
		 nor   $t9,$s0,$s2
		 andi  $t9,$t9,1
		 bne   $t9,$0,1f
		 addiu $s7,$s7,2        	 # Delay slot

		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	1:
		 m_seb   $t8,$t8
		 addu  $s7,$s7,$t8
		 andi  $t9,$s7,0x01
		 beq   $t9,$0,2f
		 subu  $a1,$s7,$s6     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $a1,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$a1,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_6300:				#:
		 or   $t9,$s0,$s2
		 bne   $t9,$0,1f
		 addiu $s7,$s7,2        	 # Delay slot

		 addiu $s7,$s7,2
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	1:
		 lh    $v0,0x00($s7)
		 addu  $s7,$s7,$v0
		 andi  $t9,$s7,0x01
		 beq   $t9,$0,2f
		 subu  $a1,$s7,$s6     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $a1,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$a1,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_6301:				#:
		 or   $t9,$s0,$s2
		 bne   $t9,$0,1f
		 addiu $s7,$s7,2        	 # Delay slot

		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	1:
		 m_seb   $t8,$t8
		 addu  $s7,$s7,$t8
		 andi  $t9,$s7,0x01
		 beq   $t9,$0,2f
		 subu  $a1,$s7,$s6     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $a1,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$a1,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_6400:				#:
		 beq   $s0,$0,1f        	 # check carry
		 addiu $s7,$s7,2        	 # Delay slot

		 addiu $s7,$s7,2
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	1:
		 lh    $v0,0x00($s7)
		 addu  $s7,$s7,$v0
		 andi  $t9,$s7,0x01
		 beq   $t9,$0,2f
		 subu  $a1,$s7,$s6     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $a1,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$a1,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_6401:				#:
		 beq   $s0,$0,1f        	 # check carry
		 addiu $s7,$s7,2        	 # Delay slot

		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	1:
		 m_seb   $t8,$t8
		 addu  $s7,$s7,$t8
		 andi  $t9,$s7,0x01
		 beq   $t9,$0,2f
		 subu  $a1,$s7,$s6     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $a1,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$a1,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_6500:				#:
		 bne   $s0,$0,1f        	 # check carry
		 addiu $s7,$s7,2        	 # Delay slot

		 addiu $s7,$s7,2
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	1:
		 lh    $v0,0x00($s7)
		 addu  $s7,$s7,$v0
		 andi  $t9,$s7,0x01
		 beq   $t9,$0,2f
		 subu  $a1,$s7,$s6     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $a1,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$a1,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_6501:				#:
		 bne   $s0,$0,1f        	 # check carry
		 addiu $s7,$s7,2        	 # Delay slot

		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	1:
		 m_seb   $t8,$t8
		 addu  $s7,$s7,$t8
		 andi  $t9,$s7,0x01
		 beq   $t9,$0,2f
		 subu  $a1,$s7,$s6     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $a1,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$a1,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_6600:				#:
		 beq   $s2,$0,1f        	 # Check zero
		 addiu $s7,$s7,2        	 # Delay slot

		 addiu $s7,$s7,2
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	1:
		 lh    $v0,0x00($s7)
		 addu  $s7,$s7,$v0
		 andi  $t9,$s7,0x01
		 beq   $t9,$0,2f
		 subu  $a1,$s7,$s6     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $a1,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$a1,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_6601:				#:
		 beq   $s2,$0,1f        	 # Check zero
		 addiu $s7,$s7,2        	 # Delay slot

		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	1:
		 m_seb   $t8,$t8
		 addu  $s7,$s7,$t8
		 andi  $t9,$s7,0x01
		 beq   $t9,$0,2f
		 subu  $a1,$s7,$s6     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $a1,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$a1,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_6700:				#:
		 bne   $s2,$0,1f        	 # Check zero
		 addiu $s7,$s7,2        	 # Delay slot

		 addiu $s7,$s7,2
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	1:
		 lh    $v0,0x00($s7)
		 addu  $s7,$s7,$v0
		 andi  $t9,$s7,0x01
		 beq   $t9,$0,2f
		 subu  $a1,$s7,$s6     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $a1,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$a1,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_6701:				#:
		 bne   $s2,$0,1f        	 # Check zero
		 addiu $s7,$s7,2        	 # Delay slot

		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	1:
		 m_seb   $t8,$t8
		 addu  $s7,$s7,$t8
		 andi  $t9,$s7,0x01
		 beq   $t9,$0,2f
		 subu  $a1,$s7,$s6     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $a1,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$a1,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_6800:				#:
		 beq   $s1,$0,1f        	 # Check Overflow
		 addiu $s7,$s7,2        	 # Delay slot

		 addiu $s7,$s7,2
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	1:
		 lh    $v0,0x00($s7)
		 addu  $s7,$s7,$v0
		 andi  $t9,$s7,0x01
		 beq   $t9,$0,2f
		 subu  $a1,$s7,$s6     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $a1,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$a1,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_6801:				#:
		 beq   $s1,$0,1f        	 # Check Overflow
		 addiu $s7,$s7,2        	 # Delay slot

		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	1:
		 m_seb   $t8,$t8
		 addu  $s7,$s7,$t8
		 andi  $t9,$s7,0x01
		 beq   $t9,$0,2f
		 subu  $a1,$s7,$s6     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $a1,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$a1,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_6900:				#:
		 bne   $s1,$0,1f        	 # Check Overflow
		 addiu $s7,$s7,2        	 # Delay slot

		 addiu $s7,$s7,2
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	1:
		 lh    $v0,0x00($s7)
		 addu  $s7,$s7,$v0
		 andi  $t9,$s7,0x01
		 beq   $t9,$0,2f
		 subu  $a1,$s7,$s6     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $a1,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$a1,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_6901:				#:
		 bne   $s1,$0,1f        	 # Check Overflow
		 addiu $s7,$s7,2        	 # Delay slot

		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	1:
		 m_seb   $t8,$t8
		 addu  $s7,$s7,$t8
		 andi  $t9,$s7,0x01
		 beq   $t9,$0,2f
		 subu  $a1,$s7,$s6     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $a1,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$a1,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_6a00:				#:
		 beq   $s3,$0,1f        	 # Check Sign
		 addiu $s7,$s7,2        	 # Delay slot

		 addiu $s7,$s7,2
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	1:
		 lh    $v0,0x00($s7)
		 addu  $s7,$s7,$v0
		 andi  $t9,$s7,0x01
		 beq   $t9,$0,2f
		 subu  $a1,$s7,$s6     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $a1,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$a1,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_6a01:				#:
		 beq   $s3,$0,1f        	 # Check Sign
		 addiu $s7,$s7,2        	 # Delay slot

		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	1:
		 m_seb   $t8,$t8
		 addu  $s7,$s7,$t8
		 andi  $t9,$s7,0x01
		 beq   $t9,$0,2f
		 subu  $a1,$s7,$s6     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $a1,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$a1,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_6b00:				#:
		 bne   $s3,$0,1f        	 # Check Sign
		 addiu $s7,$s7,2        	 # Delay slot

		 addiu $s7,$s7,2
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	1:
		 lh    $v0,0x00($s7)
		 addu  $s7,$s7,$v0
		 andi  $t9,$s7,0x01
		 beq   $t9,$0,2f
		 subu  $a1,$s7,$s6     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $a1,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$a1,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_6b01:				#:
		 bne   $s3,$0,1f        	 # Check Sign
		 addiu $s7,$s7,2        	 # Delay slot

		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	1:
		 m_seb   $t8,$t8
		 addu  $s7,$s7,$t8
		 andi  $t9,$s7,0x01
		 beq   $t9,$0,2f
		 subu  $a1,$s7,$s6     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $a1,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$a1,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_6c00:				#:
		 beq   $s3,$s1,1f
		 addiu $s7,$s7,2        	 # Delay slot

		 addiu $s7,$s7,2
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	1:
		 lh    $v0,0x00($s7)
		 addu  $s7,$s7,$v0
		 andi  $t9,$s7,0x01
		 beq   $t9,$0,2f
		 subu  $a1,$s7,$s6     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $a1,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$a1,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_6c01:				#:
		 beq   $s3,$s1,1f
		 addiu $s7,$s7,2        	 # Delay slot

		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	1:
		 m_seb   $t8,$t8
		 addu  $s7,$s7,$t8
		 andi  $t9,$s7,0x01
		 beq   $t9,$0,2f
		 subu  $a1,$s7,$s6     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $a1,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$a1,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_6d00:				#:
		 bne   $s3,$s1,1f
		 addiu $s7,$s7,2        	 # Delay slot

		 addiu $s7,$s7,2
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	1:
		 lh    $v0,0x00($s7)
		 addu  $s7,$s7,$v0
		 andi  $t9,$s7,0x01
		 beq   $t9,$0,2f
		 subu  $a1,$s7,$s6     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $a1,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$a1,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_6d01:				#:
		 bne   $s3,$s1,1f
		 addiu $s7,$s7,2        	 # Delay slot

		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	1:
		 m_seb   $t8,$t8
		 addu  $s7,$s7,$t8
		 andi  $t9,$s7,0x01
		 beq   $t9,$0,2f
		 subu  $a1,$s7,$s6     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $a1,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$a1,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_6e00:				#:
		 xor    $t9,$s3,$s1
		 or     $t9,$t9,$s2
		 beq   $t9,$0,1f
		 addiu $s7,$s7,2        	 # Delay slot

		 addiu $s7,$s7,2
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	1:
		 lh    $v0,0x00($s7)
		 addu  $s7,$s7,$v0
		 andi  $t9,$s7,0x01
		 beq   $t9,$0,2f
		 subu  $a1,$s7,$s6     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $a1,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$a1,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_6e01:				#:
		 xor    $t9,$s3,$s1
		 or     $t9,$t9,$s2
		 beq   $t9,$0,1f
		 addiu $s7,$s7,2        	 # Delay slot

		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	1:
		 m_seb   $t8,$t8
		 addu  $s7,$s7,$t8
		 andi  $t9,$s7,0x01
		 beq   $t9,$0,2f
		 subu  $a1,$s7,$s6     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $a1,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$a1,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_6f00:				#:
		 xor    $t9,$s3,$s1
		 or     $t9,$t9,$s2
		 bne   $t9,$0,1f
		 addiu $s7,$s7,2        	 # Delay slot

		 addiu $s7,$s7,2
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	1:
		 lh    $v0,0x00($s7)
		 addu  $s7,$s7,$v0
		 andi  $t9,$s7,0x01
		 beq   $t9,$0,2f
		 subu  $a1,$s7,$s6     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $a1,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$a1,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_6f01:				#:
		 xor    $t9,$s3,$s1
		 or     $t9,$t9,$s2
		 bne   $t9,$0,1f
		 addiu $s7,$s7,2        	 # Delay slot

		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	1:
		 m_seb   $t8,$t8
		 addu  $s7,$s7,$t8
		 andi  $t9,$s7,0x01
		 beq   $t9,$0,2f
		 subu  $a1,$s7,$s6     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $a1,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$a1,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_7000:				#:

		 addiu $s7,$s7,2

		 m_seb   $v0,$t8
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 addu  $t8,$t8,$s5
		 sw    $v0,0x00($t8)
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_8100:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lbu   $t0,0x00($t0)
		 addu  $t8,$t8,$s5
		 lbu   $t1,0x00($t8)
		 subu  $s1,$t1,$t0
		 subu  $s1,$s1,$s4
		 andi  $s1,$s1,0xFF
		 andi  $t2,$t0,0x0F
		 andi  $t3,$t1,0x0F
		 andi  $t0,$t0,0xF0
		 andi  $t1,$t1,0xF0
		 subu  $t3,$t3,$t2
		 subu  $t3,$t3,$s4
		 subu  $t1,$t1,$t0
		 sltiu $t4,$t3,10
		 xori  $t4,$t4,1
		 sll   $t4,$t4,1
		 subu  $t3,$t3,$t4
		 sll   $t4,$t4,1
		 subu  $t3,$t3,$t4
		 addu  $v0,$t1,$t3
		 sltiu $s0,$v0,0x9a
		 xori  $s0,$s0,1        	 # Set Carry
		 sll   $t4,$s0,5
		 addu  $v0,$v0,$t4
		 sll   $t4,$t4,2
		 addu  $v0,$v0,$t4
		 xor   $s3,$s1,$v0
		 and   $s1,$s1,$s3
		 srl   $s1,$s1,7        	 # Set Overflow
		 sltiu $t9,$v0,1
		 and   $s2,$s2,$t9       	 # Set Zero
		 srl   $s3,$v0,7         	 # Set Sign
		 or    $s4,$0,$s0       	 # Copy Carry to X
		 sb    $v0,0x00($t8)
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_8108:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t0,$0,$v0
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t1,$0,$v0
		 subu  $s1,$t1,$t0
		 subu  $s1,$s1,$s4
		 andi  $s1,$s1,0xFF
		 andi  $t2,$t0,0x0F
		 andi  $t3,$t1,0x0F
		 andi  $t0,$t0,0xF0
		 andi  $t1,$t1,0xF0
		 subu  $t3,$t3,$t2
		 subu  $t3,$t3,$s4
		 subu  $t1,$t1,$t0
		 sltiu $t4,$t3,10
		 xori  $t4,$t4,1
		 sll   $t4,$t4,1
		 subu  $t3,$t3,$t4
		 sll   $t4,$t4,1
		 subu  $t3,$t3,$t4
		 addu  $v0,$t1,$t3
		 sltiu $s0,$v0,0x9a
		 xori  $s0,$s0,1        	 # Set Carry
		 sll   $t4,$s0,5
		 addu  $v0,$v0,$t4
		 sll   $t4,$t4,2
		 addu  $v0,$v0,$t4
		 xor   $s3,$s1,$v0
		 and   $s1,$s1,$s3
		 srl   $s1,$s1,7        	 # Set Overflow
		 sltiu $t9,$v0,1
		 and   $s2,$s2,$t9       	 # Set Zero
		 srl   $s3,$v0,7         	 # Set Sign
		 or    $s4,$0,$s0       	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_810f:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t0,$0,$v0
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t1,$0,$v0
		 subu  $s1,$t1,$t0
		 subu  $s1,$s1,$s4
		 andi  $s1,$s1,0xFF
		 andi  $t2,$t0,0x0F
		 andi  $t3,$t1,0x0F
		 andi  $t0,$t0,0xF0
		 andi  $t1,$t1,0xF0
		 subu  $t3,$t3,$t2
		 subu  $t3,$t3,$s4
		 subu  $t1,$t1,$t0
		 sltiu $t4,$t3,10
		 xori  $t4,$t4,1
		 sll   $t4,$t4,1
		 subu  $t3,$t3,$t4
		 sll   $t4,$t4,1
		 subu  $t3,$t3,$t4
		 addu  $v0,$t1,$t3
		 sltiu $s0,$v0,0x9a
		 xori  $s0,$s0,1        	 # Set Carry
		 sll   $t4,$s0,5
		 addu  $v0,$v0,$t4
		 sll   $t4,$t4,2
		 addu  $v0,$v0,$t4
		 xor   $s3,$s1,$v0
		 and   $s1,$s1,$s3
		 srl   $s1,$s1,7        	 # Set Overflow
		 sltiu $t9,$v0,1
		 and   $s2,$s2,$t9       	 # Set Zero
		 srl   $s3,$v0,7         	 # Set Sign
		 or    $s4,$0,$s0       	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_8f08:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t0,$0,$v0
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t1,$0,$v0
		 subu  $s1,$t1,$t0
		 subu  $s1,$s1,$s4
		 andi  $s1,$s1,0xFF
		 andi  $t2,$t0,0x0F
		 andi  $t3,$t1,0x0F
		 andi  $t0,$t0,0xF0
		 andi  $t1,$t1,0xF0
		 subu  $t3,$t3,$t2
		 subu  $t3,$t3,$s4
		 subu  $t1,$t1,$t0
		 sltiu $t4,$t3,10
		 xori  $t4,$t4,1
		 sll   $t4,$t4,1
		 subu  $t3,$t3,$t4
		 sll   $t4,$t4,1
		 subu  $t3,$t3,$t4
		 addu  $v0,$t1,$t3
		 sltiu $s0,$v0,0x9a
		 xori  $s0,$s0,1        	 # Set Carry
		 sll   $t4,$s0,5
		 addu  $v0,$v0,$t4
		 sll   $t4,$t4,2
		 addu  $v0,$v0,$t4
		 xor   $s3,$s1,$v0
		 and   $s1,$s1,$s3
		 srl   $s1,$s1,7        	 # Set Overflow
		 sltiu $t9,$v0,1
		 and   $s2,$s2,$t9       	 # Set Zero
		 srl   $s3,$v0,7         	 # Set Sign
		 or    $s4,$0,$s0       	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_8f0f:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t0,$0,$v0
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t1,$0,$v0
		 subu  $s1,$t1,$t0
		 subu  $s1,$s1,$s4
		 andi  $s1,$s1,0xFF
		 andi  $t2,$t0,0x0F
		 andi  $t3,$t1,0x0F
		 andi  $t0,$t0,0xF0
		 andi  $t1,$t1,0xF0
		 subu  $t3,$t3,$t2
		 subu  $t3,$t3,$s4
		 subu  $t1,$t1,$t0
		 sltiu $t4,$t3,10
		 xori  $t4,$t4,1
		 sll   $t4,$t4,1
		 subu  $t3,$t3,$t4
		 sll   $t4,$t4,1
		 subu  $t3,$t3,$t4
		 addu  $v0,$t1,$t3
		 sltiu $s0,$v0,0x9a
		 xori  $s0,$s0,1        	 # Set Carry
		 sll   $t4,$s0,5
		 addu  $v0,$v0,$t4
		 sll   $t4,$t4,2
		 addu  $v0,$v0,$t4
		 xor   $s3,$s1,$v0
		 and   $s1,$s1,$s3
		 srl   $s1,$s1,7        	 # Set Overflow
		 sltiu $t9,$v0,1
		 and   $s2,$s2,$t9       	 # Set Zero
		 srl   $s3,$v0,7         	 # Set Sign
		 or    $s4,$0,$s0       	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c100:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lbu   $t0,0x00($t0)
		 addu  $t8,$t8,$s5
		 lbu   $t1,0x00($t8)
		 addu  $s1,$t1,$t0
		 addu  $s1,$s1,$s4
		 andi  $s1,$s1,0xFF
		 andi  $t2,$t0,0x0F
		 andi  $t3,$t1,0x0F
		 andi  $t0,$t0,0xF0
		 andi  $t1,$t1,0xF0
		 addu  $t3,$t3,$t2
		 addu  $t3,$t3,$s4
		 addu  $t1,$t1,$t0
		 sltiu $t4,$t3,10
		 xori  $t4,$t4,1
		 sll   $t4,$t4,1
		 addu  $t3,$t3,$t4
		 sll   $t4,$t4,1
		 addu  $t3,$t3,$t4
		 addu  $v0,$t1,$t3
		 sltiu $s0,$v0,0x9a
		 xori  $s0,$s0,1        	 # Set Carry
		 sll   $t4,$s0,5
		 subu  $v0,$v0,$t4
		 sll   $t4,$t4,2
		 subu  $v0,$v0,$t4
		 nor   $s1,$0,$s1
		 xor   $s3,$s1,$v0
		 and   $s1,$s1,$s3
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 sltiu $t9,$v0,1
		 and   $s2,$s2,$t9       	 # Set Zero
		 srl   $s3,$v0,7         	 # Set Sign
		 or    $s4,$0,$s0       	 # Copy Carry to X
		 sb    $v0,0x00($t8)
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c108:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t0,$0,$v0
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t1,$0,$v0
		 addu  $s1,$t1,$t0
		 addu  $s1,$s1,$s4
		 andi  $s1,$s1,0xFF
		 andi  $t2,$t0,0x0F
		 andi  $t3,$t1,0x0F
		 andi  $t0,$t0,0xF0
		 andi  $t1,$t1,0xF0
		 addu  $t3,$t3,$t2
		 addu  $t3,$t3,$s4
		 addu  $t1,$t1,$t0
		 sltiu $t4,$t3,10
		 xori  $t4,$t4,1
		 sll   $t4,$t4,1
		 addu  $t3,$t3,$t4
		 sll   $t4,$t4,1
		 addu  $t3,$t3,$t4
		 addu  $v0,$t1,$t3
		 sltiu $s0,$v0,0x9a
		 xori  $s0,$s0,1        	 # Set Carry
		 sll   $t4,$s0,5
		 subu  $v0,$v0,$t4
		 sll   $t4,$t4,2
		 subu  $v0,$v0,$t4
		 nor   $s1,$0,$s1
		 xor   $s3,$s1,$v0
		 and   $s1,$s1,$s3
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 sltiu $t9,$v0,1
		 and   $s2,$s2,$t9       	 # Set Zero
		 srl   $s3,$v0,7         	 # Set Sign
		 or    $s4,$0,$s0       	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c10f:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t0,$0,$v0
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t1,$0,$v0
		 addu  $s1,$t1,$t0
		 addu  $s1,$s1,$s4
		 andi  $s1,$s1,0xFF
		 andi  $t2,$t0,0x0F
		 andi  $t3,$t1,0x0F
		 andi  $t0,$t0,0xF0
		 andi  $t1,$t1,0xF0
		 addu  $t3,$t3,$t2
		 addu  $t3,$t3,$s4
		 addu  $t1,$t1,$t0
		 sltiu $t4,$t3,10
		 xori  $t4,$t4,1
		 sll   $t4,$t4,1
		 addu  $t3,$t3,$t4
		 sll   $t4,$t4,1
		 addu  $t3,$t3,$t4
		 addu  $v0,$t1,$t3
		 sltiu $s0,$v0,0x9a
		 xori  $s0,$s0,1        	 # Set Carry
		 sll   $t4,$s0,5
		 subu  $v0,$v0,$t4
		 sll   $t4,$t4,2
		 subu  $v0,$v0,$t4
		 nor   $s1,$0,$s1
		 xor   $s3,$s1,$v0
		 and   $s1,$s1,$s3
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 sltiu $t9,$v0,1
		 and   $s2,$s2,$t9       	 # Set Zero
		 srl   $s3,$v0,7         	 # Set Sign
		 or    $s4,$0,$s0       	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_cf08:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t0,$0,$v0
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t1,$0,$v0
		 addu  $s1,$t1,$t0
		 addu  $s1,$s1,$s4
		 andi  $s1,$s1,0xFF
		 andi  $t2,$t0,0x0F
		 andi  $t3,$t1,0x0F
		 andi  $t0,$t0,0xF0
		 andi  $t1,$t1,0xF0
		 addu  $t3,$t3,$t2
		 addu  $t3,$t3,$s4
		 addu  $t1,$t1,$t0
		 sltiu $t4,$t3,10
		 xori  $t4,$t4,1
		 sll   $t4,$t4,1
		 addu  $t3,$t3,$t4
		 sll   $t4,$t4,1
		 addu  $t3,$t3,$t4
		 addu  $v0,$t1,$t3
		 sltiu $s0,$v0,0x9a
		 xori  $s0,$s0,1        	 # Set Carry
		 sll   $t4,$s0,5
		 subu  $v0,$v0,$t4
		 sll   $t4,$t4,2
		 subu  $v0,$v0,$t4
		 nor   $s1,$0,$s1
		 xor   $s3,$s1,$v0
		 and   $s1,$s1,$s3
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 sltiu $t9,$v0,1
		 and   $s2,$s2,$t9       	 # Set Zero
		 srl   $s3,$v0,7         	 # Set Sign
		 or    $s4,$0,$s0       	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_cf0f:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t0,$0,$v0
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t1,$0,$v0
		 addu  $s1,$t1,$t0
		 addu  $s1,$s1,$s4
		 andi  $s1,$s1,0xFF
		 andi  $t2,$t0,0x0F
		 andi  $t3,$t1,0x0F
		 andi  $t0,$t0,0xF0
		 andi  $t1,$t1,0xF0
		 addu  $t3,$t3,$t2
		 addu  $t3,$t3,$s4
		 addu  $t1,$t1,$t0
		 sltiu $t4,$t3,10
		 xori  $t4,$t4,1
		 sll   $t4,$t4,1
		 addu  $t3,$t3,$t4
		 sll   $t4,$t4,1
		 addu  $t3,$t3,$t4
		 addu  $v0,$t1,$t3
		 sltiu $s0,$v0,0x9a
		 xori  $s0,$s0,1        	 # Set Carry
		 sll   $t4,$s0,5
		 subu  $v0,$v0,$t4
		 sll   $t4,$t4,2
		 subu  $v0,$v0,$t4
		 nor   $s1,$0,$s1
		 xor   $s3,$s1,$v0
		 and   $s1,$s1,$s3
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 sltiu $t9,$v0,1
		 and   $s2,$s2,$t9       	 # Set Zero
		 srl   $s3,$v0,7         	 # Set Sign
		 or    $s4,$0,$s0       	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_8000:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lb    $v0,0x00($t0)
		 or   $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_8010:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 or   $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_8018:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 or   $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_801f:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 or   $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_8020:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 or   $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_8027:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 or   $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_8028:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 or   $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_8030:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 or   $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_8038:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 or   $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_8039:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 or   $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_803a:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xAC($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 or   $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_803b:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xAC($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 or   $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_803c:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lb    $v0,0x00($s7)
		 addiu $s7,$s7,2
		 or   $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_8040:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lh    $v0,0x00($t0)
		 or   $t2,$t1,$v0
		 sh    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_8050:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 or   $t2,$t1,$v0
		 sh    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_8058:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 or   $t2,$t1,$v0
		 sh    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_8060:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 or   $t2,$t1,$v0
		 sh    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_8068:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 or   $t2,$t1,$v0
		 sh    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_8070:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 or   $t2,$t1,$v0
		 sh    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_8078:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 or   $t2,$t1,$v0
		 sh    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_8079:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 or   $t2,$t1,$v0
		 sh    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_807a:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 or   $t2,$t1,$v0
		 sh    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_807b:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 or   $t2,$t1,$v0
		 sh    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_807c:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 lh    $v0,0x00($s7)
		 addiu $s7,$s7,2
		 or   $t2,$t1,$v0
		 sh    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_8080:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $v0,0x00($t0)
		 or   $t2,$t1,$v0
		 sw    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_8090:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 or   $t2,$t1,$v0
		 sw    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_8098:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 or   $t2,$t1,$v0
		 sw    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_80a0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 or   $t2,$t1,$v0
		 sw    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_80a8:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 or   $t2,$t1,$v0
		 sw    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_80b0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 or   $t2,$t1,$v0
		 sw    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_80b8:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 or   $t2,$t1,$v0
		 sw    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_80b9:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 or   $t2,$t1,$v0
		 sw    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_80ba:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xB4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 or   $t2,$t1,$v0
		 sw    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_80bb:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xB4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 or   $t2,$t1,$v0
		 sw    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_80bc:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 lhu   $v0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $v0,$v0,16
		 or    $v0,$v0,$t9
		 addiu $s7,$s7,4
		 or   $t2,$t1,$v0
		 sw    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_8110:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t2,$v0
		 or   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_8118:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t2,$v0
		 or   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_811f:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t2,$v0
		 or   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_8120:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t2,$v0
		 or   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_8127:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t2,$v0
		 or   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_8128:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t2,$v0
		 or   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_8130:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t2,$v0
		 or   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_8138:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t2,$v0
		 or   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_8139:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t2,$v0
		 or   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_8150:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t2,$v0
		 or   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_8158:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t2,$v0
		 or   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_8160:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t2,$v0
		 or   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_8168:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t2,$v0
		 or   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_8170:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t2,$v0
		 or   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_8178:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t2,$v0
		 or   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_8179:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t2,$v0
		 or   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_8190:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t2,$0,$v0
		 or   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_8198:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t2,$0,$v0
		 or   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_81a0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t2,$0,$v0
		 or   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_81a8:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t2,$0,$v0
		 or   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_81b0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t2,$0,$v0
		 or   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-26
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_81b8:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t2,$0,$v0
		 or   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_81b9:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t2,$0,$v0
		 or   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-28
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_9000:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lb    $v0,0x00($t0)
		 subu  $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_9010:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 subu  $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_9018:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 subu  $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_901f:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 subu  $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_9020:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 subu  $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_9027:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 subu  $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_9028:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 subu  $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_9030:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 subu  $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_9038:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 subu  $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_9039:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 subu  $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_903a:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xAC($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 subu  $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_903b:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xAC($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 subu  $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_903c:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lb    $v0,0x00($s7)
		 addiu $s7,$s7,2
		 subu  $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_9040:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x0f
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lh    $v0,0x00($t0)
		 subu  $t2,$t1,$v0
		 sh    $t2,0x00($t8)
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_9050:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 subu  $t2,$t1,$v0
		 sh    $t2,0x00($t8)
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_9058:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 subu  $t2,$t1,$v0
		 sh    $t2,0x00($t8)
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_9060:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 subu  $t2,$t1,$v0
		 sh    $t2,0x00($t8)
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_9068:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 subu  $t2,$t1,$v0
		 sh    $t2,0x00($t8)
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_9070:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 subu  $t2,$t1,$v0
		 sh    $t2,0x00($t8)
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_9078:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 subu  $t2,$t1,$v0
		 sh    $t2,0x00($t8)
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_9079:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 subu  $t2,$t1,$v0
		 sh    $t2,0x00($t8)
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_907a:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 subu  $t2,$t1,$v0
		 sh    $t2,0x00($t8)
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_907b:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 subu  $t2,$t1,$v0
		 sh    $t2,0x00($t8)
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_907c:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 lh    $v0,0x00($s7)
		 addiu $s7,$s7,2
		 subu  $t2,$t1,$v0
		 sh    $t2,0x00($t8)
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_9080:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x0f
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $v0,0x00($t0)
		 subu  $t2,$t1,$v0
		 sw    $t2,0x00($t8)
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_9090:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $t2,$t1,$v0
		 sw    $t2,0x00($t8)
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_9098:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $t2,$t1,$v0
		 sw    $t2,0x00($t8)
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_90a0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $t2,$t1,$v0
		 sw    $t2,0x00($t8)
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_90a8:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $t2,$t1,$v0
		 sw    $t2,0x00($t8)
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_90b0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $t2,$t1,$v0
		 sw    $t2,0x00($t8)
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_90b8:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $t2,$t1,$v0
		 sw    $t2,0x00($t8)
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_90b9:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $t2,$t1,$v0
		 sw    $t2,0x00($t8)
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_90ba:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xB4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $t2,$t1,$v0
		 sw    $t2,0x00($t8)
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_90bb:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xB4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $t2,$t1,$v0
		 sw    $t2,0x00($t8)
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_90bc:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 lhu   $v0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $v0,$v0,16
		 or    $v0,$v0,$t9
		 addiu $s7,$s7,4
		 subu  $t2,$t1,$v0
		 sw    $t2,0x00($t8)
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_9110:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t2,$v0
		 subu  $v0,$t2,$t1
		 sltu  $s0,$t2,$t1       	 # Set Carry
		 xor   $s1,$t2,$t1
		 xor   $t9,$v0,$t2
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_9118:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t2,$v0
		 subu  $v0,$t2,$t1
		 sltu  $s0,$t2,$t1       	 # Set Carry
		 xor   $s1,$t2,$t1
		 xor   $t9,$v0,$t2
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_911f:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t2,$v0
		 subu  $v0,$t2,$t1
		 sltu  $s0,$t2,$t1       	 # Set Carry
		 xor   $s1,$t2,$t1
		 xor   $t9,$v0,$t2
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_9120:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t2,$v0
		 subu  $v0,$t2,$t1
		 sltu  $s0,$t2,$t1       	 # Set Carry
		 xor   $s1,$t2,$t1
		 xor   $t9,$v0,$t2
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_9127:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t2,$v0
		 subu  $v0,$t2,$t1
		 sltu  $s0,$t2,$t1       	 # Set Carry
		 xor   $s1,$t2,$t1
		 xor   $t9,$v0,$t2
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_9128:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t2,$v0
		 subu  $v0,$t2,$t1
		 sltu  $s0,$t2,$t1       	 # Set Carry
		 xor   $s1,$t2,$t1
		 xor   $t9,$v0,$t2
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_9130:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t2,$v0
		 subu  $v0,$t2,$t1
		 sltu  $s0,$t2,$t1       	 # Set Carry
		 xor   $s1,$t2,$t1
		 xor   $t9,$v0,$t2
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_9138:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t2,$v0
		 subu  $v0,$t2,$t1
		 sltu  $s0,$t2,$t1       	 # Set Carry
		 xor   $s1,$t2,$t1
		 xor   $t9,$v0,$t2
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_9139:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t2,$v0
		 subu  $v0,$t2,$t1
		 sltu  $s0,$t2,$t1       	 # Set Carry
		 xor   $s1,$t2,$t1
		 xor   $t9,$v0,$t2
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_9150:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t2,$v0
		 subu  $v0,$t2,$t1
		 sltu  $s0,$t2,$t1       	 # Set Carry
		 xor   $s1,$t2,$t1
		 xor   $t9,$v0,$t2
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_9158:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t2,$v0
		 subu  $v0,$t2,$t1
		 sltu  $s0,$t2,$t1       	 # Set Carry
		 xor   $s1,$t2,$t1
		 xor   $t9,$v0,$t2
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_9160:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t2,$v0
		 subu  $v0,$t2,$t1
		 sltu  $s0,$t2,$t1       	 # Set Carry
		 xor   $s1,$t2,$t1
		 xor   $t9,$v0,$t2
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_9168:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t2,$v0
		 subu  $v0,$t2,$t1
		 sltu  $s0,$t2,$t1       	 # Set Carry
		 xor   $s1,$t2,$t1
		 xor   $t9,$v0,$t2
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_9170:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t2,$v0
		 subu  $v0,$t2,$t1
		 sltu  $s0,$t2,$t1       	 # Set Carry
		 xor   $s1,$t2,$t1
		 xor   $t9,$v0,$t2
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_9178:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t2,$v0
		 subu  $v0,$t2,$t1
		 sltu  $s0,$t2,$t1       	 # Set Carry
		 xor   $s1,$t2,$t1
		 xor   $t9,$v0,$t2
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_9179:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t2,$v0
		 subu  $v0,$t2,$t1
		 sltu  $s0,$t2,$t1       	 # Set Carry
		 xor   $s1,$t2,$t1
		 xor   $t9,$v0,$t2
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_9190:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t2,$0,$v0
		 subu  $v0,$t2,$t1
		 sltu  $s0,$t2,$t1       	 # Set Carry
		 xor   $s1,$t2,$t1
		 xor   $t9,$v0,$t2
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_9198:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t2,$0,$v0
		 subu  $v0,$t2,$t1
		 sltu  $s0,$t2,$t1       	 # Set Carry
		 xor   $s1,$t2,$t1
		 xor   $t9,$v0,$t2
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_91a0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t2,$0,$v0
		 subu  $v0,$t2,$t1
		 sltu  $s0,$t2,$t1       	 # Set Carry
		 xor   $s1,$t2,$t1
		 xor   $t9,$v0,$t2
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_91a8:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t2,$0,$v0
		 subu  $v0,$t2,$t1
		 sltu  $s0,$t2,$t1       	 # Set Carry
		 xor   $s1,$t2,$t1
		 xor   $t9,$v0,$t2
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_91b0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t2,$0,$v0
		 subu  $v0,$t2,$t1
		 sltu  $s0,$t2,$t1       	 # Set Carry
		 xor   $s1,$t2,$t1
		 xor   $t9,$v0,$t2
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-26
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_91b8:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t2,$0,$v0
		 subu  $v0,$t2,$t1
		 sltu  $s0,$t2,$t1       	 # Set Carry
		 xor   $s1,$t2,$t1
		 xor   $t9,$v0,$t2
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_91b9:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t2,$0,$v0
		 subu  $v0,$t2,$t1
		 sltu  $s0,$t2,$t1       	 # Set Carry
		 xor   $s1,$t2,$t1
		 xor   $t9,$v0,$t2
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-28
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_90c0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x0f
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lh    $v0,0x00($t0)
		 subu  $t2,$t1,$v0
		 sw    $t2,0x20($t8)
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_90d0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 subu  $t2,$t1,$v0
		 sw    $t2,0x20($t8)
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_90d8:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 subu  $t2,$t1,$v0
		 sw    $t2,0x20($t8)
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_90e0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 subu  $t2,$t1,$v0
		 sw    $t2,0x20($t8)
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_90e8:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 subu  $t2,$t1,$v0
		 sw    $t2,0x20($t8)
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_90f0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 subu  $t2,$t1,$v0
		 sw    $t2,0x20($t8)
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_90f8:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 subu  $t2,$t1,$v0
		 sw    $t2,0x20($t8)
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_90f9:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 subu  $t2,$t1,$v0
		 sw    $t2,0x20($t8)
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_90fa:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 subu  $t2,$t1,$v0
		 sw    $t2,0x20($t8)
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_90fb:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 subu  $t2,$t1,$v0
		 sw    $t2,0x20($t8)
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_90fc:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 lh    $v0,0x00($s7)
		 addiu $s7,$s7,2
		 subu  $t2,$t1,$v0
		 sw    $t2,0x20($t8)
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_91c0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x0f
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $v0,0x00($t0)
		 subu  $t2,$t1,$v0
		 sw    $t2,0x20($t8)
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_91d0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $t2,$t1,$v0
		 sw    $t2,0x20($t8)
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_91d8:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $t2,$t1,$v0
		 sw    $t2,0x20($t8)
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_91e0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $t2,$t1,$v0
		 sw    $t2,0x20($t8)
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_91e8:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $t2,$t1,$v0
		 sw    $t2,0x20($t8)
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_91f0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $t2,$t1,$v0
		 sw    $t2,0x20($t8)
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_91f8:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $t2,$t1,$v0
		 sw    $t2,0x20($t8)
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_91f9:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $t2,$t1,$v0
		 sw    $t2,0x20($t8)
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_91fa:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xB4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $t2,$t1,$v0
		 sw    $t2,0x20($t8)
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_91fb:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xB4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $t2,$t1,$v0
		 sw    $t2,0x20($t8)
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_91fc:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 lhu   $v0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $v0,$v0,16
		 or    $v0,$v0,$t9
		 addiu $s7,$s7,4
		 subu  $t2,$t1,$v0
		 sw    $t2,0x20($t8)
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b000:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x0f
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lb    $v0,0x00($t0)
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b010:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b018:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b01f:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b020:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b027:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b028:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b030:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b038:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b039:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b03a:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xAC($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b03b:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xAC($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b03c:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lb    $v0,0x00($s7)
		 addiu $s7,$s7,2
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b040:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x0f
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lh    $v0,0x00($t0)
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b050:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b058:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b060:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b068:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b070:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b078:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b079:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b07a:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b07b:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b07c:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 lh    $v0,0x00($s7)
		 addiu $s7,$s7,2
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b080:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x0f
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $v0,0x00($t0)
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b090:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b098:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b0a0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b0a8:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b0b0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b0b8:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b0b9:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b0ba:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xB4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b0bb:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xB4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b0bc:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 lhu   $v0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $v0,$v0,16
		 or    $v0,$v0,$t9
		 addiu $s7,$s7,4
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b0c0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x0f
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lh    $v0,0x00($t0)
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b0d0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b0d8:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b0e0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b0e8:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b0f0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b0f8:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b0f9:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b0fa:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b0fb:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b0fc:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 lh    $v0,0x00($s7)
		 addiu $s7,$s7,2
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b1c0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x0f
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $v0,0x00($t0)
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b1d0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b1d8:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b1e0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b1e8:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b1f0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b1f8:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b1f9:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b1fa:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xB4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b1fb:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xB4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b1fc:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 lhu   $v0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $v0,$v0,16
		 or    $v0,$v0,$t9
		 addiu $s7,$s7,4
		 subu  $t2,$t1,$v0
		 sltu  $s0,$t1,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d0c0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x0f
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lh    $v0,0x00($t0)
		 addu  $t2,$t1,$v0
		 sw    $t2,0x20($t8)
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d0d0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 addu  $t2,$t1,$v0
		 sw    $t2,0x20($t8)
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d0d8:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 addu  $t2,$t1,$v0
		 sw    $t2,0x20($t8)
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d0e0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 addu  $t2,$t1,$v0
		 sw    $t2,0x20($t8)
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d0e8:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 addu  $t2,$t1,$v0
		 sw    $t2,0x20($t8)
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d0f0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 addu  $t2,$t1,$v0
		 sw    $t2,0x20($t8)
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d0f8:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 addu  $t2,$t1,$v0
		 sw    $t2,0x20($t8)
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d0f9:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 addu  $t2,$t1,$v0
		 sw    $t2,0x20($t8)
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d0fa:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 addu  $t2,$t1,$v0
		 sw    $t2,0x20($t8)
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d0fb:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 addu  $t2,$t1,$v0
		 sw    $t2,0x20($t8)
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d0fc:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 lh    $v0,0x00($s7)
		 addiu $s7,$s7,2
		 addu  $t2,$t1,$v0
		 sw    $t2,0x20($t8)
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d1c0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x0f
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $v0,0x00($t0)
		 addu  $t2,$t1,$v0
		 sw    $t2,0x20($t8)
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d1d0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addu  $t2,$t1,$v0
		 sw    $t2,0x20($t8)
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d1d8:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addu  $t2,$t1,$v0
		 sw    $t2,0x20($t8)
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d1e0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addu  $t2,$t1,$v0
		 sw    $t2,0x20($t8)
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d1e8:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addu  $t2,$t1,$v0
		 sw    $t2,0x20($t8)
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d1f0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addu  $t2,$t1,$v0
		 sw    $t2,0x20($t8)
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d1f8:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addu  $t2,$t1,$v0
		 sw    $t2,0x20($t8)
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d1f9:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addu  $t2,$t1,$v0
		 sw    $t2,0x20($t8)
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d1fa:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xB4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addu  $t2,$t1,$v0
		 sw    $t2,0x20($t8)
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d1fb:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xB4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addu  $t2,$t1,$v0
		 sw    $t2,0x20($t8)
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d1fc:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x20($t8)
		 lhu   $v0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $v0,$v0,16
		 or    $v0,$v0,$t9
		 addiu $s7,$s7,4
		 addu  $t2,$t1,$v0
		 sw    $t2,0x20($t8)
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b100:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lb    $t2,0x00($t0)
		 xor   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 sb    $v0,0x00($t0)
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b110:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t2,$v0
		 xor   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b118:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t2,$v0
		 xor   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b11f:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t2,$v0
		 xor   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b120:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t2,$v0
		 xor   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b127:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t2,$v0
		 xor   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b128:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t2,$v0
		 xor   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b130:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t2,$v0
		 xor   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b138:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t2,$v0
		 xor   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b139:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t2,$v0
		 xor   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b140:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lh    $t2,0x00($t0)
		 xor   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 sh    $v0,0x00($t0)
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b150:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t2,$v0
		 xor   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b158:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t2,$v0
		 xor   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b160:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t2,$v0
		 xor   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b168:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t2,$v0
		 xor   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b170:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t2,$v0
		 xor   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b178:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t2,$v0
		 xor   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b179:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t2,$v0
		 xor   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b180:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t2,0x00($t0)
		 xor   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 sw    $v0,0x00($t0)
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b190:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t2,$0,$v0
		 xor   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b198:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t2,$0,$v0
		 xor   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b1a0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t2,$0,$v0
		 xor   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b1a8:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t2,$0,$v0
		 xor   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b1b0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t2,$0,$v0
		 xor   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-26
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b1b8:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t2,$0,$v0
		 xor   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b1b9:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t2,$0,$v0
		 xor   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-28
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c000:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lb    $v0,0x00($t0)
		 and   $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c010:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 and   $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c018:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 and   $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c01f:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 and   $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c020:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 and   $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c027:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 and   $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c028:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 and   $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c030:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 and   $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c038:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 and   $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c039:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 and   $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c03a:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xAC($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 and   $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c03b:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xAC($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 and   $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c03c:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lb    $v0,0x00($s7)
		 addiu $s7,$s7,2
		 and   $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c040:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lh    $v0,0x00($t0)
		 and   $t2,$t1,$v0
		 sh    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c050:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 and   $t2,$t1,$v0
		 sh    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c058:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 and   $t2,$t1,$v0
		 sh    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c060:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 and   $t2,$t1,$v0
		 sh    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c068:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 and   $t2,$t1,$v0
		 sh    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c070:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 and   $t2,$t1,$v0
		 sh    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c078:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 and   $t2,$t1,$v0
		 sh    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c079:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 and   $t2,$t1,$v0
		 sh    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c07a:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 and   $t2,$t1,$v0
		 sh    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c07b:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 and   $t2,$t1,$v0
		 sh    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c07c:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 lh    $v0,0x00($s7)
		 addiu $s7,$s7,2
		 and   $t2,$t1,$v0
		 sh    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c080:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $v0,0x00($t0)
		 and   $t2,$t1,$v0
		 sw    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c090:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 sw    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c098:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 sw    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c0a0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 sw    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c0a8:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 sw    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c0b0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 sw    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c0b8:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 sw    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c0b9:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 sw    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c0ba:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xB4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 sw    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c0bb:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xB4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $t2,$t1,$v0
		 sw    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c0bc:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 lhu   $v0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $v0,$v0,16
		 or    $v0,$v0,$t9
		 addiu $s7,$s7,4
		 and   $t2,$t1,$v0
		 sw    $t2,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c110:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t2,$v0
		 and   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c118:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t2,$v0
		 and   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c11f:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t2,$v0
		 and   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c120:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t2,$v0
		 and   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c127:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t2,$v0
		 and   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c128:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t2,$v0
		 and   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c130:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t2,$v0
		 and   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c138:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t2,$v0
		 and   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c139:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t2,$v0
		 and   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c150:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t2,$v0
		 and   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c158:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t2,$v0
		 and   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c160:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t2,$v0
		 and   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c168:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t2,$v0
		 and   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c170:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t2,$v0
		 and   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c178:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t2,$v0
		 and   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c179:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t2,$v0
		 and   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c190:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t2,$0,$v0
		 and   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c198:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t2,$0,$v0
		 and   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c1a0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t2,$0,$v0
		 and   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c1a8:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t2,$0,$v0
		 and   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c1b0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t2,$0,$v0
		 and   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-26
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c1b8:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t2,$0,$v0
		 and   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c1b9:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t2,$0,$v0
		 and   $v0,$t2,$t1
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-28
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d000:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lb    $v0,0x00($t0)
		 addu  $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 sltu  $s0,$t2,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 nor   $s1,$0,$s1
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d010:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 addu  $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 sltu  $s0,$t2,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 nor   $s1,$0,$s1
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d018:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 addu  $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 sltu  $s0,$t2,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 nor   $s1,$0,$s1
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d01f:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 addu  $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 sltu  $s0,$t2,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 nor   $s1,$0,$s1
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d020:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 addu  $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 sltu  $s0,$t2,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 nor   $s1,$0,$s1
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d027:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 addu  $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 sltu  $s0,$t2,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 nor   $s1,$0,$s1
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d028:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 addu  $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 sltu  $s0,$t2,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 nor   $s1,$0,$s1
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d030:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 addu  $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 sltu  $s0,$t2,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 nor   $s1,$0,$s1
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d038:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 addu  $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 sltu  $s0,$t2,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 nor   $s1,$0,$s1
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d039:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 addu  $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 sltu  $s0,$t2,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 nor   $s1,$0,$s1
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d03a:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xAC($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 addu  $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 sltu  $s0,$t2,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 nor   $s1,$0,$s1
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d03b:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xAC($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 addu  $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 sltu  $s0,$t2,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 nor   $s1,$0,$s1
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d03c:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lb    $v0,0x00($s7)
		 addiu $s7,$s7,2
		 addu  $t2,$t1,$v0
		 sb    $t2,0x00($t8)
		 sltu  $s0,$t2,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 nor   $s1,$0,$s1
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d040:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x0f
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lh    $v0,0x00($t0)
		 addu  $t2,$t1,$v0
		 sh    $t2,0x00($t8)
		 sltu  $s0,$t2,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 nor   $s1,$0,$s1
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d050:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 addu  $t2,$t1,$v0
		 sh    $t2,0x00($t8)
		 sltu  $s0,$t2,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 nor   $s1,$0,$s1
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d058:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 addu  $t2,$t1,$v0
		 sh    $t2,0x00($t8)
		 sltu  $s0,$t2,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 nor   $s1,$0,$s1
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d060:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 addu  $t2,$t1,$v0
		 sh    $t2,0x00($t8)
		 sltu  $s0,$t2,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 nor   $s1,$0,$s1
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d068:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 addu  $t2,$t1,$v0
		 sh    $t2,0x00($t8)
		 sltu  $s0,$t2,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 nor   $s1,$0,$s1
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d070:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 addu  $t2,$t1,$v0
		 sh    $t2,0x00($t8)
		 sltu  $s0,$t2,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 nor   $s1,$0,$s1
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d078:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 addu  $t2,$t1,$v0
		 sh    $t2,0x00($t8)
		 sltu  $s0,$t2,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 nor   $s1,$0,$s1
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d079:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 addu  $t2,$t1,$v0
		 sh    $t2,0x00($t8)
		 sltu  $s0,$t2,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 nor   $s1,$0,$s1
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d07a:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 addu  $t2,$t1,$v0
		 sh    $t2,0x00($t8)
		 sltu  $s0,$t2,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 nor   $s1,$0,$s1
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d07b:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 addu  $t2,$t1,$v0
		 sh    $t2,0x00($t8)
		 sltu  $s0,$t2,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 nor   $s1,$0,$s1
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d07c:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 lh    $v0,0x00($s7)
		 addiu $s7,$s7,2
		 addu  $t2,$t1,$v0
		 sh    $t2,0x00($t8)
		 sltu  $s0,$t2,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 nor   $s1,$0,$s1
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$t2
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d080:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x0f
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $v0,0x00($t0)
		 addu  $t2,$t1,$v0
		 sw    $t2,0x00($t8)
		 sltu  $s0,$t2,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 nor   $s1,$0,$s1
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d090:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addu  $t2,$t1,$v0
		 sw    $t2,0x00($t8)
		 sltu  $s0,$t2,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 nor   $s1,$0,$s1
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d098:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addu  $t2,$t1,$v0
		 sw    $t2,0x00($t8)
		 sltu  $s0,$t2,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 nor   $s1,$0,$s1
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d0a0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addu  $t2,$t1,$v0
		 sw    $t2,0x00($t8)
		 sltu  $s0,$t2,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 nor   $s1,$0,$s1
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d0a8:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addu  $t2,$t1,$v0
		 sw    $t2,0x00($t8)
		 sltu  $s0,$t2,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 nor   $s1,$0,$s1
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d0b0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addu  $t2,$t1,$v0
		 sw    $t2,0x00($t8)
		 sltu  $s0,$t2,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 nor   $s1,$0,$s1
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d0b8:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addu  $t2,$t1,$v0
		 sw    $t2,0x00($t8)
		 sltu  $s0,$t2,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 nor   $s1,$0,$s1
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d0b9:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addu  $t2,$t1,$v0
		 sw    $t2,0x00($t8)
		 sltu  $s0,$t2,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 nor   $s1,$0,$s1
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d0ba:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xB4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addu  $t2,$t1,$v0
		 sw    $t2,0x00($t8)
		 sltu  $s0,$t2,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 nor   $s1,$0,$s1
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d0bb:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xB4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addu  $t2,$t1,$v0
		 sw    $t2,0x00($t8)
		 sltu  $s0,$t2,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 nor   $s1,$0,$s1
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d0bc:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 lhu   $v0,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $v0,$v0,16
		 or    $v0,$v0,$t9
		 addiu $s7,$s7,4
		 addu  $t2,$t1,$v0
		 sw    $t2,0x00($t8)
		 sltu  $s0,$t2,$v0       	 # Set Carry
		 xor   $s1,$t1,$v0
		 nor   $s1,$0,$s1
		 xor   $t9,$t2,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$t2,$0        	 # Set Sign
		 sltiu $s2,$t2,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d110:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t2,$v0
		 addu  $v0,$t2,$t1
		 sltu  $s0,$v0,$t1       	 # Set Carry
		 xor   $s1,$t2,$t1
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t2
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d118:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t2,$v0
		 addu  $v0,$t2,$t1
		 sltu  $s0,$v0,$t1       	 # Set Carry
		 xor   $s1,$t2,$t1
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t2
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d11f:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t2,$v0
		 addu  $v0,$t2,$t1
		 sltu  $s0,$v0,$t1       	 # Set Carry
		 xor   $s1,$t2,$t1
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t2
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d120:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t2,$v0
		 addu  $v0,$t2,$t1
		 sltu  $s0,$v0,$t1       	 # Set Carry
		 xor   $s1,$t2,$t1
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t2
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d127:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t2,$v0
		 addu  $v0,$t2,$t1
		 sltu  $s0,$v0,$t1       	 # Set Carry
		 xor   $s1,$t2,$t1
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t2
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d128:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t2,$v0
		 addu  $v0,$t2,$t1
		 sltu  $s0,$v0,$t1       	 # Set Carry
		 xor   $s1,$t2,$t1
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t2
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d130:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t2,$v0
		 addu  $v0,$t2,$t1
		 sltu  $s0,$v0,$t1       	 # Set Carry
		 xor   $s1,$t2,$t1
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t2
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d138:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t2,$v0
		 addu  $v0,$t2,$t1
		 sltu  $s0,$v0,$t1       	 # Set Carry
		 xor   $s1,$t2,$t1
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t2
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d139:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lb    $t1,0x00($t8)
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t2,$v0
		 addu  $v0,$t2,$t1
		 sltu  $s0,$v0,$t1       	 # Set Carry
		 xor   $s1,$t2,$t1
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t2
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d150:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t2,$v0
		 addu  $v0,$t2,$t1
		 sltu  $s0,$v0,$t1       	 # Set Carry
		 xor   $s1,$t2,$t1
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t2
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d158:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t2,$v0
		 addu  $v0,$t2,$t1
		 sltu  $s0,$v0,$t1       	 # Set Carry
		 xor   $s1,$t2,$t1
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t2
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d160:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t2,$v0
		 addu  $v0,$t2,$t1
		 sltu  $s0,$v0,$t1       	 # Set Carry
		 xor   $s1,$t2,$t1
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t2
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d168:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t2,$v0
		 addu  $v0,$t2,$t1
		 sltu  $s0,$v0,$t1       	 # Set Carry
		 xor   $s1,$t2,$t1
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t2
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d170:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t2,$v0
		 addu  $v0,$t2,$t1
		 sltu  $s0,$v0,$t1       	 # Set Carry
		 xor   $s1,$t2,$t1
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t2
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d178:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t2,$v0
		 addu  $v0,$t2,$t1
		 sltu  $s0,$v0,$t1       	 # Set Carry
		 xor   $s1,$t2,$t1
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t2
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d179:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t2,$v0
		 addu  $v0,$t2,$t1
		 sltu  $s0,$v0,$t1       	 # Set Carry
		 xor   $s1,$t2,$t1
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t2
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d190:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t2,$0,$v0
		 addu  $v0,$t2,$t1
		 sltu  $s0,$v0,$t1       	 # Set Carry
		 xor   $s1,$t2,$t1
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t2
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d198:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t2,$0,$v0
		 addu  $v0,$t2,$t1
		 sltu  $s0,$v0,$t1       	 # Set Carry
		 xor   $s1,$t2,$t1
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t2
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d1a0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t2,$0,$v0
		 addu  $v0,$t2,$t1
		 sltu  $s0,$v0,$t1       	 # Set Carry
		 xor   $s1,$t2,$t1
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t2
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d1a8:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t2,$0,$v0
		 addu  $v0,$t2,$t1
		 sltu  $s0,$v0,$t1       	 # Set Carry
		 xor   $s1,$t2,$t1
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t2
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d1b0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t2,$0,$v0
		 addu  $v0,$t2,$t1
		 sltu  $s0,$v0,$t1       	 # Set Carry
		 xor   $s1,$t2,$t1
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t2
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-26
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d1b8:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t2,$0,$v0
		 addu  $v0,$t2,$t1
		 sltu  $s0,$v0,$t1       	 # Set Carry
		 xor   $s1,$t2,$t1
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t2
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d1b9:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t1,0x00($t8)
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t2,$0,$v0
		 addu  $v0,$t2,$t1
		 sltu  $s0,$v0,$t1       	 # Set Carry
		 xor   $s1,$t2,$t1
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t2
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t1,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $t8,0x38($sp)
		 sw    $gp,0x34($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x34($sp)
		 lw    $t8,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t1,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-28
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_9100:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07

		 srl   $t8,$t8,7

		 andi  $t8,$t8,0x1C

		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lb    $t0,0x00($t0)
		 addu   $t8,$t8,$s5

		 lb    $t1,0x00($t8)

		 subu  $v0,$t1,$t0
		 subu  $v0,$v0,$s4
		 addu  $t9,$t0,$s4
		 sltu  $s0,$t1,$t9       	 # Set Carry
		 xor   $s1,$t1,$t0
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $t9,$t9,1
		 and   $s2,$s2,$t9      	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 sb    $v0,0x00($t8)

		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_9108:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07

		 srl   $t8,$t8,7

		 andi  $t8,$t8,0x1C

		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t0,$v0
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t1,$v0
		 subu  $v0,$t1,$t0
		 subu  $v0,$v0,$s4
		 addu  $t9,$t0,$s4
		 sltu  $s0,$t1,$t9       	 # Set Carry
		 xor   $s1,$t1,$t0
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $t9,$t9,1
		 and   $s2,$s2,$t9      	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_910f:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07

		 srl   $t8,$t8,7

		 andi  $t8,$t8,0x1C

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t0,$v0
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t1,$v0
		 subu  $v0,$t1,$t0
		 subu  $v0,$v0,$s4
		 addu  $t9,$t0,$s4
		 sltu  $s0,$t1,$t9       	 # Set Carry
		 xor   $s1,$t1,$t0
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $t9,$t9,1
		 and   $s2,$s2,$t9      	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_9140:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07

		 srl   $t8,$t8,7

		 andi  $t8,$t8,0x1C

		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lh    $t0,0x00($t0)
		 addu   $t8,$t8,$s5

		 lh    $t1,0x00($t8)

		 subu  $v0,$t1,$t0
		 subu  $v0,$v0,$s4
		 addu  $t9,$t0,$s4
		 sltu  $s0,$t1,$t9       	 # Set Carry
		 xor   $s1,$t1,$t0
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $t9,$t9,1
		 and   $s2,$s2,$t9      	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 sh    $v0,0x00($t8)

		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_9148:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07

		 srl   $t8,$t8,7

		 andi  $t8,$t8,0x1C

		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t0,$v0
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t1,$v0
		 subu  $v0,$t1,$t0
		 subu  $v0,$v0,$s4
		 addu  $t9,$t0,$s4
		 sltu  $s0,$t1,$t9       	 # Set Carry
		 xor   $s1,$t1,$t0
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $t9,$t9,1
		 and   $s2,$s2,$t9      	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_9180:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07

		 srl   $t8,$t8,7

		 andi  $t8,$t8,0x1C

		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t0,0x00($t0)
		 addu   $t8,$t8,$s5

		 lw    $t1,0x00($t8)

		 subu  $v0,$t1,$t0
		 subu  $v0,$v0,$s4
		 addu  $t9,$t0,$s4
		 sltu  $s0,$t1,$t9       	 # Set Carry
		 xor   $s1,$t1,$t0
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $t9,$v0,1
		 and   $s2,$s2,$t9      	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 sw    $v0,0x00($t8)

		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_9188:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07

		 srl   $t8,$t8,7

		 andi  $t8,$t8,0x1C

		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t0,$0,$v0
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($t8)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t1,$0,$v0
		 subu  $v0,$t1,$t0
		 subu  $v0,$v0,$s4
		 addu  $t9,$t0,$s4
		 sltu  $s0,$t1,$t9       	 # Set Carry
		 xor   $s1,$t1,$t0
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $t9,$v0,1
		 and   $s2,$s2,$t9      	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-30
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_9f08:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07

		 srl   $t8,$t8,7

		 andi  $t8,$t8,0x1C

		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t0,$v0
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t1,$v0
		 subu  $v0,$t1,$t0
		 subu  $v0,$v0,$s4
		 addu  $t9,$t0,$s4
		 sltu  $s0,$t1,$t9       	 # Set Carry
		 xor   $s1,$t1,$t0
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $t9,$t9,1
		 and   $s2,$s2,$t9      	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_9f0f:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07

		 srl   $t8,$t8,7

		 andi  $t8,$t8,0x1C

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t0,$v0
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t1,$v0
		 subu  $v0,$t1,$t0
		 subu  $v0,$v0,$s4
		 addu  $t9,$t0,$s4
		 sltu  $s0,$t1,$t9       	 # Set Carry
		 xor   $s1,$t1,$t0
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $t9,$t9,1
		 and   $s2,$s2,$t9      	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d100:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07

		 srl   $t8,$t8,7

		 andi  $t8,$t8,0x1C

		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lb    $t0,0x00($t0)
		 addu   $t8,$t8,$s5

		 lb    $t1,0x00($t8)

		 addu  $v0,$t1,$t0
		 addu  $v0,$v0,$s4
		 addu  $t9,$t0,$s4
		 sltu  $s0,$v0,$t9       	 # Set Carry
		 xor   $s1,$t1,$t0
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $t9,$t9,1
		 and   $s2,$s2,$t9      	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 sb    $v0,0x00($t8)

		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d108:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07

		 srl   $t8,$t8,7

		 andi  $t8,$t8,0x1C

		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t0,$v0
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t1,$v0
		 addu  $v0,$t1,$t0
		 addu  $v0,$v0,$s4
		 addu  $t9,$t0,$s4
		 sltu  $s0,$v0,$t9       	 # Set Carry
		 xor   $s1,$t1,$t0
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $t9,$t9,1
		 and   $s2,$s2,$t9      	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d10f:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07

		 srl   $t8,$t8,7

		 andi  $t8,$t8,0x1C

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t0,$v0
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t1,$v0
		 addu  $v0,$t1,$t0
		 addu  $v0,$v0,$s4
		 addu  $t9,$t0,$s4
		 sltu  $s0,$v0,$t9       	 # Set Carry
		 xor   $s1,$t1,$t0
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $t9,$t9,1
		 and   $s2,$s2,$t9      	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d140:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07

		 srl   $t8,$t8,7

		 andi  $t8,$t8,0x1C

		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lh    $t0,0x00($t0)
		 addu   $t8,$t8,$s5

		 lh    $t1,0x00($t8)

		 addu  $v0,$t1,$t0
		 addu  $v0,$v0,$s4
		 addu  $t9,$t0,$s4
		 sltu  $s0,$v0,$t9       	 # Set Carry
		 xor   $s1,$t1,$t0
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $t9,$t9,1
		 and   $s2,$s2,$t9      	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 sh    $v0,0x00($t8)

		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d148:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07

		 srl   $t8,$t8,7

		 andi  $t8,$t8,0x1C

		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t0,$v0
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t1,$v0
		 addu  $v0,$t1,$t0
		 addu  $v0,$v0,$s4
		 addu  $t9,$t0,$s4
		 sltu  $s0,$v0,$t9       	 # Set Carry
		 xor   $s1,$t1,$t0
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $t9,$t9,1
		 and   $s2,$s2,$t9      	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d180:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07

		 srl   $t8,$t8,7

		 andi  $t8,$t8,0x1C

		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t0,0x00($t0)
		 addu   $t8,$t8,$s5

		 lw    $t1,0x00($t8)

		 addu  $v0,$t1,$t0
		 addu  $v0,$v0,$s4
		 addu  $t9,$t0,$s4
		 sltu  $s0,$v0,$t9       	 # Set Carry
		 xor   $s1,$t1,$t0
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $t9,$v0,1
		 and   $s2,$s2,$t9      	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 sw    $v0,0x00($t8)

		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_d188:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07

		 srl   $t8,$t8,7

		 andi  $t8,$t8,0x1C

		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t0,$0,$v0
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($t8)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t1,$0,$v0
		 addu  $v0,$t1,$t0
		 addu  $v0,$v0,$s4
		 addu  $t9,$t0,$s4
		 sltu  $s0,$v0,$t9       	 # Set Carry
		 xor   $s1,$t1,$t0
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $t9,$v0,1
		 and   $s2,$s2,$t9      	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-30
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_df08:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07

		 srl   $t8,$t8,7

		 andi  $t8,$t8,0x1C

		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t0,$v0
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t1,$v0
		 addu  $v0,$t1,$t0
		 addu  $v0,$v0,$s4
		 addu  $t9,$t0,$s4
		 sltu  $s0,$v0,$t9       	 # Set Carry
		 xor   $s1,$t1,$t0
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $t9,$t9,1
		 and   $s2,$s2,$t9      	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_df0f:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07

		 srl   $t8,$t8,7

		 andi  $t8,$t8,0x1C

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t0,$v0
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t1,$v0
		 addu  $v0,$t1,$t0
		 addu  $v0,$v0,$s4
		 addu  $t9,$t0,$s4
		 sltu  $s0,$v0,$t9       	 # Set Carry
		 xor   $s1,$t1,$t0
		 nor   $s1,$0,$s1
		 xor   $t9,$v0,$t1
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $t9,$t9,1
		 and   $s2,$s2,$t9      	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_80c0:				#:

		 addiu $s7,$s7,2

		 addiu $t7,$t7,-133
		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lhu   $v0,0x00($t0)
		 beq   $v0,$0,9f         	 # do div by zero trap
		 nop                    	 # Delay slot
		 addu  $t8,$t8,$s5
		 lw    $t0,0x00($t8)
		 divu  $0,$t0,$v0
		 mflo  $t1
		 srl   $t0,$t1,16
		 bne   $t0,$0,8f
		 mfhi  $v1               	 # Delay slot
		 sll   $v1,$v1,16
		 or    $v0,$t1,$v1
		 sw    $v0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$t1,15         	 # Set Sign
		 sltiu $s2,$t1,1         	 # Set Zero
	7:
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	8:
		 bgez  $0,7b
		 ori   $s1,$0,1         	 # Selay slot - V flag
	9:		 # Do divide by zero trap
		 addiu $t7,$t7,95
		 m_jal   Exception
		 ori   $v0,$0,5          	 # Delay slot

		 bgez  $0,7b
		 nop                    	 # Delay slot
OP0_80d0:				#:

		 addiu $s7,$s7,2

		 addiu $t7,$t7,-137
		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 beq   $v0,$0,9f         	 # do div by zero trap
		 nop                    	 # Delay slot
		 addu  $t8,$t8,$s5
		 lw    $t0,0x00($t8)
		 divu  $0,$t0,$v0
		 mflo  $t1
		 srl   $t0,$t1,16
		 bne   $t0,$0,8f
		 mfhi  $v1               	 # Delay slot
		 sll   $v1,$v1,16
		 or    $v0,$t1,$v1
		 sw    $v0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$t1,15         	 # Set Sign
		 sltiu $s2,$t1,1         	 # Set Zero
	7:
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	8:
		 bgez  $0,7b
		 ori   $s1,$0,1         	 # Selay slot - V flag
	9:		 # Do divide by zero trap
		 addiu $t7,$t7,95
		 m_jal   Exception
		 ori   $v0,$0,5          	 # Delay slot

		 bgez  $0,7b
		 nop                    	 # Delay slot
OP0_80d8:				#:

		 addiu $s7,$s7,2

		 addiu $t7,$t7,-137
		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 beq   $v0,$0,9f         	 # do div by zero trap
		 nop                    	 # Delay slot
		 addu  $t8,$t8,$s5
		 lw    $t0,0x00($t8)
		 divu  $0,$t0,$v0
		 mflo  $t1
		 srl   $t0,$t1,16
		 bne   $t0,$0,8f
		 mfhi  $v1               	 # Delay slot
		 sll   $v1,$v1,16
		 or    $v0,$t1,$v1
		 sw    $v0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$t1,15         	 # Set Sign
		 sltiu $s2,$t1,1         	 # Set Zero
	7:
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	8:
		 bgez  $0,7b
		 ori   $s1,$0,1         	 # Selay slot - V flag
	9:		 # Do divide by zero trap
		 addiu $t7,$t7,95
		 m_jal   Exception
		 ori   $v0,$0,5          	 # Delay slot

		 bgez  $0,7b
		 nop                    	 # Delay slot
OP0_80e0:				#:

		 addiu $s7,$s7,2

		 addiu $t7,$t7,-139
		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 beq   $v0,$0,9f         	 # do div by zero trap
		 nop                    	 # Delay slot
		 addu  $t8,$t8,$s5
		 lw    $t0,0x00($t8)
		 divu  $0,$t0,$v0
		 mflo  $t1
		 srl   $t0,$t1,16
		 bne   $t0,$0,8f
		 mfhi  $v1               	 # Delay slot
		 sll   $v1,$v1,16
		 or    $v0,$t1,$v1
		 sw    $v0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$t1,15         	 # Set Sign
		 sltiu $s2,$t1,1         	 # Set Zero
	7:
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	8:
		 bgez  $0,7b
		 ori   $s1,$0,1         	 # Selay slot - V flag
	9:		 # Do divide by zero trap
		 addiu $t7,$t7,95
		 m_jal   Exception
		 ori   $v0,$0,5          	 # Delay slot

		 bgez  $0,7b
		 nop                    	 # Delay slot
OP0_80e8:				#:

		 addiu $s7,$s7,2

		 addiu $t7,$t7,-141
		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 beq   $v0,$0,9f         	 # do div by zero trap
		 nop                    	 # Delay slot
		 addu  $t8,$t8,$s5
		 lw    $t0,0x00($t8)
		 divu  $0,$t0,$v0
		 mflo  $t1
		 srl   $t0,$t1,16
		 bne   $t0,$0,8f
		 mfhi  $v1               	 # Delay slot
		 sll   $v1,$v1,16
		 or    $v0,$t1,$v1
		 sw    $v0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$t1,15         	 # Set Sign
		 sltiu $s2,$t1,1         	 # Set Zero
	7:
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	8:
		 bgez  $0,7b
		 ori   $s1,$0,1         	 # Selay slot - V flag
	9:		 # Do divide by zero trap
		 addiu $t7,$t7,95
		 m_jal   Exception
		 ori   $v0,$0,5          	 # Delay slot

		 bgez  $0,7b
		 nop                    	 # Delay slot
OP0_80f0:				#:

		 addiu $s7,$s7,2

		 addiu $t7,$t7,-145
		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 beq   $v0,$0,9f         	 # do div by zero trap
		 nop                    	 # Delay slot
		 addu  $t8,$t8,$s5
		 lw    $t0,0x00($t8)
		 divu  $0,$t0,$v0
		 mflo  $t1
		 srl   $t0,$t1,16
		 bne   $t0,$0,8f
		 mfhi  $v1               	 # Delay slot
		 sll   $v1,$v1,16
		 or    $v0,$t1,$v1
		 sw    $v0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$t1,15         	 # Set Sign
		 sltiu $s2,$t1,1         	 # Set Zero
	7:
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	8:
		 bgez  $0,7b
		 ori   $s1,$0,1         	 # Selay slot - V flag
	9:		 # Do divide by zero trap
		 addiu $t7,$t7,95
		 m_jal   Exception
		 ori   $v0,$0,5          	 # Delay slot

		 bgez  $0,7b
		 nop                    	 # Delay slot
OP0_80f8:				#:

		 addiu $s7,$s7,2

		 addiu $t7,$t7,-141
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 beq   $v0,$0,9f         	 # do div by zero trap
		 nop                    	 # Delay slot
		 addu  $t8,$t8,$s5
		 lw    $t0,0x00($t8)
		 divu  $0,$t0,$v0
		 mflo  $t1
		 srl   $t0,$t1,16
		 bne   $t0,$0,8f
		 mfhi  $v1               	 # Delay slot
		 sll   $v1,$v1,16
		 or    $v0,$t1,$v1
		 sw    $v0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$t1,15         	 # Set Sign
		 sltiu $s2,$t1,1         	 # Set Zero
	7:
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	8:
		 bgez  $0,7b
		 ori   $s1,$0,1         	 # Selay slot - V flag
	9:		 # Do divide by zero trap
		 addiu $t7,$t7,95
		 m_jal   Exception
		 ori   $v0,$0,5          	 # Delay slot

		 bgez  $0,7b
		 nop                    	 # Delay slot
OP0_80f9:				#:

		 addiu $s7,$s7,2

		 addiu $t7,$t7,-145
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 beq   $v0,$0,9f         	 # do div by zero trap
		 nop                    	 # Delay slot
		 addu  $t8,$t8,$s5
		 lw    $t0,0x00($t8)
		 divu  $0,$t0,$v0
		 mflo  $t1
		 srl   $t0,$t1,16
		 bne   $t0,$0,8f
		 mfhi  $v1               	 # Delay slot
		 sll   $v1,$v1,16
		 or    $v0,$t1,$v1
		 sw    $v0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$t1,15         	 # Set Sign
		 sltiu $s2,$t1,1         	 # Set Zero
	7:
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	8:
		 bgez  $0,7b
		 ori   $s1,$0,1         	 # Selay slot - V flag
	9:		 # Do divide by zero trap
		 addiu $t7,$t7,95
		 m_jal   Exception
		 ori   $v0,$0,5          	 # Delay slot

		 bgez  $0,7b
		 nop                    	 # Delay slot
OP0_80fa:				#:

		 addiu $s7,$s7,2

		 addiu $t7,$t7,-141
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 beq   $v0,$0,9f         	 # do div by zero trap
		 nop                    	 # Delay slot
		 addu  $t8,$t8,$s5
		 lw    $t0,0x00($t8)
		 divu  $0,$t0,$v0
		 mflo  $t1
		 srl   $t0,$t1,16
		 bne   $t0,$0,8f
		 mfhi  $v1               	 # Delay slot
		 sll   $v1,$v1,16
		 or    $v0,$t1,$v1
		 sw    $v0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$t1,15         	 # Set Sign
		 sltiu $s2,$t1,1         	 # Set Zero
	7:
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	8:
		 bgez  $0,7b
		 ori   $s1,$0,1         	 # Selay slot - V flag
	9:		 # Do divide by zero trap
		 addiu $t7,$t7,95
		 m_jal   Exception
		 ori   $v0,$0,5          	 # Delay slot

		 bgez  $0,7b
		 nop                    	 # Delay slot
OP0_80fb:				#:

		 addiu $s7,$s7,2

		 addiu $t7,$t7,-143
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 beq   $v0,$0,9f         	 # do div by zero trap
		 nop                    	 # Delay slot
		 addu  $t8,$t8,$s5
		 lw    $t0,0x00($t8)
		 divu  $0,$t0,$v0
		 mflo  $t1
		 srl   $t0,$t1,16
		 bne   $t0,$0,8f
		 mfhi  $v1               	 # Delay slot
		 sll   $v1,$v1,16
		 or    $v0,$t1,$v1
		 sw    $v0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$t1,15         	 # Set Sign
		 sltiu $s2,$t1,1         	 # Set Zero
	7:
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	8:
		 bgez  $0,7b
		 ori   $s1,$0,1         	 # Selay slot - V flag
	9:		 # Do divide by zero trap
		 addiu $t7,$t7,95
		 m_jal   Exception
		 ori   $v0,$0,5          	 # Delay slot

		 bgez  $0,7b
		 nop                    	 # Delay slot
OP0_80fc:				#:

		 addiu $s7,$s7,2

		 addiu $t7,$t7,-137
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lhu   $v0,0x00($s7)
		 addiu $s7,$s7,2
		 beq   $v0,$0,9f         	 # do div by zero trap
		 nop                    	 # Delay slot
		 addu  $t8,$t8,$s5
		 lw    $t0,0x00($t8)
		 divu  $0,$t0,$v0
		 mflo  $t1
		 srl   $t0,$t1,16
		 bne   $t0,$0,8f
		 mfhi  $v1               	 # Delay slot
		 sll   $v1,$v1,16
		 or    $v0,$t1,$v1
		 sw    $v0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$t1,15         	 # Set Sign
		 sltiu $s2,$t1,1         	 # Set Zero
	7:
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	8:
		 bgez  $0,7b
		 ori   $s1,$0,1         	 # Selay slot - V flag
	9:		 # Do divide by zero trap
		 addiu $t7,$t7,95
		 m_jal   Exception
		 ori   $v0,$0,5          	 # Delay slot

		 bgez  $0,7b
		 nop                    	 # Delay slot
OP0_81c0:				#:

		 addiu $s7,$s7,2

		 addiu $t7,$t7,-150
		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lh    $v0,0x00($t0)
		 beq   $v0,$0,9f         	 # do div by zero trap
		 nop                    	 # Delay slot
		 addu  $t8,$t8,$s5
		 lw    $t0,0x00($t8)
		 div   $0,$t0,$v0
		 mflo  $t1
		 sll   $t0,$t1,16
		 sra   $t0,$t0,16
		 bne   $t0,$t1,8f
		 andi  $t1,$t1,0xffff     	 # Delay slot
		 mfhi  $v1
		 sll   $v1,$v1,16
		 or    $v0,$t1,$v1
		 sw    $v0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$t1,15         	 # Set Sign
		 sltiu $s2,$t1,1         	 # Set Zero
	7:
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	8:
		 bgez  $0,7b
		 ori   $s1,$0,1         	 # Selay slot - V flag
	9:		 # Do divide by zero trap
		 addiu $t7,$t7,112
		 m_jal   Exception
		 ori   $v0,$0,5          	 # Delay slot

		 bgez  $0,7b
		 nop                    	 # Delay slot
OP0_81d0:				#:

		 addiu $s7,$s7,2

		 addiu $t7,$t7,-154
		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 beq   $v0,$0,9f         	 # do div by zero trap
		 nop                    	 # Delay slot
		 addu  $t8,$t8,$s5
		 lw    $t0,0x00($t8)
		 div   $0,$t0,$v0
		 mflo  $t1
		 sll   $t0,$t1,16
		 sra   $t0,$t0,16
		 bne   $t0,$t1,8f
		 andi  $t1,$t1,0xffff     	 # Delay slot
		 mfhi  $v1
		 sll   $v1,$v1,16
		 or    $v0,$t1,$v1
		 sw    $v0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$t1,15         	 # Set Sign
		 sltiu $s2,$t1,1         	 # Set Zero
	7:
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	8:
		 bgez  $0,7b
		 ori   $s1,$0,1         	 # Selay slot - V flag
	9:		 # Do divide by zero trap
		 addiu $t7,$t7,112
		 m_jal   Exception
		 ori   $v0,$0,5          	 # Delay slot

		 bgez  $0,7b
		 nop                    	 # Delay slot
OP0_81d8:				#:

		 addiu $s7,$s7,2

		 addiu $t7,$t7,-154
		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 beq   $v0,$0,9f         	 # do div by zero trap
		 nop                    	 # Delay slot
		 addu  $t8,$t8,$s5
		 lw    $t0,0x00($t8)
		 div   $0,$t0,$v0
		 mflo  $t1
		 sll   $t0,$t1,16
		 sra   $t0,$t0,16
		 bne   $t0,$t1,8f
		 andi  $t1,$t1,0xffff     	 # Delay slot
		 mfhi  $v1
		 sll   $v1,$v1,16
		 or    $v0,$t1,$v1
		 sw    $v0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$t1,15         	 # Set Sign
		 sltiu $s2,$t1,1         	 # Set Zero
	7:
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	8:
		 bgez  $0,7b
		 ori   $s1,$0,1         	 # Selay slot - V flag
	9:		 # Do divide by zero trap
		 addiu $t7,$t7,112
		 m_jal   Exception
		 ori   $v0,$0,5          	 # Delay slot

		 bgez  $0,7b
		 nop                    	 # Delay slot
OP0_81e0:				#:

		 addiu $s7,$s7,2

		 addiu $t7,$t7,-156
		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 beq   $v0,$0,9f         	 # do div by zero trap
		 nop                    	 # Delay slot
		 addu  $t8,$t8,$s5
		 lw    $t0,0x00($t8)
		 div   $0,$t0,$v0
		 mflo  $t1
		 sll   $t0,$t1,16
		 sra   $t0,$t0,16
		 bne   $t0,$t1,8f
		 andi  $t1,$t1,0xffff     	 # Delay slot
		 mfhi  $v1
		 sll   $v1,$v1,16
		 or    $v0,$t1,$v1
		 sw    $v0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$t1,15         	 # Set Sign
		 sltiu $s2,$t1,1         	 # Set Zero
	7:
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	8:
		 bgez  $0,7b
		 ori   $s1,$0,1         	 # Selay slot - V flag
	9:		 # Do divide by zero trap
		 addiu $t7,$t7,112
		 m_jal   Exception
		 ori   $v0,$0,5          	 # Delay slot

		 bgez  $0,7b
		 nop                    	 # Delay slot
OP0_81e8:				#:

		 addiu $s7,$s7,2

		 addiu $t7,$t7,-158
		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 beq   $v0,$0,9f         	 # do div by zero trap
		 nop                    	 # Delay slot
		 addu  $t8,$t8,$s5
		 lw    $t0,0x00($t8)
		 div   $0,$t0,$v0
		 mflo  $t1
		 sll   $t0,$t1,16
		 sra   $t0,$t0,16
		 bne   $t0,$t1,8f
		 andi  $t1,$t1,0xffff     	 # Delay slot
		 mfhi  $v1
		 sll   $v1,$v1,16
		 or    $v0,$t1,$v1
		 sw    $v0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$t1,15         	 # Set Sign
		 sltiu $s2,$t1,1         	 # Set Zero
	7:
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	8:
		 bgez  $0,7b
		 ori   $s1,$0,1         	 # Selay slot - V flag
	9:		 # Do divide by zero trap
		 addiu $t7,$t7,112
		 m_jal   Exception
		 ori   $v0,$0,5          	 # Delay slot

		 bgez  $0,7b
		 nop                    	 # Delay slot
OP0_81f0:				#:

		 addiu $s7,$s7,2

		 addiu $t7,$t7,-162
		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 beq   $v0,$0,9f         	 # do div by zero trap
		 nop                    	 # Delay slot
		 addu  $t8,$t8,$s5
		 lw    $t0,0x00($t8)
		 div   $0,$t0,$v0
		 mflo  $t1
		 sll   $t0,$t1,16
		 sra   $t0,$t0,16
		 bne   $t0,$t1,8f
		 andi  $t1,$t1,0xffff     	 # Delay slot
		 mfhi  $v1
		 sll   $v1,$v1,16
		 or    $v0,$t1,$v1
		 sw    $v0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$t1,15         	 # Set Sign
		 sltiu $s2,$t1,1         	 # Set Zero
	7:
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	8:
		 bgez  $0,7b
		 ori   $s1,$0,1         	 # Selay slot - V flag
	9:		 # Do divide by zero trap
		 addiu $t7,$t7,112
		 m_jal   Exception
		 ori   $v0,$0,5          	 # Delay slot

		 bgez  $0,7b
		 nop                    	 # Delay slot
OP0_81f8:				#:

		 addiu $s7,$s7,2

		 addiu $t7,$t7,-158
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 beq   $v0,$0,9f         	 # do div by zero trap
		 nop                    	 # Delay slot
		 addu  $t8,$t8,$s5
		 lw    $t0,0x00($t8)
		 div   $0,$t0,$v0
		 mflo  $t1
		 sll   $t0,$t1,16
		 sra   $t0,$t0,16
		 bne   $t0,$t1,8f
		 andi  $t1,$t1,0xffff     	 # Delay slot
		 mfhi  $v1
		 sll   $v1,$v1,16
		 or    $v0,$t1,$v1
		 sw    $v0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$t1,15         	 # Set Sign
		 sltiu $s2,$t1,1         	 # Set Zero
	7:
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	8:
		 bgez  $0,7b
		 ori   $s1,$0,1         	 # Selay slot - V flag
	9:		 # Do divide by zero trap
		 addiu $t7,$t7,112
		 m_jal   Exception
		 ori   $v0,$0,5          	 # Delay slot

		 bgez  $0,7b
		 nop                    	 # Delay slot
OP0_81f9:				#:

		 addiu $s7,$s7,2

		 addiu $t7,$t7,-162
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 beq   $v0,$0,9f         	 # do div by zero trap
		 nop                    	 # Delay slot
		 addu  $t8,$t8,$s5
		 lw    $t0,0x00($t8)
		 div   $0,$t0,$v0
		 mflo  $t1
		 sll   $t0,$t1,16
		 sra   $t0,$t0,16
		 bne   $t0,$t1,8f
		 andi  $t1,$t1,0xffff     	 # Delay slot
		 mfhi  $v1
		 sll   $v1,$v1,16
		 or    $v0,$t1,$v1
		 sw    $v0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$t1,15         	 # Set Sign
		 sltiu $s2,$t1,1         	 # Set Zero
	7:
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	8:
		 bgez  $0,7b
		 ori   $s1,$0,1         	 # Selay slot - V flag
	9:		 # Do divide by zero trap
		 addiu $t7,$t7,112
		 m_jal   Exception
		 ori   $v0,$0,5          	 # Delay slot

		 bgez  $0,7b
		 nop                    	 # Delay slot
OP0_81fa:				#:

		 addiu $s7,$s7,2

		 addiu $t7,$t7,-158
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 beq   $v0,$0,9f         	 # do div by zero trap
		 nop                    	 # Delay slot
		 addu  $t8,$t8,$s5
		 lw    $t0,0x00($t8)
		 div   $0,$t0,$v0
		 mflo  $t1
		 sll   $t0,$t1,16
		 sra   $t0,$t0,16
		 bne   $t0,$t1,8f
		 andi  $t1,$t1,0xffff     	 # Delay slot
		 mfhi  $v1
		 sll   $v1,$v1,16
		 or    $v0,$t1,$v1
		 sw    $v0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$t1,15         	 # Set Sign
		 sltiu $s2,$t1,1         	 # Set Zero
	7:
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	8:
		 bgez  $0,7b
		 ori   $s1,$0,1         	 # Selay slot - V flag
	9:		 # Do divide by zero trap
		 addiu $t7,$t7,112
		 m_jal   Exception
		 ori   $v0,$0,5          	 # Delay slot

		 bgez  $0,7b
		 nop                    	 # Delay slot
OP0_81fb:				#:

		 addiu $s7,$s7,2

		 addiu $t7,$t7,-160
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $t8,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t8,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 beq   $v0,$0,9f         	 # do div by zero trap
		 nop                    	 # Delay slot
		 addu  $t8,$t8,$s5
		 lw    $t0,0x00($t8)
		 div   $0,$t0,$v0
		 mflo  $t1
		 sll   $t0,$t1,16
		 sra   $t0,$t0,16
		 bne   $t0,$t1,8f
		 andi  $t1,$t1,0xffff     	 # Delay slot
		 mfhi  $v1
		 sll   $v1,$v1,16
		 or    $v0,$t1,$v1
		 sw    $v0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$t1,15         	 # Set Sign
		 sltiu $s2,$t1,1         	 # Set Zero
	7:
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	8:
		 bgez  $0,7b
		 ori   $s1,$0,1         	 # Selay slot - V flag
	9:		 # Do divide by zero trap
		 addiu $t7,$t7,112
		 m_jal   Exception
		 ori   $v0,$0,5          	 # Delay slot

		 bgez  $0,7b
		 nop                    	 # Delay slot
OP0_81fc:				#:

		 addiu $s7,$s7,2

		 addiu $t7,$t7,-154
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lh    $v0,0x00($s7)
		 addiu $s7,$s7,2
		 beq   $v0,$0,9f         	 # do div by zero trap
		 nop                    	 # Delay slot
		 addu  $t8,$t8,$s5
		 lw    $t0,0x00($t8)
		 div   $0,$t0,$v0
		 mflo  $t1
		 sll   $t0,$t1,16
		 sra   $t0,$t0,16
		 bne   $t0,$t1,8f
		 andi  $t1,$t1,0xffff     	 # Delay slot
		 mfhi  $v1
		 sll   $v1,$v1,16
		 or    $v0,$t1,$v1
		 sw    $v0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$t1,15         	 # Set Sign
		 sltiu $s2,$t1,1         	 # Set Zero
	7:
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	8:
		 bgez  $0,7b
		 ori   $s1,$0,1         	 # Selay slot - V flag
	9:		 # Do divide by zero trap
		 addiu $t7,$t7,112
		 m_jal   Exception
		 ori   $v0,$0,5          	 # Delay slot

		 bgez  $0,7b
		 nop                    	 # Delay slot
OP0_4840:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $v0,0x00($t8)
		 m_ror   $v0,$v0,16
		 sw    $v0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4000:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lb    $t0,0x00($t8)
		 addu  $v0,$t0,$s4
		 subu  $v0,$0,$v0
		 addu  $t9,$t0,$s4
		 sltu  $s0,$zero,$t9       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $t9,$t9,1
		 and   $s2,$s2,$t9      	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 sb    $v0,0x00($t8)
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4010:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t0,$v0
		 addu  $v0,$t0,$s4
		 subu  $v0,$0,$v0
		 addu  $t9,$t0,$s4
		 sltu  $s0,$zero,$t9       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $t9,$t9,1
		 and   $s2,$s2,$t9      	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4018:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t8)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t0,$v0
		 addu  $v0,$t0,$s4
		 subu  $v0,$0,$v0
		 addu  $t9,$t0,$s4
		 sltu  $s0,$zero,$t9       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $t9,$t9,1
		 and   $s2,$s2,$t9      	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_401f:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t0,$v0
		 addu  $v0,$t0,$s4
		 subu  $v0,$0,$v0
		 addu  $t9,$t0,$s4
		 sltu  $s0,$zero,$t9       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $t9,$t9,1
		 and   $s2,$s2,$t9      	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4020:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t0,$v0
		 addu  $v0,$t0,$s4
		 subu  $v0,$0,$v0
		 addu  $t9,$t0,$s4
		 sltu  $s0,$zero,$t9       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $t9,$t9,1
		 and   $s2,$s2,$t9      	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4027:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t0,$v0
		 addu  $v0,$t0,$s4
		 subu  $v0,$0,$v0
		 addu  $t9,$t0,$s4
		 sltu  $s0,$zero,$t9       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $t9,$t9,1
		 and   $s2,$s2,$t9      	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4028:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t0,$v0
		 addu  $v0,$t0,$s4
		 subu  $v0,$0,$v0
		 addu  $t9,$t0,$s4
		 sltu  $s0,$zero,$t9       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $t9,$t9,1
		 and   $s2,$s2,$t9      	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4030:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t0,$v0
		 addu  $v0,$t0,$s4
		 subu  $v0,$0,$v0
		 addu  $t9,$t0,$s4
		 sltu  $s0,$zero,$t9       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $t9,$t9,1
		 and   $s2,$s2,$t9      	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4038:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t0,$v0
		 addu  $v0,$t0,$s4
		 subu  $v0,$0,$v0
		 addu  $t9,$t0,$s4
		 sltu  $s0,$zero,$t9       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $t9,$t9,1
		 and   $s2,$s2,$t9      	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4039:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t0,$v0
		 addu  $v0,$t0,$s4
		 subu  $v0,$0,$v0
		 addu  $t9,$t0,$s4
		 sltu  $s0,$zero,$t9       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $t9,$t9,1
		 and   $s2,$s2,$t9      	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4040:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lh    $t0,0x00($t8)
		 addu  $v0,$t0,$s4
		 subu  $v0,$0,$v0
		 addu  $t9,$t0,$s4
		 sltu  $s0,$zero,$t9       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $t9,$t9,1
		 and   $s2,$s2,$t9      	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 sh    $v0,0x00($t8)
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4050:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t0,$v0
		 addu  $v0,$t0,$s4
		 subu  $v0,$0,$v0
		 addu  $t9,$t0,$s4
		 sltu  $s0,$zero,$t9       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $t9,$t9,1
		 and   $s2,$s2,$t9      	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4058:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t0,$v0
		 addu  $v0,$t0,$s4
		 subu  $v0,$0,$v0
		 addu  $t9,$t0,$s4
		 sltu  $s0,$zero,$t9       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $t9,$t9,1
		 and   $s2,$s2,$t9      	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4060:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t0,$v0
		 addu  $v0,$t0,$s4
		 subu  $v0,$0,$v0
		 addu  $t9,$t0,$s4
		 sltu  $s0,$zero,$t9       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $t9,$t9,1
		 and   $s2,$s2,$t9      	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4068:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t0,$v0
		 addu  $v0,$t0,$s4
		 subu  $v0,$0,$v0
		 addu  $t9,$t0,$s4
		 sltu  $s0,$zero,$t9       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $t9,$t9,1
		 and   $s2,$s2,$t9      	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4070:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t0,$v0
		 addu  $v0,$t0,$s4
		 subu  $v0,$0,$v0
		 addu  $t9,$t0,$s4
		 sltu  $s0,$zero,$t9       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $t9,$t9,1
		 and   $s2,$s2,$t9      	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4078:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t0,$v0
		 addu  $v0,$t0,$s4
		 subu  $v0,$0,$v0
		 addu  $t9,$t0,$s4
		 sltu  $s0,$zero,$t9       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $t9,$t9,1
		 and   $s2,$s2,$t9      	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4079:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t0,$v0
		 addu  $v0,$t0,$s4
		 subu  $v0,$0,$v0
		 addu  $t9,$t0,$s4
		 sltu  $s0,$zero,$t9       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $t9,$t9,1
		 and   $s2,$s2,$t9      	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4080:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t0,0x00($t8)
		 addu  $v0,$t0,$s4
		 subu  $v0,$0,$v0
		 addu  $t9,$t0,$s4
		 sltu  $s0,$zero,$t9       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $t9,$v0,1
		 and   $s2,$s2,$t9      	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 sw    $v0,0x00($t8)
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4090:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t0,$0,$v0
		 addu  $v0,$t0,$s4
		 subu  $v0,$0,$v0
		 addu  $t9,$t0,$s4
		 sltu  $s0,$zero,$t9       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $t9,$v0,1
		 and   $s2,$s2,$t9      	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4098:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($t8)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t0,$0,$v0
		 addu  $v0,$t0,$s4
		 subu  $v0,$0,$v0
		 addu  $t9,$t0,$s4
		 sltu  $s0,$zero,$t9       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $t9,$v0,1
		 and   $s2,$s2,$t9      	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_40a0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($t8)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t0,$0,$v0
		 addu  $v0,$t0,$s4
		 subu  $v0,$0,$v0
		 addu  $t9,$t0,$s4
		 sltu  $s0,$zero,$t9       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $t9,$v0,1
		 and   $s2,$s2,$t9      	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_40a8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t0,$0,$v0
		 addu  $v0,$t0,$s4
		 subu  $v0,$0,$v0
		 addu  $t9,$t0,$s4
		 sltu  $s0,$zero,$t9       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $t9,$v0,1
		 and   $s2,$s2,$t9      	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_40b0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t0,$0,$v0
		 addu  $v0,$t0,$s4
		 subu  $v0,$0,$v0
		 addu  $t9,$t0,$s4
		 sltu  $s0,$zero,$t9       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $t9,$v0,1
		 and   $s2,$s2,$t9      	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_40b8:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t0,$0,$v0
		 addu  $v0,$t0,$s4
		 subu  $v0,$0,$v0
		 addu  $t9,$t0,$s4
		 sltu  $s0,$zero,$t9       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $t9,$v0,1
		 and   $s2,$s2,$t9      	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_40b9:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t0,$0,$v0
		 addu  $v0,$t0,$s4
		 subu  $v0,$0,$v0
		 addu  $t9,$t0,$s4
		 sltu  $s0,$zero,$t9       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $t9,$v0,1
		 and   $s2,$s2,$t9      	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4200:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 and   $v0,$0,$0
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 sb    $v0,0x00($t8)
		 and   $s0,$0,$0
		 and   $s1,$0,$0
		 ori   $s2,$0,1
		 and   $s3,$0,$0
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4210:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 and   $v0,$0,$0
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0
		 and   $s1,$0,$0
		 ori   $s2,$0,1
		 and   $s3,$0,$0
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4218:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 and   $v0,$0,$0
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t8)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0
		 and   $s1,$0,$0
		 ori   $s2,$0,1
		 and   $s3,$0,$0
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_421f:				#:
		 addiu $s7,$s7,2

		 and   $v0,$0,$0
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0
		 and   $s1,$0,$0
		 ori   $s2,$0,1
		 and   $s3,$0,$0
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4220:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 and   $v0,$0,$0
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0
		 and   $s1,$0,$0
		 ori   $s2,$0,1
		 and   $s3,$0,$0
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4227:				#:
		 addiu $s7,$s7,2

		 and   $v0,$0,$0
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0
		 and   $s1,$0,$0
		 ori   $s2,$0,1
		 and   $s3,$0,$0
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4228:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 and   $v0,$0,$0
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0
		 and   $s1,$0,$0
		 ori   $s2,$0,1
		 and   $s3,$0,$0
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4230:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 and   $v0,$0,$0
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0
		 and   $s1,$0,$0
		 ori   $s2,$0,1
		 and   $s3,$0,$0
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4238:				#:
		 addiu $s7,$s7,2

		 and   $v0,$0,$0
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0
		 and   $s1,$0,$0
		 ori   $s2,$0,1
		 and   $s3,$0,$0
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4239:				#:
		 addiu $s7,$s7,2

		 and   $v0,$0,$0
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0
		 and   $s1,$0,$0
		 ori   $s2,$0,1
		 and   $s3,$0,$0
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4240:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 and   $v0,$0,$0
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 sh    $v0,0x00($t8)
		 and   $s0,$0,$0
		 and   $s1,$0,$0
		 ori   $s2,$0,1
		 and   $s3,$0,$0
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4250:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 and   $v0,$0,$0
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0
		 and   $s1,$0,$0
		 ori   $s2,$0,1
		 and   $s3,$0,$0
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4258:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 and   $v0,$0,$0
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0
		 and   $s1,$0,$0
		 ori   $s2,$0,1
		 and   $s3,$0,$0
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4260:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 and   $v0,$0,$0
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t8)
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0
		 and   $s1,$0,$0
		 ori   $s2,$0,1
		 and   $s3,$0,$0
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4268:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 and   $v0,$0,$0
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0
		 and   $s1,$0,$0
		 ori   $s2,$0,1
		 and   $s3,$0,$0
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4270:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 and   $v0,$0,$0
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0
		 and   $s1,$0,$0
		 ori   $s2,$0,1
		 and   $s3,$0,$0
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4278:				#:
		 addiu $s7,$s7,2

		 and   $v0,$0,$0
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0
		 and   $s1,$0,$0
		 ori   $s2,$0,1
		 and   $s3,$0,$0
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4279:				#:
		 addiu $s7,$s7,2

		 and   $v0,$0,$0
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0
		 and   $s1,$0,$0
		 ori   $s2,$0,1
		 and   $s3,$0,$0
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4280:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 and   $v0,$0,$0
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 sw    $v0,0x00($t8)
		 and   $s0,$0,$0
		 and   $s1,$0,$0
		 ori   $s2,$0,1
		 and   $s3,$0,$0
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4290:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 and   $v0,$0,$0
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0
		 and   $s1,$0,$0
		 ori   $s2,$0,1
		 and   $s3,$0,$0
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4298:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 and   $v0,$0,$0
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($t8)
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0
		 and   $s1,$0,$0
		 ori   $s2,$0,1
		 and   $s3,$0,$0
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_42a0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 and   $v0,$0,$0
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($t8)
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0
		 and   $s1,$0,$0
		 ori   $s2,$0,1
		 and   $s3,$0,$0
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_42a8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 and   $v0,$0,$0
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0
		 and   $s1,$0,$0
		 ori   $s2,$0,1
		 and   $s3,$0,$0
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_42b0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 and   $v0,$0,$0
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0
		 and   $s1,$0,$0
		 ori   $s2,$0,1
		 and   $s3,$0,$0
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_42b8:				#:
		 addiu $s7,$s7,2

		 and   $v0,$0,$0
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0
		 and   $s1,$0,$0
		 ori   $s2,$0,1
		 and   $s3,$0,$0
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_42b9:				#:
		 addiu $s7,$s7,2

		 and   $v0,$0,$0
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0
		 and   $s1,$0,$0
		 ori   $s2,$0,1
		 and   $s3,$0,$0
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4400:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lb    $t0,0x00($t8)
		 subu  $v0,$0,$t0
		 sltu  $s0,$zero,$t0       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 sb    $v0,0x00($t8)
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4410:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t0,$v0
		 subu  $v0,$0,$t0
		 sltu  $s0,$zero,$t0       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4418:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t8)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t0,$v0
		 subu  $v0,$0,$t0
		 sltu  $s0,$zero,$t0       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_441f:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t0,$v0
		 subu  $v0,$0,$t0
		 sltu  $s0,$zero,$t0       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4420:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t0,$v0
		 subu  $v0,$0,$t0
		 sltu  $s0,$zero,$t0       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4427:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t0,$v0
		 subu  $v0,$0,$t0
		 sltu  $s0,$zero,$t0       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4428:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t0,$v0
		 subu  $v0,$0,$t0
		 sltu  $s0,$zero,$t0       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4430:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t0,$v0
		 subu  $v0,$0,$t0
		 sltu  $s0,$zero,$t0       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4438:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t0,$v0
		 subu  $v0,$0,$t0
		 sltu  $s0,$zero,$t0       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4439:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t0,$v0
		 subu  $v0,$0,$t0
		 sltu  $s0,$zero,$t0       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4440:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lh    $t0,0x00($t8)
		 subu  $v0,$0,$t0
		 sltu  $s0,$zero,$t0       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 sh    $v0,0x00($t8)
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4450:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t0,$v0
		 subu  $v0,$0,$t0
		 sltu  $s0,$zero,$t0       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4458:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t0,$v0
		 subu  $v0,$0,$t0
		 sltu  $s0,$zero,$t0       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4460:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t0,$v0
		 subu  $v0,$0,$t0
		 sltu  $s0,$zero,$t0       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4468:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t0,$v0
		 subu  $v0,$0,$t0
		 sltu  $s0,$zero,$t0       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4470:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t0,$v0
		 subu  $v0,$0,$t0
		 sltu  $s0,$zero,$t0       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4478:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t0,$v0
		 subu  $v0,$0,$t0
		 sltu  $s0,$zero,$t0       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4479:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t0,$v0
		 subu  $v0,$0,$t0
		 sltu  $s0,$zero,$t0       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v0
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4480:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t0,0x00($t8)
		 subu  $v0,$0,$t0
		 sltu  $s0,$zero,$t0       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 sw    $v0,0x00($t8)
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4490:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t0,$0,$v0
		 subu  $v0,$0,$t0
		 sltu  $s0,$zero,$t0       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4498:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($t8)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t0,$0,$v0
		 subu  $v0,$0,$t0
		 sltu  $s0,$zero,$t0       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_44a0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($t8)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t0,$0,$v0
		 subu  $v0,$0,$t0
		 sltu  $s0,$zero,$t0       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_44a8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t0,$0,$v0
		 subu  $v0,$0,$t0
		 sltu  $s0,$zero,$t0       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_44b0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t0,$0,$v0
		 subu  $v0,$0,$t0
		 sltu  $s0,$zero,$t0       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_44b8:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t0,$0,$v0
		 subu  $v0,$0,$t0
		 sltu  $s0,$zero,$t0       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_44b9:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t0,$0,$v0
		 subu  $v0,$0,$t0
		 sltu  $s0,$zero,$t0       	 # Set Carry
		 xor   $s1,$zero,$t0
		 xor   $t9,$v0,$zero
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $s4,$0,$s0      	 # Copy Carry to X
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4600:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lb    $v0,0x00($t8)
		 nor   $v0,$0,$v0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 sb    $v0,0x00($t8)
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4610:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 nor   $v0,$0,$v0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4618:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t8)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 nor   $v0,$0,$v0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_461f:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 nor   $v0,$0,$v0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4620:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 nor   $v0,$0,$v0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4627:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 nor   $v0,$0,$v0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4628:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 nor   $v0,$0,$v0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4630:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 nor   $v0,$0,$v0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4638:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 nor   $v0,$0,$v0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4639:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 nor   $v0,$0,$v0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4640:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lh    $v0,0x00($t8)
		 nor   $v0,$0,$v0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 sh    $v0,0x00($t8)
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4650:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 nor   $v0,$0,$v0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4658:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 nor   $v0,$0,$v0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4660:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 nor   $v0,$0,$v0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4668:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 nor   $v0,$0,$v0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4670:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 nor   $v0,$0,$v0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4678:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 nor   $v0,$0,$v0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4679:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 nor   $v0,$0,$v0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4680:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $v0,0x00($t8)
		 nor   $v0,$0,$v0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 sw    $v0,0x00($t8)
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4690:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 nor   $v0,$0,$v0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4698:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($t8)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 nor   $v0,$0,$v0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_46a0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($t8)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 nor   $v0,$0,$v0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_46a8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 nor   $v0,$0,$v0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_46b0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 nor   $v0,$0,$v0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_46b8:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 nor   $v0,$0,$v0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_46b9:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 nor   $v0,$0,$v0
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4e60:				#
		 lbu   $t0,0x44($s5)
		 andi  $t0,$t0,0x20       	 # Supervisor Mode ?
		 beq   $t0,$0,9f
		 addiu $s7,$s7,2        	 # Delay slot

		 andi  $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $v0,0x20($t0)
		 sw    $v0,0x78($s5)
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	9:
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,8

		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4e68:				#
		 lbu   $t0,0x44($s5)
		 andi  $t0,$t0,0x20       	 # Supervisor Mode ?
		 beq   $t0,$0,9f
		 addiu $s7,$s7,2        	 # Delay slot

		 andi  $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $v0,0x78($s5)
		 sw    $v0,0x20($t0)
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	9:
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,8

		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4180:				#:
		 addiu $s7,$s7,2

		 srl   $t0,$t8,7
		 andi  $t0,$t0,0x1C
		 addu  $t0,$t0,$s5
		 andi  $t8,$t8,0x07
		 lh    $t0,0x00($t0)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lh    $v0,0x00($t8)
		 and   $s0,$0,$0
		 and   $s1,$0,$0
		 sltiu $s2,$t0,1
		 slt   $s3,$t0,$0
		 bne   $s3,$0,8f
		 slt   $t0,$v0,$t0         	 # Delay slot
		 bne   $t0,$0,9f
		 nop                    	 # Delay slot
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	9:
		 and   $s3,$0,$0
	8:
		 m_jal   Exception
		 ori   $v0,$0,6          	 # Delay slot

		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4190:				#:
		 addiu $s7,$s7,2

		 srl   $t0,$t8,7
		 andi  $t0,$t0,0x1C
		 addu  $t0,$t0,$s5
		 andi  $t8,$t8,0x07
		 lh    $t0,0x00($t0)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 and   $s0,$0,$0
		 and   $s1,$0,$0
		 sltiu $s2,$t0,1
		 slt   $s3,$t0,$0
		 bne   $s3,$0,8f
		 slt   $t0,$v0,$t0         	 # Delay slot
		 bne   $t0,$0,9f
		 nop                    	 # Delay slot
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	9:
		 and   $s3,$0,$0
	8:
		 m_jal   Exception
		 ori   $v0,$0,6          	 # Delay slot

		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4198:				#:
		 addiu $s7,$s7,2

		 srl   $t0,$t8,7
		 andi  $t0,$t0,0x1C
		 addu  $t0,$t0,$s5
		 andi  $t8,$t8,0x07
		 lh    $t0,0x00($t0)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 and   $s0,$0,$0
		 and   $s1,$0,$0
		 sltiu $s2,$t0,1
		 slt   $s3,$t0,$0
		 bne   $s3,$0,8f
		 slt   $t0,$v0,$t0         	 # Delay slot
		 bne   $t0,$0,9f
		 nop                    	 # Delay slot
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	9:
		 and   $s3,$0,$0
	8:
		 m_jal   Exception
		 ori   $v0,$0,6          	 # Delay slot

		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_41a0:				#:
		 addiu $s7,$s7,2

		 srl   $t0,$t8,7
		 andi  $t0,$t0,0x1C
		 addu  $t0,$t0,$s5
		 andi  $t8,$t8,0x07
		 lh    $t0,0x00($t0)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 and   $s0,$0,$0
		 and   $s1,$0,$0
		 sltiu $s2,$t0,1
		 slt   $s3,$t0,$0
		 bne   $s3,$0,8f
		 slt   $t0,$v0,$t0         	 # Delay slot
		 bne   $t0,$0,9f
		 nop                    	 # Delay slot
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	9:
		 and   $s3,$0,$0
	8:
		 m_jal   Exception
		 ori   $v0,$0,6          	 # Delay slot

		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_41a8:				#:
		 addiu $s7,$s7,2

		 srl   $t0,$t8,7
		 andi  $t0,$t0,0x1C
		 addu  $t0,$t0,$s5
		 andi  $t8,$t8,0x07
		 lh    $t0,0x00($t0)
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 and   $s0,$0,$0
		 and   $s1,$0,$0
		 sltiu $s2,$t0,1
		 slt   $s3,$t0,$0
		 bne   $s3,$0,8f
		 slt   $t0,$v0,$t0         	 # Delay slot
		 bne   $t0,$0,9f
		 nop                    	 # Delay slot
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	9:
		 and   $s3,$0,$0
	8:
		 m_jal   Exception
		 ori   $v0,$0,6          	 # Delay slot

		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_41b0:				#:
		 addiu $s7,$s7,2

		 srl   $t0,$t8,7
		 andi  $t0,$t0,0x1C
		 addu  $t0,$t0,$s5
		 andi  $t8,$t8,0x07
		 lh    $t0,0x00($t0)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 and   $s0,$0,$0
		 and   $s1,$0,$0
		 sltiu $s2,$t0,1
		 slt   $s3,$t0,$0
		 bne   $s3,$0,8f
		 slt   $t0,$v0,$t0         	 # Delay slot
		 bne   $t0,$0,9f
		 nop                    	 # Delay slot
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	9:
		 and   $s3,$0,$0
	8:
		 m_jal   Exception
		 ori   $v0,$0,6          	 # Delay slot

		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_41b8:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t8,0x00($t8)
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 and   $s0,$0,$0
		 and   $s1,$0,$0
		 sltiu $s2,$t8,1
		 slt   $s3,$t8,$0
		 bne   $s3,$0,8f
		 slt   $t0,$v0,$t8         	 # Delay slot
		 bne   $t0,$0,9f
		 nop                    	 # Delay slot
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	9:
		 and   $s3,$0,$0
	8:
		 m_jal   Exception
		 ori   $v0,$0,6          	 # Delay slot

		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_41b9:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t8,0x00($t8)
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 and   $s0,$0,$0
		 and   $s1,$0,$0
		 sltiu $s2,$t8,1
		 slt   $s3,$t8,$0
		 bne   $s3,$0,8f
		 slt   $t0,$v0,$t8         	 # Delay slot
		 bne   $t0,$0,9f
		 nop                    	 # Delay slot
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	9:
		 and   $s3,$0,$0
	8:
		 m_jal   Exception
		 ori   $v0,$0,6          	 # Delay slot

		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_41ba:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t8,0x00($t8)
		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 and   $s0,$0,$0
		 and   $s1,$0,$0
		 sltiu $s2,$t8,1
		 slt   $s3,$t8,$0
		 bne   $s3,$0,8f
		 slt   $t0,$v0,$t8         	 # Delay slot
		 bne   $t0,$0,9f
		 nop                    	 # Delay slot
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	9:
		 and   $s3,$0,$0
	8:
		 m_jal   Exception
		 ori   $v0,$0,6          	 # Delay slot

		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_41bb:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t8,0x00($t8)
		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 and   $s0,$0,$0
		 and   $s1,$0,$0
		 sltiu $s2,$t8,1
		 slt   $s3,$t8,$0
		 bne   $s3,$0,8f
		 slt   $t0,$v0,$t8         	 # Delay slot
		 bne   $t0,$0,9f
		 nop                    	 # Delay slot
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	9:
		 and   $s3,$0,$0
	8:
		 m_jal   Exception
		 ori   $v0,$0,6          	 # Delay slot

		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_41bc:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t8,0x00($t8)
		 lh    $v0,0x00($s7)
		 addiu $s7,$s7,2
		 and   $s0,$0,$0
		 and   $s1,$0,$0
		 sltiu $s2,$t8,1
		 slt   $s3,$t8,$0
		 bne   $s3,$0,8f
		 slt   $t0,$v0,$t8         	 # Delay slot
		 bne   $t0,$0,9f
		 nop                    	 # Delay slot
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	9:
		 and   $s3,$0,$0
	8:
		 m_jal   Exception
		 ori   $v0,$0,6          	 # Delay slot

		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c140:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 sll   $t0,$t0,2
		 andi  $t8,$t8,0x1C
		 addu  $t0,$t0,$s5
		 addu  $t8,$t8,$s5
		 lw    $v0,0x00($t0)
		 lw    $v1,0x00($t8)
		 sw    $v0,0x00($t8)
		 sw    $v1,0x00($t0)
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c148:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 sll   $t0,$t0,2
		 andi  $t8,$t8,0x1C
		 addu  $t0,$t0,$s5
		 addu  $t8,$t8,$s5
		 lw    $v0,0x20($t0)
		 lw    $v1,0x20($t8)
		 sw    $v0,0x20($t8)
		 sw    $v1,0x20($t0)
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c188:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 sll   $t0,$t0,2
		 andi  $t8,$t8,0x1C
		 addu  $t0,$t0,$s5
		 addu  $t8,$t8,$s5
		 lw    $v0,0x20($t0)
		 lw    $v1,0x00($t8)
		 sw    $v0,0x00($t8)
		 sw    $v1,0x20($t0)
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b108:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t0,$v0
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t8)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 subu  $v1,$v0,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$v0,$t0
		 xor   $t9,$v1,$v0
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v1
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b10f:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t0,$v0
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t8)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 subu  $v1,$v0,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$v0,$t0
		 xor   $t9,$v1,$v0
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v1
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b148:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $t0,$v0
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 subu  $v1,$v0,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$v0,$t0
		 xor   $t9,$v1,$v0
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seh  $t9,$v1
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_b188:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t0,$0,$v0
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($t8)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $v1,$v0,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$v0,$t0
		 xor   $t9,$v1,$v0
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,31        	 # Set Overflow
		 slt   $s3,$v1,$0        	 # Set Sign
		 sltiu $s2,$v1,1         	 # Set Zero
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_bf08:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t0)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t0,$v0
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 subu  $v1,$v0,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$v0,$t0
		 xor   $t9,$v1,$v0
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v1
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_bf0f:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $t0,$v0
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seb   $v0,$v0
		 subu  $v1,$v0,$t0
		 sltu  $s0,$v0,$t0       	 # Set Carry
		 xor   $s1,$v0,$t0
		 xor   $t9,$v1,$v0
		 and   $s1,$s1,$t9
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 m_seb  $t9,$v1
		 slt   $s3,$t9,$0        	 # Set Sign
		 sltiu $s2,$t9,1         	 # Set Zero
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c0c0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lhu   $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lhu   $v0,0x00($t0)
		 multu $v0,$t1
		 mflo  $t0
		 sw    $t0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t0,$0        	 # Set Sign
		 sltiu $s2,$t0,1         	 # Set Zero
		 addiu $t7,$t7,-54
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c0d0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lhu   $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 multu $v0,$t1
		 mflo  $t0
		 sw    $t0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t0,$0        	 # Set Sign
		 sltiu $s2,$t0,1         	 # Set Zero
		 addiu $t7,$t7,-58
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c0d8:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lhu   $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 multu $v0,$t1
		 mflo  $t0
		 sw    $t0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t0,$0        	 # Set Sign
		 sltiu $s2,$t0,1         	 # Set Zero
		 addiu $t7,$t7,-58
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c0e0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lhu   $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 multu $v0,$t1
		 mflo  $t0
		 sw    $t0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t0,$0        	 # Set Sign
		 sltiu $s2,$t0,1         	 # Set Zero
		 addiu $t7,$t7,-60
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c0e8:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lhu   $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 multu $v0,$t1
		 mflo  $t0
		 sw    $t0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t0,$0        	 # Set Sign
		 sltiu $s2,$t0,1         	 # Set Zero
		 addiu $t7,$t7,-62
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c0f0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lhu   $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 multu $v0,$t1
		 mflo  $t0
		 sw    $t0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t0,$0        	 # Set Sign
		 sltiu $s2,$t0,1         	 # Set Zero
		 addiu $t7,$t7,-64
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c0f8:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lhu   $t1,0x00($t8)
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 multu $v0,$t1
		 mflo  $t0
		 sw    $t0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t0,$0        	 # Set Sign
		 sltiu $s2,$t0,1         	 # Set Zero
		 addiu $t7,$t7,-62
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c0f9:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lhu   $t1,0x00($t8)
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 multu $v0,$t1
		 mflo  $t0
		 sw    $t0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t0,$0        	 # Set Sign
		 sltiu $s2,$t0,1         	 # Set Zero
		 addiu $t7,$t7,-66
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c0fa:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lhu   $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 multu $v0,$t1
		 mflo  $t0
		 sw    $t0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t0,$0        	 # Set Sign
		 sltiu $s2,$t0,1         	 # Set Zero
		 addiu $t7,$t7,-62
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c0fb:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lhu   $t1,0x00($t8)
		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 multu $v0,$t1
		 mflo  $t0
		 sw    $t0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t0,$0        	 # Set Sign
		 sltiu $s2,$t0,1         	 # Set Zero
		 addiu $t7,$t7,-64
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c0fc:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lhu   $t1,0x00($t8)
		 lhu   $v0,0x00($s7)
		 addiu $s7,$s7,2
		 multu $v0,$t1
		 mflo  $t0
		 sw    $t0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t0,$0        	 # Set Sign
		 sltiu $s2,$t0,1         	 # Set Zero
		 addiu $t7,$t7,-54
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c1c0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lh    $v0,0x00($t0)
		 mult  $v0,$t1
		 mflo  $t0
		 sw    $t0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t0,$0        	 # Set Sign
		 sltiu $s2,$t0,1         	 # Set Zero
		 addiu $t7,$t7,-54
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c1d0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 mult  $v0,$t1
		 mflo  $t0
		 sw    $t0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t0,$0        	 # Set Sign
		 sltiu $s2,$t0,1         	 # Set Zero
		 addiu $t7,$t7,-58
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c1d8:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 mult  $v0,$t1
		 mflo  $t0
		 sw    $t0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t0,$0        	 # Set Sign
		 sltiu $s2,$t0,1         	 # Set Zero
		 addiu $t7,$t7,-58
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c1e0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t0)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 mult  $v0,$t1
		 mflo  $t0
		 sw    $t0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t0,$0        	 # Set Sign
		 sltiu $s2,$t0,1         	 # Set Zero
		 addiu $t7,$t7,-60
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c1e8:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 mult  $v0,$t1
		 mflo  $t0
		 sw    $t0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t0,$0        	 # Set Sign
		 sltiu $s2,$t0,1         	 # Set Zero
		 addiu $t7,$t7,-62
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c1f0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 mult  $v0,$t1
		 mflo  $t0
		 sw    $t0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t0,$0        	 # Set Sign
		 sltiu $s2,$t0,1         	 # Set Zero
		 addiu $t7,$t7,-64
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c1f8:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 mult  $v0,$t1
		 mflo  $t0
		 sw    $t0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t0,$0        	 # Set Sign
		 sltiu $s2,$t0,1         	 # Set Zero
		 addiu $t7,$t7,-62
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c1f9:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 mult  $v0,$t1
		 mflo  $t0
		 sw    $t0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t0,$0        	 # Set Sign
		 sltiu $s2,$t0,1         	 # Set Zero
		 addiu $t7,$t7,-66
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c1fa:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 mult  $v0,$t1
		 mflo  $t0
		 sw    $t0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t0,$0        	 # Set Sign
		 sltiu $s2,$t0,1         	 # Set Zero
		 addiu $t7,$t7,-62
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c1fb:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t8,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t8,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 mult  $v0,$t1
		 mflo  $t0
		 sw    $t0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t0,$0        	 # Set Sign
		 sltiu $s2,$t0,1         	 # Set Zero
		 addiu $t7,$t7,-64
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_c1fc:				#:
		 addiu $s7,$s7,2

		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lh    $t1,0x00($t8)
		 lh    $v0,0x00($s7)
		 addiu $s7,$s7,2
		 mult  $v0,$t1
		 mflo  $t0
		 sw    $t0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$t0,$0        	 # Set Sign
		 sltiu $s2,$t0,1         	 # Set Zero
		 addiu $t7,$t7,-54
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4e77:				#:
		 lw    $s0,0x3C($s5)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$s0
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t0,$s0,6
		 sw    $t0,0x3C($s5)
		 addiu $s0,$s0,2
		 or    $s1,$0,$v0
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$s0
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t0,$0,$v0
		 or    $v0,$0,$s1
		 or    $s4,$0,$v0
		 or    $s3,$0,$v0
		 or    $s2,$0,$v0
		 or    $s1,$0,$v0
		 or    $s0,$0,$v0
		 andi  $s4,$s4,0x10
		 andi  $s3,$s3,0x08
		 andi  $s2,$s2,0x04
		 andi  $s1,$s1,0x02
		 andi  $s0,$s0,0x01
		 srl   $s4,$s4,4
		 srl   $s3,$s3,3
		 srl   $s2,$s2,2
		 srl   $s1,$s1,1
		 andi  $t9,$t0,0x01
		 beq   $t9,$0,2f
		 addu  $s7,$s6,$t0     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $t0,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$t0,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4e75:				#:
		 lw    $a0,0x3C($s5)
		 addiu $t0,$a0,4
		 sw    $t0,0x3C($s5)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t0,$0,$v0
		 andi  $t9,$t0,0x01
		 beq   $t9,$0,2f
		 addu  $s7,$s6,$t0     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $t0,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$t0,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4e90:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $a1,0x88($s5)
		 lui   $t9,0xFF00
		 lw    $a0,0x3C($s5)     	 # Push onto Stack
		 and   $a1,$a1,$t9
		 subu  $t9,$s7,$s6
		 addiu $a0,$a0,-4
		 sw    $a0,0x3C($s5)
		 or    $a1,$a1,$t9
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 andi  $t9,$t6,0x01
		 beq   $t9,$0,2f
		 addu  $s7,$s6,$t6     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $t6,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$t6,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4ea8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $a1,0x88($s5)
		 lui   $t9,0xFF00
		 lw    $a0,0x3C($s5)     	 # Push onto Stack
		 and   $a1,$a1,$t9
		 subu  $t9,$s7,$s6
		 addiu $a0,$a0,-4
		 sw    $a0,0x3C($s5)
		 or    $a1,$a1,$t9
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 andi  $t9,$t6,0x01
		 beq   $t9,$0,2f
		 addu  $s7,$s6,$t6     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-30
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $t6,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$t6,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-30
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4eb0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $a1,0x88($s5)
		 lui   $t9,0xFF00
		 lw    $a0,0x3C($s5)     	 # Push onto Stack
		 and   $a1,$a1,$t9
		 subu  $t9,$s7,$s6
		 addiu $a0,$a0,-4
		 sw    $a0,0x3C($s5)
		 or    $a1,$a1,$t9
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 andi  $t9,$t6,0x01
		 beq   $t9,$0,2f
		 addu  $s7,$s6,$t6     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-36
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $t6,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$t6,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-36
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4eb8:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $a1,0x88($s5)
		 lui   $t9,0xFF00
		 lw    $a0,0x3C($s5)     	 # Push onto Stack
		 and   $a1,$a1,$t9
		 subu  $t9,$s7,$s6
		 addiu $a0,$a0,-4
		 sw    $a0,0x3C($s5)
		 or    $a1,$a1,$t9
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 andi  $t9,$t6,0x01
		 beq   $t9,$0,2f
		 addu  $s7,$s6,$t6     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-30
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $t6,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$t6,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-30
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4eb9:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $a1,0x88($s5)
		 lui   $t9,0xFF00
		 lw    $a0,0x3C($s5)     	 # Push onto Stack
		 and   $a1,$a1,$t9
		 subu  $t9,$s7,$s6
		 addiu $a0,$a0,-4
		 sw    $a0,0x3C($s5)
		 or    $a1,$a1,$t9
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 andi  $t9,$t6,0x01
		 beq   $t9,$0,2f
		 addu  $s7,$s6,$t6     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-34
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $t6,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$t6,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-34
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4eba:				#:
		 addiu $s7,$s7,2

		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 lw    $a1,0x88($s5)
		 lui   $t9,0xFF00
		 lw    $a0,0x3C($s5)     	 # Push onto Stack
		 and   $a1,$a1,$t9
		 subu  $t9,$s7,$s6
		 addiu $a0,$a0,-4
		 sw    $a0,0x3C($s5)
		 or    $a1,$a1,$t9
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 andi  $t9,$t6,0x01
		 beq   $t9,$0,2f
		 addu  $s7,$s6,$t6     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-30
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $t6,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$t6,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-30
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4ebb:				#:
		 addiu $s7,$s7,2

		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $a1,0x88($s5)
		 lui   $t9,0xFF00
		 lw    $a0,0x3C($s5)     	 # Push onto Stack
		 and   $a1,$a1,$t9
		 subu  $t9,$s7,$s6
		 addiu $a0,$a0,-4
		 sw    $a0,0x3C($s5)
		 or    $a1,$a1,$t9
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 andi  $t9,$t6,0x01
		 beq   $t9,$0,2f
		 addu  $s7,$s6,$t6     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-32
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $t6,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$t6,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-32
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4ed0:				#:
		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 andi  $t9,$t6,0x01
		 beq   $t9,$0,2f
		 addu  $s7,$s6,$t6     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $t6,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$t6,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4ee8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 andi  $t9,$t6,0x01
		 beq   $t9,$0,2f
		 addu  $s7,$s6,$t6     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $t6,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$t6,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4ef0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 andi  $t9,$t6,0x01
		 beq   $t9,$0,2f
		 addu  $s7,$s6,$t6     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-28
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $t6,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$t6,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-28
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4ef8:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 andi  $t9,$t6,0x01
		 beq   $t9,$0,2f
		 addu  $s7,$s6,$t6     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $t6,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$t6,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4ef9:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 andi  $t9,$t6,0x01
		 beq   $t9,$0,2f
		 addu  $s7,$s6,$t6     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-26
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $t6,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$t6,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-26
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4efa:				#:
		 addiu $s7,$s7,2

		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 andi  $t9,$t6,0x01
		 beq   $t9,$0,2f
		 addu  $s7,$s6,$t6     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $t6,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$t6,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4efb:				#:
		 addiu $s7,$s7,2

		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 andi  $t9,$t6,0x01
		 beq   $t9,$0,2f
		 addu  $s7,$s6,$t6     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $t6,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$t6,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4800:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lbu   $v0,0x00($t8)
		 subu  $s1,$0,$v0
		 subu  $s1,$s1,$s4
		 andi  $s1,$s1,0xFF
		 andi  $t1,$v0,0x0F
		 andi  $t0,$v0,0xF0
		 subu  $t1,$0,$t1
		 subu  $t1,$t1,$s4
		 subu  $t0,$0,$t0
		 sltiu $t2,$t1,10
		 xori  $t2,$t2,1
		 sll   $t2,$t2,1
		 subu  $t1,$t1,$t2
		 sll   $t2,$t2,1
		 subu  $t1,$t1,$t2
		 addu  $v0,$t0,$t1
		 sltiu $s0,$v0,0x9a
		 xori  $s0,$s0,1        	 # Set Carry
		 sll   $t2,$s0,5
		 addu  $v0,$v0,$t2
		 sll   $t2,$t2,2
		 addu  $v0,$v0,$t2
		 xor   $s3,$s1,$v0
		 and   $s1,$s1,$s3
		 srl   $s1,$s1,7        	 # Set Overflow
		 sltiu $t9,$v0,1
		 and   $s2,$s2,$t9       	 # Set Zero
		 srl   $s3,$v0,7         	 # Set Sign
		 or    $s4,$0,$s0       	 # Copy Carry to X
		 sb    $v0,0x00($t8)
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4810:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $s1,$0,$v0
		 subu  $s1,$s1,$s4
		 andi  $s1,$s1,0xFF
		 andi  $t1,$v0,0x0F
		 andi  $t0,$v0,0xF0
		 subu  $t1,$0,$t1
		 subu  $t1,$t1,$s4
		 subu  $t0,$0,$t0
		 sltiu $t2,$t1,10
		 xori  $t2,$t2,1
		 sll   $t2,$t2,1
		 subu  $t1,$t1,$t2
		 sll   $t2,$t2,1
		 subu  $t1,$t1,$t2
		 addu  $v0,$t0,$t1
		 sltiu $s0,$v0,0x9a
		 xori  $s0,$s0,1        	 # Set Carry
		 sll   $t2,$s0,5
		 addu  $v0,$v0,$t2
		 sll   $t2,$t2,2
		 addu  $v0,$v0,$t2
		 xor   $s3,$s1,$v0
		 and   $s1,$s1,$s3
		 srl   $s1,$s1,7        	 # Set Overflow
		 sltiu $t9,$v0,1
		 and   $s2,$s2,$t9       	 # Set Zero
		 srl   $s3,$v0,7         	 # Set Sign
		 or    $s4,$0,$s0       	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4818:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t8)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $s1,$0,$v0
		 subu  $s1,$s1,$s4
		 andi  $s1,$s1,0xFF
		 andi  $t1,$v0,0x0F
		 andi  $t0,$v0,0xF0
		 subu  $t1,$0,$t1
		 subu  $t1,$t1,$s4
		 subu  $t0,$0,$t0
		 sltiu $t2,$t1,10
		 xori  $t2,$t2,1
		 sll   $t2,$t2,1
		 subu  $t1,$t1,$t2
		 sll   $t2,$t2,1
		 subu  $t1,$t1,$t2
		 addu  $v0,$t0,$t1
		 sltiu $s0,$v0,0x9a
		 xori  $s0,$s0,1        	 # Set Carry
		 sll   $t2,$s0,5
		 addu  $v0,$v0,$t2
		 sll   $t2,$t2,2
		 addu  $v0,$v0,$t2
		 xor   $s3,$s1,$v0
		 and   $s1,$s1,$s3
		 srl   $s1,$s1,7        	 # Set Overflow
		 sltiu $t9,$v0,1
		 and   $s2,$s2,$t9       	 # Set Zero
		 srl   $s3,$v0,7         	 # Set Sign
		 or    $s4,$0,$s0       	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_481f:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $s1,$0,$v0
		 subu  $s1,$s1,$s4
		 andi  $s1,$s1,0xFF
		 andi  $t1,$v0,0x0F
		 andi  $t0,$v0,0xF0
		 subu  $t1,$0,$t1
		 subu  $t1,$t1,$s4
		 subu  $t0,$0,$t0
		 sltiu $t2,$t1,10
		 xori  $t2,$t2,1
		 sll   $t2,$t2,1
		 subu  $t1,$t1,$t2
		 sll   $t2,$t2,1
		 subu  $t1,$t1,$t2
		 addu  $v0,$t0,$t1
		 sltiu $s0,$v0,0x9a
		 xori  $s0,$s0,1        	 # Set Carry
		 sll   $t2,$s0,5
		 addu  $v0,$v0,$t2
		 sll   $t2,$t2,2
		 addu  $v0,$v0,$t2
		 xor   $s3,$s1,$v0
		 and   $s1,$s1,$s3
		 srl   $s1,$s1,7        	 # Set Overflow
		 sltiu $t9,$v0,1
		 and   $s2,$s2,$t9       	 # Set Zero
		 srl   $s3,$v0,7         	 # Set Sign
		 or    $s4,$0,$s0       	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4820:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $s1,$0,$v0
		 subu  $s1,$s1,$s4
		 andi  $s1,$s1,0xFF
		 andi  $t1,$v0,0x0F
		 andi  $t0,$v0,0xF0
		 subu  $t1,$0,$t1
		 subu  $t1,$t1,$s4
		 subu  $t0,$0,$t0
		 sltiu $t2,$t1,10
		 xori  $t2,$t2,1
		 sll   $t2,$t2,1
		 subu  $t1,$t1,$t2
		 sll   $t2,$t2,1
		 subu  $t1,$t1,$t2
		 addu  $v0,$t0,$t1
		 sltiu $s0,$v0,0x9a
		 xori  $s0,$s0,1        	 # Set Carry
		 sll   $t2,$s0,5
		 addu  $v0,$v0,$t2
		 sll   $t2,$t2,2
		 addu  $v0,$v0,$t2
		 xor   $s3,$s1,$v0
		 and   $s1,$s1,$s3
		 srl   $s1,$s1,7        	 # Set Overflow
		 sltiu $t9,$v0,1
		 and   $s2,$s2,$t9       	 # Set Zero
		 srl   $s3,$v0,7         	 # Set Sign
		 or    $s4,$0,$s0       	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4827:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $s1,$0,$v0
		 subu  $s1,$s1,$s4
		 andi  $s1,$s1,0xFF
		 andi  $t1,$v0,0x0F
		 andi  $t0,$v0,0xF0
		 subu  $t1,$0,$t1
		 subu  $t1,$t1,$s4
		 subu  $t0,$0,$t0
		 sltiu $t2,$t1,10
		 xori  $t2,$t2,1
		 sll   $t2,$t2,1
		 subu  $t1,$t1,$t2
		 sll   $t2,$t2,1
		 subu  $t1,$t1,$t2
		 addu  $v0,$t0,$t1
		 sltiu $s0,$v0,0x9a
		 xori  $s0,$s0,1        	 # Set Carry
		 sll   $t2,$s0,5
		 addu  $v0,$v0,$t2
		 sll   $t2,$t2,2
		 addu  $v0,$v0,$t2
		 xor   $s3,$s1,$v0
		 and   $s1,$s1,$s3
		 srl   $s1,$s1,7        	 # Set Overflow
		 sltiu $t9,$v0,1
		 and   $s2,$s2,$t9       	 # Set Zero
		 srl   $s3,$v0,7         	 # Set Sign
		 or    $s4,$0,$s0       	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4828:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $s1,$0,$v0
		 subu  $s1,$s1,$s4
		 andi  $s1,$s1,0xFF
		 andi  $t1,$v0,0x0F
		 andi  $t0,$v0,0xF0
		 subu  $t1,$0,$t1
		 subu  $t1,$t1,$s4
		 subu  $t0,$0,$t0
		 sltiu $t2,$t1,10
		 xori  $t2,$t2,1
		 sll   $t2,$t2,1
		 subu  $t1,$t1,$t2
		 sll   $t2,$t2,1
		 subu  $t1,$t1,$t2
		 addu  $v0,$t0,$t1
		 sltiu $s0,$v0,0x9a
		 xori  $s0,$s0,1        	 # Set Carry
		 sll   $t2,$s0,5
		 addu  $v0,$v0,$t2
		 sll   $t2,$t2,2
		 addu  $v0,$v0,$t2
		 xor   $s3,$s1,$v0
		 and   $s1,$s1,$s3
		 srl   $s1,$s1,7        	 # Set Overflow
		 sltiu $t9,$v0,1
		 and   $s2,$s2,$t9       	 # Set Zero
		 srl   $s3,$v0,7         	 # Set Sign
		 or    $s4,$0,$s0       	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4830:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $s1,$0,$v0
		 subu  $s1,$s1,$s4
		 andi  $s1,$s1,0xFF
		 andi  $t1,$v0,0x0F
		 andi  $t0,$v0,0xF0
		 subu  $t1,$0,$t1
		 subu  $t1,$t1,$s4
		 subu  $t0,$0,$t0
		 sltiu $t2,$t1,10
		 xori  $t2,$t2,1
		 sll   $t2,$t2,1
		 subu  $t1,$t1,$t2
		 sll   $t2,$t2,1
		 subu  $t1,$t1,$t2
		 addu  $v0,$t0,$t1
		 sltiu $s0,$v0,0x9a
		 xori  $s0,$s0,1        	 # Set Carry
		 sll   $t2,$s0,5
		 addu  $v0,$v0,$t2
		 sll   $t2,$t2,2
		 addu  $v0,$v0,$t2
		 xor   $s3,$s1,$v0
		 and   $s1,$s1,$s3
		 srl   $s1,$s1,7        	 # Set Overflow
		 sltiu $t9,$v0,1
		 and   $s2,$s2,$t9       	 # Set Zero
		 srl   $s3,$v0,7         	 # Set Sign
		 or    $s4,$0,$s0       	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4838:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $s1,$0,$v0
		 subu  $s1,$s1,$s4
		 andi  $s1,$s1,0xFF
		 andi  $t1,$v0,0x0F
		 andi  $t0,$v0,0xF0
		 subu  $t1,$0,$t1
		 subu  $t1,$t1,$s4
		 subu  $t0,$0,$t0
		 sltiu $t2,$t1,10
		 xori  $t2,$t2,1
		 sll   $t2,$t2,1
		 subu  $t1,$t1,$t2
		 sll   $t2,$t2,1
		 subu  $t1,$t1,$t2
		 addu  $v0,$t0,$t1
		 sltiu $s0,$v0,0x9a
		 xori  $s0,$s0,1        	 # Set Carry
		 sll   $t2,$s0,5
		 addu  $v0,$v0,$t2
		 sll   $t2,$t2,2
		 addu  $v0,$v0,$t2
		 xor   $s3,$s1,$v0
		 and   $s1,$s1,$s3
		 srl   $s1,$s1,7        	 # Set Overflow
		 sltiu $t9,$v0,1
		 and   $s2,$s2,$t9       	 # Set Zero
		 srl   $s3,$v0,7         	 # Set Sign
		 or    $s4,$0,$s0       	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4839:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 subu  $s1,$0,$v0
		 subu  $s1,$s1,$s4
		 andi  $s1,$s1,0xFF
		 andi  $t1,$v0,0x0F
		 andi  $t0,$v0,0xF0
		 subu  $t1,$0,$t1
		 subu  $t1,$t1,$s4
		 subu  $t0,$0,$t0
		 sltiu $t2,$t1,10
		 xori  $t2,$t2,1
		 sll   $t2,$t2,1
		 subu  $t1,$t1,$t2
		 sll   $t2,$t2,1
		 subu  $t1,$t1,$t2
		 addu  $v0,$t0,$t1
		 sltiu $s0,$v0,0x9a
		 xori  $s0,$s0,1        	 # Set Carry
		 sll   $t2,$s0,5
		 addu  $v0,$v0,$t2
		 sll   $t2,$t2,2
		 addu  $v0,$v0,$t2
		 xor   $s3,$s1,$v0
		 and   $s1,$s1,$s3
		 srl   $s1,$s1,7        	 # Set Overflow
		 sltiu $t9,$v0,1
		 and   $s2,$s2,$t9       	 # Set Zero
		 srl   $s3,$v0,7         	 # Set Sign
		 or    $s4,$0,$s0       	 # Copy Carry to X
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4ac0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lbu   $v0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $v0,$v0,0x80
		 sb    $v0,0x00($t8)
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4ad0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $v0,$v0,0x80
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4ad8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t8)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $v0,$v0,0x80
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4adf:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $v0,$v0,0x80
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4ae0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $v0,$v0,0x80
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4ae7:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $v0,$v0,0x80
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4ae8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $v0,$v0,0x80
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4af0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $v0,$v0,0x80
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4af8:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $v0,$v0,0x80
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-22
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4af9:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 or    $v0,$v0,0x80
		 lw    $t9,0x9C($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-26
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4e40:				#:
		 addiu $s7,$s7,2

		 andi  $v0,$t8,0x0f
		 m_jal   Exception
		 ori   $v0,$v0,32         	 # Delay slot

		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4e76:				#
		 beq   $s1,$0,9f
		 addiu $s7,$s7,2        	 # Delay slot

		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,7

		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	9:
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4e70:				#:
		 lbu   $t0,0x44($s5)
		 andi  $t0,$t0,0x20       	 # Supervisor Mode ?
		 bne   $t0,$0,9f
		 addiu $s7,$s7,2        	 # Delay slot

		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,8

		 addiu $t7,$t7,-132
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot


	9:
		 lw    $t9,0x64($s5)
		 beq   $t9,$0,9f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 sw    $t7,m68k_ICount
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop   			 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s7,0x50($s5)
		 addu  $s7,$s7,$s6
		 lhu   $t8,0x00($s7)
	9:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp

		 lw    $a3,0x00($a3)

		 jr    $a3
		 addiu $t7,$t7,-132      	 # Delay slot
OP0_4e71:				#:
		 addiu $s7,$s7,2

		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4e72:				#:
		 lbu   $t0,0x44($s5)
		 andi  $t0,$t0,0x20       	 # Supervisor Mode ?
		 beq   $t0,$0,9f
		 addiu $s7,$s7,2        	 # Delay slot

		 lhu   $v0,0x00($s7)
		 addiu $s7,$s7,2
		 andi  $t9,$v0,0x2000    	 # User Mode ?
		 bne   $t9,$0,0f
		 or    $s2,$0,$v0       	 # Delay slot
		 lw    $s0,0x3C($s5)
		 lw    $s1,0x78($s5)
		 sw    $s0,0x40($s5)
		 sw    $s1,0x3C($s5)
	0:
		 srl   $s2,$s2,8
		 sb    $s2,0x44($s5)    	 # T, S & I
		 or    $s4,$0,$v0
		 or    $s3,$0,$v0
		 or    $s2,$0,$v0
		 or    $s1,$0,$v0
		 or    $s0,$0,$v0
		 andi  $s4,$s4,0x10
		 andi  $s3,$s3,0x08
		 andi  $s2,$s2,0x04
		 andi  $s1,$s1,0x02
		 andi  $s0,$s0,0x01
		 srl   $s4,$s4,4
		 srl   $s3,$s3,3
		 srl   $s2,$s2,2
		 srl   $s1,$s1,1
		 lbu   $v1,0x54($s5)
		 andi  $v0,$v1,0x07
		 xori  $t9,$v0,0x07       	 # Always take 7
		 bne   $t9,$0,8f
		 nop                    	 # Delay slot

		 m_j   procint
		 nop                    	 # Delay slot

	8:

		 lbu   $t0,0x44($s5)     	 # int mask
		 andi  $t0,$t0,0x07
		 subu  $t9,$t0,$v0
		 bgez  $t9,8f
		 nop                    	 # Delay slot

		 m_j   procint
		 nop                    	 # Delay slot

	8:

		 and   $t7,$0,$0
		 ori   $v1,$v1,0x80
		 sb    $v1,0x54($s5)
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	9:
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,8

		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4880:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lb    $v0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 sh    $v0,0x00($t8)
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_48c0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lh    $v0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 sw    $v0,0x00($t8)
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4e73:				#:
		 lbu   $t0,0x44($s5)
		 andi  $t0,$t0,0x20       	 # Supervisor Mode ?
		 beq   $t0,$0,9f
		 addiu $s7,$s7,2        	 # Delay slot

		 lw    $s0,0x3C($s5)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$s0
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t0,$s0,6
		 sw    $t0,0x3C($s5)
		 addiu $s0,$s0,2
		 or    $s1,$0,$v0
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$s0
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $t0,$0,$v0
		 or    $v0,$0,$s1
		 andi  $t9,$v0,0x2000    	 # User Mode ?
		 bne   $t9,$0,0f
		 or    $s2,$0,$v0       	 # Delay slot
		 lw    $s0,0x3C($s5)
		 lw    $s1,0x78($s5)
		 sw    $s0,0x40($s5)
		 sw    $s1,0x3C($s5)
	0:
		 srl   $s2,$s2,8
		 sb    $s2,0x44($s5)    	 # T, S & I
		 or    $s4,$0,$v0
		 or    $s3,$0,$v0
		 or    $s2,$0,$v0
		 or    $s1,$0,$v0
		 or    $s0,$0,$v0
		 andi  $s4,$s4,0x10
		 andi  $s3,$s3,0x08
		 andi  $s2,$s2,0x04
		 andi  $s1,$s1,0x02
		 andi  $s0,$s0,0x01
		 srl   $s4,$s4,4
		 srl   $s3,$s3,3
		 srl   $s2,$s2,2
		 srl   $s1,$s1,1
		 andi  $t9,$t0,0x01
		 beq   $t9,$0,2f
		 addu  $s7,$s6,$t0     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lbu   $a3,0x54($s5)     	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
 # Check for Interrupt waiting

		 andi  $a3,$a3,0x07
		 beq   $a3,$0,3f
		 nop                    	 # Delay slot
		 m_j   interrupt
		 nop                    	 # Delay slot

	3:
		 lhu   $t8,0x00($s7)
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $t0,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$t0,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lbu   $a3,0x54($s5)     	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
 # Check for Interrupt waiting

		 andi  $a3,$a3,0x07
		 beq   $a3,$0,3f
		 nop                    	 # Delay slot
		 m_j   interrupt
		 nop                    	 # Delay slot

	3:
		 lhu   $t8,0x00($s7)
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	9:
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,8

		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lbu   $a3,0x54($s5)     	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
 # Check for Interrupt waiting

		 andi  $a3,$a3,0x07
		 beq   $a3,$0,3f
		 nop                    	 # Delay slot
		 m_j   interrupt
		 nop                    	 # Delay slot

	3:
		 lhu   $t8,0x00($s7)
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4a00:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07

		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lbu   $v0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4a10:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07

		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4a18:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07

		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,1
		 sw    $t9,0x20($t8)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4a1f:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t9,$t6,2
		 sw    $t9,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4a20:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07

		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-1
		 sw    $t6,0x20($t8)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4a27:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)    	 # Get A7
		 addiu $t6,$t6,-2
		 sw    $t6,0x3C($s5)
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4a28:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07

		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4a30:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07

		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4a38:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4a39:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x90($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4a40:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07

		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lhu   $v0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4a50:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07

		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4a58:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07

		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4a60:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07

		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4a68:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07

		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4a70:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07

		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4a78:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4a79:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4a80:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07

		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $v0,0x00($t8)
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4a90:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07

		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4a98:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07

		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($t8)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4aa0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07

		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-4
		 sw    $t6,0x20($t8)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4aa8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07

		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4ab0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07

		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4ab8:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4ab9:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 and   $s0,$0,$0        	 # Clear Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31         	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4890:				#:
		 addiu $s7,$s7,2

		 lhu   $t4,0x00($s7)
		 addiu $s7,$s7,2
		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 ori   $t0,$0,1
		 or    $t1,$0,$s5
	9:
		 and   $t2,$t4,$t0
		 beq   $t2,$0,8f
		 lw    $a1,0x00($t1)      	 # Delay slot
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t1,0x3C($sp)
		 sw    $t4,0x38($sp)
		 sw    $t6,0x34($sp)
		 sw    $gp,0x30($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x30($sp)
		 lw    $t6,0x34($sp)
		 lw    $t4,0x38($sp)
		 lw    $t1,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t6,$t6,2
		 addiu $t7,$t7,-4
	8:
		 sll   $t0,$t0,1
		 andi  $a3,$t0,0xffff
		 bne   $a3,$0,9b
		 addiu $t1,$t1,4          	 # Delay slot
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_48a0:				#:
		 addiu $s7,$s7,2

		 lhu   $t4,0x00($s7)
		 addiu $s7,$s7,2
		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 ori   $t0,$0,1
		 addiu $t1,$s5,0x3C
	9:
		 and   $t2,$t4,$t0
		 beq   $t2,$0,8f
		 lw    $a1,0x00($t1)      	 # Delay slot
		 addiu $t6,$t6,-2
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t1,0x3C($sp)
		 sw    $t4,0x38($sp)
		 sw    $t6,0x34($sp)
		 sw    $t8,0x30($sp)
		 sw    $gp,0x2C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x2C($sp)
		 lw    $t8,0x30($sp)
		 lw    $t6,0x34($sp)
		 lw    $t4,0x38($sp)
		 lw    $t1,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-4
	8:
		 sll   $t0,$t0,1
		 andi  $a3,$t0,0xffff
		 bne   $a3,$0,9b
		 addiu $t1,$t1,-4         	 # Delay slot
		 sw    $t6,0x20($t8)
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_48a8:				#:
		 addiu $s7,$s7,2

		 lhu   $t4,0x00($s7)
		 addiu $s7,$s7,2
		 andi  $t8,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 ori   $t0,$0,1
		 or    $t1,$0,$s5
	9:
		 and   $t2,$t4,$t0
		 beq   $t2,$0,8f
		 lw    $a1,0x00($t1)      	 # Delay slot
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t1,0x3C($sp)
		 sw    $t4,0x38($sp)
		 sw    $t6,0x34($sp)
		 sw    $gp,0x30($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x30($sp)
		 lw    $t6,0x34($sp)
		 lw    $t4,0x38($sp)
		 lw    $t1,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t6,$t6,2
		 addiu $t7,$t7,-4
	8:
		 sll   $t0,$t0,1
		 andi  $a3,$t0,0xffff
		 bne   $a3,$0,9b
		 addiu $t1,$t1,4          	 # Delay slot
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_48b0:				#:
		 addiu $s7,$s7,2

		 lhu   $t4,0x00($s7)
		 addiu $s7,$s7,2
		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 ori   $t0,$0,1
		 or    $t1,$0,$s5
	9:
		 and   $t2,$t4,$t0
		 beq   $t2,$0,8f
		 lw    $a1,0x00($t1)      	 # Delay slot
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t1,0x3C($sp)
		 sw    $t4,0x38($sp)
		 sw    $t6,0x34($sp)
		 sw    $gp,0x30($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x30($sp)
		 lw    $t6,0x34($sp)
		 lw    $t4,0x38($sp)
		 lw    $t1,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t6,$t6,2
		 addiu $t7,$t7,-4
	8:
		 sll   $t0,$t0,1
		 andi  $a3,$t0,0xffff
		 bne   $a3,$0,9b
		 addiu $t1,$t1,4          	 # Delay slot
		 addiu $t7,$t7,-28
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_48b8:				#:
		 addiu $s7,$s7,2

		 lhu   $t4,0x00($s7)
		 addiu $s7,$s7,2
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 ori   $t0,$0,1
		 or    $t1,$0,$s5
	9:
		 and   $t2,$t4,$t0
		 beq   $t2,$0,8f
		 lw    $a1,0x00($t1)      	 # Delay slot
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t1,0x3C($sp)
		 sw    $t4,0x38($sp)
		 sw    $t6,0x34($sp)
		 sw    $gp,0x30($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x30($sp)
		 lw    $t6,0x34($sp)
		 lw    $t4,0x38($sp)
		 lw    $t1,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t6,$t6,2
		 addiu $t7,$t7,-4
	8:
		 sll   $t0,$t0,1
		 andi  $a3,$t0,0xffff
		 bne   $a3,$0,9b
		 addiu $t1,$t1,4          	 # Delay slot
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_48b9:				#:
		 addiu $s7,$s7,2

		 lhu   $t4,0x00($s7)
		 addiu $s7,$s7,2
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 ori   $t0,$0,1
		 or    $t1,$0,$s5
	9:
		 and   $t2,$t4,$t0
		 beq   $t2,$0,8f
		 lw    $a1,0x00($t1)      	 # Delay slot
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t1,0x3C($sp)
		 sw    $t4,0x38($sp)
		 sw    $t6,0x34($sp)
		 sw    $gp,0x30($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x30($sp)
		 lw    $t6,0x34($sp)
		 lw    $t4,0x38($sp)
		 lw    $t1,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t6,$t6,2
		 addiu $t7,$t7,-4
	8:
		 sll   $t0,$t0,1
		 andi  $a3,$t0,0xffff
		 bne   $a3,$0,9b
		 addiu $t1,$t1,4          	 # Delay slot
		 addiu $t7,$t7,-28
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_48d0:				#:
		 addiu $s7,$s7,2

		 lhu   $t4,0x00($s7)
		 addiu $s7,$s7,2
		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 ori   $t0,$0,1
		 or    $t1,$0,$s5
	9:
		 and   $t2,$t4,$t0
		 beq   $t2,$0,8f
		 lw    $a1,0x00($t1)      	 # Delay slot
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t1,0x3C($sp)
		 sw    $t4,0x38($sp)
		 sw    $t6,0x34($sp)
		 sw    $gp,0x30($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x30($sp)
		 lw    $t6,0x34($sp)
		 lw    $t4,0x38($sp)
		 lw    $t1,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t6,$t6,4
		 addiu $t7,$t7,-8
	8:
		 sll   $t0,$t0,1
		 andi  $a3,$t0,0xffff
		 bne   $a3,$0,9b
		 addiu $t1,$t1,4          	 # Delay slot
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_48e0:				#:
		 addiu $s7,$s7,2

		 lhu   $t4,0x00($s7)
		 addiu $s7,$s7,2
		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 ori   $t0,$0,1
		 addiu $t1,$s5,0x3C
	9:
		 and   $t2,$t4,$t0
		 beq   $t2,$0,8f
		 lw    $a1,0x00($t1)      	 # Delay slot
		 addiu $t6,$t6,-4
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t1,0x3C($sp)
		 sw    $t4,0x38($sp)
		 sw    $t6,0x34($sp)
		 sw    $t8,0x30($sp)
		 sw    $gp,0x2C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x2C($sp)
		 lw    $t8,0x30($sp)
		 lw    $t6,0x34($sp)
		 lw    $t4,0x38($sp)
		 lw    $t1,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-8
	8:
		 sll   $t0,$t0,1
		 andi  $a3,$t0,0xffff
		 bne   $a3,$0,9b
		 addiu $t1,$t1,-4         	 # Delay slot
		 sw    $t6,0x20($t8)
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_48e8:				#:
		 addiu $s7,$s7,2

		 lhu   $t4,0x00($s7)
		 addiu $s7,$s7,2
		 andi  $t8,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 ori   $t0,$0,1
		 or    $t1,$0,$s5
	9:
		 and   $t2,$t4,$t0
		 beq   $t2,$0,8f
		 lw    $a1,0x00($t1)      	 # Delay slot
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t1,0x3C($sp)
		 sw    $t4,0x38($sp)
		 sw    $t6,0x34($sp)
		 sw    $gp,0x30($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x30($sp)
		 lw    $t6,0x34($sp)
		 lw    $t4,0x38($sp)
		 lw    $t1,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t6,$t6,4
		 addiu $t7,$t7,-8
	8:
		 sll   $t0,$t0,1
		 andi  $a3,$t0,0xffff
		 bne   $a3,$0,9b
		 addiu $t1,$t1,4          	 # Delay slot
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_48f0:				#:
		 addiu $s7,$s7,2

		 lhu   $t4,0x00($s7)
		 addiu $s7,$s7,2
		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 ori   $t0,$0,1
		 or    $t1,$0,$s5
	9:
		 and   $t2,$t4,$t0
		 beq   $t2,$0,8f
		 lw    $a1,0x00($t1)      	 # Delay slot
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t1,0x3C($sp)
		 sw    $t4,0x38($sp)
		 sw    $t6,0x34($sp)
		 sw    $gp,0x30($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x30($sp)
		 lw    $t6,0x34($sp)
		 lw    $t4,0x38($sp)
		 lw    $t1,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t6,$t6,4
		 addiu $t7,$t7,-8
	8:
		 sll   $t0,$t0,1
		 andi  $a3,$t0,0xffff
		 bne   $a3,$0,9b
		 addiu $t1,$t1,4          	 # Delay slot
		 addiu $t7,$t7,-28
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_48f8:				#:
		 addiu $s7,$s7,2

		 lhu   $t4,0x00($s7)
		 addiu $s7,$s7,2
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 ori   $t0,$0,1
		 or    $t1,$0,$s5
	9:
		 and   $t2,$t4,$t0
		 beq   $t2,$0,8f
		 lw    $a1,0x00($t1)      	 # Delay slot
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t1,0x3C($sp)
		 sw    $t4,0x38($sp)
		 sw    $t6,0x34($sp)
		 sw    $gp,0x30($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x30($sp)
		 lw    $t6,0x34($sp)
		 lw    $t4,0x38($sp)
		 lw    $t1,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t6,$t6,4
		 addiu $t7,$t7,-8
	8:
		 sll   $t0,$t0,1
		 andi  $a3,$t0,0xffff
		 bne   $a3,$0,9b
		 addiu $t1,$t1,4          	 # Delay slot
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_48f9:				#:
		 addiu $s7,$s7,2

		 lhu   $t4,0x00($s7)
		 addiu $s7,$s7,2
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 ori   $t0,$0,1
		 or    $t1,$0,$s5
	9:
		 and   $t2,$t4,$t0
		 beq   $t2,$0,8f
		 lw    $a1,0x00($t1)      	 # Delay slot
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t1,0x3C($sp)
		 sw    $t4,0x38($sp)
		 sw    $t6,0x34($sp)
		 sw    $gp,0x30($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x30($sp)
		 lw    $t6,0x34($sp)
		 lw    $t4,0x38($sp)
		 lw    $t1,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t6,$t6,4
		 addiu $t7,$t7,-8
	8:
		 sll   $t0,$t0,1
		 andi  $a3,$t0,0xffff
		 bne   $a3,$0,9b
		 addiu $t1,$t1,4          	 # Delay slot
		 addiu $t7,$t7,-28
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4c90:				#:
		 addiu $s7,$s7,2

		 lhu   $t4,0x00($s7)
		 addiu $s7,$s7,2
		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 ori   $t0,$0,1
		 or    $t1,$0,$s5
	9:
		 and   $t2,$t4,$t0
		 beq   $t2,$0,8f
		 nop                    	 # Delay slot
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t1,0x3C($sp)
		 sw    $t4,0x38($sp)
		 sw    $t6,0x34($sp)
		 sw    $gp,0x30($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x30($sp)
		 lw    $t6,0x34($sp)
		 lw    $t4,0x38($sp)
		 lw    $t1,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 sll   $v0,$v0,16
		 sra   $v0,$v0,16
		 sw    $v0,0x00($t1)
		 addiu $t6,$t6,2
		 addiu $t7,$t7,-4
	8:
		 sll   $t0,$t0,1
		 andi  $a3,$t0,0xffff
		 bne   $a3,$0,9b
		 addiu $t1,$t1,4          	 # Delay slot
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4c98:				#:
		 addiu $s7,$s7,2

		 lhu   $t4,0x00($s7)
		 addiu $s7,$s7,2
		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($t8)
		 ori   $t0,$0,1
		 or    $t1,$0,$s5
	9:
		 and   $t2,$t4,$t0
		 beq   $t2,$0,8f
		 nop                    	 # Delay slot
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t1,0x3C($sp)
		 sw    $t4,0x38($sp)
		 sw    $t6,0x34($sp)
		 sw    $t8,0x30($sp)
		 sw    $gp,0x2C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x2C($sp)
		 lw    $t8,0x30($sp)
		 lw    $t6,0x34($sp)
		 lw    $t4,0x38($sp)
		 lw    $t1,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 sll   $v0,$v0,16
		 sra   $v0,$v0,16
		 sw    $v0,0x00($t1)
		 addiu $t6,$t6,2
		 addiu $t7,$t7,-4
	8:
		 sll   $t0,$t0,1
		 andi  $a3,$t0,0xffff
		 bne   $a3,$0,9b
		 addiu $t1,$t1,4          	 # Delay slot
		 sw    $t6,0x20($t8)
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4ca8:				#:
		 addiu $s7,$s7,2

		 lhu   $t4,0x00($s7)
		 addiu $s7,$s7,2
		 andi  $t8,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 ori   $t0,$0,1
		 or    $t1,$0,$s5
	9:
		 and   $t2,$t4,$t0
		 beq   $t2,$0,8f
		 nop                    	 # Delay slot
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t1,0x3C($sp)
		 sw    $t4,0x38($sp)
		 sw    $t6,0x34($sp)
		 sw    $gp,0x30($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x30($sp)
		 lw    $t6,0x34($sp)
		 lw    $t4,0x38($sp)
		 lw    $t1,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 sll   $v0,$v0,16
		 sra   $v0,$v0,16
		 sw    $v0,0x00($t1)
		 addiu $t6,$t6,2
		 addiu $t7,$t7,-4
	8:
		 sll   $t0,$t0,1
		 andi  $a3,$t0,0xffff
		 bne   $a3,$0,9b
		 addiu $t1,$t1,4          	 # Delay slot
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4cb0:				#:
		 addiu $s7,$s7,2

		 lhu   $t4,0x00($s7)
		 addiu $s7,$s7,2
		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 ori   $t0,$0,1
		 or    $t1,$0,$s5
	9:
		 and   $t2,$t4,$t0
		 beq   $t2,$0,8f
		 nop                    	 # Delay slot
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t1,0x3C($sp)
		 sw    $t4,0x38($sp)
		 sw    $t6,0x34($sp)
		 sw    $gp,0x30($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x30($sp)
		 lw    $t6,0x34($sp)
		 lw    $t4,0x38($sp)
		 lw    $t1,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 sll   $v0,$v0,16
		 sra   $v0,$v0,16
		 sw    $v0,0x00($t1)
		 addiu $t6,$t6,2
		 addiu $t7,$t7,-4
	8:
		 sll   $t0,$t0,1
		 andi  $a3,$t0,0xffff
		 bne   $a3,$0,9b
		 addiu $t1,$t1,4          	 # Delay slot
		 addiu $t7,$t7,-28
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4cb8:				#:
		 addiu $s7,$s7,2

		 lhu   $t4,0x00($s7)
		 addiu $s7,$s7,2
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 ori   $t0,$0,1
		 or    $t1,$0,$s5
	9:
		 and   $t2,$t4,$t0
		 beq   $t2,$0,8f
		 nop                    	 # Delay slot
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t1,0x3C($sp)
		 sw    $t4,0x38($sp)
		 sw    $t6,0x34($sp)
		 sw    $gp,0x30($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x30($sp)
		 lw    $t6,0x34($sp)
		 lw    $t4,0x38($sp)
		 lw    $t1,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 sll   $v0,$v0,16
		 sra   $v0,$v0,16
		 sw    $v0,0x00($t1)
		 addiu $t6,$t6,2
		 addiu $t7,$t7,-4
	8:
		 sll   $t0,$t0,1
		 andi  $a3,$t0,0xffff
		 bne   $a3,$0,9b
		 addiu $t1,$t1,4          	 # Delay slot
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4cb9:				#:
		 addiu $s7,$s7,2

		 lhu   $t4,0x00($s7)
		 addiu $s7,$s7,2
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 ori   $t0,$0,1
		 or    $t1,$0,$s5
	9:
		 and   $t2,$t4,$t0
		 beq   $t2,$0,8f
		 nop                    	 # Delay slot
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t1,0x3C($sp)
		 sw    $t4,0x38($sp)
		 sw    $t6,0x34($sp)
		 sw    $gp,0x30($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x30($sp)
		 lw    $t6,0x34($sp)
		 lw    $t4,0x38($sp)
		 lw    $t1,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 sll   $v0,$v0,16
		 sra   $v0,$v0,16
		 sw    $v0,0x00($t1)
		 addiu $t6,$t6,2
		 addiu $t7,$t7,-4
	8:
		 sll   $t0,$t0,1
		 andi  $a3,$t0,0xffff
		 bne   $a3,$0,9b
		 addiu $t1,$t1,4          	 # Delay slot
		 addiu $t7,$t7,-28
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4cba:				#:
		 addiu $s7,$s7,2

		 lhu   $t4,0x00($s7)
		 addiu $s7,$s7,2
		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 ori   $t0,$0,1
		 or    $t1,$0,$s5
	9:
		 and   $t2,$t4,$t0
		 beq   $t2,$0,8f
		 nop                    	 # Delay slot
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t1,0x3C($sp)
		 sw    $t4,0x38($sp)
		 sw    $t6,0x34($sp)
		 sw    $gp,0x30($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x30($sp)
		 lw    $t6,0x34($sp)
		 lw    $t4,0x38($sp)
		 lw    $t1,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 sll   $v0,$v0,16
		 sra   $v0,$v0,16
		 sw    $v0,0x00($t1)
		 addiu $t6,$t6,2
		 addiu $t7,$t7,-4
	8:
		 sll   $t0,$t0,1
		 andi  $a3,$t0,0xffff
		 bne   $a3,$0,9b
		 addiu $t1,$t1,4          	 # Delay slot
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4cbb:				#:
		 addiu $s7,$s7,2

		 lhu   $t4,0x00($s7)
		 addiu $s7,$s7,2
		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 ori   $t0,$0,1
		 or    $t1,$0,$s5
	9:
		 and   $t2,$t4,$t0
		 beq   $t2,$0,8f
		 nop                    	 # Delay slot
		 lw    $t9,0xB0($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t1,0x3C($sp)
		 sw    $t4,0x38($sp)
		 sw    $t6,0x34($sp)
		 sw    $gp,0x30($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x30($sp)
		 lw    $t6,0x34($sp)
		 lw    $t4,0x38($sp)
		 lw    $t1,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 sll   $v0,$v0,16
		 sra   $v0,$v0,16
		 sw    $v0,0x00($t1)
		 addiu $t6,$t6,2
		 addiu $t7,$t7,-4
	8:
		 sll   $t0,$t0,1
		 andi  $a3,$t0,0xffff
		 bne   $a3,$0,9b
		 addiu $t1,$t1,4          	 # Delay slot
		 addiu $t7,$t7,-26
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4cd0:				#:
		 addiu $s7,$s7,2

		 lhu   $t4,0x00($s7)
		 addiu $s7,$s7,2
		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 ori   $t0,$0,1
		 or    $t1,$0,$s5
	9:
		 and   $t2,$t4,$t0
		 beq   $t2,$0,8f
		 nop                    	 # Delay slot
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t1,0x3C($sp)
		 sw    $t4,0x38($sp)
		 sw    $t6,0x34($sp)
		 sw    $gp,0x30($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x30($sp)
		 lw    $t6,0x34($sp)
		 lw    $t4,0x38($sp)
		 lw    $t1,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 sw    $v0,0x00($t1)
		 addiu $t6,$t6,4
		 addiu $t7,$t7,-8
	8:
		 sll   $t0,$t0,1
		 andi  $a3,$t0,0xffff
		 bne   $a3,$0,9b
		 addiu $t1,$t1,4          	 # Delay slot
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4cd8:				#:
		 addiu $s7,$s7,2

		 lhu   $t4,0x00($s7)
		 addiu $s7,$s7,2
		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,4
		 sw    $t9,0x20($t8)
		 ori   $t0,$0,1
		 or    $t1,$0,$s5
	9:
		 and   $t2,$t4,$t0
		 beq   $t2,$0,8f
		 nop                    	 # Delay slot
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t1,0x3C($sp)
		 sw    $t4,0x38($sp)
		 sw    $t6,0x34($sp)
		 sw    $t8,0x30($sp)
		 sw    $gp,0x2C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x2C($sp)
		 lw    $t8,0x30($sp)
		 lw    $t6,0x34($sp)
		 lw    $t4,0x38($sp)
		 lw    $t1,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 sw    $v0,0x00($t1)
		 addiu $t6,$t6,4
		 addiu $t7,$t7,-8
	8:
		 sll   $t0,$t0,1
		 andi  $a3,$t0,0xffff
		 bne   $a3,$0,9b
		 addiu $t1,$t1,4          	 # Delay slot
		 sw    $t6,0x20($t8)
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4ce8:				#:
		 addiu $s7,$s7,2

		 lhu   $t4,0x00($s7)
		 addiu $s7,$s7,2
		 andi  $t8,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 ori   $t0,$0,1
		 or    $t1,$0,$s5
	9:
		 and   $t2,$t4,$t0
		 beq   $t2,$0,8f
		 nop                    	 # Delay slot
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t1,0x3C($sp)
		 sw    $t4,0x38($sp)
		 sw    $t6,0x34($sp)
		 sw    $gp,0x30($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x30($sp)
		 lw    $t6,0x34($sp)
		 lw    $t4,0x38($sp)
		 lw    $t1,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 sw    $v0,0x00($t1)
		 addiu $t6,$t6,4
		 addiu $t7,$t7,-8
	8:
		 sll   $t0,$t0,1
		 andi  $a3,$t0,0xffff
		 bne   $a3,$0,9b
		 addiu $t1,$t1,4          	 # Delay slot
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4cf0:				#:
		 addiu $s7,$s7,2

		 lhu   $t4,0x00($s7)
		 addiu $s7,$s7,2
		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 ori   $t0,$0,1
		 or    $t1,$0,$s5
	9:
		 and   $t2,$t4,$t0
		 beq   $t2,$0,8f
		 nop                    	 # Delay slot
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t1,0x3C($sp)
		 sw    $t4,0x38($sp)
		 sw    $t6,0x34($sp)
		 sw    $gp,0x30($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x30($sp)
		 lw    $t6,0x34($sp)
		 lw    $t4,0x38($sp)
		 lw    $t1,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 sw    $v0,0x00($t1)
		 addiu $t6,$t6,4
		 addiu $t7,$t7,-8
	8:
		 sll   $t0,$t0,1
		 andi  $a3,$t0,0xffff
		 bne   $a3,$0,9b
		 addiu $t1,$t1,4          	 # Delay slot
		 addiu $t7,$t7,-28
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4cf8:				#:
		 addiu $s7,$s7,2

		 lhu   $t4,0x00($s7)
		 addiu $s7,$s7,2
		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 ori   $t0,$0,1
		 or    $t1,$0,$s5
	9:
		 and   $t2,$t4,$t0
		 beq   $t2,$0,8f
		 nop                    	 # Delay slot
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t1,0x3C($sp)
		 sw    $t4,0x38($sp)
		 sw    $t6,0x34($sp)
		 sw    $gp,0x30($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x30($sp)
		 lw    $t6,0x34($sp)
		 lw    $t4,0x38($sp)
		 lw    $t1,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 sw    $v0,0x00($t1)
		 addiu $t6,$t6,4
		 addiu $t7,$t7,-8
	8:
		 sll   $t0,$t0,1
		 andi  $a3,$t0,0xffff
		 bne   $a3,$0,9b
		 addiu $t1,$t1,4          	 # Delay slot
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4cf9:				#:
		 addiu $s7,$s7,2

		 lhu   $t4,0x00($s7)
		 addiu $s7,$s7,2
		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 ori   $t0,$0,1
		 or    $t1,$0,$s5
	9:
		 and   $t2,$t4,$t0
		 beq   $t2,$0,8f
		 nop                    	 # Delay slot
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t1,0x3C($sp)
		 sw    $t4,0x38($sp)
		 sw    $t6,0x34($sp)
		 sw    $gp,0x30($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x30($sp)
		 lw    $t6,0x34($sp)
		 lw    $t4,0x38($sp)
		 lw    $t1,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 sw    $v0,0x00($t1)
		 addiu $t6,$t6,4
		 addiu $t7,$t7,-8
	8:
		 sll   $t0,$t0,1
		 andi  $a3,$t0,0xffff
		 bne   $a3,$0,9b
		 addiu $t1,$t1,4          	 # Delay slot
		 addiu $t7,$t7,-28
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4cfa:				#:
		 addiu $s7,$s7,2

		 lhu   $t4,0x00($s7)
		 addiu $s7,$s7,2
		 lh    $a3,0x00($s7)
		 subu  $t9,$s7,$s6
		 addu  $t6,$t9,$a3       	 # Add Offset to PC
		 addiu $s7,$s7,2
		 ori   $t0,$0,1
		 or    $t1,$0,$s5
	9:
		 and   $t2,$t4,$t0
		 beq   $t2,$0,8f
		 nop                    	 # Delay slot
		 lw    $t9,0xB4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t1,0x3C($sp)
		 sw    $t4,0x38($sp)
		 sw    $t6,0x34($sp)
		 sw    $gp,0x30($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x30($sp)
		 lw    $t6,0x34($sp)
		 lw    $t4,0x38($sp)
		 lw    $t1,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 sw    $v0,0x00($t1)
		 addiu $t6,$t6,4
		 addiu $t7,$t7,-8
	8:
		 sll   $t0,$t0,1
		 andi  $a3,$t0,0xffff
		 bne   $a3,$0,9b
		 addiu $t1,$t1,4          	 # Delay slot
		 addiu $t7,$t7,-24
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4cfb:				#:
		 addiu $s7,$s7,2

		 lhu   $t4,0x00($s7)
		 addiu $s7,$s7,2
		 subu  $t6,$s7,$s6       	 # Get PC
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 ori   $t0,$0,1
		 or    $t1,$0,$s5
	9:
		 and   $t2,$t4,$t0
		 beq   $t2,$0,8f
		 nop                    	 # Delay slot
		 lw    $t9,0xB4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t1,0x3C($sp)
		 sw    $t4,0x38($sp)
		 sw    $t6,0x34($sp)
		 sw    $gp,0x30($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x30($sp)
		 lw    $t6,0x34($sp)
		 lw    $t4,0x38($sp)
		 lw    $t1,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 sw    $v0,0x00($t1)
		 addiu $t6,$t6,4
		 addiu $t7,$t7,-8
	8:
		 sll   $t0,$t0,1
		 andi  $a3,$t0,0xffff
		 bne   $a3,$0,9b
		 addiu $t1,$t1,4          	 # Delay slot
		 addiu $t7,$t7,-26
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4e50:				#:
		 addiu $s7,$s7,2

		 lw    $t6,0x3C($s5)
		 andi  $t8,$t8,0x07
		 addiu $t6,$t6,-4
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $a1,0x20($t8)
		 sw    $t6,0x20($t8)
		 lw    $t9,0xA4($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 lh    $v0,0x00($s7)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$v0
		 sw    $t6,0x3C($s5)
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4e58:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $t6,0x20($t0)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $t0,0x40($sp)
		 sw    $t6,0x3C($sp)
		 sw    $gp,0x38($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x38($sp)
		 lw    $t6,0x3C($sp)
		 lw    $t0,0x40($sp)
		 lw    $t7,m68k_ICount
		 sw    $v0,0x20($t0)
		 addiu $t6,$t6,4
		 sw    $t6,0x3C($s5)
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e000:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lb    $v0,0x00($t0)
		 and   $s1,$0,$0        	 # Clear Overflow
		 beq   $t8,$0,9f
		 and   $s0,$0,$0        	 # Delay slot - Clear Carry
		 sll   $t9,$t8,1
		 subu  $t7,$t7,$t9
		 addiu $t1,$t8,-1
		 sll   $t9,$t8,26
		 sra   $t9,$t9,31
		 andi  $t9,$t9,0x1f
		 or    $t8,$t8,$t9
		 sll   $t9,$t1,26
		 sra   $t9,$t9,31
		 andi  $t9,$t9,0x1f
		 or    $t1,$t1,$t9
		 srlv  $s0,$v0,$t1
		 andi  $s0,$s0,0x01     	 # Set Carry
		 srav  $v0,$v0,$t8
		 sb    $v0,0x00($t0)
		 or    $s4,$0,$s0       	 # Copy Carry to X
	9:
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e020:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t8,0x00($t8)
		 andi  $t8,$t8,0x3F
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lb    $v0,0x00($t0)
		 and   $s1,$0,$0        	 # Clear Overflow
		 beq   $t8,$0,9f
		 and   $s0,$0,$0        	 # Delay slot - Clear Carry
		 sll   $t9,$t8,1
		 subu  $t7,$t7,$t9
		 addiu $t1,$t8,-1
		 sll   $t9,$t8,26
		 sra   $t9,$t9,31
		 andi  $t9,$t9,0x1f
		 or    $t8,$t8,$t9
		 sll   $t9,$t1,26
		 sra   $t9,$t9,31
		 andi  $t9,$t9,0x1f
		 or    $t1,$t1,$t9
		 srlv  $s0,$v0,$t1
		 andi  $s0,$s0,0x01     	 # Set Carry
		 srav  $v0,$v0,$t8
		 sb    $v0,0x00($t0)
		 or    $s4,$0,$s0       	 # Copy Carry to X
	9:
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e040:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lh    $v0,0x00($t0)
		 and   $s1,$0,$0        	 # Clear Overflow
		 beq   $t8,$0,9f
		 and   $s0,$0,$0        	 # Delay slot - Clear Carry
		 sll   $t9,$t8,1
		 subu  $t7,$t7,$t9
		 addiu $t1,$t8,-1
		 sll   $t9,$t8,26
		 sra   $t9,$t9,31
		 andi  $t9,$t9,0x1f
		 or    $t8,$t8,$t9
		 sll   $t9,$t1,26
		 sra   $t9,$t9,31
		 andi  $t9,$t9,0x1f
		 or    $t1,$t1,$t9
		 srlv  $s0,$v0,$t1
		 andi  $s0,$s0,0x01     	 # Set Carry
		 srav  $v0,$v0,$t8
		 sh    $v0,0x00($t0)
		 or    $s4,$0,$s0       	 # Copy Carry to X
	9:
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e060:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t8,0x00($t8)
		 andi  $t8,$t8,0x3F
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lh    $v0,0x00($t0)
		 and   $s1,$0,$0        	 # Clear Overflow
		 beq   $t8,$0,9f
		 and   $s0,$0,$0        	 # Delay slot - Clear Carry
		 sll   $t9,$t8,1
		 subu  $t7,$t7,$t9
		 addiu $t1,$t8,-1
		 sll   $t9,$t8,26
		 sra   $t9,$t9,31
		 andi  $t9,$t9,0x1f
		 or    $t8,$t8,$t9
		 sll   $t9,$t1,26
		 sra   $t9,$t9,31
		 andi  $t9,$t9,0x1f
		 or    $t1,$t1,$t9
		 srlv  $s0,$v0,$t1
		 andi  $s0,$s0,0x01     	 # Set Carry
		 srav  $v0,$v0,$t8
		 sh    $v0,0x00($t0)
		 or    $s4,$0,$s0       	 # Copy Carry to X
	9:
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e080:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $v0,0x00($t0)
		 and   $s1,$0,$0        	 # Clear Overflow
		 beq   $t8,$0,9f
		 and   $s0,$0,$0        	 # Delay slot - Clear Carry
		 sll   $t9,$t8,1
		 subu  $t7,$t7,$t9
		 addiu $t1,$t8,-1
		 sll   $t9,$t8,26
		 sra   $t9,$t9,31
		 andi  $t9,$t9,0x1f
		 or    $t8,$t8,$t9
		 sll   $t9,$t1,26
		 sra   $t9,$t9,31
		 andi  $t9,$t9,0x1f
		 or    $t1,$t1,$t9
		 srlv  $s0,$v0,$t1
		 andi  $s0,$s0,0x01     	 # Set Carry
		 srav  $v0,$v0,$t8
		 sw    $v0,0x00($t0)
		 or    $s4,$0,$s0       	 # Copy Carry to X
	9:
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e0a0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t8,0x00($t8)
		 andi  $t8,$t8,0x3F
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $v0,0x00($t0)
		 and   $s1,$0,$0        	 # Clear Overflow
		 beq   $t8,$0,9f
		 and   $s0,$0,$0        	 # Delay slot - Clear Carry
		 sll   $t9,$t8,1
		 subu  $t7,$t7,$t9
		 addiu $t1,$t8,-1
		 sll   $t9,$t8,26
		 sra   $t9,$t9,31
		 andi  $t9,$t9,0x1f
		 or    $t8,$t8,$t9
		 sll   $t9,$t1,26
		 sra   $t9,$t9,31
		 andi  $t9,$t9,0x1f
		 or    $t1,$t1,$t9
		 srlv  $s0,$v0,$t1
		 andi  $s0,$s0,0x01     	 # Set Carry
		 srav  $v0,$v0,$t8
		 sw    $v0,0x00($t0)
		 or    $s4,$0,$s0       	 # Copy Carry to X
	9:
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e100:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lb    $v0,0x00($t0)
		 and   $s1,$0,$0        	 # Clear Overflow
		 beq   $t8,$0,9f
		 and   $s0,$0,$0        	 # Delay slot - Clear Carry
		 sll   $t9,$t8,1
		 subu  $t7,$t7,$t9
		 sll   $t9,$t8,26
		 sra   $t9,$t9,31
		 andi  $t9,$t9,0x1f
		 or    $t9,$t8,$t9
		 clz   $t1,$v0
		 clo   $t2,$v0
		 addiu $t1,$t1,-24
		 addiu $t2,$t2,-24
		 sltu  $t1,$t1,$t9
		 sltu  $t2,$t2,$t9
		 or    $s1,$t1,$t2        	 # Set Overflow
		 subu  $t1,$0,$t8
		 andi  $t1,$t1,0x07
		 srlv   $s0,$v0,$t1
		 andi  $s0,$s0,0x01     	 # Set Carry
		 sllv   $v0,$v0,$t8
		 sb    $v0,0x00($t0)
		 or    $s4,$0,$s0       	 # Copy Carry to X
	9:
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e120:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t8,0x00($t8)
		 andi  $t8,$t8,0x3F
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lb    $v0,0x00($t0)
		 and   $s1,$0,$0        	 # Clear Overflow
		 beq   $t8,$0,9f
		 and   $s0,$0,$0        	 # Delay slot - Clear Carry
		 sll   $t9,$t8,1
		 subu  $t7,$t7,$t9
		 sll   $t9,$t8,26
		 sra   $t9,$t9,31
		 andi  $t9,$t9,0x1f
		 or    $t9,$t8,$t9
		 clz   $t1,$v0
		 clo   $t2,$v0
		 addiu $t1,$t1,-24
		 addiu $t2,$t2,-24
		 sltu  $t1,$t1,$t9
		 sltu  $t2,$t2,$t9
		 or    $s1,$t1,$t2        	 # Set Overflow
		 subu  $t1,$0,$t8
		 andi  $t1,$t1,0x07
		 srlv   $s0,$v0,$t1
		 andi  $s0,$s0,0x01     	 # Set Carry
		 sllv   $v0,$v0,$t8
		 sb    $v0,0x00($t0)
		 or    $s4,$0,$s0       	 # Copy Carry to X
	9:
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e140:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lh    $v0,0x00($t0)
		 and   $s1,$0,$0        	 # Clear Overflow
		 beq   $t8,$0,9f
		 and   $s0,$0,$0        	 # Delay slot - Clear Carry
		 sll   $t9,$t8,1
		 subu  $t7,$t7,$t9
		 sll   $t9,$t8,26
		 sra   $t9,$t9,31
		 andi  $t9,$t9,0x1f
		 or    $t9,$t8,$t9
		 clz   $t1,$v0
		 clo   $t2,$v0
		 addiu $t1,$t1,-16
		 addiu $t2,$t2,-16
		 sltu  $t1,$t1,$t9
		 sltu  $t2,$t2,$t9
		 or    $s1,$t1,$t2        	 # Set Overflow
		 subu  $t1,$0,$t8
		 andi  $t1,$t1,0x0F
		 srlv   $s0,$v0,$t1
		 andi  $s0,$s0,0x01     	 # Set Carry
		 sllv   $v0,$v0,$t8
		 sh    $v0,0x00($t0)
		 or    $s4,$0,$s0       	 # Copy Carry to X
	9:
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e160:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t8,0x00($t8)
		 andi  $t8,$t8,0x3F
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lh    $v0,0x00($t0)
		 and   $s1,$0,$0        	 # Clear Overflow
		 beq   $t8,$0,9f
		 and   $s0,$0,$0        	 # Delay slot - Clear Carry
		 sll   $t9,$t8,1
		 subu  $t7,$t7,$t9
		 sll   $t9,$t8,26
		 sra   $t9,$t9,31
		 andi  $t9,$t9,0x1f
		 or    $t9,$t8,$t9
		 clz   $t1,$v0
		 clo   $t2,$v0
		 addiu $t1,$t1,-16
		 addiu $t2,$t2,-16
		 sltu  $t1,$t1,$t9
		 sltu  $t2,$t2,$t9
		 or    $s1,$t1,$t2        	 # Set Overflow
		 subu  $t1,$0,$t8
		 andi  $t1,$t1,0x0F
		 srlv   $s0,$v0,$t1
		 andi  $s0,$s0,0x01     	 # Set Carry
		 sllv   $v0,$v0,$t8
		 sh    $v0,0x00($t0)
		 or    $s4,$0,$s0       	 # Copy Carry to X
	9:
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e180:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $v0,0x00($t0)
		 and   $s1,$0,$0        	 # Clear Overflow
		 beq   $t8,$0,9f
		 and   $s0,$0,$0        	 # Delay slot - Clear Carry
		 sll   $t9,$t8,1
		 subu  $t7,$t7,$t9
		 sll   $t9,$t8,26
		 sra   $t9,$t9,31
		 andi  $t9,$t9,0x1f
		 or    $t9,$t8,$t9
		 clz   $t1,$v0
		 clo   $t2,$v0
		 sltu  $t1,$t1,$t9
		 sltu  $t2,$t2,$t9
		 or    $s1,$t1,$t2        	 # Set Overflow
		 subu  $t1,$0,$t8
		 srlv   $s0,$v0,$t1
		 andi  $s0,$s0,0x01     	 # Set Carry
		 sllv   $v0,$v0,$t8
		 sw    $v0,0x00($t0)
		 or    $s4,$0,$s0       	 # Copy Carry to X
	9:
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e1a0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t8,0x00($t8)
		 andi  $t8,$t8,0x3F
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $v0,0x00($t0)
		 and   $s1,$0,$0        	 # Clear Overflow
		 beq   $t8,$0,9f
		 and   $s0,$0,$0        	 # Delay slot - Clear Carry
		 sll   $t9,$t8,1
		 subu  $t7,$t7,$t9
		 sll   $t9,$t8,26
		 sra   $t9,$t9,31
		 andi  $t9,$t9,0x1f
		 or    $t9,$t8,$t9
		 clz   $t1,$v0
		 clo   $t2,$v0
		 sltu  $t1,$t1,$t9
		 sltu  $t2,$t2,$t9
		 or    $s1,$t1,$t2        	 # Set Overflow
		 andi  $t1,$t8,0x20
		 movn  $v0,$0,$t1
		 subu  $t1,$0,$t8
		 srlv   $s0,$v0,$t1
		 andi  $s0,$s0,0x01     	 # Set Carry
		 sllv   $v0,$v0,$t8
		 sw    $v0,0x00($t0)
		 or    $s4,$0,$s0       	 # Copy Carry to X
	9:
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e300:				#:
		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lbu   $v1,0x00($t8)

		 addiu $s7,$s7,2

		 sll   $v0,$v1,1
		 sb    $v0,0x00($t8)
		 srl   $s0,$v1,7        	 # Set Carry
		 xor   $s1,$v0,$v1
		 srl   $s1,$s1,7
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 andi  $s2,$v0,0xFF
		 sltiu $s2,$s2,1        	 # Set Zero
		 srl   $s3,$v0,7        	 # Set Sign
		 andi  $s3,$s3,1
		 or    $s4,$0,$s0       	 # Copy Carry to X
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e340:				#:
		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lhu   $v1,0x00($t8)

		 addiu $s7,$s7,2

		 sll   $v0,$v1,1
		 sh    $v0,0x00($t8)
		 srl   $s0,$v1,15        	 # Set Carry
		 xor   $s1,$v0,$v1
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 andi  $s2,$v0,0xFFFF
		 sltiu $s2,$s2,1        	 # Set Zero
		 srl   $s3,$v0,15        	 # Set Sign
		 andi  $s3,$s3,1
		 or    $s4,$0,$s0       	 # Copy Carry to X
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e380:				#:
		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $v1,0x00($t8)

		 addiu $s7,$s7,2

		 sll   $v0,$v1,1
		 sw    $v0,0x00($t8)
		 srl   $s0,$v1,31        	 # Set Carry
		 xor   $s1,$v0,$v1
		 srl   $s1,$s1,31       	 # Set Overflow
		 or    $s2,$0,$v0
		 sltiu $s2,$s2,1        	 # Set Zero
		 srl   $s3,$v0,31        	 # Set Sign
		 andi  $s3,$s3,1
		 or    $s4,$0,$s0       	 # Copy Carry to X
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e0d0:				#:

		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 andi  $s0,$v0,0x01      	 # Set Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 sra   $v0,$v0,1
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1        	 # Set Zero
		 or    $s4,$0,$s0       	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e0d8:				#:

		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 andi  $s0,$v0,0x01      	 # Set Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 sra   $v0,$v0,1
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1        	 # Set Zero
		 or    $s4,$0,$s0       	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e0e0:				#:

		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 andi  $s0,$v0,0x01      	 # Set Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 sra   $v0,$v0,1
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1        	 # Set Zero
		 or    $s4,$0,$s0       	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e0e8:				#:

		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 andi  $s0,$v0,0x01      	 # Set Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 sra   $v0,$v0,1
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1        	 # Set Zero
		 or    $s4,$0,$s0       	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e0f0:				#:

		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 andi  $s0,$v0,0x01      	 # Set Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 sra   $v0,$v0,1
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1        	 # Set Zero
		 or    $s4,$0,$s0       	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e0f8:				#:

		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 andi  $s0,$v0,0x01      	 # Set Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 sra   $v0,$v0,1
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1        	 # Set Zero
		 or    $s4,$0,$s0       	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e0f9:				#:

		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 m_seh   $v0,$v0
		 andi  $s0,$v0,0x01      	 # Set Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 sra   $v0,$v0,1
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1        	 # Set Zero
		 or    $s4,$0,$s0       	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e1d0:				#:

		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $v1,$0,$v0
		 sll   $v0,$v1,1
		 srl   $s0,$v1,15        	 # Set Carry
		 xor   $s1,$v0,$v1
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 andi  $v0,$v0,0xffff
		 srl   $s3,$v0,15        	 # Set Sign
		 sltiu $s2,$v0,1        	 # Set Zero
		 or    $s4,$0,$s0       	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e1d8:				#:

		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $v1,$0,$v0
		 sll   $v0,$v1,1
		 srl   $s0,$v1,15        	 # Set Carry
		 xor   $s1,$v0,$v1
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 andi  $v0,$v0,0xffff
		 srl   $s3,$v0,15        	 # Set Sign
		 sltiu $s2,$v0,1        	 # Set Zero
		 or    $s4,$0,$s0       	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e1e0:				#:

		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $v1,$0,$v0
		 sll   $v0,$v1,1
		 srl   $s0,$v1,15        	 # Set Carry
		 xor   $s1,$v0,$v1
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 andi  $v0,$v0,0xffff
		 srl   $s3,$v0,15        	 # Set Sign
		 sltiu $s2,$v0,1        	 # Set Zero
		 or    $s4,$0,$s0       	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e1e8:				#:

		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $v1,$0,$v0
		 sll   $v0,$v1,1
		 srl   $s0,$v1,15        	 # Set Carry
		 xor   $s1,$v0,$v1
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 andi  $v0,$v0,0xffff
		 srl   $s3,$v0,15        	 # Set Sign
		 sltiu $s2,$v0,1        	 # Set Zero
		 or    $s4,$0,$s0       	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e1f0:				#:

		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $v1,$0,$v0
		 sll   $v0,$v1,1
		 srl   $s0,$v1,15        	 # Set Carry
		 xor   $s1,$v0,$v1
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 andi  $v0,$v0,0xffff
		 srl   $s3,$v0,15        	 # Set Sign
		 sltiu $s2,$v0,1        	 # Set Zero
		 or    $s4,$0,$s0       	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e1f8:				#:

		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $v1,$0,$v0
		 sll   $v0,$v1,1
		 srl   $s0,$v1,15        	 # Set Carry
		 xor   $s1,$v0,$v1
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 andi  $v0,$v0,0xffff
		 srl   $s3,$v0,15        	 # Set Sign
		 sltiu $s2,$v0,1        	 # Set Zero
		 or    $s4,$0,$s0       	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e1f9:				#:

		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 or    $v1,$0,$v0
		 sll   $v0,$v1,1
		 srl   $s0,$v1,15        	 # Set Carry
		 xor   $s1,$v0,$v1
		 srl   $s1,$s1,15
		 andi  $s1,$s1,0x01     	 # Set Overflow
		 andi  $v0,$v0,0xffff
		 srl   $s3,$v0,15        	 # Set Sign
		 sltiu $s2,$v0,1        	 # Set Zero
		 or    $s4,$0,$s0       	 # Copy Carry to X
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e010:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lbu   $v0,0x00($t0)
		 beq   $t8,$0,9f
		 or    $s0,$0,$s4       	 # Delay slot - Copy X to Carry
		 sll   $t9,$t8,1
		 andi  $t1,$t8,1
		 bne   $t1,$t8,7f
		 subu  $t7,$t7,$t9       	 # Delay slot
		 andi  $s4,$v0,0x01      	 # Set X
		 sll   $s0,$s0,7
		 srl   $v0,$v0,1
		 bgez  $0,6f
		 or    $v0,$v0,$s0        	 # Delay slot
	7:
		 ori   $t3,$0,9
	8:
		 subu  $t1,$t8,$t3
		 addiu $t9,$t1,-1
		 slt   $t9,$t9,$0
		 beq   $t9,$0,8b
		 movz  $t8,$t1,$t9        	 # Delay slot
		 subu  $t3,$t3,$t8
		 addiu $t1,$t8,-1
		 addiu $t2,$t3,-1
		 sllv  $s0,$s0,$t2
		 srlv  $s4,$v0,$t1
		 andi  $s4,$s4,0x01     	 # Set X
		 sllv  $t9,$v0,$t3
		 srlv  $v0,$v0,$t8
		 or    $v0,$v0,$t9
		 or    $v0,$v0,$s0
	6:
		 or    $s0,$0,$s4       	 # Copy X to Carry
	9:
		 andi  $v0,$v0,0xFF
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 sb    $v0,0x00($t0)
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e030:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t8,0x00($t8)
		 andi  $t8,$t8,0x3F
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lbu   $v0,0x00($t0)
		 beq   $t8,$0,9f
		 or    $s0,$0,$s4       	 # Delay slot - Copy X to Carry
		 sll   $t9,$t8,1
		 andi  $t1,$t8,1
		 bne   $t1,$t8,7f
		 subu  $t7,$t7,$t9       	 # Delay slot
		 andi  $s4,$v0,0x01      	 # Set X
		 sll   $s0,$s0,7
		 srl   $v0,$v0,1
		 bgez  $0,6f
		 or    $v0,$v0,$s0        	 # Delay slot
	7:
		 ori   $t3,$0,9
	8:
		 subu  $t1,$t8,$t3
		 addiu $t9,$t1,-1
		 slt   $t9,$t9,$0
		 beq   $t9,$0,8b
		 movz  $t8,$t1,$t9        	 # Delay slot
		 subu  $t3,$t3,$t8
		 addiu $t1,$t8,-1
		 addiu $t2,$t3,-1
		 sllv  $s0,$s0,$t2
		 srlv  $s4,$v0,$t1
		 andi  $s4,$s4,0x01     	 # Set X
		 sllv  $t9,$v0,$t3
		 srlv  $v0,$v0,$t8
		 or    $v0,$v0,$t9
		 or    $v0,$v0,$s0
	6:
		 or    $s0,$0,$s4       	 # Copy X to Carry
	9:
		 andi  $v0,$v0,0xFF
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 sb    $v0,0x00($t0)
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e050:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lhu   $v0,0x00($t0)
		 beq   $t8,$0,9f
		 or    $s0,$0,$s4       	 # Delay slot - Copy X to Carry
		 sll   $t9,$t8,1
		 andi  $t1,$t8,1
		 bne   $t1,$t8,7f
		 subu  $t7,$t7,$t9       	 # Delay slot
		 andi  $s4,$v0,0x01      	 # Set X
		 sll   $s0,$s0,15
		 srl   $v0,$v0,1
		 bgez  $0,6f
		 or    $v0,$v0,$s0        	 # Delay slot
	7:
		 ori   $t3,$0,17
	8:
		 subu  $t1,$t8,$t3
		 addiu $t9,$t1,-1
		 slt   $t9,$t9,$0
		 beq   $t9,$0,8b
		 movz  $t8,$t1,$t9        	 # Delay slot
		 subu  $t3,$t3,$t8
		 addiu $t1,$t8,-1
		 addiu $t2,$t3,-1
		 sllv  $s0,$s0,$t2
		 srlv  $s4,$v0,$t1
		 andi  $s4,$s4,0x01     	 # Set X
		 sllv  $t9,$v0,$t3
		 srlv  $v0,$v0,$t8
		 or    $v0,$v0,$t9
		 or    $v0,$v0,$s0
	6:
		 or    $s0,$0,$s4       	 # Copy X to Carry
	9:
		 andi  $v0,$v0,0xFFFF
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 sh    $v0,0x00($t0)
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e070:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t8,0x00($t8)
		 andi  $t8,$t8,0x3F
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lhu   $v0,0x00($t0)
		 beq   $t8,$0,9f
		 or    $s0,$0,$s4       	 # Delay slot - Copy X to Carry
		 sll   $t9,$t8,1
		 andi  $t1,$t8,1
		 bne   $t1,$t8,7f
		 subu  $t7,$t7,$t9       	 # Delay slot
		 andi  $s4,$v0,0x01      	 # Set X
		 sll   $s0,$s0,15
		 srl   $v0,$v0,1
		 bgez  $0,6f
		 or    $v0,$v0,$s0        	 # Delay slot
	7:
		 ori   $t3,$0,17
	8:
		 subu  $t1,$t8,$t3
		 addiu $t9,$t1,-1
		 slt   $t9,$t9,$0
		 beq   $t9,$0,8b
		 movz  $t8,$t1,$t9        	 # Delay slot
		 subu  $t3,$t3,$t8
		 addiu $t1,$t8,-1
		 addiu $t2,$t3,-1
		 sllv  $s0,$s0,$t2
		 srlv  $s4,$v0,$t1
		 andi  $s4,$s4,0x01     	 # Set X
		 sllv  $t9,$v0,$t3
		 srlv  $v0,$v0,$t8
		 or    $v0,$v0,$t9
		 or    $v0,$v0,$s0
	6:
		 or    $s0,$0,$s4       	 # Copy X to Carry
	9:
		 andi  $v0,$v0,0xFFFF
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 sh    $v0,0x00($t0)
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e090:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $v0,0x00($t0)
		 beq   $t8,$0,9f
		 or    $s0,$0,$s4       	 # Delay slot - Copy X to Carry
		 sll   $t9,$t8,1
		 andi  $t1,$t8,1
		 bne   $t1,$t8,7f
		 subu  $t7,$t7,$t9       	 # Delay slot
		 andi  $s4,$v0,0x01      	 # Set X
		 sll   $s0,$s0,31
		 srl   $v0,$v0,1
		 bgez  $0,6f
		 or    $v0,$v0,$s0        	 # Delay slot
	7:
		 ori   $t3,$0,33
	8:
		 subu  $t1,$t8,$t3
		 addiu $t9,$t1,-1
		 slt   $t9,$t9,$0
		 beq   $t9,$0,8b
		 movz  $t8,$t1,$t9        	 # Delay slot
		 subu  $t3,$t3,$t8
		 addiu $t1,$t8,-1
		 addiu $t2,$t3,-1
		 sllv  $s0,$s0,$t2
		 srlv  $s4,$v0,$t1
		 andi  $s4,$s4,0x01     	 # Set X
		 sllv  $t9,$v0,$t3
		 srlv  $v0,$v0,$t8
		 or    $v0,$v0,$t9
		 or    $v0,$v0,$s0
	6:
		 or    $s0,$0,$s4       	 # Copy X to Carry
	9:
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 sw    $v0,0x00($t0)
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e0b0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t8,0x00($t8)
		 andi  $t8,$t8,0x3F
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $v0,0x00($t0)
		 beq   $t8,$0,9f
		 or    $s0,$0,$s4       	 # Delay slot - Copy X to Carry
		 sll   $t9,$t8,1
		 andi  $t1,$t8,1
		 bne   $t1,$t8,7f
		 subu  $t7,$t7,$t9       	 # Delay slot
		 andi  $s4,$v0,0x01      	 # Set X
		 sll   $s0,$s0,31
		 srl   $v0,$v0,1
		 bgez  $0,6f
		 or    $v0,$v0,$s0        	 # Delay slot
	7:
		 ori   $t3,$0,33
	8:
		 subu  $t1,$t8,$t3
		 addiu $t9,$t1,-1
		 slt   $t9,$t9,$0
		 beq   $t9,$0,8b
		 movz  $t8,$t1,$t9        	 # Delay slot
		 subu  $t3,$t3,$t8
		 addiu $t1,$t8,-1
		 addiu $t2,$t3,-1
		 sllv  $s0,$s0,$t2
		 srlv  $s4,$v0,$t1
		 andi  $s4,$s4,0x01     	 # Set X
		 sllv  $t9,$v0,$t3
		 srlv  $v0,$v0,$t8
		 or    $v0,$v0,$t9
		 or    $v0,$v0,$s0
	6:
		 or    $s0,$0,$s4       	 # Copy X to Carry
	9:
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 sw    $v0,0x00($t0)
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e110:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lbu   $v0,0x00($t0)
		 beq   $t8,$0,9f
		 or    $s0,$0,$s4       	 # Delay slot - Copy X to Carry
		 sll   $t9,$t8,1
		 andi  $t1,$t8,1
		 bne   $t1,$t8,7f
		 subu  $t7,$t7,$t9       	 # Delay slot
		 srl   $s4,$v0,7        	 # Set X
		 sll   $v0,$v0,1
		 bgez  $0,6f
		 or    $v0,$v0,$s0        	 # Delay slot
	7:
		 ori   $t3,$0,9
	8:
		 subu  $t1,$t8,$t3
		 addiu $t9,$t1,-1
		 slt   $t9,$t9,$0
		 beq   $t9,$0,8b
		 movz  $t8,$t1,$t9        	 # Delay slot
		 subu  $t3,$t3,$t8
		 addiu $t1,$t8,-1
		 sllv  $s0,$s0,$t1
		 srlv  $s4,$v0,$t3
		 andi  $s4,$s4,0x01     	 # Set X
		 srlv  $t9,$v0,$t3
		 sllv  $v0,$v0,$t8
		 or    $v0,$v0,$t9
		 or    $v0,$v0,$s0
	6:
		 or    $s0,$0,$s4       	 # Copy X to Carry
	9:
		 andi  $v0,$v0,0xFF
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 sb    $v0,0x00($t0)
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e130:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t8,0x00($t8)
		 andi  $t8,$t8,0x3F
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lbu   $v0,0x00($t0)
		 beq   $t8,$0,9f
		 or    $s0,$0,$s4       	 # Delay slot - Copy X to Carry
		 sll   $t9,$t8,1
		 andi  $t1,$t8,1
		 bne   $t1,$t8,7f
		 subu  $t7,$t7,$t9       	 # Delay slot
		 srl   $s4,$v0,7        	 # Set X
		 sll   $v0,$v0,1
		 bgez  $0,6f
		 or    $v0,$v0,$s0        	 # Delay slot
	7:
		 ori   $t3,$0,9
	8:
		 subu  $t1,$t8,$t3
		 addiu $t9,$t1,-1
		 slt   $t9,$t9,$0
		 beq   $t9,$0,8b
		 movz  $t8,$t1,$t9        	 # Delay slot
		 subu  $t3,$t3,$t8
		 addiu $t1,$t8,-1
		 sllv  $s0,$s0,$t1
		 srlv  $s4,$v0,$t3
		 andi  $s4,$s4,0x01     	 # Set X
		 srlv  $t9,$v0,$t3
		 sllv  $v0,$v0,$t8
		 or    $v0,$v0,$t9
		 or    $v0,$v0,$s0
	6:
		 or    $s0,$0,$s4       	 # Copy X to Carry
	9:
		 andi  $v0,$v0,0xFF
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 sb    $v0,0x00($t0)
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e150:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lhu   $v0,0x00($t0)
		 beq   $t8,$0,9f
		 or    $s0,$0,$s4       	 # Delay slot - Copy X to Carry
		 sll   $t9,$t8,1
		 andi  $t1,$t8,1
		 bne   $t1,$t8,7f
		 subu  $t7,$t7,$t9       	 # Delay slot
		 srl   $s4,$v0,15        	 # Set X
		 sll   $v0,$v0,1
		 bgez  $0,6f
		 or    $v0,$v0,$s0        	 # Delay slot
	7:
		 ori   $t3,$0,17
	8:
		 subu  $t1,$t8,$t3
		 addiu $t9,$t1,-1
		 slt   $t9,$t9,$0
		 beq   $t9,$0,8b
		 movz  $t8,$t1,$t9        	 # Delay slot
		 subu  $t3,$t3,$t8
		 addiu $t1,$t8,-1
		 sllv  $s0,$s0,$t1
		 srlv  $s4,$v0,$t3
		 andi  $s4,$s4,0x01     	 # Set X
		 srlv  $t9,$v0,$t3
		 sllv  $v0,$v0,$t8
		 or    $v0,$v0,$t9
		 or    $v0,$v0,$s0
	6:
		 or    $s0,$0,$s4       	 # Copy X to Carry
	9:
		 andi  $v0,$v0,0xFFFF
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 sh    $v0,0x00($t0)
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e170:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t8,0x00($t8)
		 andi  $t8,$t8,0x3F
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lhu   $v0,0x00($t0)
		 beq   $t8,$0,9f
		 or    $s0,$0,$s4       	 # Delay slot - Copy X to Carry
		 sll   $t9,$t8,1
		 andi  $t1,$t8,1
		 bne   $t1,$t8,7f
		 subu  $t7,$t7,$t9       	 # Delay slot
		 srl   $s4,$v0,15        	 # Set X
		 sll   $v0,$v0,1
		 bgez  $0,6f
		 or    $v0,$v0,$s0        	 # Delay slot
	7:
		 ori   $t3,$0,17
	8:
		 subu  $t1,$t8,$t3
		 addiu $t9,$t1,-1
		 slt   $t9,$t9,$0
		 beq   $t9,$0,8b
		 movz  $t8,$t1,$t9        	 # Delay slot
		 subu  $t3,$t3,$t8
		 addiu $t1,$t8,-1
		 sllv  $s0,$s0,$t1
		 srlv  $s4,$v0,$t3
		 andi  $s4,$s4,0x01     	 # Set X
		 srlv  $t9,$v0,$t3
		 sllv  $v0,$v0,$t8
		 or    $v0,$v0,$t9
		 or    $v0,$v0,$s0
	6:
		 or    $s0,$0,$s4       	 # Copy X to Carry
	9:
		 andi  $v0,$v0,0xFFFF
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 sh    $v0,0x00($t0)
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e190:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $v0,0x00($t0)
		 beq   $t8,$0,9f
		 or    $s0,$0,$s4       	 # Delay slot - Copy X to Carry
		 sll   $t9,$t8,1
		 andi  $t1,$t8,1
		 bne   $t1,$t8,7f
		 subu  $t7,$t7,$t9       	 # Delay slot
		 srl   $s4,$v0,31        	 # Set X
		 sll   $v0,$v0,1
		 bgez  $0,6f
		 or    $v0,$v0,$s0        	 # Delay slot
	7:
		 ori   $t3,$0,33
	8:
		 subu  $t1,$t8,$t3
		 addiu $t9,$t1,-1
		 slt   $t9,$t9,$0
		 beq   $t9,$0,8b
		 movz  $t8,$t1,$t9        	 # Delay slot
		 subu  $t3,$t3,$t8
		 addiu $t1,$t8,-1
		 sllv  $s0,$s0,$t1
		 srlv  $s4,$v0,$t3
		 andi  $s4,$s4,0x01     	 # Set X
		 srlv  $t9,$v0,$t3
		 sllv  $v0,$v0,$t8
		 or    $v0,$v0,$t9
		 or    $v0,$v0,$s0
	6:
		 or    $s0,$0,$s4       	 # Copy X to Carry
	9:
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 sw    $v0,0x00($t0)
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e1b0:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t8,0x00($t8)
		 andi  $t8,$t8,0x3F
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $v0,0x00($t0)
		 beq   $t8,$0,9f
		 or    $s0,$0,$s4       	 # Delay slot - Copy X to Carry
		 sll   $t9,$t8,1
		 andi  $t1,$t8,1
		 bne   $t1,$t8,7f
		 subu  $t7,$t7,$t9       	 # Delay slot
		 srl   $s4,$v0,31        	 # Set X
		 sll   $v0,$v0,1
		 bgez  $0,6f
		 or    $v0,$v0,$s0        	 # Delay slot
	7:
		 ori   $t3,$0,33
	8:
		 subu  $t1,$t8,$t3
		 addiu $t9,$t1,-1
		 slt   $t9,$t9,$0
		 beq   $t9,$0,8b
		 movz  $t8,$t1,$t9        	 # Delay slot
		 subu  $t3,$t3,$t8
		 addiu $t1,$t8,-1
		 sllv  $s0,$s0,$t1
		 srlv  $s4,$v0,$t3
		 andi  $s4,$s4,0x01     	 # Set X
		 srlv  $t9,$v0,$t3
		 sllv  $v0,$v0,$t8
		 or    $v0,$v0,$t9
		 or    $v0,$v0,$s0
	6:
		 or    $s0,$0,$s4       	 # Copy X to Carry
	9:
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 sw    $v0,0x00($t0)
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e210:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lbu   $v0,0x00($t8)
		 sll   $t1,$s4,7
		 andi  $s4,$v0,0x01      	 # Set X
		 srl   $v0,$v0,1
		 or    $v0,$v0,$t1
		 or    $s0,$0,$s4       	 # Copy X to Carry
		 andi  $v0,$v0,0xFF
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 sb    $v0,0x00($t8)
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e250:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lhu   $v0,0x00($t8)
		 sll   $t1,$s4,15
		 andi  $s4,$v0,0x01      	 # Set X
		 srl   $v0,$v0,1
		 or    $v0,$v0,$t1
		 or    $s0,$0,$s4       	 # Copy X to Carry
		 andi  $v0,$v0,0xFFFF
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 sh    $v0,0x00($t8)
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e290:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $v0,0x00($t8)
		 sll   $t1,$s4,31
		 andi  $s4,$v0,0x01      	 # Set X
		 srl   $v0,$v0,1
		 or    $v0,$v0,$t1
		 or    $s0,$0,$s4       	 # Copy X to Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 sw    $v0,0x00($t8)
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e310:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lbu   $v0,0x00($t8)
		 or    $t1,$0,$s4
		 srl   $s4,$v0,7        	 # Set X
		 sll   $v0,$v0,1
		 or    $v0,$v0,$t1
		 or    $s0,$0,$s4       	 # Copy X to Carry
		 andi  $v0,$v0,0xFF
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 sb    $v0,0x00($t8)
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e350:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lhu   $v0,0x00($t8)
		 or    $t1,$0,$s4
		 srl   $s4,$v0,15        	 # Set X
		 sll   $v0,$v0,1
		 or    $v0,$v0,$t1
		 or    $s0,$0,$s4       	 # Copy X to Carry
		 andi  $v0,$v0,0xFFFF
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 sh    $v0,0x00($t8)
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e390:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $v0,0x00($t8)
		 or    $t1,$0,$s4
		 srl   $s4,$v0,31        	 # Set X
		 sll   $v0,$v0,1
		 or    $v0,$v0,$t1
		 or    $s0,$0,$s4       	 # Copy X to Carry
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,31        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 sw    $v0,0x00($t8)
		 addiu $t7,$t7,-10
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e4d0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 sll   $t1,$s4,15
		 andi  $s4,$v0,0x01      	 # Set X
		 srl   $v0,$v0,1
		 or    $v0,$v0,$t1
		 or    $s0,$0,$s4       	 # Copy X to Carry
		 andi  $v0,$v0,0xffff
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e4d8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 sll   $t1,$s4,15
		 andi  $s4,$v0,0x01      	 # Set X
		 srl   $v0,$v0,1
		 or    $v0,$v0,$t1
		 or    $s0,$0,$s4       	 # Copy X to Carry
		 andi  $v0,$v0,0xffff
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e4e0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 sll   $t1,$s4,15
		 andi  $s4,$v0,0x01      	 # Set X
		 srl   $v0,$v0,1
		 or    $v0,$v0,$t1
		 or    $s0,$0,$s4       	 # Copy X to Carry
		 andi  $v0,$v0,0xffff
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e4e8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 sll   $t1,$s4,15
		 andi  $s4,$v0,0x01      	 # Set X
		 srl   $v0,$v0,1
		 or    $v0,$v0,$t1
		 or    $s0,$0,$s4       	 # Copy X to Carry
		 andi  $v0,$v0,0xffff
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e4f0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 sll   $t1,$s4,15
		 andi  $s4,$v0,0x01      	 # Set X
		 srl   $v0,$v0,1
		 or    $v0,$v0,$t1
		 or    $s0,$0,$s4       	 # Copy X to Carry
		 andi  $v0,$v0,0xffff
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e4f8:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 sll   $t1,$s4,15
		 andi  $s4,$v0,0x01      	 # Set X
		 srl   $v0,$v0,1
		 or    $v0,$v0,$t1
		 or    $s0,$0,$s4       	 # Copy X to Carry
		 andi  $v0,$v0,0xffff
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e4f9:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 sll   $t1,$s4,15
		 andi  $s4,$v0,0x01      	 # Set X
		 srl   $v0,$v0,1
		 or    $v0,$v0,$t1
		 or    $s0,$0,$s4       	 # Copy X to Carry
		 andi  $v0,$v0,0xffff
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e5d0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 andi  $t1,$s4,0x01
		 srl   $s4,$v0,15        	 # Set X
		 sll   $v0,$v0,1
		 or    $v0,$v0,$t1
		 or    $s0,$0,$s4       	 # Copy X to Carry
		 andi  $v0,$v0,0xffff
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e5d8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 andi  $t1,$s4,0x01
		 srl   $s4,$v0,15        	 # Set X
		 sll   $v0,$v0,1
		 or    $v0,$v0,$t1
		 or    $s0,$0,$s4       	 # Copy X to Carry
		 andi  $v0,$v0,0xffff
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e5e0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 andi  $t1,$s4,0x01
		 srl   $s4,$v0,15        	 # Set X
		 sll   $v0,$v0,1
		 or    $v0,$v0,$t1
		 or    $s0,$0,$s4       	 # Copy X to Carry
		 andi  $v0,$v0,0xffff
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e5e8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 andi  $t1,$s4,0x01
		 srl   $s4,$v0,15        	 # Set X
		 sll   $v0,$v0,1
		 or    $v0,$v0,$t1
		 or    $s0,$0,$s4       	 # Copy X to Carry
		 andi  $v0,$v0,0xffff
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e5f0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 andi  $t1,$s4,0x01
		 srl   $s4,$v0,15        	 # Set X
		 sll   $v0,$v0,1
		 or    $v0,$v0,$t1
		 or    $s0,$0,$s4       	 # Copy X to Carry
		 andi  $v0,$v0,0xffff
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e5f8:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 andi  $t1,$s4,0x01
		 srl   $s4,$v0,15        	 # Set X
		 sll   $v0,$v0,1
		 or    $v0,$v0,$t1
		 or    $s0,$0,$s4       	 # Copy X to Carry
		 andi  $v0,$v0,0xffff
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e5f9:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 andi  $t1,$s4,0x01
		 srl   $s4,$v0,15        	 # Set X
		 sll   $v0,$v0,1
		 or    $v0,$v0,$t1
		 or    $s0,$0,$s4       	 # Copy X to Carry
		 andi  $v0,$v0,0xffff
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e008:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 sll   $t9,$t8,1
		 subu  $t7,$t7,$t9
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lbu   $v0,0x00($t0)
		 andi  $t1,$t8,0x20
		 addiu $t2,$t8,-1
		 andi  $t3,$t2,0x20
		 srlv  $s0,$v0,$t2
		 andi  $s0,$s0,0x01     	 # Set Carry
		 srlv  $v0,$v0,$t8
		 andi  $v0,$v0,0xFF
		 movn  $v0,$0,$t1         	 # Return 0 if shift by 32-63
		 movn  $s0,$0,$t3       	 # Clear Carry if shift by >32
		 movz  $s0,$0,$t8       	 # Clear Carry if shift by 0
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 movn  $s4,$s0,$t8      	 # Set X if not shift by 0
		 sb    $v0,0x00($t0)
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e028:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t8,0x00($t8)
		 andi  $t8,$t8,0x3F
		 sll   $t9,$t8,1
		 subu  $t7,$t7,$t9
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lbu   $v0,0x00($t0)
		 andi  $t1,$t8,0x20
		 addiu $t2,$t8,-1
		 andi  $t3,$t2,0x20
		 srlv  $s0,$v0,$t2
		 andi  $s0,$s0,0x01     	 # Set Carry
		 srlv  $v0,$v0,$t8
		 andi  $v0,$v0,0xFF
		 movn  $v0,$0,$t1         	 # Return 0 if shift by 32-63
		 movn  $s0,$0,$t3       	 # Clear Carry if shift by >32
		 movz  $s0,$0,$t8       	 # Clear Carry if shift by 0
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 movn  $s4,$s0,$t8      	 # Set X if not shift by 0
		 sb    $v0,0x00($t0)
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e048:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 sll   $t9,$t8,1
		 subu  $t7,$t7,$t9
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lhu   $v0,0x00($t0)
		 andi  $t1,$t8,0x20
		 addiu $t2,$t8,-1
		 andi  $t3,$t2,0x20
		 srlv  $s0,$v0,$t2
		 andi  $s0,$s0,0x01     	 # Set Carry
		 srlv  $v0,$v0,$t8
		 andi  $v0,$v0,0xFFFF
		 movn  $v0,$0,$t1         	 # Return 0 if shift by 32-63
		 movn  $s0,$0,$t3       	 # Clear Carry if shift by >32
		 movz  $s0,$0,$t8       	 # Clear Carry if shift by 0
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 movn  $s4,$s0,$t8      	 # Set X if not shift by 0
		 sh    $v0,0x00($t0)
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e068:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t8,0x00($t8)
		 andi  $t8,$t8,0x3F
		 sll   $t9,$t8,1
		 subu  $t7,$t7,$t9
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lhu   $v0,0x00($t0)
		 andi  $t1,$t8,0x20
		 addiu $t2,$t8,-1
		 andi  $t3,$t2,0x20
		 srlv  $s0,$v0,$t2
		 andi  $s0,$s0,0x01     	 # Set Carry
		 srlv  $v0,$v0,$t8
		 andi  $v0,$v0,0xFFFF
		 movn  $v0,$0,$t1         	 # Return 0 if shift by 32-63
		 movn  $s0,$0,$t3       	 # Clear Carry if shift by >32
		 movz  $s0,$0,$t8       	 # Clear Carry if shift by 0
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 movn  $s4,$s0,$t8      	 # Set X if not shift by 0
		 sh    $v0,0x00($t0)
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e088:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 sll   $t9,$t8,1
		 subu  $t7,$t7,$t9
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $v0,0x00($t0)
		 andi  $t1,$t8,0x20
		 addiu $t2,$t8,-1
		 andi  $t3,$t2,0x20
		 srlv  $s0,$v0,$t2
		 andi  $s0,$s0,0x01     	 # Set Carry
		 srlv  $v0,$v0,$t8
		 movn  $v0,$0,$t1         	 # Return 0 if shift by 32-63
		 movn  $s0,$0,$t3       	 # Clear Carry if shift by >32
		 movz  $s0,$0,$t8       	 # Clear Carry if shift by 0
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 movn  $s4,$s0,$t8      	 # Set X if not shift by 0
		 sw    $v0,0x00($t0)
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e0a8:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t8,0x00($t8)
		 andi  $t8,$t8,0x3F
		 sll   $t9,$t8,1
		 subu  $t7,$t7,$t9
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $v0,0x00($t0)
		 andi  $t1,$t8,0x20
		 addiu $t2,$t8,-1
		 andi  $t3,$t2,0x20
		 srlv  $s0,$v0,$t2
		 andi  $s0,$s0,0x01     	 # Set Carry
		 srlv  $v0,$v0,$t8
		 movn  $v0,$0,$t1         	 # Return 0 if shift by 32-63
		 movn  $s0,$0,$t3       	 # Clear Carry if shift by >32
		 movz  $s0,$0,$t8       	 # Clear Carry if shift by 0
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 movn  $s4,$s0,$t8      	 # Set X if not shift by 0
		 sw    $v0,0x00($t0)
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e108:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 sll   $t9,$t8,1
		 subu  $t7,$t7,$t9
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lbu   $v0,0x00($t0)
		 andi  $t1,$t8,0x20
		 addiu $t2,$t8,-1
		 andi  $t3,$t2,0x20
		 sll   $s0,$v0,24
		 sllv  $s0,$v0,$t2
		 srl   $s0,$s0,31       	 # Set Carry
		 sllv  $v0,$v0,$t8
		 andi  $v0,$v0,0xFF
		 movn  $v0,$0,$t1         	 # Return 0 if shift by 32-63
		 movn  $s0,$0,$t3       	 # Clear Carry if shift by >32
		 movz  $s0,$0,$t8       	 # Clear Carry if shift by 0
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 movn  $s4,$s0,$t8      	 # Set X if not shift by 0
		 sb    $v0,0x00($t0)
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e128:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t8,0x00($t8)
		 andi  $t8,$t8,0x3F
		 sll   $t9,$t8,1
		 subu  $t7,$t7,$t9
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lbu   $v0,0x00($t0)
		 andi  $t1,$t8,0x20
		 addiu $t2,$t8,-1
		 andi  $t3,$t2,0x20
		 sll   $s0,$v0,24
		 sllv  $s0,$v0,$t2
		 srl   $s0,$s0,31       	 # Set Carry
		 sllv  $v0,$v0,$t8
		 andi  $v0,$v0,0xFF
		 movn  $v0,$0,$t1         	 # Return 0 if shift by 32-63
		 movn  $s0,$0,$t3       	 # Clear Carry if shift by >32
		 movz  $s0,$0,$t8       	 # Clear Carry if shift by 0
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,7        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 movn  $s4,$s0,$t8      	 # Set X if not shift by 0
		 sb    $v0,0x00($t0)
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e148:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 sll   $t9,$t8,1
		 subu  $t7,$t7,$t9
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lhu   $v0,0x00($t0)
		 andi  $t1,$t8,0x20
		 addiu $t2,$t8,-1
		 andi  $t3,$t2,0x20
		 sll   $s0,$v0,16
		 sllv  $s0,$v0,$t2
		 srl   $s0,$s0,31       	 # Set Carry
		 sllv  $v0,$v0,$t8
		 andi  $v0,$v0,0xFFFF
		 movn  $v0,$0,$t1         	 # Return 0 if shift by 32-63
		 movn  $s0,$0,$t3       	 # Clear Carry if shift by >32
		 movz  $s0,$0,$t8       	 # Clear Carry if shift by 0
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 movn  $s4,$s0,$t8      	 # Set X if not shift by 0
		 sh    $v0,0x00($t0)
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e168:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t8,0x00($t8)
		 andi  $t8,$t8,0x3F
		 sll   $t9,$t8,1
		 subu  $t7,$t7,$t9
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lhu   $v0,0x00($t0)
		 andi  $t1,$t8,0x20
		 addiu $t2,$t8,-1
		 andi  $t3,$t2,0x20
		 sll   $s0,$v0,16
		 sllv  $s0,$v0,$t2
		 srl   $s0,$s0,31       	 # Set Carry
		 sllv  $v0,$v0,$t8
		 andi  $v0,$v0,0xFFFF
		 movn  $v0,$0,$t1         	 # Return 0 if shift by 32-63
		 movn  $s0,$0,$t3       	 # Clear Carry if shift by >32
		 movz  $s0,$0,$t8       	 # Clear Carry if shift by 0
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 movn  $s4,$s0,$t8      	 # Set X if not shift by 0
		 sh    $v0,0x00($t0)
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e188:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 sll   $t9,$t8,1
		 subu  $t7,$t7,$t9
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $v0,0x00($t0)
		 andi  $t1,$t8,0x20
		 addiu $t2,$t8,-1
		 andi  $t3,$t2,0x20
		 sllv  $s0,$v0,$t2
		 srl   $s0,$s0,31       	 # Set Carry
		 sllv  $v0,$v0,$t8
		 movn  $v0,$0,$t1         	 # Return 0 if shift by 32-63
		 movn  $s0,$0,$t3       	 # Clear Carry if shift by >32
		 movz  $s0,$0,$t8       	 # Clear Carry if shift by 0
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 movn  $s4,$s0,$t8      	 # Set X if not shift by 0
		 sw    $v0,0x00($t0)
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e1a8:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t8,0x00($t8)
		 andi  $t8,$t8,0x3F
		 sll   $t9,$t8,1
		 subu  $t7,$t7,$t9
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $v0,0x00($t0)
		 andi  $t1,$t8,0x20
		 addiu $t2,$t8,-1
		 andi  $t3,$t2,0x20
		 sllv  $s0,$v0,$t2
		 srl   $s0,$s0,31       	 # Set Carry
		 sllv  $v0,$v0,$t8
		 movn  $v0,$0,$t1         	 # Return 0 if shift by 32-63
		 movn  $s0,$0,$t3       	 # Clear Carry if shift by >32
		 movz  $s0,$0,$t8       	 # Clear Carry if shift by 0
		 and   $s1,$0,$0        	 # Clear Overflow
		 slt   $s3,$v0,$0        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 movn  $s4,$s0,$t8      	 # Set X if not shift by 0
		 sw    $v0,0x00($t0)
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e2d0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 andi  $s0,$v0,0x01      	 # Set Carry
		 srl   $v0,$v0,1
		 andi  $v0,$v0,0xffff
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 movn  $s4,$s0,$t8      	 # Set X if not shift by 0
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e2d8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 andi  $s0,$v0,0x01      	 # Set Carry
		 srl   $v0,$v0,1
		 andi  $v0,$v0,0xffff
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 movn  $s4,$s0,$t8      	 # Set X if not shift by 0
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e2e0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 andi  $s0,$v0,0x01      	 # Set Carry
		 srl   $v0,$v0,1
		 andi  $v0,$v0,0xffff
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 movn  $s4,$s0,$t8      	 # Set X if not shift by 0
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e2e8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 andi  $s0,$v0,0x01      	 # Set Carry
		 srl   $v0,$v0,1
		 andi  $v0,$v0,0xffff
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 movn  $s4,$s0,$t8      	 # Set X if not shift by 0
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e2f0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 andi  $s0,$v0,0x01      	 # Set Carry
		 srl   $v0,$v0,1
		 andi  $v0,$v0,0xffff
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 movn  $s4,$s0,$t8      	 # Set X if not shift by 0
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e2f8:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 andi  $s0,$v0,0x01      	 # Set Carry
		 srl   $v0,$v0,1
		 andi  $v0,$v0,0xffff
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 movn  $s4,$s0,$t8      	 # Set X if not shift by 0
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e2f9:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 andi  $s0,$v0,0x01      	 # Set Carry
		 srl   $v0,$v0,1
		 andi  $v0,$v0,0xffff
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 movn  $s4,$s0,$t8      	 # Set X if not shift by 0
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e3d0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 srl   $s0,$v0,15        	 # Set Carry
		 sll   $v0,$v0,1
		 andi  $v0,$v0,0xffff
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 movn  $s4,$s0,$t8      	 # Set X if not shift by 0
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e3d8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 srl   $s0,$v0,15        	 # Set Carry
		 sll   $v0,$v0,1
		 andi  $v0,$v0,0xffff
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 movn  $s4,$s0,$t8      	 # Set X if not shift by 0
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e3e0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 srl   $s0,$v0,15        	 # Set Carry
		 sll   $v0,$v0,1
		 andi  $v0,$v0,0xffff
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 movn  $s4,$s0,$t8      	 # Set X if not shift by 0
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e3e8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 srl   $s0,$v0,15        	 # Set Carry
		 sll   $v0,$v0,1
		 andi  $v0,$v0,0xffff
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 movn  $s4,$s0,$t8      	 # Set X if not shift by 0
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e3f0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 srl   $s0,$v0,15        	 # Set Carry
		 sll   $v0,$v0,1
		 andi  $v0,$v0,0xffff
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 movn  $s4,$s0,$t8      	 # Set X if not shift by 0
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e3f8:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 srl   $s0,$v0,15        	 # Set Carry
		 sll   $v0,$v0,1
		 andi  $v0,$v0,0xffff
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 movn  $s4,$s0,$t8      	 # Set X if not shift by 0
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e3f9:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 srl   $s0,$v0,15        	 # Set Carry
		 sll   $v0,$v0,1
		 andi  $v0,$v0,0xffff
		 and   $s1,$0,$0        	 # Clear Overflow
		 srl   $s3,$v0,15        	 # Set Sign
		 sltiu $s2,$v0,1         	 # Set Zero
		 movn  $s4,$s0,$t8      	 # Set X if not shift by 0
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e018:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lbu   $v0,0x00($t0)
		 beq   $t8,$0,9f
		 and   $s0,$0,$0        	 # Delay slot - Clear Carry
		 sll   $t9,$t8,1
		 subu  $t7,$t7,$t9
		 andi  $t8,$t8,0x07
		 subu  $t9,$0,$t8
		 andi  $t9,$t9,0x07
		 sllv  $t9,$v0,$t9
		 srlv  $v0,$v0,$t8
		 or    $v0,$v0,$t9
		 srl   $s0,$v0,7        	 # Set Carry
	9:
		 andi  $v0,$v0,0xFF
		 and   $s1,$0,$0        	 # Clear Overflow
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $s3,$v0,7        	 # Set Sign
		 sb    $v0,0x00($t0)
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e038:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t8,0x00($t8)
		 andi  $t8,$t8,0x3F
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lbu   $v0,0x00($t0)
		 beq   $t8,$0,9f
		 and   $s0,$0,$0        	 # Delay slot - Clear Carry
		 sll   $t9,$t8,1
		 subu  $t7,$t7,$t9
		 andi  $t8,$t8,0x07
		 subu  $t9,$0,$t8
		 andi  $t9,$t9,0x07
		 sllv  $t9,$v0,$t9
		 srlv  $v0,$v0,$t8
		 or    $v0,$v0,$t9
		 srl   $s0,$v0,7        	 # Set Carry
	9:
		 andi  $v0,$v0,0xFF
		 and   $s1,$0,$0        	 # Clear Overflow
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $s3,$v0,7        	 # Set Sign
		 sb    $v0,0x00($t0)
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e058:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lhu   $v0,0x00($t0)
		 beq   $t8,$0,9f
		 and   $s0,$0,$0        	 # Delay slot - Clear Carry
		 sll   $t9,$t8,1
		 subu  $t7,$t7,$t9
		 andi  $t8,$t8,0x0F
		 subu  $t9,$0,$t8
		 andi  $t9,$t9,0x0F
		 sllv  $t9,$v0,$t9
		 srlv  $v0,$v0,$t8
		 or    $v0,$v0,$t9
		 srl   $s0,$v0,15        	 # Set Carry
	9:
		 andi  $v0,$v0,0xFFFF
		 and   $s1,$0,$0        	 # Clear Overflow
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $s3,$v0,15        	 # Set Sign
		 sh    $v0,0x00($t0)
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e078:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t8,0x00($t8)
		 andi  $t8,$t8,0x3F
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lhu   $v0,0x00($t0)
		 beq   $t8,$0,9f
		 and   $s0,$0,$0        	 # Delay slot - Clear Carry
		 sll   $t9,$t8,1
		 subu  $t7,$t7,$t9
		 andi  $t8,$t8,0x0F
		 subu  $t9,$0,$t8
		 andi  $t9,$t9,0x0F
		 sllv  $t9,$v0,$t9
		 srlv  $v0,$v0,$t8
		 or    $v0,$v0,$t9
		 srl   $s0,$v0,15        	 # Set Carry
	9:
		 andi  $v0,$v0,0xFFFF
		 and   $s1,$0,$0        	 # Clear Overflow
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $s3,$v0,15        	 # Set Sign
		 sh    $v0,0x00($t0)
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e098:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $v0,0x00($t0)
		 beq   $t8,$0,9f
		 and   $s0,$0,$0        	 # Delay slot - Clear Carry
		 sll   $t9,$t8,1
		 subu  $t7,$t7,$t9
		 m_rorv  $v0,$v0,$t8
		 srl   $s0,$v0,31        	 # Set Carry
	9:
		 and   $s1,$0,$0        	 # Clear Overflow
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $s3,$v0,31        	 # Set Sign
		 sw    $v0,0x00($t0)
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e0b8:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t8,0x00($t8)
		 andi  $t8,$t8,0x3F
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $v0,0x00($t0)
		 beq   $t8,$0,9f
		 and   $s0,$0,$0        	 # Delay slot - Clear Carry
		 sll   $t9,$t8,1
		 subu  $t7,$t7,$t9
		 m_rorv  $v0,$v0,$t8
		 srl   $s0,$v0,31        	 # Set Carry
	9:
		 and   $s1,$0,$0        	 # Clear Overflow
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $s3,$v0,31        	 # Set Sign
		 sw    $v0,0x00($t0)
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e118:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lbu   $v0,0x00($t0)
		 beq   $t8,$0,9f
		 and   $s0,$0,$0        	 # Delay slot - Clear Carry
		 sll   $t9,$t8,1
		 subu  $t7,$t7,$t9
		 andi  $t8,$t8,0x07
		 subu  $t9,$0,$t8
		 andi  $t9,$t9,0x07
		 srlv  $t9,$v0,$t9
		 sllv  $v0,$v0,$t8
		 or    $v0,$v0,$t9
		 andi  $s0,$v0,0x01      	 # Set Carry
	9:
		 andi  $v0,$v0,0xFF
		 and   $s1,$0,$0        	 # Clear Overflow
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $s3,$v0,7        	 # Set Sign
		 sb    $v0,0x00($t0)
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e138:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t8,0x00($t8)
		 andi  $t8,$t8,0x3F
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lbu   $v0,0x00($t0)
		 beq   $t8,$0,9f
		 and   $s0,$0,$0        	 # Delay slot - Clear Carry
		 sll   $t9,$t8,1
		 subu  $t7,$t7,$t9
		 andi  $t8,$t8,0x07
		 subu  $t9,$0,$t8
		 andi  $t9,$t9,0x07
		 srlv  $t9,$v0,$t9
		 sllv  $v0,$v0,$t8
		 or    $v0,$v0,$t9
		 andi  $s0,$v0,0x01      	 # Set Carry
	9:
		 andi  $v0,$v0,0xFF
		 and   $s1,$0,$0        	 # Clear Overflow
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $s3,$v0,7        	 # Set Sign
		 sb    $v0,0x00($t0)
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e158:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lhu   $v0,0x00($t0)
		 beq   $t8,$0,9f
		 and   $s0,$0,$0        	 # Delay slot - Clear Carry
		 sll   $t9,$t8,1
		 subu  $t7,$t7,$t9
		 andi  $t8,$t8,0x0F
		 subu  $t9,$0,$t8
		 andi  $t9,$t9,0x0F
		 srlv  $t9,$v0,$t9
		 sllv  $v0,$v0,$t8
		 or    $v0,$v0,$t9
		 andi  $s0,$v0,0x01      	 # Set Carry
	9:
		 andi  $v0,$v0,0xFFFF
		 and   $s1,$0,$0        	 # Clear Overflow
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $s3,$v0,15        	 # Set Sign
		 sh    $v0,0x00($t0)
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e178:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t8,0x00($t8)
		 andi  $t8,$t8,0x3F
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lhu   $v0,0x00($t0)
		 beq   $t8,$0,9f
		 and   $s0,$0,$0        	 # Delay slot - Clear Carry
		 sll   $t9,$t8,1
		 subu  $t7,$t7,$t9
		 andi  $t8,$t8,0x0F
		 subu  $t9,$0,$t8
		 andi  $t9,$t9,0x0F
		 srlv  $t9,$v0,$t9
		 sllv  $v0,$v0,$t8
		 or    $v0,$v0,$t9
		 andi  $s0,$v0,0x01      	 # Set Carry
	9:
		 andi  $v0,$v0,0xFFFF
		 and   $s1,$0,$0        	 # Clear Overflow
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $s3,$v0,15        	 # Set Sign
		 sh    $v0,0x00($t0)
		 addiu $t7,$t7,-6
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e198:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,9
		 addiu $t8,$t8,-1       	 # Move range down
		 andi  $t8,$t8,0x07     	 # Mask out lower bits
		 addiu $t8,$t8,1        	 # correct range
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $v0,0x00($t0)
		 beq   $t8,$0,9f
		 and   $s0,$0,$0        	 # Delay slot - Clear Carry
		 sll   $t9,$t8,1
		 subu  $t7,$t7,$t9
		 subu  $t9,$0,$t8
		 m_rorv  $v0,$v0,$t9
		 andi  $s0,$v0,0x01      	 # Set Carry
	9:
		 and   $s1,$0,$0        	 # Clear Overflow
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $s3,$v0,31        	 # Set Sign
		 sw    $v0,0x00($t0)
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e1b8:				#:
		 addiu $s7,$s7,2

		 andi  $t0,$t8,0x07
		 srl   $t8,$t8,7
		 andi  $t8,$t8,0x1C
		 addu  $t8,$t8,$s5
		 lw    $t8,0x00($t8)
		 andi  $t8,$t8,0x3F
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $v0,0x00($t0)
		 beq   $t8,$0,9f
		 and   $s0,$0,$0        	 # Delay slot - Clear Carry
		 sll   $t9,$t8,1
		 subu  $t7,$t7,$t9
		 subu  $t9,$0,$t8
		 m_rorv  $v0,$v0,$t9
		 andi  $s0,$v0,0x01      	 # Set Carry
	9:
		 and   $s1,$0,$0        	 # Clear Overflow
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $s3,$v0,31        	 # Set Sign
		 sw    $v0,0x00($t0)
		 addiu $t7,$t7,-8
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e6d0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 sll   $t9,$v0,15
		 srl   $v0,$v0,1
		 or    $v0,$v0,$t9
		 srl   $s0,$v0,15        	 # Set Carry
		 andi  $v0,$v0,0xffff
		 and   $s1,$0,$0        	 # Clear Overflow
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $s3,$v0,15        	 # Set Sign
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e6d8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 sll   $t9,$v0,15
		 srl   $v0,$v0,1
		 or    $v0,$v0,$t9
		 srl   $s0,$v0,15        	 # Set Carry
		 andi  $v0,$v0,0xffff
		 and   $s1,$0,$0        	 # Clear Overflow
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $s3,$v0,15        	 # Set Sign
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e6e0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 sll   $t9,$v0,15
		 srl   $v0,$v0,1
		 or    $v0,$v0,$t9
		 srl   $s0,$v0,15        	 # Set Carry
		 andi  $v0,$v0,0xffff
		 and   $s1,$0,$0        	 # Clear Overflow
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $s3,$v0,15        	 # Set Sign
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e6e8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 sll   $t9,$v0,15
		 srl   $v0,$v0,1
		 or    $v0,$v0,$t9
		 srl   $s0,$v0,15        	 # Set Carry
		 andi  $v0,$v0,0xffff
		 and   $s1,$0,$0        	 # Clear Overflow
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $s3,$v0,15        	 # Set Sign
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e6f0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 sll   $t9,$v0,15
		 srl   $v0,$v0,1
		 or    $v0,$v0,$t9
		 srl   $s0,$v0,15        	 # Set Carry
		 andi  $v0,$v0,0xffff
		 and   $s1,$0,$0        	 # Clear Overflow
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $s3,$v0,15        	 # Set Sign
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e6f8:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 sll   $t9,$v0,15
		 srl   $v0,$v0,1
		 or    $v0,$v0,$t9
		 srl   $s0,$v0,15        	 # Set Carry
		 andi  $v0,$v0,0xffff
		 and   $s1,$0,$0        	 # Clear Overflow
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $s3,$v0,15        	 # Set Sign
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e6f9:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 sll   $t9,$v0,15
		 srl   $v0,$v0,1
		 or    $v0,$v0,$t9
		 srl   $s0,$v0,15        	 # Set Carry
		 andi  $v0,$v0,0xffff
		 and   $s1,$0,$0        	 # Clear Overflow
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $s3,$v0,15        	 # Set Sign
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e7d0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 srl   $t9,$v0,15
		 sll   $v0,$v0,1
		 or    $v0,$v0,$t9
		 andi  $s0,$v0,0x01      	 # Set Carry
		 andi  $v0,$v0,0xffff
		 and   $s1,$0,$0        	 # Clear Overflow
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $s3,$v0,15        	 # Set Sign
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e7d8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t9,$t6,2
		 sw    $t9,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 srl   $t9,$v0,15
		 sll   $v0,$v0,1
		 or    $v0,$v0,$t9
		 andi  $s0,$v0,0x01      	 # Set Carry
		 andi  $v0,$v0,0xffff
		 and   $s1,$0,$0        	 # Clear Overflow
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $s3,$v0,15        	 # Set Sign
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-12
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e7e0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $t6,$t6,-2
		 sw    $t6,0x20($t8)
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 srl   $t9,$v0,15
		 sll   $v0,$v0,1
		 or    $v0,$v0,$t9
		 andi  $s0,$v0,0x01      	 # Set Carry
		 andi  $v0,$v0,0xffff
		 and   $s1,$0,$0        	 # Clear Overflow
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $s3,$v0,15        	 # Set Sign
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-14
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e7e8:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 lh    $a3,0x00($s7)
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 addiu $s7,$s7,2
		 addu  $t6,$t6,$a3
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 srl   $t9,$v0,15
		 sll   $v0,$v0,1
		 or    $v0,$v0,$t9
		 andi  $s0,$v0,0x01      	 # Set Carry
		 andi  $v0,$v0,0xffff
		 and   $s1,$0,$0        	 # Clear Overflow
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $s3,$v0,15        	 # Set Sign
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e7f0:				#:
		 addiu $s7,$s7,2

		 andi  $t8,$t8,0x07
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 lw    $t6,0x20($t8)
		 lhu   $a3,0x00($s7)
		 addiu $s7,$s7,2
		 m_seb   $a2,$a3
		 or    $t9,$0,$a3
		 srl   $a3,$a3,12
		 andi  $t9,$t9,0x0800
		 sll   $a3,$a3,2
		 addu  $a3,$a3,$s5
		 bne   $t9,$0,0f
		 lw    $t9,0x00($a3)      	 # Delay slot
		 m_seh   $t9,$t9
	0:
		 addu  $t9,$t6,$t9
		 addu  $t6,$t9,$a2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 srl   $t9,$v0,15
		 sll   $v0,$v0,1
		 or    $v0,$v0,$t9
		 andi  $s0,$v0,0x01      	 # Set Carry
		 andi  $v0,$v0,0xffff
		 and   $s1,$0,$0        	 # Clear Overflow
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $s3,$v0,15        	 # Set Sign
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-18
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e7f8:				#:
		 addiu $s7,$s7,2

		 lh    $t6,0x00($s7)
		 addiu $s7,$s7,2
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 srl   $t9,$v0,15
		 sll   $v0,$v0,1
		 or    $v0,$v0,$t9
		 andi  $s0,$v0,0x01      	 # Set Carry
		 andi  $v0,$v0,0xffff
		 and   $s1,$0,$0        	 # Clear Overflow
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $s3,$v0,15        	 # Set Sign
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_e7f9:				#:
		 addiu $s7,$s7,2

		 lhu   $t6,0x00($s7)
		 lhu   $t9,0x02($s7)
		 sll   $t6,$t6,16
		 or    $t6,$t6,$t9
		 addiu $s7,$s7,4
		 lw    $t9,0x94($s5)
		 sw    $t7,m68k_ICount
		 sw    $t6,0x40($sp)
		 sw    $gp,0x3C($sp)
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x3C($sp)
		 lw    $t6,0x40($sp)
		 lw    $t7,m68k_ICount
		 srl   $t9,$v0,15
		 sll   $v0,$v0,1
		 or    $v0,$v0,$t9
		 andi  $s0,$v0,0x01      	 # Set Carry
		 andi  $v0,$v0,0xffff
		 and   $s1,$0,$0        	 # Clear Overflow
		 sltiu $s2,$v0,1         	 # Set Zero
		 srl   $s3,$v0,15        	 # Set Sign
		 lw    $t9,0xA0($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 or    $a1,$0,$v0
		 or    $a0,$0,$t6
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 addiu $t7,$t7,-20
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_a000:				#:
		 addiu $s7,$s7,2

		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,10

		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_f000:				#:
		 addiu $s7,$s7,2

		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,11

		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

ILLEGAL:
		 subu  $t9,$s7,$s6
		 sw    $t8,illegal_op
		 sw    $t9,illegal_pc
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,4

		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4e74:				#:
		 lw    $t9,0x84($s5)
		 bne   $t9,$0,0f
		 nop                    	 # Delay slot
		 m_j   ILLEGAL
	0:
		 nop                    	 # Delay slot

		 lw    $a0,0x3C($s5)
		 lw    $t9,0x98($s5)
		 sw    $t7,m68k_ICount
		 sw    $gp,0x40($sp)
		 .set  noat
		 subu  $at,$s7,$s6
		 sw    $at,0x50($s5)
		 .set  at
		 jalr  $t9
		 nop    			 # Delay slot
		 lw    $gp,0x40($sp)
		 lw    $t7,m68k_ICount
		 lh    $t0,0x00($s7)
		 lw    $t1,0x3C($s5)
		 addiu $t0,$t0,4
		 addu  $t1,$t0,$t1
		 sw    $t1,0x3C($s5)
		 andi  $t9,$v0,0x01
		 beq   $t9,$0,2f
		 addu  $s7,$s6,$v0     	 # Delay slot
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,3

		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	2:
		 sw    $v0,0x88($s5)
		 lw    $a2,mem_amask
		 lw    $t9,0xA8($s5)
		 and   $s7,$v0,$a2
		 sw    $t7,m68k_ICount
		 jalr  $t9
		 or    $a0,$0,$s7   	 # Delay slot
		 lw    $t7,m68k_ICount
		 lw    $s6,OP_ROM
		 addu  $s7,$s7,$s6
 # End of Banking code:
		 addiu $t7,$t7,-16
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4e7a:				#:
		 lw    $t9,0x84($s5)
		 bne   $t9,$0,0f
		 nop                    	 # Delay slot
		 m_j   ILLEGAL
	0:
		 nop                    	 # Delay slot

		 lbu   $t0,0x44($s5)
		 andi  $t0,$t0,0x20       	 # Supervisor Mode ?
		 beq   $t0,$0,9f
		 addiu $s7,$s7,2        	 # Delay slot
		 lhu   $t0,0x00($s7)
		 addiu $s7,$s7,2
		 andi  $v0,$t0,0x01
		 srl   $t8,$t0,10
		 andi  $t8,$t8,0x02
		 or    $t8,$t8,$v0
		 srl   $t0,$t0,12
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 addiu $t8,$t8,112
		 lw    $v0,0x00($t8)
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 sw    $v0,0x00($t0)
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	9:
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,8

		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

OP0_4e7b:				#:
		 lw    $t9,0x84($s5)
		 bne   $t9,$0,0f
		 nop                    	 # Delay slot
		 m_j   ILLEGAL
	0:
		 nop                    	 # Delay slot

		 lbu   $t0,0x44($s5)
		 andi  $t0,$t0,0x20       	 # Supervisor Mode ?
		 beq   $t0,$0,9f
		 addiu $s7,$s7,2        	 # Delay slot
		 lhu   $t0,0x00($s7)
		 addiu $s7,$s7,2
		 andi  $v0,$t0,0x01
		 srl   $t8,$t0,10
		 andi  $t8,$t8,0x02
		 or    $t8,$t8,$v0
		 srl   $t0,$t0,12
		 sll   $t0,$t0,2
		 addu  $t0,$t0,$s5
		 lw    $v0,0x00($t0)
		 andi  $t1,$t8,0x02
		 bne   $t1,$0,8f
		 nop                    	 # Delay slot
		 andi  $v0,$v0,0x07
	8:
		 sll   $t8,$t8,2
		 addu  $t8,$t8,$s5
		 addiu $t8,$t8,112
		 sw    $v0,0x00($t8)
		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

	9:
		 addiu $s7,$s7,-2
		 m_jal   Exception
		 ori   $v0,$0,8

		 addiu $t7,$t7,-4
		 bgez  $t7,3f
		 lhu   $t8,0x00($s7)    	 # Delay slot
		 m_j   MainExit
		 nop                    	 # Delay slot

	3:
		 sll   $a3,$t8,2
		 addu  $a3,$a3,$fp
		 lw    $a3,0x00($a3)
		 jr    $a3
		 nop                    	 # Delay slot

		 .data

		 .align 4


 # Register Structure

M68000_regs:
R_D0:	 .word 0			 # Data Registers
R_D1:	 .word 0
R_D2:	 .word 0
R_D3:	 .word 0
R_D4:	 .word 0
R_D5:	 .word 0
R_D6:	 .word 0
R_D7:	 .word 0

R_A0:	 .word 0			 # Address Registers
R_A1:	 .word 0
R_A2:	 .word 0
R_A3:	 .word 0
R_A4:	 .word 0
R_A5:	 .word 0
R_A6:	 .word 0
R_A7:	 .word 0

R_ISP:	 .word 0			 # Supervisor Stack
R_SR_H:	 .word 0			 # Status Register High TuSuuIII
R_CCR:	 .word 0
R_XC:	 .word 0
R_PC:	 .word 0			 # Program Counter
R_IRQ:	 .word 0			 # IRQ Request Level

R_SR:	 .word 0			 # Motorola Format SR

R_IRQ_CALLBACK:	 .word 0			 # irq callback (get vector)

R_PPC:	 .word 0			 # Previous Program Counter
R_RESET_CALLBACK:	 .word 0			 # Reset Callback
R_RTE_CALLBACK:	 .word 0
R_CMP_CALLBACK:	 .word 0
R_SFC:	 .word 0			 # Source Function Call
R_DFC:	 .word 0			 # Destination Function Call
R_USP:	 .word 0			 # User Stack
R_VBR:	 .word 0			 # Vector Base
asmbank:	 .word 0

CPUversion:	 .word 0

FullPC:	 .word 0

a68k_memory_intf:
	.rept 13
		.word 0
	.endr


exception_cycles:
		 .byte 0, 0, 0, 0, 38, 42, 44, 38, 38, 0, 38, 38, 0, 0, 0, 0
		 .byte 0, 0, 0, 0, 0, 0, 0, 0, 46, 46, 46, 46, 46, 46, 46, 46
		 .byte 38, 38, 38, 38, 38, 38, 38, 38, 38, 38, 38, 38, 38, 38, 38, 38

		 .align 4
 # Jump Table

M68000_OPCODETABLE:
.include "a68ktbl.inc"

